; This file contains modules for the "Analysis" menu of LISAPP

;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Navigation Data Routines ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;

pro plot_nav, event, idxmin, idxmax
	common sensor_read, this_sensor, file_format
	common datafiles, orbit, fname
	common otd_boom, boom
	common lisbase, lisb
	common lisapp_colors,red,green,blue
	common navstuff, navelements, navlow, navhigh, navwin, navb
	common navvals, xval, yval, xlo, xhi, ylo, yhi, xname, yname, cval
	common navbounds, navmin, navmax
	
	navmin=idxmin & navmax = idxmax
	navelements = strarr(19)
	navelements = ['Time','Position(0)','Position(1)','Position(2)','Velocity(0)', $
			'Velocity(1)','Velocity(2)','Boresight Lat','Boresight Lon', $
			'Geolocn Error','Transform(0)','Transform(1)','Transform(2)', $
			'Transform(3)','Transform(4)','Transform(5)','Transform(6)', $
			'Transform(7)','Transform(8)']
	navcolelements = strarr(20)
	navcolelements = ['(Solid)','Time','Position(0)','Position(1)','Position(2)','Velocity(0)', $
			'Velocity(1)','Velocity(2)','Boresight Lat','Boresight Lon', $
			'Geolocn Error','Transform(0)','Transform(1)','Transform(2)', $
			'Transform(3)','Transform(4)','Transform(5)','Transform(6)', $
			'Transform(7)','Transform(8)']
	navlow = fltarr(19)
	navlow = [navmin,-10000000,-10000000.,-10000000.,-10000.,-10000.,-10000., $
			-90.0,-180.0,-1.0,-1.0,-1.0,-1.0,-1.0,-1.0,-1.0,-1.0,-1.0,-1.0]
	navhigh = fltarr(19)
	navhigh = [navmax,10000000.,10000000.,10000000.,10000.,10000.,10000., $
			90.0,180.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0]

	xval = orbit.one_second(idxmin:idxmax).TAI93_time-orbit.one_second(0).TAI93_time
	yval = orbit.one_second(idxmin:idxmax).position_vector(0)
	xlo = navlow(0) & xhi = navhigh(0)
	ylo = navlow(1) & yhi = navhigh(1)
	xname = navelements(0) & yname = navelements(1)
	cval = bytarr(orbit.orbit_summary.one_second_count)
	cval(*) = 64

	navb = widget_base(column=2,group_leader=lisb,title='Navigation data plot')
	navb1 = widget_base(navb,row=4,/align_center)
	navb11 = widget_droplist(navb1,title='X Axis',/align_center,value=navelements,event_pro='navdlxhandler')
	navb12 = widget_droplist(navb1,title='Y Axis',/align_center,value=navelements,event_pro='navdlyhandler')
	navb13 = widget_droplist(navb1,title='Colors',/align_center,value=navcolelements,event_pro='navdlchandler')
	navb14 = widget_base(navb1,column=2,/align_center)
	navb141 = widget_button(navb14,value='Edit & Print',/align_center)
	navb142 = widget_button(navb14,value='Cancel',/align_center)
	navb2 = widget_draw(navb,colors=220,xsize=400,ysize=300,retain=2)
	widget_control,navb11,set_droplist_select=0
	widget_control,navb12,set_droplist_select=1
	widget_control,navb,/realize
	widget_control,navb2,get_value=navwin
	wset,navwin & 	if (not iswindows(0)) then device,pseu=8
	do_nav_plot
	xmanager,'Navigation Plots',navb,event_handler='navhandler'
end

pro navhandler, event
	common navstuff, navelements, navlow, navhigh, navwin, navb
	common sensor_read, this_sensor, file_format
	common datafiles, orbit, fname
	common otd_boom, boom
	common lisbase, lisb
	common lisapp_colors,red,green,blue
	common navvals, xval, yval, xlo, xhi, ylo, yhi, xname, yname, cval
	common navbounds, navmin, navmax

	widget_control, event.id, get_value=buttonvalue
	
	case buttonvalue of
		'Edit & Print' :  begin
				; do_nav_plot
				live_plot,yval,independent=xval
			  end
		'Cancel': begin
				navelements = 0
				widget_control,navb,/destroy
			  end
	endcase
end


pro navdlxhandler, event
	common navstuff, navelements, navlow, navhigh, navwin, navb
	common sensor_read, this_sensor, file_format
	common datafiles, orbit, fname
	common otd_boom, boom
	common lisbase, lisb
	common lisapp_colors,red,green,blue
	common navvals, xval, yval, xlo, xhi, ylo, yhi, xname, yname, cval
	common navbounds, navmin, navmax

	lo = navlow(event.index) & hi = navhigh(event.index)
	name = navelements(event.index)

	case event.index of 
		0: begin
			val = orbit.one_second(navmin:navmax).TAI93_time-orbit.one_second(0).TAI93_time
		   end
		1: begin
			val = orbit.one_second(navmin:navmax).position_vector(0)
		   end
		2: begin	
			val = orbit.one_second(navmin:navmax).position_vector(1)
		   end
		3: begin
			val = orbit.one_second(navmin:navmax).position_vector(2)
		   end
		4: begin
			val = orbit.one_second(navmin:navmax).velocity_vector(0)
		   end
		5: begin
			val = orbit.one_second(navmin:navmax).velocity_vector(1)
		   end
		6: begin
			val = orbit.one_second(navmin:navmax).velocity_vector(2)
		   end
		7: begin
			nadirs = fltarr(2,navmax-navmin+1)
			get_nadir_locations, orbit.one_second(navmin:navmax).TAI93_time, nadirs
			temp = fltarr(navmax-navmin+1)
			temp(*) = nadirs(0,*)
			val = temp
		   end
		8: begin
			nadirs = fltarr(2,navmax-navmin+1)
			get_nadir_locations, orbit.one_second(navmin:navmax).TAI93_time, nadirs
			temp = fltarr(navmax-navmin+1)
			temp(*) = nadirs(1,*)
			val = temp
		   end
		9: begin
		   end
		10: begin
			val = orbit.one_second(navmin:navmax).transform_matrix(0)
		    end
		11: begin
			val = orbit.one_second(navmin:navmax).transform_matrix(1)
		    end
		12: begin
			val = orbit.one_second(navmin:navmax).transform_matrix(2)
		    end
		13: begin
			val = orbit.one_second(navmin:navmax).transform_matrix(3)
		    end
		14: begin
			val = orbit.one_second(navmin:navmax).transform_matrix(4)
		    end
		15: begin
			val = orbit.one_second(navmin:navmax).transform_matrix(5)
		    end
		16: begin
			val = orbit.one_second(navmin:navmax).transform_matrix(6)
		    end
		17: begin
			val = orbit.one_second(navmin:navmax).transform_matrix(7)
		    end
		18: begin
			val = orbit.one_second(navmin:navmax).transform_matrix(8)
		    end
	endcase

	xval = val & xlo = lo & xhi = hi & xname=name

	do_nav_plot

end

pro navdlyhandler, event
	common navstuff, navelements, navlow, navhigh, navwin, navb
	common sensor_read, this_sensor, file_format
	common datafiles, orbit, fname
	common otd_boom, boom
	common lisbase, lisb
	common lisapp_colors,red,green,blue
	common navvals, xval, yval, xlo, xhi, ylo, yhi, xname, yname, cval
	common navbounds, navmin, navmax

	lo = navlow(event.index) & hi = navhigh(event.index)
	name = navelements(event.index)

	case event.index of 
		0: begin
			val = orbit.one_second(navmin:navmax).TAI93_time-orbit.one_second(0).TAI93_time
		   end
		1: begin
			val = orbit.one_second(navmin:navmax).position_vector(0)
		   end
		2: begin	
			val = orbit.one_second(navmin:navmax).position_vector(1)
		   end
		3: begin
			val = orbit.one_second(navmin:navmax).position_vector(2)
		   end
		4: begin
			val = orbit.one_second(navmin:navmax).velocity_vector(0)
		   end
		5: begin
			val = orbit.one_second(navmin:navmax).velocity_vector(1)
		   end
		6: begin
			val = orbit.one_second(navmin:navmax).velocity_vector(2)
		   end
		7: begin
			nadirs = fltarr(2,navmax-navmin+1)
			get_nadir_locations, orbit.one_second(navmin:navmax).TAI93_time, nadirs
			temp = fltarr(navmax-navmin+1)
			temp(*) = nadirs(0,*)
			val = temp
		   end
		8: begin
			nadirs = fltarr(2,navmax-navmin+1)
			get_nadir_locations, orbit.one_second(navmin:navmax).TAI93_time, nadirs
			temp = fltarr(navmax-navmin+1)
			temp(*) = nadirs(1,*)
			val = temp
		   end
		9: begin
		   end
		10: begin
			val = orbit.one_second(navmin:navmax).transform_matrix(0)
		    end
		11: begin
			val = orbit.one_second(navmin:navmax).transform_matrix(1)
		    end
		12: begin
			val = orbit.one_second(navmin:navmax).transform_matrix(2)
		    end
		13: begin
			val = orbit.one_second(navmin:navmax).transform_matrix(3)
		    end
		14: begin
			val = orbit.one_second(navmin:navmax).transform_matrix(4)
		    end
		15: begin
			val = orbit.one_second(navmin:navmax).transform_matrix(5)
		    end
		16: begin
			val = orbit.one_second(navmin:navmax).transform_matrix(6)
		    end
		17: begin
			val = orbit.one_second(navmin:navmax).transform_matrix(7)
		    end
		18: begin
			val = orbit.one_second(navmin:navmax).transform_matrix(8)
		    end
	endcase

	yval = val & ylo = lo & yhi = hi & yname=name

	do_nav_plot

end

pro navdlchandler, event
	common navstuff, navelements, navlow, navhigh, navwin, navb
	common sensor_read, this_sensor, file_format
	common datafiles, orbit, fname
	common otd_boom, boom
	common lisbase, lisb
	common lisapp_colors,red,green,blue
	common navvals, xval, yval, xlo, xhi, ylo, yhi, xname, yname, cval
	common navbounds, navmin, navmax

	if (event.index ne 0) then begin
		lo = navlow(event.index-1) & hi = navhigh(event.index-1)
		name = navelements(event.index-1)
	endif else begin
		lo = 64 & hi = 65 & name = 'Solid'
	endelse

	case event.index of 
		0: begin
		   	val = bytarr(navmax-navmin+1)
			val(*) = 64
		   end
		1: begin
			val = orbit.one_second(navmin:navmax).TAI93_time-orbit.one_second(0).TAI93_time
		   end
		2: begin
			val = orbit.one_second(navmin:navmax).position_vector(0)
		   end
		3: begin	
			val = orbit.one_second(navmin:navmax).position_vector(1)
		   end
		4: begin
			val = orbit.one_second(navmin:navmax).position_vector(2)
		   end
		5: begin
			val = orbit.one_second(navmin:navmax).velocity_vector(0)
		   end
		6: begin
			val = orbit.one_second(navmin:navmax).velocity_vector(1)
		   end
		7: begin
			val = orbit.one_second(navmin:navmax).velocity_vector(2)
		   end
		8: begin
			nadirs = fltarr(2,navmax-navmin+1)
			get_nadir_locations, orbit.one_second(navmin:navmax).TAI93_time, nadirs
			temp = fltarr(navmax-navmin+1)
			temp(*) = nadirs(0,*)
			val = temp
		   end
		9: begin
			nadirs = fltarr(2,navmax-navmin+1)
			get_nadir_locations, orbit.one_second(navmin:navmax).TAI93_time, nadirs
			temp = fltarr(navmax-navmin+1)
			temp(*) = nadirs(1,*)
			val = temp
		   end
		10: begin
		   end
		11: begin
			val = orbit.one_second(navmin:navmax).transform_matrix(0)
		    end
		12: begin
			val = orbit.one_second(navmin:navmax).transform_matrix(1)
		    end
		13: begin
			val = orbit.one_second(navmin:navmax).transform_matrix(2)
		    end
		14: begin
			val = orbit.one_second(navmin:navmax).transform_matrix(3)
		    end
		15: begin
			val = orbit.one_second(navmin:navmax).transform_matrix(4)
		    end
		16: begin
			val = orbit.one_second(navmin:navmax).transform_matrix(5)
		    end
		17: begin
			val = orbit.one_second(navmin:navmax).transform_matrix(6)
		    end
		18: begin
			val = orbit.one_second(navmin:navmax).transform_matrix(7)
		    end
		19: begin
			val = orbit.one_second(navmin:navmax).transform_matrix(8)
		    end
	endcase

	cval = fix(65.+63.*((val-lo)/(hi-lo)))
	; print,lo,min(val),max(val),hi,min(cval),max(cval)
	do_nav_plot

end

pro do_nav_plot
	common navstuff, navelements, navlow, navhigh, navwin, navb
	common sensor_read, this_sensor, file_format
	common datafiles, orbit, fname
	common otd_boom, boom
	common lisbase, lisb
	common lisapp_colors,red,green,blue
	common navvals, xval, yval, xlo, xhi, ylo, yhi, xname, yname, cval
	common navbounds, navmin, navmax

	wset, navwin & 	if (not iswindows(0)) then device,pseu=8
	tvlct,red,green,blue
	; print,'Plotting',xname,' ',yname,' '
	plot,xval,yval,/nodata,xrange=[xlo,xhi],yrange=[ylo,yhi],xtitle=xname,ytitle=yname,/xstyle,/ystyle,color=64,font=0
	plots,xval,yval,psym=3,color=cval
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Pixel Animation Routines ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;

pro animate_events, event, idxmin, idxmax

	common datafiles, orbit, fname
	common lisapp_colors,red,green,blue
	common lisbase, lisb
	common evanim, aeb, aebctl, aebsld1b, aebsld1, aebsld1col, aebdat1, aebsld2b, aebsld2, aebsld2col, $
				aebdat2, aebsld3b, aebsld3, aebdat3, aebprevbut, aebanimbut, aebpausbut, aebdonebut, aebdrw, aeb1, $
				aeb2, aed1, aed2, startidx, startdt, endidx, enddt, currentidx, currentdt
	common evanim3, aebmmfbut
	common nadirlocs, nadirs
	common evanimstatus, status
	common evanim2, aebbgbut
	common evsubcom, baseidxmin, baseidxmax, grinplot, evinplot, grinplotct, evinplotct
	
	baseidxmin = idxmin & baseidxmax = idxmax
	

	
	grinplot = where(orbit.point.lightning.group(*).TAI93_time ge orbit.one_second(idxmin).TAI93_time and $
				 orbit.point.lightning.group(*).TAI93_time le orbit.one_second(idxmax).TAI93_time,grinplotct)
	if (grinplotct gt 1) then begin
	
		sgt = sort(orbit.point.lightning.group(grinplot).TAI93_time)
		startdt = new_datetime_structure(1)
		enddt = new_datetime_structure(1)
		
		startdt.tai93 = orbit.point.lightning.group(grinplot(sgt(0))).TAI93_time
		enddt.tai93   = orbit.point.lightning.group(grinplot(sgt(grinplotct-1))).TAI93_time
	
		
		startidx = long(startdt.tai93-orbit.one_second(0).TAI93_time)
		endidx   = long(enddt.tai93-orbit.one_second(0).TAI93_time)
		currentidx = startidx
		
		baseidxmax = endidx & baseidxmin = startidx
		; print,sgt(0),sgt(orbit.point.point_summary.group_count-1)
		 print,format='((F14.3),(F14.3))',startdt.tai93,enddt.tai93
		; print,startidx,endidx

		aeb = widget_base(column=2,title='Event Animation',group_leader=lisb,ysize=560,scroll=0,/base_align_right)
		aebctl = widget_base(aeb,row=12,xsize=230,/align_left,frame=0)
		aebdrw = widget_base(aeb,row=2,xsize=370,/align_right,frame=0,/base_align_right)
		
		aebinfob1 = widget_base(aebctl,column=2)
		aebdat1 = widget_label(aebinfob1,value='00:00:00.000 UTC')
		aebsld1col = widget_draw(aebinfob1,colors=220,frame=0,xsize=15,ysize=15,retain=2)
		aebsld1b = widget_base(aebctl,column=1,frame=0)
		aebsld1 = widget_slider(aebsld1b,frame=0,xsize=220,minimum=startidx,maximum=endidx,title='Start',event_pro='aesliderupdate')
		
		aebdat3 = widget_label(aebctl,value='00:00:00.000 UTC')
		aebsld3b = widget_base(aebctl,column=1,frame=0)
		aebsld3 = widget_slider(aebsld3b,frame=0,xsize=220,minimum=startidx,maximum=endidx,title='Current')
		widget_control,aebsld3,sensitive=0
		
		aebinfob2 = widget_base(aebctl,column=2)
		aebdat2 = widget_label(aebinfob2,value='00:00:00.000 UTC')
		aebsld2col = widget_draw(aebinfob2,colors=220,frame=0,xsize=15,ysize=15,retain=2)
		aebsld2b = widget_base(aebctl,column=1,frame=0)
		aebsld2 = widget_slider(aebsld2b,frame=0,xsize=220,minimum=startidx,maximum=endidx,title='End',event_pro='aesliderupdate')
		
		aebprevbut	=	widget_button(aebctl,value='Preview',xsize=220)
		aebanimbut = 	widget_button(aebctl,value='Animate',xsize=220)
		;aebbgbut 	=	widget_button(aebctl,value='--> Geographic Plot',xsize=220)
		;aebmmfbut = widget_button(aebctl,value='--> Make Movie Frames',xsize=220)
		aebpausbut =	widget_button(aebctl,value='Reset',xsize=220)
		aebdonebut	=	widget_button(aebctl,value='Done',xsize=220)
		
		aeb2 = widget_base(aebdrw,column=1,/align_center,/base_align_center,xsize=360,ysize=180)
		aeb1 = widget_base(aebdrw,column=1,/align_center,/base_align_center,xsize=360,ysize=360)
		aed2 = widget_draw(aeb2,/align_center,colors=220,xsize=360,ysize=180,retain=2,frame=0)
		aed1 = widget_draw(aeb1,colors=220,/align_center,xsize=360,ysize=360,retain=2,frame=0)
		
		widget_control,aeb,/realize
		Widget_Control, aed1, get_value=aed1_id
		widget_control, aed2, get_value=aed2_id
		widget_control, aebsld2, set_value=endidx
	
		calc_datetime,startdt
		widget_control,aebdat1,set_value=string(format='((I2.2),":",(I2.2),":",(F6.3)," UTC")',startdt.utc.hour,startdt.utc.minute,startdt.utc.second)
		widget_control,aebsld1,set_value=startidx
		
		calc_datetime,enddt
		widget_control,aebdat2,set_value=string(format='((I2.2),":",(I2.2),":",(F6.3)," UTC")',enddt.utc.hour,enddt.utc.minute,enddt.utc.second)
		widget_control,aebsld2,set_value=endidx
		
		widget_control,aebdat3,set_value=string(format='((I2.2),":",(I2.2),":",(F6.3)," UTC")',startdt.utc.hour,startdt.utc.minute,startdt.utc.second)
		widget_control,aebsld3,set_value=startidx
		
		; widget_control,aebbgbut,sensitive=0
		
		wset,aed2_id &	if (not iswindows(0)) then device,pseu=8
		tvlct,red,green,blue
		map_set,/continents,limit=[-90,-180,90,180],xmargin=[0,0],ymargin=[0,0],/noerase,/noborder,con_color=48
	
		nadirs = fltarr(2,orbit.orbit_summary.one_second_count)
		get_nadir_locations, orbit.one_second(*).TAI93_time, nadirs
	
		times = long(orbit.point.lightning.group(grinplot).TAI93_time-orbit.one_second(0).TAI93_time)
		lons=fltarr(grinplotct) & lats=fltarr(grinplotct)
		lons(0:grinplotct-1) = nadirs(1,times) 
		lats(0:grinplotct-1) = nadirs(0,times)
		pix_colors=fix(65.+63.*(float(times-startidx)/float(endidx-startidx+1)))
		; print,min(pix_colors),max(pix_colors)
		plots,lons,lats,psym=4,color=pix_colors
		
		wset,aed1_id & if (not iswindows(0)) then device,pseu=8 & 	erase,63
		usersym,sin(findgen(21)/10.*!pi),cos(findgen(21)/10.*!pi),/fill
		uscale = usersym_scale(0)
		tvlct,red,green,blue
		evinplot = where(orbit.point.lightning.event(*).TAI93_time ge orbit.one_second(startidx).TAI93_time and $
					 orbit.point.lightning.event(*).TAI93_time le orbit.one_second(endidx).TAI93_time,evinplotct)
		deltinplot = orbit.one_second[endidx].TAI93_time-orbit.one_second[startidx].TAI93_time
		; print,deltinplot,startidx,endidx
		pix_colors = ((63.*((orbit.point.lightning.event(evinplot).TAI93_time-orbit.one_second[startidx].TAI93_time) / $
				  deltinplot))+65.)
		plot,intarr(2),intarr(2),xrange=[0,127],yrange=[0,127],position=[0,0,1,1],/nodata,xstyle=1,ystyle=1,xmargin=[0,0],ymargin=[0,0],/noerase
		ssize = .05+fix(720/200)*.10*uscale
		plots, orbit.point.lightning.event(evinplot).x_pixel, orbit.point.lightning.event(evinplot).y_pixel, psym=8, $
			  color=pix_colors, symsize=ssize
	
		aesliderupdate
		widget_control,aebsld1,set_value=startidx+1
		aesliderupdate
		widget_control,aebsld1,set_value=startidx-1
		aesliderupdate
		wset,aed1_id
		xmanager,'event_animation_controls',aeb,event_handler='ae_handler',/no_block
	endif
end

pro ae_handler, event
	common datafiles, orbit, fname
	common evanim, aeb, aebctl, aebsld1b, aebsld1, aebsld1col, aebdat1, aebsld2b, aebsld2, aebsld2col, $
				aebdat2, aebsld3b, aebsld3, aebdat3, aebprevbut,  aebanimbut, aebpausbut, aebdonebut, aebdrw, aeb1, $
				aeb2, aed1, aed2, startidx, startdt, endidx, enddt, currentidx, currentdt
	common evanimstatus, status
	common nadirlocs, nadirs
	common lisapp_colors, red,green,blue
	common evanim2, aebbgbut
	common evanim3, aebmmfbut
	common lisbase, lisb
	common evsubcom, baseidxmin, baseidxmax, grinplot, evinplot, grinplotct, evinplotct

	widget_control, event.id, get_value=buttonvalue
	; print,buttonvalue
	
	case buttonvalue of 
		'Done': begin
				grinplot = 0 & evinplot = 0 & nadirs = 0
				widget_control,aeb,/destroy
			   end
		'Animate': begin
				;widget_control,aed2,get_value=aed2_id
				;wset,aed2_id &	if (not iswindows(0)) then device,pseu=8
				;tvlct,red,green,blue
				;map_set,/continents,limit=[-90,-180,90,180],xmargin=[0,0],ymargin=[0,0],/noborder
				; widget_control,aebanimbut,set_value='Resume'
				widget_control,aebprevbut,sensitive=0
				widget_control,aebanimbut,sensitive=0
				widget_control,aebpausbut,sensitive=0
				widget_control,aebdonebut,sensitive=0
				; widget_control,aebbgbut,sensitive=0
				widget_control,aebsld1,sensitive=0
				widget_control,aebsld2,sensitive=0
				widget_control,aebsld1,get_value=startidx
				widget_control,aebsld2,get_value=endidx
				widget_control,aebsld3,get_value=currentidx
				do_event_animation
				widget_control,aebprevbut,/sensitive
				widget_control,aebsld1,/sensitive
				widget_control,aebsld2,/sensitive
				widget_control,aebdonebut,/sensitive
				widget_control,aebpausbut,/sensitive
				widget_control,aebanimbut,/sensitive
				; widget_control,aebbgbut,/sensitive
				; widget_control,aebbgbut,/sensitive
				currentidx=startidx
			    end
		'Reset': begin
				widget_control,aed2,get_value=aed2_id
				sgt = sort(orbit.point.lightning.group(grinplot).TAI93_time)
				startdt = new_datetime_structure(1)
				enddt = new_datetime_structure(1)
				startdt.tai93 = orbit.point.lightning.group(grinplot(sgt(0))).TAI93_time
				enddt.tai93   = orbit.point.lightning.group(grinplot(sgt(grinplotct-1))).TAI93_time
				startidx = long(startdt.tai93-orbit.one_second(0).TAI93_time)
				endidx   = long(enddt.tai93-orbit.one_second(0).TAI93_time)
				currentidx = baseidxmin
				calc_datetime,startdt
				widget_control,aebdat1,set_value=string(format='((I2.2),":",(I2.2),":",(F6.3)," UTC")',startdt.utc.hour,startdt.utc.minute,startdt.utc.second)
				widget_control,aebsld1,set_value=baseidxmin
				calc_datetime,enddt
				widget_control,aebdat2,set_value=string(format='((I2.2),":",(I2.2),":",(F6.3)," UTC")',enddt.utc.hour,enddt.utc.minute,enddt.utc.second)
				widget_control,aebsld2,set_value=baseidxmax
				widget_control,aebdat3,set_value=string(format='((I2.2),":",(I2.2),":",(F6.3)," UTC")',startdt.utc.hour,startdt.utc.minute,startdt.utc.second)
				widget_control,aebsld3,set_value=baseidxmin
				wset,aed2_id &	if (not iswindows(0)) then device,pseu=8
				tvlct,red,green,blue
				map_set,/continents,limit=[-90,-180,90,180],xmargin=[0,0],ymargin=[0,0],/noborder,con_color=48
				nadirs = fltarr(2,orbit.orbit_summary.one_second_count)
				get_nadir_locations, orbit.one_second(*).TAI93_time, nadirs
				times = long(orbit.point.lightning.group(grinplot).TAI93_time-orbit.one_second(0).TAI93_time)
				lons=fltarr(grinplotct) & lats=fltarr(grinplotct)
				lons(0:grinplotct-1) = nadirs(1,times) 
				lats(0:grinplotct-1) = nadirs(0,times)
				pix_colors=fix(65.+63.*(float(times-baseidxmin)/float(baseidxmax-baseidxmin+1)))
				; print,min(pix_colors),max(pix_colors)
				plots,lons,lats,psym=4,color=pix_colors
				
				widget_control,aed1,get_value=aed1_id
				wset,aed1_id &	if (not iswindows(0)) then device,pseu=8 & erase,63
				tvlct,red,green,blue
				usersym,sin(findgen(21)/10.*!pi),cos(findgen(21)/10.*!pi),/fill
				uscale = usersym_scale(0)
				evinplot = where(orbit.point.lightning.event(*).TAI93_time ge orbit.one_second(baseidxmin).TAI93_time and $
							 orbit.point.lightning.event(*).TAI93_time le orbit.one_second(baseidxmax).TAI93_time,evinplotct)
				deltinplot = orbit.one_second[baseidxmax].TAI93_time-orbit.one_second[baseidxmin].TAI93_time
				; print,deltinplot,idxmin,idxmax
				pix_colors = ((63.*((orbit.point.lightning.event(evinplot).TAI93_time-orbit.one_second[baseidxmin].TAI93_time) / $
						  deltinplot))+65.)
				plot,intarr(2),intarr(2),xrange=[0,127],yrange=[0,127],position=[0,0,1,1],/nodata,xstyle=1,ystyle=1,xmargin=[0,0],ymargin=[0,0],/noerase
				ssize = .05+fix(720/200)*.10*uscale
				plots, orbit.point.lightning.event(evinplot).x_pixel, orbit.point.lightning.event(evinplot).y_pixel, psym=8, $
					  color=pix_colors, symsize=ssize

				aesliderupdate
				widget_control,aebsld1,set_value=baseidxmin+1
				aesliderupdate
				widget_control,aebsld1,set_value=baseidxmin
				aesliderupdate
				widget_control,aebsld2,set_value=baseidxmax-1
				aesliderupdate
				widget_control,aebsld2,set_value=baseidxmax
				aesliderupdate
				end
		'Preview':begin
					do_event_anim_preview
					; widget_control,aebbgbut,/sensitive
					; widget_control,aebmmfbut,/sensitive
				end
;		'--> Geographic Plot':begin
;					allbginfo = orbit.point.bg_summary(*)
;					evinrange = where((orbit.point.lightning.event(*).TAI93_time ge startdt.tai93) and $
;								  (orbit.point.lightning.event(*).TAI93_time le enddt.tai93),evinrangect)
;					if (evinrangect gt 0) then begin
;						firstbg = min(orbit.point.lightning.event(evinrange).TAI93_time)
;						lastbg = max(orbit.point.lightning.event(evinrange).TAI93_time)
;						bginrange = where((orbit.point.bg_summary(*).TAI93_time ge firstbg) and $
;									 (orbit.point.bg_summary(*).TAI93_time le lastbg),bginrangect)
;						if (bginrangect gt 0) then begin
;							bgs_selected = bytarr(orbit.point.point_summary.bg_count)
;							bgs_selected(*) = 0
;							bgs_selected(bginrange) = 1
;							plot_bgs,allbginfo,bgs_selected,'animate_events'
;						endif
;					endif
;				end
;		'--> Make Movie Frames':begin
;					movie_prefs,lisb,startdt.tai93,enddt.tai93
;				end
				
	endcase
end

pro aesliderupdate, event
	common datafiles, orbit, fname
	common evanim, aeb, aebctl, aebsld1b, aebsld1, aebsld1col, aebdat1, aebsld2b, aebsld2, aebsld2col, $
				aebdat2, aebsld3b, aebsld3, aebdat3, aebprevbut,  aebanimbut, aebpausbut, aebdonebut, aebdrw, aeb1, $
				aeb2, aed1, aed2, startidx, startdt, endidx, enddt, currentidx, currentdt
	common evanimstatus, status
	common nadirlocs, nadirs
	common lisapp_colors, red,green,blue
	common evsubcom, baseidxmin, baseidxmax, grinplot, evinplot, grinplotct, evinplotct
	
	widget_control,aebsld1,get_value=new_start_idx
	widget_control,aebsld2,get_value=new_end_idx
	
	if (new_start_idx ne startidx) then begin
		startdt = new_datetime_structure(1)
		startdt.tai93 = double(orbit.one_second(0).tai93_time)+double(new_start_idx)
		calc_datetime,startdt
		startidx = new_start_idx
		widget_control,aebdat1,set_value=string(format='((I2.2),":",(I2.2),":",(F6.3)," UTC")',startdt.utc.hour,startdt.utc.minute,startdt.utc.second)
		pixcol=fix(65.+63.*(float(startidx-baseidxmin)/float(baseidxmax-baseidxmin+1)))
		startbox = bytarr(15,15)
		startbox(*,*)=pixcol
		widget_control,aebsld1col,get_value=drawwin
		wset,drawwin & tvlct, red,green,blue & tv,startbox
		if (startidx gt endidx) then begin
			endidx = startidx
			enddt = new_datetime_structure(1)
			enddt.tai93 = double(orbit.one_second(0).tai93_time)+double(endidx)
			calc_datetime,enddt
			widget_control,aebdat2,set_value=string(format='((I2.2),":",(I2.2),":",(F6.3)," UTC")',enddt.utc.hour,enddt.utc.minute,enddt.utc.second)
			widget_control,aebsld2,set_value=endidx
			pixcol=fix(65.+63.*(float(endidx-baseidxmin)/float(baseidxmax-baseidxmin+1)))
			endbox = bytarr(15,15)
			endbox(*,*)=pixcol
			widget_control,aebsld2col,get_value=drawwin
			wset,drawwin & tvlct, red,green,blue & tv,endbox
		endif
	endif else begin
		if (new_end_idx ne endidx) then begin
			enddt = new_datetime_structure(1)
			enddt.tai93 = double(orbit.one_second(0).tai93_time)+double(new_end_idx)
			calc_datetime,enddt
			endidx = new_end_idx
			widget_control,aebdat2,set_value=string(format='((I2.2),":",(I2.2),":",(F6.3)," UTC")',enddt.utc.hour,enddt.utc.minute,enddt.utc.second)
			pixcol=fix(65.+63.*(float(endidx-baseidxmin)/float(baseidxmax-baseidxmin+1)))
			endbox = bytarr(15,15)
			endbox(*,*)=pixcol
			widget_control,aebsld2col,get_value=drawwin
			wset,drawwin & tvlct, red,green,blue & tv,endbox
			if (endidx lt new_start_idx) then begin
				startidx = endidx
				startdt = new_datetime_structure(1)
				startdt.tai93 = double(orbit.one_second(0).tai93_time)+double(startidx)
				calc_datetime,startdt
				widget_control,aebdat1,set_value=string(format='((I2.2),":",(I2.2),":",(F6.3)," UTC")',startdt.utc.hour,startdt.utc.minute,startdt.utc.second)
				widget_control,aebsld1,set_value=startidx
				pixcol=fix(65.+63.*(float(startidx-baseidxmin)/float(baseidxmax-baseidxmin+1)))
				startbox = bytarr(15,15)
				startbox(*,*)=pixcol
				widget_control,aebsld1col,get_value=drawwin
				wset,drawwin & tvlct, red,green,blue & tv,startbox
			endif
		endif
	endelse
	currentidx=startidx
	currentdt = startdt
	widget_control,aebdat3,set_value=string(format='((I2.2),":",(I2.2),":",(F6.3)," UTC")',startdt.utc.hour,startdt.utc.minute,startdt.utc.second)
	widget_control,aebsld3,set_value=currentidx
end

pro do_event_animation, current
	common datafiles, orbit, fname
	common evanim, aeb, aebctl, aebsld1b, aebsld1, aebsld1col, aebdat1, aebsld2b, aebsld2, aebsld2col, $
				aebdat2, aebsld3b, aebsld3, aebdat3, aebprevbut,  aebanimbut, aebpausbut, aebdonebut, aebdrw, aeb1, $
				aeb2, aed1, aed2, startidx, startdt, endidx, enddt, currentidx, currentdt
	common evanimstatus, status
	common nadirlocs, nadirs
	common lisapp_colors,red,green,blue
	common lisapp_prefs, app_prefs
	
	widget_control,aed1,get_value=aed1_id
	widget_control,aed2,get_value=aed2_id
	wset,aed1_id  &	if (not iswindows(0)) then device,pseu=8 & erase,63
	tvlct,red,green,blue
	sleepfac=.0001
	persist = app_prefs.graphics.persistence
	usersym,sin(findgen(21)/10.*!pi),cos(findgen(21)/10.*!pi),/fill
	uscale = usersym_scale(0)

	
	sgt = sort(orbit.point.lightning.group(*).TAI93_time)
	plot,findgen(2),findgen(2),/nodata,xrange=[0,127],yrange=[0,127],xstyle=1,ystyle=1,xmargin=[0,0],ymargin=[0,0],/noerase
	
	plot_p = !P & plot_x = !X & plot_y = !Y & plot_z = !Z & plot_D = !D
	
	
	;for i=long(1),orbit.point.point_summary.group_count-1 do begin
	
	inanim = where((orbit.point.lightning.group(sgt(*)).tai93_time ge startdt.tai93) and $
			    (orbit.point.lightning.group(sgt(*)).tai93_time le enddt.tai93),incount)
	if (incount gt 0) then begin
		einanim = where((orbit.point.lightning.event(*).tai93_time ge startdt.tai93) and $
					(orbit.point.lightning.event(*).tai93_time le enddt.tai93))
		colorbase = orbit.point.lightning.event(*).radiance
		cbmin = alog10(max([min( colorbase(where(colorbase ne 0))  ),4e4]))
		cbmax = alog10(max([max( colorbase(where(colorbase ne 0))  ),4e4]))
		cbdel = cbmax-cbmin

		maxlat=min([max(orbit.point.lightning.event(einanim).location(0))+2.,90.0])
		maxlon=min([max(orbit.point.lightning.event(einanim).location(1))+2.,180.0])
		minlat=max([min(orbit.point.lightning.event(einanim).location(0))-2.,-90.0])
		minlon=max([min(orbit.point.lightning.event(einanim).location(1))-2.,-180.0])
		latdel = maxlat-minlat
		londel = maxlon-minlon
		if (londel gt 2.*latdel) then begin
			newlatdel = .5*londel
			latpad = .5*(newlatdel-latdel)
			minlat = minlat - latpad
			maxlat = maxlat + latpad
		endif else begin
			newlondel = 2.*latdel
			lonpad = .5*(newlondel-londel)
			minlon = minlon - lonpad
			maxlon = maxlon + lonpad
		endelse
		wset,aed2_id &	if (not iswindows(0)) then device,pseu=8
		tvlct,red,green,blue
		map_set,/continents,limit=[minlat,minlon,maxlat,maxlon],/isotropic,xmargin=[0,0],ymargin=[0,0],con_color=48
		map_p = !P & map_x = !X & map_y = !Y & map_Z = !Z & map_D = !D
		wset,aed1_id &	if (not iswindows(0)) then device,pseu=8
		!P = plot_p & !X = plot_x & !Y = plot_y & !Z = plot_z 

		startidx = inanim(0)
		endidx = inanim(incount-1)
		i = currentidx
		
		;while (status ne 'pause') do begin
		; print,inanim(0),inanim(incount-1),incount,"***"
		ssize = .05+fix(720/200)*.10*uscale
		for i=inanim(0),inanim(incount-1),1 do begin
			onesecidx = long(orbit.point.lightning.group(sgt(i)).TAI93_time-orbit.one_second(0).TAI93_time)
			widget_control, aebsld3, set_value=onesecidx
			dt = new_datetime_structure(1)
			dt.tai93 = orbit.point.lightning.group(sgt(i)).TAI93_time
			calc_datetime,dt
			current_time = string(format='((I2.2),":",(I2.2),":",(F6.3)," UTC")',dt.utc.hour,dt.utc.minute,dt.utc.second)
			widget_control,aebdat3,set_value=current_time
			
			; In pixarray window
			
			wset,aed1_id & 	if (not iswindows(0)) then device,pseu=8
			
			; Turn off persistent groups
			
			!P = plot_p & !X = plot_x & !Y = plot_y & !Z = plot_z 
			if (i ge persist) then begin
				chad = orbit.point.lightning.group(sgt(i-persist)).child_address
				for j = 0, orbit.point.lightning.group(sgt(i-persist)).child_count-1, 1 do begin
					plots,orbit.point.lightning.event(chad+j).x_pixel, $
						 orbit.point.lightning.event(chad+j).y_pixel, $
						 color=48,psym=8,symsize=ssize
				endfor
			endif
			
			; wset,aed1_id & 	if (not iswindows(0)) then device,pseu=8
			;!P = plot_p & !X = plot_x & !Y = plot_y & !Z = plot_z 
			
			; Turn on next group
			
			chad = orbit.point.lightning.group(sgt(i)).child_address
			for j = 0, orbit.point.lightning.group(sgt(i)).child_count-1,1 do begin
				;pix_colors = ((64.*((orbit.point.lightning.event(chad+j).TAI93_time-orbit.one_second(0).TAI93_time) / $
				;	(orbit.orbit_summary.TAI93_end-orbit.one_second(0).TAI93_time)))+64.)
				usecolor = fix(65.+63.*(float(orbit.point.lightning.event(chad+j).amplitude)/128.))
				plots,orbit.point.lightning.event(chad+j).x_pixel, $
					 orbit.point.lightning.event(chad+j).y_pixel, $
					 color=usecolor,psym=8,symsize=ssize
			endfor
			
			; In map window
			
			wset,aed2_id & 	if (not iswindows(0)) then device,pseu=8
			!P = map_p & !X = map_x & !Y = map_y & !Z = map_z 		
			map_set,limit=[minlat,minlon,maxlat,maxlon],/isotropic,xmargin=[0,0],ymargin=[0,0],/noerase,/noborder,con_color=48
			
			; Turn on this group
						
			for j = 0, orbit.point.lightning.group(sgt(i)).child_count-1,1 do begin
				; usecolor = fix(65.+63.*(alog10(orbit.point.lightning.event(chad+j).radiance)-cbmin)/cbdel)
				usecolor = fix(65.+63.*(float(orbit.point.lightning.event(chad+j).amplitude)/128.))
				plots,orbit.point.lightning.event(chad+j).location(1), $
					orbit.point.lightning.event(chad+j).location(0), $
					color=usecolor,psym=8,symsize=ssize

			endfor
			
			; Add a delay if requested
			
			if (app_prefs.graphics.delay gt 0) then begin
				wait, float(app_prefs.graphics.delay)*.0001
			endif
			
			; wset,aed2_id & 	if (not iswindows(0)) then device,pseu=8
			; !P = map_p & !X = map_x & !Y = map_y & !Z = map_z 
			; map_set,limit=[minlat,minlon,maxlat,maxlon],/isotropic,xmargin=[0,0],ymargin=[0,0],/noerase,/noborder, con_color=48
	
			thistime = long(orbit.point.lightning.group(sgt(i)).TAI93_time-orbit.one_second(0).TAI93_time)
			if ((thistime ge 0) and (thistime lt orbit.orbit_summary.one_second_count)) then begin
				lons=fltarr(1) & lats=fltarr(1)
				lons(0) = nadirs(1,thistime) & lats(0) = nadirs(0,thistime)
				oplot,lons,lats,psym=3,color=pix_colors
			endif
			;if (i gt endidx) then begin
			;	 turn off any persistent events
			;	evinanim = where((orbit.point.lightning.event(*).tai93_time ge startdt.tai93) and $
			;    (orbit.point.lightning.event(*).tai93_time le enddt.tai93),incount)
			;	wset,aed1_id
			;	!P = plot_p & !X = plot_x & !Y = plot_y & !Z = plot_z 
			;	plots,orbit.point.lightning.event(evinanim).x_pixel, $
			;		orbit.point.lightning.event(evinanim).y_pixel,color=0,psym=1
			;	 loop over again
			;	 i = startidx
			;endif
		endfor
	endif
end

pro do_event_anim_preview, current
	common datafiles, orbit, fname
	common evanim, aeb, aebctl, aebsld1b, aebsld1, aebsld1col, aebdat1, aebsld2b, aebsld2, aebsld2col, $
				aebdat2, aebsld3b, aebsld3, aebdat3, aebprevbut,  aebanimbut, aebpausbut, aebdonebut, aebdrw, aeb1, $
				aeb2, aed1, aed2, startidx, startdt, endidx, enddt, currentidx, currentdt
	common evanimstatus, status
	common nadirlocs, nadirs
	common lisapp_colors,red,green,blue
	common evsubcom, baseidxmin, baseidxmax, grinplot, evinplot, grinplotct, evinplotct
		
	widget_control,aed1,get_value=aed1_id
	widget_control,aed2,get_value=aed2_id
	wset,aed1_id &	if (not iswindows(0)) then device,pseu=8 & erase,63
	tvlct,red,green,blue
	usersym,sin(findgen(21)/10.*!pi),cos(findgen(21)/10.*!pi),/fill
	uscale = usersym_scale(0)
      ssize = .05+fix(720/200)*.10*uscale
	
	sgt = sort(orbit.point.lightning.group(grinplot).TAI93_time)
	plot,findgen(2),findgen(2),/nodata,xrange=[0,127],yrange=[0,127],xstyle=1,ystyle=1,xmargin=[0,0],ymargin=[0,0],/noerase
	
	plot_p = !P & plot_x = !X & plot_y = !Y & plot_z = !Z & plot_D = !D
	
	inanim = where((orbit.point.lightning.group(grinplot(sgt(*))).tai93_time ge startdt.tai93) and $
			    (orbit.point.lightning.group(grinplot(sgt(*))).tai93_time le enddt.tai93),incount)
	if (incount gt 0) then begin
		sevt = sort(orbit.point.lightning.event(evinplot).TAI93_time)
		einanim = where((orbit.point.lightning.event(evinplot(sevt(*))).tai93_time ge startdt.tai93) and $
					(orbit.point.lightning.event(evinplot(sevt(*))).tai93_time le enddt.tai93),eincount)
		if (eincount gt 0) then begin
			;colorbase = orbit.point.lightning.event(evinplot(sevt(einanim(*)))).radiance
			;cbmin = alog10(max([min( colorbase(where(colorbase ne 0))  ),4e4]))
			;cbmax = alog10(max([max( colorbase(where(colorbase ne 0))  ),4e4]))
			;cbdel = cbmax-cbmin
	
			maxlat=min([max(orbit.point.lightning.event(evinplot(sevt(einanim))).location(0))+2.,90.0])
			maxlon=min([max(orbit.point.lightning.event(evinplot(sevt(einanim))).location(1))+2.,180.0])
			minlat=max([min(orbit.point.lightning.event(evinplot(sevt(einanim))).location(0))-2.,-90.0])
			minlon=max([min(orbit.point.lightning.event(evinplot(sevt(einanim))).location(1))-2.,-180.0])
			latdel = maxlat-minlat
			londel = maxlon-minlon
			if (londel gt 2.*latdel) then begin
				newlatdel = .5*londel
				latpad = .5*(newlatdel-latdel)
				minlat = minlat - latpad
				maxlat = maxlat + latpad
			endif else begin
				newlondel = 2.*latdel
				lonpad = .5*(newlondel-londel)
				minlon = minlon - lonpad
				maxlon = maxlon + lonpad
			endelse
			wset,aed2_id &	if (not iswindows(0)) then device,pseu=8
			tvlct,red,green,blue
			map_set,/continents,limit=[minlat,minlon,maxlat,maxlon],/isotropic,xmargin=[0,0],ymargin=[0,0],con_color=48
			map_p = !P & map_x = !X & map_y = !Y & map_Z = !Z & map_D = !D
			usecolor = fix(65.+63.*(orbit.point.lightning.event(evinplot(sevt(einanim))).amplitude/127.))
			plots,orbit.point.lightning.event(evinplot(sevt(einanim))).location(1), $
				orbit.point.lightning.event(evinplot(sevt(einanim))).location(0), $
				color=usecolor,psym=4
	
			wset,aed1_id &	if (not iswindows(0)) then device,pseu=8
			!P = plot_p & !X = plot_x & !Y = plot_y & !Z = plot_z 
			pix_colors = ((64.*((orbit.point.lightning.event(evinplot(sevt(einanim))).TAI93_time-orbit.one_second[baseidxmin].TAI93_time) / $
				(orbit.one_second[baseidxmax].TAI93_time-orbit.one_second[baseidxmin].TAI93_time)))+64.)
			plots,orbit.point.lightning.event(evinplot(sevt(einanim))).x_pixel, $
				 orbit.point.lightning.event(evinplot(sevt(einanim))).y_pixel, $
				 color=pix_colors,psym=8,symsize=ssize
		endif
	endif
end

function event_anim_poll
	common evanim, aeb, aebctl, aebsld1b, aebsld1, aebsld1col, aebdat1, aebsld2b, aebsld2, aebsld2col, $
				aebdat2, aebsld3b, aebsld3, aebdat3, aebprevbut,  aebanimbut, aebpausbut, aebdonebut, aebdrw, aeb1, $
				aeb2, aed1, aed2, startidx, startdt, endidx, enddt, currentidx, currentdt
	common evanimstatus, status

	return,status
end
	
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Thresholds and Glint Routines ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

pro examine_bgs, event, first, last
	common sensor_read, this_sensor, file_format
	common datafiles, orbit, fname
	common lisbase, lisb
	common lisapp_colors,red,green,blue
	common exbgcommon3, calidx, maxrads, count_to_rad, rad_to_count, exbgb, exbg_map_id, exbg_cal_id, exbg_raw_id, exbg_ev_id, exbg_glint_id, $
 						dc, bgidx, ctmaskval, radmaskval, event_window, exbgctl_cttab, exbgctl_radtab, $
 						exbgctl_bgsld, exbgctl_evsld, exbgctl_ctsld, exbgctl_rasld, exbgctl_bgsld_tai, inbg, calbg, exbgctl_donebut, exbgctl_glintbut
	common exbgcommon4, exbgctl_xlab,exbgctl_ylab,exbgctl_ctlab,exbgctl_radlab,exbgctl_evlab,exbgctl_glintlab,evgrid,glintgrid,exbgd_raw,exbgd_cal,exbgd_ev,exbgd_glint
	common prefs, readparams, readparamsotd, setparams, setparamsotd, gotparams, gotparamsotd
	common exbgcommon5, exbgctl_threshbut
	common exbgcommon6, exbgctl_evbgbut,maxvals
	common exbgcommon7, exbgctl_evtabbut
	common exbgcommon8, exbgd_calpd, exbgd_rawpd, exbgd_evpd, exbgd_glintpd
	common exbgcommon9, exbgctl_pixinfo6
	common bg_info, fnamestub, candobgs, bgname, bgpathtouse
	common bg_info2, firstbg, lastbg

	if (this_sensor eq 'otd') then begin
		calidx = 0
	endif else begin
		calidx = 1
	endelse
	
	win1elements = strarr(1)
	win2elements = strarr(1)
	win3elements = strarr(1)
	win4elements = strarr(4)
	win1elements = ['Calibrated Radiance']
	win2elements = ['12-bit Counts']
	win3elements = ['Events in Window']
	win4elements = ['(Choose...)','Glint','Thresholds','Histograms']
	
	maxrads = [520,460]
	emptarr = intarr(128,128)
	emptarr2 = fltarr(128,128)
	exbgb = widget_base(group_leader=lisb,column=2,title='Examine Background Images')
	exbgctl = widget_base(exbgb,row=10,/base_align_center)
	exbgdc  = widget_base(exbgb,column=2,/grid_layout)
	exbgdr1  = widget_base(exbgdc,row=4)
	exbgdr2  = widget_base(exbgdc,row=4)
	exbgd_calpd   = widget_droplist(exbgdr1,/align_center,value=win1elements)
	exbgd_cal     = widget_draw(exbgdr1,colors=220,xsize=256,ysize=256,/motion_events,retain=2)
	exbgd_rawpd   = widget_droplist(exbgdr2,/align_center,value=win2elements)
	exbgd_raw     = widget_draw(exbgdr2,colors=220,xsize=256,ysize=256,/motion_events,retain=2)
	exbgd_evpd    = widget_droplist(exbgdr1,/align_center,value=win3elements)
	exbgd_ev      = widget_draw(exbgdr1,colors=220,xsize=256,ysize=256,/motion_events,retain=2)
	exbgd_glintpd = widget_droplist(exbgdr2,/align_center,value=win4elements)
	exbgd_glint   = widget_draw(exbgdr2,colors=220,xsize=256,ysize=256,/motion_events,retain=2)
	exbgctl_map = widget_draw(exbgctl,colors=220,xsize=256,ysize=128,retain=2)
	exbgctl_bgsld_tai = widget_label(exbgctl,value='00:00:00.000 UTC')
	exbgctl_bgsld = widget_slider(exbgctl,xsize=256,title='Background Image Number',minimum=first,maximum=last)
	exbgctl_evsld = widget_slider(exbgctl,xsize=256,title='Event Window (sec)',minimum=0,maximum=60)
	exbgctl_ctsld = widget_slider(exbgctl,xsize=256,title='12-bit Count Mask',minimum=0,maximum=4096)
	exbgctl_rasld = widget_slider(exbgctl,xsize=256,title='BG Radiance Mask',minimum=0,maximum=maxrads(calidx))
;	exbgctl_cttab = widget_table(exbgctl,value=emptarr,alignment=1,/scroll,/resizeable_columns,scr_xsize=256,scr_ysize=64,xsize=3,ysize=3,format='(F4.0)',/no_header)
;	exbgctl_radtab = widget_table(exbgctl,value=emptarr2,alignment=1,/scroll,/resizeable_columns,scr_xsize=256,scr_ysize=64,xsize=3,ysize=3,format='(F5.1)',/no_header)
	exbgctl_pixinfob = widget_base(exbgctl,column=2,/base_align_center,/grid_layout)
	exbgctl_pixinfolabs = widget_base(exbgctl_pixinfob,row=6,/base_align_left,/grid_layout)
	exbgctl_pixinfo1 = widget_label(exbgctl_pixinfolabs,value='X Pixel')
	exbgctl_pixinfo2 = widget_label(exbgctl_pixinfolabs,value='Y Pixel')
	exbgctl_pixinfo3 = widget_label(exbgctl_pixinfolabs,value='Radiance')
	exbgctl_pixinfo4 = widget_label(exbgctl_pixinfolabs,value='12-bit count')
	exbgctl_pixinfo5 = widget_label(exbgctl_pixinfolabs,value='Events')
	exbgctl_pixinfo6 = widget_label(exbgctl_pixinfolabs,value='            ')
	exbgctl_pixinfovals = widget_base(exbgctl_pixinfob,row=6,/base_align_center,/grid_layout)
	exbgctl_xlab = widget_label(exbgctl_pixinfovals,  value='----')
	exbgctl_ylab = widget_label(exbgctl_pixinfovals,  value='----')
	exbgctl_radlab = widget_label(exbgctl_pixinfovals,value='------')
	exbgctl_ctlab = widget_label(exbgctl_pixinfovals, value='-----')
	exbgctl_evlab = widget_label(exbgctl_pixinfovals, value='-----')
	exbgctl_glintlab = widget_label(exbgctl_pixinfovals,value='      ')
	exbgctl_evtabbut = widget_button(exbgctl,xsize=128,value='Event Table')

	exbgctl_donebut = widget_button(exbgctl,xsize=128,value='Done')
	
	
	widget_control,exbgb,/realize


	widget_control,exbgctl_map,get_value=exbg_map_id
	widget_control,exbgd_cal,get_value=exbg_cal_id
	widget_control,exbgd_raw,get_value=exbg_raw_id
	widget_control,exbgd_ev,get_value=exbg_ev_id
	widget_control,exbgd_glint,get_value=exbg_glint_id
	widget_control,exbgd_glintpd,set_uvalue=0
	wset,exbg_map_id
	tvlct,red,green,blue
	map_set,/continents,xmargin=[0,0],ymargin=[0,0],/noerase,color=64,con_color=48,/noborder
	map_p = !P & map_x = !X & map_y = !Y & map_Z = !Z & map_D = !D
	
	waitb = widget_base(title='Working',tlb_frame_attr=0,scroll=0,/modal,column=1,group_leader=lisb)
	waitm = widget_label(waitb,value='Reading background images...',/align_center)
	widget_control,waitb,/realize
	
	widget_control,exbgb,sensitive=0
		
	filename = bgname
	
	fileid=HDF_OPEN(filename,/read)

	sd_id=HDF_SD_START(filename,/read)
	HDF_SD_FILEINFO,sd_id,nmfsds,nglobatts
		
	for i=0,nmfsds-1 do begin
		sds_id=HDF_SD_SELECT(sd_id,i)
	     	HDF_SD_GETINFO,sds_id,name=n,ndims=r,type=t,natts=nats,$
	                         hdf_type=h,unit=u
	          	          	
	     	if r gt 0 then HDF_SD_GETINFO,sds_id,dims=dims else dims=0
	
		for j=0,nats-1 do begin
		 	HDF_SD_ATTRINFO,sds_id,j,name=n,type=t,count=c,data=d,$
	                            hdf_type=h
	     	endfor
	       		  
          	inbg = intarr(128,128,last-first+1)
          	maxvals = intarr(last-first+1)
          	
		temp = intarr(128,128)
		for kk = first,last,1 do begin
			hdf_sd_getdata,sds_id,temp,start=[0,0,kk],count=[128,128,1]
			inbg(*,*,kk-first) = temp
		endfor

           	; hdf_sd_getdata,sds_id,inbg,start=[0,0,first],count=[128,128,last-first+1]

		HDF_SD_ENDACCESS,sds_id
	endfor
	for i=0,nglobatts-1 do begin
		HDF_SD_ATTRINFO,sd_id,i,name=n,type=t,count=c,data=d
	endfor
			   
	HDF_SD_END,sd_id


	do_calibration = dialog_message('Calibrate backgrounds',/question)
	
	if (do_calibration eq 'Yes') then begin
		widget_control,waitm,set_value='Loading calibration data...'
		
		read_cal_hdfs,ac,dc
		
		widget_control,waitm,set_value='Calibrating backgrounds...'
	endif

	wset,exbg_raw_id
	raw_p = !P & raw_x = !X & raw_y = !Y & raw_z = !Z & raw_D = !D
	wset,exbg_cal_id
	cal_p = !P & cal_x = !X & cal_y = !Y & cal_z = !Z & cal_D = !D
	
	
	calbg = fltarr(128,128,last-first+1)
	rawbg = intarr(128,128)
	for i=0,last-first,1 do begin
		rawbg = inbg(*,*,i)
		maxvals(i) = max(rawbg)
		wset,exbg_raw_id
		!P = raw_p & !X = raw_x & !Y = raw_y & !Z = raw_Z
		dum1 = rawbg/maxvals(i)
		dum2 = 64.*dum1
		dum3 = fix(dum2)
		dum4 = rebin(dum3,256,256,/sample)
		tv,dum4
		;tv,rebin(fix(64.*rawbg/maxvals(i)),256,256,/sample)
		widget_control,exbgctl_bgsld,set_value=first+i
		dt = new_datetime_structure(1)
		dt.tai93 = orbit.point.bg_summary(first+i).TAI93_time
		calc_datetime,dt
		msgstr = string(format='((I2.2),":",(I2.2),":",(F6.3)," UTC")', $
										dt.utc.hour, dt.utc.minute, dt.utc.second)
		widget_control,exbgctl_bgsld_tai,set_value=msgstr
		if (do_calibration eq 'Yes') then begin
			if (maxvals(i) gt 200) then begin
				calbg(*,*,i) = dc(calidx,*,*,0)+inbg(*,*,i)*dc(calidx,*,*,1)+(inbg(*,*,i)*inbg(*,*,i)*dc(calidx,*,*,2))
			endif else begin
				calbg(*,*,i) = 0
			endelse
			wset,exbg_cal_id
			!P = cal_p & !X = cal_x & !Y = cal_y & !Z = cal_z
			tv,rebin(fix(64.*calbg(*,*,i)/300.),256,256,/sample)
		endif
		wset,exbg_map_id
		!P = map_p & !X = map_x & !Y = map_y & !Z = map_Z
		; map_set,xmargin=[0,0],ymargin=[0,0],/noerase,/noborder,con_color=48
		if (i gt 0) then begin
			plots,orbit.point.bg_summary(first+i-1).boresight(1),orbit.point.bg_summary(first+i-1).boresight(0),psym=4,color=131
		endif
		plots,orbit.point.bg_summary(first+i).boresight(1),orbit.point.bg_summary(first+i).boresight(0),psym=4,color=129
	endfor
	plots,orbit.point.bg_summary(first+i-1).boresight(1),orbit.point.bg_summary(first+i-1).boresight(0),psym=4,color=131
	widget_control,waitm,set_value='Finishing up...'
	
	; Set to first BG
	bgidx = first
	wset,exbg_map_id
	map_set,xmargin=[0,0],ymargin=[0,0],/noerase,/noborder,con_color=48
	plots,orbit.point.bg_summary(bgidx).boresight(1),orbit.point.bg_summary(bgidx).boresight(0),psym=4,color=129
	widget_control,exbgctl_bgsld,set_value=bgidx
	widget_control,exbgctl_bgsld,set_uvalue=bgidx
	
	; Compute bg cal lookup table
;	ct12 = findgen(4096)
;	count_to_rad = fltarr(4096)
;	rad_to_count = fltarr(maxrads(calidx))
;	for i=0,4095,1 do begin
;		thisrad = dc(calidx,*,*,0)+float(i)*dc(calidx,*,*,1)+float(i)*float(i)*dc(calidx,*,*,2)
;		thisradmom = moment(thisrad)
;		count_to_rad(i) = thisradmom(0)
;		rad_to_count(max([fix(thisradmom(0)),0])) = i
;	endfor

	ctmaskval = 3000
;	radmaskval = fix(count_to_rad(ctmaskval))
	radmaskval = 330
	widget_control,exbgctl_ctsld,set_value=ctmaskval
	widget_control,exbgctl_ctsld,set_uvalue=ctmaskval
	widget_control,exbgctl_rasld,set_value=radmaskval
	widget_control,exbgctl_rasld,set_uvalue=radmaskval
	
	; Draw first BG map
	wset,exbg_raw_id
	tv,rebin(fix(64.*float(inbg(*,*,bgidx-first))/float(ctmaskval)),256,256,/sample)

	; Draw calibrated BG map
	
	wset,exbg_cal_id
	tv,rebin(fix(64.*float(calbg(*,*,bgidx-first))/float(radmaskval)),256,256,/sample)
	
	; Calculate initial event window = average inter-bg duration
	
	if (gotparams(3) ne 0) then begin
		delt = 0.
		for i=first+1,last-1,1 do begin
			delt = delt + orbit.point.bg_summary(i).TAI93_time - orbit.point.bg_summary(i-1).TAI93_time
		endfor
		avdelt = delt / float(orbit.point.point_summary.bg_count)
		
		event_window = fix(avdelt)
		widget_control,exbgctl_evsld,set_value=avdelt
		
		inplot = where((orbit.point.lightning.event(*).TAI93_time ge (orbit.point.bg_summary(bgidx).TAI93_time-(event_window/2.))) and $
				    (orbit.point.lightning.event(*).TAI93_time le (orbit.point.bg_summary(bgidx).TAI93_time+(event_window/2.))),inplot_count)
		evgrid = lonarr(128,128)
		outevgrid = bytarr(128,128)
		if (inplot_count gt 0) then begin
			for i=long(0),inplot_count-long(1),long(1) do begin
				xpix = orbit.point.lightning.event(inplot(i)).x_pixel
				ypix = orbit.point.lightning.event(inplot(i)).y_pixel
				evgrid(xpix,ypix) = evgrid(xpix,ypix) + 1
			endfor
			outevgrid = fix(64.+62.*float(evgrid)/max(float(evgrid)))
		endif else begin
			outevgrid(*,*) = 64.
		endelse
		wset,exbg_ev_id
		tv,rebin(outevgrid,256,256,/sample)
	endif
	
	; Load the tables
	
;	widget_control, exbgctl_cttab, set_value=inbg(*,*,bgidx)
;	widget_control, exbgctl_radtab, set_value=inbg(*,*,bgidx)
	
	; Ready for events
	
	if (inplot_count gt 0) then begin
		widget_control,exbgctl_evtabbut,/sensitive
	endif else begin
		widget_control,exbgctl_evtabbut,sensitive=0
	endelse
	
	widget_control,waitb,/destroy
	widget_control,exbgb,/sensitive
	if (gotparams(3) eq 0) then begin
		widget_control,exbgctl_evsld, sensitive=0
	endif
	if (gotparams(5) eq 0) then begin
		;widget_control,exbgctl_glintbut, sensitive=0
	endif

	if ((maxvals(bgidx-first) gt 200) and (inplot_count gt 0)) then begin
		;widget_control,exbgctl_evbgbut,/sensitive
	endif else begin
		;widget_control,exbgctl_evbgbut,sensitive=0
	endelse

	firstbg = first & lastbg = last
	
	xmanager, 'examine_bgs',exbgb, event='exbg_handler',/no_block

end

pro exbg_handler,event
	common exbgcommon3, calidx, maxrads, count_to_rad, rad_to_count, exbgb, exbg_map_id, exbg_cal_id, exbg_raw_id, exbg_ev_id, exbg_glint_id, $
 						dc, bgidx, ctmaskval, radmaskval, event_window, exbgctl_cttab, exbgctl_radtab, $
 						exbgctl_bgsld, exbgctl_evsld, exbgctl_ctsld, exbgctl_rasld, exbgctl_bgsld_tai, inbg, calbg, exbgctl_donebut, exbgctl_glintbut
	common exbgcommon4, exbgctl_xlab,exbgctl_ylab,exbgctl_ctlab,exbgctl_radlab,exbgctl_evlab,exbgctl_glintlab,evgrid,glintgrid,exbgd_raw,exbgd_cal,exbgd_ev,exbgd_glint
	common datafiles, orbit, fname
	common sensor_read, this_sensor, file_format
	common prefs, readparams, readparamsotd, setparams, setparamsotd, gotparams, gotparamsotd
	common lisapp_colors,red,green,blue
	common exbgcommon5, exbgctl_threshbut
	common exbgcommon6, exbgctl_evbgbut,maxvals
	common exbgcommon7, exbgctl_evtabbut
	common exbgcommon8, exbgd_calpd, exbgd_rawpd, exbgd_evpd, exbgd_glintpd
	common exbgcommon9, exbgctl_pixinfo6
	common bg_info2, firstbg, lastbg

		case 1 of 
		(event.id eq exbgctl_donebut) : begin
					inbg = 0 & calbg = 0 & firstbg = 0 & lastbg = 0 & maxvals = 0 & dc = 0
					widget_control, exbgb, /destroy
				    end
		(event.id eq exbgctl_bgsld): begin
					widget_control,exbgctl_bgsld,get_uvalue=oldbgidx
					widget_control,exbgctl_bgsld,get_value=bgidx
					wset,exbg_map_id
					map_set,xmargin=[0,0],ymargin=[0,0],/noerase,/noborder,con_color=48
					plots,orbit.point.bg_summary(oldbgidx).boresight(1),orbit.point.bg_summary(oldbgidx).boresight(0),psym=4,color=131
					plots,orbit.point.bg_summary(bgidx).boresight(1),orbit.point.bg_summary(bgidx).boresight(0),psym=4,color=129
					widget_control,exbgctl_bgsld,set_uvalue=bgidx
					dt = new_datetime_structure(1)
					dt.tai93 = orbit.point.bg_summary(bgidx).TAI93_time
					calc_datetime,dt
					msgstr = string(format='((I2.2),":",(I2.2),":",(F6.3)," UTC")', dt.utc.hour, dt.utc.minute, dt.utc.second)		
					widget_control,exbgctl_bgsld_tai,set_value=msgstr					
					widget_control,exbgctl_ctsld,get_value=ctmaskval
					widget_control,exbgctl_rasld,get_value=radmaskval
					wset,exbg_raw_id
					tv,rebin(fix(64.*float(inbg(*,*,bgidx-firstbg))/float(ctmaskval)),256,256,/sample)
					wset,exbg_cal_id
					tv,rebin(fix(64.*float(calbg(*,*,bgidx-firstbg))/float(radmaskval)),256,256,/sample)
					if (gotparams(3) ne 0) then begin
						widget_control,exbgctl_evsld,get_value=event_window	
						inplot = where((orbit.point.lightning.event(*).TAI93_time ge (orbit.point.bg_summary(bgidx).TAI93_time-(event_window/2.))) and $
								    (orbit.point.lightning.event(*).TAI93_time le (orbit.point.bg_summary(bgidx).TAI93_time+(event_window/2.))),inplot_count)
						evgrid = lonarr(128,128)
						outevgrid = bytarr(128,128)
						if (inplot_count gt 0) then begin
							for i=long(0),inplot_count-long(1),long(1) do begin
								xpix = orbit.point.lightning.event(inplot(i)).x_pixel
								ypix = orbit.point.lightning.event(inplot(i)).y_pixel
								evgrid(xpix,ypix) = evgrid(xpix,ypix) + 1
							endfor
							outevgrid = fix(64.+62.*float(evgrid)/max(float(evgrid)))
						endif else begin
							outevgrid(*,*) = 64.
						endelse
						wset,exbg_ev_id
						tv,rebin(outevgrid,256,256,/sample)
					endif
					emptarr=bytarr(256,256)
					tvlct,red,green,blue
					wset,exbg_glint_id
					tv,emptarr
;					widget_control, exbgctl_cttab, set_value=inbg(*,*,bgidx)
;					widget_control, exbgctl_radtab, set_value=inbg(*,*,bgidx)
					if ((maxvals(bgidx-firstbg) gt 200) and (inplot_count gt 0)) then begin
						;widget_control,exbgctl_evbgbut,/sensitive
					endif else begin
						;widget_control,exbgctl_evbgbut,sensitive=0
					endelse
					if (inplot_count gt 0) then begin
						widget_control,exbgctl_evtabbut,/sensitive
					endif else begin
						widget_control,exbgctl_evtabbut,sensitive=0
					endelse
					widget_control,exbgd_glintpd,set_droplist_select=0
					widget_control,exbgctl_pixinfo6,set_value='          '
					widget_control,exbgctl_glintlab,set_value='      '
				    end
		((event.id eq exbgd_cal) or (event.id eq exbgd_raw) or (event.id eq exbgd_ev) or (event.id eq exbgd_glint)): begin
					xpix = fix(event.x/2.)
					ypix = fix(event.y/2.)
					msg = string(format='((I3))',xpix)
					widget_control,exbgctl_xlab,set_value=msg
					msg = string(format='((I3))',ypix)
					widget_control,exbgctl_ylab,set_value=msg
					msg = string(format='((I5))',fix(inbg(xpix,ypix,bgidx-firstbg)))
					widget_control,exbgctl_ctlab,set_value=msg
					msg = string(format='((F5.1))',calbg(xpix,ypix,bgidx-firstbg))
					widget_control,exbgctl_radlab,set_value=msg
					if (gotparams(3) ne 0) then begin
						msg = string(format='((I5))',evgrid(xpix,ypix))
						widget_control,exbgctl_evlab,set_value=msg
					endif
					widget_control,exbgd_glintpd,get_uvalue=which4
					if (which4 eq 1 or which4 eq 2) then begin
						if (which4 eq 1) then begin
							outval = fix(glintgrid(xpix,ypix))
						endif else begin
							outval = fix(glintgrid(xpix,ypix))
						endelse
						msg = string(format='((I5))',outval)
						widget_control,exbgctl_glintlab,set_value=msg
					endif
				    end	
		(event.id eq exbgctl_evsld): begin
					widget_control, exbgctl_evsld, get_value=event_window
					inplot = where((orbit.point.lightning.event(*).TAI93_time ge (orbit.point.bg_summary(bgidx).TAI93_time-(event_window/2.))) and $
							    (orbit.point.lightning.event(*).TAI93_time le (orbit.point.bg_summary(bgidx).TAI93_time+(event_window/2.))),inplot_count)
					evgrid = lonarr(128,128)
					outevgrid = bytarr(128,128)
					if (inplot_count gt 0) then begin
						for i=long(0),inplot_count-long(1),long(1) do begin
							xpix = orbit.point.lightning.event(inplot(i)).x_pixel
							ypix = orbit.point.lightning.event(inplot(i)).y_pixel
							evgrid(xpix,ypix) = evgrid(xpix,ypix) + 1
						endfor
						outevgrid = fix(64.+62.*float(evgrid)/max(float(evgrid)))
						;widget_control,exbgctl_evtabbut,/sensitive
					endif else begin
						outevgrid(*,*) = 64.
						;widget_control,exbgctl_evtabbut,sensitive=0
					endelse
					wset,exbg_ev_id
					tv,rebin(outevgrid,256,256,/sample)
					if ((maxvals(bgidx-firstbg) gt 200) and (inplot_count gt 0)) then begin
						;widget_control,exbgctl_evbgbut,/sensitive
					endif else begin
						;widget_control,exbgctl_evbgbut,sensitive=0
					endelse
					
				    end
		(event.id eq exbgctl_ctsld): begin
					widget_control,exbgctl_ctsld,get_value=ctmaskval
					wset,exbg_raw_id
					tv,rebin(fix(64.*float(inbg(*,*,bgidx-firstbg))/float(ctmaskval)),256,256,/sample)
				    end
		(event.id eq exbgctl_rasld):  begin
					widget_control,exbgctl_rasld,get_value=radmaskval
					wset,exbg_cal_id
					tv,rebin(fix(64.*float(calbg(*,*,bgidx-firstbg))/float(radmaskval)),256,256,/sample)
				    end  
		(event.id eq exbgctl_evtabbut):begin
					inplot = where((orbit.point.lightning.event(*).TAI93_time ge (orbit.point.bg_summary(bgidx).TAI93_time-(event_window/2.))) and $
						    (orbit.point.lightning.event(*).TAI93_time le (orbit.point.bg_summary(bgidx).TAI93_time+(event_window/2.))),inplot_count)
					if (inplot_count gt 1) then begin
						ev_table,event,inplot
					endif
					end	
		(event.id eq exbgd_calpd or event.id eq exbgd_rawpd or event.id eq exbgd_evpd):begin
					end
		(event.id eq exbgd_glintpd):begin
					case event.index of
						0:begin
							widget_control,exbgd_glintpd,set_uvalue=0
						  end
						1:begin
							widget_control,exbgctl_bgsld,get_value=bgidx
						  	array_glint_factor,orbit.point.bg_summary(bgidx).TAI93_time,this_sensor,glint_angle
						  	cos_glint_angle = cos(!DTOR*glint_angle)
						  	glintgrid = acos(cos_glint_angle)/!dtor
						  	dark = where(cos_glint_angle lt 0, darkct)
						  	if (darkct gt 0) then begin
						  		cos_glint_angle(dark) = -1
						  	endif
						  	wset,exbg_glint_id
						  	tv,rebin(65.+64.*((cos_glint_angle)),256,256,/sample)
						  	widget_control,exbgctl_pixinfo6,set_value='Glint'
						  	widget_control,exbgctl_glintlab,set_value='-----'
						  	widget_control,exbgd_glintpd,set_uvalue=1
						  	cos_glint_angle = 0
						  end
						2:begin
							widget_control,exbgctl_bgsld,get_value=bgidx
							thrarr = bytarr(128,128)
							thr = bytarr(16)
							tidx = long(orbit.point.bg_summary(bgidx).TAI93_time)-long(orbit.one_second(0).TAI93_time)
							thr(0:11) = orbit.one_second(tidx).thresholds(0:11)
							thr(12:15) = 63
							for y=0,127,1 do begin
								for x=0,127,1 do begin
									thrarr(x,y) = thr(fix(inbg(x,y,bgidx-firstbg)/256))
								endfor
							endfor
							thrarr = 65+temporary(thrarr)
							wset,exbg_glint_id
							tv,rebin(thrarr,256,256,/sample)
						  	widget_control,exbgctl_pixinfo6,set_value='Threshold'
						  	widget_control,exbgctl_glintlab,set_value='-----'
						  	widget_control,exbgd_glintpd,set_uvalue=2
						  	glintgrid = thrarr-65 & thrarr = 0
						  end
						3:begin
							wset,exbg_glint_id
							xvert = fltarr(5) & yvert = fltarr(5)
							xvert(0) = 0 & xvert(1)=0 & xvert(2) = 256 & xvert(3) = 256 & xvert(4) = 0
							yvert(0) = 0 & yvert(1)=256 & yvert(2) = 256 & yvert(3) = 0 & yvert(4) = 0
							polyfill,xvert,yvert,color=133,/device
							widget_control,exbgctl_bgsld,get_value=bgidx
							widget_control,exbgctl_ctsld,get_value=ctmaskval
							inplot = where((orbit.point.lightning.event(*).TAI93_time ge (orbit.point.bg_summary(bgidx).TAI93_time-(event_window/2.))) and $
								    (orbit.point.lightning.event(*).TAI93_time le (orbit.point.bg_summary(bgidx).TAI93_time+(event_window/2.))),inplot_count)
							if (inplot_count gt 1) then begin
								plotmin = min([min(inbg(*,*,bgidx-firstbg)),min(orbit.point.lightning.event(inplot).bg_value)])
								plotmax = max([maxvals(bgidx-firstbg),max(orbit.point.lightning.event(inplot).bg_value)])
							endif else begin
								plotmin = min(inbg(*,*,bgidx-firstbg))
								plotmax = maxvals(bgidx-firstbg)
							endelse
							plotmin = 10*fix(float(plotmin)/10.)
							plotmax = 10*(fix(float(plotmax)/10.)+1)
							numbars = fix((plotmax-plotmin)/10.)
							bghist = histogram(inbg(*,*,bgidx-firstbg),min=plotmin,max=plotmax,binsize=10)
							if (inplot_count gt 1) then begin
								evhist = histogram(orbit.point.lightning.event(inplot).bg_value,min=plotmin,max=plotmax,binsize=10)
							endif
							xvert = fltarr(5) & yvert = fltarr(5)
							for i=0,numbars-1,1 do begin
								yvert(0) = 128
								yvert(1) = 128+(float(bghist(i))/float(max(bghist(*))))*128
								yvert(2) = yvert(1)
								yvert(3) = 128
								yvert(4) = 128
								xvert(0) = 256*(float(i)/float(numbars))
								xvert(1) = xvert(0)
								xvert(2) = 256*(float(i+1)/float(numbars))
								xvert(3) = xvert(2)
								xvert(4) = xvert(0)
								polyfill,xvert,yvert,color=fix(64.*(float(plotmin+i*10+5)/float(ctmaskval))),/device
								if (inplot_count gt 1) then begin
									yvert(0) = 0
									yvert(1) = (float(evhist(i))/float(max(evhist(*))))*128
									yvert(2) = yvert(1)
									yvert(3) = 0
									yvert(4) = 0
									polyfill,xvert,yvert,color=fix(64.*(float(plotmin+i*10+5)/float(ctmaskval))),/device		
								endif				
							endfor
						  	widget_control,exbgctl_pixinfo6,set_value='          '
						  	widget_control,exbgctl_glintlab,set_value='     '
							widget_control,exbgd_glintpd,set_uvalue=3
						  end
					endcase
				end				
	endcase
 						
 end
 
;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Flash Analysis Routines ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;

  pro area_quickplot, theareas, numareas
	common datafiles, orbit, fname
	common lisbase, lisb
	common lisapp_colors,red,green,blue
	common sensor_read, this_sensor, file_format
	
	earliest_time = 9e20
	latest_time = -9e20
	num_flashes = 0
	
	for act = 0, numareas-1, 1 do begin	
		thisarea = theareas(act)
		if (thisarea.TAI93_time lt earliest_time) then begin
			earliest_time = thisarea.TAI93_time
		endif
		if (thisarea.TAI93_time + thisarea.observe_time gt latest_time) then begin
			latest_time = thisarea.TAI93_time + thisarea.observe_time
		endif
		num_flashes = num_flashes + thisarea.child_count
	endfor
	
	if (latest_time eq earliest_time) then begin
		latest_time = earliest_time + 1.
	endif

	theevents = replicate({an_event},total(theareas(*).greatgrandchild_count))
	thegroups = replicate({a_group},total(theareas(*).grandchild_count))
	eidx = 0L & gidx = 0L
	for act=0L,numareas-1L,1L do begin
		thisarea=theareas(act)
		for fct=0L,thisarea.child_count-1L,1L do begin
			thisflash = orbit.point.lightning.flash(thisarea.child_address+fct)
			for gct = 0L,thisflash.child_count-1L,1L do begin
				thisgroup = orbit.point.lightning.group(thisflash.child_address+gct)
				thegroups(gidx) = thisgroup
				gidx = gidx+1
				for ect = 0L, thisgroup.child_count-1L,1L do begin
					thisevent = orbit.point.lightning.event(thisgroup.child_address+ect)
					theevents(eidx) = thisevent
					eidx = eidx + 1L
				endfor
			endfor
		endfor
	endfor
	
	; scale to full screen for ms per x pixel
	msperpix = (latest_time-earliest_time)/2000.
	
	xsz = long((latest_time-earliest_time)/msperpix)+1
	; 1 flash per 5 y pixels
	ysz = 10*num_flashes
	; print,xsz,ysz

	window,18,colors=220,xsize=256,ysize=256,retain=2
	minlat = 0 & minlon = 0 & maxlat = 0 & maxlon = 0
	geo_plot,1,theevents,18,0,minlat,maxlat,minlon,maxlon,bgpixmap


end

pro geo_plot, type, theevents, win_id, do_bg, minlat,maxlat,minlon,maxlon,bgpixmap
	case type of
		1: begin
		   	; Simple PLOTS
		   	geo_superquickplot, theevents, win_id, do_bg, minlat, maxlat, minlon, maxlon, passbgpixmap
		   	bgpixmap = passbgpixmap
		   end
		2: begin
			; Footprint, sorted rad
			geo_quickplot, theevents, win_id, do_bg, minlat, maxlat, minlon, maxlon, passbgpixmap
			bgpixmap = passbgpixmap
		   end
		3: begin
			; Footprint, total rad
		   end
	endcase
end

pro geo_superquickplot, theevents, win_id, do_bg, minlat, maxlat, minlon, maxlon, passbgpixmap
	common datafiles, orbit, fname
	common lisbase, lisb
	common lisapp_colors,red,green,blue
	common sensor_read, this_sensor, file_format
	uscale = usersym_scale(0)
	if (minlat eq 0 and minlon eq 0 and maxlat eq 0 and maxlon eq 0) then begin
		dellat = max(theevents(*).location(0))-min(theevents(*).location(0))+1.25;
		dellon = max(theevents(*).location(1))-min(theevents(*).location(1))+1.25;
		ctrlat = (max(theevents(*).location(0))+min(theevents(*).location(0)))/2.
		ctrlon = (max(theevents(*).location(1))+min(theevents(*).location(1)))/2.
		if (dellat gt dellon) then begin
			dellon = dellat
		endif else begin
			dellat = dellon
		endelse
		minlat = ctrlat - dellat/2.
		maxlat = ctrlat + dellat/2.
		minlon = ctrlon - dellon/2.
		maxlon = ctrlon + dellon/2.
	endif
	eidx = 0L
	sorteidx = sort(theevents(*).amplitude)
	wset,win_id
	tvlct,red,green,blue
	if (do_bg eq 1) then begin
		plot,findgen(2),findgen(2),position=[0,0,1,1],/nodata,xstyle=1,ystyle=1,xmargin=[0,0],ymargin=[0,0],xrange=[minlon,maxlon],yrange=[minlat,maxlat]
	endif else begin
		plot,findgen(2),findgen(2),position=[0,0,1,1],/nodata,/noerase,xstyle=1,ystyle=1,xmargin=[0,0],ymargin=[0,0],xrange=[minlon,maxlon],yrange=[minlat,maxlat]
	endelse
	ssize = .25*uscale
	plots,theevents(sorteidx).location(1),theevents(sorteidx).location(0), $
				psym=4,symsize=ssize, $
				color=fix(65.+theevents(sorteidx).amplitude/2)
	passbgpixmap = tvrd()
	eventlocs = where(passbgpixmap ge 65,inct)
	if (inct gt 0) then begin
		passbgpixmap(eventlocs) = 24
	endif

end

pro geo_quickplot, theevents, win_id, do_bg, minlat,maxlat,minlon,maxlon,passbgpixmap
	common datafiles, orbit, fname
	common lisbase, lisb
	common lisapp_colors,red,green,blue
	common sensor_read, this_sensor, file_format
	common bg_info, fnamestub, candobgs, bgname, bgpathtouse

	if (minlat eq 0 and minlon eq 0 and maxlat eq 0 and maxlon eq 0) then begin
		dellat = max(theevents(*).location(0))-min(theevents(*).location(0))+1.25;
		dellon = max(theevents(*).location(1))-min(theevents(*).location(1))+1.25;
		ctrlat = (max(theevents(*).location(0))+min(theevents(*).location(0)))/2.
		ctrlon = (max(theevents(*).location(1))+min(theevents(*).location(1)))/2.
		if (dellat gt dellon) then begin
			dellon = dellat
		endif else begin
			dellat = dellon
		endelse
		minlat = ctrlat - dellat/2.
		maxlat = ctrlat + dellat/2.
		minlon = ctrlon - dellon/2.
		maxlon = ctrlon + dellon/2.
	endif
	;hiresgridsize = .01
	;fpgridsz = fix(dellat/hiresgridsize)
	fpgridsz = 256
	fpgrid = lonarr(fpgridsz,fpgridsz)
	wset,win_id
	tvlct,red,green,blue
	if (do_bg eq 1) then begin
		plot,findgen(2),findgen(2),position=[0,0,1,1],/nodata,xstyle=1,ystyle=1,xmargin=[0,0],ymargin=[0,0],xrange=[minlon,maxlon],yrange=[minlat,maxlat]
	endif else begin
		plot,findgen(2),findgen(2),position=[0,0,1,1],/nodata,/noerase,xstyle=1,ystyle=1,xmargin=[0,0],ymargin=[0,0],xrange=[minlon,maxlon],yrange=[minlat,maxlat]
	endelse
	evsz = size(theevents)
	num_events = evsz(1)
	; num_events = total(theareas(*).greatgrandchild_count)
	xverts = fltarr(5)
	yverts = fltarr(5)
	xpix = fltarr(5)
	ypix = fltarr(5)
	
	if (do_bg eq 1 and candobgs) then begin
		; Find closest background
		
		if (num_events gt 1) then begin
			timmom = moment(theevents(*).TAI93_time)
			meantime = timmom(0)
		endif else begin
			meantime = theevents(0).TAI93_time
		endelse
		timdiff = abs(orbit.point.bg_summary(*).TAI93_time-meantime)
		stimdiff = sort(timdiff)
		closest = stimdiff(0)
		nextclosest = stimdiff(1)
		closidx = lonarr(2)
		closidx = [closest,nextclosest]
		filename = bgname		
		fileid=HDF_OPEN(filename,/read)
	
		sd_id=HDF_SD_START(filename,/read)
		HDF_SD_FILEINFO,sd_id,nmfsds,nglobatts
		
		for i=0,nmfsds-1 do begin
			sds_id=HDF_SD_SELECT(sd_id,i)
		     	HDF_SD_GETINFO,sds_id,name=n,ndims=r,type=t,natts=nats,$
		                         hdf_type=h,unit=u
		          	          	
		     	if r gt 0 then HDF_SD_GETINFO,sds_id,dims=dims else dims=0
		
			for j=0,nats-1 do begin
			 	HDF_SD_ATTRINFO,sds_id,j,name=n,type=t,count=c,data=d,$
		                            hdf_type=h
		     	endfor
		       		  
	          	inbg0 = intarr(128,128)
	 		hdf_sd_getdata,sds_id,inbg0,start=[0,0,closidx(0)],count=[128,128,1]
	 		inbg1 = intarr(128,128)
	 		hdf_sd_getdata,sds_id,inbg1,start=[0,0,closidx(1)],count=[128,128,1]
	
			HDF_SD_ENDACCESS,sds_id
		endfor
		for i=0,nglobatts-1 do begin
			HDF_SD_ATTRINFO,sd_id,i,name=n,type=t,count=c,data=d
		endfor
				   
		HDF_SD_END,sd_id
	
		xgrid = bytarr(128,128)
		ygrid = bytarr(128,128)
		for yidx = 0,127,1 do begin
			for xidx = 0,127,1 do begin
				xgrid(xidx,yidx) = xidx
				ygrid(xidx,yidx) = yidx
			endfor
		endfor
		for bg=0,1,1 do begin
			if (bg eq 0) then begin
				inbg = inbg0
			endif else begin
				inbg = inbg1
			endelse
			;window,19,xsize=128,ysize=128,retain=2
			;tvscl,inbg
			
			get_array_pointing_vector, this_sensor, array_look_vector
			get_array_earth_intersection,array_look_vector,orbit.point.bg_summary(closidx(bg)).TAI93_time,array_location,0.0D ,return_code
			lats = fltarr(128,128) & lons = fltarr(128,128)
			lats = array_location(0:127,0:127,0) & lons = array_location(0:127,0:127,1)
			inplot = where(lons gt minlon and lons lt maxlon and $
					    lats gt minlat and lats lt maxlat,inct)
			if (inct gt 0) then begin
				; print,inct
				times = replicate(orbit.point.bg_summary(closidx(bg)).TAI93_time,5)
				for inidx = 0, inct-1, 1 do begin
					xpix(0) = float(xgrid(inplot(inidx)))-.5
					xpix(1) = float(xgrid(inplot(inidx)))+.5
					xpix(2) = xpix(1)
					xpix(3) = xpix(0)
					xpix(4) = xpix(0)
					ypix(0) = float(ygrid(inplot(inidx)))-.5
					ypix(1) = ypix(0)
					ypix(2) = float(ygrid(inplot(inidx)))+.5
					ypix(3) = ypix(2)
					ypix(4) = ypix(0)
					get_vector_pointing_vector, this_sensor, xpix, ypix, look_vector
					get_vector_earth_intersection, look_vector, times, vector_location, 12000.0D, return_code
					for ict = 0,4,1 do begin
						xverts(ict) = vector_location(ict,1)
						yverts(ict) = vector_location(ict,0)
					endfor
					polyfill,xverts,yverts,color=fix(64.*float(inbg(xgrid(inplot(inidx)),ygrid(inplot(inidx))))/3500.)
				endfor
			
			endif
		endfor
	endif
	eidx = 0L
	sorteidx = sort(theevents(*).amplitude)
	for ect = 0L, num_events-1L,1L do begin
		times = replicate(theevents(sorteidx(ect)).TAI93_time,5)
		thisevent = theevents(sorteidx(ect))
		xpix(0) = float(thisevent.x_pixel)-.5
		xpix(1) = float(thisevent.x_pixel)+.5
		xpix(2) = xpix(1)
		xpix(3) = xpix(0)
		xpix(4) = xpix(0)
		ypix(0) = float(thisevent.y_pixel)-.5
		ypix(1) = ypix(0)
		ypix(2) = float(thisevent.y_pixel)+.5
		ypix(3) = ypix(2)
		ypix(4) = ypix(0)
		get_vector_pointing_vector, this_sensor, xpix, ypix, look_vector
		get_vector_earth_intersection, look_vector, times, vector_location, 12000.0D, return_code
		for ict = 0,4,1 do begin
			xverts(ict) = vector_location(ict,1)
			yverts(ict) = vector_location(ict,0)
		endfor
		; print,' '
		eidx = eidx+1L
		polyfill,xverts,yverts,color=fix(65.+theevents(sorteidx(ect)).amplitude/2)
	endfor
	passbgpixmap = tvrd()
	eventlocs = where(passbgpixmap ge 65,inct)
	if (inct gt 0) then begin
		passbgpixmap(eventlocs) = 24
	endif

end

pro area_barplot, theareas, numareas
	common datafiles, orbit, fname
	common lisbase, lisb
	common lisapp_colors,red,green,blue
	common sensor_read, this_sensor, file_format
	common bg_info, fnamestub, candobgs, bgname, bgpathtouse
	
	earliest_time = 9e20
	latest_time = -9e20
	num_flashes = 0
	
	for act = 0, numareas-1, 1 do begin	
		thisarea = theareas(act)
		if (thisarea.TAI93_time lt earliest_time) then begin
			earliest_time = thisarea.TAI93_time
		endif
		if (thisarea.TAI93_time + thisarea.observe_time gt latest_time) then begin
			latest_time = thisarea.TAI93_time + thisarea.observe_time
		endif
		num_flashes = num_flashes + thisarea.child_count
	endfor
	
	if (latest_time eq earliest_time) then begin
		latest_time = earliest_time + 1.
	endif
	
	; scale to full screen for ms per x pixel
	msperpix = (latest_time-earliest_time)/2000.
	
	xsz = long((latest_time-earliest_time)/msperpix)+1
	; 1 flash per 5 y pixels
	ysz = 10*num_flashes
	; print,xsz,ysz

	window,15,colors=220,xsize=xsz,ysize=ysz,retain=2
	tvlct,red,green,blue
	polyfill,[0,0,xsz-1,xsz-1,0],[0,ysz-1,ysz-1,0,0],color=16,/device
	cbmin = alog10(1e4)
	cbmax = alog10(1e6)
	cbdel = cbmax-cbmin
	for sectime = long(earliest_time),long(latest_time),1L do begin
		thisx = long((double(sectime)-earliest_time)/msperpix)+1
		thisyb = 0
		thisyt = (total(theareas(*).child_count)+1)*10-1
		;print,thisx,thisyb,thisyt
		plots,[thisx,thisx],[thisyb,thisyt],color=48,/device
	endfor
	
	flashct = 0
	for act = 0, numareas-1, 1 do begin
		thisarea = theareas(act)
		for fct = 0, thisarea.child_count-1, 1 do begin
			thisflash = orbit.point.lightning.flash(thisarea.child_address+fct)
			thisxl = long((thisflash.TAI93_time-earliest_time)/msperpix)+1
			thisxr = long((thisflash.TAI93_time+thisflash.delta_time-earliest_time)/msperpix)+1
			thisyb = flashct*10
			thisyt = (flashct+1)*10-1
			xvert = [thisxl,thisxl,thisxr,thisxr,thisxl]
			yvert = [thisyb,thisyt,thisyt,thisyb,thisyb]
			usecolor = fix(65.+63.*(alog10(thisflash.radiance)-cbmin)/cbdel)
			polyfill,xvert,yvert,color=usecolor,/device
			flashct = flashct+1
		endfor
	endfor
	
	; Histogram
	
	theevents = replicate({an_event},total(theareas(*).greatgrandchild_count))
	thegroups = replicate({a_group},total(theareas(*).grandchild_count))
	eidx = 0L & gidx = 0L
	for act=0L,numareas-1L,1L do begin
		thisarea=theareas(act)
		for fct=0L,thisarea.child_count-1L,1L do begin
			thisflash = orbit.point.lightning.flash(thisarea.child_address+fct)
			for gct = 0L,thisflash.child_count-1L,1L do begin
				thisgroup = orbit.point.lightning.group(thisflash.child_address+gct)
				thegroups(gidx) = thisgroup
				gidx = gidx+1
				for ect = 0L, thisgroup.child_count-1L,1L do begin
					thisevent = orbit.point.lightning.event(thisgroup.child_address+ect)
					theevents(eidx) = thisevent
					eidx = eidx + 1
				endfor
			endfor
		endfor
	endfor
	
	if (eidx gt 1) then begin
		h = histogram(theevents(*).amplitude,max=128)
		xvert = intarr(4*130) & yvert=intarr(4*130)
		for hct = 0,128,1 do begin
			xvert(hct*4) = hct
			xvert(hct*4+1) = hct
			xvert(hct*4+2) = hct+1
			xvert(hct*4+3) = hct+1
			yvert(hct*4) = 0
			yvert(hct*4+1) = h(hct)
			yvert(hct*4+2) = h(hct)
			yvert(hct*4+3) = 0
		endfor
		xvert(129*4) = 0 & yvert(129*4)=0
	endif else begin
		h = intarr(128)
		xvert = intarr(4) & yvert = intarr(4)
		h(theevents(0).amplitude) = theevents(0).amplitude
		xvert(0) = theevents(0).amplitude
		xvert(1) = theevents(0).amplitude
		xvert(2) = theevents(0).amplitude+1
		xvert(3) = theevents(0).amplitude+1
		yvert(0) = 0
		yvert(1) = 1
		yvert(2) = 1
		yvert(3) = 0
	endelse
	window,17,colors=220,xsize=600,ysize=400,retain=2
	plot,h,/xstyle,psym=3
	polyfill,xvert,yvert,/data,color=129
		
	dellat = max(theevents(*).location(0))-min(theevents(*).location(0))+1.25;
	dellon = max(theevents(*).location(1))-min(theevents(*).location(1))+1.25;
	ctrlat = (max(theevents(*).location(0))+min(theevents(*).location(0)))/2.
	ctrlon = (max(theevents(*).location(1))+min(theevents(*).location(1)))/2.
	if (dellat gt dellon) then begin
		dellon = dellat
	endif else begin
		dellat = dellon
	endelse
	minlat = ctrlat - dellat/2.
	maxlat = ctrlat + dellat/2.
	minlon = ctrlon - dellon/2.
	maxlon = ctrlon + dellon/2.
	; print,minlat,minlon,maxlat,maxlon
	;hiresgridsize = .01
	;fpgridsz = fix(dellat/hiresgridsize)
	fpgridsz = 256
	hiresgridsize = float(dellat)/float(fpgridsz)
	fpgrid = lonarr(fpgridsz,fpgridsz)
	window,18,colors=220,xsize=fpgridsz,ysize=fpgridsz,retain=2
	
	for gct = 0, gidx-1, 1 do begin
		tmpgrid = lonarr(fpgridsz,fpgridsz)
		evct = thegroups(gct).child_count
		xpix = fltarr(13*13*evct)
		ypix = fltarr(13*13*evct)
		subidx = 0L
		for ect = 0, evct-1, 1 do begin
			thisevent = orbit.point.lightning.event(thegroups(gct).child_address+ect)
			for xdel = -.45,.45,.075 do begin
				for ydel = -.45,.45,.075 do begin
					xpix(subidx) = thisevent.x_pixel+xdel
					ypix(subidx) = thisevent.y_pixel+ydel
					subidx = subidx + 1L
				endfor
			endfor
		endfor
		get_vector_pointing_vector, this_sensor, xpix, ypix, look_vector
		times = replicate(thegroups(gct).TAI93_time,13*13*evct)
		get_vector_earth_intersection, look_vector, times, vector_location, 12000.0D, return_code
		xgridloc = (vector_location(*,1)-minlon)/hiresgridsize
		ygridloc = (vector_location(*,0)-minlat)/hiresgridsize
		tmpgrid(xgridloc,ygridloc) = temporary(tmpgrid(xgridloc,ygridloc))+thegroups(gct).radiance
		;window,17,xsize=fpgridsz,ysize=fpgridsz,retain=2
		;tvscl,tmpgrid
		fpgrid = fpgrid+tmpgrid
		
		tvscl,fpgrid
	endfor
	
	; Find closest background
	
	if (numareas gt 1) then begin
		timmom = moment(theareas(*).TAI93_time)
		meantime = timmom(0)
	endif else begin
		meantime = theareas(0).TAI93_time
	endelse
	timdiff = abs(orbit.point.bg_summary(*).TAI93_time-meantime)
	stimdiff = sort(timdiff)
	closest = stimdiff(0)
	nextclosest = stimdiff(1)
	closidx = lonarr(2)
	closidx = [closest,nextclosest]
	
	filename = bgname
	fileid=HDF_OPEN(filename,/read)

	sd_id=HDF_SD_START(filename,/read)
	HDF_SD_FILEINFO,sd_id,nmfsds,nglobatts
	
	for i=0,nmfsds-1 do begin
		sds_id=HDF_SD_SELECT(sd_id,i)
	     	HDF_SD_GETINFO,sds_id,name=n,ndims=r,type=t,natts=nats,$
	                         hdf_type=h,unit=u
	          	          	
	     	if r gt 0 then HDF_SD_GETINFO,sds_id,dims=dims else dims=0
	
		for j=0,nats-1 do begin
		 	HDF_SD_ATTRINFO,sds_id,j,name=n,type=t,count=c,data=d,$
	                            hdf_type=h
	     	endfor
	       		  
          	inbg0 = intarr(128,128)
 		hdf_sd_getdata,sds_id,inbg0,start=[0,0,closidx(0)],count=[128,128,1]
 		inbg1 = intarr(128,128)
 		hdf_sd_getdata,sds_id,inbg1,start=[0,0,closidx(1)],count=[128,128,1]

		HDF_SD_ENDACCESS,sds_id
	endfor
	for i=0,nglobatts-1 do begin
		HDF_SD_ATTRINFO,sd_id,i,name=n,type=t,count=c,data=d
	endfor
			   
	HDF_SD_END,sd_id

	xgrid = bytarr(128,128)
	ygrid = bytarr(128,128)
	for yidx = 0,127,1 do begin
		for xidx = 0,127,1 do begin
			xgrid(xidx,yidx) = xidx
			ygrid(xidx,yidx) = yidx
		endfor
	endfor
	bgfpgrid = fltarr(fpgridsz,fpgridsz)
	
	for bg=0,1,1 do begin
		if (bg eq 0) then begin
			inbg = inbg0
		endif else begin
			inbg = inbg1
		endelse
		window,19,colors=220,xsize=128,ysize=128,retain=2
		tvscl,inbg
		
		get_array_pointing_vector, this_sensor, array_look_vector
		get_array_earth_intersection,array_look_vector,orbit.point.bg_summary(closidx(bg)).TAI93_time,array_location,0.0D ,return_code
		lats = fltarr(128,128) & lons = fltarr(128,128)
		lats = array_location(0:127,0:127,0) & lons = array_location(0:127,0:127,1)
		inplot = where(lons gt minlon and lons lt maxlon and $
				    lats gt minlat and lats lt maxlat,inct)
		if (inct gt 0) then begin
			; print,inct
			times = replicate(orbit.point.bg_summary(closidx(bg)).TAI93_time,13*13)
			for inidx = 0, inct-1, 1 do begin
				xpix = fltarr(13*13)
				ypix = fltarr(13*13)
				subidx = 0
				for xdel = -.45,.45,.075 do begin
					for ydel = -.45,.45,.075 do begin
						xpix(subidx) = float(xgrid(inplot(inidx)))+xdel
						ypix(subidx) = float(ygrid(inplot(inidx)))+ydel
						subidx = subidx + 1
					endfor
				endfor
				get_vector_pointing_vector, this_sensor, xpix, ypix, look_vector
				get_vector_earth_intersection, look_vector, times, vector_location, 0.0D, return_code
				xgridloc = (vector_location(*,1)-minlon)/hiresgridsize
				ygridloc = (vector_location(*,0)-minlat)/hiresgridsize
				for idx = 0,13*13-1,1 do begin
					if (xgridloc(idx) ge 0 and xgridloc(idx) lt fpgridsz and $
					    ygridloc(idx) ge 0 and ygridloc(idx) lt fpgridsz) then begin
						if (bgfpgrid(xgridloc(idx),ygridloc(idx)) eq 0) then begin
							bgfpgrid(xgridloc(idx),ygridloc(idx)) = temporary(bgfpgrid(xgridloc(idx),ygridloc(idx))) + $
									inbg(xgrid(inplot(inidx)),ygrid(inplot(inidx)))
						endif
					endif
				endfor
			endfor
		
		endif
	endfor
	
	window,20,colors=220,xsize=fpgridsz,ysize=fpgridsz,retain=2
	tvscl,bgfpgrid
	window,21,colors=220,xsize=fpgridsz,ysize=fpgridsz,retain=2
	wset,18
	evimg = bytarr(fpgridsz,fpgridsz)
	evimg = tvrd()
	wset,20
	bgimg = bytarr(fpgridsz,fpgridsz)
	bgimg = tvrd()
	evimg = fix(evimg*64./max(evimg))
	bgimg = fix(bgimg*64./max(bgimg))
	newimg = bytarr(fpgridsz,fpgridsz)
	whereevents = where(evimg ne 0,whevct)
	wherenoevents = where(evimg eq 0,whnoevct)
	newimg(wherenoevents) = bgimg(wherenoevents)
	newimg(whereevents) = newimg(whereevents)+evimg(whereevents)+65.
	wset,21
	tv,newimg
	newimg = 0 & bgimg = 0 & evimg = 0 & bgfpgrid = 0 & inbg = 0 & inbg0 = 0 & inbg1 = 0 & xgridloc = 0 & ygridloc = 0 
	tmpgrid = 0 & lats = 0 & lons = 0
end
		
pro flash_barplot, theflashes, numflashes
	common datafiles, orbit, fname
	common lisbase, lisb
	common lisapp_colors,red,green,blue
	common sensor_read, this_sensor, file_format
	
	tvlct,red,green,blue
	
	earliest_time = 9e20
	latest_time = -9e20
	num_groups = 0

	for fct = 0, numflashes-1, 1 do begin	
		thisflash = theflashes(fct)
		if (thisflash.TAI93_time lt earliest_time) then begin
			earliest_time = thisflash.TAI93_time
		endif
		if (thisflash.TAI93_time + thisflash.delta_time gt latest_time) then begin
			latest_time = thisflash.TAI93_time + thisflash.delta_time
		endif
		num_groups = num_groups + thisflash.child_count
	endfor
	
	if (latest_time eq earliest_time) then begin
		latest_time = earliest_time + 1.
	endif

	delms = long((latest_time-earliest_time)*1000.0D)
	
	; Histogram
	
	theevents = replicate({an_event},total(theflashes(*).grandchild_count))
	thegroups = replicate({a_group},total(theflashes(*).child_count))
	eidx = 0L & gidx = 0L
	for fct=0L,numflashes-1L,1L do begin
		thisflash = theflashes(fct)
		for gct = 0L,thisflash.child_count-1L,1L do begin
			thisgroup = orbit.point.lightning.group(thisflash.child_address+gct)
			thegroups(gidx) = thisgroup
			gidx = gidx+1
			for ect = 0L, thisgroup.child_count-1L,1L do begin
				thisevent = orbit.point.lightning.event(thisgroup.child_address+ect)
				theevents(eidx) = thisevent
				eidx = eidx + 1L
			endfor
		endfor
	endfor
	
	if (eidx gt 1) then begin
		h = histogram(theevents(*).amplitude,max=128)
		xvert = intarr(4*130) & yvert=intarr(4*130)
		for hct = 0,128,1 do begin
			xvert(hct*4) = hct
			xvert(hct*4+1) = hct
			xvert(hct*4+2) = hct+1
			xvert(hct*4+3) = hct+1
			yvert(hct*4) = 0
			yvert(hct*4+1) = h(hct)
			yvert(hct*4+2) = h(hct)
			yvert(hct*4+3) = 0
		endfor
		xvert(129*4) = 0 & yvert(129*4)=0
	endif else begin
		h = intarr(128)
		xvert = intarr(4) & yvert = intarr(4)
		h(theevents(0).amplitude) = theevents(0).amplitude
		xvert(0) = theevents(0).amplitude
		xvert(1) = theevents(0).amplitude
		xvert(2) = theevents(0).amplitude+1
		xvert(3) = theevents(0).amplitude+1
		yvert(0) = 0
		yvert(1) = 1
		yvert(2) = 1
		yvert(3) = 0
	endelse
	window,17,colors=220,xsize=600,ysize=400,retain=2
	plot,h,/xstyle,psym=3
	polyfill,xvert,yvert,/data,color=129

	; Barplot
	
	pysize = 250
	
	window,22,colors=220,xsize=delms+2,ysize=pysize,retain=2
	
	totradiance = 0
	for ms = 0, delms, 2 do begin
		time1 = ms
		time2 = ms+1
		grthisms = where(long((thegroups.TAI93_time-earliest_time)*1000.D) eq time1 or $
					 long((thegroups.TAI93_time-earliest_time)*1000.D) eq time2, grthismsct)
		if (grthismsct gt 0) then begin
			evthisms = 0
			for gct = 0, grthismsct-1, 1 do begin
				totradiance = totradiance + $
						thegroups(grthisms(gct)).radiance
				evthisms = evthisms+thegroups(grthisms(gct)).child_count
			endfor
			vertcolor = fix((float(evthisms)/128.)*64. + 65.)
		endif else begin
			vertcolor = 48
		endelse
		verticesx = [ms,ms,ms+2,ms+2]
		if (totradiance eq 0) then begin
			yrad = 0
		endif else begin
			;yrad = fix((alog10(totradiance)-3.0)*pysize/6.)
			yrad = fix((pysize-5.)*totradiance/total(theflashes(*).radiance)+5.)
		endelse
		verticesy = [0,yrad,yrad,0]
		polyfill,verticesx,verticesy,color=vertcolor,/device
		; print,ms,verticesx,verticesy,vertcolor
	endfor

	prevsec = long(earliest_time)
	nextsec = long(latest_time+1)
	
	for onesectic = prevsec,nextsec,1L do begin
		for tenthsectic = 1,9,1 do begin
			tenthticx = long((double(onesectic) + double(tenthsectic)/10. - earliest_time)*1000.0D)			
			; print,onesectic,tenthsectic,tenthticx
			if (tenthticx gt 0 and tenthticx lt (delms+2)) then begin
				plots,[tenthticx,tenthticx],[0,pysize],linestyle=1,color=32,/device
			endif
		endfor
		onesecticx = long((double(onesectic) - earliest_time)*1000.0D)
		if (onesecticx gt 0 and onesecticx lt (delms+2)) then begin
			plots,[onesecticx,onesecticx],[0,pysize],linestyle=4,color=32,/device
		endif
	endfor
	
		
	
end

function select_f_toggle, event

	common datafiles, orbit, fname
	common lisbase, lisb
	common a_selected,f_selected,g_selected,e_selected,f2_selected,g2_selected,aselect,fselect,gselect,eselect,f2select,g2select
	common lists, alist,flist,flist2,glist,glist2,elist
	common listids, fbflashlist, fbflash2list, fgrouplist, fbgroup2list
	common lisapp_colors, red,green,blue
	
	widget_control, event.id, get_value=buttonvalue
	widget_control, event.id, get_uvalue=fidx
	
	if (f_selected(fidx) ne (1-buttonvalue)) then begin
		f_selected(fidx) = 1-buttonvalue
		toggle_flist_entry,fidx
		widget_control,fbflashlist,set_list_select=fidx
	endif
	return, fidx
end

pro flash_window, theseflashes, numflashes,caller

	common datafiles, orbit, fname
	common lisbase, lisb
	common lisapp_colors,red,green,blue
	common sensor_read, this_sensor, file_format
	common a_selected,f_selected,g_selected,e_selected,f2_selected,g2_selected,aselect,fselect,gselect,eselect,f2select,g2select
	common listids, fbflashlist, fbflash2list, fgrouplist, fbgroup2list
	common lists, alist,flist,flist2,glist,glist2,elist
	common vallabs, fbcval1, fbcval2, fbcval3, fbcval4, fbcval5, fbcval6, fbcval7, fbcval8, fbcval9, fbcval10
	common drawwins2, fbt2td1, fbt2td2, fbt2bd1, fbbd1
	common geobounds, minlat,maxlat,minlon,maxlon
	common hist_coords, hist_p, hist_x, hist_y, hist_z
	common bar_coords, bar_p, bar_x, bar_y, bar_z
	common bar_timestuff, TAIstart, msinc
	common flash_buts, fselect_all, fdeselect_all, g2select_all, g2deselect_all, fbcbut_gran, fbcbut_evtab, fbcbut_done
	common pointdata, theareas, theflashes, thegroups, theevents
	common fwbg, bgpixmap
	common plotwins, footprint_id, thresh_id, bar_id, hist_id
	common plotpds, pixpd, latlonpd, histpd, barpd
	common fwcaller, thecaller
	common fwcom2, fbcbut_fplot
		
	thecaller = caller
	widget_control,thecaller,sensitive=0
	
	thegroups = replicate({a_group},total(theseflashes(*).child_count))
	gidx=0
	glist2 = strarr(total(theseflashes(*).child_count))
	flist = strarr(numflashes)
	for i=0,numflashes-1,1 do begin
		dtarr = new_datetime_structure(1)
		dtarr.TAI93 = theseflashes(i).TAI93_time
		calc_datetime,dtarr
		flist(i) = string(format='("  ",(I5.5)," ",(I2.2),":",(I2.2),":",(F6.3)," ",(F6.2)," ",(F7.2)," ",(I3))', $
			theseflashes(i).address, dtarr.utc.hour, dtarr.utc.minute, dtarr.utc.second, 			  $
			theseflashes(i).location(0), theseflashes(i).location(1), theseflashes(i).child_count)
		for j=0L,theseflashes(i).child_count-1L,1L do begin
			thegroups(gidx) = orbit.point.lightning.group(theseflashes(i).child_address+j)
			dtarr = new_datetime_structure(1)
			dtarr.TAI93 = thegroups(gidx).TAI93_time
			calc_datetime,dtarr
			glist2(gidx) = string(format='("  ",(I5.5)," ",(I2.2),":",(I2.2),":",(F6.3)," ",(F6.2)," ",(F7.2)," ",(I3))', $
			theseflashes(i).child_address+j, dtarr.utc.hour, dtarr.utc.minute, dtarr.utc.second, 			  $
			thegroups(gidx).location(0), thegroups(gidx).location(1), thegroups(gidx).child_count)
			gidx = gidx+1
		endfor
	endfor

	emptylist = [' ']
	
	f_selected = bytarr(numflashes)
	f_selected(*) = 1
	g2_selected = bytarr(gidx)
	g2_selected(*) =1

	fbt2td1_choices = ['Pix, MaxRad']
	fbt2td2_choices = ['Geo,Pt,MaxRad','Geo,Fp,MaxRad', 'Geo,Fp,MaxRad,BG']
	fbt2bd1_choices = ['Event Amplitude PDF','Event Log Amplitude PDF','Event Log Amplitude PDF (2)',$
					'Event Amplitude CDF','Event Log Amplitude CDF']
	fbbd1_choices   = ['Flash Time Series Plot']
	
	fb0  = widget_base(group_leader=lisb,column=2,title='Flash Analysis',space=1)
	 fbc =  widget_base(fb0,row=2,/align_center)
	  fbcbut = widget_base(fbc,row=11,/align_center)
	   fbutbase = widget_base(fbcbut,frame=1,row=4)
	    flab = widget_label(fbutbase,value='Flashes')
	    fselect = cw_bgroup(fbutbase,['Selected','Deselected'],/exclusive,event_func='select_f_toggle',/no_release,xsize=100)
	    fselect_all = widget_button(fbutbase,value='Choose All',/align_center,xsize=100)
	    fdeselect_all = widget_button(fbutbase,value='Clear All',/align_center,xsize=100)
	   gbutbase = widget_base(fbcbut,frame=1,row=4)
	    g2lab = widget_label(gbutbase,value='Groups')
	    g2select = cw_bgroup(gbutbase,['Selected','Deselected'],/exclusive,event_func='select_g2_toggle',/no_release,xsize=100)
	    g2select_all = widget_button(gbutbase,value='Choose All',/align_center,xsize=100)
	    g2deselect_all = widget_button(gbutbase,value='Clear All',/align_center,xsize=100)
	   obutbase = widget_base(fbcbut,frame=1,row=5)
	    olab = widget_label(obutbase,value='Actions',xsize=100)		
	    fbcbut_gran   = widget_button(obutbase,value='Make Movie',xsize=100)
	    fbcbut_fplot = widget_button(obutbase,value='Fancy Plot',xsize=100)
	    fbcbut_evtab  = widget_button(obutbase,value='Event Vdata',xsize=100)
	    fbcbut_done   = widget_button(obutbase,value='Done',xsize=100)
	  fbclab = widget_base(fbc,row=8,/align_center)
	   fbclab1 = widget_label(fbclab,value='(X,Y):  ')
	   fbcval1 = widget_label(fbclab,value='-----------------')
	   fbclab2 = widget_label(fbclab,value='(Lat,Lon):  ')
	   fbcval2 = widget_label(fbclab,value='-----------------')
	   fbclab3 = widget_label(fbclab,value='TAI93 Time:')
	   fbcval3 = widget_label(fbclab,value='-----------------')
	   fbclab4 = widget_label(fbclab,value='UTC Time:')
	   fbcval4 = widget_label(fbclab,value='-----------------')
	 fb  = widget_base(fb0,row=2,/align_left,space=1)
	  fbt = widget_base(fb,column=2,/align_center,/base_align_center,space=1)
	   fbt1 = widget_base(fbt,row=4,/align_center,/base_align_center)
	    fbt1flashlab = widget_label(fbt1,value='Flashes')
	    fbflashlist = widget_list(fbt1,ysize=16,value=flist,/align_center,xsize=40,event_pro='handle_flist')
	    fbt1grouplab = widget_label(fbt1,value='Groups')
	    fbgroup2list = widget_list(fbt1,ysize=16,value=emptylist,/align_center,xsize=40,event_pro='handle_g2list')
	   fbt2 = widget_base(fbt,row=2,space=1)
	    fbt2t = widget_base(fbt2,column=2,space=1)
	     fbt2t1 = widget_base(fbt2t,row=2)
	      pixpd  = widget_droplist(fbt2t1,/align_center,value=fbt2td1_choices)
	      fbt2td1 = widget_draw(fbt2t1,colors=220,xsize=256,ysize=256,retain=2,/motion_events)
	      fbt2t2 = widget_base(fbt2t,row=2)
	      latlonpd = widget_droplist(fbt2t2,/align_center,value=fbt2td2_choices)
	      fbt2td2 = widget_draw(fbt2t2,colors=220,xsize=256,ysize=256,retain=2,/motion_events)
	    fbt2b = widget_base(fbt2,row=2,space=1)
	     histpd = widget_droplist(fbt2b,/align_center,value=fbt2bd1_choices)
	     fbt2bd1 = widget_draw(fbt2b,colors=220,xsize=522,ysize=150,retain=2,/motion_events)
	  fbb = widget_base(fb,row=2,/align_center,/base_align_center)  
	   barpd = widget_droplist(fbb,/align_center,value=fbbd1_choices)		
	   fbbd1 = widget_draw(fbb,retain=2,colors=220,xsize=800,ysize=136,/motion_events)

	widget_control,fb0,/realize
	xmanager,'Flash Analysis',fb0,event='flash_window_events',/no_block
	widget_control,fbt2td1,get_value=footprint_id
	widget_control,fbt2td2,get_value=thresh_id
	widget_control,fbt2bd1,get_value=hist_id
	widget_control,fbbd1,get_value=bar_id
	widget_control,fselect,sensitive=1
	widget_control,g2select,sensitive=1
	
	
	theevents = replicate({an_event},total(theseflashes(*).grandchild_count))
	thegroups = replicate({a_group},total(theseflashes(*).child_count))

	eidx = 0L & gidx = 0L
	for fct=0L,numflashes-1L,1L do begin
		thisflash = theseflashes(fct)
		for gct = 0L,thisflash.child_count-1L,1L do begin
			thisgroup = orbit.point.lightning.group(thisflash.child_address+gct)
			thegroups(gidx) = thisgroup
			gidx = gidx+1L
			for ect = 0L, thisgroup.child_count-1L,1L do begin
				thisevent = orbit.point.lightning.event(thisgroup.child_address+ect)
				theevents(eidx) = thisevent
				eidx = eidx + 1L
			endfor
		endfor
	endfor
	TAIstart = min(theevents.TAI93_time)
	
	; Histogram

	selection=0
	case selection of 
		0: begin
				event_histogram, theevents, hist_id
		   end
		1: begin
				event_loghistogram, theevents, hist_id
		   end
		2: begin
				event_loghistogram2, theevents, hist_id
		   end
		3: begin
				event_cdf, theevents, hist_id
		   end
		4: begin
				event_logcdf, theevents, hist_id
		   end
	endcase
	widget_control, histpd, set_uvalue=0
		
	; Pixel plot
	
	pixel_plot, theevents, footprint_id

	; Barplot
	
	flash_barplot, theseflashes, thegroups, theevents, bar_id
	
	; Geo plot

	minlat = 0 & minlon = 0 & maxlat = 0 & maxlon = 0
	
	widget_control,latlonpd,set_uvalue=0
	geo_plot,1, theevents, thresh_id,0,minlat,maxlat,minlon,maxlon,bgpixmap
	
	f_selected(*) = 0 & g2_selected(*) = 0
	
	theflashes = theseflashes
	
end

pro flash_window_events, event
	common datafiles, orbit, fname
	common lisbase, lisb
	common fullpaths, app_path, data_path, bg_path, output_path
	common lisapp_colors,red,green,blue
	common sensor_read, this_sensor, file_format
	common a_selected,f_selected,g_selected,e_selected,f2_selected,g2_selected,aselect,fselect,gselect,eselect,f2select,g2select
	common listids, fbflashlist, fbflash2list, fgrouplist, fbgroup2list
	common lists, alist,flist,flist2,glist,glist2,elist
	common drawwins2, fbt2td1, fbt2td2, fbt2bd1, fbbd1
	common vallabs, fbcval1, fbcval2, fbcval3, fbcval4, fbcval5, fbcval6, fbcval7, fbcval8, fbcval9, fbcval10
	common geobounds, minlat,maxlat,minlon,maxlon
	common hist_coords, hist_p, hist_x, hist_y, hist_z
	common bar_coords, bar_p, bar_x, bar_y, bar_z
	common bar_timestuff, TAIstart, msinc
	common flash_buts, fselect_all, fdeselect_all, g2select_all, g2deselect_all, fbcbut_gran, fbcbut_evtab, fbcbut_done
	common pointdata, theareas, theflashes, thegroups, theevents
	common fwbg, bgpixmap
	common plotwins, footprint_id, thresh_id, bar_id, hist_id
	common plotpds, pixpd, latlonpd, histpd, barpd
	common fwcaller, thecaller
	common fwcom2, fbcbut_fplot
	
	case 1 of 
		(event.id eq fbcbut_done):begin
			widget_control,thecaller,/sensitive
			theareas = 0 & theflashes = 0 & thegroups = 0 & theevents = 0 & bgpixmap = 0 
			a_selected = 0 & f_selected = 0 & g_selected = 0 & e_selected = 0 
			f2_selected = 0 & g2_selected = 0 & aselect = 0 & fselect = 0 & gselect = 0 & eselect = 0
			fbflashlist = 0 & fbflash2list = 0 & fgrouplist = 0 & fbgroup2list = 0
			widget_control,event.top,/destroy
			end
		(event.id eq pixpd):begin
		end
		(event.id eq fbcbut_fplot): begin
			show_density_map, fname, orbit, (maxlat+minlat)/2., (maxlon+minlon)/2., lisb
		end
		(event.id eq histpd):begin
			wset,hist_id
			widget_control,histpd,set_uvalue=event.index
			xvert = intarr(5) & yvert = intarr(5)	
			xvert(0) = 0 & xvert(1)=0 & xvert(2) = 256 & xvert(3) = 256 & xvert(4) = 0
			yvert(0) = 0 & yvert(1)=256 & yvert(2) = 256 & yvert(3) = 0 & yvert(4) = 0
			polyfill,xvert,yvert,color=0,/device
			case event.index of 
				0: begin
						event_histogram, theevents, hist_id
				   end
				1: begin
						event_loghistogram, theevents, hist_id
				   end
				2: begin
						event_loghistogram2, theevents, hist_id
				   end
				3: begin
						event_cdf, theevents, hist_id
				   end
				4: begin
						event_logcdf, theevents, hist_id
				   end
			endcase
		end
		(event.id eq barpd):begin
		end
		(event.id eq latlonpd):begin
			wset,thresh_id
			widget_control,latlonpd,set_uvalue=event.index
			xvert = intarr(5) & yvert = intarr(5)
			xvert(0) = 0 & xvert(1)=0 & xvert(2) = 256 & xvert(3) = 256 & xvert(4) = 0
			yvert(0) = 0 & yvert(1)=256 & yvert(2) = 256 & yvert(3) = 0 & yvert(4) = 0
			polyfill,xvert,yvert,color=0,/device
			case event.index of 
				0: begin
					geo_plot, 1, theevents, thresh_id, 0, minlat,maxlat,minlon,maxlon, bgpixmap
				   end
				1: begin
					geo_plot, 2, theevents, thresh_id, 0, minlat,maxlat,minlon,maxlon, bgpixmap
				   end
				2: begin
					geo_plot, 2, theevents, thresh_id, 1, minlat,maxlat,minlon,maxlon, bgpixmap
				   end
			endcase
		end
		(event.id eq fbt2td1): begin
			;help,/struct,event
			xpix = fix(event.x/2.)
			ypix = fix(event.y/2.)
			msg = string(format='("(",(I3),",",(I3),")")',xpix,ypix)
			widget_control,fbcval1,set_value=msg
			msg = '     '
			widget_control,fbcval2,set_value=msg
			widget_control,fbcval3,set_value=msg
			widget_control,fbcval4,set_value=msg
		end
		(event.id eq fbt2td2): begin
			dellat = maxlat-minlat & dellon = maxlon-minlon
			lat = minlat + (event.y/256.)*dellat
			lon = minlon + (event.x/256.)*dellon
			msg = string(format='("(",(F6.2),",",(F7.2),")")',lat,lon)
			widget_control,fbcval2,set_value=msg
			msg = '     '
			widget_control,fbcval1,set_value=msg
			widget_control,fbcval3,set_value=msg
			widget_control,fbcval4,set_value=msg
		end
		(event.id eq fbt2bd1): begin
			msg = '    '
			widget_control,fbcval1,set_value=msg
			widget_control,fbcval2,set_value=msg
			widget_control,fbcval3,set_value=msg
			widget_control,fbcval4,set_value=msg
		end
		(event.id eq fbbd1): begin
			TAI93 = TAIstart + double(event.x*msinc*.001)/800.0
			msg = string(format='((F13.3))',TAI93)
			widget_control,fbcval3,set_value=msg
			dt = new_datetime_structure(1)
			dt.tai93 = TAI93
			calc_datetime,dt
			msg = string(format='((I2.2),":",(I2.2),":",(F6.3))',dt.utc.hour,dt.utc.minute,dt.utc.second)
			widget_control,fbcval4,set_value=msg
			msg = '     '
			widget_control,fbcval1,set_value=msg
			widget_control,fbcval2,set_value=msg
		end
		(event.id eq fbcbut_gran):begin
				; zoom_overpass, min(theevents[*].TAI93_time),max(theevents[*].TAI93_time),.5D,output_path
				movie_prefs,lisb,min(theevents[*].TAI93_time),max(theevents[*].TAI93_time)
			end
		(event.id eq fbcbut_evtab):begin
			evsort = sort(theevents(*).TAI93_time)
			evindices = theevents(evsort).address
			ev_table,event,orbit.point.lightning.event(evindices).address
			end
		(event.id eq fselect_all):begin
			fssize = size(f_selected)
			if (fssize(0) eq 0) then begin
				num_flashes = 1
			endif else begin
				num_flashes = fssize(1)
			endelse
			f_selected(*)=1
			widget_control,fselect,set_value=0
			for i=0,num_flashes-1,1 do begin
				entry = flist(i)
				retval = string(entry)
				bretval = byte(retval)
				bretval(0) = 42
				entry = string(bretval)
				flist(i)=entry
			endfor
			widget_control,fbflashlist,set_value=flist							
			chosen = where(f_selected eq 1, chosen_ct)
			if (chosen_ct ge 1) then begin
				theevents = replicate({an_event},total(theflashes(chosen).grandchild_count))
				thegroups = replicate({a_group},total(theflashes(chosen).child_count))
				glist2 = strarr(total(theflashes(chosen).child_count))
				eidx = 0L & gidx = 0L
				for fct=0L,num_flashes-1L,1L do begin
					if (f_selected(fct) eq 1) then begin
						thisflash = theflashes(fct)
						for gct = 0L,thisflash.child_count-1L,1L do begin
							thisgroup = orbit.point.lightning.group(thisflash.child_address+gct)
							thegroups(gidx) = thisgroup
							dtarr = new_datetime_structure(1)
							dtarr.TAI93 = thegroups(gidx).TAI93_time
							calc_datetime,dtarr					
							glist2(gidx) = string(format='("* ",(I5.5)," ",(I2.2),":",(I2.2),":",(F6.3)," ",(F6.2)," ",(F7.2)," ",(I3))', $
							thisflash.child_address+gct, dtarr.utc.hour, dtarr.utc.minute, dtarr.utc.second, 			  $
							thegroups(gidx).location(0), thegroups(gidx).location(1), thegroups(gidx).child_count)
				 					gidx = gidx+1L
							for ect = 0L, thisgroup.child_count-1L,1L do begin
								thisevent = orbit.point.lightning.event(thisgroup.child_address+ect)
								theevents(eidx) = thisevent
								eidx = eidx + 1L
							endfor
						endfor
					endif
				endfor		
				g2_selected = bytarr(gidx)
				g2_selected(*) =1
				widget_control,fbgroup2list,set_value=glist2
				
				widget_control,histpd,get_uvalue=selection
				case selection of 
					0: begin
							event_histogram, theevents, hist_id
					   end
					1: begin
							event_loghistogram, theevents, hist_id
					   end
					2: begin
							event_loghistogram2, theevents, hist_id
					   end
					3: begin
							event_cdf, theevents, hist_id
					   end
					4: begin
							event_logcdf, theevents, hist_id
					   end
				endcase
										
				pixel_plot, theevents, footprint_id
				
				flash_barplot, theflashes(chosen), thegroups, theevents, bar_id
				
				wset, thresh_id
				tv,bgpixmap	
				widget_control,latlonpd,get_uvalue=selection
				case selection of 
					0: begin
						geo_plot, 1, theevents, thresh_id, 0, minlat,maxlat,minlon,maxlon, bgpixmap
					   end
					1: begin
						geo_plot, 2, theevents, thresh_id, 0, minlat,maxlat,minlon,maxlon, bgpixmap
					   end
					2: begin
						geo_plot, 2, theevents, thresh_id, 1, minlat,maxlat,minlon,maxlon, bgpixmap
					   end
				endcase
			endif
			end
		(event.id eq fdeselect_all):begin
			fssize = size(f_selected)
			if (fssize(0) eq 0) then begin
				num_flashes = 1
			endif else begin
				num_flashes = fssize(1)
			endelse
			f_selected(*)=0
			widget_control,fselect,set_value=1
			for i=0,num_flashes-1,1 do begin
				entry = flist(i)
				retval = string(entry)
				bretval = byte(retval)
				bretval(0) = 32
				entry = string(bretval)
				flist(i)=entry
			endfor
			widget_control,fbflashlist,set_value=flist	
			wset,thresh_id		
			tv,bgpixmap
			wset,bar_id
			polyfill,[0,0,1,1,0],[0,1,1,0,0],/normal,color=0
			wset,hist_id
			polyfill,[0,0,1,1,0],[0,1,1,0,0],/normal,color=0				
			theevents = replicate({an_event},1)
			thegroups = replicate({a_group},1)
			glist2 = strarr(1)
			glist2(0) = '    '		
			g2_selected = bytarr(1)
			g2_selected(*) =0
			widget_control,fbgroup2list,set_value=glist2
			end
		(event.id eq g2select_all):begin
			g2ssize = size(g2_selected)
			if (g2ssize(0) eq 0) then begin
				num_groups = 1
			endif else begin
				num_groups = g2ssize(1)
			endelse
			g2_selected(*)=1
			widget_control,g2select,set_value=0
			for i=0,num_groups-1,1 do begin
				entry = glist2(i)
				retval = string(entry)
				bretval = byte(retval)
				bretval(0) = 42
				entry = string(bretval)
				glist2(i)=entry
			endfor
			widget_control,fbgroup2list,set_value=glist2							
			end
		(event.id eq g2deselect_all):begin
			g2ssize = size(g2_selected)
			if (g2ssize(0) eq 0) then begin
				num_groups = 1
			endif else begin
				num_groups = g2ssize(1)
			endelse
			g2_selected(*)=0
			widget_control,g2select,set_value=1
			for i=0,num_groups-1,1 do begin
				entry = glist2(i)
				retval = string(entry)
				bretval = byte(retval)
				bretval(0) = 32
				entry = string(bretval)
				glist2(i)=entry
			endfor
			widget_control,fbgroup2list,set_value=glist2							
			end
				
		
	endcase
end

pro event_histogram, theevents, hist_id
	common datafiles, orbit, fname
	common lisbase, lisb
	common lisapp_colors,red,green,blue
	common sensor_read, this_sensor, file_format
	common hist_coords, hist_p, hist_x, hist_y, hist_z

	esz = size(theevents)
	if (esz(0) ne 0) then begin
		if (esz(1) gt 1) then begin
			h = histogram(theevents(*).amplitude,max=128)
			xvert = intarr(4*130) & yvert=intarr(4*130)
			for hct = 0,128,1 do begin
				xvert(hct*4) = hct
				xvert(hct*4+1) = hct
				xvert(hct*4+2) = hct+1
				xvert(hct*4+3) = hct+1
				yvert(hct*4) = 0
				yvert(hct*4+1) = h(hct)
				yvert(hct*4+2) = h(hct)
				yvert(hct*4+3) = 0
			endfor
			xvert(129*4) = 0 & yvert(129*4)=0
		endif else begin
			h = intarr(128)
			xvert = intarr(4) & yvert = intarr(4)
			h(theevents(0).amplitude) = theevents(0).amplitude
			xvert(0) = theevents(0).amplitude
			xvert(1) = theevents(0).amplitude
			xvert(2) = theevents(0).amplitude+1
			xvert(3) = theevents(0).amplitude+1
			yvert(0) = 0
			yvert(1) = 1
			yvert(2) = 1
			yvert(3) = 0		
		endelse
	endif else begin
		h = intarr(128)
		xvert = intarr(4) & yvert = intarr(4)
		h(theevents(0).amplitude) = theevents(0).amplitude
		xvert(0) = theevents(0).amplitude
		xvert(1) = theevents(0).amplitude
		xvert(2) = theevents(0).amplitude+1
		xvert(3) = theevents(0).amplitude+1
		yvert(0) = 0
		yvert(1) = 1
		yvert(2) = 1
		yvert(3) = 0
	endelse
	wset,hist_id
	tvlct,red,green,blue
	xtva =[8,16,24,32,40,48,56,64,72,80,88,96,104,112,120,128]
	plot,h,/xstyle,psym=3,xmargin=[5,1],ymargin=[2,1],$
				xticklen=1, xticks=15, xtickv=xtva,color=63
	polyfill,xvert,yvert,/data,color=129
	hist_p = !P & hist_x = !X & hist_y = !Y & hist_z = !Z
end

pro event_cdf, theevents, hist_id
	common datafiles, orbit, fname
	common lisbase, lisb
	common lisapp_colors,red,green,blue
	common sensor_read, this_sensor, file_format
	common hist_coords, hist_p, hist_x, hist_y, hist_z

	esz = size(theevents)
	if (esz(0) ne 0) then begin
		if (esz(1) gt 1) then begin
			h0 = histogram(theevents(*).amplitude,max=128)
			h = fltarr(129)
			xvals = findgen(128)
			lxvals = fltarr(128)
			lxvals(7:127) = alog10(xvals(7:127))
			lxvals(0:6) = lxvals(7)
			for i=0,128,1 do begin
				h(i) = float(total(h0(0:i)))/float(total(h0(*)))
			endfor
			xvert = fltarr(4*130) & yvert=fltarr(4*130)
			for hct = 1,128,1 do begin
				xvert(hct*4) = (float(hct))
				xvert(hct*4+1) = (float(hct))
				xvert(hct*4+2) =(float(hct+1))
				xvert(hct*4+3) = (float(hct+1))
				yvert(hct*4) = 0
				yvert(hct*4+1) = h(hct)
				yvert(hct*4+2) = h(hct)
				yvert(hct*4+3) = 0
			endfor
			xvert(129*4) = 0 & yvert(129*4)=0
		endif else begin
			h = intarr(128)
			xvert = intarr(4) & yvert = intarr(4)
			h(theevents(0).amplitude) = theevents(0).amplitude
			xvert(0) = theevents(0).amplitude
			xvert(1) = theevents(0).amplitude
			xvert(2) = theevents(0).amplitude+1
			xvert(3) = theevents(0).amplitude+1
			yvert(0) = 0
			yvert(1) = 1
			yvert(2) = 1
			yvert(3) = 0		
		endelse
	endif else begin
		h = intarr(128)
		xvert = intarr(4) & yvert = intarr(4)
		h(theevents(0).amplitude) = theevents(0).amplitude
		xvert(0) = theevents(0).amplitude
		xvert(1) = theevents(0).amplitude
		xvert(2) = theevents(0).amplitude+1
		xvert(3) = theevents(0).amplitude+1
		yvert(0) = 0
		yvert(1) = 1
		yvert(2) = 1
		yvert(3) = 0
	endelse
	wset,hist_id
	tvlct,red,green,blue
	xtva =[8,16,24,32,40,48,56,64,72,80,88,96,104,112,120,128]
	plot,h,/xstyle,psym=3,xmargin=[5,3],ymargin=[2,1],yticklen=1,yticks=4,ytickv=[.1,.25,.5,.75,.9],$
		xticklen=1, xticks=15, xtickv=xtva,color=63
;	lxtva = alog10(xtva)
;	plot,lxvals,h,/xstyle,psym=3,xmargin=[5,3],ymargin=[2,1],yticklen=1,yticks=4,ytickv=[.1,.25,.5,.75,.9],$
;		xticklen=1, xticks=15, xtickv=lxtva, XTICKFORMAT='(F4.2)',color=63
;	PAT = BYTARR(10,10)	;Define pattern array as 10 by 10.
;	PAT(5,5) = 255	;Set center pixel to bright.
	polyfill,xvert,yvert,/data,color=129
	;,PAT = PAT
	hist_p = !P & hist_x = !X & hist_y = !Y & hist_z = !Z
end

pro event_logcdf, theevents, hist_id
	common datafiles, orbit, fname
	common lisbase, lisb
	common lisapp_colors,red,green,blue
	common sensor_read, this_sensor, file_format
	common hist_coords, hist_p, hist_x, hist_y, hist_z

	esz = size(theevents)
	if (esz(0) ne 0) then begin
		if (esz(1) gt 1) then begin
			h0 = histogram(theevents(*).amplitude,max=128)
			h = fltarr(129)
			xvals = findgen(128)
			lxvals = fltarr(128)
			lxvals(7:127) = alog10(xvals(7:127))
			lxvals(0:6) = lxvals(7)
			for i=0,128,1 do begin
				h(i) = float(total(h0(0:i)))/float(total(h0(*)))
			endfor
			xvert = fltarr(4*130) & yvert=fltarr(4*130)
			for hct = 1,128,1 do begin
				xvert(hct*4) = alog10(float(hct))
				xvert(hct*4+1) = alog10(float(hct))
				xvert(hct*4+2) = alog10(float(hct+1))
				xvert(hct*4+3) = alog10(float(hct+1))
				yvert(hct*4) = 0
				yvert(hct*4+1) = h(hct)
				yvert(hct*4+2) = h(hct)
				yvert(hct*4+3) = 0
			endfor
			xvert(129*4) = 0 & yvert(129*4)=0
		endif else begin
			h = intarr(128)
			xvert = intarr(4) & yvert = intarr(4)
			h(theevents(0).amplitude) = theevents(0).amplitude
			xvert(0) = theevents(0).amplitude
			xvert(1) = theevents(0).amplitude
			xvert(2) = theevents(0).amplitude+1
			xvert(3) = theevents(0).amplitude+1
			yvert(0) = 0
			yvert(1) = 1
			yvert(2) = 1
			yvert(3) = 0		
		endelse
	endif else begin
		h = intarr(128)
		xvert = intarr(4) & yvert = intarr(4)
		h(theevents(0).amplitude) = theevents(0).amplitude
		xvert(0) = theevents(0).amplitude
		xvert(1) = theevents(0).amplitude
		xvert(2) = theevents(0).amplitude+1
		xvert(3) = theevents(0).amplitude+1
		yvert(0) = 0
		yvert(1) = 1
		yvert(2) = 1
		yvert(3) = 0
	endelse
	wset,hist_id
	tvlct,red,green,blue
	xtva =[8,16,24,32,40,48,56,64,72,80,88,96,104,112,120,128]
;	plot,h,/xstyle,psym=3,xmargin=[5,3],ymargin=[2,1],yticklen=1,yticks=4,ytickv=[.1,.25,.5,.75,.9],$
;		xticklen=1, xticks=15, xtickv=xtva,color=63
	lxtva = alog10(xtva)
	plot,lxvals,h,/xstyle,psym=3,xmargin=[5,3],ymargin=[2,1],yticklen=1,yticks=4,ytickv=[.1,.25,.5,.75,.9],$
		xticklen=1, xticks=15, xtickv=lxtva, XTICKFORMAT='(F4.2)',color=63
;	PAT = BYTARR(10,10)	;Define pattern array as 10 by 10.
;	PAT(5,5) = 255	;Set center pixel to bright.
	polyfill,xvert,yvert,/data,color=129
	;,PAT = PAT
	hist_p = !P & hist_x = !X & hist_y = !Y & hist_z = !Z
end

pro event_loghistogram, theevents, hist_id
	common datafiles, orbit, fname
	common lisbase, lisb
	common lisapp_colors,red,green,blue
	common sensor_read, this_sensor, file_format
	common hist_coords, hist_p, hist_x, hist_y, hist_z

	esz = size(theevents)
	if (esz(0) ne 0) then begin
		if (esz(1) gt 1) then begin
			h = histogram(theevents(*).amplitude,max=128)
			xvals = findgen(128)
			lxvals = fltarr(128)
			lxvals(7:127) = alog10(xvals(7:127))
			lxvals(0:6) = lxvals(7)
			xvert = fltarr(4*130) & yvert=fltarr(4*130)
			for hct = 1,128,1 do begin
				xvert(hct*4) = alog10(float(hct))
				xvert(hct*4+1) = alog10(float(hct))
				xvert(hct*4+2) = alog10(float(hct+1))
				xvert(hct*4+3) = alog10(float(hct+1))
				yvert(hct*4) = 0
				yvert(hct*4+1) = h(hct)
				yvert(hct*4+2) = h(hct)
				yvert(hct*4+3) = 0
			endfor
			xvert(129*4) = 0 & yvert(129*4)=0
		endif else begin
			h = intarr(128)
			xvert = intarr(4) & yvert = intarr(4)
			h(theevents(0).amplitude) = theevents(0).amplitude
			xvert(0) = theevents(0).amplitude
			xvert(1) = theevents(0).amplitude
			xvert(2) = theevents(0).amplitude+1
			xvert(3) = theevents(0).amplitude+1
			yvert(0) = 0
			yvert(1) = 1
			yvert(2) = 1
			yvert(3) = 0		
		endelse
	endif else begin
		h = intarr(128)
		xvert = intarr(4) & yvert = intarr(4)
		h(theevents(0).amplitude) = theevents(0).amplitude
		xvert(0) = theevents(0).amplitude
		xvert(1) = theevents(0).amplitude
		xvert(2) = theevents(0).amplitude+1
		xvert(3) = theevents(0).amplitude+1
		yvert(0) = 0
		yvert(1) = 1
		yvert(2) = 1
		yvert(3) = 0
	endelse
	wset,hist_id
	tvlct,red,green,blue
	xtva =[8,16,24,32,40,48,56,64,72,80,88,96,104,112,120,128]
;	plot,h,/xstyle,psym=3,xmargin=[5,3],ymargin=[2,1],yticklen=1,yticks=4,ytickv=[.1,.25,.5,.75,.9],$
;		xticklen=1, xticks=15, xtickv=xtva,color=63
	lxtva = alog10(xtva)
	plot,lxvals,h,/xstyle,psym=3,xmargin=[5,3],ymargin=[2,1],$
		xticklen=1, xticks=15, xtickv=lxtva, XTICKFORMAT='(F4.2)',color=63

;	plot,h,/xstyle,psym=3,xmargin=[5,1],ymargin=[2,1]
	polyfill,xvert,yvert,/data,color=129
	hist_p = !P & hist_x = !X & hist_y = !Y & hist_z = !Z
end

pro event_loghistogram2, theevents, hist_id
	common datafiles, orbit, fname
	common lisbase, lisb
	common lisapp_colors,red,green,blue
	common sensor_read, this_sensor, file_format
	common hist_coords, hist_p, hist_x, hist_y, hist_z

	esz = size(theevents)
	if (esz(0) ne 0) then begin
		if (esz(1) gt 1) then begin
			loghist = lonarr(15)
			histvals = bytarr(15)
			histvals = [7,8,11,16,23,32,45,64,77,81,87,96,108,125,128]	
			xvert = intarr(4*15) & yvert = intarr(4*15)
			for i=0,13,1 do begin
				einbin = where(theevents[*].amplitude gt histvals[i] and $
						    theevents[*].amplitude le histvals[i+1],einbinct)
				einbinct = max([einbinct,0])
				loghist[i] = einbinct
				xvert[i*4] = histvals[i]
				xvert[i*4+1] = histvals[i]
				xvert[i*4+2] = histvals[i+1]
				xvert[i*4+3] = histvals[i+1]
				yvert[i*4] = 0
				yvert[i*4+1] = loghist[i]
				yvert[i*4+2] = loghist[i]
				yvert[i*4+3] = 0
			endfor
			xvert[14*4] = 0 & yvert[14*4] = 0
		endif else begin
			loghist = lonarr(14)
			xvert = intarr(4) & yvert = intarr[4]
			einval = where(histvals gt theevents[0].amplitude)
			loghist[einval-1] = theevents[0].amplitude
			xvert[0] = histvals[einval-1]
			xvert[1] = histvals[einval-1]
			xvert[2] = histvals[einval]
			xvert[3] = histvals[einval]
			yvert[0] = 0
			yvert[1] = 1
			yvert[2] = 1
			yvert[3] = 0
		endelse
	endif else begin
		loghist = lonarr(14)
		xvert = intarr(4) & yvert = intarr[4]
		einval = where(histvals gt theevents[0].amplitude)
		loghist[einval-1] = theevents[0].amplitude
		xvert[0] = histvals[einval-1]
		xvert[1] = histvals[einval-1]
		xvert[2] = histvals[einval]
		xvert[3] = histvals[einval]
		yvert[0] = 0
		yvert[1] = 1
		yvert[2] = 1
		yvert[3] = 0
	endelse
	loghist[14] = 0
	wset,hist_id
	tvlct,red,green,blue
	xtva =[8,11,16,23,32,45,64,77,81,87,96,108,125]
	plot,histvals,loghist,/xstyle,psym=3,xmargin=[5,1],ymargin=[2,1],$
					xticklen=1, xticks=12, xtickv=xtva,color=63
	polyfill,xvert,yvert,/data,color=129
	hist_p = !P & hist_x = !X & hist_y = !Y & hist_z = !Z
end
		
pro pixel_plot, theevents, win_id
	common datafiles, orbit, fname
	common lisbase, lisb
	common lisapp_colors,red,green,blue
	common sensor_read, this_sensor, file_format

	wset, win_id
	tvlct,red,green,blue

	pixx = theevents(*).x_pixel
	pixy = theevents(*).y_pixel
	pixc = fix(65.+(theevents(*).amplitude/2))
	sortidx = sort(pixc)
	plot,intarr(2),intarr(2),xrange=[0,127],yrange=[0,127],position=[0,0,1,1],/nodata,xstyle=1,ystyle=1,xmargin=[0,0],ymargin=[0,0]	
	plots,pixx(sortidx),pixy(sortidx),color=pixc(sortidx),psym=3

end			

pro flash_barplot, theflashes, thegroups, theevents, win_id
	common datafiles, orbit, fname
	common lisbase, lisb
	common lisapp_colors,red,green,blue
	common sensor_read, this_sensor, file_format
	common bar_coords, bar_p, bar_x, bar_y, bar_z
	common bar_timestuff, TAIstart, msinc
	
	pysize = 100
	
	wset,win_id
	tvlct,red,green,blue

	earliest_time = 9e20
	latest_time = -9e20
	num_groups = 0

	fsz = size(theflashes)
	if (fsz(0) eq 0) then begin
		numflashes = 1
	endif else begin
		numflashes = fsz(1)
	endelse
	
	gsz = size(thegroups)
	if (gsz(0) eq 0) then begin
		numgroups = 1
	endif else begin
		numgroups = gsz(1)
	endelse
	
	esz = size(theevents)
	if (esz(0) eq 0) then begin
		numevents = 1
	endif else begin
		numevents = esz(1)
	endelse
	
	for fct = 0, numflashes-1, 1 do begin	
		thisflash = theflashes(fct)
		if (thisflash.TAI93_time lt earliest_time) then begin
			earliest_time = thisflash.TAI93_time
		endif
		if (thisflash.TAI93_time + thisflash.delta_time gt latest_time) then begin
			latest_time = thisflash.TAI93_time + thisflash.delta_time
		endif
		num_groups = num_groups + thisflash.child_count
	endfor
	
	if (latest_time eq earliest_time) then begin
		latest_time = earliest_time + 1.
	endif

	delms = long((latest_time-earliest_time)*1000.0D)


	plot,findgen(2),findgen(2),position=[0,0,1,1],/nodata,xstyle=5,ystyle=5,xmargin=[0,0],ymargin=[0,0],xrange=[0,delms+2],yrange=[0,100]
	totradiance = 0
	if (long(delms) lt 2000L) then begin
		msinc = 2L
	endif else begin if (long(delms) lt 10000L) then begin
			msinc = 10L 
		endif else begin if (long(delms) lt 50000L) then begin
			 	msinc = 25L
			 endif else begin if (long(delms) lt 100000L) then begin
			 		msinc = 100L
			 	endif else begin if (long(delms) lt 500000L) then begin
			 			msinc = 500L
			 		endif else begin
						msinc = 1000L
					endelse
				endelse
			 endelse
		endelse
	endelse
	for ms = 0L, long(delms), msinc do begin
		time1 = ms
		time2 = ms+msinc-1
		grthisms = where(long((thegroups.TAI93_time-earliest_time)*1000.D) ge time1 and $
					 long((thegroups.TAI93_time-earliest_time)*1000.D) le time2, grthismsct)
		if (grthismsct gt 0) then begin
			evthisms = 0
			for gct = 0, grthismsct-1, 1 do begin
				totradiance = totradiance + $
						thegroups(grthisms(gct)).radiance
				evthisms = evthisms+thegroups(grthisms(gct)).child_count
			endfor
			vertcolor = fix((float(evthisms)/(float(msinc)*32.))*64. + 65.)
		endif else begin
			vertcolor = 48
		endelse
		verticesx = [ms,ms,ms+msinc,ms+msinc]
		if (totradiance eq 0) then begin
			yrad = 0
		endif else begin
			;yrad = fix((alog10(totradiance)-3.0)*pysize/6.)
			yrad = fix((90.)*totradiance/total(theflashes(*).radiance)+5.)+11
		endelse
		verticesy = [0,yrad,yrad,0]
		polyfill,verticesx,verticesy,color=vertcolor,/data
		; print,ms,verticesx,verticesy,vertcolor
	endfor

	thems = lindgen(delms)
	theTAI = dblarr(delms)
	theTAI(*) = earliest_time + thems/1000.
	theindices = fix(theTAI-long(orbit.orbit_summary.TAI93_start))
	uniqvals = uniq(theindices)
	szuv = size(uniqvals)
	case szuv[0] of
		0:begin
			numuv = 1
		  end
		1:begin
			numuv = szuv[1]
		  end
	endcase
	fatal1 = (orbit.one_second(theindices[uniqvals]).alert_summary and 1)
	warn1 = (orbit.one_second(theindices[uniqvals]).alert_summary and 2)/2
	ok1 = ((warn1 or fatal1) eq 0)*129
	warn1 = warn1 * 130 & fatal1 = fatal1 * 131
	
	fatal2 = (orbit.one_second(theindices[uniqvals]).alert_summary and 4)/4
	warn2 = (orbit.one_second(theindices[uniqvals]).alert_summary and 8)/8
	ok2 = ((warn2 or fatal2) eq 0)*129
	warn2 = warn2 * 130 & fatal2 = fatal2 * 131
	
	fatal3 = (orbit.one_second(theindices[uniqvals]).alert_summary and 16)/16
	warn3 = (orbit.one_second(theindices[uniqvals]).alert_summary and 32)/32
	ok3 = ((warn3 or fatal3) eq 0)*129
	warn3 = warn3 * 130 & fatal3 = fatal3 * 131
	
	fatal4 = (orbit.one_second(theindices[uniqvals]).alert_summary and 64)/64
	warn4 = (orbit.one_second(theindices[uniqvals]).alert_summary and 128)/128
	ok4 = ((warn4 or fatal4) eq 0)*129
	warn4 = warn4 * 130 & fatal4 = fatal4 * 131

	for uvidx = 0, numuv-2, 1 do begin
		verticesx = [thems[uniqvals[uvidx]]-1000, thems[uniqvals[uvidx]]-1000, thems[uniqvals[uvidx+1]], thems[uniqvals[uvidx+1]]]
		verticesy = [0,.025*136,.025*136,0]
		thiscol = max([ok1[uvidx],warn1[uvidx],fatal1[uvidx]])
		polyfill,verticesx,verticesy,color=thiscol,/data
		verticesy = [.025*136,.05*136,.05*136,.025*136]
		thiscol = max([ok2[uvidx],warn2[uvidx],fatal2[uvidx]])
		polyfill,verticesx,verticesy,color=thiscol,/data
		verticesy = [.05*136,.075*136,.075*136,.05*136]
		thiscol = max([ok3[uvidx],warn3[uvidx],fatal3[uvidx]])
		polyfill,verticesx,verticesy,color=thiscol,/data
		verticesy = [.075*136,.1*136,.1*136,.075*136]
		thiscol = max([ok4[uvidx],warn4[uvidx],fatal4[uvidx]])
		polyfill,verticesx,verticesy,color=thiscol,/data		
	endfor
	prevsec = long(earliest_time)
	nextsec = long(latest_time+1)
	
	for onesectic = prevsec,nextsec,1L do begin
		if (long(delms) lt 10000L) then begin
			for tenthsectic = 1,9,1 do begin
				tenthticx = long((double(onesectic) + double(tenthsectic)/10. - earliest_time)*1000.0D)			
				; print,onesectic,tenthsectic,tenthticx
				if (tenthticx gt 0 and tenthticx lt (delms+2)) then begin
					plots,[tenthticx,tenthticx],[0,pysize],linestyle=1,color=32,/data
				endif
			endfor
		endif
		onesecticx = long((double(onesectic) - earliest_time)*1000.0D)
		if (onesecticx gt 0 and onesecticx lt (delms+2)) then begin
			plots,[onesecticx,onesecticx],[0,pysize],linestyle=4,color=32,/data
		endif
	endfor


	bar_p = !P & bar_x = !X & bar_y = !Y & bar_z = !Z
	msinc = delms
	
	
end

pro handle_flist, event

	common datafiles, orbit, fname
	common lisbase, lisb
	common a_selected,f_selected,g_selected,e_selected,f2_selected,g2_selected,aselect,fselect,gselect,eselect,f2select,g2select
	common lists, alist,flist,flist2,glist,glist2,elist
	common listids, fbflashlist, fbflash2list, fgrouplist, fbgroup2list
	common lisapp_colors, red,green,blue
	
	fidx = event.index
	fclix = event.clicks
	
	widget_control,fselect,/sensitive
	
	if (fclix eq 1) then begin
	
		widget_control,fselect,set_value=1-f_selected(fidx)
		widget_control,fselect,set_uvalue=fidx
	
	endif else begin
	
		f_selected(fidx) = 1-f_selected(fidx)
		widget_control,fselect,set_value=1-f_selected(fidx)
		toggle_flist_entry,fidx
		widget_control,fbflashlist,set_list_select=fidx
		widget_control,fselect,set_uvalue=fidx
		
	endelse
end

function select_f_toggle, event

	common datafiles, orbit, fname
	common lisbase, lisb
	common a_selected,f_selected,g_selected,e_selected,f2_selected,g2_selected,aselect,fselect,gselect,eselect,f2select,g2select
	common lists, alist,flist,flist2,glist,glist2,elist
	common listids, fbflashlist, fbflash2list, fgrouplist, fbgroup2list
	common lisapp_colors, red,green,blue
	
	widget_control, event.id, get_value=buttonvalue
	widget_control, event.id, get_uvalue=fidx
	
	if (f_selected(fidx) ne (1-buttonvalue)) then begin
		f_selected(fidx) = 1-buttonvalue
		toggle_flist_entry,fidx
		widget_control,fbflashlist,set_list_select=fidx
	endif
	return, fidx
end
pro change_entry_string, entry

		retval = string(entry)
		bretval = byte(retval)
		if (bretval(0) eq 42) then begin
			bretval(0) = 32
		endif else begin
			bretval(0) = 42
		endelse
		entry = string(bretval)
end

pro toggle_flist_entry, idx

	common datafiles, orbit, fname
	common lisbase, lisb
	common a_selected,f_selected,g_selected,e_selected,f2_selected,g2_selected,aselect,fselect,gselect,eselect,f2select,g2select
	common lists, alist,flist,flist2,glist,glist2,elist
	common listids, fbflashlist, fbflash2list, fgrouplist, fbgroup2list
	common lisapp_colors, red,green,blue
	common pointdata, theareas, theflashes, thegroups, theevents
	common geobounds, minlat,maxlat,minlon,maxlon
	common plotwins, footprint_id, thresh_id, bar_id, hist_id
	common fwbg, bgpixmap
	common plotpds, pixpd, latlonpd, histpd, barpd
	
	entry = flist(idx)
	change_entry_string, entry
	flist(idx) = entry
	widget_control,fbflashlist,set_value=flist


	fssize = size(flist)
	if (fssize(0) eq 0) then begin
		numflashes = fssize
	endif else begin
		numflashes = fssize(1)
	endelse
	
	chosen = where(f_selected eq 1, chosen_ct)
	if (chosen_ct ge 1) then begin
	
		theevents = replicate({an_event},total(theflashes(chosen).grandchild_count))
		thegroups = replicate({a_group},total(theflashes(chosen).child_count))
		glist2 = strarr(total(theflashes(chosen).child_count))
	
		eidx = 0L & gidx = 0L
		for fct=0L,numflashes-1L,1L do begin
			if (f_selected(fct) eq 1) then begin
				thisflash = theflashes(fct)
				for gct = 0L,thisflash.child_count-1L,1L do begin
					thisgroup = orbit.point.lightning.group(thisflash.child_address+gct)
					thegroups(gidx) = thisgroup
					dtarr = new_datetime_structure(1)
					dtarr.TAI93 = thegroups(gidx).TAI93_time
					calc_datetime,dtarr					
					glist2(gidx) = string(format='("* ",(I5.5)," ",(I2.2),":",(I2.2),":",(F6.3)," ",(F6.2)," ",(F7.2)," ",(I3))', $
					thisflash.child_address+gct, dtarr.utc.hour, dtarr.utc.minute, dtarr.utc.second, 			  $
					thegroups(gidx).location(0), thegroups(gidx).location(1), thegroups(gidx).child_count)
		 					gidx = gidx+1L
					for ect = 0L, thisgroup.child_count-1L,1L do begin
						thisevent = orbit.point.lightning.event(thisgroup.child_address+ect)
						theevents(eidx) = thisevent
						eidx = eidx + 1L
					endfor
				endfor
			endif
		endfor
	
		g2_selected = bytarr(gidx)
		g2_selected(*) =1
		widget_control,fbgroup2list,set_value=glist2
		
		widget_control,histpd,get_uvalue=selection
		case selection of 
			0: begin
					event_histogram, theevents, hist_id
			   end
			1: begin
					event_loghistogram, theevents, hist_id
			   end
			2: begin
					event_loghistogram2, theevents, hist_id
			   end
			3: begin
					event_cdf, theevents, hist_id
			   end
			4: begin
					event_logcdf, theevents, hist_id
			   end
		endcase
				
		pixel_plot, theevents, footprint_id
		
		flash_barplot, theflashes(chosen), thegroups, theevents, bar_id
		
		wset, thresh_id
		tv,bgpixmap	
		widget_control,latlonpd,get_uvalue=selection
		case selection of 
			0: begin
				geo_plot, 1, theevents, thresh_id, 0, minlat,maxlat,minlon,maxlon, bgpixmap
			   end
			1: begin
				geo_plot, 2, theevents, thresh_id, 0, minlat,maxlat,minlon,maxlon, bgpixmap
			   end
			2: begin
				geo_plot, 2, theevents, thresh_id, 1, minlat,maxlat,minlon,maxlon, bgpixmap
			   end
		endcase
	endif
	
end

pro handle_g2list, event

	common datafiles, orbit, fname
	common lisbase, lisb
	common a_selected,f_selected,g_selected,e_selected,f2_selected,g2_selected,aselect,fselect,gselect,eselect,f2select,g2select
	common lists, alist,flist,flist2,glist,glist2,elist
	common listids, fbflashlist, fbflash2list, fgrouplist, fbgroup2list
	common lisapp_colors, red,green,blue
	
	g2idx = event.index
	g2clix = event.clicks
	
	widget_control,g2select,/sensitive
	
	if (g2clix eq 1) then begin
	
		widget_control,g2select,set_value=1-g2_selected(g2idx)
		widget_control,g2select,set_uvalue=g2idx
	
	endif else begin
	
		g2_selected(g2idx) = 1-g2_selected(g2idx)
		widget_control,g2select,set_value=1-g2_selected(g2idx)
		toggle_g2list_entry,g2idx
		widget_control,fbgroup2list,set_list_select=g2idx
		widget_control,g2select,set_uvalue=g2idx
		
	endelse

end

function select_g2_toggle, event

	common datafiles, orbit, fname
	common lisbase, lisb
	common a_selected,f_selected,g_selected,e_selected,f2_selected,g2_selected,aselect,fselect,gselect,eselect,f2select,g2select
	common lists, alist,flist,flist2,glist,glist2,elist
	common listids, fbflashlist, fbflash2list, fgrouplist, fbgroup2list
	common lisapp_colors, red,green,blue
	
	widget_control, event.id, get_value=buttonvalue
	widget_control, event.id, get_uvalue=g2idx
	
	if (g2_selected(g2idx) ne (1-buttonvalue)) then begin
		g2_selected(g2idx) = 1-buttonvalue
		toggle_g2list_entry,g2idx
		widget_control,fbgroup2list,set_list_select=g2idx
	endif
	return, g2idx
end

pro toggle_g2list_entry, idx

	common datafiles, orbit, fname
	common lisbase, lisb
	common a_selected,f_selected,g_selected,e_selected,f2_selected,g2_selected,aselect,fselect,gselect,eselect,f2select,g2select
	common lists, alist,flist,flist2,glist,glist2,elist
	common listids, fbflashlist, fbflash2list, fgrouplist, fbgroup2list
	common lisapp_colors, red,green,blue
	common pointdata, theareas, theflashes, thegroups, theevents
	common geobounds, minlat,maxlat,minlon,maxlon
	common plotwins, footprint_id, thresh_id, bar_id, hist_id
	common fwbg, bgpixmap
	common plotpds, pixpd, latlonpd, histpd, barpd
	
	entry = glist2(idx)
	change_entry_string, entry
	glist2(idx) = entry
	widget_control,fbgroup2list,set_value=glist2
	fchosen = where(f_selected eq 1, fchosen_ct)
	chosen = where(g2_selected eq 1, chosen_ct)
	if (chosen_ct ge 1) then begin
		eidx = 0L
		theevents = replicate({an_event},total(thegroups(chosen).child_count))
		for gct = 0L, chosen_ct-1L, 1L do begin
			thisgroup = thegroups(chosen(gct))
			for ect = 0L, thisgroup.child_count-1L,1L do begin
				thisevent = orbit.point.lightning.event(thisgroup.child_address+ect)
				theevents(eidx) = thisevent
				eidx = eidx + 1L
			endfor
		endfor
	endif

	widget_control,histpd,get_uvalue=selection
	case selection of 
		0: begin
				event_histogram, theevents, hist_id
		   end
		1: begin
				event_loghistogram, theevents, hist_id
		   end
		2: begin
				event_loghistogram2, theevents, hist_id
		   end
		3: begin
				event_cdf, theevents, hist_id
		   end
		4: begin
				event_logcdf, theevents, hist_id
		   end
	endcase
			
	pixel_plot, theevents, footprint_id
	
	flash_barplot, theflashes(fchosen), thegroups(chosen), theevents, bar_id
	
	wset, thresh_id
	tv,bgpixmap	
	widget_control,latlonpd,get_uvalue=selection
	case selection of 
		0: begin
			geo_plot, 1, theevents, thresh_id, 0, minlat,maxlat,minlon,maxlon, bgpixmap
		   end
		1: begin
			geo_plot, 2, theevents, thresh_id, 0, minlat,maxlat,minlon,maxlon, bgpixmap
		   end
		2: begin
			geo_plot, 2, theevents, thresh_id, 1, minlat,maxlat,minlon,maxlon, bgpixmap
		   end
	endcase
end

PRO glint_factor, time, sensor, xpix, ypix, glint_angle

  common datafiles,orbit,fname
  EARTH_RADIUS=6370000.D0

  get_pointing_vector, sensor, xpix, ypix, look_vector

  get_earth_intersection, look_vector, time, location, 0.0D, return_code

  if (return_code ne 0) then begin
    glint_angle = -999.
    return
  endif

  ; determine vector normal to the earth's surface at the point
  ; being observed by the instrument
  Nx=cos(location[1]*!dtor)*cos(location[0]*!dtor)
  Ny=sin(location[1]*!dtor)*cos(location[0]*!dtor)
  Nz=sin(location[0]*!dtor)

  tidx = long(time-orbit.one_second(0).TAI93_time)
  
  ; determine vector for the sun
  Sx=orbit.one_second(tidx).solar_vector[0]
  Sy=orbit.one_second(tidx).solar_vector[1]
  Sz=orbit.one_second(tidx).solar_vector[2]

  ; the sun vector (S), the normal vector (N), and the reflection vector (R)
  ; must lie in the same plane.  Given this fact, the vectors must conform
  ; to the equation S+R=kN , where S, R, & N are unit vectors.  
  ; k can be determined as follows:
  a=(Nx*Nx+Ny*Ny+Nz*Nz)
  b=2.*(Nx*Sx+Ny*Sy+Nz*Sz)
  k=b/a

  ; the reflection vector is given by:
  Rx=k*Nx-Sx
  Ry=k*Ny-Sy
  Rz=k*Nz-Sz

  ; now determine the angle between this reflection angle and
  ; the observation vector (from the location of the instrument to 
  ; the reflection point on the earth)

  ; first, the location of the instrument (note it is not a normalized vector)
  Px= orbit.one_second[tidx].POSITION_VECTOR[0]
  Py= orbit.one_second[tidx].POSITION_VECTOR[1]
  Pz= orbit.one_second[tidx].POSITION_VECTOR[2]
  
  ; next, the instrument-to-earth observation vector (O)
  Ox= Px-Nx*EARTH_RADIUS
  Oy= Py-Ny*EARTH_RADIUS
  Oz= Pz-Nz*EARTH_RADIUS
  Norm=sqrt(Ox*Ox+Oy*Oy+Oz*Oz)
  Ox=Ox/Norm
  Oy=Oy/Norm
  Oz=Oz/Norm
  
  ; the glint angle is determine from the reflection vector and
  ; the instrument-to-earth observation vector
  glint_angle=acos(Rx*Ox+Ry*Oy+Rz*Oz)/!dtor

END

PRO array_glint_factor, time, sensor, glint_angle

  common datafiles,orbit,fname
  EARTH_RADIUS=6370000.D0

  xpix = bytarr(128,128)
  ypix = bytarr(128,128)
  glint_angle = fltarr(128,128)
  
  for x=0,127,1 do begin
  	xpix(x,*) = x
  	ypix(*,x) = x
  endfor
  
  get_array_pointing_vector, sensor, look_vector

  get_array_earth_intersection, look_vector, time, location, 0.0D, return_code

  if (return_code ne 0) then begin
    glint_angle(*,*) = -999.
    return
  endif

  ; determine vector normal to the earth's surface at the point
  ; being observed by the instrument
  Nx=cos(location[*,*,1]*!dtor)*cos(location[*,*,0]*!dtor)
  Ny=sin(location[*,*,1]*!dtor)*cos(location[*,*,0]*!dtor)
  Nz=sin(location[*,*,0]*!dtor)

  tidx = long(time-orbit.one_second(0).TAI93_time)
  
  ; determine vector for the sun
  Sx = fltarr(128,128)
  Sy = fltarr(128,128)
  Sz = fltarr(128,128)
  
  Sx(*,*)=orbit.one_second(tidx).solar_vector[0]
  Sy(*,*)=orbit.one_second(tidx).solar_vector[1]
  Sz(*,*)=orbit.one_second(tidx).solar_vector[2]

  ; the sun vector (S), the normal vector (N), and the reflection vector (R)
  ; must lie in the same plane.  Given this fact, the vectors must conform
  ; to the equation S+R=kN , where S, R, & N are unit vectors.  
  ; k can be determined as follows:
  a=(Nx*Nx+Ny*Ny+Nz*Nz)
  b=2.*(Nx*Sx+Ny*Sy+Nz*Sz)
  k=b/a

  ; the reflection vector is given by:
  Rx=k*Nx-Sx
  Ry=k*Ny-Sy
  Rz=k*Nz-Sz

  Sx = 0 & Sy = 0 & Sz = 0
  
  ; now determine the angle between this reflection angle and
  ; the observation vector (from the location of the instrument to 
  ; the reflection point on the earth)

  ; first, the location of the instrument (note it is not a normalized vector)
  Px = fltarr(128,128)
  Py = fltarr(128,128)
  Pz = fltarr(128,128)
  
  Px(*,*)= orbit.one_second[tidx].POSITION_VECTOR[0]
  Py(*,*)= orbit.one_second[tidx].POSITION_VECTOR[1]
  Pz(*,*)= orbit.one_second[tidx].POSITION_VECTOR[2]
  
  ; next, the instrument-to-earth observation vector (O)
  Ox= Px-Nx*EARTH_RADIUS
  Oy= Py-Ny*EARTH_RADIUS
  Oz= Pz-Nz*EARTH_RADIUS
  
  Px = 0 & Py = 0 & Pz = 0
  Nx = 0 & Ny = 0 & Nz = 0
  
  Norm=sqrt(Ox*Ox+Oy*Oy+Oz*Oz)
  Ox=Ox/Norm
  Oy=Oy/Norm
  Oz=Oz/Norm
  
  ; the glint angle is determine from the reflection vector and
  ; the instrument-to-earth observation vector
  glint_angle=acos(Rx*Ox+Ry*Oy+Rz*Oz)/!dtor
  
  Ox = 0 & Oy = 0 & Oz = 0 & Rx = 0 & Ry = 0 & Rz = 0 & a=0 & b=0 & k=0 & Norm = 0
  

END
