@lisapp_GlobalRoutines.pro

pro plot_array

	common datafiles, orbit, fname
	common lisbase, lisb
	common active_windows, plot_array_flag, plot_geographic_flag
	common lisapp_colors, red,green,blue
	
	plot_array_flag = 1
	
	arrbase = widget_base(column=1,xsize=300,ysize=300,/align_center,title='Events in array',group_leader=lisb,kill_notify='plot_array_off')
	arrwin  = widget_draw(arrbase,colors=220,/align_center,xsize=300,ysize=300,retain=2)	
	widget_control,arrbase,/realize
	Widget_Control, arrwin, get_value=arr_id

	wset,arr_id & 	if (not iswindows(0)) then device,pseu=8
	tvlct,red,green,blue
	plot,findgen(2),findgen(2),/nodata,xrange=[0,127],yrange=[0,127],xstyle=5,ystyle=5,xmargin=[0,0],ymargin=[0,0]
	
	pix_colors = ((64.*((orbit.point.lightning.event(*).TAI93_time-orbit.orbit_summary.TAI93_start) / $
			  (orbit.orbit_summary.TAI93_end-orbit.orbit_summary.TAI93_start)))+64.)
	plots, orbit.point.lightning.event(*).x_pixel, orbit.point.lightning.event(*).y_pixel, psym=3, $
		  color=pix_colors
		
end

pro plot_array_off, event
	common active_windows, plot_array_flag, plot_geographic_flag
	
	plot_array_flag = 0
end

pro quicklook_browse_image, event
	common datafiles, orbit, fname
	common lisbase,lisb
	common lisapp_colors,red,green,blue
	which_sensor, fname, file_format
	if (file_format eq 'lis') then begin
		getvdcounts, fname, vdsizes
		READ_orbit_summary,fname,orbit_summary,status=status,_EXTRA=e
		TAI93_start = orbit_summary.TAI93_start
		TAI93_end   = orbit_summary.TAI93_end
		dt = new_datetime_structure(2)
		dt(0).tai93 = tai93_start
		dt(1).tai93 = tai93_end
		calc_datetime,dt
		msgstr = strarr(10)
		msgstr(0) = fname
		msgstr(1) = string(format='("   Start:  ",(I2.2),"/",(I2.2),"/",(I4)," (",(I3.3),")   ",(I2.2),":",(I2.2),":",(F6.3)," UTC")', $
				dt(0).utc.month, dt(0).utc.day, dt(0).utc.year, dt(0).utc.doy, dt(0).utc.hour, dt(0).utc.minute, dt(0).utc.second)
		msgstr(2) = string(format='("   End  :  ",(I2.2),"/",(I2.2),"/",(I4)," (",(I3.3),")   ",(I2.2),":",(I2.2),":",(F6.3)," UTC")', $
				dt(1).utc.month, dt(1).utc.day, dt(1).utc.year, dt(1).utc.doy, dt(1).utc.hour, dt(1).utc.minute, dt(1).utc.second)
		msgstr(3)  = string(format='((I6),"    Areas")',vdsizes(2))
		msgstr(4)  = string(format='((I6),"    Flashes")',vdsizes(3))
		msgstr(5)  = string(format='((I6),"    Groups")',vdsizes(4))
		msgstr(6)  = string(format='((I6),"    Events")',vdsizes(5))
		msgstr(7)  = string(format='((I6),"    Viewtimes")',vdsizes(6))
		msgstr(8)  = string(format='((I6),"    Backgrounds")',vdsizes(7))
		msgstr(9)  = string(format='((I6),"    One-Seconds")',vdsizes(8))
		numlines = 10
		has_raster='false'
		h = hdf_open(fname,/read)
		if (h ne -1) then begin
			hdf_close,h
			h = hdf_open(fname,/read)
			r8n = hdf_dfr8_nimages(fname)
			hdf_close,h
			hdf_dfr8_restart
			if (r8n gt 0) then begin
				h = hdf_open(fname,/read)
				hdf_dfr8_getinfo,fname,r8w,r8h,r8hp
				hdf_dfr8_restart
				hdf_close,h
				has_raster='true'
				; print,r8w,r8h,r8hp
				h = hdf_open(fname,/read)
				browseimage=bytarr(r8w,r8h)
				hdf_dfr8_getimage,fname,browseimage,palette
				hdf_dfr8_restart
				hdf_close,h
			endif
		endif
	endif else begin
		if (file_format eq 'otd') then begin
			getoldvdcounts, fname, vdsizes
			READ_OLD_orbit_attributes,fname,orbit_attributes,status=status,_EXTRA=e
			TAI93_start = orbit_attributes.TAI93_start
			TAI93_end   = orbit_attributes.TAI93_end
			dt = new_datetime_structure(2)
			dt(0).tai93 = tai93_start
			dt(1).tai93 = tai93_end
			calc_datetime,dt							
			msgstr = strarr(10)
			msgstr(0) = fname
			msgstr(1) = string(format='("   Start:  ",(I2.2),"/",(I2.2),"/",(I4)," (",(I3.3),")   ",(I2.2),":",(I2.2),":",(F6.3)," UTC")', $
					dt(0).utc.month, dt(0).utc.day, dt(0).utc.year, dt(0).utc.doy, dt(0).utc.hour, dt(0).utc.minute, dt(0).utc.second)
			msgstr(2) = string(format='("   End  :  ",(I2.2),"/",(I2.2),"/",(I4)," (",(I3.3),")   ",(I2.2),":",(I2.2),":",(F6.3)," UTC")', $
					dt(1).utc.month, dt(1).utc.day, dt(1).utc.year, dt(1).utc.doy, dt(1).utc.hour, dt(1).utc.minute, dt(1).utc.second)
			msgstr(3)  = string(format='((I6),"    Areas")',vdsizes(0))
			msgstr(4)  = string(format='((I6),"    Flashes")',vdsizes(1))
			msgstr(5)  = string(format='((I6),"    Groups")',vdsizes(2))
			msgstr(6)  = string(format='((I6),"    Events")',vdsizes(3))
			msgstr(7)  = string(format='((I6),"    Viewtimes")',vdsizes(8))
			msgstr(8)  = string(format='((I6),"    Backgrounds")',vdsizes(6))
			msgstr(9)  = string(format='((I6),"    One-Seconds")',6100)
			numlines = 10
			has_raster='false'						
		endif else begin
			retval = dialog_message('This does not appear to be a LIS/OTD HDF file!',/error)
			numlines = 0
		endelse
	endelse
	if (numlines gt 0) then begin
		infob = widget_base(group_leader=lisb,/modal,tlb_frame_attr=1,scroll=0,title='HDF File Information',/base_align_center,row=5)
		infol = lonarr(numlines)
		for i=0,2,1 do begin
			infol(i) = widget_label(infob,value=msgstr(i),/align_center)
		endfor
		infob2 = widget_base(infob,column=2)
		infob3 = widget_base(infob2,row=numlines-3,/base_align_center,/align_center)
		if (has_raster eq 'true') then begin
			infob4 = widget_draw(infob2,colors=220,/align_center,xsize=r8w,ysize=r8h,retain=2)
		endif
		for i=3,numlines-1,1 do begin
			infol(i) = widget_label(infob3,value=msgstr(i),/align_center)
		endfor
		
		infodone = widget_button(infob,value='Done',/align_center)
		widget_control,infob,/realize
		if (has_raster eq 'true') then begin
			widget_control,infob4,get_value=browse_win_id
			wset,browse_win_id
			tvlct,palette(0,*),palette(1,*),palette(2,*)
			tv,rotate(browseimage,7)
		endif
		widget_control,infob,cancel_button=infodone
		xmanager, 'hdf_file_info', infob, event='selfkill'
	endif

	
end

pro quicklook_viewtimes, event
	common datafiles, orbit, fname
	common lisbase, lisb
	common vtgrids, vtgrid, vtgridmask
	common lisapp_colors,red,green,blue	

	waitb = widget_base(title='Working',tlb_frame_attr=0,scroll=0,/modal,column=1,group_leader=lisb)
	waitm = widget_label(waitb,value='Preparing viewtime plot ... this may take a moment...',/align_center)
	widget_control,waitb,/realize

	if (vtgrid(0,0) eq -1) then begin
		vtgrid = fltarr(360,720)
		vtgridmask = fltarr(360,720)
		vtgridmask(*,*)=64
		for i = long(0),orbit.point.point_summary.vt_count-1,long(1) do begin
			thisy = fix(orbit.point.viewtime(i).location(0)*2+180)
			thisx = fix(orbit.point.viewtime(i).location(1)*2+360)
			vtgrid(thisy,thisx) = vtgrid(thisy,thisx) + orbit.point.viewtime(i).effective_obs
			vtgridmask(thisy,thisx) = 48
		endfor
		vtgrid2 = fix((58.*vtgrid/float(max(vtgrid)))+64.)
		vtgrid = vtgrid2
	endif

	outvt = transpose(vtgrid)
	
	window,colors=220,0,xsize=360*2,ysize=360*1,retain=2,/pixmap
	if (not iswindows(0)) then device,pseu=8
	tvlct,red,green,blue
	tv,outvt
	map_set,/continents,E_Continents={countries:1},/usa,xmargin=[0,0],ymargin=[0,0],/noerase,color=0,con_color=0,/noborder
	imgv = tvrd()
	tstr = fname+' (Viewtimes)'
;	slide_image,imgv,group=lisb,/register,title=tstr,yvisible=380,xvisible=380
	qlv = widget_base(group=lisb,xsize=720,ysize=360,title='Viewtime Quicklook')
	qlvd = widget_draw(qlv,xsize=720,ysize=360,retain=2)
	widget_control,qlv,/realize
	widget_control,qlvd,get_value=qlvdid
	wset,qlvdid
	tvlct,red,green,blue
	tv,imgv
	
	widget_control,waitb,/destroy


end

pro quicklook_bg_footprints

	common datafiles, orbit, fname
	common sensor_read, this_sensor, file_format
	common lisbase, lisb
	common bggrids, bggrid, bggridmask
	common lisapp_colors,red,green,blue	
	
	waitb = widget_base(title='Working',tlb_frame_attr=0,scroll=0,/modal,column=1,group_leader=lisb)
	waitm = widget_label(waitb,value='Geolocating background footprint   1 of xxx',/align_center)
	widget_control,waitb,/realize

	if(bggrid(0,0) eq -1) then begin
		bggrid = bytarr(360,720)
		
		get_array_pointing_vector,this_sensor,array_look_vector
		
		for i = 0, orbit.point.point_summary.bg_count-1,1 do begin
			msgstr = string(format='("Geolocating background footprint ",(I3)," of ",(I3))',i+1, $
					orbit.point.point_summary.bg_count)
			widget_control,waitm,set_value=msgstr
			get_array_earth_intersection,array_look_vector,orbit.point.bg_summary(i).TAI93_time,array_location,0.0D ,return_code
			if (return_code eq 0) then begin
				xlocns = reform(fix(array_location(*,*,1)*2+360),1,16384)
				ylocns = reform(fix(array_location(*,*,0)*2+180),1,16384)
				bggrid(ylocns,xlocns) = $
					bggrid(ylocns,xlocns) + 1
				bggridmask(ylocns,xlocns) = 1
			endif 
		endfor
	endif
	widget_control,waitm,set_value='Drawing image'

	outgrid = transpose(bggrid)
	outgrid = fix((58.*outgrid/float(max(outgrid)))+64.)

	window,colors=220,0,xsize=360*2,ysize=360*1,retain=2,/pixmap
	if (not iswindows(0)) then device,pseu=8
	tvlct,red,green,blue	
	tv,outgrid
	map_set,/continents,E_Continents={countries:1},/usa,xmargin=[0,0],ymargin=[0,0],/noerase,/noborder,color=0,con_color=0
	imgb = tvrd()
	tstr = fname+' (Background Image Footprints)'
;	slide_image,imgv,group=lisb,/register,title=tstr,xvisible=380,yvisible=380
	qlb = widget_base(group=lisb,xsize=720,ysize=360,title='Background Footprint Quicklook')
	qlbd = widget_draw(qlb,xsize=720,ysize=360,retain=2)
	widget_control,qlb,/realize
	widget_control,qlbd,get_value=qlbdid
	wset,qlbdid
	tvlct,red,green,blue
	tv,imgb
	
	widget_control,waitb,/destroy
		
end

pro quicklook_vt_no_bg

	common vtgrids, vtgrid, vtgridmask
	common bggrids, bggrid, bggridmask
	common datafiles, orbit, fname
	common sensor_read, this_sensor, file_format
	common lisbase, lisb

	if (vtgridmask(0,0) eq -1) then begin
		; print,'Computing VT grid'
		vtgrid = fltarr(360,720)
		vtgridmask = bytarr(360,720)
		vtgridmask(*,*)=64
		for i = long(0),long(orbit.point.point_summary.vt_count-1) do begin
			thisy = fix(orbit.point.viewtime(i).location(0)*2+180)
			thisx = fix(orbit.point.viewtime(i).location(1)*2+360)
			vtgrid(thisy,thisx) = vtgrid(thisy,thisx) + orbit.point.viewtime(i).effective_obs
			vtgridmask(thisy,thisx) = 48
		endfor
	endif

	if(bggridmask(0,0) eq -1) then begin
		; print, 'Computing BG grid'
		bggrid = bytarr(360,720)
		bggridmask = bytarr(360,720)
		
		get_array_pointing_vector,this_sensor,array_look_vector
		
		for i = 0, orbit.point.point_summary.bg_count-1,1 do begin
			get_array_earth_intersection,array_look_vector,orbit.point.bg_summary(i).TAI93_time,array_location, 0.0D, return_code
			if (return_code eq 0) then begin
				xlocns = reform(fix(array_location(*,*,1)*2+360),1,16384)
				ylocns = reform(fix(array_location(*,*,0)*2+180),1,16384)
				; print,i,max(xlocns),max(ylocns)
				bggrid(ylocns,xlocns) = $
					bggrid(ylocns,xlocns) + 1
				bggridmask(ylocns,xlocns) = 1
			endif 
		endfor
	endif

	diffgrid = bytarr(360,720)
	locns = where((vtgridmask gt 0) and (bggridmask eq 0),count)
	if (count ne 0) then begin
		; print,count
		diffgrid(locns)=128
		locns = where((vtgridmask gt 0) and (bggridmask gt 0),count)
		if (count ne 0) then begin
			; print,count
			diffgrid(locns) = 32
		endif
	endif else begin
		locns = where((vtgridmask gt 0) and (bggridmask gt 0),count)
		; print,'All viewtimes have backgrounds'
		if (count ne 0) then begin
			diffgrid(locns) = 32
		endif
	endelse
	loadct,39
	window,colors=220,0,xsize=360*4,ysize=360*2,retain=2,/pixmap
	if (not iswindows(0)) then device,pseu=8
	tvscl,rebin(transpose(diffgrid),360*4,360*2)
	map_set,/continents,E_Continents={countries:1},/usa,xmargin=[0,0],ymargin=[0,0],/noerase,/noborder
	imgv = tvrd()
	tstr = fname+' (Viewtimes without Backgrounds)'
	slide_image,imgv,group=lisb,/register,title=tstr

end

pro quicklook_alert_flags
	common datafiles, orbit, fname
	common sensor_read, this_sensor, file_format
	common lisbase, lisb
	common lisapp_colors, red, green, blue

	waitb = widget_base(title='Working',tlb_frame_attr=0,scroll=0,/modal,column=1,group_leader=lisb)
	waitm = widget_label(waitb,value='Preparing alert summary flag plot...',/align_center)
	widget_control,waitb,/realize
     usersym,sin(findgen(21)/10.*!pi),cos(findgen(21)/10.*!pi),/fill
	uscale = usersym_scale(0)
	
;	nadirs = fltarr(2,orbit.orbit_summary.one_second_count)
;	get_nadir_locations, orbit.one_second(*).TAI93_time, nadirs

	fatal1 = (orbit.one_second(*).alert_summary and 1)
	warn1 = (orbit.one_second(*).alert_summary and 2)
	
	fatal2 = (orbit.one_second(*).alert_summary and 4)
	warn2 = (orbit.one_second(*).alert_summary and 8)
	
	fatal3 = (orbit.one_second(*).alert_summary and 16)
	warn3 = (orbit.one_second(*).alert_summary and 32)
	
	fatal4 = (orbit.one_second(*).alert_summary and 64)
	warn4 = (orbit.one_second(*).alert_summary and 128)
	
	times = orbit.one_second(*).TAI93_time
	numos = orbit.orbit_summary.one_second_count
	xpix = intarr(numos)
	xpix(*) = 64
	ypix1 = intarr(numos)
	ypix1(*) = 16
	ypix2 = intarr(numos)
	ypix2(*) = 48
	ypix3 = intarr(numos)
	ypix3(*) = 80
	ypix4 = intarr(numos)
	ypix4(*) = 112
	
	get_vector_pointing_vector,this_sensor,xpix,ypix1,lv1
	get_vector_earth_intersection,lv1,times,vl1,0,rc
	
	get_vector_pointing_vector,this_sensor,xpix,ypix2,lv2
	get_vector_earth_intersection,lv2,times,vl2,0,rc
	
	get_vector_pointing_vector,this_sensor,xpix,ypix3,lv3
	get_vector_earth_intersection,lv3,times,vl3,0,rc
	
	get_vector_pointing_vector,this_sensor,xpix,ypix4,lv4
	get_vector_earth_intersection,lv4,times,vl4,0,rc
	
	window,colors=220,0,xsize=360*2,ysize=360*1,retain=2,/pixmap
	if (not iswindows(0)) then device,pseu=8
	tvlct,red,green,blue
	map_set,/continents,E_Continents={countries:1},/usa,xmargin=[0,0],ymargin=[0,0],/noerase,/noborder,color=64,con_color=64
	
	plots,vl1(*,1),vl1(*,0),psym=3,symsize=uscale,color=129
	plots,vl2(*,1),vl2(*,0),psym=3,symsize=uscale,color=129
	plots,vl3(*,1),vl3(*,0),psym=3,symsize=uscale,color=129
	plots,vl4(*,1),vl4(*,0),psym=3,symsize=uscale,color=129
	w1 = where(warn1 gt 0,w1ct)
	f1 = where(fatal1 gt 0,f1ct)
	ssize = .05+fix(360*2/200)*.10*uscale
	if (w1ct gt 0) then begin
		plots,vl1(w1,1),vl1(w1,0),psym=8,symsize=ssize,color=130
	endif
	if (f1ct gt 0) then begin
		plots,vl1(f1,1),vl1(f1,0),psym=8,symsize=ssize,color=131
	endif
	
	w2 = where(warn2 gt 0,w2ct)
	f2 = where(fatal2 gt 0,f2ct)
	if (w2ct gt 0) then begin
		plots,vl2(w2,1),vl2(w2,0),psym=8,symsize=ssize,color=130
	endif
	if (f2ct gt 0) then begin
		plots,vl2(f2,1),vl2(f2,0),psym=8,symsize=ssize,color=131
	endif

	w3 = where(warn3 gt 0,w3ct)
	f3 = where(fatal3 gt 0,f3ct)
	if (w3ct gt 0) then begin
		plots,vl3(w3,1),vl3(w3,0),psym=8,symsize=ssize,color=130
	endif
	if (f3ct gt 0) then begin
		plots,vl3(f3,1),vl3(f3,0),psym=8,symsize=ssize,color=131
	endif

	w4 = where(warn4 gt 0,w4ct)
	f4 = where(fatal4 gt 0,f4ct)
	ssize = .05+fix(360*2/200)*.10*uscale
	if (w4ct gt 0) then begin
		plots,vl4(w4,1),vl4(w4,0),psym=8,symsize=ssize,color=130
	endif
	if (f4ct gt 0) then begin
		plots,vl4(f4,1),vl4(f4,0),psym=8,symsize=ssize,color=131
	endif

;	afcolors = bytarr(orbit.orbit_summary.one_second_count)
;	
;	flagged = where(orbit.one_second(*).alert_summary ne 0,count)
;	notflagged = where(orbit.one_second(*).alert_summary eq 0,clear_count)
;	if (count ne 0) then begin
;		if (count ne orbit.orbit_summary.one_second_count) then begin
;			afc2 = bytarr(clear_count)
;			afc2(*) = 32
;			plots,nadirs(1,notflagged),nadirs(0,notflagged),psym=3,symsize=.5,color=afc2
;			; print,clear_count,' not flagged'
;		endif 
;		
;		warn = where( ((orbit.one_second(*).alert_summary and 1) ne 0) or $
;				   ((orbit.one_second(*).alert_summary and 4) ne 0) or $
;				   ((orbit.one_second(*).alert_summary and 16) ne 0) or $
;				   ((orbit.one_second(*).alert_summary and 64) ne 0), warn_count)
;		fatal = where(  ((orbit.one_second(*).alert_summary and 2) ne 0) or $
;					((orbit.one_second(*).alert_summary and 8) ne 0) or $
;					((orbit.one_second(*).alert_summary and 32) ne 0) or $
;					((orbit.one_second(*).alert_summary and 128) ne 0), fatal_count)
;		
;		
;		if (warn_count ne 0) then begin
;			; print,warn_count,' flagged WARN'
;			afc3 = bytarr(warn_count)
;			; afc3(*) = fix(((orbit.one_second(flagged).alert_summary/256.)*64.)+64.)
;			afc3(*) = 88
;			plots,nadirs(1,warn),nadirs(0,warn),psym=1,symsize=1,color=afc3		
;		endif
;		if (fatal_count ne 0) then begin
;			; print,fatal_count,' flagged FATAL'
;			afc4 = bytarr(fatal_count)
;			afc4(*) = 120
;			plots,nadirs(1,fatal),nadirs(0,fatal),psym=1,symsize=1,color=afc4
;		endif
;	endif else begin
;		plots,nadirs(1,*),nadirs(0,*),psym=3,symsize=.5,color=32
;	endelse	
;
	imgq = tvrd()
	tstr = fname+' (Alert Summaries)'
;	slide_image,imgv,group=lisb,/register,title=tstr,xvisible=380,yvisible=380
	qlq = widget_base(group=lisb,xsize=720,ysize=360,title='Alert Flag Quicklook')
	qlqd = widget_draw(qlq,xsize=720,ysize=360,retain=2)
	widget_control,qlq,/realize
	widget_control,qlqd,get_value=qlqdid
	wset,qlqdid
	tvlct,red,green,blue
	tv,imgq

	widget_control,waitb,/destroy
end	
	
		
pro quicklook_areas, event
	common datafiles, orbit, fname
	common lisbase, lisb
	common vtgrids, vtgrid, vtgridmask
	common lisapp_colors,red,green,blue
	
     usersym,sin(findgen(21)/10.*!pi),cos(findgen(21)/10.*!pi),/fill
	uscale = usersym_scale(0)
	waitb = widget_base(title='Working',tlb_frame_attr=0,scroll=0,/modal,column=1,group_leader=lisb)
	waitm = widget_label(waitb,value='Preparing area plot...     ',/align_center)
	widget_control,waitb,/realize

	if (vtgridmask(0,0) eq -1) then begin
		widget_control,waitm,set_value='Computing viewtime mask...'
		vtgridmask = fltarr(360,720)
		vtgridmask(*,*) = 64
		vtgridmask(fix(orbit.point.viewtime(*).location(0)*2+180),fix(orbit.point.viewtime(*).location(1)*2+360)) = 48
	endif
	
	widget_control,waitm,set_value='Preparing area plot...     '
	
	window,colors=220,0,xsize=720,ysize=360,retain=2,/pixmap
	if (not iswindows(0)) then device,pseu=8
	tvlct,red,green,blue
	
	;tv,rebin(transpose(vtgridmask),360*2,360*1)
	tv,transpose(vtgridmask)
	
	map_set,/continents,E_Continents={countries:1},/usa,xmargin=[0,0],ymargin=[0,0],/noerase,/noborder,color=16,con_color=16
	
	maxchildren = float(max(orbit.point.lightning.area(*).child_count))
	
	colors=fix(65.+(64.*orbit.point.lightning.area(*).child_count/min(25.,maxchildren)))
	
	ssize = .05+fix(360*4/200)*.10*uscale
	plots,orbit.point.lightning.area(*).location(1),orbit.point.lightning.area(*).location(0),psym=8, $
			color=colors,symsize=ssize
	imga = tvrd()
	tstr = fname+' (Areas)'
	;slide_image,imga,group=lisb,/register,title=tstr,xvisible=380,yvisible=380
	qla = widget_base(group=lisb,xsize=720,ysize=360,title='Area Quicklook')
	qlad = widget_draw(qla,xsize=720,ysize=360,retain=2)
	widget_control,qla,/realize
	widget_control,qlad,get_value=qladid
	wset,qladid
	tvlct,red,green,blue
	tv,imga
	
	widget_control,waitb,/destroy
end

pro quicklook_flashes, event
	common datafiles, orbit, fname
	common lisbase, lisb
	common vtgrids, vtgrid, vtgridmask
	common lisapp_colors,red,green,blue
	
     usersym,sin(findgen(21)/10.*!pi),cos(findgen(21)/10.*!pi),/fill
	uscale = usersym_scale(0)
	waitb = widget_base(title='Working',tlb_frame_attr=0,scroll=0,/modal,column=1,group_leader=lisb)
	waitm = widget_label(waitb,value='Preparing flash plot...     ',/align_center)
	widget_control,waitb,/realize

	if (vtgridmask(0,0) eq -1) then begin
		widget_control,waitm,set_value='Computing viewtime mask...'
		vtgridmask = fltarr(360,720)
		vtgridmask(*,*) = 64
		vtgridmask(fix(orbit.point.viewtime(*).location(0)*2+180),fix(orbit.point.viewtime(*).location(1)*2+360)) = 48
	endif
	
	widget_control,waitm,set_value='Preparing flash plot...     '
	
	window,colors=220,0,xsize=720,ysize=360,retain=2,/pixmap
	if (not iswindows(0)) then device,pseu=8
	tvlct,red,green,blue
	
	tv,transpose(vtgridmask)
	
	map_set,/continents,E_Continents={countries:1},/usa,xmargin=[0,0],ymargin=[0,0],/noerase,/noborder,color=16,con_color=16
	
	maxchildren = float(max(orbit.point.lightning.flash(*).child_count))
	
	colors=fix(65.+(64.*orbit.point.lightning.flash(*).child_count/min(25.,maxchildren)))

	ssize = .05+fix(360*4/200)*.10*uscale
	plots,orbit.point.lightning.flash(*).location(1),orbit.point.lightning.flash(*).location(0),psym=8, $
			color=colors,symsize=ssize
	imgf = tvrd()
	tstr = fname+' (Flashes)'
;	slide_image,imgf,group=lisb,/register,title=tstr,xvisible=380,yvisible=380
	qlf = widget_base(group=lisb,xsize=720,ysize=360,title='Flash Quicklook')
	qlfd = widget_draw(qlf,xsize=720,ysize=360,retain=2)
	widget_control,qlf,/realize
	widget_control,qlfd,get_value=qlfdid
	wset,qlfdid
	tvlct,red,green,blue
	tv,imgf

	widget_control,waitb,/destroy
end

pro quicklook_groups, event
	common datafiles, orbit, fname
	common lisbase, lisb
	common vtgrids, vtgrid, vtgridmask
	common lisapp_colors,red,green,blue
	
     usersym,sin(findgen(21)/10.*!pi),cos(findgen(21)/10.*!pi),/fill
	uscale = usersym_scale(0)
	waitb = widget_base(title='Working',tlb_frame_attr=0,scroll=0,/modal,column=1,group_leader=lisb)
	waitm = widget_label(waitb,value='Preparing group plot...     ',/align_center)
	widget_control,waitb,/realize

	if (vtgridmask(0,0) eq -1) then begin
		widget_control,waitm,set_value='Computing viewtime mask...'
		vtgridmask = fltarr(360,720)
		vtgridmask(*,*) = 64
		vtgridmask(fix(orbit.point.viewtime(*).location(0)*2+180),fix(orbit.point.viewtime(*).location(1)*2+360)) = 48
	endif
	
	widget_control,waitm,set_value='Preparing group plot...     '
	
	window,colors=220,0,xsize=360*2,ysize=360*1,retain=2,/pixmap
	if (not iswindows(0)) then device,pseu=8
	tvlct,red,green,blue
	
	tv,transpose(vtgridmask)
	
	map_set,/continents,E_Continents={countries:1},/usa,xmargin=[0,0],ymargin=[0,0],/noerase,/noborder,color=16,con_color=16
	
	maxchildren = float(max(orbit.point.lightning.group(*).child_count))
	
	colors=fix(65.+(64.*orbit.point.lightning.group(*).child_count/min(25.,maxchildren)))

	ssize = .05+fix(360*4/200)*.10*uscale
	plots,orbit.point.lightning.group(*).location(1),orbit.point.lightning.group(*).location(0),psym=8, $
			color=colors,symsize=ssize
	imgg = tvrd()
	tstr = fname+' (Groups)'
;	slide_image,imgf,group=lisb,/register,title=tstr,xvisible=380,yvisible=380
	qlg = widget_base(group=lisb,xsize=720,ysize=360,title='Group Quicklook')
	qlgd = widget_draw(qlg,xsize=720,ysize=360,retain=2)
	widget_control,qlg,/realize
	widget_control,qlgd,get_value=qlgdid
	wset,qlgdid
	tvlct,red,green,blue
	tv,imgg

	widget_control,waitb,/destroy
	
end

pro quicklook_events, event
	common datafiles, orbit, fname
	common lisbase, lisb
	common vtgrids, vtgrid, vtgridmask
	common lisapp_colors,red,green,blue
	
     usersym,sin(findgen(21)/10.*!pi),cos(findgen(21)/10.*!pi),/fill
	uscale = usersym_scale(0)
	waitb = widget_base(title='Working',tlb_frame_attr=0,scroll=0,/modal,column=1,group_leader=lisb)
	waitm = widget_label(waitb,value='Preparing event plot...     ',/align_center)
	widget_control,waitb,/realize

	if (vtgridmask(0,0) eq -1) then begin
		widget_control,waitm,set_value='Computing viewtime mask...'
		vtgridmask = fltarr(360,720)
		vtgridmask(*,*) = 64
		vtgridmask(fix(orbit.point.viewtime(*).location(0)*2+180),fix(orbit.point.viewtime(*).location(1)*2+360)) = 48
	endif
	
	widget_control,waitm,set_value='Preparing event plot...     '
	
	window,colors=220,0,xsize=360*2,ysize=360*1,retain=2,/pixmap
	if (not iswindows(0)) then device,pseu=8
	tvlct,red,green,blue
	
	tv,transpose(vtgridmask)
	
	map_set,/continents,E_Continents={countries:1},/usa,xmargin=[0,0],ymargin=[0,0],/noerase,/noborder,color=16,con_color=16
		
	colors=fix(65.+(64.*orbit.point.lightning.event(*).amplitude/128.))
	ssize = .05+fix(360*4/200)*.10*uscale
	plots,orbit.point.lightning.event(*).location(1),orbit.point.lightning.event(*).location(0),psym=8, $
			color=colors,symsize=ssize
	imge = tvrd()
	tstr = fname+' (Events)'
;	slide_image,img,group=lisb,/register,title=tstr,xvisible=380,yvisible=380
	qle = widget_base(group=lisb,xsize=720,ysize=360,title='Event Quicklook')
	qled = widget_draw(qle,xsize=720,ysize=360,retain=2)
	widget_control,qle,/realize
	widget_control,qled,get_value=qledid
	wset,qledid
	tvlct,red,green,blue
	tv,imge

	widget_control,waitb,/destroy
end

pro animate_bgs, event
	common sensor_read, this_sensor, file_format
	common datafiles, orbit, fname
	common otd_boom, boom
	common lisbase, lisb
	common lisapp_colors,red,green,blue
	common bg_info, fnamestub, candobgs, bgname, bgpathtouse

	filename = bgname
		
	fileid=HDF_OPEN(filename,/read)

	sd_id=HDF_SD_START(filename,/read)
	HDF_SD_FILEINFO,sd_id,nmfsds,nglobatts
		
	for i=0,nmfsds-1 do begin
		sds_id=HDF_SD_SELECT(sd_id,i)
	     	HDF_SD_GETINFO,sds_id,name=n,ndims=r,type=t,natts=nats,$
	                         hdf_type=h,unit=u
	          	          	
	     	if r gt 0 then HDF_SD_GETINFO,sds_id,dims=dims else dims=0
	
		for j=0,nats-1 do begin
		 	HDF_SD_ATTRINFO,sds_id,j,name=n,type=t,count=c,data=d,$
	                            hdf_type=h
	     	endfor
	       		  
	 	temp = intarr(128,128)		   
          	inbg = intarr(128,128,orbit.point.point_summary.bg_count)
		first = 0
		last = orbit.point.point_summary.bg_count-1

		for kk = first,last,1 do begin
			hdf_sd_getdata,sds_id,temp,start=[0,0,kk],count=[128,128,1]
			inbg(*,*,kk-first) = temp
		endfor
		
           	;hdf_sd_getdata,sds_id,inbg,start=[0,0,0],count=[128,128,orbit.point.point_summary.bg_count]

		HDF_SD_ENDACCESS,sds_id
	endfor
	for i=0,nglobatts-1 do begin
		HDF_SD_ATTRINFO,sd_id,i,name=n,type=t,count=c,data=d
	endfor
			   
	HDF_SD_END,sd_id
	
	for i=0,orbit.point.point_summary.bg_count-1,1 do begin
		; print,i,min(inbg(*,*,i)),max(inbg(*,*,i))
		inbg(*,*,i) = byte((float(temporary(inbg(*,*,i)))/3000.)*64.)
	endfor
	bgab = widget_base(group_leader=lisb,title='Background Image Animation')
	animate = cw_animate(bgab,128,128,orbit.point.point_summary.bg_count,/track)
	widget_control,/realize,bgab
	for i=0,orbit.point.point_summary.bg_count-1,1 do begin
		cw_animate_load, animate, frame=i, image=inbg(*,*,i)
	endfor
	; cw_animate_getp, animate, pixmap_vect
	tvlct,red,green,blue
	cw_animate_run, animate
	xmanager,'Background Image Animation',bgab,event_handler='bgahandler'
end

pro bgahandler, event
	widget_control,/destroy,event.top
end
