@lisapp_DateTime.pro

; ***********************************************************************************************
; *					Basic Interface			 			*
; ***********************************************************************************************


PRO LISOTD

; *** Calling interface is simple: just:
; ***	READ_orbit, filename, somevariable
; ***   WRITE_orbit, filename, somevariable
	
;	READ_orbit, "new_otd.hdf", orbit 
	READ_orbit, "/usr9/data/12_14/TRMM_LIS_SC.03.2_1997.348.00855", orbit 
	
;	WRITE_orbit, "trmm.test.idl", orbit
	
;	summarize_hdf, "trmm.test.idl"	
	
END

; ***********************************************************************************************
; *			Examples of use of IDL LIS-OTD HDF Interface Libraries 			*
; ***********************************************************************************************


pro  IDL_EXAMPLES, fname

; *** Calling interface is simple: just:
; ***	READ_orbit, filename, somevariable

	READ_orbit, fname, orbit

; *** In this case, the variable "orbit" is a structure of structures.
; ***	Its format mirrors the HDF file structure
; ***	See: http://atmos.uah.edu/~driscoll/HDF/hdf_format.html
; ***        and the HDF file summary provided by Johnny

; ***
; *** EXAMPLE: Print the locations of all flashes
; ***

;	for f = 0, orbit.point.point_summary.flash_count-1, 1 do begin
;		print, f, orbit.point.lightning.flash(f).location(0), $
;		          orbit.point.lightning.flash(f).location(1)
;	endfor

; ***
; *** EXAMPLE: Plot the viewtimes on a world map
; ***

	loadct,39
	vtgrid = fltarr(360,720)
	for i = long(0),long(orbit.point.point_summary.vt_count-1) do begin
		thisy = fix(orbit.point.viewtime(i).location(0)*2+180)
		thisx = fix(orbit.point.viewtime(i).location(1)*2+360)
		vtgrid(thisy,thisx) = vtgrid(thisy,thisx) + orbit.point.viewtime(i).effective_obs
	endfor
	window,0,colors=256,xsize=360*4,ysize=360*2,retain=2,/pixmap
	tvscl,rebin(transpose(vtgrid),360*4,360*2)
	map_set,/continents,xmargin=[0,0],ymargin=[0,0],/noerase
	img = tvrd()
	slide_image,img,/register
; ***
; *** EXAMPLE: Plot up the alert summary flag for this orbit
; ***


; *** 
; *** EXAMPLE: Show all pixels lit up during this orbit (spooky - it's a ghost)
; ***

	window,2,colors=256,xsize=300,ysize=300,retain=2
	plot, orbit.point.lightning.event(*).x_pixel, orbit.point.lightning.event(*).y_pixel, psym=3, $
		xrange=[0,127],yrange=[0,127],/xstyle,/ystyle,xtitle='X Pixel',ytitle='Y Pixel'

; ***
; *** EXAMPLE: Traverse the point data tree
; ***

;	for a = 0, orbit.point.point_summary.area_count-1, 1 do begin

;		print, format='("A ",I5,F7.2,F7.2)',                 $ 
;		       a, orbit.point.lightning.area(a).location(0), $
;			  orbit.point.lightning.area(a).location(1)
;		
;		for f = orbit.point.lightning.area(a).child_address,  $
;			orbit.point.lightning.area(a).child_address + $
;			orbit.point.lightning.area(a).child_count - 1 do begin
;		
;			print, 	format='("     F ",I5,F7.2,F7.2)',               $
;				f-orbit.point.lightning.area(a).child_address, $
;				orbit.point.lightning.flash(f).location(0),    $
;				orbit.point.lightning.flash(f).location(1)
;				 
;			for g = orbit.point.lightning.flash(f).child_address,   $
;				orbit.point.lightning.flash(f).child_address +  $
;				orbit.point.lightning.flash(f).child_count - 1 do begin		
;				
;				print,  format='("          G ",I5,F7.2,F7.2)',         $
;					g-orbit.point.lightning.flash(f).child_address, $
;					orbit.point.lightning.group(g).location(0),     $
;					orbit.point.lightning.group(g).location(1)
;					  
;				for e = orbit.point.lightning.group(g).child_address,  $
;					orbit.point.lightning.group(g).child_address + $
;					orbit.point.lightning.group(g).child_count - 1 do begin
;					
;					print, 	format='("              E ",I5,F7.2,F7.2)',     $
;						e-orbit.point.lightning.group(g).child_address, $
;						orbit.point.lightning.event(e).location(0),  	$
;						orbit.point.lightning.event(e).location(1)
;						  
;				endfor
;		
;			endfor
;			
;		endfor
;		
;	endfor

; ***
; *** Don't want to waste time reading in viewtimes and one-second data?  Access any subelement directly:
; ***

	READ_groups, fname, groups
	
	; Whoops - since we didn't read a point summary, gotta figure out how many groups we got.
	
	gsize = size(groups)
	num_groups = gsize(1)	
	print,num_groups,' groups were read from ',fname
	
	; Here is some group data...
	
		group0_lat = groups(0).location(0)
		group0_lon = groups(0).location(1)
		
	;	etc.
		
	; See the library modules below for other READ_ routines

; ***
; *** Play around for yourself using this dataset...
; ***

	print,'Structure definition for type AN_ORBIT'
	help,orbit,/structures
	print,' '
	
	print,'Structure definition for type A_POINT'
	help,orbit.point,/structures
	print,' '
	
	print,'Structure definition for type A_LIGHTNING'
	help,orbit.point.lightning,/structures
	print,' '
	
	print,'Strucutre definition for type AN_EVENT'
	help,orbit.point.lightning.event,/structures
	print,' '
	
	print,'*** Variable "orbit" is still in memory - play with it!'
	
	; You get the picture!  Program will now stop; variable "orbit" is still in memory...
	
	STOP

END

pro which_sensor, fname, sensor

	hdffile = hdf_open(fname)
	if (hdffile le long(0)) then begin
		sensor = 'unknown'
		return
	endif
	vdref = hdf_vd_find(hdffile,"bg_summary")
	if (vdref ne 0) then begin
		sensor = 'lis'
	endif else begin
		vdref = hdf_vd_find(hdffile,"ephemeris")
		if (vdref ne 0) then begin
			sensor = 'otd'
		endif else begin
			sensor = 'unknown'
		endelse
	endelse
	hdf_close,hdffile

	return
end

function get_version

	vers = str_sep(string(!version.release),'.')
	
	version = vers(0)
		
	return, version

end

; ***********************************************************************************************
; *   				LISOTD Read Library routines; 10/27/97 - DJB			*
; ***********************************************************************************************


pro getvdsizes,fname,vdsizes
	hdffile = hdf_open(fname)
	if (hdffile eq long(0)) then begin
	   print,"Error opening file *"+fname+"*"
	   hdf_close,hdffile
	   stop
	endif
	vdname = ["orbit_summary","point_summary","area","flash","group","event","viewtime","bg_summary","one_second"]
	vdsizes = lonarr(9)
	for vdn=0,8,1 do begin
		vdref = hdf_vd_find(hdffile,vdname(vdn))
		if (vdref ne 0) then begin
			vd = hdf_vd_attach(hdffile,vdref,/read)
			hdf_vd_get,vd,class=cl,count=ct,fields=flds,interlace=ilc,name=nm,nfields=nflds,ref=rf,size=sz,tag=tg
			for i=0,nflds-1 do begin
			   hdf_vd_getinfo,vd,i,name=n,type=t,order=o
			endfor
			vdsizes(vdn) = long(sz);
			hdf_vd_detach,vd
		endif else begin
			vdsizes(vdn) = -1;
		endelse
	endfor
	hdf_close,hdffile
	return
end

pro getosvdsize,fname,vdsizes
	hdffile = hdf_open(fname)
	if (hdffile eq long(0)) then begin
	   print,"Error opening file *"+fname+"*"
	   hdf_close,hdffile
	   stop
	endif
	vdname = ["one_second"]
	vdsizes = lonarr(9)
	for vdn=0,0,1 do begin
		vdref = hdf_vd_find(hdffile,vdname(vdn))
		vd = hdf_vd_attach(hdffile,vdref,/read)
		hdf_vd_get,vd,class=cl,count=ct,fields=flds,interlace=ilc,name=nm,nfields=nflds,ref=rf,size=sz,tag=tg
		for i=0,nflds-1 do begin
		   hdf_vd_getinfo,vd,i,name=n,type=t,order=o
		endfor
		vdsizes(vdn) = long(sz);
		hdf_vd_detach,vd
	endfor
	hdf_close,hdffile
	return
end

pro getvtvdsize,fname,vdsizes
	hdffile = hdf_open(fname)
	if (hdffile eq long(0)) then begin
	   print,"Error opening file *"+fname+"*"
	   hdf_close,hdffile
	   stop
	endif
	vdname = ["viewtime"]
	vdsizes = lonarr(9)
	for vdn=0,0,1 do begin
		vdref = hdf_vd_find(hdffile,vdname(vdn))
		vd = hdf_vd_attach(hdffile,vdref,/read)
		hdf_vd_get,vd,class=cl,count=ct,fields=flds,interlace=ilc,name=nm,nfields=nflds,ref=rf,size=sz,tag=tg
		for i=0,nflds-1 do begin
		   hdf_vd_getinfo,vd,i,name=n,type=t,order=o
		endfor
		vdsizes(vdn) = long(sz);
		hdf_vd_detach,vd
	endfor
	hdf_close,hdffile
	return
end

pro getvdcounts,fname,vdsizes
	hdffile = hdf_open(fname)
	if (hdffile eq long(0)) then begin
	   print,"Error opening file *"+fname+"*"
	   hdf_close,hdffile
	   stop
	endif
	vdname = ["orbit_summary","point_summary","area","flash","group","event","viewtime","bg_summary","one_second"]
	vdsizes = lonarr(9)
	for vdn=0,8,1 do begin
		vdref = hdf_vd_find(hdffile,vdname(vdn))
		vd = hdf_vd_attach(hdffile,vdref,/read)
		hdf_vd_get,vd,class=cl,count=ct,fields=flds,interlace=ilc,name=nm,nfields=nflds,ref=rf,size=sz,tag=tg
		for i=0,nflds-1 do begin
		   hdf_vd_getinfo,vd,i,name=n,type=t,order=o
		endfor
		vdsizes(vdn) = long(ct);
		hdf_vd_detach,vd
	endfor
	hdf_close,hdffile
	return
end

pro getoscounts,fname,vdsizes
	hdffile = hdf_open(fname)
	if (hdffile eq long(0)) then begin
	   print,"Error opening file *"+fname+"*"
	   hdf_close,hdffile
	   stop
	endif
	vdname = ["one_second"]
	vdsizes = lonarr(9)
	for vdn=0,0,1 do begin
		vdref = hdf_vd_find(hdffile,vdname(vdn))
		vd = hdf_vd_attach(hdffile,vdref,/read)
		hdf_vd_get,vd,class=cl,count=ct,fields=flds,interlace=ilc,name=nm,nfields=nflds,ref=rf,size=sz,tag=tg
		for i=0,nflds-1 do begin
		   hdf_vd_getinfo,vd,i,name=n,type=t,order=o
		endfor
		vdsizes(vdn) = long(ct);
		hdf_vd_detach,vd
	endfor
	hdf_close,hdffile
	return
end

pro getvtcounts,fname,vdsizes
	hdffile = hdf_open(fname)
	if (hdffile eq long(0)) then begin
	   print,"Error opening file *"+fname+"*"
	   hdf_close,hdffile
	   stop
	endif
	vdname = ["viewtime"]
	vdsizes = lonarr(9)
	for vdn=0,0,1 do begin
		vdref = hdf_vd_find(hdffile,vdname(vdn))
		vd = hdf_vd_attach(hdffile,vdref,/read)
		hdf_vd_get,vd,class=cl,count=ct,fields=flds,interlace=ilc,name=nm,nfields=nflds,ref=rf,size=sz,tag=tg
		for i=0,nflds-1 do begin
		   hdf_vd_getinfo,vd,i,name=n,type=t,order=o
		endfor
		vdsizes(vdn) = long(ct);
		hdf_vd_detach,vd
	endfor
	hdf_close,hdffile
	return
end

pro READ_orbit, fname, this_orbit, NO_ONE_SECONDS = no_one_seconds, NATIVE=native, QUIET=quiet, _EXTRA=e
						
	if (keyword_set(quiet)) then begin
		quiet=1
	endif else begin
		quiet=0
	endelse
					if (quiet eq 0) then begin
						status_title = 'Reading "'+fname+'"'
						if (get_version() ge 5) then begin
							dummy_base = Widget_Base()
							status_base = Widget_Base(Column=1,title=status_title,uvalue=[0,0],tlb_frame_attr=0,scroll=0,/modal,group_leader=dummy_base)
						endif else begin
							status_base = Widget_Base(Row=1,title=status_title,uvalue=[0,0],tlb_frame_attr=0,scroll=0)
						endelse
;						status = CW_Field(status_base, value='Status:  Reading ORBIT',title='Status:',xsize=1.5*strlen(fname),/noedit)
						status = Widget_label(status_base, value='Status:  Reading ORBIT',xsize=12*strlen(fname),/align_left)
						Widget_Control, status_base, /realize
						Widget_Control, status, Set_value='Status:  ( 0% done) -> Reading ORBIT_SUMMARY'
					endif
	READ_orbit_summary,fname,orbit_summary,status=status,_EXTRA=e
					if (quiet eq 0) then begin	
						Widget_Control, status, Set_value='Status:  ( 0% done) -> Reading POINT'
						pct_done = [84L,84L+(1-keyword_set(no_one_seconds))*5400L*119L]
						Widget_Control, status, Set_UValue = pct_done
					endif	
	this_os = !version.os
	call_nat = keyword_set(native)

	if (call_nat eq 1) then begin
		READ_point,fname,point,status=status,native=native,QUIET=quiet,_EXTRA=e
	endif else begin
		READ_point,fname,point,status=status,QUIET=quiet,_EXTRA=e
	endelse
	
	if keyword_set(no_one_seconds) then begin
		this_one_second = {a_one_second, TAI93_time:double(0), alert_summary:byte(0), 	$
				instrument_alert:byte(0), platform_alert:byte(0),	$
				external_alert:byte(0), processing_alert:byte(0),	$
				position_vector:fltarr(3), velocity_vector:fltarr(3),	$
				transform_matrix:fltarr(9), solar_vector:fltarr(3),	$
				ephemeris_quality_flag:long(0), attitude_quality_flag:long(0), $
				boresight_threshold:byte(0), thresholds:bytarr(16), 	$
				noise_index:byte(0), event_count:intarr(6)}
		one_second = replicate(this_one_second,1)
		orbit_summary.one_second_count = 1
	endif else begin
					if (quiet eq 0) then begin
						Widget_Control, status, Get_UValue=pct_done
						status_msg = string(100*pct_done(0)/pct_done(1), '("Status:  (",I2,"% done) -> Reading ONE SECONDs...")')
						Widget_Control, status, Set_Value=status_msg
					endif
		if (call_nat eq 0) then begin
			read_one_second_data,fname,one_second,status=status,_EXTRA=e
		endif else begin
			read_one_second_data,fname,one_second,status=status,native=native,_EXTRA=e
		endelse
					if (quiet eq 0) then begin
						pct_done(0) = pct_done(0)+5400L*119L
						Widget_Control, status, Set_UValue = pct_done
					endif
	endelse
	
	this_orbit = {	orbit_summary:{an_orbit_summary},			$
				point:replicate(point,1),					$
				one_second:replicate(one_second(0),max([orbit_summary.one_second_count,1]))}
	this_orbit.orbit_summary	= orbit_summary
	this_orbit.point		= point
	this_orbit.one_second		= one_second
	if (quiet eq 0) then begin
		Widget_Control, status, Set_value='Status:  Done reading ORBIT.'
							;print,'Done Reading ORBIT ',fname
							;print,' '
		Widget_Control, status_base, /Destroy	
	endif
	return
end

pro READ_point, fname, this_point, status=status, NO_VIEWTIMES = no_viewtimes, NATIVE=native, QUIET=quiet, _EXTRA=e

	if (keyword_set(quiet)) then begin
		quiet=1
	endif else begin
		quiet=0
	endelse
	this_os = !version.os
	call_nat = keyword_set(native)

					if (quiet eq 0) then begin
						Widget_Control, status, Set_value='Status:  --> Reading POINT_SUMMARY ( 0% done)'
					endif
					
	READ_point_summary,fname,point_summary,status=status,_EXTRA=e

					if (quiet eq 0) then begin
						if keyword_set(e) then begin
							usea = (where(tag_names(e) eq 'NO_AREAS') eq -1)
							usef = (where(tag_names(e) eq 'NO_FLAHSES') eq -1)
							useg = (where(tag_names(e) eq 'NO_GROUPS') eq -1)
							usee = (where(tag_names(e) eq 'NO_EVENTS') eq -1)
						endif else begin
							usea = 1 & usef = 1 & useg = 1 & usee = 1
						endelse
						
						usev = 1-keyword_set(no_viewtimes)
					
						Widget_Control, status, Get_UValue = pct_done
						
						pct_done(1) = pct_done(1) + 52L + point_summary.area_count*68L*usea + point_summary.flash_count*68L*usef + $
									point_summary.group_count*68L*useg + point_summary.event_count*60L*usee + 	   $
									point_summary.bg_count*52L + point_summary.vt_count*22L*usev 
						pct_done(0) = 84L+52L
						Widget_Control, status, Set_UValue = pct_done
						status_msg = string(100*pct_done(0)/pct_done(1), '("Status:  (",I2,"% done) -> Reading LIGHTNING...")')
						Widget_Control, status, Set_Value=status_msg
					endif
					
	anyltg = ((point_summary.area_count gt 0) and (point_summary.flash_count gt 0) and $
			(point_summary.group_count gt 0) and (point_summary.event_count gt 0))
			
	if (call_nat) then begin
		READ_lightning,fname,lightning,point_summary,status=status,native=native,QUIET=quiet,_EXTRA=e,anyltg=anyltg
	endif else begin
		READ_lightning,fname,lightning,point_summary,status=status,QUIET=quiet,_EXTRA=e,anyltg=anyltg
	endelse
		
	if keyword_set(no_viewtimes) then begin
		this_viewtime = {a_viewtime, location:fltarr(2), TAI93_start:long(0), TAI93_end:long(0), effective_obs:float(0), $
				alert_flag:byte(0), approx_threshold:byte(0)}
		viewtime = replicate(this_viewtime,1)
		point_summary.vt_count = 1
	endif else begin
					if (quiet eq 0) then begin
						Widget_Control, status, Get_UValue=pct_done
						status_msg = string(100*pct_done(0)/pct_done(1), '("Status:  (",I2,"% done) --> Reading VIEWTIMEs...")')
						Widget_Control, status, Set_Value=status_msg
					endif
		if (call_nat) then begin
			READ_viewtimes,fname,viewtime,status=status,native=native
		endif else begin
			READ_viewtimes,fname,viewtime,status=status
		endelse
		
					if (quiet eq 0) then begin
						pct_done(0) = pct_done(0) + point_summary.vt_count*22L
						Widget_Control, status, Set_UValue=pct_done
					endif		
	endelse
					if (quiet eq 0) then begin
						Widget_Control, status, Get_UValue=pct_done
						status_msg = string(100*pct_done(0)/pct_done(1), '("Status:  (",I2,"% done) --> Reading BG_SUMMARYs...")')
						Widget_Control, status, Set_Value=status_msg
					endif
					
	READ_bg_summaries,fname,bg_summary,status=status,_EXTRA=e
	
					if (quiet eq 0) then begin
						pct_done(0) = pct_done(0) + point_summary.bg_count*52L
						Widget_Control, status, Set_UValue=pct_done
					endif
											
	this_point = {		point_summary:{a_point_summary},				$ 	
				lightning:replicate(lightning,1),				$
				viewtime:replicate(viewtime(0), max([point_summary.vt_count,1])), 	$
				bg_summary:replicate(bg_summary(0),max([point_summary.bg_count,1]))}
	this_point.point_summary 	= point_summary
	this_point.lightning		= lightning
	this_point.viewtime		= viewtime
	this_point.bg_summary		= bg_summary
	return
end
				
pro READ_lightning, fname, this_lightning, point_summary, status=status, NO_AREAS=no_areas,NO_FLASHES=no_flashes,NO_GROUPS=no_groups,NO_EVENTS=no_events,NATIVE=native,QUIET=quiet,_EXTRA=e, anyltg=anyltg
	if (keyword_set(quiet)) then begin
		quiet=1
	endif else begin
		quiet=0
	endelse
	this_os = !version.os
	call_nat = keyword_set(native)
	
	if (keyword_set(no_areas) or (anyltg eq 0)) then begin
		if (call_nat eq 1 and this_os eq 'MacOS') then begin
			this_area = {an_area,TAI93_time:0D,delta_time:0.0,observe_time:0,location:fltarr(2),net_radiance:0.0, $
					footprint:0.0,address:0L,parent_address:0L,child_address:0L,child_count:0L,       $
					grandchild_count:0L,greatgrandchild_count:0L,approx_threshold:byte(0),alert_flag:byte(0),cluster_index: $
					byte(0),density_index:byte(0),noise_index:byte(0),grouping_status:byte(0)}
		endif else begin
			this_area = {an_area,TAI93_time:double(0),delta_time:float(0),observe_time:fix(0),location:fltarr(2),net_radiance:float(0), $
					footprint:float(0),address:long(0),parent_address:long(0),child_address:long(0),child_count:long(0),       $
					grandchild_count:long(0),greatgrandchild_count:long(0),approx_threshold:byte(0),alert_flag:byte(0),cluster_index: $
					byte(0),density_index:byte(0),noise_index:byte(0),oblong_index:float(0),grouping_sequence:long(0),         $
					grouping_status:byte(0)}
		endelse
		arearead = replicate(this_area,1)
		point_summary.area_count = 1	
	endif else begin
					if (quiet eq 0) then begin
						Widget_Control, status, Get_UValue=pct_done
						status_msg = string(100*pct_done(0)/pct_done(1), '("Status:  (",I2,"% done) ---> Reading AREAs...")')
						Widget_Control, status, Set_Value=status_msg
					endif
		if (call_nat eq 1) then begin
			;print,'Caling READ_areas'
			READ_areas,fname,arearead,status=status,native=native
		endif else begin
			;print,'Native IDL read_areas'
			READ_areas,fname,arearead,status=status
		endelse
					if (quiet eq 0) then begin
						pct_done(0) = pct_done(0) + point_summary.area_count*68L
						Widget_Control, status, Set_UValue=pct_done
					endif
	endelse
	if (keyword_set(no_flashes)  or (anyltg eq 0)) then begin
		if (call_nat eq 1 and this_os eq 'MacOS') then begin
			this_flash = {a_flash, TAI93_time:0D, delta_time:0.0, observe_time:0, location:fltarr(2) ,radiance:0.0, $
					footprint:0.0, address:0L, parent_address:0L, child_address:0L, child_count:0L,      $
					grandchild_count:0L, approx_threshold:byte(0), alert_flag:byte(0), cluster_index:byte(0), density_index: $
					byte(0), noise_index:byte(0), glint_index:0.0, grouping_sequence:0L,       $
					grouping_status:byte(0)}
		endif else begin
			this_flash = {a_flash, TAI93_time:double(0), delta_time:float(0), observe_time:fix(0), location:fltarr(2) ,radiance:float(0), $
					footprint:float(0), address:long(0), parent_address:long(0), child_address:long(0), child_count:long(0),      $
					grandchild_count:long(0), approx_threshold:byte(0), alert_flag:byte(0), cluster_index:byte(0), density_index: $
					byte(0), noise_index:byte(0), glint_index:float(0), oblong_index:float(0), grouping_sequence:long(0),         $
					grouping_status:byte(0)}
		endelse
		flash = replicate(this_flash,1)
		point_summary.flash_count = 1	
	endif else begin
					if (quiet eq 0) then begin
						Widget_Control, status, Get_UValue=pct_done
						status_msg = string(100*pct_done(0)/pct_done(1), '("Status:  (",I2,"% done) --> Reading FLASHes...")')
						Widget_Control, status, Set_Value=status_msg
					endif
		if (call_nat eq 1) then begin
			READ_flashes,fname,flash,status=status,native=native
		endif else begin
			READ_flashes,fname,flash,status=status
		endelse
					if (quiet eq 0) then begin
						pct_done(0) = pct_done(0) + point_summary.flash_count*68L
						Widget_Control, status, Set_UValue=pct_done
					endif
	endelse
	if (keyword_set(no_groups)  or (anyltg eq 0)) then begin
		if (call_nat eq 1 and this_os eq 'MacOS') then begin
			this_group = {a_group, TAI93_time:double(0), observe_time:fix(0), location:fltarr(2) ,radiance:float(0),                 $
					footprint:float(0), address:long(0), parent_address:long(0), child_address:long(0), child_count:long(0), $
					approx_threshold:byte(0), alert_flag:byte(0), cluster_index:byte(0), density_index:                      $
					byte(0), noise_index:byte(0), glint_index:float(0), grouping_sequence:long(0),    $
					grouping_status:byte(0)}
		endif else begin
			this_group = {a_group, TAI93_time:0.0D, observe_time:0, location:fltarr(2) ,radiance:0.0,                 $
					footprint:0.0, address:0L, parent_address:0L, child_address:0L, child_count:0L, $
					approx_threshold:byte(0), alert_flag:byte(0), cluster_index:byte(0), density_index:                      $
					byte(0), noise_index:byte(0), glint_index:0.0, oblong_index:0.0, grouping_sequence:0L,    $
					grouping_status:byte(0)}
		endelse
		group = replicate(this_group,1)
		point_summary.group_count = 1
	endif else begin
					if (quiet eq 0) then begin
						Widget_Control, status, Get_UValue=pct_done
						status_msg = string(100*pct_done(0)/pct_done(1), '("Status:  (",I2,"% done) ---> Reading GROUPs...")')
						Widget_Control, status, Set_Value=status_msg
					endif
		if (call_nat eq 1) then begin
			READ_groups,fname,group,status=status,native=native
		endif else begin
			READ_groups,fname,group,status=status
		endelse
					if (quiet eq 0) then begin
						pct_done(0) = pct_done(0) + point_summary.group_count*60L
						Widget_Control, status, Set_UValue=pct_done
					endif
	endelse
	if (keyword_set(no_events) or (anyltg eq 0)) then begin
		this_event = {an_event, TAI93_time:double(0), observe_time:fix(0), location:fltarr(2) ,radiance:float(0),       $
				footprint:float(0), address:long(0), parent_address:long(0), x_pixel:byte(0), y_pixel:byte(0),  $
				bg_value:fix(0), bg_radiance:fix(0), amplitude:byte(0), sza_index:byte(0), glint_index:byte(0), $
				approx_threshold:byte(0), alert_flag:byte(0), cluster_index:byte(0), density_index:                      $
				byte(0), noise_index:byte(0), bg_value_flag:byte(0), grouping_sequence:long(0)}
		event = replicate(this_event,1)
		point_summary.event_count = 1
	endif else begin
					if (quiet eq 0) then begin
						Widget_Control, status, Get_UValue=pct_done
						status_msg = string(100*pct_done(0)/pct_done(1), '("Status:  (",I2,"% done) ---> Reading EVENTs...")')
						Widget_Control, status, Set_Value=status_msg
					endif
		if (call_nat eq 1) then begin
			READ_events,fname,event,status=status,native=native
		endif else begin
			READ_events,fname,event,status=status
		endelse
					if (quiet eq 0) then begin
						pct_done(0) = pct_done(0) + point_summary.event_count*53L
						Widget_Control, status, Set_UValue=pct_done
					endif
	endelse
	this_lightning = {	  area:replicate(arearead(0),  max([point_summary.area_count,1])), 	$
				       flash:replicate(flash(0), max([point_summary.flash_count,1])),   $
				       group:replicate(group(0), max([point_summary.group_count,1])),   $
				       event:replicate(event(0), max([point_summary.event_count,1]))}
	this_lightning.area  = arearead
	this_lightning.flash = flash
	this_lightning.group = group
	this_lightning.event = event
	return
end
		       	

pro READ_orbit_summary,fname,this_orbit_summary, status=status,_EXTRA=e
	; Read in Orbit Summary
	getvdsizes,fname,vdsizes
	hdffile = hdf_open(fname)
	if (hdffile eq long(0)) then begin
	   print,"Error opening file *"+fname+"*"
	   hdf_close,hdffile
	   stop
	endif
	vdref = hdf_vd_find(hdffile,"orbit_summary")
	this_orbit_summary = {an_orbit_summary,id_number:long(0),TAI93_start:double(0),UTC_start:bytarr(28),GPS_start:double(0), $
				TAI93_end:double(0),start_longitude:float(0),end_longitude:float(0),point_data_count:fix(0), $
				point_data_address:fix(0),one_second_count:long(0),one_second_address:long(0),summary_image_count: $
				fix(0),summary_image_address:fix(0),inspection_code:fix(0),configuration_code:fix(0)}
	vd = hdf_vd_attach(hdffile,vdref,/read)
	nread = hdf_vd_read(vd,temp)
	this_orbit_summary.id_number = long(temp,0)
	this_orbit_summary.TAI93_start = double(temp,4)
	for i=0,27 do begin
	  this_orbit_summary.UTC_start(i) = byte(temp,12+i)
	endfor
	this_orbit_summary.GPS_start = double(temp,40)
	this_orbit_summary.TAI93_end = double(temp,48)
	this_orbit_summary.start_longitude = float(temp,56)
	this_orbit_summary.end_longitude = float(temp,60)
	this_orbit_summary.point_data_count = fix(temp,64)
	this_orbit_summary.point_data_address = fix(temp,66)
	this_orbit_summary.one_second_count = long(temp,68)
	this_orbit_summary.one_second_address = long(temp,72)
	this_orbit_summary.summary_image_count = fix(temp,76)
	this_orbit_summary.summary_image_address = fix(temp,78)
	this_orbit_summary.inspection_code = fix(temp,80)
	this_orbit_summary.configuration_code = fix(temp,82)
	hdf_vd_detach,vd
	hdf_close,hdffile
	return
end

pro READ_one_second_data,fname,these_one_seconds,status=status,native=native,_EXTRA=e

	this_os = !version.os
	call_nat = keyword_set(native)

	this_one_second = {a_one_second, TAI93_time:double(0), alert_summary:byte(0), 	$
					instrument_alert:byte(0), platform_alert:byte(0),	$
					external_alert:byte(0), processing_alert:byte(0),	$
					position_vector:fltarr(3), velocity_vector:fltarr(3),	$
					transform_matrix:fltarr(9), solar_vector:fltarr(3),	$
					ephemeris_quality_flag:long(0), attitude_quality_flag:long(0), $
					boresight_threshold:byte(0), thresholds:bytarr(16), 	$
					noise_index:byte(0), event_count:intarr(6)}

	if (call_nat eq 1) then begin
		READ_orbit_summary,fname,os,status=status,_EXTRA=e
		nread = os.one_second_count
		these_one_seconds = replicate({a_one_second},max([nread,1]))
		if (nread gt 0) then begin
			res = CALL_EXTERNAL(native,'glis_one_second',fname,these_one_seconds)
		endif
	endif else begin
		getosvdsize,fname,vdsizes
		hdffile = hdf_open(fname)
		if (hdffile eq long(0)) then begin
		   print,"Error opening file *"+fname+"*"
		   hdf_close,hdffile
		   stop
		endif
		vdref = hdf_vd_find(hdffile,"one_second")
		vd = hdf_vd_attach(hdffile,vdref,/read)
		nread = hdf_vd_read(vd,tempone_second)
		newone_second = bytarr(vdsizes(0),nread)
		insize = size(tempone_second)
		inwidth = insize(1)
		these_one_seconds = replicate({a_one_second},max([nread,1]))
		; Note - IDL reads things in on even byte boundaries, must repack the byte array to the correct HDF size
;		for jj = long(0),long(nread*vdsizes(0)-1),long(1) do begin
;			newrecord  = jj / vdsizes(0)
;			newelement = jj mod vdsizes(0)
;			oldrecord  = jj / inwidth
;			oldelement = jj mod inwidth
;			newone_second(newelement,newrecord) = tempone_second(oldelement,oldrecord)
;		endfor
		jj = lindgen(nread*vdsizes(0))
		newone_second(jj) = tempone_second(jj)
		jj = 0
		tempone_second = 0
		for i = long(0), long(nread-1), long(1) do begin
			thisone_second = bytarr(vdsizes(0))
			thisone_second = newone_second(*,i)
			these_one_seconds(i).TAI93_time		= double(thisone_second,0);
			these_one_seconds(i).alert_summary	= byte(thisone_second,8);
			these_one_seconds(i).instrument_alert	= byte(thisone_second,9);
			these_one_seconds(i).platform_alert	= byte(thisone_second,10);
			these_one_seconds(i).external_alert	= byte(thisone_second,11);
			these_one_seconds(i).processing_alert	= byte(thisone_second,12);
			these_one_seconds(i).position_vector(0)	= float(thisone_second,13);
			these_one_seconds(i).position_vector(1)	= float(thisone_second,17);
			these_one_seconds(i).position_vector(2)	= float(thisone_second,21);
			these_one_seconds(i).velocity_vector(0)	= float(thisone_second,25);
			these_one_seconds(i).velocity_vector(1)	= float(thisone_second,29);
			these_one_seconds(i).velocity_vector(2)	= float(thisone_second,33);
			these_one_seconds(i).transform_matrix(0)= float(thisone_second,37);
			these_one_seconds(i).transform_matrix(1)= float(thisone_second,41);
			these_one_seconds(i).transform_matrix(2)= float(thisone_second,45);
			these_one_seconds(i).transform_matrix(3)= float(thisone_second,49);
			these_one_seconds(i).transform_matrix(4)= float(thisone_second,53);
			these_one_seconds(i).transform_matrix(5)= float(thisone_second,57);
			these_one_seconds(i).transform_matrix(6)= float(thisone_second,61);
			these_one_seconds(i).transform_matrix(7)= float(thisone_second,65);
			these_one_seconds(i).transform_matrix(8)= float(thisone_second,69);
			these_one_seconds(i).solar_vector(0)	= float(thisone_second,73);
			these_one_seconds(i).solar_vector(1)	= float(thisone_second,77);
			these_one_seconds(i).solar_vector(2)	= float(thisone_second,81);
			these_one_seconds(i).ephemeris_quality_flag=long(thisone_second,85);
			these_one_seconds(i).attitude_quality_flag= long(thisone_second,89);
			these_one_seconds(i).boresight_threshold= byte(thisone_second,93);
			these_one_seconds(i).thresholds(0)	= byte(thisone_second,94);
			these_one_seconds(i).thresholds(1)	= byte(thisone_second,95);
			these_one_seconds(i).thresholds(2)	= byte(thisone_second,96);
			these_one_seconds(i).thresholds(3)	= byte(thisone_second,97);
			these_one_seconds(i).thresholds(4)	= byte(thisone_second,98);
			these_one_seconds(i).thresholds(5)	= byte(thisone_second,99);
			these_one_seconds(i).thresholds(6)	= byte(thisone_second,100);
			these_one_seconds(i).thresholds(7)	= byte(thisone_second,101);
			these_one_seconds(i).thresholds(8)	= byte(thisone_second,102);
			these_one_seconds(i).thresholds(9)	= byte(thisone_second,103);
			these_one_seconds(i).thresholds(10)	= byte(thisone_second,104);
			these_one_seconds(i).thresholds(11)	= byte(thisone_second,105);
			these_one_seconds(i).thresholds(12)	= byte(thisone_second,106);
			these_one_seconds(i).thresholds(13)	= byte(thisone_second,107);
			these_one_seconds(i).thresholds(14)	= byte(thisone_second,108);
			these_one_seconds(i).thresholds(15)	= byte(thisone_second,109);
			these_one_seconds(i).noise_index	= byte(thisone_second,110);
			these_one_seconds(i).event_count(0)	= fix(thisone_second,111);
			these_one_seconds(i).event_count(1)	= fix(thisone_second,113);
			these_one_seconds(i).event_count(2)	= fix(thisone_second,115);
			these_one_seconds(i).event_count(3)	= fix(thisone_second,117);
			these_one_seconds(i).event_count(4)	= fix(thisone_second,119);
			these_one_seconds(i).event_count(5)	= fix(thisone_second,121);
		endfor
		hdf_vd_detach,vd
		hdf_close,hdffile
	endelse
	return
end

pro READ_point_summary,fname,this_point_summary,status=status,_EXTRA=e
	; Read in Point Data Summary
	getvdsizes,fname,vdsizes
	hdffile = hdf_open(fname)
	if (hdffile eq long(0)) then begin
	   print,"Error opening file *"+fname+"*"
	   hdf_close,hdffile
	   stop
	endif
	vdref = hdf_vd_find(hdffile,"point_summary")
	this_point_summary = {a_point_summary,parent_address:long(0),event_count:long(0),event_address:long(0),group_count:long(0), $
				group_address:long(0),flash_count:long(0),flash_address:long(0),area_count:long(0),area_address:long(0), $
				bg_count:long(0),bg_address:long(0),vt_count:long(0),vt_address:long(0)}
	vd = hdf_vd_attach(hdffile,vdref,/read)
	nread = hdf_vd_read(vd,temp)
	this_point_summary.parent_address = long(temp,0)
	this_point_summary.event_count    = long(temp,4)
	this_point_summary.event_address  = long(temp,8)
	this_point_summary.group_count    = long(temp,12)
	this_point_summary.group_address  = long(temp,16)
	this_point_summary.flash_count    = long(temp,20)
	this_point_summary.flash_address  = long(temp,24)
	this_point_summary.area_count     = long(temp,28)
	this_point_summary.area_address   = long(temp,32)
	this_point_summary.bg_count       = long(temp,36)
	this_point_summary.bg_address     = long(temp,40)
	this_point_summary.vt_count       = long(temp,44)
	this_point_summary.vt_address     = long(temp,48)
	hdf_vd_detach,vd
	hdf_close,hdffile
	return
end

pro READ_areas,fname,these_areas,status=status,native=native,_EXTRA=e

	this_os = !version.os
	call_nat = keyword_set(native)
	if (call_nat eq 1 and this_os eq 'MacOS') then begin
		;print,'Shared libraries in MacOS'
		this_area = {an_area,TAI93_time:0D,delta_time:0.0,observe_time:0,location:fltarr(2),net_radiance:0.0, $
				footprint:0.0,address:0L,parent_address:0L,child_address:0L,child_count:0L,       $
				grandchild_count:0L,greatgrandchild_count:0L,approx_threshold:byte(0),alert_flag:byte(0),cluster_index: $
				byte(0),density_index:byte(0),noise_index:byte(0),grouping_status:byte(0)}
	endif else begin
		this_area = {an_area,TAI93_time:double(0),delta_time:float(0),observe_time:fix(0),location:fltarr(2),net_radiance:float(0), $
				footprint:float(0),address:long(0),parent_address:long(0),child_address:long(0),child_count:long(0),       $
				grandchild_count:long(0),greatgrandchild_count:long(0),approx_threshold:byte(0),alert_flag:byte(0),cluster_index: $
				byte(0),density_index:byte(0),noise_index:byte(0),oblong_index:float(0),grouping_sequence:long(0),         $
				grouping_status:byte(0)}
	endelse

	if (call_nat eq 1) then begin
		READ_point_summary,fname,ps,status=status,_EXTRA=e
		nread = ps.area_count
		these_areas = replicate({an_area},max([nread,1]))
		;print,'About to call glis_area',nread
        	if (nread gt 0) then begin
			res=CALL_EXTERNAL(native,'glis_area',fname,these_areas)
		endif
	endif else begin
		;print,'Non-native II'
		getvdsizes,fname,vdsizes
		hdffile = hdf_open(fname)
		if (hdffile eq long(0)) then begin
		   print,"Error opening file *"+fname+"*"
		   hdf_close,hdffile
		   stop
		endif
		vdref = hdf_vd_find(hdffile,"area")
		vdct = 0 
		vd = hdf_vd_attach(hdffile,vdref,/read)
		hdf_vd_get,vd,count=vdct
		if (vdref gt 0 and vdct gt 0 and vd gt 0) then begin
			;print,vdref,vdct,vd
			nread = hdf_vd_read(vd,temparea)
			;print,nread,vdsizes(2)
			newarea = bytarr(vdsizes(2),nread)
			; help,newarea
			insize = size(temparea)
			inwidth = insize(1)
			these_areas = replicate({an_area},max([nread,1]))
			; Note - IDL reads things in on even byte boundaries, must repack the byte array to the correct HDF size
;			for jj = long(0),long(nread*vdsizes(2)-1),long(1) do begin
;			   newrecord  = jj / vdsizes(2)
;			   newelement = jj mod vdsizes(2)
;			   oldrecord  = jj / inwidth
;			   oldelement = jj mod inwidth
;			   newarea(newelement,newrecord) = temparea(oldelement,oldrecord)
;			endfor 
			jj = lindgen(nread*vdsizes(2))
			newarea(jj) = temparea(jj)
			jj = 0
			temparea = 0
			for i = 0,nread-1,1 do begin
				thisarea = bytarr(vdsizes(2))
				thisarea = newarea(*,i)
				these_areas(i).TAI93_time   = double(thisarea,0)
				these_areas(i).delta_time   = float(thisarea,8)
				these_areas(i).observe_time = fix(thisarea,12)
				these_areas(i).location(0)  = float(thisarea,14)
				these_areas(i).location(1)  = float(thisarea,18)
				these_areas(i).net_radiance = float(thisarea,22)
				these_areas(i).footprint    = float(thisarea,26)
				these_areas(i).address      = long(thisarea,30)
				these_areas(i).parent_address=long(thisarea,34)
				these_areas(i).child_address= long(thisarea,38)
				these_areas(i).child_count  = long(thisarea,42)
				these_areas(i).grandchild_count = long(thisarea,46)
				these_areas(i).greatgrandchild_count = long(thisarea,50)
				these_areas(i).approx_threshold = byte(thisarea,54)
				these_areas(i).alert_flag       = byte(thisarea,55)
				these_areas(i).cluster_index    = byte(thisarea,56)
				these_areas(i).density_index    = byte(thisarea,57)
				these_areas(i).noise_index      = byte(thisarea,58)
				these_areas(i).oblong_index     = float(thisarea,59)
				these_areas(i).grouping_sequence = long(thisarea,63)
				these_areas(i).grouping_status   = byte(thisarea,67)
			endfor
			hdf_vd_detach,vd
			hdf_close,hdffile
		endif else begin
			these_areas = replicate({an_area},1)
		endelse
	endelse
	return
end

pro READ_flashes,fname,these_flashes,status=status,native=native,_EXTRA=e

	this_os = !version.os
	call_nat = keyword_set(native)

	if (call_nat eq 1 and this_os eq 'MacOS') then begin
		this_flash = {a_flash, TAI93_time:0D, delta_time:0.0, observe_time:0, location:fltarr(2) ,radiance:0.0, $
				footprint:0.0, address:0L, parent_address:0L, child_address:0L, child_count:0L,      $
				grandchild_count:0L, approx_threshold:byte(0), alert_flag:byte(0), cluster_index:byte(0), density_index: $
				byte(0), noise_index:byte(0), glint_index:0.0, grouping_sequence:0L,       $
				grouping_status:byte(0)}
	endif else begin
		this_flash = {a_flash, TAI93_time:double(0), delta_time:float(0), observe_time:fix(0), location:fltarr(2) ,radiance:float(0), $
				footprint:float(0), address:long(0), parent_address:long(0), child_address:long(0), child_count:long(0),      $
				grandchild_count:long(0), approx_threshold:byte(0), alert_flag:byte(0), cluster_index:byte(0), density_index: $
				byte(0), noise_index:byte(0), glint_index:float(0), oblong_index:float(0), grouping_sequence:long(0),         $
				grouping_status:byte(0)}
	endelse

	if (call_nat eq 1) then begin
		READ_point_summary,fname,ps,status=status,_EXTRA=e
		nread = ps.flash_count
		these_flashes = replicate({a_flash},max([nread,1]))
        	if (nread gt 0) then begin
			res=CALL_EXTERNAL(native,'glis_flash',fname,these_flashes)
		endif
	endif else begin
		getvdsizes,fname,vdsizes
		hdffile = hdf_open(fname)
		if (hdffile eq long(0)) then begin
		   print,"Error opening file *"+fname+"*"
		   hdf_close,hdffile
		   stop
		endif
		vdref = hdf_vd_find(hdffile,"flash")
		vdct = 0 
		vd = hdf_vd_attach(hdffile,vdref,/read)
		hdf_vd_get,vd,count=vdct
		if (vdref gt 0 and vdct gt 0 and vd gt 0) then begin
			nread = hdf_vd_read(vd,tempflash)
			newflash = bytarr(vdsizes(3),nread)
			insize = size(tempflash)
			inwidth = insize(1)
			these_flashes = replicate({a_flash},max([nread,1]))	
			; Note - IDL reads things in on even byte boundaries, must repack the byte array to the correct HDF size
;			for jj = long(0),long(nread*vdsizes(3)-1),long(1) do begin
;				newrecord  = jj / vdsizes(3)
;				newelement = jj mod vdsizes(3)
;				oldrecord  = jj / inwidth
;				oldelement = jj mod inwidth
;				newflash(newelement,newrecord) = tempflash(oldelement,oldrecord)
;			endfor	
			jj = lindgen(nread*vdsizes(3))
			newflash(jj) = tempflash(jj)
			jj = 0
			tempflash = 0
			for i = 0, nread-1, 1 do begin
				thisflash = bytarr(vdsizes(3))
				thisflash = newflash(*,i)
				these_flashes(i).TAI93_time	= double(thisflash,0)
				these_flashes(i).delta_time	= float(thisflash,8)
				these_flashes(i).observe_time	= fix(thisflash,12)
				these_flashes(i).location(0)	= float(thisflash,14)
				these_flashes(i).location(1)	= float(thisflash,18)
				these_flashes(i).radiance	= float(thisflash,22)
				these_flashes(i).footprint	= float(thisflash,26)
				these_flashes(i).address	= long(thisflash,30)
				these_flashes(i).parent_address = long(thisflash,34)
				these_flashes(i).child_address	= long(thisflash,38)
				these_flashes(i).child_count	= long(thisflash,42)
				these_flashes(i).grandchild_count = long(thisflash,46)
				these_flashes(i).approx_threshold= byte(thisflash,50)
				these_flashes(i).alert_flag	= byte(thisflash,51)
				these_flashes(i).cluster_index  = byte(thisflash,52)
				these_flashes(i).density_index	= byte(thisflash,53)
				these_flashes(i).noise_index    = byte(thisflash,54)
				these_flashes(i).glint_index    = float(thisflash,55)
				these_flashes(i).oblong_index	= float(thisflash,59)
				these_flashes(i).grouping_sequence = long(thisflash,63)
				these_flashes(i).grouping_status   = byte(thisflash,67)
			endfor
			hdf_vd_detach,vd
			hdf_close,hdffile
		endif else begin
			these_flashes = replicate({a_flash},1)
		endelse
	endelse
	return
end

pro READ_groups,fname,these_groups,status=status,native=native,_EXTRA=e

	this_os = !version.os
	call_nat = keyword_set(native)

	if (call_nat eq 1 and this_os eq 'MacOS') then begin
		this_group = {a_group, TAI93_time:double(0), observe_time:fix(0), location:fltarr(2) ,radiance:float(0),                 $
				footprint:float(0), address:long(0), parent_address:long(0), child_address:long(0), child_count:long(0), $
				approx_threshold:byte(0), alert_flag:byte(0), cluster_index:byte(0), density_index:                      $
				byte(0), noise_index:byte(0), glint_index:float(0), grouping_sequence:long(0),    $
				grouping_status:byte(0)}
	endif else begin
		this_group = {a_group, TAI93_time:0.0D, observe_time:0, location:fltarr(2) ,radiance:0.0,                 $
					footprint:0.0, address:0L, parent_address:0L, child_address:0L, child_count:0L, $
					approx_threshold:byte(0), alert_flag:byte(0), cluster_index:byte(0), density_index:                      $
					byte(0), noise_index:byte(0), glint_index:0.0, oblong_index:0.0, grouping_sequence:0L,    $
					grouping_status:byte(0)}
	endelse

	if (call_nat eq 1) then begin
		READ_point_summary,fname,ps,status=status,_EXTRA=e
		nread = ps.group_count
		these_groups = replicate({a_group},max([nread,1]))
		if (nread gt 0) then begin
			res=CALL_EXTERNAL(native,'glis_group',fname,these_groups)
		endif
	endif else begin
		getvdsizes,fname,vdsizes
		hdffile = hdf_open(fname)
		if (hdffile eq long(0)) then begin
		   print,"Error opening file *"+fname+"*"
		   hdf_close,hdffile
		   stop
		endif
		vdref = hdf_vd_find(hdffile,"group")
		vdct = 0 
		vd = hdf_vd_attach(hdffile,vdref,/read)
		hdf_vd_get,vd,count=vdct
		if (vdref gt 0 and vdct gt 0 and vd gt 0) then begin
			nread = hdf_vd_read(vd,tempgroup)
			newgroup = bytarr(vdsizes(4),nread)
			insize = size(tempgroup)
			inwidth = insize(1)
			these_groups = replicate({a_group},max([nread,1]))
			; Note - IDL reads things in on even byte boundaries, must repack the byte array to the correct HDF size
;			for jj = long(0),long(nread*vdsizes(4)-1),long(1) do begin
;				newrecord  = jj / vdsizes(4)
;				newelement = jj mod vdsizes(4)
;				oldrecord  = jj / inwidth
;				oldelement = jj mod inwidth
;				newgroup(newelement,newrecord) = tempgroup(oldelement,oldrecord)
;			endfor	
			jj = lindgen(nread*vdsizes(4))
			newgroup(jj) = tempgroup(jj)
			jj = 0
			tempgroup = 0
			for i = long(0), long(nread-1), long(1) do begin
				thisgroup = bytarr(vdsizes(4))
				thisgroup = newgroup(*,i)
				these_groups(i).TAI93_time	= double(thisgroup,0)
				these_groups(i).observe_time	= fix(thisgroup,8)
				these_groups(i).location(0)	= float(thisgroup,10)
				these_groups(i).location(1)	= float(thisgroup,14)
				these_groups(i).radiance	= float(thisgroup,18)
				these_groups(i).footprint	= float(thisgroup,22)
				these_groups(i).address		= long(thisgroup,26)
				these_groups(i).parent_address  = long(thisgroup,30)
				these_groups(i).child_address	= long(thisgroup,34)
				these_groups(i).child_count	= long(thisgroup,38)
				these_groups(i).approx_threshold= byte(thisgroup,42)
				these_groups(i).alert_flag	= byte(thisgroup,43)
				these_groups(i).cluster_index  	= byte(thisgroup,44)
				these_groups(i).density_index	= byte(thisgroup,45)
				these_groups(i).noise_index    	= byte(thisgroup,46)
				these_groups(i).glint_index   	= float(thisgroup,47)
				these_groups(i).oblong_index	= float(thisgroup,51)
				these_groups(i).grouping_sequence = long(thisgroup,55)
				these_groups(i).grouping_status   = byte(thisgroup,59)
			endfor
			hdf_vd_detach,vd
			hdf_close,hdffile
		endif else begin
			these_groups = replicate({a_group},1)
		endelse
	endelse
	return
end

pro READ_events,fname,these_events,status=status,native=native,_EXTRA=e
	this_os = !version.os
	call_nat = keyword_set(native)

	this_event = {an_event, TAI93_time:double(0), observe_time:fix(0), location:fltarr(2) ,radiance:float(0),       $
			footprint:float(0), address:long(0), parent_address:long(0), x_pixel:byte(0), y_pixel:byte(0),  $
			bg_value:fix(0), bg_radiance:fix(0), amplitude:byte(0), sza_index:byte(0), glint_index:byte(0), $
			approx_threshold:byte(0), alert_flag:byte(0), cluster_index:byte(0), density_index:                      $
			byte(0), noise_index:byte(0), bg_value_flag:byte(0), grouping_sequence:long(0)}

	if (call_nat eq 1) then begin
		READ_point_summary,fname,ps,status=status,_EXTRA=e
		nread = ps.event_count
		these_events = replicate({an_event},max([nread,1]))
		if (nread gt 0) then begin
			res=CALL_EXTERNAL(native,'glis_event',fname,these_events)
		endif
	endif else begin
		getvdsizes,fname,vdsizes
		hdffile = hdf_open(fname)
		if (hdffile eq long(0)) then begin
		   print,"Error opening file *"+fname+"*"
		   hdf_close,hdffile
		   stop
		endif
		vdref = hdf_vd_find(hdffile,"event")
		vdct = 0 
		vd = hdf_vd_attach(hdffile,vdref,/read)
		hdf_vd_get,vd,count=vdct
		; print,systime(),' Before hdf_vd_read'
		if (vdref gt 0 and vdct gt 0 and vd gt 0) then begin
			nread = hdf_vd_read(vd,tempevent)
			; print,systime(),' After hdf_vd_read'
			newevent = bytarr(vdsizes(5),nread)
			insize = size(tempevent)
			inwidth = insize(1)
			these_events = replicate({an_event},max([nread,1]))
			; Note - IDL reads things in on even byte boundaries, must repack the byte array to the correct HDF size
;			jj = long(0)
;			for jj = long(0),long(nread*vdsizes(5)-1),long(1) do begin
;				newrecord  = jj / vdsizes(5)
;				newelement = jj mod vdsizes(5)
;				oldrecord  = jj / inwidth
;				oldelement = jj mod inwidth
;				newevent(newelement,newrecord) = tempevent(oldelement,oldrecord)
;			endfor	
			; Wow.  Forgot IDL can reference arrays as vectors - this
			;  speeds up the read process by 2x over the approach above
			jj = lindgen(nread*vdsizes(5))
			newevent(jj) = tempevent(jj)
			jj = 0
			tempevent = 0
			; Pretty good :-)
			; print,systime(),' After repacking'
			for i = long(0), long(nread-1), long(1) do begin
				thisevent = bytarr(vdsizes(5))
				thisevent = newevent(*,i)
				these_events(i).TAI93_time	= double(thisevent,0)
				these_events(i).observe_time	= fix(thisevent,8)
				these_events(i).location(0)	= float(thisevent,10)
				these_events(i).location(1)	= float(thisevent,14)
				these_events(i).radiance	= float(thisevent,18)
				these_events(i).footprint	= float(thisevent,22)
				these_events(i).address		= long(thisevent,26)
				these_events(i).parent_address  = long(thisevent,30)
			 	these_events(i).x_pixel		= byte(thisevent,34)
			 	these_events(i).y_pixel		= byte(thisevent,35)
			 	these_events(i).bg_value	= fix(thisevent,36)
			 	these_events(i).bg_radiance	= fix(thisevent,38)
			 	these_events(i).amplitude	= byte(thisevent,40)
			 	these_events(i).sza_index	= byte(thisevent,41)
			 	these_events(i).glint_index	= byte(thisevent,42)
			 	these_events(i).approx_threshold= byte(thisevent,43)
				these_events(i).alert_flag	= byte(thisevent,44)
				these_events(i).cluster_index  	= byte(thisevent,45)
				these_events(i).density_index	= byte(thisevent,46)
				these_events(i).noise_index    	= byte(thisevent,47)
				these_events(i).bg_value_flag	= byte(thisevent,48)
				these_events(i).grouping_sequence = long(thisevent,49)
			endfor
			hdf_vd_detach,vd
			hdf_close,hdffile
			; print,systime(),' After extraction'
		endif else begin
			these_events = replicate({an_event},1)
		endelse
	endelse
	return
end

pro READ_viewtimes,fname,these_viewtimes,status=status,native=native,_EXTRA=e

	this_os = !version.os
	call_nat = keyword_set(native)

	this_viewtime = {a_viewtime, location:fltarr(2), TAI93_start:long(0), TAI93_end:long(0), effective_obs:float(0), $
				alert_flag:byte(0), approx_threshold:byte(0)}

	if (call_nat eq 1) then begin
		READ_point_summary,fname,ps,status=status,_EXTRA=e
		nread = ps.vt_count
		these_viewtimes = replicate({a_viewtime},max([nread,1]))
        	if (nread gt 0) then begin
			res=CALL_EXTERNAL(native,'glis_viewtime',fname,these_viewtimes)
		endif
	endif else begin
		getvtvdsize,fname,vdsizes
		hdffile = hdf_open(fname)
		if (hdffile eq long(0)) then begin
		   print,"Error opening file *"+fname+"*"
		   hdf_close,hdffile
		   stop
		endif
		vdref = hdf_vd_find(hdffile,"viewtime")
		vd = hdf_vd_attach(hdffile,vdref,/read)
		nread = hdf_vd_read(vd,tempviewtime)
		newviewtime = bytarr(vdsizes(0),nread)
		insize = size(tempviewtime)
		inwidth = insize(1)	
		these_viewtimes = replicate({a_viewtime},max([nread,1]))
		; Note - IDL reads things in on even byte boundaries, must repack the byte array to the correct HDF size
;		for jj = long(0),long(nread*vdsizes(0)-1),long(1) do begin
;			newrecord  = jj / vdsizes(0)
;			newelement = jj mod vdsizes(0)
;			oldrecord  = jj / inwidth
;			oldelement = jj mod inwidth
;			newviewtime(newelement,newrecord) = tempviewtime(oldelement,oldrecord)
;		endfor	
		jj = lindgen(nread*vdsizes(0))
		newviewtime(jj) = tempviewtime(jj)
		jj = 0
		tempviewtime = 0
		for i = long(0), long(nread-1), long(1) do begin
			thisviewtime = bytarr(vdsizes(0))
			thisviewtime = newviewtime(*,i)
			these_viewtimes(i).location(0)	= float(thisviewtime,0)
			these_viewtimes(i).location(1)	= float(thisviewtime,4)
			these_viewtimes(i).TAI93_start	= long(thisviewtime,8)
			these_viewtimes(i).TAI93_end    = long(thisviewtime,12)
			these_viewtimes(i).effective_obs= float(thisviewtime,16)
			these_viewtimes(i).alert_flag   = byte(thisviewtime,20)
			these_viewtimes(i).approx_threshold = byte(thisviewtime,21)
		endfor
		hdf_vd_detach,vd
		hdf_close,hdffile
	endelse
	return
end

pro READ_bg_summaries,fname,these_bg_summaries,status=status,native=native,_EXTRA=e
	this_os = !version.os
	call_nat = keyword_set(native)

	this_bg_summary = {a_bg_summary, TAI93_time:double(0),address:long(0),boresight:fltarr(2), corners:fltarr(8)}

	if (call_nat eq 1) then begin
		READ_point_summary,fname,ps,status=status,_EXTRA=e
		nread = ps.bg_count
		these_bg_summaries = replicate({a_bg_summary},max([nread,1]))
        	res=CALL_EXTERNAL(native,'glis_bg_summaries',fname,these_bg_summaries)
	endif else begin
		; Read in BG Summaries
		getvdsizes,fname,vdsizes
		hdffile = hdf_open(fname)
		if (hdffile eq long(0)) then begin
		   print,"Error opening file *"+fname+"*"
		   hdf_close,hdffile
		   stop
		endif
		vdref = hdf_vd_find(hdffile,"bg_summary")
		vd = hdf_vd_attach(hdffile,vdref,/read)
		nread = hdf_vd_read(vd,tempbg_summary)
		newbg_summary = bytarr(vdsizes(7),nread)
		insize = size(tempbg_summary)
		inwidth = insize(1)
		these_bg_summaries = replicate({a_bg_summary},max([nread,1]))
		; Note - IDL reads things in on even byte boundaries, must repack the byte array to the correct HDF size
;		for jj = 0,nread*vdsizes(7)-1,1 do begin
;			newrecord  = jj / vdsizes(7)
;			newelement = jj mod vdsizes(7)
;			oldrecord  = jj / inwidth
;			oldelement = jj mod inwidth
;			newbg_summary(newelement,newrecord) = tempbg_summary(oldelement,oldrecord)
;		endfor
		jj = lindgen(nread*vdsizes(7))
		newbg_summary(jj) = tempbg_summary(jj)
		jj = 0
		tempbg_summary = 0
		for i = 0, nread-1, 1 do begin
			thisbg_summary = bytarr(vdsizes(7))
			thisbg_summary = newbg_summary(*,i)
			these_bg_summaries(i).TAI93_time 	= double(thisbg_summary,0);
			these_bg_summaries(i).address	 	= long(thisbg_summary,8);
			these_bg_summaries(i).boresight(0)	= float(thisbg_summary,12);
			these_bg_summaries(i).boresight(1) 	= float(thisbg_summary,16);
			these_bg_summaries(i).corners(0)	= float(thisbg_summary,20);
			these_bg_summaries(i).corners(1)	= float(thisbg_summary,24);
			these_bg_summaries(i).corners(2)	= float(thisbg_summary,28);
			these_bg_summaries(i).corners(3)	= float(thisbg_summary,32);
			these_bg_summaries(i).corners(4)	= float(thisbg_summary,36);
			these_bg_summaries(i).corners(5)	= float(thisbg_summary,40);
			these_bg_summaries(i).corners(6)	= float(thisbg_summary,44);
			these_bg_summaries(i).corners(7)	= float(thisbg_summary,48);
		endfor
		hdf_vd_detach,vd
		hdf_close,hdffile
	endelse
	return
end


pro summarize_hdf, fname
	hdffile = hdf_open(fname)
	if (hdffile eq long(0)) then begin
	   print,"Error opening file *"+fname+"*"
	   hdf_close,hdffile
	   stop
	endif
	
	vgid = hdf_vg_getid(hdffile,-1)
	vg   = hdf_vg_attach(hdffile,vgid,/read)
	hdf_vg_getinfo,vg,class=c,name=nm,nentries=n
	print,'VGROUP'
	print,"Class   =",c
	print,"Name    =",nm
	print,"Entries =",n
	hdf_vg_gettrs,vg,vgtags,vgrefs
	print,"Tags    =",vgtags
	print,"Refs    =",vgrefs
	print,' '
	hdf_vg_detach,vg
	
	oldvgid = vgid
	vgid = hdf_vg_getid(hdffile,oldvgid)
	vg   = hdf_vg_attach(hdffile,vgid,/read)
	hdf_vg_getinfo,vg,class=c,name=nm,nentries=n
	print,'VGROUP'
	print,"Class   =",c
	print,"Name    =",nm
	print,"Entries =",n
	hdf_vg_gettrs,vg,vgtags,vgrefs
	print,"Tags    =",vgtags
	print,"Refs    =",vgrefs
	print,' '
	hdf_vg_detach,vg
	
	oldvgid = vgid
	vgid = hdf_vg_getid(hdffile,oldvgid)
	vg   = hdf_vg_attach(hdffile,vgid,/read)
	hdf_vg_getinfo,vg,class=c,name=nm,nentries=n
	print,'VGROUP'
	print,"Class   =",c
	print,"Name    =",nm
	print,"Entries =",n
	hdf_vg_gettrs,vg,vgtags,vgrefs
	print,"Tags    =",vgtags
	print,"Refs    =",vgrefs
	print,' '
	hdf_vg_detach,vg
	
	vdname = ["orbit_summary","point_summary","area","flash","group","event","viewtime","bg_summary","one_second"]
	vdsizes = intarr(9)
	for vdn=0,8,1 do begin
		vdref = hdf_vd_find(hdffile,vdname(vdn))
		vd = hdf_vd_attach(hdffile,vdref,/read)
		hdf_vd_get,vd,class=cl,count=ct,fields=flds,interlace=ilc,name=nm,nfields=nflds,ref=rf,size=sz,tag=tg
		print,'VDATA'
		print,format='("CLASS = ",A20,"  NAME = ",A20)',c,nm
		print,"Fields  = ",nflds
		print,"Size    = ",sz
		print,format='("TagRef  = ",I4,"/",I2)',tg,rf
		for i=0,nflds-1 do begin
		   hdf_vd_getinfo,vd,i,name=n,type=t,order=o
		   print,format='("Name = ",A32," Type = ",A6," Order = ",I2)',n,t,o
		endfor
		print,' '
		vdsizes(vdn) = fix(sz);
		hdf_vd_detach,vd
	endfor
	hdf_close,hdffile
end

; ***********************************************************************************************
; *   				LISOTD Write Library routines; 10/28/97 - DJB			*
; ***********************************************************************************************

pro WRITE_orbit, fname, orbit

	; The routines below are _not_ standalone - they depend on vgroup IDs returned
	;    by WRITE_skeleton.  They are modular simply for programming and debugging
	;    convenience.
	     
	     						print,' '
	    						print,'Writing HDF file ',fname
	WRITE_create_file,	fname,orbit,hdffile,ovg,pvg,lvg
							print,'Writing ORBIT_SUMMARY'
	WRITE_orbit_summary,	fname,orbit,hdffile,ovg,pvg,lvg
							print,'Writing ONE_SECONDs'
	WRITE_one_second,	fname,orbit,hdffile,ovg,pvg,lvg
							print,'   Writing POINT_SUMMARY'
	WRITE_point_summary,	fname,orbit,hdffile,ovg,pvg,lvg
							print,'      Writing AREAs'
	WRITE_areas,		fname,orbit,hdffile,ovg,pvg,lvg
							print,'      Writing FLASHes'
	WRITE_flashes,		fname,orbit,hdffile,ovg,pvg,lvg
							print,'      Writing GROUPs'
	WRITE_groups,		fname,orbit,hdffile,ovg,pvg,lvg
							print,'      Writing EVENTs'
	WRITE_events,		fname,orbit,hdffile,ovg,pvg,lvg
							print,'   Writing VIEWTIMEs'
	WRITE_viewtimes,	fname,orbit,hdffile,ovg,pvg,lvg
							print,'   Writing BG_SUMMARYs'
	WRITE_bg_summaries,	fname,orbit,hdffile,ovg,pvg,lvg
							print,'Closing out the file.'
	WRITE_close_file,	fname,orbit,hdffile,ovg,pvg,lvg
							print,'Done Writing ',fname
							print,' '
							
	return
end

pro WRITE_create_file,fname,orbit,hdffile,ovg,pvg,lvg

	hdffile = hdf_open(fname,/create,/write)

	ovg = hdf_vg_attach(hdffile,-1,/write)
	hdf_vg_setinfo, ovg, class='data', name='orbit'
	
	pvg = hdf_vg_attach(hdffile,-1,/write)
	hdf_vg_setinfo, pvg, class='data', name='point'
	
	hdf_vg_insert, ovg, pvg
	
	lvg = hdf_vg_attach(hdffile,-1,/write)
	hdf_vg_setinfo, lvg, class='data', name='lightning'
	
	hdf_vg_insert, pvg, lvg

	; Leave the vgroups attached, once we close them we can no longer
	;    insert vdatas...
	
	return
end
	
pro WRITE_orbit_summary,fname,orbit,hdffile,ovg,pvg,lvg

	; Insert orbit summary vdata into orbit vgroup
		
	vd = hdf_vd_attach(hdffile,-1,/write)
	hdf_vd_setinfo, vd, class='data', name='orbit_summary'	
	hdf_vd_fdefine,vd,'id_number',/long,order=1
	hdf_vd_fdefine,vd,'TAI93_start',/double,order=1
	hdf_vd_fdefine,vd,'UTC_start',/byte,order=28
	hdf_vd_fdefine,vd,'GPS_start',/double,order=1
	hdf_vd_fdefine,vd,'TAI93_end',/double,order=1
	hdf_vd_fdefine,vd,'start_longitude',/float,order=1
	hdf_vd_fdefine,vd,'end_longitude',/float,order=1
	hdf_vd_fdefine,vd,'point_data_count',/int,order=1
	hdf_vd_fdefine,vd,'point_data_address',/int,order=1
	hdf_vd_fdefine,vd,'one_second_count',/long,order=1
	hdf_vd_fdefine,vd,'one_second_address',/long,order=1
	hdf_vd_fdefine,vd,'summary_image_count',/int,order=1
	hdf_vd_fdefine,vd,'summary_image_address',/int,order=1
	hdf_vd_fdefine,vd,'inspection_code',/int,order=1
	hdf_vd_fdefine,vd,'configuration_code',/int,order=1
	out = bytarr(84)
	out( 0: 3) = byte(orbit.orbit_summary.id_number,		0,4)
	out( 4:11) = byte(orbit.orbit_summary.TAI93_start,		0,8)
	out(12:39) = orbit.orbit_summary.UTC_start(0:27)
	out(40:47) = byte(orbit.orbit_summary.GPS_start,		0,8)
	out(48:55) = byte(orbit.orbit_summary.TAI93_end,		0,8)
	out(56:59) = byte(orbit.orbit_summary.start_longitude,		0,4)
	out(60:63) = byte(orbit.orbit_summary.end_longitude,		0,4)
	out(64:65) = byte(orbit.orbit_summary.point_data_count,		0,2)
	out(66:67) = byte(orbit.orbit_summary.point_data_address,	0,2)
	out(68:71) = byte(orbit.orbit_summary.one_second_count,		0,4)
	out(72:75) = byte(orbit.orbit_summary.one_second_address,	0,4)
	out(76:77) = byte(orbit.orbit_summary.summary_image_count,	0,2)
	out(78:79) = byte(orbit.orbit_summary.summary_image_address,	0,2)
	out(80:81) = byte(orbit.orbit_summary.inspection_code,		0,2)
	out(82:83) = byte(orbit.orbit_summary.configuration_code,	0,2)
	hdf_vd_write,vd, $
	   'id_number,TAI93_start,UTC_start,GPS_start,TAI93_end,start_longitude,end_longitude,point_data_count,'+$
	   'point_data_address,one_second_count,one_second_address,summary_image_count,summary_image_address,'+$
	   'inspection_code,configuration_code', $
	   out
	hdf_vg_insert,ovg,vd
	hdf_vd_detach,vd
		
	return
end

pro WRITE_one_second,fname,orbit,hdffile,ovg,pvg,lvg

	; Insert one_second vdata into orbit vgroup

	vd = hdf_vd_attach(hdffile,-1,/write)
	hdf_vd_setinfo, vd, class='data', name='one_second'
	hdf_vd_fdefine,vd,'TAI93_time',/double,order=1
	hdf_vd_fdefine,vd,'alert_summary',/byte,order=1
	hdf_vd_fdefine,vd,'instrument_alert',/byte,order=1
	hdf_vd_fdefine,vd,'platform_alert',/byte,order=1
	hdf_vd_fdefine,vd,'external_alert',/byte,order=1
	hdf_vd_fdefine,vd,'processing_alert',/byte,order=1
	hdf_vd_fdefine,vd,'position_vector',/float,order=3
	hdf_vd_fdefine,vd,'velocity_vector',/float,order=3
	hdf_vd_fdefine,vd,'transform_matrix',/float,order=9
	hdf_vd_fdefine,vd,'solar_vector',/float,order=3
	hdf_vd_fdefine,vd,'ephemeris_quality_flag',/long,order=1
	hdf_vd_fdefine,vd,'attitude_quality_flag',/long,order=1
	hdf_vd_fdefine,vd,'boresight_threshold',/byte,order=1
	hdf_vd_fdefine,vd,'thresholds',/byte,order=12
	hdf_vd_fdefine,vd,'noise_index',/byte,order=1
	hdf_vd_fdefine,vd,'event_count',/int,order=6
	out = bytarr(119, orbit.orbit_summary.one_second_count)
	for i = long(0), long(orbit.orbit_summary.one_second_count-1), long(1) do begin
		out( 0: 7,i) = byte(orbit.one_second(i).TAI93_time,			0,8)
		out(    8,i) = byte(orbit.one_second(i).alert_summary,			0,1)
		out(    9,i) = byte(orbit.one_second(i).instrument_alert,		0,1)
		out(   10,i) = byte(orbit.one_second(i).platform_alert,			0,1)
		out(   11,i) = byte(orbit.one_second(i).external_alert,			0,1)
		out(   12,i) = byte(orbit.one_second(i).processing_alert,		0,1)
		out(13:16,i) = byte(orbit.one_second(i).position_vector(0),		0,4)
		out(17:20,i) = byte(orbit.one_second(i).position_vector(1),		0,4)
		out(21:24,i) = byte(orbit.one_second(i).position_vector(2),		0,4)
		out(25:28,i) = byte(orbit.one_second(i).velocity_vector(0),		0,4)
		out(29:32,i) = byte(orbit.one_second(i).velocity_vector(1),		0,4)
		out(33:36,i) = byte(orbit.one_second(i).velocity_vector(2),		0,4)
		for j = 0, 8, 1 do begin
	           out(37+(j*4):40+(j*4),i) = byte(orbit.one_second(i).transform_matrix(j),0,4)
		endfor
		out(73:76,i) = byte(orbit.one_second(i).solar_vector(0),		0,4)
		out(77:80,i) = byte(orbit.one_second(i).solar_vector(1),		0,4)
		out(81:84,i) = byte(orbit.one_second(i).solar_vector(2),		0,4)
		out(85:88,i) = byte(orbit.one_second(i).ephemeris_quality_flag,		0,4)
		out(89:92,i) = byte(orbit.one_second(i).attitude_quality_flag,		0,4)
		out(   93,i) = byte(orbit.one_second(i).boresight_threshold,		0,1)
		for j = 0, 11, 1 do begin
			out(94+j,i) = byte(orbit.one_second(i).thresholds(j),		0,1)
		endfor
		out(  106,i) = byte(orbit.one_second(i).noise_index,			0,1)
		for j = 0, 5, 1 do begin
		   out(107+(j*2):108+(j*2),i) = byte(orbit.one_second(i).event_count(j),0,2)
		endfor
	endfor
	hdf_vd_write,vd, $
	   'TAI93_time,alert_summary,instrument_alert,platform_alert,external_alert,processing_alert,position_vector,velocity_vector,'+$
	   'transform_matrix,solar_vector,ephemeris_quality_flag,attitude_quality_flag,boresight_threshold,thresholds,'+$
	   'noise_index,event_count', $
	   out, nrec=orbit.orbit_summary.one_second_count
	hdf_vg_insert,ovg,vd
	hdf_vd_detach,vd
		
	return
end

pro WRITE_point_summary,fname,orbit,hdffile,ovg,pvg,lvg

	; Insert point_summary into point vgroup.  
	;   Defer insertion of viewtime and bg_summary vdatas till after lightning data in order
	;   to match Johnny's insertion flow.
		
	vd = hdf_vd_attach(hdffile,-1,/write)
	hdf_vd_setinfo, vd, class='data', name='point_summary'
	hdf_vd_fdefine,vd,'parent_address',	/long,order=1
	hdf_vd_fdefine,vd,'event_count',	/long,order=1
	hdf_vd_fdefine,vd,'event_address',	/long,order=1
	hdf_vd_fdefine,vd,'group_count',	/long,order=1
	hdf_vd_fdefine,vd,'group_address',	/long,order=1
	hdf_vd_fdefine,vd,'flash_count',	/long,order=1
	hdf_vd_fdefine,vd,'flash_address',	/long,order=1
	hdf_vd_fdefine,vd,'area_count',		/long,order=1
	hdf_vd_fdefine,vd,'area_address',	/long,order=1
	hdf_vd_fdefine,vd,'bg_count',		/long,order=1
	hdf_vd_fdefine,vd,'bg_address',		/long,order=1
	hdf_vd_fdefine,vd,'vt_count',		/long,order=1
	hdf_vd_fdefine,vd,'vt_address',		/long,order=1
	nout = lonarr(13,1)
	nout(0) = orbit.point.point_summary.parent_address
	nout(1) = orbit.point.point_summary.event_count
	nout(2) = orbit.point.point_summary.event_address
	nout(3) = orbit.point.point_summary.group_count
	nout(4) = orbit.point.point_summary.group_address
	nout(5) = orbit.point.point_summary.flash_count
	nout(6) = orbit.point.point_summary.flash_address
	nout(7) = orbit.point.point_summary.area_count
	nout(8) = orbit.point.point_summary.area_address
	nout(9) = orbit.point.point_summary.bg_count
	nout(10)= orbit.point.point_summary.bg_address
	nout(11)= orbit.point.point_summary.vt_count
	nout(12)= orbit.point.point_summary.vt_address
	hdf_vd_write,vd, $
	   'parent_address, event_count, event_address, group_count, group_address, flash_count, flash_address, area_count, area_address, bg_count, bg_address, vt_count, vt_address', $
	   nout
	hdf_vg_insert,pvg,vd
	hdf_vd_detach,vd
	 	
	return
end

pro WRITE_areas,fname,orbit,hdffile,ovg,pvg,lvg

	; Insert area vdata into lightning vgroup
	
	vd = hdf_vd_attach(hdffile,-1,/write)
	hdf_vd_setinfo,vd,class='data',name='area'
	hdf_vd_fdefine,vd,'TAI93_time',/double,order=1
	hdf_vd_fdefine,vd,'delta_time',/float,order=1
	hdf_vd_fdefine,vd,'observe_time',/int,order=1
	hdf_vd_fdefine,vd,'location',/float,order=2
	hdf_vd_fdefine,vd,'net_radiance',/float,order=1
	hdf_vd_fdefine,vd,'footprint',/float,order=1
	hdf_vd_fdefine,vd,'address',/long,order=1
	hdf_vd_fdefine,vd,'parent_address',/long,order=1
	hdf_vd_fdefine,vd,'child_address',/long,order=1
	hdf_vd_fdefine,vd,'child_count',/long,order=1
	hdf_vd_fdefine,vd,'grandchild_count',/long,order=1
	hdf_vd_fdefine,vd,'greatgrandchild_count',/long,order=1
	hdf_vd_fdefine,vd,'approx_threshold',/byte,order=1
	hdf_vd_fdefine,vd,'alert_flag',/byte,order=1
	hdf_vd_fdefine,vd,'cluster_index',/byte,order=1
	hdf_vd_fdefine,vd,'density_index',/byte,order=1
	hdf_vd_fdefine,vd,'noise_index',/byte,order=1
	hdf_vd_fdefine,vd,'oblong_index',/float,order=1
	hdf_vd_fdefine,vd,'grouping_sequence',/long,order=1
	hdf_vd_fdefine,vd,'grouping_status',/byte,order=1
	out = bytarr(68,orbit.point.point_summary.area_count)
	for i = long(0), long(orbit.point.point_summary.area_count-1), long(1) do begin
		out( 0: 7,i) = byte(orbit.point.lightning.area(i).TAI93_time,		0,8)
		out( 8:11,i) = byte(orbit.point.lightning.area(i).delta_time,		0,4)
		out(12:13,i) = byte(orbit.point.lightning.area(i).observe_time,		0,2)
		out(14:17,i) = byte(orbit.point.lightning.area(i).location(0),		0,4)
		out(18:21,i) = byte(orbit.point.lightning.area(i).location(1),		0,4)
		out(22:25,i) = byte(orbit.point.lightning.area(i).net_radiance,		0,4)
		out(26:29,i) = byte(orbit.point.lightning.area(i).footprint,		0,4)
		out(30:33,i) = byte(orbit.point.lightning.area(i).address,		0,4)
		out(34:37,i) = byte(orbit.point.lightning.area(i).parent_address,	0,4)
		out(38:41,i) = byte(orbit.point.lightning.area(i).child_address,	0,4)
		out(42:45,i) = byte(orbit.point.lightning.area(i).child_count,		0,4)
		out(46:49,i) = byte(orbit.point.lightning.area(i).grandchild_count,	0,4)
		out(50:53,i) = byte(orbit.point.lightning.area(i).greatgrandchild_count,0,4)
		out(   54,i) = byte(orbit.point.lightning.area(i).approx_threshold,	0,1)
		out(   55,i) = byte(orbit.point.lightning.area(i).alert_flag,		0,1)
		out(   56,i) = byte(orbit.point.lightning.area(i).cluster_index,	0,1)
		out(   57,i) = byte(orbit.point.lightning.area(i).density_index,	0,1)
		out(   58,i) = byte(orbit.point.lightning.area(i).noise_index,		0,1)
		out(59:62,i) = byte(orbit.point.lightning.area(i).oblong_index,		0,4)
		out(63:66,i) = byte(orbit.point.lightning.area(i).grouping_sequence,	0,4)
		out(   67,i) = byte(orbit.point.lightning.area(i).grouping_status,	0,1)
	endfor	
	fldnms1 = 'TAI93_time,delta_time,observe_time,location,net_radiance,footprint,address,parent_address,child_address,child_count,grandchild_count,greatgrandchild_count,'
	fldnms2 = 'approx_threshold,alert_flag,cluster_index,density_index,noise_index,oblong_index,grouping_sequence,grouping_status'
	fldnms  = fldnms1+fldnms2	
	hdf_vd_write,vd, fldnms, out, nrec = orbit.point.point_summary.area_count
	hdf_vg_insert, lvg, vd
	hdf_vd_detach,vd	   
		
	return
end

pro WRITE_flashes,fname,orbit,hdffile,ovg,pvg,lvg

	; Insert flash vdata into lightning vgroup 
	
	vd = hdf_vd_attach(hdffile,-1,/write)
	hdf_vd_setinfo,vd,class='data',name='flash'
	hdf_vd_fdefine,vd,'TAI93_time',/double,order=1
	hdf_vd_fdefine,vd,'delta_time',/float,order=1
	hdf_vd_fdefine,vd,'observe_time',/int,order=1
	hdf_vd_fdefine,vd,'location',/float,order=2
	hdf_vd_fdefine,vd,'radiance',/float,order=1
	hdf_vd_fdefine,vd,'footprint',/float,order=1
	hdf_vd_fdefine,vd,'address',/long,order=1
	hdf_vd_fdefine,vd,'parent_address',/long,order=1
	hdf_vd_fdefine,vd,'child_address',/long,order=1
	hdf_vd_fdefine,vd,'child_count',/long,order=1
	hdf_vd_fdefine,vd,'grandchild_count',/long,order=1
	hdf_vd_fdefine,vd,'approx_threshold',/byte,order=1
	hdf_vd_fdefine,vd,'alert_flag',/byte,order=1
	hdf_vd_fdefine,vd,'cluster_index',/byte,order=1
	hdf_vd_fdefine,vd,'density_index',/byte,order=1
	hdf_vd_fdefine,vd,'noise_index',/byte,order=1
	hdf_vd_fdefine,vd,'glint_index',/float,order=1
	hdf_vd_fdefine,vd,'oblong_index',/float,order=1
	hdf_vd_fdefine,vd,'grouping_sequence',/long,order=1
	hdf_vd_fdefine,vd,'grouping_status',/byte,order=1
	out = bytarr(68,orbit.point.point_summary.flash_count)
	for i = long(0), long(orbit.point.point_summary.flash_count-1), long(1) do begin
		out( 0: 7,i) = byte(orbit.point.lightning.flash(i).TAI93_time,		0,8)
		out( 8:11,i) = byte(orbit.point.lightning.flash(i).delta_time,		0,4)
		out(12:13,i) = byte(orbit.point.lightning.flash(i).observe_time,	0,2)
		out(14:17,i) = byte(orbit.point.lightning.flash(i).location(0),		0,4)
		out(18:21,i) = byte(orbit.point.lightning.flash(i).location(1),		0,4)
		out(22:25,i) = byte(orbit.point.lightning.flash(i).radiance,		0,4)
		out(26:29,i) = byte(orbit.point.lightning.flash(i).footprint,		0,4)
		out(30:33,i) = byte(orbit.point.lightning.flash(i).address,		0,4)
		out(34:37,i) = byte(orbit.point.lightning.flash(i).parent_address,	0,4)
		out(38:41,i) = byte(orbit.point.lightning.flash(i).child_address,	0,4)
		out(42:45,i) = byte(orbit.point.lightning.flash(i).child_count,		0,4)
		out(46:49,i) = byte(orbit.point.lightning.flash(i).grandchild_count,	0,4)
		out(   50,i) = byte(orbit.point.lightning.flash(i).approx_threshold,	0,1)
		out(   51,i) = byte(orbit.point.lightning.flash(i).alert_flag,		0,1)
		out(   52,i) = byte(orbit.point.lightning.flash(i).cluster_index,	0,1)
		out(   53,i) = byte(orbit.point.lightning.flash(i).density_index,	0,1)
		out(   54,i) = byte(orbit.point.lightning.flash(i).noise_index,		0,1)
		out(55:58,i) = byte(orbit.point.lightning.flash(i).glint_index,		0,4)
		out(59:62,i) = byte(orbit.point.lightning.flash(i).oblong_index,	0,4)
		out(63:66,i) = byte(orbit.point.lightning.flash(i).grouping_sequence,	0,4)
		out(   67,i) = byte(orbit.point.lightning.flash(i).grouping_status,	0,1)
	endfor
	fldnms1 = 'TAI93_time,delta_time,observe_time,location,radiance,footprint,address,parent_address,child_address,child_count,grandchild_count,'
	fldnms2 = 'approx_threshold,alert_flag,cluster_index,density_index,noise_index,glint_index,oblong_index,grouping_sequence,grouping_status'
	fldnms = fldnms1+fldnms2
	hdf_vd_write,vd, fldnms, out, nrec = orbit.point.point_summary.flash_count
	hdf_vg_insert, lvg, vd
	hdf_vd_detach,vd	   
	
	return
end

pro WRITE_groups,fname,orbit,hdffile,ovg,pvg,lvg

	; Insert group vdata into lightning group 

	vd = hdf_vd_attach(hdffile,-1,/write)
	hdf_vd_setinfo,vd,class='data',name='group'
	hdf_vd_fdefine,vd,'TAI93_time',/double,order=1
	hdf_vd_fdefine,vd,'observe_time',/int,order=1
	hdf_vd_fdefine,vd,'location',/float,order=2
	hdf_vd_fdefine,vd,'radiance',/float,order=1
	hdf_vd_fdefine,vd,'footprint',/float,order=1
	hdf_vd_fdefine,vd,'address',/long,order=1
	hdf_vd_fdefine,vd,'parent_address',/long,order=1
	hdf_vd_fdefine,vd,'child_address',/long,order=1
	hdf_vd_fdefine,vd,'child_count',/long,order=1
	hdf_vd_fdefine,vd,'approx_threshold',/byte,order=1
	hdf_vd_fdefine,vd,'alert_flag',/byte,order=1
	hdf_vd_fdefine,vd,'cluster_index',/byte,order=1
	hdf_vd_fdefine,vd,'density_index',/byte,order=1
	hdf_vd_fdefine,vd,'noise_index',/byte,order=1
	hdf_vd_fdefine,vd,'glint_index',/float,order=1
	hdf_vd_fdefine,vd,'oblong_index',/float,order=1
	hdf_vd_fdefine,vd,'grouping_sequence',/long,order=1
	hdf_vd_fdefine,vd,'grouping_status',/byte,order=1
	out = bytarr(60,orbit.point.point_summary.group_count)
	for i = long(0), long(orbit.point.point_summary.group_count-1), long(1) do begin
		out( 0: 7,i) = byte(orbit.point.lightning.group(i).TAI93_time,		0,8)
		out( 8: 9,i) = byte(orbit.point.lightning.group(i).observe_time,	0,2)
		out(10:13,i) = byte(orbit.point.lightning.group(i).location(0),		0,4)
		out(14:17,i) = byte(orbit.point.lightning.group(i).location(1),		0,4)
		out(18:21,i) = byte(orbit.point.lightning.group(i).radiance,		0,4)
		out(22:25,i) = byte(orbit.point.lightning.group(i).footprint,		0,4)
		out(26:29,i) = byte(orbit.point.lightning.group(i).address,		0,4)
		out(30:33,i) = byte(orbit.point.lightning.group(i).parent_address,	0,4)
		out(34:37,i) = byte(orbit.point.lightning.group(i).child_address,	0,4)
		out(38:41,i) = byte(orbit.point.lightning.group(i).child_count,		0,4)
		out(   42,i) = byte(orbit.point.lightning.group(i).approx_threshold,	0,1)
		out(   43,i) = byte(orbit.point.lightning.group(i).alert_flag,		0,1)
		out(   44,i) = byte(orbit.point.lightning.group(i).cluster_index,	0,1)
		out(   45,i) = byte(orbit.point.lightning.group(i).density_index,	0,1)
		out(   46,i) = byte(orbit.point.lightning.group(i).noise_index,		0,1)
		out(47:50,i) = byte(orbit.point.lightning.group(i).glint_index,		0,4)
		out(51:54,i) = byte(orbit.point.lightning.group(i).oblong_index,	0,4)
		out(55:58,i) = byte(orbit.point.lightning.group(i).grouping_sequence,	0,4)
		out(   59,i) = byte(orbit.point.lightning.group(i).grouping_status,	0,1)
	endfor
	fldnms1 = 'TAI93_time,observe_time,location,radiance,footprint,address,parent_address,child_address,child_count,approx_threshold,alert_flag,cluster_index,'
	fldnms2 = 'density_index,noise_index,glint_index,oblong_index,grouping_sequence,grouping_status'
	fldnms = fldnms1+fldnms2
	hdf_vd_write,vd, fldnms, out, nrec = orbit.point.point_summary.group_count
	hdf_vg_insert, lvg, vd
	hdf_vd_detach,vd
		   
	return
end

pro WRITE_events,fname,orbit,hdffile,ovg,pvg,lvg

	; Insert event vdata into lightning vgroup	

	vd = hdf_vd_attach(hdffile,-1,/write)
	hdf_vd_setinfo,vd,class='data',name='event'
	hdf_vd_fdefine,vd,'TAI93_time',/DOUBLE,ORDER=1
	hdf_vd_fdefine,vd,'observe_time',/INT,ORDER=1
	hdf_vd_fdefine,vd,'location',/FLOAT,ORDER=2
	hdf_vd_fdefine,vd,'radiance',/FLOAT,ORDER=1
	hdf_vd_fdefine,vd,'footprint',/FLOAT,ORDER=1
	hdf_vd_fdefine,vd,'address',/LONG,ORDER=1
	hdf_vd_fdefine,vd,'parent_address',/LONG,ORDER=1
	hdf_vd_fdefine,vd,'x_pixel',/BYTE,ORDER=1
	hdf_vd_fdefine,vd,'y_pixel',/BYTE,ORDER=1
	hdf_vd_fdefine,vd,'bg_value',/INT,ORDER=1
	hdf_vd_fdefine,vd,'bg_radiance',/INT,ORDER=1
	hdf_vd_fdefine,vd,'amplitude',/BYTE,ORDER=1
	hdf_vd_fdefine,vd,'sza_index',/BYTE,ORDER=1
	hdf_vd_fdefine,vd,'glint_index',/BYTE,ORDER=1
	hdf_vd_fdefine,vd,'approx_threshold',/BYTE,ORDER=1
	hdf_vd_fdefine,vd,'alert_flag',/BYTE,ORDER=1
	hdf_vd_fdefine,vd,'cluster_index',/BYTE,ORDER=1
	hdf_vd_fdefine,vd,'density_index',/BYTE,ORDER=1
	hdf_vd_fdefine,vd,'noise_index',/BYTE,ORDER=1
	hdf_vd_fdefine,vd,'bg_value_flag',/BYTE,ORDER=1
	hdf_vd_fdefine,vd,'grouping_sequence',/LONG,ORDER=1	
	out = bytarr(53,orbit.point.point_summary.event_count)
	for i = long(0), long(orbit.point.point_summary.event_count-1), long(1) do begin	
		out( 0: 7,i) = byte(orbit.point.lightning.event(i).TAI93_time,		0,8)
		out( 8: 9,i) = byte(orbit.point.lightning.event(i).observe_time,	0,2)
		out(10:13,i) = byte(orbit.point.lightning.event(i).location(0),		0,4)
		out(14:17,i) = byte(orbit.point.lightning.event(i).location(1),		0,4)
		out(18:21,i) = byte(orbit.point.lightning.event(i).radiance,		0,4)
		out(22:25,i) = byte(orbit.point.lightning.event(i).footprint,		0,4)
		out(26:29,i) = byte(orbit.point.lightning.event(i).address,		0,4)
		out(30:33,i) = byte(orbit.point.lightning.event(i).parent_address,	0,4)
		out(   34,i) = byte(orbit.point.lightning.event(i).x_pixel,		0,1)
		out(   35,i) = byte(orbit.point.lightning.event(i).y_pixel,		0,1)
		out(36:37,i) = byte(orbit.point.lightning.event(i).bg_value,		0,2)
		out(38:39,i) = byte(orbit.point.lightning.event(i).bg_radiance,		0,2)
		out(   40,i) = byte(orbit.point.lightning.event(i).amplitude,		0,1)
		out(   41,i) = byte(orbit.point.lightning.event(i).sza_index,		0,1)
		out(   42,i) = byte(orbit.point.lightning.event(i).glint_index,		0,1)
		out(   43,i) = byte(orbit.point.lightning.event(i).approx_threshold,	0,1)
		out(   44,i) = byte(orbit.point.lightning.event(i).alert_flag,		0,1)
		out(   45,i) = byte(orbit.point.lightning.event(i).cluster_index,	0,1)
		out(   46,i) = byte(orbit.point.lightning.event(i).density_index,	0,1)
		out(   47,i) = byte(orbit.point.lightning.event(i).noise_index,		0,1)
		out(   48,i) = byte(orbit.point.lightning.event(i).bg_value_flag,	0,1)
		out(49:52,i) = byte(orbit.point.lightning.event(i).grouping_sequence,	0,4)
	endfor 		
	fldnms1 = 'TAI93_time,observe_time,location,radiance,footprint,address,parent_address,x_pixel,y_pixel,bg_value,bg_radiance,amplitude,'
	fldnms2 = 'sza_index,glint_index,approx_threshold,alert_flag,cluster_index,density_index,noise_index,bg_value_flag,grouping_sequence'
	fldnms = fldnms1+fldnms2
	hdf_vd_write,vd, fldnms, out,nrec=orbit.point.point_summary.event_count
	hdf_vg_insert, lvg, vd
	hdf_vd_detach,vd
	
	return
end

pro WRITE_viewtimes,fname,orbit,hdffile,ovg,pvg,lvg

	; Finally insert viewtime vdata into point vgroup
	
	vd = hdf_vd_attach(hdffile,-1,/write)
	hdf_vd_setinfo, vd, class='data', name='viewtime'
	hdf_vd_fdefine,vd,'location',/float,order=2
	hdf_vd_fdefine,vd,'TAI93_start',/long,order=1
	hdf_vd_fdefine,vd,'TAI93_end',/long,order=1
	hdf_vd_fdefine,vd,'effective_obs',/float,order=1
	hdf_vd_fdefine,vd,'alert_flag',/byte,order=1
	hdf_vd_fdefine,vd,'approx_threshold',/byte,order=1
	out = bytarr(22, orbit.point.point_summary.vt_count)
	for i = long(0), long(orbit.point.point_summary.vt_count-1), long(1) do begin
		out( 0: 3,i) = byte(orbit.point.viewtime(i).location(0),		0,4)
		out( 4: 7,i) = byte(orbit.point.viewtime(i).location(1),		0,4)
		out( 8:11,i) = byte(orbit.point.viewtime(i).TAI93_start,		0,4)
		out(12:15,i) = byte(orbit.point.viewtime(i).TAI93_end,			0,4)
		out(16:19,i) = byte(orbit.point.viewtime(i).effective_obs,		0,4)
		out(   20,i) = byte(orbit.point.viewtime(i).alert_flag,			0,1)
		out(   21,i) = byte(orbit.point.viewtime(i).approx_threshold,		0,1)
	endfor
	hdf_vd_write, vd, $
	   'location,TAI93_start,TAI93_end,effective_obs,alert_flag,approx_threshold', $
	   out, nrec = orbit.point.point_summary.vt_count
	hdf_vg_insert,pvg,vd
	hdf_vd_detach,vd
		
	return
end

pro WRITE_bg_summaries,fname,orbit,hdffile,ovg,pvg,lvg

	; Finally insert background summary vdata into point vgroup
	
	vd = hdf_vd_attach(hdffile,-1,/write)
	hdf_vd_setinfo, vd, class='data', name='bg_summary'
	hdf_vd_fdefine,vd,'TAI93_time',/double,order=1
	hdf_vd_fdefine,vd,'address',/long,order=1
	hdf_vd_fdefine,vd,'boresight',/float,order=2
	hdf_vd_fdefine,vd,'corners',/float,order=8
	out = bytarr(52, orbit.point.point_summary.bg_count)
	for i = long(0), long(orbit.point.point_summary.bg_count-1), long(1) do begin
		out( 0: 7,i) = byte(orbit.point.bg_summary(i).TAI93_time,		0,8)
		out( 8:11,i) = byte(orbit.point.bg_summary(i).address,			0,4)
		out(12:15,i) = byte(orbit.point.bg_summary(i).boresight(0),		0,4)
		out(16:19,i) = byte(orbit.point.bg_summary(i).boresight(1),		0,4)
		for j = 0,7,1 do begin
		   out(20+(j*4):23+(j*4),i) = byte(orbit.point.bg_summary(i).corners(j),0,4)
		endfor
	endfor
	hdf_vd_write, vd, $
	   'TAI93_time,address,boresight,corners', out, nrec=orbit.point.point_summary.bg_count
	hdf_vg_insert,pvg,vd
	hdf_vd_detach,vd

	return
end

pro WRITE_close_file,fname,orbit,hdffile,ovg,pvg,lvg

	hdf_vg_detach, lvg
	hdf_vg_detach, pvg
	hdf_vg_detach, ovg
	
	hdf_close,hdffile
	
	return
end

; ***********************************************************************************************
; *   				Old OTD Read Library routines; 10/27/97 - DJB			*
; ***********************************************************************************************

pro READ_OLD_orbit, fname, this_orbit, old_orbit, NO_AREAS=no_areas, NO_FLASHES = no_flashes, $
	       NO_GROUPS = no_groups, NO_EVENTS = no_events, native=native, _EXTRA=e
	
						status_title = 'Reading old OTD file "'+fname+'"'
						if (get_version() ge 5) then begin
							dummy_base = Widget_Base()
							status_base = Widget_Base(Row=1,title=status_title,uvalue=[0,0],tlb_frame_attr=0,scroll=0,/modal,group_leader=dummy_base)
						endif else begin
							status_base = Widget_Base(Row=1,title=status_title,uvalue=[0,0],tlb_frame_attr=0,scroll=0)
						endelse
;						status = CW_Field(status_base, value='Status:  Reading ORBIT',title='Status:',xsize=1.5*strlen(fname),/noedit)
						status = Widget_label(status_base, value='Status:  Reading ORBIT',xsize=12*strlen(fname),/align_left)

						Widget_Control, status_base, /realize
						Widget_Control, status, Set_value='Status:  Status:  ( 0% done) -> Reading ORBIT_ATTRIBUTES...'

	READ_OLD_orbit_attributes, fname, old_orbit_attributes, status=status, native=native, _EXTRA=e
	
						Widget_Control, status, Set_value='Status:  ( 0% done) -> Reading ORBIT_SUMMARY...'

	READ_OLD_orbit_summary, fname, old_orbit_summary, status=status, native=native, _EXTRA=e
	
;	READ_OLD_thresholds, fname, has_thresholds, old_thresholds, status=status, native=native, _EXTRA=e
		
						Widget_Control, status, Get_UValue = pct_done
						status_msg = string(100*pct_done(0)/pct_done(1), '("Status:  (",I2,"% done) -> Reading BROWSE...")')
						Widget_Control, status, Set_Value=status_msg
						
	READ_OLD_browse, fname, old_browse, status=status, native=native, _EXTRA=e
	
						Widget_Control, status, Get_UValue = pct_done
						status_msg = string(100*pct_done(0)/pct_done(1), '("Status:  (",I2,"% done) -> Reading FLASH_DENSITY...")')
						Widget_Control, status, Set_Value=status_msg

	READ_OLD_flash_density, fname, old_flash_density,status=status, native=native, _EXTRA=e
	
	if keyword_set(no_areas) then begin
		old_areas = {an_old_area, seq:long(0), TAI93:double(0), delta:float(0), view:float(0), 	$
			s_z_a:fix(0), d_n_t:byte(0), end_status:byte(0), events:fix(0),		$
			cent:fltarr(2), stdev:fltarr(2), loc_count:fix(0), rad:float(0),	$
			orbit_id:long(0), day:long(0), children:long(0), child_seq:long(0),	$
			child_rec:long(0), QA:intarr(4)}
		old_orbit_summary.summary.areas = 1
	endif else begin
						Widget_Control, status, Get_UValue = pct_done
						status_msg = string(100*pct_done(0)/pct_done(1), '("Status:  (",I2,"% done) -> Reading AREAs...")')
						Widget_Control, status, Set_Value=status_msg
		READ_OLD_areas, fname, old_areas, status=status, native=native
						pct_done(0) = pct_done(0) + old_orbit_summary.summary.areas * 76L
						Widget_Control, status, Set_UValue = pct_done
	endelse
	
	if keyword_set(no_flashes) then begin
		old_flashes = {an_old_flash, seq:long(0), TAI93:double(0), delta:float(0), view:float(0), 	$
			s_z_a:fix(0), d_n_t:byte(0), end_status:byte(0), events:fix(0),		$
			cent:fltarr(2), stdev:fltarr(2), loc_count:fix(0), rad:float(0),	$
			parent_seq:long(0), parent_rec:long(0), children:long(0), 		$
			child_seq:long(0), child_rec:long(0), QA:intarr(4)}
		old_orbit_summary.summary.flashes = 1
	endif else begin
						Widget_Control, status, Get_UValue = pct_done
						status_msg = string(100*pct_done(0)/pct_done(1), '("Status:  (",I2,"% done) -> Reading FLASHes...")')
						Widget_Control, status, Set_Value=status_msg
		READ_OLD_flashes, fname, old_flashes, status=status, native=native
						pct_done(0) = pct_done(0) + old_orbit_summary.summary.flashes * 76L
						Widget_Control, status, Set_UValue = pct_done
	endelse
	
	if keyword_set(no_groups) then begin
		old_groups = {an_old_group, seq:long(0), TAI93:double(0), delta:float(0), view:float(0), 	$
			s_z_a:fix(0), d_n_t:byte(0), end_status:byte(0), events:fix(0),		$
			cent:fltarr(2), stdev:fltarr(2), loc_count:fix(0), rad:float(0),	$
			parent_seq:long(0), parent_rec:long(0), children:long(0), 		$
			child_seq:long(0), child_rec:long(0), QA:intarr(4)}
		old_orbit_summary.summary.groups = 1
	endif else begin
						Widget_Control, status, Get_UValue = pct_done
						status_msg = string(100*pct_done(0)/pct_done(1), '("Status:  (",I2,"% done) -> Reading GROUPs...")')
						Widget_Control, status, Set_Value=status_msg
		READ_OLD_groups, fname, old_groups, status=status, native=native
						pct_done(0) = pct_done(0) + old_orbit_summary.summary.groups * 76L
						Widget_Control, status, Set_UValue = pct_done
	endelse
	
	if keyword_set(no_events) then begin
		old_events = {an_old_event, event_num:long(0), seq_num:long(0), TAI93:double(0),		$
			s_z_a:fix(0), d_n_t:byte(0), x_pixel:fix(0), y_pixel:fix(0),		$
			raw_radiance:fix(0), cal_radiance:float(0), parent_seq:long(0), 	$
			parent_rec:long(0), QA:intarr(4), location:fltarr(2)}
		old_orbit_summary.summary.events = 1
	endif else begin
						Widget_Control, status, Get_UValue = pct_done
						status_msg = string(100*pct_done(0)/pct_done(1), '("Status:  (",I2,"% done) -> Reading EVENTs...")')
						Widget_Control, status, Set_Value=status_msg
		READ_OLD_events, fname, old_events, status=status, native=native
						pct_done(0) = pct_done(0) + old_orbit_summary.summary.events * 53L
						Widget_Control, status, Set_UValue = pct_done
	endelse
	
	; Create the old format structure
		
	some_old_area_vg  = {area:replicate(old_areas(0),old_orbit_summary.summary.areas)}
	some_old_flash_vg = {flash:replicate(old_flashes(0),old_orbit_summary.summary.flashes)}
	some_old_group_vg = {group:replicate(old_groups(0),old_orbit_summary.summary.groups)}
	some_old_event_vg = {event:replicate(old_events(0),old_orbit_summary.summary.events)}
	
	old_orbit = {		   area:replicate(some_old_area_vg,1), $
				   flash:replicate(some_old_flash_vg,1), $
				   group:replicate(some_old_group_vg,1), $
				   event:replicate(some_old_event_vg,1), $
				   browse:replicate(old_browse,1),	 $
				   flash_density:replicate(old_flash_density,1),	$
				   orbit_summary:replicate(old_orbit_summary,1),	$
				   orbit:{an_orbit_attributes}}
			
	old_orbit.area.area 		= old_areas
	old_orbit.flash.flash 		= old_flashes
	old_orbit.group.group		= old_groups
	old_orbit.event.event 		= old_events
	old_orbit.browse		= old_browse
	old_orbit.flash_density		= old_flash_density
	old_orbit.orbit_summary		= old_orbit_summary
	old_orbit.orbit			= old_orbit_attributes
			
	; Now stuff these into a new format structure
	
	Widget_Control, status, Set_value='Status:  Done reading ORBIT.'
						;print,'Done Reading ORBIT ',fname
						;print,' '
	Widget_Control, status_base, /Destroy	

						status_title = 'Converting '+fname+' to new format'
						status_base = Widget_Base(Row=1,frame=4,title=status_title,uvalue=[0,0],tlb_frame_attr=8,scroll=0)
;						status = CW_Field(status_base, value='Status:  Converting.',title='Status:',xsize=36,/noedit)
						status = Widget_label(status_base, value='Status:  Converting',xsize=12*strlen(status_title),/align_left)
						widget_control,status_base,/realize
	; print,'Converting old orbit to new format...'
	OLDOTD_to_LISOTD, this_orbit, old_orbit, NO_AREAS=no_areas, NO_FLASHES = no_flashes, $
				NO_GROUPS = no_groups, NO_EVENTS = no_events, status=status, native=native, _EXTRA=e
						widget_control, status_base, /destroy
	; print,'Done converting.'
	
	return
	
end


pro getoldvdsizes,fname,vdsizes
	hdffile = hdf_open(fname)
	if (hdffile eq long(0)) then begin
	   print,'Error opening file *'+fname+'*'
	   hdf_close,hdffile
	   stop
	endif
	vdname = ['area','flash','group','event','browse','vector','image','2.5 degree','view time', $
			'500km','summary','rate_f','rate_e','ephemeris','orbit','metadata','threshold']
	vdsizes = lonarr(17)
	for vdn = 0,16,1 do begin
		vdref = hdf_vd_find(hdffile,vdname(vdn))
		vd = hdf_vd_attach(hdffile,vdref,/read)
		hdf_vd_get,vd,class=cl,count=ct,fields=flds,interlace=ilc,name=nm,nfields=nflds, $
				ref=rf,size=sz,tag=tg
		for i=0,nflds-1 do begin
			hdf_vd_getinfo,vd,i,name=n,type=t,order=o
		endfor
		vdsizes(vdn) = long(sz)
		hdf_vd_detach,vd
	endfor
	hdf_close,hdffile
	return
end

pro getoldvdcounts,fname,vdsizes
	hdffile = hdf_open(fname)
	if (hdffile eq long(0)) then begin
	   print,'Error opening file *'+fname+'*'
	   hdf_close,hdffile
	   stop
	endif
	vdname = ['area','flash','group','event','browse','vector','image','2.5 degree','view time', $
			'500km','summary','rate_f','rate_e','ephemeris','orbit','metadata','threshold']
	vdsizes = lonarr(17)
	for vdn = 0,16,1 do begin
		vdref = hdf_vd_find(hdffile,vdname(vdn))
		vd = hdf_vd_attach(hdffile,vdref,/read)
		hdf_vd_get,vd,class=cl,count=ct,fields=flds,interlace=ilc,name=nm,nfields=nflds, $
				ref=rf,size=sz,tag=tg
		for i=0,nflds-1 do begin
			hdf_vd_getinfo,vd,i,name=n,type=t,order=o
		endfor
		vdsizes(vdn) = long(ct)
		hdf_vd_detach,vd
	endfor
	hdf_close,hdffile
	return
end

pro READ_OLD_orbit_attributes, fname, this_orbit_attributes, status=status, native=native, _EXTRA=e
	hdffile = hdf_open(fname)
	if (hdffile eq long(0)) then begin
	   print,'Error opening file *'+fname+'*'
	   hdf_close,hdffile
	   stop
	endif
	getoldvdsizes,fname,vdsizes
	vdref = hdf_vd_find(hdffile,'orbit')
	this_orbit_attributes = {an_orbit_attributes,orbit_ID:long(0),TAI93_start:double(0), 	$
				   start_location:fltarr(2),TAI93_end:double(0),end_location:fltarr(2)}
	vd = hdf_vd_attach(hdffile,vdref,/read)
	nread = hdf_vd_read(vd,temp)
	this_orbit_attributes.orbit_ID 		= long(temp,0)
	this_orbit_attributes.TAI93_start	= double(temp,4)
	this_orbit_attributes.start_location(0)	= float(temp,12)
	this_orbit_attributes.start_location(1) = float(temp,16)
	this_orbit_attributes.TAI93_end		= double(temp,20)
	this_orbit_attributes.end_location(0)	= float(temp,28)
	this_orbit_attributes.end_location(1)	= float(temp,32)
	hdf_vd_detach,vd
	hdf_close,hdffile
	return
end
	
pro READ_OLD_orbit_summary, fname, this_orbit_summary, status=status, NO_ONE_SECONDS=no_one_seconds, $
					native=native, _EXTRA=e
	
						Widget_Control, status, Set_value='Status:  ( 0% done) -> Reading SUMMARY...'
	READ_OLD_summary, fname, old_summary, status=status, native=native

	usea = 1-keyword_set(no_areas)
	usef = 1-keyword_set(no_flashes)
	useg = 1-keyword_set(no_groups)
	usee = 1-keyword_set(no_events)
	useo = 1-keyword_set(no_one_seconds)
	if keyword_set(e) then begin
		usev = (where(tag_names(e) eq 'NO_VIEWTIMES') eq -1)
	endif else begin
		usev = 1
	endelse
	
	total_bytes = usea * old_summary.areas * 76L + $
		      usef * old_summary.flashes * 76L + $
		      useg * old_summary.groups * 76L + $
		      usee * old_summary.events * 53L + $
		      usev * 55000L * 18L + $
		      useo * 12200L * 19L + useo * 6000L * 128L
	pct_done = [0L,total_bytes]
	
						Widget_Control, status, Set_UValue=pct_done
						
	; Ignore rate_f vdata for now
	
	if keyword_set(no_one_seconds) then begin
		some_old_event_rates = {a_rate_e,count:intarr(6100)}
		old_event_rates = replicate(some_old_event_rates,19)
		old_ephemerises = {an_ephemeris,GPS_time:double(0),spice_kernel:dblarr(15)}
	endif else begin
						Widget_Control, status, Set_value='Status:  ( 0% done) -> Reading EVENT_RATES...'
		READ_OLD_event_rates, fname, old_event_rates, status=status, native=native
						Widget_Control, status, Get_UValue = pct_done
						pct_done(0) = pct_done(0) + 12200L*19L
						Widget_Control, status, Set_UValue = pct_done
						status_msg = string(100*pct_done(0)/pct_done(1), '("Status:  (",I2,"% done) -> Reading EPHEMERISES...")')
						Widget_Control, status, Set_Value = status_msg
		READ_OLD_ephemeris, fname, old_ephemerises, status=status, native=native
						pct_done(0) = pct_done(0) + 6000L*128L
						Widget_Control, status, Set_UValue = pct_done
	endelse
	
	esize = size(old_ephemerises)
	elength = esize(1)
	ersize = size(old_event_rates)
	erlength = ersize(1)
	
	this_orbit_summary = {  summary:{a_summary}, $
				rate_e:replicate(old_event_rates(0),erlength), $
				ephemeris:replicate(old_ephemerises(0),elength)}
	
	this_orbit_summary.summary 	= old_summary
	this_orbit_summary.rate_e	= old_event_rates
	this_orbit_summary.ephemeris 	= old_ephemerises
	
	return
end

pro READ_OLD_summary, fname, this_summary, status=status, native=native, _EXTRA=e
	hdffile = hdf_open(fname)
	if (hdffile eq long(0)) then begin
	   print,"Error opening file *"+fname+"*"
	   hdf_close,hdffile
	   stop
	endif
	getoldvdsizes,fname,vdsizes
	vdref = hdf_vd_find(hdffile,'summary')
	this_summary = {a_summary,areas:long(0),flashes:long(0),groups:long(0),events:long(0), $
					backgrounds:long(0),total_cal_radiance:float(0)}
	vd = hdf_vd_attach(hdffile,vdref,/read)
	nread = hdf_vd_read(vd,temp)
	this_summary.areas			= long(temp,0)
	this_summary.flashes 		= long(temp,4)
	this_summary.groups		= long(temp,8)
	this_summary.events		= long(temp,12)
	this_summary.backgrounds	= long(temp,16)
	this_summary.total_cal_radiance= float(temp,20)
	hdf_vd_detach,vd
	hdf_close,hdffile
	return
end

pro READ_OLD_event_rates, fname, these_event_rates, status=status, native=native, _EXTRA=e
	hdffile = hdf_open(fname)
	if (hdffile eq long(0)) then begin
	   print,"Error opening file *"+fname+"*"
	   hdf_close,hdffile
	   stop
	endif
	getoldvdsizes,fname,vdsizes
	vdref = hdf_vd_find(hdffile,'rate_e')
	some_event_rate = {a_rate_e,count:intarr(6100)}
	vd = hdf_vd_attach(hdffile,vdref,/read)
	nread = hdf_vd_read(vd,temp)
	these_event_rates = replicate({a_rate_e},max([nread,1]))
	for i = long(0), long(nread-1), long(1) do begin
		these_event_rates(i).count = temp(*,i)
	endfor
	hdf_vd_detach,vd
	hdf_close,hdffile
	return
end

pro READ_OLD_ephemeris, fname, these_ephemerises, status=status, native=native, _EXTRA=e
	this_os = !version.os
	call_nat = keyword_set(native)

	some_ephemeris = {an_ephemeris,GPS_time:double(0),spice_kernel:dblarr(15)}

	if (call_nat eq 1) then begin
		res = CALL_EXTERNAL(native,'gotd_ephemeris_count',fname)
		nread = res
		; print,nread
		these_ephemerises = replicate({an_ephemeris},max([nread,1]))
		res=CALL_EXTERNAL(native,'gotd_ephemeris',fname,these_ephemerises)
	endif else begin
		hdffile = hdf_open(fname)
		if (hdffile eq long(0)) then begin
		   print,"Error opening file *"+fname+"*"
		   hdf_close,hdffile
		   stop
		endif
		getoldvdsizes,fname,vdsizes
		vdref = hdf_vd_find(hdffile,'ephemeris')
		some_ephemeris = {an_ephemeris,GPS_time:double(0),spice_kernel:dblarr(15)}
		vd = hdf_vd_attach(hdffile,vdref,/read)
		nread = hdf_vd_read(vd,temp)
		newtemp = dblarr(16,nread)
		insize = size(temp)
		inwidth = insize(1)
		these_ephemerises = replicate({an_ephemeris},max([nread,1]))
;		for jj = long(0),long(nread*16-1),long(1) do begin
;			newrecord  = jj / 16
;			newelement = jj mod 16
;			oldrecord  = jj / inwidth
;			oldelement = jj mod inwidth
;			newtemp(newelement,newrecord) = temp(oldelement,oldrecord)
;		endfor
		jj = lindgen(nread*16)
		newtemp(jj) = temp(jj)
		jj = 0
		tempbg_summary = 0
		for i = long(0), long(nread-1), long(1) do begin
			some_ephemeris = bytarr(128)
			some_ephemeris = newtemp(*,i)
			these_ephemerises(i).GPS_time = double(some_ephemeris,0)
			for j = 0, 14, 1 do begin
				these_ephemerises(i).spice_kernel(j) = double(some_ephemeris,8+j*8)
			endfor
		endfor
		hdf_vd_detach,vd
		hdf_close,hdffile
	endelse
	return
end

pro READ_OLD_browse, fname, this_browse, status=status, native=native, _EXTRA=e

	; Ignore EGFA browses for now
	
	; Ignore vector statistics for now
						Widget_Control, status, Get_UValue = pct_done
						status_msg = string(100*pct_done(0)/pct_done(1), '("Status:  (",I2,"% done) --> Reading IMAGE_ATTRIBUTES...")')
						Widget_Control, status, Set_Value = status_msg
	READ_OLD_image_attributes, fname, old_image_attributes, status=status	, native=native
	oiasize  = size(old_image_attributes)
	oialength = oiasize(1)
	this_browse = {image:replicate(old_image_attributes(0),oialength)}
	this_browse.image = old_image_attributes
	return
end

pro READ_OLD_image_attributes, fname, these_image_attributes, status=status, native=native, _EXTRA=e
	hdffile = hdf_open(fname)
	if (hdffile eq long(0)) then begin
	   print,"Error opening file *"+fname+"*"
	   hdf_close,hdffile
	   stop
	endif
	getoldvdsizes,fname,vdsizes
	vdref = hdf_vd_find(hdffile,'image')
	some_image_attribute = {an_image,TAI93:double(0),NW_location:fltarr(2), 	$
				NE_location:fltarr(2),SW_location:fltarr(2),SE_location:fltarr(2),	$
				nadir_location:fltarr(2),image_pointer:long(0),calibration_number:long(0), $
				spice_pointer:long(0)}
	vd = hdf_vd_attach(hdffile,vdref,/read)
	nread = hdf_vd_read(vd,temp)
	newtemp = bytarr(60,nread)
	insize = size(temp)
	inwidth = insize(1)
	these_image_attributes = replicate({an_image},max([nread,1]))
;	for jj = long(0),long(nread*60-1),long(1) do begin
;		newrecord  = jj / 60
;		newelement = jj mod 60
;		oldrecord  = jj / inwidth
;		oldelement = jj mod inwidth
;		newtemp(newelement,newrecord) = temp(oldelement,oldrecord)
;	endfor
		jj = lindgen(nread*60)
		newtemp(jj) = temp(jj)
		jj = 0
		tempbg_summary = 0
	for i = long(0), long(nread-1), long(1) do begin
		some_image_attribute = bytarr(60)
		some_image_attribute = newtemp(*,i)
		these_image_attributes(i).TAI93			= double(some_image_attribute,0)
		these_image_attributes(i).NW_location(0)	= float(some_image_attribute,8)
		these_image_attributes(i).NW_location(1)	= float(some_image_attribute,12)
		these_image_attributes(i).NE_location(0)	= float(some_image_attribute,16)
		these_image_attributes(i).NE_location(1)	= float(some_image_attribute,20)
		these_image_attributes(i).SW_location(0)	= float(some_image_attribute,24)
		these_image_attributes(i).SW_location(1)	= float(some_image_attribute,28)
		these_image_attributes(i).SE_location(0)	= float(some_image_attribute,32)
		these_image_attributes(i).SE_location(1)	= float(some_image_attribute,36)
		these_image_attributes(i).nadir_location(0)	= float(some_image_attribute,40)
		these_image_attributes(i).nadir_location(1)	= float(some_image_attribute,44)
		these_image_attributes(i).image_pointer		= long(some_image_attribute,48)
		these_image_attributes(i).calibration_number	= long(some_image_attribute,52)
		these_image_attributes(i).spice_pointer		= long(some_image_attribute,56)
	endfor
	hdf_vd_detach,vd
	hdf_close,hdffile
	return
end

pro READ_OLD_flash_density, fname, this_flash_density, NO_VIEWTIMES = no_viewtimes, status=status, native=native, _EXTRA=e

	; Ignore 2.5 degree grid for now
	
	if keyword_set(no_viewtimes) then begin
		old_view_times = {a_view_time,grid_point:fltarr(2),TAI93_start:double(0),duration:fix(0)}
	endif else begin
						Widget_Control, status, Get_UValue = pct_done
						status_msg = string(100*pct_done(0)/pct_done(1), '("Status:  (",I2,"% done) --> Reading VIEWTIMEs...")')
						Widget_Control, status, Set_Value=status_msg
		READ_OLD_viewtimes, fname, old_view_times, status=status, native=native
						pct_done(0) = pct_done(0) + 55000L*18L
						Widget_Control, status, Set_UValue = pct_done
	endelse
	
	; Ignore 500km grid for now
	
	vtsize = size(old_view_times)
	vtlength = vtsize(1)
	this_flash_density = {view_time:replicate(old_view_times(0),vtlength)}
	this_flash_density.view_time = old_view_times
	this_flash_density.view_time(*).grid_point(*) = $
		this_flash_density.view_time(*).grid_point(*) + .25
	return
end

pro READ_OLD_viewtimes, fname, these_view_times, status=status, native=native, _EXTRA=e
	this_os = !version.os
	call_nat = keyword_set(native)

	some_view_time = {a_view_time,grid_point:fltarr(2),TAI93_start:double(0),duration:fix(0)}

	if (call_nat eq 1) then begin
		res = CALL_EXTERNAL(native,'gotd_viewtime_count',fname)
		nread = res
		; print,nread
		these_view_times = replicate({a_view_time},max([nread,1]))
		res=CALL_EXTERNAL(native,'gotd_viewtime',fname,these_view_times)
	endif else begin
		hdffile = hdf_open(fname)
		if (hdffile eq long(0)) then begin
		   print,"Error opening file *"+fname+"*"
		   hdf_close,hdffile
		   stop
		endif
		getoldvdsizes,fname,vdsizes
		vdref = hdf_vd_find(hdffile,'view time')
		some_view_time = {a_view_time,grid_point:fltarr(2),TAI93_start:double(0),duration:fix(0)}
		vd = hdf_vd_attach(hdffile,vdref,/read)
		nread = hdf_vd_read(vd,temp)
		newtemp = bytarr(18,nread)
		insize = size(temp)
		inwidth = insize(1)
		these_view_times = replicate({a_view_time},max([nread,1]))
;		for jj = long(0),long(nread*18-1),long(1) do begin
;			newrecord  = jj / 18
;			newelement = jj mod 18
;			oldrecord  = jj / inwidth
;			oldelement = jj mod inwidth
;			newtemp(newelement,newrecord) = temp(oldelement,oldrecord)
;		endfor
		jj = lindgen(nread*18)
		newtemp(jj) = temp(jj)
		jj = 0
		tempbg_summary = 0
		for i = long(0), long(nread-1), long(1) do begin
			some_view_time = bytarr(18)
			some_view_time = newtemp(*,i)
			these_view_times(i).grid_point(0)		= float(some_view_time,0)
			these_view_times(i).grid_point(1)		= float(some_view_time,4)
			these_view_times(i).TAI93_start			= double(some_view_time,8)
			these_view_times(i).duration			= fix(some_view_time,16)
		endfor
		hdf_vd_detach,vd
		hdf_close,hdffile
	endelse
	return
end

pro READ_OLD_areas, fname, these_areas, native=native, _EXTRA=e
	this_os = !version.os
	call_nat = keyword_set(native)

	some_area = {an_old_area, seq:long(0), TAI93:double(0), delta:float(0), view:float(0), 	$
			s_z_a:fix(0), d_n_t:byte(0), end_status:byte(0), events:fix(0),		$
			cent:fltarr(2), stdev:fltarr(2), loc_count:fix(0), rad:float(0),	$
			orbit_id:long(0), day:long(0), children:long(0), child_seq:long(0),	$
			child_rec:long(0), QA:intarr(4)}

	if (call_nat eq 1) then begin
		READ_old_summary,fname,summ,status=status,_EXTRA=e
		nread = summ.areas
		these_areas = replicate({an_old_area},max([nread,1]))
		res=CALL_EXTERNAL(native,'gotd_area',fname,these_areas)
	endif else begin
		hdffile = hdf_open(fname)
		if (hdffile eq long(0)) then begin
		   print,"Error opening file *"+fname+"*"
		   hdf_close,hdffile
		   stop
		endif
		getoldvdsizes,fname,vdsizes
		vdref = hdf_vd_find(hdffile,'area')
		some_area = {an_old_area, seq:long(0), TAI93:double(0), delta:float(0), view:float(0), 	$
				s_z_a:fix(0), d_n_t:byte(0), end_status:byte(0), events:fix(0),		$
				cent:fltarr(2), stdev:fltarr(2), loc_count:fix(0), rad:float(0),	$
				orbit_id:long(0), day:long(0), children:long(0), child_seq:long(0),	$
				child_rec:long(0), QA:intarr(4)}
		vd = hdf_vd_attach(hdffile,vdref,/read)
		nread = hdf_vd_read(vd,temp)
		newtemp = bytarr(76,nread)
		insize = size(temp)
		inwidth = insize(1)
		these_areas = replicate({an_old_area},max([nread,1]))
;		for jj = long(0),long(nread*76-1),long(1) do begin
;			newrecord  = jj / 76
;			newelement = jj mod 76
;			oldrecord  = jj / inwidth
;			oldelement = jj mod inwidth
;			newtemp(newelement,newrecord) = temp(oldelement,oldrecord)
;		endfor
		jj = lindgen(nread*76)
		newtemp(jj) = temp(jj)
		jj = 0
		tempbg_summary = 0
		for i = long(0), long(nread-1), long(1) do begin
			some_area = bytarr(76)
			some_area = newtemp(*,i)
			these_areas(i).seq				= long(some_area,0)
			these_areas(i).TAI93				= double(some_area,4)
			these_areas(i).delta				= float(some_area,12)
			these_areas(i).view				= float(some_area,16)
			these_areas(i).s_z_a				= fix(some_area,20)
			these_areas(i).d_n_t				= byte(some_area,22)
			these_areas(i).end_status			= byte(some_area,23)
			these_areas(i).events				= fix(some_area,24)
			these_areas(i).cent(0)				= float(some_area,26)
			these_areas(i).cent(1)				= float(some_area,30)
			these_areas(i).stdev(0)				= float(some_area,34)
			these_areas(i).stdev(1)				= float(some_area,38)
			these_areas(i).loc_count			= fix(some_area,42)
			these_areas(i).rad				= float(some_area,44)
			these_areas(i).orbit_id				= long(some_area,48)
			these_areas(i).day				= long(some_area,52)
			these_areas(i).children				= long(some_area,56)
			these_areas(i).child_seq			= long(some_area,60)
			these_areas(i).child_rec			= long(some_area,64)
			these_areas(i).QA(0)				= fix(some_area,68)
			these_areas(i).QA(1)				= fix(some_area,70)
			these_areas(i).QA(2)				= fix(some_area,72)
			these_areas(i).QA(3)				= fix(some_area,74)			
		endfor
		hdf_vd_detach,vd
		hdf_close,hdffile
	endelse
	return
end

pro READ_OLD_flashes, fname, these_flashes, native=native, _EXTRA=e
	this_os = !version.os
	call_nat = keyword_set(native)

	some_flash = {an_old_flash, seq:long(0), TAI93:double(0), delta:float(0), view:float(0), 	$
			s_z_a:fix(0), d_n_t:byte(0), end_status:byte(0), events:fix(0),		$
			cent:fltarr(2), stdev:fltarr(2), loc_count:fix(0), rad:float(0),	$
			parent_seq:long(0), parent_rec:long(0), children:long(0), 		$
			child_seq:long(0), child_rec:long(0), QA:intarr(4)}

	if (call_nat eq 1) then begin
		READ_old_summary,fname,summ,status=status,_EXTRA=e
		nread = summ.flashes
		these_flashes = replicate({an_old_flash},max([nread,1]))
		res=CALL_EXTERNAL(native,'gotd_flash',fname,these_flashes)
	endif else begin
		hdffile = hdf_open(fname)
		if (hdffile eq long(0)) then begin
		   print,"Error opening file *"+fname+"*"
		   hdf_close,hdffile
		   stop
		endif
		getoldvdsizes,fname,vdsizes
		vdref = hdf_vd_find(hdffile,'flash')
		some_flash = {an_old_flash, seq:long(0), TAI93:double(0), delta:float(0), view:float(0), 	$
				s_z_a:fix(0), d_n_t:byte(0), end_status:byte(0), events:fix(0),		$
				cent:fltarr(2), stdev:fltarr(2), loc_count:fix(0), rad:float(0),	$
				parent_seq:long(0), parent_rec:long(0), children:long(0), 		$
				child_seq:long(0), child_rec:long(0), QA:intarr(4)}
		vd = hdf_vd_attach(hdffile,vdref,/read)
		nread = hdf_vd_read(vd,temp)
		newtemp = bytarr(76,nread)
		insize = size(temp)
		inwidth = insize(1)
		these_flashes = replicate({an_old_flash},max([nread,1]))
;		for jj = long(0),long(nread*76-1),long(1) do begin
;			newrecord  = jj / 76
;			newelement = jj mod 76
;			oldrecord  = jj / inwidth
;			oldelement = jj mod inwidth
;			newtemp(newelement,newrecord) = temp(oldelement,oldrecord)
;		endfor
		jj = lindgen(nread*76)
		newtemp(jj) = temp(jj)
		jj = 0
		tempbg_summary = 0
		for i = long(0), long(nread-1), long(1) do begin
			some_flash = bytarr(76)
			some_flash = newtemp(*,i)
			these_flashes(i).seq				= long(some_flash,0)
			these_flashes(i).TAI93				= double(some_flash,4)
			these_flashes(i).delta				= float(some_flash,12)
			these_flashes(i).view				= float(some_flash,16)
			these_flashes(i).s_z_a				= fix(some_flash,20)
			these_flashes(i).d_n_t				= byte(some_flash,22)
			these_flashes(i).end_status			= byte(some_flash,23)
			these_flashes(i).events				= fix(some_flash,24)
			these_flashes(i).cent(0)			= float(some_flash,26)
			these_flashes(i).cent(1)			= float(some_flash,30)
			these_flashes(i).stdev(0)			= float(some_flash,34)
			these_flashes(i).stdev(1)			= float(some_flash,38)
			these_flashes(i).loc_count			= fix(some_flash,42)
			these_flashes(i).rad				= float(some_flash,44)
			these_flashes(i).parent_seq			= long(some_flash,48)
			these_flashes(i).parent_rec			= long(some_flash,52)
			these_flashes(i).children			= long(some_flash,56)
			these_flashes(i).child_seq			= long(some_flash,60)
			these_flashes(i).child_rec			= long(some_flash,64)
			these_flashes(i).QA(0)				= fix(some_flash,68)
			these_flashes(i).QA(1)				= fix(some_flash,70)
			these_flashes(i).QA(2)				= fix(some_flash,72)
			these_flashes(i).QA(3)				= fix(some_flash,74)			
		endfor
		hdf_vd_detach,vd
		hdf_close,hdffile
	endelse
	return
end

pro READ_OLD_groups, fname, these_groups, native=native, _EXTRA=e
	this_os = !version.os
	call_nat = keyword_set(native)

	some_group = {an_old_group, seq:long(0), TAI93:double(0), delta:float(0), view:float(0), 	$
			s_z_a:fix(0), d_n_t:byte(0), end_status:byte(0), events:fix(0),		$
			cent:fltarr(2), stdev:fltarr(2), loc_count:fix(0), rad:float(0),	$
			parent_seq:long(0), parent_rec:long(0), children:long(0), 		$
			child_seq:long(0), child_rec:long(0), QA:intarr(4)}

	if (call_nat eq 1) then begin
		READ_old_summary,fname,summ,status=status,_EXTRA=e
		nread = summ.groups
		these_groups = replicate({an_old_group},max([nread,1]))
		res=CALL_EXTERNAL(native,'gotd_group',fname,these_groups)
	endif else begin
		hdffile = hdf_open(fname)
		if (hdffile eq long(0)) then begin
		   print,"Error opening file *"+fname+"*"
		   hdf_close,hdffile
		   stop
		endif
		getoldvdsizes,fname,vdsizes
		vdref = hdf_vd_find(hdffile,'group')
		some_group = {an_old_group, seq:long(0), TAI93:double(0), delta:float(0), view:float(0), 	$
				s_z_a:fix(0), d_n_t:byte(0), end_status:byte(0), events:fix(0),		$
				cent:fltarr(2), stdev:fltarr(2), loc_count:fix(0), rad:float(0),	$
				parent_seq:long(0), parent_rec:long(0), children:long(0), 		$
				child_seq:long(0), child_rec:long(0), QA:intarr(4)}
		vd = hdf_vd_attach(hdffile,vdref,/read)
		nread = hdf_vd_read(vd,temp)
		newtemp = bytarr(76,nread)
		insize = size(temp)
		inwidth = insize(1)
		these_groups = replicate({an_old_group},max([nread,1]))
;		for jj = long(0),long(nread*76-1),long(1) do begin
;			newrecord  = jj / 76
;			newelement = jj mod 76
;			oldrecord  = jj / inwidth
;			oldelement = jj mod inwidth
;			newtemp(newelement,newrecord) = temp(oldelement,oldrecord)
;		endfor
		jj = lindgen(nread*76)
		newtemp(jj) = temp(jj)
		jj = 0
		tempbg_summary = 0
		for i = long(0), long(nread-1), long(1) do begin
			some_group = bytarr(76)
			some_group = newtemp(*,i)
			these_groups(i).seq				= long(some_group,0)
			these_groups(i).TAI93				= double(some_group,4)
			these_groups(i).delta				= float(some_group,12)
			these_groups(i).view				= float(some_group,16)
			these_groups(i).s_z_a				= fix(some_group,20)
			these_groups(i).d_n_t				= byte(some_group,22)
			these_groups(i).end_status			= byte(some_group,23)
			these_groups(i).events				= fix(some_group,24)
			these_groups(i).cent(0)				= float(some_group,26)
			these_groups(i).cent(1)				= float(some_group,30)
			these_groups(i).stdev(0)			= float(some_group,34)
			these_groups(i).stdev(1)			= float(some_group,38)
			these_groups(i).loc_count			= fix(some_group,42)
			these_groups(i).rad				= float(some_group,44)
			these_groups(i).parent_seq			= long(some_group,48)
			these_groups(i).parent_rec			= long(some_group,52)
			these_groups(i).children			= long(some_group,56)
			these_groups(i).child_seq			= long(some_group,60)
			these_groups(i).child_rec			= long(some_group,64)
			these_groups(i).QA(0)				= fix(some_group,68)
			these_groups(i).QA(1)				= fix(some_group,70)
			these_groups(i).QA(2)				= fix(some_group,72)
			these_groups(i).QA(3)				= fix(some_group,74)			
		endfor
		hdf_vd_detach,vd
		hdf_close,hdffile
	endelse
	return
end

pro READ_OLD_events, fname, these_events, native=native, _EXTRA=e
	this_os = !version.os
	call_nat = keyword_set(native)

	some_event = {an_old_event, event_num:long(0), seq_num:long(0), TAI93:double(0),		$
			s_z_a:fix(0), d_n_t:byte(0), x_pixel:fix(0), y_pixel:fix(0),		$
			raw_radiance:fix(0), cal_radiance:float(0), parent_seq:long(0), 	$
			parent_rec:long(0), QA:intarr(4), location:fltarr(2)}

	if (call_nat eq 1) then begin
		READ_old_summary,fname,summ,status=status,_EXTRA=e
		nread = summ.events
		these_events = replicate({an_old_event},max([nread,1]))
		res=CALL_EXTERNAL(native,'gotd_event',fname,these_events)
	endif else begin
		hdffile = hdf_open(fname)
		if (hdffile eq long(0)) then begin
		   print,"Error opening file *"+fname+"*"
		   hdf_close,hdffile
		   stop
		endif
		getoldvdsizes,fname,vdsizes
		vdref = hdf_vd_find(hdffile,'event')
		some_event = {an_old_event, event_num:long(0), seq_num:long(0), TAI93:double(0),		$
				s_z_a:fix(0), d_n_t:byte(0), x_pixel:fix(0), y_pixel:fix(0),		$
				raw_radiance:fix(0), cal_radiance:float(0), parent_seq:long(0), 	$
				parent_rec:long(0), QA:intarr(4), location:fltarr(2)}
		vd = hdf_vd_attach(hdffile,vdref,/read)
		nread = hdf_vd_read(vd,temp)
		newtemp = bytarr(53,nread)
		insize = size(temp)
		inwidth = insize(1)
		these_events = replicate({an_old_event},max([nread,1]))
;		for jj = long(0),long(nread*53-1),long(1) do begin
;			newrecord  = jj / 53
;			newelement = jj mod 53
;			oldrecord  = jj / inwidth
;			oldelement = jj mod inwidth
;			newtemp(newelement,newrecord) = temp(oldelement,oldrecord)
;		endfor
		jj = lindgen(nread*53)
		newtemp(jj) = temp(jj)
		jj = 0
		tempbg_summary = 0
		for i = long(0), long(nread-1), long(1) do begin
			some_event = bytarr(53)
			some_event = newtemp(*,i)
			these_events(i).event_num			= long(some_event,0)
			these_events(i).seq_num				= long(some_event,4)
			these_events(i).TAI93				= double(some_event,8)
			these_events(i).s_z_a				= fix(some_event,16)
			these_events(i).d_n_t				= byte(some_event,18)
			these_events(i).x_pixel				= fix(some_event,19)
			these_events(i).y_pixel				= fix(some_event,21)
			these_events(i).raw_radiance			= fix(some_event,23)
			these_events(i).cal_radiance			= float(some_event,25)
			these_events(i).parent_seq			= long(some_event,29)
			these_events(i).parent_rec			= long(some_event,33)
			these_events(i).QA(0)				= fix(some_event,37)
			these_events(i).QA(1)				= fix(some_event,39)
			these_events(i).QA(2)				= fix(some_event,41)
			these_events(i).QA(3)				= fix(some_event,43)
			these_events(i).location(0)			= float(some_event,45)
			these_events(i).location(1)			= float(some_event,49)		
		endfor
		hdf_vd_detach,vd
		hdf_close,hdffile
	endelse
	return
end

; ***********************************************************************************************
; *   			Old OTD to New LISOTD Conversion; 10/30/97 - DJB			*
; ***********************************************************************************************

pro OLDOTD_to_LISOTD, new, old, NO_ONE_SECONDS = no_one_seconds, NO_VIEWTIMES = no_viewtimes, NO_AREAS = no_areas, $
			        NO_FLASHES = no_flashes, NO_GROUPS = no_groups, NO_EVENTS = no_events, status=status, native=native, _EXTRA=e
	this_os = !version.os
	call_nat = keyword_set(native)

	; Set up structure definitions for new style HDF
	
	this_orbit_summary = {an_orbit_summary,id_number:long(0),TAI93_start:double(0),UTC_start:bytarr(28),GPS_start:double(0), $
				TAI93_end:double(0),start_longitude:float(0),end_longitude:float(0),point_data_count:fix(0), $
				point_data_address:fix(0),one_second_count:long(0),one_second_address:long(0),summary_image_count: $
				fix(0),summary_image_address:fix(0),inspection_code:fix(0),configuration_code:fix(0)}
	this_point_summary = {a_point_summary,parent_address:long(0),event_count:long(0),event_address:long(0),group_count:long(0), $
				group_address:long(0),flash_count:long(0),flash_address:long(0),area_count:long(0),area_address:long(0), $
				bg_count:long(0),bg_address:long(0),vt_count:long(0),vt_address:long(0)}
	
	if (call_nat eq 1 and this_os eq 'MacOS') then begin
		this_area = {an_area,TAI93_time:0D,delta_time:0.0,observe_time:0,location:fltarr(2),net_radiance:0.0, $
				footprint:0.0,address:0L,parent_address:0L,child_address:0L,child_count:0L,       $
				grandchild_count:0L,greatgrandchild_count:0L,approx_threshold:byte(0),alert_flag:byte(0),cluster_index: $
				byte(0),density_index:byte(0),noise_index:byte(0),         $
				grouping_status:byte(0)}
	endif else begin
		this_area = {an_area,TAI93_time:double(0),delta_time:float(0),observe_time:fix(0),location:fltarr(2),net_radiance:float(0), $
				footprint:float(0),address:long(0),parent_address:long(0),child_address:long(0),child_count:long(0),       $
				grandchild_count:long(0),greatgrandchild_count:long(0),approx_threshold:byte(0),alert_flag:byte(0),cluster_index: $
				byte(0),density_index:byte(0),noise_index:byte(0),oblong_index:float(0),grouping_sequence:long(0),         $
				grouping_status:byte(0)}
	endelse
	
	if (call_nat eq 1 and this_os eq 'MacOS') then begin
		this_flash = {a_flash, TAI93_time:0D, delta_time:0.0, observe_time:0, location:fltarr(2) ,radiance:0.0, $
				footprint:0.0, address:0L, parent_address:0L, child_address:0L, child_count:0L,      $
				grandchild_count:0L, approx_threshold:byte(0), alert_flag:byte(0), cluster_index:byte(0), density_index: $
				byte(0), noise_index:byte(0), glint_index:0.0, grouping_sequence:0L,       $
				grouping_status:byte(0)}
	endif else begin
		this_flash = {a_flash, TAI93_time:double(0), delta_time:float(0), observe_time:fix(0), location:fltarr(2) ,radiance:float(0), $
				footprint:float(0), address:long(0), parent_address:long(0), child_address:long(0), child_count:long(0),      $
				grandchild_count:long(0), approx_threshold:byte(0), alert_flag:byte(0), cluster_index:byte(0), density_index: $
				byte(0), noise_index:byte(0), glint_index:float(0), oblong_index:float(0), grouping_sequence:long(0),         $
				grouping_status:byte(0)}
	endelse

	if (call_nat eq 1 and this_os eq 'MacOS') then begin
		this_group = {a_group, TAI93_time:double(0), observe_time:fix(0), location:fltarr(2) ,radiance:float(0),                 $
				footprint:float(0), address:long(0), parent_address:long(0), child_address:long(0), child_count:long(0), $
				approx_threshold:byte(0), alert_flag:byte(0), cluster_index:byte(0), density_index:                      $
				byte(0), noise_index:byte(0), glint_index:float(0), grouping_sequence:long(0),    $
				grouping_status:byte(0)}
	endif else begin
		this_group = {a_group, TAI93_time:double(0), observe_time:fix(0), location:fltarr(2) ,radiance:float(0),                 $
				footprint:float(0), address:long(0), parent_address:long(0), child_address:long(0), child_count:long(0), $
				approx_threshold:byte(0), alert_flag:byte(0), cluster_index:byte(0), density_index:                      $
				byte(0), noise_index:byte(0), glint_index:float(0), oblong_index:float(0), grouping_sequence:long(0),    $
				grouping_status:byte(0)}
	endelse

	this_event = {an_event, TAI93_time:double(0), observe_time:fix(0), location:fltarr(2) ,radiance:float(0),       $
			footprint:float(0), address:long(0), parent_address:long(0), x_pixel:byte(0), y_pixel:byte(0),  $
			bg_value:fix(0), bg_radiance:fix(0), amplitude:byte(0), sza_index:byte(0), glint_index:byte(0), $
			approx_threshold:byte(0), alert_flag:byte(0), cluster_index:byte(0), density_index:                      $
			byte(0), noise_index:byte(0), bg_value_flag:byte(0), grouping_sequence:long(0)}
	this_viewtime = {a_viewtime, location:fltarr(2), TAI93_start:long(0), TAI93_end:long(0), effective_obs:float(0), $
				alert_flag:byte(0), approx_threshold:byte(0)}
	this_bg_summary = {a_bg_summary, TAI93_time:double(0),address:long(0),boresight:fltarr(2), corners:fltarr(8)}

	this_one_second = {a_one_second, TAI93_time:double(0), alert_summary:byte(0), 	$
					instrument_alert:byte(0), platform_alert:byte(0),	$
					external_alert:byte(0), processing_alert:byte(0),	$
					position_vector:fltarr(3), velocity_vector:fltarr(3),	$
					transform_matrix:fltarr(9), solar_vector:fltarr(3),	$
					ephemeris_quality_flag:long(0), attitude_quality_flag:long(0), $
					boresight_threshold:byte(0), thresholds:bytarr(16), 	$
					noise_index:byte(0), event_count:intarr(6)}

	this_lightning = {	   area:replicate(this_area,  old.orbit_summary.summary.areas), 	$
				       flash:replicate(this_flash,  old.orbit_summary.summary.flashes),   $
				       group:replicate(this_group,  old.orbit_summary.summary.groups),   $
				       event:replicate(this_event, old.orbit_summary.summary.events)}
	szvt = size(old.flash_density.view_time)
	nvt  = szvt(1)
	szia = size(old.browse.image)
	nia  = szia(1)
	
	this_point = {		point_summary:{a_point_summary},				$ 	
				lightning:replicate(this_lightning,1),					$
				viewtime:replicate(this_viewtime, nvt), 				$
				bg_summary:replicate(this_bg_summary,nia)}
	new = {			orbit_summary:{an_orbit_summary},				$
				point:replicate(this_point,1),						$
				one_second:replicate(this_one_second,6100)}
	
	; Populate orbit_summary
	
	new.orbit_summary.id_number		=	old.orbit.orbit_ID
	new.orbit_summary.TAI93_start		= 	old.orbit.TAI93_start
	dt = new_datetime_structure(1)
	dt.tai93 = new.orbit_summary.TAI93_start
	calc_datetime,dt
	new.orbit_summary.UTC_start = byte(string(format=$
		'((I4),"-",(I2.2),"-",(I2.2),"T",(I2.2),":",(I2.2),":",(F9.6),"Z")', $
		dt.utc.year,dt.utc.month,dt.utc.day,dt.utc.hour,dt.utc.minute,dt.utc.second))
	new.orbit_summary.GPS_start = dt.GPS
	new.orbit_summary.TAI93_end		= 	old.orbit.TAI93_end
	new.orbit_summary.start_longitude	= 	old.orbit.start_location(1)
	new.orbit_summary.end_longitude		=	old.orbit.end_location(1)
	new.orbit_summary.point_data_count	=	1
	new.orbit_summary.point_data_address	= 	1
	new.orbit_summary.one_second_count	=	6100
	new.orbit_summary.one_second_address	= 	1
	new.orbit_summary.summary_image_count	=	0
	new.orbit_summary.summary_image_address =	0
	; Inspection code NYI
	; Configuration code NYI
	
	; Populate point_summary
	
	new.point.point_summary.parent_address	=	1
	new.point.point_summary.event_count		= 	old.orbit_summary.summary.events
	new.point.point_summary.event_address		=	1
	new.point.point_summary.group_count		=	old.orbit_summary.summary.groups
	new.point.point_summary.group_address		=	1
	new.point.point_summary.flash_count		=	old.orbit_summary.summary.flashes
	new.point.point_summary.flash_address		=	1
	new.point.point_summary.area_count		=	old.orbit_summary.summary.areas
	new.point.point_summary.area_address		=	1
	new.point.point_summary.bg_count		=	nia
	new.point.point_summary.bg_address		=	1
	new.point.point_summary.vt_count		=	nvt
	new.point.point_summary.vt_address		=	1
	
	; Populate areas
	
	new.point.lightning.area(*).TAI93_time		= old.area.area(*).TAI93
	new.point.lightning.area(*).delta_time		= old.area.area(*).delta
	new.point.lightning.area(*).observe_time	= old.area.area(*).view
	new.point.lightning.area(*).location		= old.area.area(*).cent
	new.point.lightning.area(*).net_radiance	= old.area.area(*).rad
	; .footprint NYI
	new.point.lightning.area(*).address		= old.area.area(*).seq
	new.point.lightning.area(*).parent_address	= new.orbit_summary.id_number
	new.point.lightning.area(*).child_address	= old.area.area(*).child_rec
	new.point.lightning.area(*).child_count		= old.area.area(*).children
	; .grandchild_count is set below
	; .greatgrandchild_count is set below
	; .approx_threshold is set below
	; .alert_flag is set below
	new.point.lightning.area(*).cluster_index = old.area.area(*).QA(0)
	sat = where(old.area.area(*).QA(3) gt 255,satct)
	if (satct gt 0) then begin
		old.area.area(sat).QA(3) = 255
	endif
	new.point.lightning.area(*).density_index = old.area.area(*).QA(3)
	new.point.lightning.area(*).noise_index = old.area.area(*).QA(2)/10
	; .oblong_index NYI
	if (call_nat eq 1 and this_os eq 'MacOS') then begin
	endif else begin
		new.point.lightning.area(*).grouping_sequence	= old.area.area(*).seq
	endelse
	new.point.lightning.area(*).grouping_status	= old.area.area(*).end_status
	
	; Populate flashes
	
	new.point.lightning.flash(*).TAI93_time		= old.flash.flash(*).TAI93
	new.point.lightning.flash(*).delta_time		= old.flash.flash(*).delta
	new.point.lightning.flash(*).observe_time	= old.flash.flash(*).view
	new.point.lightning.flash(*).location		= old.flash.flash(*).cent
	new.point.lightning.flash(*).radiance		= old.flash.flash(*).rad
	; .footprint NYI
	new.point.lightning.flash(*).address		= old.flash.flash(*).seq
	new.point.lightning.flash(*).parent_address	= old.flash.flash(*).parent_rec
	new.point.lightning.flash(*).child_address	= old.flash.flash(*).child_rec
	new.point.lightning.flash(*).child_count	= old.flash.flash(*).children
	; .grandchild_count is set below
	; .approx_threshold is set below
	; .alert_flag is set below
	new.point.lightning.flash(*).cluster_index = old.flash.flash(*).QA(0)
	sat = where(old.flash.flash(*).QA(3) gt 255,satct)
	if (satct gt 0) then begin
		old.flash.flash(sat).QA(3) = 255
	endif
	new.point.lightning.flash(*).density_index = old.flash.flash(*).QA(3)
	new.point.lightning.flash(*).noise_index = old.flash.flash(*).QA(2)/10
	; .glint_index NYI
	; .oblong_index NYI
	if (call_nat eq 1 and this_os eq 'MacOS') then begin
	endif else begin
		new.point.lightning.flash(*).grouping_sequence	= old.flash.flash(*).seq
	endelse
	new.point.lightning.flash(*).grouping_status	= old.flash.flash(*).end_status
		
	; Populate groups
	
	new.point.lightning.group(*).TAI93_time		= old.group.group(*).TAI93
	new.point.lightning.group(*).observe_time	= old.group.group(*).view
	new.point.lightning.group(*).location		= old.group.group(*).cent
	new.point.lightning.group(*).radiance		= old.group.group(*).rad
	; .footprint NYI
	new.point.lightning.group(*).address		= old.group.group(*).seq
	new.point.lightning.group(*).parent_address	= old.group.group(*).parent_rec
	new.point.lightning.group(*).child_address	= old.group.group(*).child_rec
	new.point.lightning.group(*).child_count	= old.group.group(*).children
	; .approx_threshold is set below
	; .alert_flag is set below
	new.point.lightning.group(*).cluster_index = old.group.group(*).QA(0)
	sat = where(old.group.group(*).QA(3) gt 255,satct)
	if (satct gt 0) then begin
		old.group.group(sat).QA(3) = 255
	endif
	new.point.lightning.group(*).density_index = old.group.group(*).QA(3)
	new.point.lightning.group(*).noise_index = old.group.group(*).QA(2)/10
	; .glint_index NYI
	; .oblong_index NYI
	if (call_nat eq 1 and this_os eq 'MacOS') then begin
	endif else begin
		new.point.lightning.group(*).grouping_sequence	= old.group.group(*).seq
	endelse
	new.point.lightning.group(*).grouping_status	= old.group.group(*).end_status

	Widget_Control, status, Get_Value=msg
	msg = msg + '.'
	Widget_Control, status, Set_Value=msg
	
	; Populate events
	
	new.point.lightning.event(*).TAI93_time		= old.event.event(*).TAI93
	; .observe_time is set below
	new.point.lightning.event(*).location		= old.event.event(*).location
	new.point.lightning.event(*).radiance		= old.event.event(*).cal_radiance
	; .footprint NYI
	new.point.lightning.event(*).address		= old.event.event(*).event_num
	new.point.lightning.event(*).parent_address	= old.event.event(*).parent_rec
	new.point.lightning.event(*).x_pixel		= old.event.event(*).x_pixel
	new.point.lightning.event(*).y_pixel		= old.event.event(*).y_pixel
	; .bg_value NYI
	; .bg_radiance NYI
	new.point.lightning.event(*).amplitude		= old.event.event(*).raw_radiance
	new.point.lightning.event(*).sza_index		= byte(old.event.event(*).s_z_a)
	new.point.lightning.event(*).glint_index	= old.event.event(*).QA(1)	
	; .approx_threshold is set below
	; .alert_flag is set below
	new.point.lightning.event(*).cluster_index = old.event.event(*).QA(0)
 	sat = where(old.event.event(*).QA(3) gt 255,satct)
	if (satct gt 0) then begin
		old.event.event(sat).QA(3) = 255
	endif
	new.point.lightning.event(*).density_index = old.event.event(*).QA(3)
	new.point.lightning.event(*).noise_index = old.event.event(*).QA(2)/10
	; .bg_value_flag NYI
	new.point.lightning.event(*).grouping_sequence	= old.event.event(*).seq_num
	new.point.lightning.event(*).observe_time	= new.point.lightning.group(new.point.lightning.event(*).parent_address).observe_time

	Widget_Control, status, Get_Value=msg
	msg = msg + '.'
	Widget_Control, status, Set_Value=msg
		
	; Populate view_times
	
	new.point.viewtime(*).location			= old.flash_density.view_time(*).grid_point
	new.point.viewtime(*).TAI93_start		= long(old.flash_density.view_time(*).TAI93_start)
	new.point.viewtime(*).TAI93_end			= long(old.flash_density.view_time(*).TAI93_start + $
 							       old.flash_density.view_time(*).duration)
	new.point.viewtime(*).effective_obs		= float(old.flash_density.view_time(*).duration)
	; .alert_flag is set below
	; .approx_threshold is set below
	
	Widget_Control, status, Get_Value=msg
	msg = msg + '.'
	Widget_Control, status, Set_Value=msg

	; Populate bg_summarys
	
	new.point.bg_summary(*).TAI93_time		= old.browse.image(*).TAI93
	new.point.bg_summary(*).address			= old.browse.image(*).image_pointer
	new.point.bg_summary(*).boresight		= old.browse.image(*).nadir_location
	new.point.bg_summary(*).corners(0:1)		= old.browse.image(*).NW_location
	new.point.bg_summary(*).corners(2:3)		= old.browse.image(*).NE_location
	new.point.bg_summary(*).corners(4:5)		= old.browse.image(*).SW_location
	new.point.bg_summary(*).corners(6:7)		= old.browse.image(*).SE_location
	
	; Populate one_seconds
	
	if (keyword_set(no_one_seconds)) then begin
		print,'   Note - one-second data has not been read; flags, event rates, ephemeris data unavailable'
	endif else begin
		
		; Arbitrary subsampling of the 19 old event rate arrays
		new.one_second(*).event_count(0)		= old.orbit_summary.rate_e(1).count
		new.one_second(*).event_count(1)		= old.orbit_summary.rate_e(3).count
		new.one_second(*).event_count(2)		= old.orbit_summary.rate_e(5).count
		new.one_second(*).event_count(3)		= old.orbit_summary.rate_e(7).count
		new.one_second(*).event_count(4)		= old.orbit_summary.rate_e(10).count
		new.one_second(*).event_count(5)		= old.orbit_summary.rate_e(12).count
		Widget_Control, status, Get_Value=msg
		msg = msg + '.'
		Widget_Control, status, Set_Value=msg

		;   Populate the nav elements of the one second data
		;   OTD ephems are not guaranteed to match the one-second array times, or even live within
		;	their temporal bounds.  Insert them as needed, leaving unrecorded ones blank
		
		ephsize = size(old.orbit_summary.ephemeris)
		ephlen = ephsize(1)
		os_start = long(old.orbit.TAI93_start)
		os_end   = long(old.orbit.TAI93_end)
		
		badephems = 0	
		for i=long(0), long(ephlen-1), long(1) do begin
			ephtaitime = long(old.orbit_summary.ephemeris(i).GPS_time)  ; This is not really GPS time
			ephidx = long(ephtaitime-os_start)
			if (ephidx ge 0 and ephidx lt 6100) then begin
				new.one_second(ephidx).position_vector(0:2) = old.orbit_summary.ephemeris(i).spice_kernel(0:2)
				new.one_second(ephidx).velocity_vector(0:2) = old.orbit_summary.ephemeris(i).spice_kernel(3:5)
				new.one_second(ephidx).transform_matrix(0:8) = old.orbit_summary.ephemeris(i).spice_kernel(6:14)
			endif else begin
				badephems = badephems+1
			endelse
		endfor
		if (badephems gt 0) then begin
			print,'         *** There were ',badephems,' ephemeris entries outside the time bounds of this orbit'
		endif
		indices = lindgen(6100)
		new.one_second(*).TAI93_time = os_start+double(indices)
		
		; Populate FIFO full flags
		
		badFIFO = where((old.orbit_summary.rate_e(16).count and 1) eq 1,count)
		if (count ne 0) then begin
			new.one_second(badFIFO).instrument_alert = new.one_second(badFIFO).instrument_alert + 4
			print,count,' FIFO full flags were set'
		endif
		
		; Populate Instrument Out of Control flags
		
		badOOC = where((old.orbit_summary.rate_e(16).count and 2) eq 1,count)
		if (count ne 0) then begin
			new.one_second(badOOC).platform_alert = new.one_second(badOOC).platform_alert + 4 + 16
			print,count,' Instrument out of control flags were set'
		endif
		
		; Populate No Ephemeris flags
		
		badNE = where( ( (old.orbit_summary.rate_e(16).count and 4) eq 1) or $
			((new.one_second(*).position_vector(0) eq 0) and (new.one_second(*).position_vector(1) eq 0)),count)
		if (count ne 0) then begin
			new.one_second(badNE).platform_alert = new.one_second(badNE).platform_alert + 2
			print,count,' No ephemeris flags were set or inferred'
		endif
		
		; Populate Warmup flags
		
		badW = where((old.orbit_summary.rate_e(16).count and 8) eq 1,count)
		if (count ne 0) then begin
			new.one_second(badW).instrument_alert = new.one_second(badW).instrument_alert + 16
			print,count,' Warmup flags were set'
		endif
		
		; Populate Connector flags
		
		badC = where((old.orbit_summary.rate_e(16).count and 16) eq 1,count)
		if (count ne 0) then begin
			new.one_second(badC).instrument_alert = new.one_second(badC).instrument_alert + 32
			print,count,' Connector problem flags wre set'
		endif
		
		print,'         ----->'

		Widget_Control, status, Get_Value=msg
		msg = msg + '.'
		Widget_Control, status, Set_Value=msg

		; Populate alert_summary flag
		
		FatalInst = where( ( (new.one_second.instrument_alert and 1)) or $
		                   ( (new.one_second.instrument_alert and 2)) or $
		                   ( (new.one_second.instrument_alert and 4)) or $
		                   ( (new.one_second.instrument_alert and 16)) or $
		                   ( (new.one_second.instrument_alert and 32)) or $
		                   ( (new.one_second.instrument_alert and 64)),count)
		if (count ne 0) then begin
			new.one_second(FatalInst).alert_summary = new.one_second(FatalInst).alert_summary + 1
			print,count,' Instrument/Fatal bits were set'
		endif
		
		WarnInst  = where( ( (new.one_second.instrument_alert and 4)) or $
				   ( (new.one_second.instrument_alert and 8)) or $
				   ( (new.one_second.instrument_alert and 16)) or $
				   ( (new.one_second.instrument_alert and 32)),count)
		if (count ne 0) then begin
			new.one_second(WarnInst).alert_summary = new.one_second(WarnInst).alert_summary + 2
			print,count,' Instrument/Warning bits were set'
		endif
		
		FatalPlat = where( ( (new.one_second.platform_alert and 2)) or $
				   ( (new.one_second.platform_alert and 8)) or $
				   ( (new.one_second.platform_alert and 32)) or $
				   ( (new.one_second.platform_alert and 128)),count) 
		if (count ne 0) then begin
			new.one_second(FatalPlat).alert_summary = new.one_second(FatalPlat).alert_summary + 4
			print,count,' Platform/Fatal bits were set'
		endif
		
		WarnPlat  = where( ( (new.one_second.platform_alert and 1)) or $
				   ( (new.one_second.platform_alert and 4)) or $
				   ( (new.one_second.platform_alert and 16)) or $
				   ( (new.one_second.platform_alert and 64)),count)
		if (count ne 0) then begin
			new.one_second(WarnPlat).alert_summary = new.one_second(WarnPlat).alert_summary + 8
			print,count,' Platform/Warning bits were set'
		endif
		
		; No Fatal External flags defined yet
		
		WarnExt   = where( ( (new.one_second.external_alert and 1)) or $
				   ( (new.one_second.external_alert and 2)) or $
				   ( (new.one_second.external_alert and 4)),count)
		if (count ne 0) then begin
			new.one_second(WarnExt).alert_summary = new.one_second(WarnExt).alert_summary + 32
			print,count,' External/Warning bits were set'
		endif
		
		FatalProc = where( ( (new.one_second.processing_alert and 2)) or $
				   ( (new.one_second.processing_alert and 4)) or $
				   ( (new.one_second.processing_alert and 8)) or $
				   ( (new.one_second.processing_alert and 16)),count)
		if (count ne 0) then begin
			new.one_second(FatalProc).alert_summary = new.one_second(FatalProc).alert_summary + 64
			print,count,' Processing/Fatal bits were set'
		endif
		
		WarnProc  = where( ( (new.one_second.processing_alert and 1)) or $
				   ( (new.one_second.processing_alert and 16)) or $
				   ( (new.one_second.processing_alert and 32)) or $
				   ( (new.one_second.processing_alert and 64)),count)
		if (count ne 0) then begin
			new.one_second(WarnProc).alert_summary = new.one_second(WarnProc).alert_summary + 128
			print,count,' Processing/Warning bits were set'
		endif
	
		; Now populate the alert summary into relevant point data
		
		if (not keyword_set(no_areas)) then begin
			indices = long(new.point.lightning.area(*).TAI93_time - new.orbit_summary.TAI93_start)
			goodones = where(indices gt 0,goodct)
			if (goodct gt 0) then begin
			new.point.lightning.area(goodones).alert_flag = new.one_second(indices[goodones]).alert_summary
			endif
		endif
		if (not keyword_set(no_flashes)) then begin
			indices = long(new.point.lightning.flash(*).TAI93_time - new.orbit_summary.TAI93_start)
			goodones = where(indices gt 0, goodct)
			if (goodct gt 0) then begin
			new.point.lightning.flash(goodones).alert_flag = new.one_second(indices[goodones]).alert_summary
			endif
		endif
		if (not keyword_set(no_groups)) then begin
			indices = long(new.point.lightning.group(*).TAI93_time - new.orbit_summary.TAI93_start)
			goodones = where(indices gt 0, goodct)
			if (goodct gt 0) then begin
			new.point.lightning.group(goodones).alert_flag = new.one_second(indices[goodones]).alert_summary
			endif
		endif
		if (not keyword_set(no_events)) then begin
			indices = long(new.point.lightning.event(*).TAI93_time - new.orbit_summary.TAI93_start)
			goodones = where(indices gt 0, goodct)
			if (goodct gt 0) then begin
			new.point.lightning.event(goodones).alert_flag = new.one_second(indices[goodones]).alert_summary
			endif
		endif
		if (not keyword_set(no_viewtimes)) then begin
			indices = long(new.point.viewtime(*).TAI93_start - new.orbit_summary.TAI93_start)
			indices2= long(new.point.viewtime(*).TAI93_end - new.orbit_summary.TAI93_end)
			for i = long(0), long(new.point.point_summary.vt_count-1), long(1) do begin
				mask = 0
				for j = indices(i), indices2(i), 1 do begin
					mask = mask or new.one_second(j).alert_summary
				endfor
				new.point.viewtime(i).alert_flag = mask
			endfor
		endif
		
	endelse

	; Clean up some; populate the chillun's
	
	if (not keyword_set(no_flashes)) then begin
		for fidx=0L, new.point.point_summary.flash_count-1L, 1L do begin
			gstart = new.point.lightning.flash(fidx).child_address
			ect = 0L
			if ((not keyword_set(no_events)) and (not keyword_set(no_groups))) then begin
				for gidx=0L, new.point.lightning.flash(fidx).child_count-1L,1L do begin
					ect = ect + new.point.lightning.group(gstart+gidx).child_count
				endfor
		     endif
			new.point.lightning.flash(fidx).grandchild_count = ect
		endfor
	endif
	
	if (not keyword_set(no_areas)) then begin
		for aidx=0L, new.point.point_summary.area_count-1L,1L do begin
			fstart = new.point.lightning.area(aidx).child_address
			gct = 0L
			ect = 0L
			if (not keyword_set(no_flashes)) then begin
				for fidx=0L, new.point.lightning.area(aidx).child_count-1L,1L do begin
					gct = gct + new.point.lightning.flash(fstart+fidx).child_count
					ect = ect + new.point.lightning.flash(fstart+fidx).grandchild_count
				endfor
			endif
			new.point.lightning.area(aidx).grandchild_count = gct
			new.point.lightning.area(aidx).greatgrandchild_count = ect
		endfor
	endif
	
	Widget_Control, status, Get_Value=msg
	msg = msg + '.'
	Widget_Control, status, Set_Value=msg
	
	; Set thresholds based on known update history
	
	thr_changes_TAI93 = [70848002.0D, 76876442.0D, 77155202.0D, 80392502.0D, 120199443.0D]
	thr_settings      = [14, 20, 18, 17, 15]
	
	xx = where(thr_changes_TAI93 le new.orbit_summary.TAI93_start)
	xidx = max(xx)
	thresh8 = thr_settings(xidx)
	new.point.lightning.area(*).approx_threshold = thresh8
	new.point.lightning.flash(*).approx_threshold = thresh8
	new.point.lightning.group(*).approx_threshold = thresh8
	new.point.lightning.event(*).approx_threshold = thresh8

	Widget_Control, status, Get_Value=msg
	msg = msg + '.'
	Widget_Control, status, Set_Value=msg

	new.point.viewtime(*).approx_threshold = thresh8
	Widget_Control, status, Get_Value=msg
	msg = msg + '.'

	Widget_Control, status, Set_Value=msg

	new.one_second(*).thresholds = replicate(thresh8,12)

	Widget_Control, status, Get_Value=msg
	msg = msg + '.'
	Widget_Control, status, Set_Value=msg
		
	return
end

function otd_qa,fname,databasename

	retval = intarr(5)
	
	; Load the OTD QA database
	numindatabase = 2220
	qadata = intarr(8,numindatabase)
	openr,lun,databasename,/get_lun
	readf,lun,qadata
	close,lun,/all
	free_lun,lun
	
	; Parse the OTD QA name ... extract year, jday, orbit number
	
	; Pick the right directory separator
	if (!version.os_family eq 'MacOS') then begin
		seperator = 58
	endif else begin 
		if (!version.os_family eq 'unix') then begin
			seperator =47
		endif else begin
			seperator = 92
		endelse
	endelse
	
	; Strip off the path info
	fnameb = byte(fname)
	dirmarks = max(where(fnameb eq seperator))
	fnamesz = size(fnameb)
	fnamelen = fnamesz(1)
	if (dirmarks eq -1) then begin
		fnamestub = fname
	endif else begin
		fnamebstub = fnameb(dirmarks+1:fnamelen-1)
		fnamestub = string(fnamebstub)
	endelse
	
	bfnamestub = byte(fnamestub)
	bfnamestubsz = size(bfnamestub)
	bfnamestublen = bfnamestubsz(1)
	if (bfnamestublen ne 26) then begin
		retval(*) = -1
		stop
		return,retval
	endif
	
	byr = bfnamestub(15:16)
	bdy = bfnamestub(18:20)
	bor = bfnamestub(22:25)
	
	yr = fix(string(byr))
	dy = fix(string(bdy))
	orb = fix(string(bor))
	
	years = qadata(0,*)
	days = qadata(1,*)
	orbits = qadata(2,*)
	
	indatabase = where((yr eq years) and (dy eq days) and (orb eq orbits),indatabasect)
	
	if (indatabasect le 0) then begin
		retval(*) = 0
		return,retval
	endif else begin
		for i = 0, indatabasect-1, 1 do begin
			retval(*) = retval(*) + qadata(3:7,indatabase(i))
		endfor
		return,retval
	endelse

end

function make_readstring,fname,sensor
	common prefs, readparams, readparamsotd, setparams, setparamsotd, gotparams, gotparamsotd
	common lisapp_prefs, app_prefs

	if (app_prefs.shlb.use_shlb eq 1) then begin
		shlbname=strip_132(app_prefs.shlb.shlb)
		use_shlb = 1
	endif else begin
		use_shlb = 0
	endelse 
	
	if (sensor eq 'lis') then begin
		readstring = 'read_orbit,"'+fname+'",orbit'
		useparams = readparams
	endif else begin
		readstring = 'read_old_orbit,"'+fname+'",orbit,old_orbit'
		useparams = readparamsotd
	endelse
	
	if (useparams(0) ne 1) then readstring = readstring + ',/no_areas'
	if (useparams(1) ne 1) then readstring = readstring + ',/no_flashes'
	if (useparams(2) ne 1) then readstring = readstring + ',/no_groups'
	if (useparams(3) ne 1) then readstring = readstring + ',/no_events'
	if (useparams(4) ne 1) then readstring = readstring + ',/no_viewtimes'
	if (useparams(5) ne 1) then readstring = readstring + ',/no_one_seconds'
	if (use_shlb) then begin
		readstring = readstring+',native="'+shlbname+'"'
	endif
	
;	print, readstring
	
	return,readstring
end

pro add_vdata, orb, filename, which_vdata
	common prefs, readparams, readparamsotd, setparams, setparamsotd, gotparams, gotparamsotd

	close,/all
	which_sensor,filename,sensor
	templis = readparams & tempotd = readparamsotd
	readparams[*] = 0 & readparamsotd[*] = 0
	
	case which_vdata of
		'area':begin
				readparams[0] = 1 & readparamsotd[0] = 1
				rs = make_readstring(filename,sensor)
				res = execute(rs)
;				READ_orbit,filename,partial_orb,status=status,/no_flashes,/no_groups,/no_events,/no_viewtimes,/no_one_seconds
			  end
		'flash':begin
				readparams[1] = 1 & readparamsotd[1] = 1
				rs = make_readstring(filename,sensor)
				res = execute(rs)
;				READ_orbit,filename,partial_orb,status=status,/no_areas,/no_groups,/no_events,/no_viewtimes,/no_one_seconds
			  end
		'group':begin
				readparams[2] = 1 & readparamsotd[2] = 1
				rs = make_readstring(filename,sensor)
				res = execute(rs)
;				READ_orbit,filename,partial_orb,status=status,/no_areas,/no_flashes,/no_events,/no_viewtimes,/no_one_seconds
			  end
		'event':begin
				readparams[3] = 1 & readparamsotd[3] = 1
				rs = make_readstring(filename,sensor)
				res = execute(rs)
;				READ_orbit,filename,partial_orb,status=status,/no_areas,/no_flashes,/no_groups,/no_viewtimes,/no_one_seconds
			   end
		'viewtime':begin
				readparams[4] = 1 & readparamsotd[4] = 1
				rs = make_readstring(filename,sensor)
				res = execute(rs)
;				READ_orbit,filename,partial_orb,status=status,/no_areas,/no_flashes,/no_groups,/no_events,/no_one_seconds
			   end
		'one_second':begin
				readparams[5] = 1 & readparamsotd[5] = 1
				rs = make_readstring(filename,sensor)
				res = execute(rs)
;				READ_orbit,filename,partial_orb,status=status,/no_areas,/no_flashes,/no_groups,/no_events,/no_viewtimes
			   end
	endcase
	
	readparams = templis & readparamsotd = tempotd
	partial_orb = orbit
	
	; Recreate lightning vgroup
	
	case which_vdata of
		'one_second':begin
					lightning = orb.point.lightning
					num_areas = orb.point.point_summary.area_count
					num_flashes = orb.point.point_summary.flash_count
					num_groups = orb.point.point_summary.group_count
					num_events = orb.point.point_summary.event_count
				  end
		'viewtime':  begin
					lightning = orb.point.lightning
					num_areas = orb.point.point_summary.area_count
					num_flashes = orb.point.point_summary.flash_count
					num_groups = orb.point.point_summary.group_count
					num_events = orb.point.point_summary.event_count
				  end
		'area':      begin
					num_areas = partial_orb.point.point_summary.area_count
					num_flashes = orb.point.point_summary.flash_count
					num_groups = orb.point.point_summary.group_count
					num_events = orb.point.point_summary.event_count
					lightning = {area:replicate(partial_orb.point.lightning.area(0),num_areas), $
							  flash:replicate(orb.point.lightning.flash(0),num_flashes), $
							  group:replicate(orb.point.lightning.group(0),num_groups), $
							  event:replicate(orb.point.lightning.event(0),num_events)}
					lightning.area = partial_orb.point.lightning.area
					lightning.flash = orb.point.lightning.flash
					lightning.group = orb.point.lightning.group
					lightning.event = orb.point.lightning.event
				   end
		'flash':	   begin
					num_areas = orb.point.point_summary.area_count
					num_flashes = partial_orb.point.point_summary.flash_count
					num_groups = orb.point.point_summary.group_count
					num_events = orb.point.point_summary.event_count
					lightning = {area:replicate(orb.point.lightning.area(0),num_areas),$
							  flash:replicate(partial_orb.point.lightning.flash(0),num_flashes),$
							  group:replicate(orb.point.lightning.group(0),num_groups),$
							  event:replicate(orb.point.lightning.event(0),num_events)}
					lightning.area = orb.point.lightning.area
					lightning.flash = partial_orb.point.lightning.flash
					lightning.group = orb.point.lightning.group
					lightning.event = orb.point.lightning.event		
				   end
		'group':	   begin
					num_areas = orb.point.point_summary.area_count
					num_flashes = orb.point.point_summary.flash_count
					num_groups = partial_orb.point.point_summary.group_count
					num_events = orb.point.point_summary.event_count
					lightning = {area:replicate(orb.point.lightning.area(0),num_areas),$
							  flash:replicate(orb.point.lightning.flash(0),num_flashes),$
							  group:replicate(partial_orb.point.lightning.group(0),num_groups),$
							  event:replicate(orb.point.lightning.event(0),num_events)}
					lightning.area = orb.point.lightning.area
					lightning.flash = orb.point.lightning.flash
					lightning.group = partial_orb.point.lightning.group
					lightning.event = orb.point.lightning.event		
				   end
		'event':	   begin
					num_areas = orb.point.point_summary.area_count
					num_flashes = orb.point.point_summary.flash_count
					num_groups = orb.point.point_summary.group_count
					num_events = partial_orb.point.point_summary.event_count
					lightning = {area:replicate(orb.point.lightning.area(0),num_areas),$
							  flash:replicate(orb.point.lightning.flash(0),num_flashes),$
							  group:replicate(orb.point.lightning.group(0),num_groups),$
							  event:replicate(partial_orb.point.lightning.event(0),num_events)}
					lightning.area = orb.point.lightning.area
					lightning.flash = orb.point.lightning.flash
					lightning.group = orb.point.lightning.group
					lightning.event = partial_orb.point.lightning.event		
				   end
	endcase

	; Other point data
	
	num_bgs = orb.point.point_summary.bg_count
	
	if (which_vdata eq 'viewtime') then  begin
		num_vts = partial_orb.point.point_summary.vt_count
		viewtime = partial_orb.point.viewtime		
	endif else begin
		num_vts = orb.point.point_summary.vt_count
		viewtime = orb.point.viewtime
	endelse

	point_summary = orb.point.point_summary
	point_summary.area_count = num_areas
	point_summary.flash_count= num_flashes
	point_summary.group_count= num_groups
	point_summary.event_count= num_events
	point_summary.vt_count   = num_vts
	point_summary.bg_count   = num_bgs
	
	bg_summary = orb.point.bg_summary
	
	point = {		point_summary:{a_point_summary},				$ 	
				lightning:replicate(lightning,1),				$
				viewtime:replicate(viewtime(0), point_summary.vt_count), 	$
				bg_summary:replicate(bg_summary(0),point_summary.bg_count)}
				
	point.point_summary 	= point_summary
	point.lightning 	= lightning
	point.viewtime 		= viewtime
	point.bg_summary	= bg_summary
	
	; One second
	
	orbit_summary = orb.orbit_summary
	
	if (which_vdata eq 'one_second') then begin
		num_onesec = partial_orb.orbit_summary.one_second_count
		one_second = partial_orb.one_second
	endif else begin
		num_onesec = orb.orbit_summary.one_second_count
		one_second = orb.one_second
	endelse

	orbit_summary.one_second_count = num_onesec

	partial_orb = 0

	new_orb = {	orbit_summary:{an_orbit_summary},			$
				point:replicate(point,1),					$
				one_second:replicate(one_second(0),orbit_summary.one_second_count)}

	new_orb.orbit_summary 	= orbit_summary
	new_orb.point 			= point
	new_orb.one_second		= one_second
	
	orb = new_orb
	
	new_orb = 0
	point = 0
	lightning = 0
	viewtime = 0
			   
end


;+  READ_ORBIT; READ_OLD_ORBIT; WRITE_ORBIT
;  --------------------------------------------------------------------------------
;  
;  The READ_ORBIT procedure loads all data in a LISOTD HDF file into a single
;  variable, an IDL "structure of structures".  The data hierarchy of the
;  HDF file is mirrored almost exactly in memory.  HDF Vgroups such as "Orbit",
;  "Point" and "Lightning" are elements of the parent structure, and are 
;  anonymous IDL structures.  HDF Vdata such as "Area", "Flash", etc. are named 
;  IDL structures.
;  
;  The READ_OLD_ORBIT procedure loads all data from an old OTD HDF file into a
;  new format LISOTD structure.  The old-format structure may also be returned.
;  
;  The WRITE_ORBIT procedure writes LISOTD IDL structures to LISOTD-format HDF
;  files.  Note that this may be used to rewrite old OTD files in new LISOTD
;  format, or to "edit" a LISOTD HDF file.
;     
;  Calling Sequence
;  
;  	READ_ORBIT, FILENAME, LIS_OTD_ORBIT_VAR
;
;  	READ_OLD_ORBIT, FILENAME, LIS_OTD_ORBIT_VAR [, OLD_OTD_ORBIT_VAR]
;
;  	WRITE_ORBIT, FILENAME, LIS_OTD_ORBIT_VAR
;  	
;  Arguments
;  
;  	FILENAME
;  	Character string containing the HDF file name.
;  	
;  	LIS_OTD_ORBIT_VAR
;  	Name of a new IDL structure variable containing the HDF data.
;  	
;  	OLD_OTD_ORBIT_VAR
;  	Optional IDL structure variable containing old OTD data in its 
;  	original (messy) HDF structure.
;  	
;  Keywords
;  
;  	/NO_VIEWTIMES
;  	Do not read in viewtime granules upon input; this saves input time.
;  	(Read routines only)
;  	
;  	/NO_ONE_SECONDS
;  	Do not read in one second array data upon input; this saves input time.
;  	(Read routines only)
;  	
;  	/NO_AREAS
;  	/NO_FLASHES
;  	/NO_GROUPS
;  	/NO_EVENTS
;  	Do not read in the relevant (area,flash,group,event) point data; 
;  		this saves input time.
;  	(Read routines only)
;  	
;  Example
;  
;  	Read all the data in file "trmm.test.hdf" into the variable ORBIT:
;  	
;  	READ_ORBIT,'trmm.test.hdf',ORBIT
;  	
;  	Examine the data structure:
;  	
;  	HELP, /STRUCTURES, ORBIT			(the orbit vgroup)
;  	HELP, /STRUCTURES, ORBIT.POINT		        (the point vgroup)
;  	HELP, /STRUCTURES, ORBIT.POINT.LIGHTNING	(the point vgroup)
;  	HELP, /STRUCTURES, ORBIT.POINT.LIGHTNING.AREA	(the area  vdata)
;  	
;  	Plot the alert summary of this file:
;  	
;  	WINDOW, 1, XSIZE=600, YSIZE=300, RETAIN=2
;  	PLOT, ORBIT.ONE_SECOND(*).ALERT_SUMMARY, PSYM=3
;  	
;  	Show all pixels hit during this orbit:
;  	
;  	WINDOW, 2, XSIZE=256, YSIZE=256, RETAIN=2
;  	PLOT, ORBIT.POINT.LIGHTNING.EVENT(*).X_PIXEL, $
;  	      ORBIT.POINT.LIGHTNING.EVENT(*).Y_PIXEL, $
;  	      XRANGE=[0,127], YRANGE=[0,127],/XSTYLE,/YSTYLE, PSYM=3
;  	      
;  	Read the data in file "trmm.test.hdf" into the variable ORBIT, 
;  	ignoring viewtime granules:
;  	
;  	READ_ORBIT,'trmm.test.hdf',ORBIT,/NO_VIEWTIMES
;       
;	Read the old OTD data in file "otd.test.hdf" into a new LISOTD
;	style structure.  Retain the old structure for kicks.  Ignore 
;	event data:
;
;	READ_OLD_ORBIT,'otd.test.hdf',NEW,OLD,/NO_EVENTS
;
;	Write this data back out in LISHDF format:
;
;	WRITE_ORBIT,'newotd.test.hdf',NEW
;  	
;  Notes
;  
;  	Note that the original HDF format of OTD and LISOTD files uses
;  	HDF 4.0r1 and 4.1r1.  However, WRITE_ORBIT will create files
;  	of type 3.3r4 for IDL 4.x and 4.0r2 for IDL 5.x.  The files are
;  	all backward-compatible and (if unaltered before writing) identical
;  	in data content, but may differ in byte size due to different
;  	housekeeping overhead between the various HDF revisions.
;  	
;  	Note also that IDL does not recognize unsigned data types, and
;  	any vdata elements of type "unsigned" will be changed to "signed"
;  	when WRITE_DATA is used.  This will not affect the content of
;  	the data, merely its tagging in the HDF file.
;-
