/*  Routines to incorporate shared memory between C and IDL
    Reading in the LIS hdf files via C is much faster than
    using the IDL hdf read routines.
    To compile this use: 

	cc -ansiposix -lm -KPIC -c all_struct.c liblis_readhdf.c \
	-I/usr/people/buechler/lis/inc \
	-I/usr1/local/sdptk5.2.1/TOOLKIT/hdf/sgi32/HDF4.1r1/include 

	ld -shared -o all_struct.so all_struct.o liblis_readhdf.o \
	/usr/local/lib/libdf.a \
	/usr/local/lib/libmfhdf.a \
	/usr/local/lib/libdf.a \
	/usr/local/lib/libz.a

    To read one of the structures (event, group, flash, area, one_second, viewtime)
    using IDL, use the following example for flashes:

                this_flash = {a_flash, TAI93_time:double(0), delta_time:float(0), observe_time:fix(0), $
                             location:fltarr(2) ,radiance:float(0), $
                             footprint:float(0), address:long(0), parent_address:long(0), $
                             child_address:long(0), child_count:long(0),      $
                             grandchild_count:long(0), approx_threshold:byte(0), alert_flag:byte(0), $
                             cluster_index:byte(0), density_index: $
                             byte(0), noise_index:byte(0), glint_index:float(0), oblong_index:float(0), $
                             grouping_sequence:long(0), grouping_status:byte(0)}

	sdf = '/usr9/data/12_14/TRMM_LIS_SC.03.2_1997.348.00855'
	flashh = replicate(this_flash,102)
	res=CALL_EXTERNAL('all_struct.so','gflash',sdf,flashh)
     
   The CALL_EXTERNAL call consists of the shared object ('all_struct.so'), the entry
   point name in the shared object ('gflash'), the file name (sdf), and the returned
   structure (flashh).  Note the structure must be allocated in IDL to the correct 
   length before calling CALL_EXTERNAL. 

   The entry points in this program are called:

	gevent                      
	ggroup
	gflash
	garea
	g_one_second
        gviewtime
        

*/
/*
#include <windows.h>
#define __dest_os __win32_os
*/

/*use liblis_readhdf.c for Johnny's lib routines */
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <math.h>
#include <errno.h>
#include "liblisotd_readhdf.h"
#include "liblisotd_err_defs.h"
#include "liblisotd_idl_prototypes.h"


struct lis_area lis_AREA;
struct lis_flash lis_FLASH;
struct lis_group lis_GROUP;
struct lis_event lis_EVENT;
struct lis_viewtime lis_VIEWTIME;
struct lis_one_second lis_ONE_SECOND;

char fname[256];
char timebuf[256];
long lis_area_COUNT(long fid);
long lis_flash_COUNT(long fid);
long lis_group_COUNT(long fid);
long lis_event_COUNT(long fid);
long j,i,lis_areas,lis_flashes,lis_groups,lis_events,lis_bg_summaries;
long lis_viewtimes,lis_one_seconds,fid;

void display_lis_area();
void display_lis_flash(long i,long j,char *c);
void display_lis_group(long i,long j,char *c);
void display_lis_event(long i,long j,char *c);
void display_lis_viewtime();
void display_lis_one_second();

typedef struct {
   unsigned short slen;     /* length of string */
   short stype;             /* type of string */
   char *s;                 /* pointer to character array */
} STRING;

#if defined(MAC) || defined(macintosh)	
main() {return(0);}
#endif

glis_group(argc,argv)
int argc;
void *argv[];

{
        STRING  *string_var;
        char *string2;
        struct lis_group *lis_group_var; /* declare flash structure pointer */

        string_var = (STRING *) argv[0];
        lis_group_var = (struct lis_group *) argv[1];

        string2 = strcpy(fname,string_var->s); 

        fid = OPEN_lis_orbit_hdf(fname);  
                        if((fid<0)&&((fid&0xffffff00)!=lis_NoMetadata)){
                                printf("OPEN_lis_orbit_hdf failed group %s : %08x\n",fname,fid);
       							if(CLOSE_lis_orbit_hdf(fid)!=lis_operation_SUCCESSFUL )printf("close %s file failed\n",fname);
                                return(0); 
                                /* fid = fid&0xff; */
                        }
                        if(((fid&0xffffff00)==lis_NoMetadata))fid = fid^lis_NoMetadata;

        lis_groups = lis_group_COUNT(fid); 
        if(lis_groups!=0){
				i = READ_lis_groups(fid,lis_group_var,0,lis_groups);
        }
        if(CLOSE_lis_orbit_hdf(fid)!=lis_operation_SUCCESSFUL )printf("close %s file failed\n",fname);
        return (1);
}
glis_flash(argc,argv)
int argc;
void *argv[];

{
        STRING  *string_var;
        char *string2;
        struct lis_flash *lis_flash_var; /* declare flash structure pointer */

        string_var = (STRING *) argv[0];
        lis_flash_var = (struct lis_flash *) argv[1];

        string2 = strcpy(fname,string_var->s);
        fid = OPEN_lis_orbit_hdf(fname);
                        if((fid<0)&&((fid&0xffffff00)!=lis_NoMetadata)){
                                printf("OPEN_lis_orbit_hdf failed gflash %s : %08x\n",fname,fid);
                                if(CLOSE_lis_orbit_hdf(fid)!=lis_operation_SUCCESSFUL )printf("close %s file failed\n",fname);
                                return(0); 
                                /* fid = fid&0xff; */
                        }
                        if(((fid&0xffffff00)==lis_NoMetadata))fid = fid^lis_NoMetadata;


        lis_flashes = lis_flash_COUNT(fid);
        if(lis_flashes!=0){
			i = READ_lis_flashes(fid,lis_flash_var,0,lis_flashes); 
        }
        if(CLOSE_lis_orbit_hdf(fid)!=lis_operation_SUCCESSFUL )printf("close %s file failed\n",fname);
        return (1);
}

glis_area(argc,argv)
int argc;
void *argv[];

{
        STRING  *string_var;
        char *string2;
        struct lis_area *lis_area_var; /* declare area structure pointer */

        string_var = (STRING *) argv[0];
        lis_area_var = (struct lis_area *) argv[1];

        string2 = strcpy(fname,string_var->s);

        fid = OPEN_lis_orbit_hdf(fname);
                        if((fid<0)&&((fid&0xffffff00)!=lis_NoMetadata)){
                                printf("OPEN_lis_orbit_hdf failed area %s : %08x\n",fname,fid);
                                if(CLOSE_lis_orbit_hdf(fid)!=lis_operation_SUCCESSFUL )printf("close %s file failed\n",fname);
                                return(0); 
                                /* fid = fid&0xff; */
                        }
                        if(((fid&0xffffff00)==lis_NoMetadata))fid = fid^lis_NoMetadata;
        lis_areas = lis_area_COUNT(fid);
        if(lis_areas!=0){
			i = READ_lis_areas(fid,lis_area_var,0,lis_areas); 
        }
        if(CLOSE_lis_orbit_hdf(fid)!=lis_operation_SUCCESSFUL )printf("close %s file failed\n",fname);
        return (1);
}

glis_event(argc,argv)
int argc;
void *argv[];

{
        STRING  *string_var;
        char *string2;
        struct lis_event *lis_event_var; /* declare flash structure pointer */

        string_var = (STRING *) argv[0];
        lis_event_var = (struct lis_event *) argv[1];

        string2 = strcpy(fname,string_var->s);

        fid = OPEN_lis_orbit_hdf(fname);
                        if((fid<0)&&((fid&0xffffff00)!=lis_NoMetadata)){
                                printf("OPEN_lis_orbit_hdf failed event %s : %08x\n",fname,fid);
                                if(CLOSE_lis_orbit_hdf(fid)!=lis_operation_SUCCESSFUL )printf("close %s file failed\n",fname);
                                return(0); 
                                /* fid = fid&0xff; */
                        }
                        if(((fid&0xffffff00)==lis_NoMetadata))fid = fid^lis_NoMetadata;
        lis_events = lis_event_COUNT(fid);
        if(lis_events!=0){
                 i = READ_lis_events(fid,lis_event_var,0,lis_events);
        }
        if(CLOSE_lis_orbit_hdf(fid)!=lis_operation_SUCCESSFUL )printf("close %s file failed\n",fname);
        return (1);
}

glis_one_second(argc,argv)
int argc;
void *argv[];

{
        STRING  *string_var;
        char *string2;
        struct lis_one_second *lis_one_second_var; /* declare flash structure pointer */

        string_var = (STRING *) argv[0];
        lis_one_second_var = (struct lis_one_second *) argv[1];

        string2 = strcpy(fname,string_var->s);
        fid = OPEN_lis_orbit_hdf(fname);
                        if((fid<0)&&((fid&0xffffff00)!=lis_NoMetadata)&&
                        		((fid&0xffffff00)!=lis_MissingVdata)  &&
                        		((fid&0xffffff00)!=lis_NoMetadataMissingVdata)){
                                printf("OPEN_lis_orbit_hdf failed  one sec %s : %08x\n",fname,fid);
                                if(CLOSE_lis_orbit_hdf(fid)!=lis_operation_SUCCESSFUL )printf("close %s file failed\n",fname);
                                return(0); 
                                /* fid = fid&0xff; */
                        }
                         if(((fid&0xffffff00)==lis_NoMetadataMissingVdata))fid = fid^lis_NoMetadataMissingVdata;
                       if(((fid&0xffffff00)==lis_NoMetadata))fid = fid^lis_NoMetadata;
                        if(((fid&0xffffff00)==lis_MissingVdata))fid = fid^lis_MissingVdata;
        lis_one_seconds = lis_one_second_COUNT(fid);
        if(lis_one_seconds!=0){
               i = READ_lis_one_seconds(fid,lis_one_second_var,0,lis_one_seconds);
        }
        if(CLOSE_lis_orbit_hdf(fid)!=lis_operation_SUCCESSFUL )printf("close %s file failed\n",fname);
        return (1);
}

glis_viewtime(argc,argv)
int argc;
void *argv[];

{
        STRING  *string_var;
        char *string2;
        struct lis_viewtime *lis_viewtime_var; /* declare flash structure pointer */

        string_var = (STRING *) argv[0];
        lis_viewtime_var  = (struct lis_viewtime *) argv[1];

        string2 = strcpy(fname,string_var->s);

        fid = OPEN_lis_orbit_hdf(fname);
                          if((fid<0)&&((fid&0xffffff00)!=lis_NoMetadata)&&
                        		((fid&0xffffff00)!=lis_MissingVdata)  &&
                        		((fid&0xffffff00)!=lis_NoMetadataMissingVdata)){
                             	printf("OPEN_lis_orbit_hdf failed viewtime %s : %08x\n",fname,fid);
                                if(CLOSE_lis_orbit_hdf(fid)!=lis_operation_SUCCESSFUL )printf("close %s file failed\n",fname);
                                return(0); 
                                /* fid = fid&0xff; */
                        }
                        if(((fid&0xffffff00)==lis_NoMetadataMissingVdata))fid = fid^lis_NoMetadataMissingVdata;
                        if(((fid&0xffffff00)==lis_NoMetadata))fid = fid^lis_NoMetadata;
                        if(((fid&0xffffff00)==lis_MissingVdata))fid = fid^lis_MissingVdata;


        lis_viewtimes = lis_viewtime_COUNT(fid);

        if(lis_viewtimes!=0){
			i = READ_lis_viewtimes(fid,lis_viewtime_var,0,lis_viewtimes); 
        }
        if(CLOSE_lis_orbit_hdf(fid)!=lis_operation_SUCCESSFUL )printf("close %s file failed\n",fname);
        return (1);
}

glis_bg_summary(argc,argv)
int argc;
void *argv[];

{
        STRING  *string_var;
        char *string2;
        struct lis_bg_summary *lis_bg_summary_var; /* declare flash structure pointer */

        string_var = (STRING *) argv[0];
        lis_bg_summary_var  = (struct lis_bg_summary *) argv[1];

        string2 = strcpy(fname,string_var->s);

        fid = OPEN_lis_orbit_hdf(fname);
                        if((fid<0)&&((fid&0xffffff00)!=lis_NoMetadata)&&
                        		((fid&0xffffff00)!=lis_MissingVdata)  &&
                        		((fid&0xffffff00)!=lis_NoMetadataMissingVdata)){
                                printf("OPEN_lis_orbit_hdf failed viewtime %s : %08x\n",fname,fid);
                                if(CLOSE_lis_orbit_hdf(fid)!=lis_operation_SUCCESSFUL )printf("close %s file failed\n",fname);
                                return(0); 
                                /* fid = fid&0xff; */
                        }
                          if(((fid&0xffffff00)==lis_NoMetadataMissingVdata))fid = fid^lis_NoMetadataMissingVdata;
                      if(((fid&0xffffff00)==lis_NoMetadata))fid = fid^lis_NoMetadata;
                        if(((fid&0xffffff00)==lis_MissingVdata))fid = fid^lis_MissingVdata;


        lis_bg_summaries = lis_bg_summary_COUNT(fid);

        if(lis_bg_summaries!=0){
			i = READ_lis_bg_summaries(fid,lis_bg_summary_var,0,lis_bg_summaries); 
        }
        if(CLOSE_lis_orbit_hdf(fid)!=lis_operation_SUCCESSFUL )printf("close %s file failed\n",fname);
        return (1);
}





/* OTD */

#define otd_NoMetadata (FAIL|otd_inventory_metadata_FAIL|otd_archived_metadata_FAIL)

struct otd_AREA otd_area;
struct otd_FLASH otd_flash;
struct otd_GROUP otd_group;
struct otd_EVENT otd_event;
struct otd_BROWSE otd_browse;
struct otd_VECTOR otd_vector;
struct otd_IMAGE_ATTRIBUTE otd_image_attribute;
struct otd_THRESHOLD otd_threshold;
struct otd_DENSITY otd_density;
struct otd_Orbit_attributes otd_orbit_attribute;
struct otd_View_time otd_viewtime;
struct otd_EPHEMERIS otd_ephemeris;

char fname[256];
char timebuf[256];
long get_total_otd_areas();
long get_total_otd_flashes();
long get_total_otd_groups();
long get_total_otd_events();

long get_total_otd_metadata();
long get_total_otd_threshold();
long get_total_otd_browse();
long get_total_otd_image();
long get_total_otd_25_grid();
long get_total_otd_500_km();
long get_total_otd_view_time();
long get_total_otd_ephemeris();
long get_total_otd_flash_rate();
long get_total_otd_event_rate();
long j,i,otd_areas,otd_flashes,otd_groups,otd_events;
long fid;
long otd_browses,otd_vectors,otd_image_attributes, otd_thresholds,
	 otd_densities, otd_orbit_attributes, otd_viewtimes, otd_ephemerises;
	 

gotd_area(argc,argv)
int argc;
void *argv[];

{
        STRING  *string_var;
        char *string2;
        struct otd_AREA *otd_area_var; /* declare area structure pointer */

        string_var = (STRING *) argv[0];
        otd_area_var = (struct otd_AREA *) argv[1];

        string2 = strcpy(fname,string_var->s);

        fid = open_OTD_HDF_READONLY(fname);
                        if((fid<0)&&((fid&0xffffff00)!=otd_NoMetadata)){
                                printf("OPEN_lis_orbit_hdf failed area %s : %08x\n",fname,fid);
                                if(close_OTD_HDF_FILE()!=otd_SUCCESS )printf("close %s file failed\n",fname);
                                return(0); 
                                /* fid = fid&0xff; */
                        }
                        if(((fid&0xffffff00)==otd_NoMetadata))fid = fid^otd_NoMetadata;
        otd_areas = get_total_otd_areas();
        if(otd_areas!=0){
                i = READ_otd_areas(otd_area_var,0,otd_areas);
        }
        if(close_OTD_HDF_FILE()!=otd_SUCCESS )printf("close %s file failed\n",fname);
        return (1);
}

gotd_flash(argc,argv)
int argc;
void *argv[];

{
        STRING  *string_var;
        char *string2;
        struct otd_FLASH *otd_flash_var; /* declare flash structure pointer */

        string_var = (STRING *) argv[0];
        otd_flash_var = (struct otd_FLASH *) argv[1];

        string2 = strcpy(fname,string_var->s);

        fid = open_OTD_HDF_READONLY(fname);
                        if((fid<0)&&((fid&0xffffff00)!=otd_NoMetadata)){
                                printf("OPEN_lis_orbit_hdf failed flash %s : %08x\n",fname,fid);
                                if(close_OTD_HDF_FILE()!=otd_SUCCESS )printf("close %s file failed\n",fname);
                                return(0); 
                                /* fid = fid&0xff; */
                        }
                        if(((fid&0xffffff00)==otd_NoMetadata))fid = fid^otd_NoMetadata;
        otd_flashes = get_total_otd_flashes();
        if(otd_flashes!=0){
                i = READ_otd_flashes(otd_flash_var,0,otd_flashes);
        }
        if(close_OTD_HDF_FILE()!=otd_SUCCESS )printf("close %s file failed\n",fname);
        return (1);
}

gotd_group(argc,argv)
int argc;
void *argv[];

{
        STRING  *string_var;
        char *string2;
        struct otd_GROUP *otd_group_var; /* declare flash structure pointer */

        string_var = (STRING *) argv[0];
        otd_group_var = (struct otd_GROUP *) argv[1];

        string2 = strcpy(fname,string_var->s);

        fid = open_OTD_HDF_READONLY(fname);
                        if((fid<0)&&((fid&0xffffff00)!=otd_NoMetadata)){
                                printf("OPEN_lis_orbit_hdf failed group %s : %08x\n",fname,fid);
                                if(close_OTD_HDF_FILE()!=otd_SUCCESS )printf("close %s file failed\n",fname);
                                return(0); 
                                /* fid = fid&0xff; */
                        }
                        if(((fid&0xffffff00)==otd_NoMetadata))fid = fid^otd_NoMetadata;
        otd_groups = get_total_otd_groups();
        if(otd_groups!=0){
                i = READ_otd_groups(otd_group_var,0,otd_groups);
        }
        if(close_OTD_HDF_FILE()!=otd_SUCCESS )printf("close %s file failed\n",fname);
        return (1);
}

gotd_event(argc,argv)
int argc;
void *argv[];

{
        STRING  *string_var;
        char *string2;
        struct otd_EVENT *otd_event_var; /* declare flash structure pointer */

        string_var = (STRING *) argv[0];
        otd_event_var = (struct otd_EVENT *) argv[1];

        string2 = strcpy(fname,string_var->s);

        fid = open_OTD_HDF_READONLY(fname);
                        if((fid<0)&&((fid&0xffffff00)!=otd_NoMetadata)){
                                printf("OPEN_lis_orbit_hdf failed event %s : %08x\n",fname,fid);
                                if(close_OTD_HDF_FILE()!=otd_SUCCESS )printf("close %s file failed\n",fname);
                                return(0); 
                               /*  fid = fid&0xff; */
                        }
                        if(((fid&0xffffff00)==otd_NoMetadata))fid = fid^otd_NoMetadata;
        otd_events = get_total_otd_events();
        if(otd_events!=0){
                i = READ_otd_events(otd_event_var,0,otd_events);
        }
        if(close_OTD_HDF_FILE()!=otd_SUCCESS )printf("close %s file failed\n",fname);
        return (1);
}

gotd_viewtime(argc,argv)
int argc;
void *argv[];

{
        STRING  *string_var;
        char *string2;
        struct otd_View_time *otd_viewtime_var; /* declare flash structure pointer */

        string_var = (STRING *) argv[0];
        otd_viewtime_var = (struct otd_View_time *) argv[1];

        string2 = strcpy(fname,string_var->s);

        fid = open_OTD_HDF_READONLY(fname);
                        if((fid<0)&&((fid&0xffffff00)!=otd_NoMetadata)){
                                printf("OPEN_lis_orbit_hdf failed viewtime %s : %08x\n",fname,fid);
                                if(close_OTD_HDF_FILE()!=otd_SUCCESS )printf("close %s file failed\n",fname);
                                return(0); 
                                /* fid = fid&0xff; */
                        }
                        if(((fid&0xffffff00)==otd_NoMetadata))fid = fid^otd_NoMetadata;
        otd_viewtimes = get_total_otd_view_time();
        if(otd_viewtimes!=0){
               i = READ_otd_viewtimes(otd_viewtime_var,0,otd_viewtimes);
        }
        if(close_OTD_HDF_FILE()!=otd_SUCCESS )printf("close %s file failed\n",fname);
        return (1);
}

long gotd_viewtime_count(argc,argv)
int argc;
void *argv[];
{
        STRING  *string_var;
        char *string2;

        string_var = (STRING *) argv[0];
        string2 = strcpy(fname,string_var->s);
	    
	    fid = open_OTD_HDF_READONLY(fname);
                        if((fid<0)&&((fid&0xffffff00)!=otd_NoMetadata)){
                                printf("OPEN_lis_orbit_hdf failed event %s : %08x\n",fname,fid);
                                if(close_OTD_HDF_FILE()!=otd_SUCCESS )printf("close %s file failed\n",fname);
                                return(0); 
                                /* fid = fid&0xff; */
                        }
                        if(((fid&0xffffff00)==otd_NoMetadata))fid = fid^otd_NoMetadata;
        otd_viewtimes = get_total_otd_view_time();
        if(close_OTD_HDF_FILE()!=otd_SUCCESS )printf("close %s file failed\n",fname);
		return(otd_viewtimes);
}

gotd_ephemeris(argc,argv)
int argc;
void *argv[];

{
        STRING  *string_var;
        char *string2;
        struct otd_EPHEMERIS *otd_ephemeris_var; /* declare flash structure pointer */

        string_var = (STRING *) argv[0];
        otd_ephemeris_var = (struct otd_EPHEMERIS *) argv[1];

        string2 = strcpy(fname,string_var->s);

        fid = open_OTD_HDF_READONLY(fname);
                        if((fid<0)&&((fid&0xffffff00)!=otd_NoMetadata)){
                                printf("OPEN_lis_orbit_hdf failed ephemeris %s : %08x\n",fname,fid);
                                if(close_OTD_HDF_FILE()!=otd_SUCCESS )printf("close %s file failed\n",fname);
                                return(0); 
                                /* fid = fid&0xff; */
                        }
                        if(((fid&0xffffff00)==otd_NoMetadata))fid = fid^otd_NoMetadata;
        otd_ephemerises = get_total_otd_ephemeris();
        if(otd_ephemerises!=0){
                i = READ_otd_ephemerises(otd_ephemeris_var,0,otd_ephemerises);
        }
        if(close_OTD_HDF_FILE()!=otd_SUCCESS )printf("close %s file failed\n",fname);
        return (1);
}

long gotd_ephemeris_count(argc,argv)
int argc;
void *argv[];
{
        STRING  *string_var;
        char *string2;

        string_var = (STRING *) argv[0];
        string2 = strcpy(fname,string_var->s);
	    
	    fid = open_OTD_HDF_READONLY(fname);
                        if((fid<0)&&((fid&0xffffff00)!=otd_NoMetadata)){
                                printf("OPEN_lis_orbit_hdf failed event %s : %08x\n",fname,fid);
                                if(close_OTD_HDF_FILE()!=otd_SUCCESS )printf("close %s file failed\n",fname);
                                return(0); 
                                /* fid = fid&0xff; */
                        }
                        if(((fid&0xffffff00)==otd_NoMetadata))fid = fid^otd_NoMetadata;
        otd_ephemerises = get_total_otd_ephemeris();
        if(close_OTD_HDF_FILE()!=otd_SUCCESS )printf("close %s file failed\n",fname);
		return(otd_ephemerises);
}
