#include "liblisotd_read_LISOTD.h"

struct acal calibs[] = {0.0059,	    -.3749,	26.614,	    1022.7,
			0.0056,	    -.2725,	15.627,	    1332.7,
			0.0019,	    0.1644,	5.704,	    1243.4, 
			0.006,	    -0.397,	26.742,	    992.47, 
			
			0.0065,	    -0.4495,	30.177,	    917.69, 
			0.006,	    -.3529,	22.334,	    1131.9, 
			0.0031,	    -0.0079,	13.694,	    1123.2, 
			0.0054,	    -0.2959,	21.659,	    1008.2, 
			
			0.0067,	    -0.426,	27.575,	    1008.5, 
			0.0062,	    -.3818,	25.19,	    1085.2, 
			0.0032,	    0.0638,	6.977,	    1331.0, 
			0.0103,	    -0.9366,	46.545,	    832.27, 
			
			0.0102,	    -0.8809,	45.927,	    903.15, 
			0.0081,	    -.5868,	32.363,	    1064.0, 
			0.0144,	    -1.3521,	61.285,	    865.7, 
			0.0115,	    -0.9295,	40.904,	    1087.2, 
			
			0.0077,	    -.2375,	15.559,	    1327.3, 
			0.016,	    -1.2677,	48.402,	    1092.5, 
			-0.0112,    2.2663,	-76.254,    2341.4, 
			0.0109,	    -0.0871,	-3.4631,    1756.4};


/* Calculate the total calibrated flash radiance.  Level B5 is used
 *  for daytime,  B3 for twilight,  B1 for night.
 */
 
double flash_rad(struct otd_FLASH thisFlash)
{
	int num_groups,group;
	long groupid;
	struct otd_GROUP agroup;
	double flashrad = 0.0;
	double group_rad();
	
	num_groups = thisFlash.children;
	groupid = thisFlash.child_rec;

	for (group = 1; group <= num_groups; group++) {
		read_otd_group_struct(&agroup,groupid);
		flashrad += group_rad(agroup);
		groupid++;
	}

	return(flashrad);
}

double Cflash_rad(struct anOTDDataset *TheData, struct CFlash thisFlash)
{
	long groupid;
	double flashrad=0.0;

	groupid = thisFlash.FirstGroup;
	while (groupid < thisFlash.FirstGroup+thisFlash.Groups_Accepted) {
	    flashrad += Cgroup_rad(TheData, TheData->Groups[groupid]);
	    groupid++;
	}
	return(flashrad);
}

double Cgroup_rad(struct anOTDDataset *TheData, struct CGroup thisGroup)
{
    long eventid;
    double grouprad = 0.0;
    
    eventid = thisGroup.FirstEvent;
    while (eventid < thisGroup.FirstEvent+thisGroup.Events_Accepted) {
	grouprad += event_rad(TheData->Events[eventid].HDFEvent);
	eventid++;
    }
    return(grouprad);
}

/* Calculate the total calibrated group radiance.  Level B5 is used
 *  for daytime,  B3 for twilight,  B1 for night.
 */
 
double group_rad(struct otd_GROUP thisGroup)
{
	int num_events,event;
	long eventid;
	struct otd_EVENT anevent;
	double grouprad = 0.0;
	double event_rad();

	num_events = thisGroup.children;
	eventid = thisGroup.child_rec;

	for (event = 1; event <= num_events; event++) {
		read_otd_event_struct(&anevent,eventid);
		grouprad += event_rad(anevent);
		eventid++;
	}
	
	return(grouprad);
}

/* Calculate a calibrated event radiance.  Level B5 is used
 *  for daytime,  B3 for twilight,  B1 for night.
 */
 
double event_rad(struct otd_EVENT thisEvent)
{
	int q, callevel;
	float counts;
	double eventrad;

	q = getquad(thisEvent.x_pixel,thisEvent.y_pixel)-1;
	if (thisEvent.d_n_t == 'd') callevel=4;
	else if (thisEvent.d_n_t == 't') callevel=2;
	else if (thisEvent.d_n_t == 'n') callevel=0;
	counts = (float)thisEvent.raw_radiance;

	eventrad = counts * (calibs[callevel].quad[q].c1 +
		    counts * (calibs[callevel].quad[q].c2 +
		    counts * (calibs[callevel].quad[q].c3 +
		    counts * (calibs[callevel].quad[q].c4))));
	return(eventrad);
}

/* Convert a radiance to an optical energy out of the top of the
 * cloud.  We assume:
 *  (1) Uniform 10 km x 10 km pixels
 *  (2) Emitting surface composed of individual,  pixellated 
 *	isotropic emitters
 *  (3) Our bandpass is ~10% of the optical band
 * The conversion is thus simply a constant scaling of individual
 * or summed radiances.
 */
 
double rad_to_energy(double radiance)
{
	double scalfac = 8.45e-4;
	double microJ_to_J = 1e-6;
	double pi = 3.14159265;
	double area = 10.*1e3*10.*1e3;
	double bwcorr = 10.;
	double energy_J;
	
	energy_J = radiance*area*pi*microJ_to_J*scalfac*bwcorr;
	
	return(energy_J);
	
}

/* Convert an energy to a power given an estimate of the sub-event
 *   (< 1.9 ms) pulse duration in microseconds.
 */
 
double energy_to_power(double energy_J, double pulsewidth_micsec)
{
    double power_W;
    double micsec_to_sec = 1e-6;
    
    power_W = energy_J / (pulsewidth_micsec*micsec_to_sec);
    
    return(power_W);
}

/* Quick guess at a flash's area: multiply the x,y standard deviations
 * about the flash centroid.  Return in km^2 units.
 */
 
double approx_flash_area(struct otd_FLASH thisFlash)
{
    float stdx, stdy;
    double approx_area_kmsq;
    
    stdx = thisFlash.stdev[1];
    stdy = thisFlash.stdev[0];
    
    if (stdx <= 1.) stdx = 10.;
    if (stdy <= 1.) stdy = 10.;
    
    approx_area_kmsq = stdx*stdy;

    return(approx_area_kmsq);
}

double peak_group_time(flash, Data, pgr)
long flash;
struct anOTDDataset *Data;
double *pgr;
{
    double pgt = 0.0;
    long   group;
    
    *pgr = 0.0;
    for (group = Data->Flashes[flash].FirstGroup;
	    group < (Data->Flashes[flash].FirstGroup + Data->Flashes[flash].Groups_Accepted);
	       group++) {
       if (Data->Groups[group].radiance > *pgr) {
	   *pgr = Data->Groups[group].radiance;
	   pgt = Data->Groups[group].HDFGroup.TAI93;
       }
   }
   return(pgt);
}


/* Area of the largest stroke in a flash (i.e., maximum number of events
 * in a group).  Return in km^2 units.
 */
 
double max_stroke_area(thisFlash)
struct otd_FLASH thisFlash;
{
    int num_groups,group;
    long groupid;
    struct otd_GROUP agroup;
    double maxarea = 0.0;
    
    num_groups = thisFlash.children;
    groupid = thisFlash.child_rec;

    for (group = 1; group <= num_groups; group++) {
	    read_otd_group_struct(&agroup,groupid);
	    if ((double)agroup.children >= maxarea) 
		    maxarea = (double)agroup.children;
	    groupid++;
    }

    return(maxarea*10.*10.);
}

double Cmax_stroke_area(struct anOTDDataset TheData, struct CFlash thisFlash)
{
    long groupid;
    double maxarea = 0.0;
    
    groupid = thisFlash.FirstGroup;
    while (groupid < thisFlash.FirstGroup+thisFlash.Groups_Accepted) {
	if ((double)TheData.Groups[groupid].Events_Accepted >= maxarea)
	    maxarea = (double)TheData.Groups[groupid].Events_Accepted;
	groupid++;
    }
    return(maxarea*10.0*10.0); 
}

/* Total pixel area spanned by a flash.  We assume flashes to be no
 *    longer than ~ 1 second and thus negligible geographic translation
 *    due to OTD's motion.  Thus,  simply sum (once) all pixels 
 *    illuminated during the flash.  Again,  assume uniform size pixels.
 */
 /* Modified 2/27/98 to use actual pixel resolutions */
 
double total_flash_area(struct otd_FLASH thisFlash)
{
    int num_groups, group, num_events, event;
    unsigned char array[128][128];
    long groupid,  eventid;
    double totalarea = 0.0;
    int i, j;
    struct otd_GROUP agroup;
    struct otd_EVENT anevent;
    float pixres;
    
    for (i=0;i<=127;i++) { 
	for (j=0;j<=127;j++) {
	    array[i][j] = 0;
	}
    }
    
    num_groups = thisFlash.children;
    groupid = thisFlash.child_rec;
    for (group = 1; group <= num_groups; group++) {
	read_otd_group_struct(&agroup, groupid);
	num_events = agroup.children;
	eventid = agroup.child_rec;
	for (event = 1; event <= num_events; event++) {
	    read_otd_event_struct(&anevent, eventid);
	    array[anevent.x_pixel][anevent.y_pixel] = 1;
	    eventid++;
	}
	groupid++;
    }
    
    for (i=0;i<=127;i++) {
	for (j=0; j<=127; j++) {
		if (array[i][j] > 0) {
			/*
	    	totalarea += (double)(array[i][j]);
	    	*/
	    	pixres = PixelResolutionKm(i,j,"OTD");
	    	totalarea += pixres*pixres*(float)(array[i][j]);
	    }
	}
    }
    /*
    return(totalarea*10.*10.);
    */
    return(totalarea);
}
		  
/* Get a sensor quadrant from a pixel x,y location */

int getquad(int x, int y)
{
    if (x <= 63 && y <= 63) return(2);
    else if (x <= 63 && y >= 63) return(3);
    else if (x >= 63 && y <= 63) return(1);
    else if (x >= 63 && y >= 63) return(4);
    else return(-1);
}
