#include "hdf.h"
#include "liblisotd_hdf.h"
#include "liblisotd_err_defs.h"
#include "liblisotd_readhdf.h"
#include <fcntl.h>
#include <stdlib.h>

long READ_lis_SDS(void *databuffer,long sds_id, long srecord);
long READ_lis_VSGROUP(void *databuffer,long vsref,long *vsrecord);
long READ_otd_SDS(void *databuffer,long sds_id, long srecord);
long READ_otd_VSGROUP(void *databuffer,long vsref,long *vsrecord);
long READ_lis_VSGROUPS(void *databuffer,long vsref,long *vsrecord,long numrec);
long READ_otd_VSGROUPS(void *databuffer,long vsref, long *vsrecord, long numrec);
long get_lis_totals(long vsref);
long get_otd_totals(long vsref);

long lis_hdf_id[MAX_FILE]=	{-1,-1,-1,-1,-1,-1,-1,-1,
				-1,-1,-1,-1,-1,-1,-1,-1,
				-1,-1,-1,-1,-1,-1,-1,-1,
				-1,-1,-1,-1,-1,-1,-1,-1};



long OPEN_lis_orbit_hdf(char *hdf_fname)
/****************************************************************************************/
/*											*/
/*	Open a ORBIT HDF file for read access						*/
/*											*/
/*	returns file number least significant byte					*/
/*	three most significant bytes are error status bits				*/
/*											*/
/****************************************************************************************/
/****************************************************************************************/
/*											*/
/*	Compatible with HDF version 4.0						*/
/*											*/
/*											*/
/*											*/
/*	LIS/OTD HDF database access library						*/
/*											*/
/*	FUNCTION NAME		TYPE	DESCRIPTION					*/
/*	-------------------	----	----------------------------------------------- */
/*	create_OTD_HDF_FILE	long	Create and initialize LIS/OTD archive file	*/
/*	close_OTD_HDF_FILE	long	Gracefully detach and close archive file	*/
/*											*/
/*	write_otd_IMAGE		long	Write background image (SDS) to archive file	*/
/*	write_otd_ORBIT_DATA	long	Write TRMM orbit data to archive file		*/
/*	write_otd_METADATA		long	Write the Metadata to archive file		*/
/*	write_otd_ORBIT_SUMMARY	long	Write the orbit summary data to archive file	*/
/*	write_otd_VIEW		long	Write the view time data to archive file	*/
/*	write_otd_GRID_25		long	Write the 2.5 deg. data to archive file		*/
/*	write_otd_GRID_500_KM	long	Write the 500 km data to archive file		*/
/*	write_otd_AREA		long	Write AREA data to archive file			*/
/*	write_otd_FLASH		long	Write FLASH data to archive file		*/
/*	write_otd_GROUP		long	Write GROUP data to archive file		*/
/*	write_otd_EVENT		long	Write the EVENT data to archive file		*/
/*	write_otd_RATE		long	Write the EVENT rate data to archive file	*/
/*	write_otd_VECTOR_PRODUCTS	long	Write VECTOR PRODUCTS data to archive file	*/
/*	write_otd_IMAGE_ATTRIBUTE	long	Write background image data to archive file	*/
/*	write_otd_BROWSE_DATA	long	Write BROWSE DATA to archive file		*/
/*	write_otd_EPHEMERIS		long	Write SPICE ephemeris data to archive file	*/
/*	write_otd_THRESHOLD		long	Write sensor threshold data to archive file	*/
/*											*/
/*											*/
/*											*/
/*	write_otd_VSGROUP		long	generic VSwrite routine	(hiddden from user)	*/
/*	write_otd_SDS		long    generic SDwritedata routine (hidden from user)	*/
/*											*/
/*											*/
/*											*/
/*											*/
/*	*********************************************************			*/
/*	* NOTE!!					      	*			*/
/*	*	All functions return FAIL (-1) on error		*			*/
/*	*********************************************************			*/
/*											*/
/****************************************************************************************/


/****************************************************************************************/
/**************************************		*****************************************/
/************************************** HISTORY	*****************************************/
/**************************************		*****************************************/
/****************************************************************************************/
/*											*/
/*	Initial library development 9/1/94	John Hall/CSC (jmh)			*/
/*						e-mail:					*/
/*						       	jhall@deltaq.msfc.nasa.gov	*/
/*						       	johnny.hall@msfc.nasa.gov	*/
/*						phone:					*/
/*						  	(205) 922-5972			*/
/*											*/
/*											*/
/* Change History:									*/
/*											*/
/*											*/
/* Date		Module			Change description			initail */
/* --------	-------------------	-----------------------------------	-------	*/
/* 1/13/95	Create/write funcs.	Major change request from D Mach	jmh	*/
/*					virtual re-write of code!			*/
/*											*/
/* 2/22/95	Create/write funcs	Created orbit_summary,area_flash_rate,	jmh	*/
/*					unique_event_location,2.5_grid,			*/
/*					500km_grid,Vdatas				*/
/*											*/
/* 10/3/95	Removed			unique_event_location			jmh	*/
/*					removed locations and pointers from		*/
/*					groups, flashes, and areas 			*/
/*											*/
/*		Added			QA,TAI,location to event vdata		jmh	*/
/*											*/
/*											*/
/* 10/4/95	Added			View time vset				jmh	*/
/*											*/
/*											*/
/*											*/
/*											*/
/*											*/
/*											*/
/*											*/
/****************************************************************************************/
{
	long hid_tmp,i,file_number,success_flag;
 

/********************************* initialize H interface *******************************/
	success_flag = 0;
	hid_tmp = (Hopen(hdf_fname,DFACC_RDONLY,256)); 
	if(hid_tmp==FAIL)return hid_tmp;
	file_number = -1;
	for(i=0;i<MAX_FILE;i++){
		if(lis_hdf_id[i]==-1){
			file_number = i;
			break;
		}
	}
	if(file_number==-1){
		success_flag = lis_operation_FAIL|lis_MAX_FILE_FAIL;
		return(success_flag);
	}
	if(Vstart(hid_tmp)==FAIL) return lis_operation_FAIL|lis_Vstart_FAIL;
	lis_hdf_id[file_number] = hid_tmp;

	/****************************** attach vdatas *******************************************/
	/************************ search for individual vsrefs **************************/

	lis_area_vsref[file_number] = VSattach(lis_hdf_id[file_number],VSfind(lis_hdf_id[file_number],"area"),"r");
	lis_flash_vsref[file_number] = VSattach(lis_hdf_id[file_number],VSfind(lis_hdf_id[file_number],"flash"),"r");
	lis_group_vsref[file_number] = VSattach(lis_hdf_id[file_number],VSfind(lis_hdf_id[file_number],"group"),"r");
	lis_event_vsref[file_number] = VSattach(lis_hdf_id[file_number],VSfind(lis_hdf_id[file_number],"event"),"r");
	lis_viewtime_vsref[file_number] = VSattach(lis_hdf_id[file_number],VSfind(lis_hdf_id[file_number],"viewtime"),"r");
	lis_point_summary_vsref[file_number] = VSattach(lis_hdf_id[file_number],VSfind(lis_hdf_id[file_number],"point_summary"),"r");
	lis_bg_summary_vsref[file_number] = VSattach(lis_hdf_id[file_number],VSfind(lis_hdf_id[file_number],"bg_summary"),"r");
	lis_orbit_summary_vsref[file_number] = VSattach(lis_hdf_id[file_number],VSfind(lis_hdf_id[file_number],"orbit_summary"),"r");
	lis_one_second_vsref[file_number] = VSattach(lis_hdf_id[file_number],VSfind(lis_hdf_id[file_number],"one_second"),"r");

	lis_inventory_metadata_vsref[file_number] = VSattach(lis_hdf_id[file_number],VSfind(lis_hdf_id[file_number],"coremetadata.0"),"r");
	lis_archived_metadata_vsref[file_number] = VSattach(lis_hdf_id[file_number],VSfind(lis_hdf_id[file_number],"productmetadata.0"),"r");

	if(lis_area_vsref[file_number]==FAIL)
		{
#ifdef DEBUG
printf("area_vsref failed\n");
#endif
		success_flag|= lis_area_VSattach_FAIL;
	}
	if(lis_flash_vsref[file_number]==FAIL)
		{
#ifdef DEBUG
printf("flash_vsref failed\n");
#endif
		success_flag|= lis_flash_VSattach_FAIL;
	}
	if(lis_group_vsref[file_number]==FAIL)
		{
#ifdef DEBUG
printf("group_vsref failed\n");
#endif
		success_flag|= lis_group_VSattach_FAIL;
	}
	if(lis_event_vsref[file_number]==FAIL)
		{
#ifdef DEBUG
printf("event_vsref failed\n");
#endif
		success_flag|= lis_event_VSattach_FAIL;
	}
	if(lis_point_summary_vsref[file_number]==FAIL)
		{
#ifdef DEBUG
printf("point_summary_vsref failed\n");
#endif
			success_flag|= lis_point_summary_VSattach_FAIL;
		}
	if(lis_viewtime_vsref[file_number]==FAIL)
		{
#ifdef DEBUG
printf("viewtime_vsref failed\n");
#endif
		success_flag|= lis_viewtime_VSattach_FAIL;
	}
	if(lis_bg_summary_vsref[file_number]==FAIL)
		{
#ifdef DEBUG
printf("bg_summary_vsref failed\n");
#endif
		success_flag|= lis_bg_summary_VSattach_FAIL;
	}
	if(lis_orbit_summary_vsref[file_number]==FAIL)
		{
#ifdef DEBUG
printf("orbit_summary_vsref failed\n");
#endif
		success_flag |= lis_orbit_summary_VSattach_FAIL;
	}
	if(lis_one_second_vsref[file_number]==FAIL)
		{
#ifdef DEBUG
printf("one_second_vsref failed\n");
#endif
		success_flag |= lis_one_second_VSattach_FAIL;
	}


	if(lis_inventory_metadata_vsref[file_number]==FAIL)
		{
#ifdef DEBUG
printf("inventory metadata_vsref failed\n");
#endif
		success_flag|= lis_inventory_metadata_FAIL;
	}
	if(lis_archived_metadata_vsref[file_number]==FAIL){
#ifdef DEBUG
printf("archived metadata_vsref failed\n");
#endif
		success_flag|= lis_archived_metadata_FAIL;
	}



	/**************** select fields to access ***********************/
	/********************** ALL *************************************/
	if(VSsetfields(lis_orbit_summary_vsref[file_number],
		"id_number,TAI93_start,UTC_start,GPS_start,TAI93_end,start_longitude,end_longitude,point_data_count,point_data_address,one_second_count,one_second_address,summary_image_count,summary_image_address,inspection_code,configuration_code")==FAIL){
		success_flag |=  lis_VSsetfields_FAIL; 
#ifdef DEBUG
printf("orbit_summary_vsref VSetfields failed\n");
#endif
	}
	if(VSsetfields(lis_one_second_vsref[file_number],
		"TAI93_time,alert_summary,instrument_alert,platform_alert,external_alert,processing_alert,position_vector,velocity_vector,transform_matrix,solar_vector,ephemeris_quality_flag,attitude_quality_flag,boresight_threshold,thresholds,noise_index,event_count")==FAIL){
#ifdef DEBUG
printf("one_second_vsref VSetfields failed\n");
#endif
		success_flag |=  lis_VSsetfields_FAIL; 
	}
	if(VSsetfields(lis_point_summary_vsref[file_number],
		"parent_address,event_count,event_address,group_count,group_address,flash_count,flash_address,area_count,area_address,bg_count,bg_address,vt_count,vt_address")==FAIL){
#ifdef DEBUG
printf("point_summary_vsref VSetfields failed\n");
#endif
		success_flag |=  lis_VSsetfields_FAIL; 
	}
	if(VSsetfields(lis_area_vsref[file_number],
		"TAI93_time,delta_time,observe_time,location,net_radiance,footprint,address,parent_address,child_address,child_count,grandchild_count,greatgrandchild_count,approx_threshold,alert_flag,cluster_index,density_index,noise_index,oblong_index,grouping_sequence,grouping_status")==FAIL){
#ifdef DEBUG
printf("area_vsref VSetfields failed\n");
#endif
		success_flag |=  lis_VSsetfields_FAIL; 
	}
	if(VSsetfields(lis_flash_vsref[file_number],
		"TAI93_time,delta_time,observe_time,location,radiance,footprint,address,parent_address,child_address,child_count,grandchild_count,approx_threshold,alert_flag,cluster_index,density_index,noise_index,glint_index,oblong_index,grouping_sequence,grouping_status")==FAIL){
#ifdef DEBUG
printf("flash_vsref VSetfields failed\n");
#endif
		success_flag |=  lis_VSsetfields_FAIL; 
	}
	if(VSsetfields(lis_group_vsref[file_number],
		"TAI93_time,observe_time,location,radiance,footprint,address,parent_address,child_address,child_count,approx_threshold,alert_flag,cluster_index,density_index,noise_index,glint_index,oblong_index,grouping_sequence,grouping_status")==FAIL){
#ifdef DEBUG
printf("group_vsref VSetfields failed\n");
#endif
		success_flag |=  lis_VSsetfields_FAIL; 
	}
	if(VSsetfields(lis_event_vsref[file_number],
		"TAI93_time,observe_time,location,radiance,footprint,address,parent_address,x_pixel,y_pixel,bg_value,bg_radiance,amplitude,sza_index,glint_index,approx_threshold,alert_flag,cluster_index,density_index,noise_index,bg_value_flag,grouping_sequence")==FAIL){
#ifdef DEBUG
printf("event_vsref VSetfields failed\n");
#endif
		success_flag |=  lis_VSsetfields_FAIL; 
	}

	if(VSsetfields(lis_viewtime_vsref[file_number],
		"location,TAI93_start,TAI93_end,effective_obs,alert_flag,approx_threshold")==FAIL){
#ifdef DEBUG
printf("viewtime_vsref VSetfields failed\n");
#endif
		success_flag |=  lis_VSsetfields_FAIL; 
	}
	if(VSsetfields(lis_bg_summary_vsref[file_number],
		"TAI93_time,address,boresight,corners")==FAIL){
#ifdef DEBUG
printf("bg_summary_vsref VSetfields failed\n");
#endif
		success_flag |=  lis_VSsetfields_FAIL; 
	}



	if((success_flag&0xffffff00)!=0)
		success_flag = success_flag|lis_operation_FAIL;
	return (success_flag|file_number); 
}


long OPEN_lis_background_hdf(char *hdf_fname){

	long hid_tmp,i,file_number,success_flag;
 
	success_flag = 0;

/********************************* initialize H interface *******************************/
	
	hid_tmp = (Hopen(hdf_fname,DFACC_RDONLY,256)); 
	if(hid_tmp==FAIL){
#ifdef DEBUG
printf("Hopen failed\n");
#endif
		return (FAIL);   
	}
	file_number = -1;
	for(i=0;i<MAX_FILE;i++){
		if(lis_hdf_id[i]==-1){
			file_number = i;
			break;
		}
	}
	if(file_number==-1){
#ifdef DEBUG
printf("MAX_FILE exceeded\n");
#endif
		success_flag = lis_operation_FAIL|lis_MAX_FILE_FAIL;
		return(success_flag);
	}
	if(Vstart(hid_tmp)==FAIL){
#ifdef DEBUG
printf("Vstart  failed\n");
#endif
		success_flag = lis_operation_FAIL|lis_Vstart_FAIL;
		return(success_flag);
	}
	lis_hdf_id[file_number] = hid_tmp;

	/****************************** attach vdatas *******************************************/



	lis_bg_orbit_summary_vsref[file_number] = VSattach(lis_hdf_id[file_number],VSfind(lis_hdf_id[file_number],"bg_orbit_summary"),"r");
	if(lis_bg_orbit_summary_vsref[file_number]==FAIL)
		{
#ifdef DEBUG
printf("bg_orbit_summary_vsref failed\n");
#endif
		success_flag|= lis_bg_orbit_summary_VSattach_FAIL;
	}
	lis_bg_info_vsref[file_number] = VSattach(lis_hdf_id[file_number],VSfind(lis_hdf_id[file_number],"bg_info"),"r");
	if(lis_bg_info_vsref[file_number]==FAIL)
		{
#ifdef DEBUG
printf("bg_info_vsref failed\n");
#endif
		success_flag|= lis_bg_info_VSattach_FAIL;
	}
	lis_bg_data_summary_vsref[file_number] = VSattach(lis_hdf_id[file_number],VSfind(lis_hdf_id[file_number],"bg_data_summary"),"r");
	if(lis_bg_data_summary_vsref[file_number]==FAIL)
		{
#ifdef DEBUG
printf("bg_data_summary_vsref failed\n");
#endif
		success_flag|= lis_bg_data_summary_VSattach_FAIL;
	}

/************** open hdf file for reading  scientific data sets (SDS) *******************/
	lis_image_srecord[file_number] = 0;
	lis_image_sds_id[file_number] = FAIL;
	lis_sd_id[file_number] = SDstart(hdf_fname,DFACC_RDONLY);
	if (lis_sd_id[file_number]==FAIL){
#ifdef DEBUG
printf(" SDstart failed\n");
#endif
	success_flag |= lis_SDstart_FAIL; }
	if((lis_image_sds_id[file_number] = SDselect(lis_sd_id[file_number],SDnametoindex(lis_sd_id[file_number],"bg_data")))==FAIL){
#ifdef DEBUG
printf("image_sds_id failed\n");
#endif
		success_flag =  success_flag|=lis_SDselect_FAIL; 
	}
	if(VSsetfields(lis_bg_data_summary_vsref[file_number],
		"TAI93_time,address,boresight,corners")==FAIL){
#ifdef DEBUG
printf("bg_summary_vsref VSetfields failed\n");
#endif
		success_flag |=  lis_VSsetfields_FAIL; 
	}

	if(VSsetfields(lis_bg_info_vsref[file_number],
		"TAI93_time,alert_summary,instrument_alert,platform_alert,external_alert,processing_alert,position_vector,velocity_vector,transform_matrix,solar_vector,ephemeris_quality_flag,attitude_quality_flag,bg_value,noise_index,event_count")==FAIL){
#ifdef DEBUG
printf("bg_info_vsref VSetfields failed\n");
#endif
		success_flag |=  lis_VSsetfields_FAIL; 
	}

	if(VSsetfields(lis_bg_orbit_summary_vsref[file_number],
		"id_number,TAI93_start,UTC_start,GPS_start,TAI93_end,start_longitude,end_longitude,image_summary_count,image_summary_address,image_info_count,image_info_address")==FAIL){
#ifdef DEBUG
printf("bg_orbit_summary_vsref VSetfields failed\n");
#endif
		success_flag |=  lis_VSsetfields_FAIL; 
	}

	if((success_flag&0xffffff00)!=0)
		success_flag = success_flag|lis_operation_FAIL;
	return (success_flag|file_number); 

}

long CLOSE_lis_orbit_hdf(long file_number)

/****************************************************************************************/
/*                                                                             		*/
/*	CLOSE_hdf_file									*/
/*											*/
/*	returns 0 if all vgroups, vsets and the file itself are successfully    	*/
/*	closed										*/
/*	returns FAIL(-1) on error							*/
/*											*/
/*	Date last modified 6/1/96							*/
/****************************************************************************************/

{
	long return_status;
	return_status = 0;

        if(lis_inventory_metadata_vsref[file_number]!= FAIL)VSdetach(lis_inventory_metadata_vsref[file_number]);		/* metadata */
        if(lis_archived_metadata_vsref[file_number]!= FAIL)VSdetach(lis_archived_metadata_vsref[file_number]);		/* metadata */

	if(lis_area_vsref[file_number]!= FAIL)VSdetach(lis_area_vsref[file_number]);			
	if(lis_flash_vsref[file_number]!= FAIL)VSdetach(lis_flash_vsref[file_number]);			
	if(lis_group_vsref[file_number]!= FAIL)VSdetach(lis_group_vsref[file_number]);			
        if(lis_event_vsref[file_number]!= FAIL)VSdetach(lis_event_vsref[file_number]);			
        if(lis_point_summary_vsref[file_number]!= FAIL)VSdetach(lis_point_summary_vsref[file_number]);			
        if(lis_viewtime_vsref[file_number]!= FAIL)VSdetach(lis_viewtime_vsref[file_number]);			
        if(lis_bg_summary_vsref[file_number]!= FAIL)VSdetach(lis_bg_summary_vsref[file_number]);			
        if(lis_orbit_summary_vsref[file_number]!= FAIL)VSdetach(lis_orbit_summary_vsref[file_number]);			
        if(lis_one_second_vsref[file_number]!= FAIL)VSdetach(lis_one_second_vsref[file_number]);			

	if(lis_orbit_data_vgroup_id[file_number]!= FAIL)Vdetach(lis_orbit_data_vgroup_id[file_number]);
	if(lis_point_vgroup_id[file_number]!= FAIL)Vdetach(lis_point_vgroup_id[file_number]);
	if(lis_lightning_vgroup_id[file_number]!= FAIL)Vdetach(lis_lightning_vgroup_id[file_number]);

	if(lis_hdf_id[file_number]!=FAIL)Vend(lis_hdf_id[file_number]);



	if(lis_hdf_id[file_number] !=FAIL)if(Hclose(lis_hdf_id[file_number])==FAIL)return_status = FAIL;	

	lis_hdf_id[file_number] = -1;	/* free file numer */
	
	return (return_status);						/* success*/

}

long CLOSE_lis_background_hdf(long file_number){
	long return_status;
	return_status = 0;


	if(lis_bg_orbit_summary_vsref[file_number]!= FAIL)VSdetach(lis_bg_orbit_summary_vsref[file_number]);
	if(lis_bg_info_vsref[file_number]!= FAIL)VSdetach(lis_bg_info_vsref[file_number]);			
	if(lis_bg_data_summary_vsref[file_number]!= FAIL)VSdetach(lis_bg_data_summary_vsref[file_number]);			

	if(lis_orbit_data_vgroup_id[file_number]!= FAIL)Vdetach(lis_orbit_data_vgroup_id[file_number]);


 	if(lis_sd_id[file_number]!=FAIL)if (SDend(lis_sd_id[file_number])==FAIL)return_status = FAIL;					/* SD id */

	if(lis_hdf_id[file_number] !=FAIL)if(Hclose(lis_hdf_id[file_number])==FAIL)return_status = FAIL;	
	lis_hdf_id[file_number] = -1;	/* free file numer */
	return (return_status);						

}

long READ_lis_background_data(long file_number,short int *image,long *vsrecord)

/****************************************************************************************/
/*											*/
/*	read Background image (SDS) from HDF file					*/
/*											*/
/*	returns zero if successful							*/
/*	returns operation_FAIL|SDreaddata_FAIL on error					*/
/*											*/
/*	Date last modified	6/1/96							*/
/****************************************************************************************/


{
	start[0] = *vsrecord;
	start[1]=start[2]=0;

	endr[0]=1;
	endr[1]=endr[2]=128;


/*	if(image_count[file_number]<*vsrecord)return (FAIL);*/
	if(SDreaddata(lis_image_sds_id[file_number],start,NULL,endr,image)==FAIL)
		return (lis_operation_FAIL|lis_SDreaddata_FAIL);
	return (0);	
}

long READ_lis_VSGROUP(void *databuffer,long vsref,long *vsrecord)

/****************************************************************************************/
/*											*/
/*	read data from HDF file								*/
/*											*/
/*	returns zero if successful							*/
/*	returns operation_FAIL|VSread_FAIL on error					*/
/*											*/
/*	Date last modified	6/1/96							*/
/****************************************************************************************/


{int j;
	if(vsref==FAIL)return(FAIL);
	if (VSelts(vsref) <= *vsrecord){
#ifdef DEBUG
printf("VEOF:: VSelts(%08x) = %d attempt to read record %d\n",vsref,VSelts(vsref),*vsrecord);
#endif
		return (lis_operation_FAIL|lis_VEOF);
	}
	if (VSseek(vsref,*vsrecord)==FAIL){
#ifdef DEBUG
printf("VSseek failed on record %d\n",*vsrecord);
#endif
		 return (lis_operation_FAIL|lis_VSseek_FAIL);
	}
 	if((j=VSread(vsref,databuffer,1,FULL_INTERLACE))==FAIL){
#ifdef DEBUF
printf("VSread FAILED\n");
#endif 
		return (lis_operation_FAIL|lis_VSread_FAIL);
	}
	return (*vsrecord);
}

long READ_lis_VSGROUPS(void *databuffer,long vsref,long *vsrecord,long numrec)

/****************************************************************************************/
/*											*/
/*	read data from HDF file								*/
/*											*/
/*	returns zero if successful							*/
/*	returns operation_FAIL|VSread_FAIL on error					*/
/*											*/
/*	Date last modified	6/1/96							*/
/****************************************************************************************/


{int j;
	if(vsref==FAIL)return(FAIL);
	if (VSelts(vsref) <= *vsrecord){
#ifdef DEBUG
printf("VEOF:: VSelts(%08x) = %d attempt to read record %d\n",vsref,VSelts(vsref),*vsrecord);
#endif
		return (lis_operation_FAIL|lis_VEOF);
	}
	if (VSseek(vsref,*vsrecord)==FAIL){
#ifdef DEBUG
printf("VSseek failed on record %d\n",*vsrecord);
#endif
		 return (lis_operation_FAIL|lis_VSseek_FAIL);
	}
 	if((j=VSread(vsref,databuffer,numrec,FULL_INTERLACE))==FAIL){
#ifdef DEBUF
printf("VSread FAILED\n");
#endif 
		return (lis_operation_FAIL|lis_VSread_FAIL);
	}
	return (*vsrecord);
}

long lis_area_COUNT(long file_number)			{return (get_lis_totals(lis_area_vsref[file_number]));}
long lis_flash_COUNT(long file_number)			{return (get_lis_totals(lis_flash_vsref[file_number]));}
long lis_group_COUNT(long file_number)			{return (get_lis_totals(lis_group_vsref[file_number]));}
long lis_event_COUNT(long file_number)			{return (get_lis_totals(lis_event_vsref[file_number]));}
long lis_point_summary_COUNT(long file_number)		{return (get_lis_totals(lis_point_summary_vsref[file_number]));}
long lis_viewtime_COUNT(long file_number)		{return (get_lis_totals(lis_viewtime_vsref[file_number]));}
long lis_bg_summary_COUNT(long file_number)		{return (get_lis_totals(lis_bg_summary_vsref[file_number]));}
long lis_orbit_summary_COUNT(long file_number)		{return (get_lis_totals(lis_orbit_summary_vsref[file_number]));}
long lis_one_second_COUNT(long file_number)		{return (get_lis_totals(lis_one_second_vsref[file_number]));}

long lis_bg_orbit_summary_COUNT(long file_number)	{return (get_lis_totals(lis_bg_orbit_summary_vsref[file_number]));}
long lis_bg_info_COUNT(long file_number)		{return (get_lis_totals(lis_bg_info_vsref[file_number]));}
long lis_bg_data_summary_COUNT(long file_number)	{return (get_lis_totals(lis_bg_data_summary_vsref[file_number]));}


/****************************************************************************************/
/*											*/
/*	read vdata record number 							*/
/*											*/
/*	returns total Vdata record # if successful					*/
/*	returns FAIL (-1) on error							*/
/*											*/
/*	Date last modified	6/1/96							*/
/****************************************************************************************/

long get_lis_totals(long vsref)
{
 long return_flag;
 int32 record_COUNT;
	if(vsref==FAIL)return (FAIL);
	return_flag = VSinquire(vsref,&record_COUNT,NULL,NULL,NULL,NULL);
	if (return_flag == -1)
		return (return_flag);
	else
		return (record_COUNT);
}

long READ_lis_area(long file_number,struct lis_area *lis_AREA,long record){
	unsigned char buf[128];
	long status;
		if((status=READ_lis_VSGROUP(buf,lis_area_vsref[file_number],&record))!=record)
			return status;
#ifdef DEBUG
printf("%d:: ",record);
for(i=0;i<68;i++)printf("%02x ",buf[i]);
printf("\n");
#endif
		memcpy(&lis_AREA->TAI93_time,&buf[0],8);
		memcpy(&lis_AREA->delta_time,&buf[8],4);
		memcpy(&lis_AREA->observe_time,&buf[12],2);
		memcpy(&lis_AREA->location,&buf[14],8);
		memcpy(&lis_AREA->net_radiance,&buf[22],4);
		memcpy(&lis_AREA->footprint,&buf[26],4);
		memcpy(&lis_AREA->address,&buf[30],4);
		memcpy(&lis_AREA->parent_address,&buf[34],4);
		memcpy(&lis_AREA->child_address,&buf[38],4);
		memcpy(&lis_AREA->child_count,&buf[42],4);
		memcpy(&lis_AREA->grandchild_count,&buf[46],4);
		memcpy(&lis_AREA->greatgrandchild_count,&buf[50],4);
		memcpy(&lis_AREA->approx_threshold,&buf[54],1);
		memcpy(&lis_AREA->alert_flag,&buf[55],1);
		memcpy(&lis_AREA->cluster_index,&buf[56],1);
		memcpy(&lis_AREA->density_index,&buf[57],1);
		memcpy(&lis_AREA->noise_index,&buf[58],1);
#if defined(MAC) || defined(macintosh)	
/*
		memcpy(&lis_AREA->oblong_index,&buf[59],4);
		memcpy(&lis_AREA->grouping_sequence,&buf[63],4);
*/
#else
		memcpy(&lis_AREA->oblong_index,&buf[59],4);
		memcpy(&lis_AREA->grouping_sequence,&buf[63],4);	
#endif
		memcpy(&lis_AREA->grouping_status,&buf[67],1);
		
		return 0;
}

long READ_lis_areas(long file_number,struct lis_area *lis_area_var,long record,long numrec){
	unsigned char *buf;
	long status,i;
		buf = malloc((size_t)(68*sizeof(unsigned char)*numrec));
		status = READ_lis_VSGROUPS(buf,lis_area_vsref[file_number],&record,numrec);
		for (i=0; i<numrec; i++) {		
			memcpy(&lis_area_var[i].TAI93_time,&buf[i*68+0],8);
			memcpy(&lis_area_var[i].delta_time,&buf[i*68+8],4);
			memcpy(&lis_area_var[i].observe_time,&buf[i*68+12],2);
			memcpy(&lis_area_var[i].location,&buf[i*68+14],8);
			memcpy(&lis_area_var[i].net_radiance,&buf[i*68+22],4);
			memcpy(&lis_area_var[i].footprint,&buf[i*68+26],4);
			memcpy(&lis_area_var[i].address,&buf[i*68+30],4);
			memcpy(&lis_area_var[i].parent_address,&buf[i*68+34],4);
			memcpy(&lis_area_var[i].child_address,&buf[i*68+38],4);
			memcpy(&lis_area_var[i].child_count,&buf[i*68+42],4);
			memcpy(&lis_area_var[i].grandchild_count,&buf[i*68+46],4);
			memcpy(&lis_area_var[i].greatgrandchild_count,&buf[i*68+50],4);
			memcpy(&lis_area_var[i].approx_threshold,&buf[i*68+54],1);
			memcpy(&lis_area_var[i].alert_flag,&buf[i*68+55],1);
			memcpy(&lis_area_var[i].cluster_index,&buf[i*68+56],1);
			memcpy(&lis_area_var[i].density_index,&buf[i*68+57],1);
			memcpy(&lis_area_var[i].noise_index,&buf[i*68+58],1);
#if defined(MAC) || defined(macintosh)	
	/*
			memcpy(&lis_area_var[i].oblong_index,&buf[i*68+59],4);
			memcpy(&lis_area_var[i].grouping_sequence,&buf[i*68+63],4);
	*/
#else
			memcpy(&lis_area_var[i].oblong_index,&buf[i*68+59],4);
			memcpy(&lis_area_var[i].grouping_sequence,&buf[i*68+63],4);
#endif
			memcpy(&lis_area_var[i].grouping_status,&buf[i*68+67],1);
		}
		free(buf);
		return 0;
}

long READ_lis_flash(long file_number,struct lis_flash *lis_FLASH,long record){
	unsigned char buf[128];
	long status;
		if((status=READ_lis_VSGROUP(buf,lis_flash_vsref[file_number],&record))!=record)
			return status;
#ifdef DEBUG
printf("%d:: ",record);
for(i=0;i<68;i++)printf("%02x ",buf[i]);
printf("\n");
#endif

		memcpy(&lis_FLASH->TAI93_time,&buf[0],8);
		memcpy(&lis_FLASH->delta_time,&buf[8],4);
		memcpy(&lis_FLASH->observe_time,&buf[12],2);
		memcpy(&lis_FLASH->location,&buf[14],8);
		memcpy(&lis_FLASH->radiance,&buf[22],4);
		memcpy(&lis_FLASH->footprint,&buf[26],4);
		memcpy(&lis_FLASH->address,&buf[30],4);
		memcpy(&lis_FLASH->parent_address,&buf[34],4);
		memcpy(&lis_FLASH->child_address,&buf[38],4);
		memcpy(&lis_FLASH->child_count,&buf[42],4);
		memcpy(&lis_FLASH->grandchild_count,&buf[46],4);
		memcpy(&lis_FLASH->approx_threshold,&buf[50],1);
		memcpy(&lis_FLASH->alert_flag,&buf[51],1);
		memcpy(&lis_FLASH->cluster_index,&buf[52],1);
		memcpy(&lis_FLASH->density_index,&buf[53],1);
		memcpy(&lis_FLASH->noise_index,&buf[54],1);
		memcpy(&lis_FLASH->glint_index,&buf[55],4);
#if defined(MAC) || defined(macintosh)	
/*
		memcpy(&lis_FLASH->oblong_index,&buf[59],4);
		memcpy(&lis_FLASH->grouping_sequence,&buf[63],4);
*/
#else
		memcpy(&lis_FLASH->oblong_index,&buf[59],4);
		memcpy(&lis_FLASH->grouping_sequence,&buf[63],4);
#endif
		memcpy(&lis_FLASH->grouping_status,&buf[67],1);
		return 0;
}

long READ_lis_flashes(long file_number,struct lis_flash *lis_flash_var,long record,long numrec){
	unsigned char *buf;
	long status,i;
		buf = malloc((size_t)(68*sizeof(unsigned char)*numrec));
		status = READ_lis_VSGROUPS(buf,lis_flash_vsref[file_number],&record,numrec);
		for (i=0; i<numrec; i++) {		
			memcpy(&lis_flash_var[i].TAI93_time,&buf[i*68+0],8);
			memcpy(&lis_flash_var[i].delta_time,&buf[i*68+8],4);
			memcpy(&lis_flash_var[i].observe_time,&buf[i*68+12],2);
			memcpy(&lis_flash_var[i].location,&buf[i*68+14],8);
			memcpy(&lis_flash_var[i].radiance,&buf[i*68+22],4);
			memcpy(&lis_flash_var[i].footprint,&buf[i*68+26],4);
			memcpy(&lis_flash_var[i].address,&buf[i*68+30],4);
			memcpy(&lis_flash_var[i].parent_address,&buf[i*68+34],4);
			memcpy(&lis_flash_var[i].child_address,&buf[i*68+38],4);
			memcpy(&lis_flash_var[i].child_count,&buf[i*68+42],4);
			memcpy(&lis_flash_var[i].grandchild_count,&buf[i*68+46],4);
			memcpy(&lis_flash_var[i].approx_threshold,&buf[i*68+50],1);
			memcpy(&lis_flash_var[i].alert_flag,&buf[i*68+51],1);
			memcpy(&lis_flash_var[i].cluster_index,&buf[i*68+52],1);
			memcpy(&lis_flash_var[i].density_index,&buf[i*68+53],1);
			memcpy(&lis_flash_var[i].noise_index,&buf[i*68+54],1);
			memcpy(&lis_flash_var[i].glint_index,&buf[i*68+55],4);
#if defined(MAC) || defined(macintosh)	
	/*
			memcpy(&lis_flash_var[i].oblong_index,&buf[i*68+59],4);
			memcpy(&lis_flash_var[i].grouping_sequence,&buf[i*68+63],4);
	*/
#else
			memcpy(&lis_flash_var[i].oblong_index,&buf[i*68+59],4);
			memcpy(&lis_flash_var[i].grouping_sequence,&buf[i*68+63],4);
#endif
			memcpy(&lis_flash_var[i].grouping_status,&buf[i*68+67],1);		
		}
		free(buf);
		return 0;
}


long READ_lis_group(long file_number,struct lis_group *lis_GROUP,long record){
	unsigned char buf[128];
	long status;
		if((status=READ_lis_VSGROUP(buf,lis_group_vsref[file_number],&record))!=record)
			return status;
#ifdef DEBUG
printf("%d:: ",record);
for(i=0;i<60;i++)printf("%02x ",buf[i]);
printf("\n");
#endif
		memcpy(&lis_GROUP->TAI93_time,&buf[0],8);
		memcpy(&lis_GROUP->observe_time,&buf[8],2);
		memcpy(&lis_GROUP->location,&buf[10],8);
		memcpy(&lis_GROUP->radiance,&buf[18],4);
		memcpy(&lis_GROUP->footprint,&buf[22],4);
		memcpy(&lis_GROUP->address,&buf[26],4);
		memcpy(&lis_GROUP->parent_address,&buf[30],4);
		memcpy(&lis_GROUP->child_address,&buf[34],4);
		memcpy(&lis_GROUP->child_count,&buf[38],4);
		memcpy(&lis_GROUP->approx_threshold,&buf[42],1);
		memcpy(&lis_GROUP->alert_flag,&buf[43],1);
		memcpy(&lis_GROUP->cluster_index,&buf[44],1);
		memcpy(&lis_GROUP->density_index,&buf[45],1);
		memcpy(&lis_GROUP->noise_index,&buf[46],1);
		memcpy(&lis_GROUP->glint_index,&buf[47],4);
#if defined(MAC) || defined(macintosh)	
		/*
		memcpy(&lis_GROUP->oblong_index,&buf[51],4);
		*/
#else
		memcpy(&lis_GROUP->oblong_index,&buf[51],4);
#endif
		memcpy(&lis_GROUP->grouping_sequence,&buf[55],4);
		memcpy(&lis_GROUP->grouping_status,&buf[59],1);
		return 0;
}

long READ_lis_groups(long file_number,struct lis_group *lis_group_var,long record,long numrec){
	unsigned char *buf;
	long status,i;
		buf = malloc((size_t)(60*sizeof(unsigned char)*numrec));
		status = READ_lis_VSGROUPS(buf,lis_group_vsref[file_number],&record,numrec);
		for (i=0; i<numrec; i++) {		
			memcpy(&lis_group_var[i].TAI93_time,&buf[i*60+0],8);
			memcpy(&lis_group_var[i].observe_time,&buf[i*60+8],2);
			memcpy(&lis_group_var[i].location,&buf[i*60+10],8);
			memcpy(&lis_group_var[i].radiance,&buf[i*60+18],4);
			memcpy(&lis_group_var[i].footprint,&buf[i*60+22],4);
			memcpy(&lis_group_var[i].address,&buf[i*60+26],4);
			memcpy(&lis_group_var[i].parent_address,&buf[i*60+30],4);
			memcpy(&lis_group_var[i].child_address,&buf[i*60+34],4);
			memcpy(&lis_group_var[i].child_count,&buf[i*60+38],4);
			memcpy(&lis_group_var[i].approx_threshold,&buf[i*60+42],1);
			memcpy(&lis_group_var[i].alert_flag,&buf[i*60+43],1);
			memcpy(&lis_group_var[i].cluster_index,&buf[i*60+44],1);
			memcpy(&lis_group_var[i].density_index,&buf[i*60+45],1);
			memcpy(&lis_group_var[i].noise_index,&buf[i*60+46],1);
			memcpy(&lis_group_var[i].glint_index,&buf[i*60+47],4);
#if defined(MAC) || defined(macintosh)	
/*
			memcpy(&lis_group_var[i].oblong_index,&buf[i*60+51],4);
			memcpy(&lis_group_var[i].grouping_sequence,&buf[i*60+55],4);
*/
#else
			memcpy(&lis_group_var[i].oblong_index,&buf[i*60+51],4);
			memcpy(&lis_group_var[i].grouping_sequence,&buf[i*60+55],4);
#endif
			memcpy(&lis_group_var[i].grouping_status,&buf[i*60+59],1);
		}
		free(buf);
		return 0;
}

long READ_lis_event(long file_number,struct lis_event *lis_EVENT,long record){
	unsigned char buf[128];
	long status;
		if((status=READ_lis_VSGROUP(buf,lis_event_vsref[file_number],&record))!=record)
			return status;
#ifdef DEBUG
printf("%d:: ",record);
for(i=0;i<53;i++)printf("%02x ",buf[i]);
printf("\n");
#endif
		memcpy(&lis_EVENT->TAI93_time,&buf[0],8);
		memcpy(&lis_EVENT->observe_time,&buf[8],2);
		memcpy(&lis_EVENT->location,&buf[10],8);
		memcpy(&lis_EVENT->radiance,&buf[18],4);
		memcpy(&lis_EVENT->footprint,&buf[22],4);
		memcpy(&lis_EVENT->address,&buf[26],4);
		memcpy(&lis_EVENT->parent_address,&buf[30],4);
		memcpy(&lis_EVENT->x_pixel,&buf[34],1);
		memcpy(&lis_EVENT->y_pixel,&buf[35],1);
		memcpy(&lis_EVENT->bg_value,&buf[36],2);
		memcpy(&lis_EVENT->bg_radiance,&buf[38],2);
		memcpy(&lis_EVENT->amplitude,&buf[40],1);
		memcpy(&lis_EVENT->sza_index,&buf[41],1);
		memcpy(&lis_EVENT->glint_index,&buf[42],1);
		memcpy(&lis_EVENT->approx_threshold,&buf[43],1);
		memcpy(&lis_EVENT->alert_flag,&buf[44],1);
		memcpy(&lis_EVENT->cluster_index,&buf[45],1);
		memcpy(&lis_EVENT->density_index,&buf[46],1);
		memcpy(&lis_EVENT->noise_index,&buf[47],1);
		memcpy(&lis_EVENT->bg_value_flag,&buf[48],1);
		memcpy(&lis_EVENT->grouping_sequence,&buf[49],4);
		return 0;
}

long READ_lis_events(long file_number,struct lis_event *lis_event_var,long record,long numrec){
	unsigned char *buf;
	long status,i;
	
		buf = malloc((size_t)(53*sizeof(unsigned char)*numrec));
		
		status=READ_lis_VSGROUPS(buf,lis_event_vsref[file_number],&record,numrec);
		for (i=0; i<numrec; i++) {		
			memcpy(&lis_event_var[i].TAI93_time,&buf[i*53+0],8);
			memcpy(&lis_event_var[i].observe_time,&buf[i*53+8],2);
			memcpy(&lis_event_var[i].location,&buf[i*53+10],8);
			memcpy(&lis_event_var[i].radiance,&buf[i*53+18],4);
			memcpy(&lis_event_var[i].footprint,&buf[i*53+22],4);
			memcpy(&lis_event_var[i].address,&buf[i*53+26],4);
			memcpy(&lis_event_var[i].parent_address,&buf[i*53+30],4);
			memcpy(&lis_event_var[i].x_pixel,&buf[i*53+34],1);
			memcpy(&lis_event_var[i].y_pixel,&buf[i*53+35],1);
			memcpy(&lis_event_var[i].bg_value,&buf[i*53+36],2);
			memcpy(&lis_event_var[i].bg_radiance,&buf[i*53+38],2);
			memcpy(&lis_event_var[i].amplitude,&buf[i*53+40],1);
			memcpy(&lis_event_var[i].sza_index,&buf[i*53+41],1);
			memcpy(&lis_event_var[i].glint_index,&buf[i*53+42],1);
			memcpy(&lis_event_var[i].approx_threshold,&buf[i*53+43],1);
			memcpy(&lis_event_var[i].alert_flag,&buf[i*53+44],1);
			memcpy(&lis_event_var[i].cluster_index,&buf[i*53+45],1);
			memcpy(&lis_event_var[i].density_index,&buf[i*53+46],1);
			memcpy(&lis_event_var[i].noise_index,&buf[i*53+47],1);
			memcpy(&lis_event_var[i].bg_value_flag,&buf[i*53+48],1);
			memcpy(&lis_event_var[i].grouping_sequence,&buf[i*53+49],4);
		}
		free(buf);
		return 0;
}

long READ_lis_point_summary(long file_number,struct lis_point_summary *lis_POINT_SUMMARY,long record){
	unsigned char buf[128];
	long status;
		if((status=READ_lis_VSGROUP(buf,lis_point_summary_vsref[file_number],&record))!=record)
			return status;
#ifdef DEBUG
printf("%d:: ",record);
for(i=0;i<52;i++)printf("%02x ",buf[i]);
printf("\n");
#endif

		memcpy(&lis_POINT_SUMMARY->parent_address,&buf[0],4);
		memcpy(&lis_POINT_SUMMARY->event_count,&buf[4],4);
		memcpy(&lis_POINT_SUMMARY->event_address,&buf[8],4);
		memcpy(&lis_POINT_SUMMARY->group_count,&buf[12],4);
		memcpy(&lis_POINT_SUMMARY->group_address,&buf[16],4);
		memcpy(&lis_POINT_SUMMARY->flash_count,&buf[20],4);
		memcpy(&lis_POINT_SUMMARY->flash_address,&buf[24],4);
		memcpy(&lis_POINT_SUMMARY->area_count,&buf[28],4);
		memcpy(&lis_POINT_SUMMARY->area_address,&buf[32],4);
		memcpy(&lis_POINT_SUMMARY->bg_count,&buf[36],4);
		memcpy(&lis_POINT_SUMMARY->bg_address,&buf[40],4);
		memcpy(&lis_POINT_SUMMARY->vt_count,&buf[44],4);
		memcpy(&lis_POINT_SUMMARY->vt_address,&buf[48],4);

		return 0;
}
long READ_lis_viewtime(long file_number,struct lis_viewtime *lis_VIEWTIME,long record){
	unsigned char buf[128];
	long status;
		if((status=READ_lis_VSGROUP(buf,lis_viewtime_vsref[file_number],&record))!=record)
			return status;
#ifdef DEBUG
printf("%d:: ",record);
for(i=0;i<22;i++)printf("%02x ",buf[i]);
printf("\n");
#endif
		memcpy(&lis_VIEWTIME->location,&buf[0],8);
		memcpy(&lis_VIEWTIME->TAI93_start,&buf[8],4);
		memcpy(&lis_VIEWTIME->TAI93_end,&buf[12],4);
		memcpy(&lis_VIEWTIME->effective_obs,&buf[16],4);
		memcpy(&lis_VIEWTIME->alert_flag,&buf[20],1);
		memcpy(&lis_VIEWTIME->approx_threshold,&buf[21],1);
		return 0;
}

long READ_lis_viewtimes(long file_number,struct lis_viewtime *lis_viewtime_var,long record, long numrec){
	unsigned char *buf;
	long status,i;
		buf = malloc((size_t)(22*sizeof(unsigned char)*numrec));
		status = READ_lis_VSGROUPS(buf,lis_viewtime_vsref[file_number],&record,numrec);
		for (i=0; i<numrec; i++) {		
			memcpy(&lis_viewtime_var[i].location,&buf[i*22+0],8);
			memcpy(&lis_viewtime_var[i].TAI93_start,&buf[i*22+8],4);
			memcpy(&lis_viewtime_var[i].TAI93_end,&buf[i*22+12],4);
			memcpy(&lis_viewtime_var[i].effective_obs,&buf[i*22+16],4);
			memcpy(&lis_viewtime_var[i].alert_flag,&buf[i*22+20],1);
			memcpy(&lis_viewtime_var[i].approx_threshold,&buf[i*22+21],1);
		}
		free(buf);
		return 0;
}

long READ_lis_bg_summary(long file_number,struct lis_bg_summary *lis_BG_SUMMARY,long record){
	unsigned char buf[128];
	long status;
		if((status=READ_lis_VSGROUP(buf,lis_bg_summary_vsref[file_number],&record))!=record)
			return status;
#ifdef DEBUG
printf("%d:: ",record);
for(i=0;i<52;i++)printf("%02x ",buf[i]);
printf("\n");
#endif
		memcpy(&lis_BG_SUMMARY->TAI93_time,&buf[0],8);
		memcpy(&lis_BG_SUMMARY->address,&buf[8],4);
		memcpy(&lis_BG_SUMMARY->boresight,&buf[12],8);
		memcpy(&lis_BG_SUMMARY->corners,&buf[20],32);
		return 0;
}

long READ_lis_bg_summaries(long file_number,struct lis_bg_summary *lis_bg_summary_var,long record, long numrec){
	unsigned char *buf;
	long status,i;
		buf = malloc((size_t)(52*sizeof(unsigned char)*numrec));
		status = READ_lis_VSGROUPS(buf,lis_bg_summary_vsref[file_number],&record,numrec);
		for (i=0; i<numrec; i++) {		
			memcpy(&lis_bg_summary_var[i].TAI93_time,&buf[i*52+0],8);
			memcpy(&lis_bg_summary_var[i].address,&buf[i*52+8],4);
			memcpy(&lis_bg_summary_var[i].boresight,&buf[i*52+12],8);
			memcpy(&lis_bg_summary_var[i].corners,&buf[i*52+20],32);
		}
		free(buf);
		return 0;
}


long READ_lis_orbit_summary(long file_number,struct lis_orbit_summary *lis_ORBIT_SUMMARY,long record){
	unsigned char buf[128];
	long status;
		if((status=READ_lis_VSGROUP(buf,lis_orbit_summary_vsref[file_number],&record))!=record)
			return status;
#ifdef DEBUG
printf("%d:: ",record);
for(i=0;i<84;i++)printf("%02x ",buf[i]);
printf("\n");
#endif
		memcpy(&lis_ORBIT_SUMMARY->id_number,&buf[0],4);
		memcpy(&lis_ORBIT_SUMMARY->TAI93_start,&buf[4],8);
		memcpy(&lis_ORBIT_SUMMARY->UTC_start,&buf[12],28);
		memcpy(&lis_ORBIT_SUMMARY->GPS_start,&buf[40],8);
		memcpy(&lis_ORBIT_SUMMARY->TAI93_end,&buf[48],8);
		memcpy(&lis_ORBIT_SUMMARY->start_longitude,&buf[56],4);
		memcpy(&lis_ORBIT_SUMMARY->end_longitude,&buf[60],4);
		memcpy(&lis_ORBIT_SUMMARY->point_data_count,&buf[64],2);
		memcpy(&lis_ORBIT_SUMMARY->point_data_address,&buf[66],2);
		memcpy(&lis_ORBIT_SUMMARY->one_second_count,&buf[68],4);
		memcpy(&lis_ORBIT_SUMMARY->one_second_address,&buf[72],4);
		memcpy(&lis_ORBIT_SUMMARY->summary_image_count,&buf[76],2);
		memcpy(&lis_ORBIT_SUMMARY->summary_image_address,&buf[78],2);
		memcpy(&lis_ORBIT_SUMMARY->inspection_code,&buf[80],2);
		memcpy(&lis_ORBIT_SUMMARY->configuration_code,&buf[82],2);


		return 0;
}
long READ_lis_one_second(long file_number,struct lis_one_second *lis_ONE_SECOND,long record){
	unsigned char buf[256];
	long status;
		if((status=READ_lis_VSGROUP(buf,lis_one_second_vsref[file_number],&record))!=record)
			return status;
#ifdef DEBUG
printf("%d:: ",record);
for(i=0;i<119;i++)printf("%02x ",buf[i]);
printf("\n");
#endif
		memcpy(&lis_ONE_SECOND->TAI93_time,&buf[0],8);
		memcpy(&lis_ONE_SECOND->alert_summary,&buf[8],1);
		memcpy(&lis_ONE_SECOND->instrument_alert,&buf[9],1);
		memcpy(&lis_ONE_SECOND->platform_alert,&buf[10],1);
		memcpy(&lis_ONE_SECOND->external_alert,&buf[11],1);
		memcpy(&lis_ONE_SECOND->processing_alert,&buf[12],1);
		memcpy(&lis_ONE_SECOND->position_vector,&buf[13],12);
		memcpy(&lis_ONE_SECOND->velocity_vector,&buf[25],12);
		memcpy(&lis_ONE_SECOND->transform_matrix,&buf[37],36);
		memcpy(&lis_ONE_SECOND->solar_vector,&buf[73],12);
		memcpy(&lis_ONE_SECOND->ephemeris_quality_flag,&buf[85],4);
		memcpy(&lis_ONE_SECOND->attitude_quality_flag,&buf[89],4);
		memcpy(&lis_ONE_SECOND->boresight_threshold,&buf[93],1);
		memcpy(&lis_ONE_SECOND->thresholds,&buf[94],12);
		memcpy(&lis_ONE_SECOND->noise_index,&buf[110],1);
#if defined(MAC) || defined(macintosh)	
/*
		memcpy(&lis_ONE_SECOND->event_count,&buf[111],12);
*/
#else
		memcpy(&lis_ONE_SECOND->event_count,&buf[111],12);
#endif

		return 0;
}

long READ_lis_one_seconds(long file_number,struct lis_one_second *lis_one_second_var,long record,long numrec){
	unsigned char *buf;
	long status,i;
		buf = malloc((size_t)(123*sizeof(unsigned char)*numrec));
		status = READ_lis_VSGROUPS(buf,lis_one_second_vsref[file_number],&record,numrec);
		for (i=0; i<numrec; i++) {
			memcpy(&lis_one_second_var[i].TAI93_time,&buf[i*123+0],8);
			memcpy(&lis_one_second_var[i].alert_summary,&buf[i*123+8],1);
			memcpy(&lis_one_second_var[i].instrument_alert,&buf[i*123+9],1);
			memcpy(&lis_one_second_var[i].platform_alert,&buf[i*123+10],1);
			memcpy(&lis_one_second_var[i].external_alert,&buf[i*123+11],1);
			memcpy(&lis_one_second_var[i].processing_alert,&buf[i*123+12],1);
			memcpy(&lis_one_second_var[i].position_vector,&buf[i*123+13],12);
			memcpy(&lis_one_second_var[i].velocity_vector,&buf[i*123+25],12);
			memcpy(&lis_one_second_var[i].transform_matrix,&buf[i*123+37],36);
			memcpy(&lis_one_second_var[i].solar_vector,&buf[i*123+73],12);
			memcpy(&lis_one_second_var[i].ephemeris_quality_flag,&buf[i*123+85],4);
			memcpy(&lis_one_second_var[i].attitude_quality_flag,&buf[i*123+89],4);
			memcpy(&lis_one_second_var[i].boresight_threshold,&buf[i*123+93],1);
			memcpy(&lis_one_second_var[i].thresholds,&buf[i*123+94],16);
			memcpy(&lis_one_second_var[i].noise_index,&buf[i*123+110],1);
			memcpy(&lis_one_second_var[i].event_count,&buf[i*123+111],12);
		}
		free(buf);
		return 0;
}
long READ_lis_bg_orbit_summary(long file_number,struct lis_bg_orbit_summary *lis_BG_ORBIT_SUMMARY,long record){
	unsigned char buf[128];
	long status;
		if((status=READ_lis_VSGROUP(buf,lis_bg_orbit_summary_vsref[file_number],&record))!=record)
			return status;
#ifdef DEBUG
printf("%d:: ",record);
for(i=0;i<72;i++)printf("%02x ",buf[i]);
printf("\n");
#endif
		memcpy(&lis_BG_ORBIT_SUMMARY->id_number,&buf[0],4);
		memcpy(&lis_BG_ORBIT_SUMMARY->TAI93_start,&buf[4],8);
		memcpy(&lis_BG_ORBIT_SUMMARY->UTC_start,&buf[12],28);
		memcpy(&lis_BG_ORBIT_SUMMARY->GPS_start,&buf[40],8);
		memcpy(&lis_BG_ORBIT_SUMMARY->TAI93_end,&buf[48],8);
		memcpy(&lis_BG_ORBIT_SUMMARY->start_longitude,&buf[56],4);
		memcpy(&lis_BG_ORBIT_SUMMARY->end_longitude,&buf[60],4);
		memcpy(&lis_BG_ORBIT_SUMMARY->image_summary_count,&buf[64],2);
		memcpy(&lis_BG_ORBIT_SUMMARY->image_summary_address,&buf[66],2);
		memcpy(&lis_BG_ORBIT_SUMMARY->image_info_count,&buf[68],2);
		memcpy(&lis_BG_ORBIT_SUMMARY->image_info_address,&buf[70],2);

		return 0;
}


long READ_lis_bg_info(long file_number,struct lis_bg_info *lis_BG_INFO,long record){
	unsigned char buf[256];
	long status;
		if((status=READ_lis_VSGROUP(buf,lis_bg_info_vsref[file_number],&record))!=record)
			return status;
#ifdef DEBUG
printf("%d:: ",record);
for(i=0;i<108;i++)printf("%02x ",buf[i]);
printf("\n");
#endif
		memcpy(&lis_BG_INFO->TAI93_time,&buf[0],8);
		memcpy(&lis_BG_INFO->alert_summary,&buf[8],1);
		memcpy(&lis_BG_INFO->instrument_alert,&buf[9],1);
		memcpy(&lis_BG_INFO->platform_alert,&buf[10],1);
		memcpy(&lis_BG_INFO->external_alert,&buf[11],1);
		memcpy(&lis_BG_INFO->processing_alert,&buf[12],1);
		memcpy(&lis_BG_INFO->position_vector,&buf[13],12);
		memcpy(&lis_BG_INFO->velocity_vector,&buf[25],12);
		memcpy(&lis_BG_INFO->transform_matrix,&buf[37],36);
		memcpy(&lis_BG_INFO->solar_vector,&buf[73],12);
		memcpy(&lis_BG_INFO->ephemeris_quality_flag,&buf[85],4);
		memcpy(&lis_BG_INFO->attitude_quality_flag,&buf[89],4);
		memcpy(&lis_BG_INFO->bg_value,&buf[93],2);
		memcpy(&lis_BG_INFO->noise_index,&buf[95],1);
		memcpy(&lis_BG_INFO->event_count,&buf[96],12);

		return 0;
}


long READ_lis_bg_data_summary(long file_number,struct lis_bg_data_summary *lis_BG_DATA_SUMMARY,long record){
	unsigned char buf[128];
	long status;
		if((status=READ_lis_VSGROUP(buf,lis_bg_data_summary_vsref[file_number],&record))!=record)
			return status;
#ifdef DEBUG
printf("%d:: ",record);
for(i=0;i<52;i++)printf("%02x ",buf[i]);
printf("\n");
#endif
		memcpy(&lis_BG_DATA_SUMMARY->TAI93_time,&buf[0],8);
		memcpy(&lis_BG_DATA_SUMMARY->address,&buf[8],4);
		memcpy(&lis_BG_DATA_SUMMARY->boresight,&buf[12],8);
		memcpy(&lis_BG_DATA_SUMMARY->corners,&buf[20],32);
		return 0;
}

#define DBG FALSE

#ifdef DEBUG
#define DBG TRUE
#endif


long read_otd_SDS(void *databuffer,long sds_id, long srecord);
long read_otd_VSGROUP(void *databuffer,long vsref,long *vsrecord);

long get_otd_totals(long vsref);

long open_OTD_HDF_READONLY(char *hdf_fname)
/****************************************************************************************/
/*											*/
/*	Open a HDF file for read access							*/
/*											*/
/*	returns the zero (0) id if successful						*/
/*	returns nonzero on error							*/
/*											*/
/*	Date last modified	6/1/96							*/
/****************************************************************************************/
{
	long temp_id,temp_vref,success_flag;
	char temp_space[1024], *temp_name;
	long get_total_otd_image();
	

/************** open hdf file for reading  scientific data sets (SDS) *******************/
	success_flag = otd_SUCCESS;
	otd_hdf_id = FAIL;
	otd_image_srecord = 0;
	otd_image_sds_id = FAIL;
	otd_sd_id = FAIL;
	otd_sd_id = SDstart(hdf_fname,DFACC_RDONLY);
	if (otd_sd_id==FAIL){if(DBG)printf(" SDstart failed\n");success_flag = otd_SDstart_FAIL; }
	if((otd_image_sds_id = SDselect(otd_sd_id,SDnametoindex(otd_sd_id,"Background images")))==FAIL){
		if(DBG)printf("image_sds_id failed\n");
		success_flag =  success_flag|=otd_SDselect_FAIL; 
	}
/*
	if(SDreadattr(image_sds_id,SDfindattr(image_sds_id,"total images"),&image_count)==FAIL){
		return  (FAIL);  
*/

/************** open hdf file for reading Vsets and Vdatas ******************************/

	otd_hdf_id = FAIL;
	otd_hdf_id = Hopen(hdf_fname,DFACC_RDONLY,0); 
	if(otd_hdf_id==FAIL){
		if(DBG)printf("Hopen failed\n");
		success_flag|=otd_Hopen_FAIL;
		return  (success_flag); 
	}
	Vstart(otd_hdf_id);

/****************************** attach vgroups ******************************************/


	/********************* initailize vgroup id to FAIL ***************************/

	otd_area_vgroup_id = FAIL;
	otd_flash_vgroup_id = FAIL;
	otd_group_vgroup_id = FAIL;
	otd_event_vgroup_id = FAIL;
	otd_browse_vgroup_id = FAIL;
	otd_orbit_summary_vgroup_id = FAIL;
	otd_flash_density_vgroup_id = FAIL;
	
	/****************** search for individual vgroup ids **************************************/

	temp_name = &temp_space[0];
	temp_vref = -1;

	if((temp_vref=Vgetid(otd_hdf_id,temp_vref))==FAIL)
		{if(DBG)printf("Vgetid search failed\n");
		success_flag|=otd_area_vg_search_FAIL;
		success_flag|=otd_flash_vg_search_FAIL;
		success_flag|=otd_group_vg_search_FAIL;
		success_flag|=otd_event_vg_search_FAIL;
		success_flag|=otd_browse_vg_search_FAIL;
		success_flag|=otd_orbit_summary_vg_search_FAIL;
		success_flag|=otd_flash_density_vg_search_FAIL;
	 }

	temp_id = Vattach(otd_hdf_id,temp_vref,"r");

	if(temp_id!=FAIL)Vgetname(temp_id,temp_space);

	while (temp_vref !=FAIL){



		if(strcmp(temp_name,"Area")==0)
			otd_area_vgroup_id = temp_id;
		else if(strcmp(temp_name,"Flash")==0)
			otd_flash_vgroup_id = temp_id;
		else if(strcmp(temp_name,"Group")==0)
			otd_group_vgroup_id = temp_id;
		else if(strcmp(temp_name,"Event")==0)
			otd_event_vgroup_id = temp_id;
		else if(strcmp(temp_name,"Browse")==0)
			otd_browse_vgroup_id = temp_id;
		else if(strcmp(temp_name,"Orbit summary")==0)
			otd_orbit_summary_vgroup_id = temp_id;
		else if(strcmp(temp_name,"Flash Density")==0)
			otd_flash_density_vgroup_id = temp_id;
		else{
			if(temp_id!=FAIL)Vdetach(temp_id);
		}
		temp_vref=Vgetid(otd_hdf_id,temp_vref);



		if(temp_vref!=FAIL){;
			temp_id = Vattach(otd_hdf_id,temp_vref,"r");
			if(temp_id!=FAIL)Vgetname(temp_id,temp_space);

		}
	}

	if(otd_area_vgroup_id == FAIL)
		{if(DBG)printf("area_vgroup_id failed\n");success_flag|=otd_area_vg_search_FAIL;}
	if(otd_flash_vgroup_id == FAIL)
		{if(DBG)printf("flash_vgroup_id failed\n");success_flag|=otd_flash_vg_search_FAIL;}
	if(otd_group_vgroup_id == FAIL)
		{if(DBG)printf("group_vgroup_id failed\n");success_flag|=otd_group_vg_search_FAIL;}
	if(otd_event_vgroup_id == FAIL)
		{if(DBG)printf("event_vgroup_id failed\n");success_flag|=otd_event_vg_search_FAIL;}

	if(otd_browse_vgroup_id == FAIL)
		{if(DBG)printf("browse_vgroup_id failed\n");success_flag|=otd_browse_vg_search_FAIL;}
	if(otd_flash_density_vgroup_id == FAIL)
		{if(DBG)printf("flash_density_vgroup_id failed\n");success_flag|=otd_flash_density_vg_search_FAIL;}
	if(otd_orbit_summary_vgroup_id == FAIL)
		{if(DBG)printf("orbit_summary_vgroup_id failed\n");success_flag|=otd_orbit_summary_vg_search_FAIL;}


	/****************************** attach vdatas *******************************************/


	/*********************** initialize to FAIL *************************************/

	otd_area_vsref = FAIL;
	otd_flash_vsref = FAIL;
	otd_group_vsref = FAIL;
	otd_event_vsref = FAIL;
	otd_browse_vsref = FAIL;
	otd_image_vsref = FAIL;
	otd_vector_vsref = FAIL;
	otd_metadata_vsref = FAIL;
	otd_orbit_vsref = FAIL;
	otd_orbit_summary_vsref = FAIL;
	otd_orbit_flash_rate_vsref = FAIL;
	otd_orbit_event_rate_vsref = FAIL;
	otd_ephemeris_vsref = FAIL;
	otd_view_vsref = FAIL;
	otd_grid_25_vsref = FAIL;
	otd_grid_500_km_vsref = FAIL;
	otd_threshold_vsref = FAIL;

	/************************ search for individual vsrefs **************************/

	otd_area_vsref = VSattach(otd_hdf_id,VSfind(otd_hdf_id,"area"),"r");
	otd_flash_vsref = VSattach(otd_hdf_id,VSfind(otd_hdf_id,"flash"),"r");
	otd_group_vsref = VSattach(otd_hdf_id,VSfind(otd_hdf_id,"group"),"r");
	otd_event_vsref = VSattach(otd_hdf_id,VSfind(otd_hdf_id,"event"),"r");
	otd_browse_vsref = VSattach(otd_hdf_id,VSfind(otd_hdf_id,"browse"),"r");
	otd_image_vsref = VSattach(otd_hdf_id,VSfind(otd_hdf_id,"image"),"r");
	otd_vector_vsref = VSattach(otd_hdf_id,VSfind(otd_hdf_id,"vector"),"r");
	otd_metadata_vsref = VSattach(otd_hdf_id,VSfind(otd_hdf_id,"metadata"),"r");
	otd_inventory_metadata_vsref = VSattach(otd_hdf_id,VSfind(otd_hdf_id,"coremetadata.0"),"r");
	otd_archived_metadata_vsref = VSattach(otd_hdf_id,VSfind(otd_hdf_id,"productmetadata.0"),"r");
	otd_orbit_vsref = VSattach(otd_hdf_id,VSfind(otd_hdf_id,"orbit"),"r");
	otd_orbit_summary_vsref = VSattach(otd_hdf_id,VSfind(otd_hdf_id,"summary"),"r");
	otd_orbit_flash_rate_vsref = VSattach(otd_hdf_id,VSfind(otd_hdf_id,"rate_f"),"r");
	otd_orbit_event_rate_vsref = VSattach(otd_hdf_id,VSfind(otd_hdf_id,"rate_e"),"r");
	otd_ephemeris_vsref = VSattach(otd_hdf_id,VSfind(otd_hdf_id,"ephemeris"),"r");
	otd_view_vsref = VSattach(otd_hdf_id,VSfind(otd_hdf_id,"view time"),"r");
	otd_grid_25_vsref = VSattach(otd_hdf_id,VSfind(otd_hdf_id,"2.5 degree"),"r");
	otd_grid_500_km_vsref = VSattach(otd_hdf_id,VSfind(otd_hdf_id,"500km"),"r");
	otd_threshold_vsref = VSattach(otd_hdf_id,VSfind(otd_hdf_id,"threshold"),"r");

	if(otd_area_vsref==FAIL)
		{if(DBG)printf("area_vsref failed\n");success_flag|= otd_area_VSattach_FAIL;}
	if(otd_flash_vsref==FAIL)
		{if(DBG)printf("flash_vsref failed\n");success_flag|= otd_flash_VSattach_FAIL;}
	if(otd_group_vsref==FAIL)
		{if(DBG)printf("group_vsref failed\n");success_flag|= otd_group_VSattach_FAIL;}
	if(otd_event_vsref==FAIL)
		{if(DBG)printf("event_vsref failed\n");success_flag|= otd_event_VSattach_FAIL;}
	if(otd_image_vsref==FAIL)
		{if(DBG)printf("image_vsref failed\n");success_flag|= otd_image_VSattach_FAIL;}
	if(otd_vector_vsref==FAIL)
		{if(DBG)printf("vector_vsref failed\n");success_flag|= otd_vector_VSattach_FAIL;}
	if(otd_metadata_vsref==FAIL)
		{if(DBG)printf("metadata_vsref failed\n");success_flag|= otd_metadata_VSattach_FAIL;}
	if(otd_inventory_metadata_vsref==FAIL)
		{if(DBG)printf("inventory metadata_vsref failed\n");success_flag|= otd_inventory_metadata_FAIL;}
	if(otd_archived_metadata_vsref==FAIL)
		{if(DBG)printf("archived metadata_vsref failed\n");success_flag|= otd_archived_metadata_FAIL;}
	if(otd_orbit_vsref==FAIL)
		{if(DBG)printf("orbit_vsref failed\n");success_flag|= otd_orbit_VSattach_FAIL;}
	if(otd_orbit_summary_vsref==FAIL)
		{if(DBG)printf("orbit_summary_vsref failed\n");success_flag |= otd_orbit_summary_VSattach_FAIL;}
	if(otd_grid_25_vsref==FAIL)
		{if(DBG)printf("grid_25_vsref failed\n");success_flag |= otd_grid_25_deg_VSattach_FAIL;}
	if(otd_view_vsref==FAIL)
		{if(DBG)printf("view_vsref failed\n");success_flag |=otd_view_VSattach_FAIL;}
	if(otd_grid_500_km_vsref==FAIL)
		{if(DBG)printf("grid_500_km_vsref failed\n");success_flag |= otd_grid_500_km_VSattach_FAIL;}
	if(otd_ephemeris_vsref==FAIL)
		{if(DBG)printf("ephemeris_vsref failed\n");success_flag |= otd_ephemeris_VSattach_FAIL;}
	if(otd_threshold_vsref==FAIL)
		{if(DBG)printf("threshold_vsref failed\n");success_flag |= otd_threshold_VSattach_FAIL;}
	if(otd_orbit_event_rate_vsref==FAIL)
		{if(DBG)printf("orbit_event_rate_vsref failed\n");success_flag |= otd_orbit_event_rate_VSattach_FAIL;}
	if(otd_orbit_flash_rate_vsref==FAIL)
		{if(DBG)printf("orbit_flash_rate_vsref failed\n");success_flag |= otd_orbit_flash_rate_VSattach_FAIL;}


	/**************** select fields to access ***********************/
	/********************** ALL *************************************/

	if(VSsetfields(otd_orbit_vsref,"orbit ID,TAI93 start,start location,TAI93 end,end location")==FAIL){
		if(DBG)printf("orbit_vsref VSsetfields failed\n");
		success_flag |=  otd_VSsetfields_FAIL; 
	}
	if(VSsetfields(otd_metadata_vsref,"text")==FAIL){
		if(DBG)printf("metadata_vsref VSsetfields failed\n");
		success_flag |=  otd_VSsetfields_FAIL; 
	}
	if(VSsetfields(otd_inventory_metadata_vsref,"VALUES")==FAIL){
		if(DBG)printf("inventory_metadata_vsref VSsetfields failed\n");
		success_flag |=  otd_VSsetfields_FAIL; 
	}
	if(VSsetfields(otd_archived_metadata_vsref,"VALUES")==FAIL){
		if(DBG)printf("archived_metadata_vsref VSsetfields failed\n");
		success_flag |=  otd_VSsetfields_FAIL; 
	}
	if(VSsetfields(otd_area_vsref,
		"seq,TAI93,delta,view,s-z-a,d-n-t,end status,events,cent,stdev,loc count,rad,orbit id,day,children,child seq,child rec,QA")==FAIL){
		if(DBG)printf("area_vsref VSsetfields failed\n");
		success_flag |=  otd_VSsetfields_FAIL; 
	}
	if(VSsetfields(otd_flash_vsref,
		"seq,TAI93,delta,view,s-z-a,d-n-t,end status,events,cent,stdev,loc count,rad,parent seq,parent rec,children,child seq,child rec,QA")==FAIL){   
		if(DBG)printf("flash_vsref VSsetfields failed\n");
		success_flag |=  otd_VSsetfields_FAIL; 
	}
	if(VSsetfields(otd_group_vsref,
		"seq,TAI93,delta,view,s-z-a,d-n-t,end status,events,cent,stdev,loc count,rad,parent seq,parent rec,children,child seq,child rec,QA")==FAIL){
		if(DBG)printf("group_vsref VSsetfields failed\n");
		success_flag |=  otd_VSsetfields_FAIL; 
	}
	if(VSsetfields(otd_event_vsref,
		"event #,seq. #,TAI93,s-z-a,d-n-t,x pixel,y pixel,raw radiance,cal radiance,parent seq,parent rec,QA,location")==FAIL){
		if(DBG)printf("event_vsref VSsetfields failed\n");
		success_flag |=  otd_VSsetfields_FAIL; 
	}
	if(VSsetfields(otd_browse_vsref,
		"location,EGFAs,EGFA cal rad,max rate,1 event areas,highres locs,1st highres,bkgnds,iloper,bkgnd pointer,duration")==FAIL){
		if(DBG)printf("browse_vsref VSsetfields failed\n");
		success_flag |=  otd_VSsetfields_FAIL; 
	}
	if(VSsetfields(otd_vector_vsref,
		"location,EGFA counts,EGFA cal radiances,max flash rate,single event areas")==FAIL){
		if(DBG)printf("vector_vsref VSsetfields failed\n");
		success_flag |=  otd_VSsetfields_FAIL; 
	}
	if(VSsetfields(otd_image_vsref,
		"TAI93,NW location,NE location, SW location,SE location,nadir location,image pointer,calibration number,spice pointer")==FAIL){
		if(DBG)printf("image_vsref VSsetfields failed\n");
		success_flag =  FAIL; 
	}
	if(VSsetfields(otd_orbit_summary_vsref,"areas,flashes,groups,events,backgrounds,total cal radiance")==FAIL){
		if(DBG)printf("orbit_summary_vsref VSsetfields failed\n");
		success_flag |=  otd_VSsetfields_FAIL; 
	}
	if(VSsetfields(otd_orbit_flash_rate_vsref,"flash rate")==FAIL){
		if(DBG)printf("orbit_flash_rate_vsref VSsetfields failed\n");
		success_flag |=  otd_VSsetfields_FAIL; 
	}
	if(VSsetfields(otd_orbit_event_rate_vsref,"count")==FAIL){
		if(DBG)printf("orbit_event_rate_vsref VSsetfields failed\n");
		success_flag |=  otd_VSsetfields_FAIL; 
	} 
	if(VSsetfields(otd_ephemeris_vsref,"GPS time,spice kernel")==FAIL){
		if(DBG)printf("ephemeris_vsref VSsetfields failed\n");
		success_flag |=  otd_VSsetfields_FAIL; 
	}
	if(VSsetfields(otd_view_vsref,"grid point,TAI93 start,duration")==FAIL){
		if(DBG)printf("view_vsref VSsetfields failed\n");
		success_flag |=  otd_VSsetfields_FAIL; 
	}  
	if(VSsetfields(otd_grid_25_vsref,"location,flash count")==FAIL){
		if(DBG)printf("grid_25_vsref VSsetfields failed\n");
		success_flag |=  otd_VSsetfields_FAIL; 
	}
	if(VSsetfields(otd_grid_500_km_vsref,"location,flash count")==FAIL){
		if(DBG)printf("grid_500_km_vsref VSsetfields failed\n");
		success_flag |=  otd_VSsetfields_FAIL; 
	} 
	if(VSsetfields(otd_threshold_vsref,"TAI93,values")==FAIL){
		if(DBG)printf("threshold_vsref VSsetfields failed\n");
		success_flag |=  otd_VSsetfields_FAIL; 
	}

	otd_image_count = get_total_otd_image();
	return (success_flag); 
}


long close_OTD_HDF_FILE()

/****************************************************************************************/
/*                                                                             		*/
/*	close_hdf_file									*/
/*											*/
/*	returns 0 if all vgroups, vsets and the file itself are successfully    	*/
/*	closed										*/
/*	returns FAIL(-1) on error							*/
/*											*/
/*	Date last modified 6/1/96							*/
/****************************************************************************************/

{
	long return_status;
	return_status = 0;

        if(otd_orbit_vsref!= FAIL)VSdetach(otd_orbit_vsref);			/* orbit */
 
        if(otd_metadata_vsref!= FAIL)VSdetach(otd_metadata_vsref);		/* metadata */
        if(otd_inventory_metadata_vsref!= FAIL)VSdetach(otd_inventory_metadata_vsref);		/* metadata */
        if(otd_archived_metadata_vsref!= FAIL)VSdetach(otd_archived_metadata_vsref);		/* metadata */

        if(otd_threshold_vsref!= FAIL)VSdetach(otd_threshold_vsref);		/* threshold */
 
        if(otd_event_vsref!= FAIL)VSdetach(otd_event_vsref);			/* Events */
	if(otd_event_vgroup_id!= FAIL)Vdetach(otd_event_vgroup_id);

	if(otd_group_vsref!= FAIL)VSdetach(otd_group_vsref);			/* Group */
	if(otd_group_vgroup_id!= FAIL)Vdetach(otd_group_vgroup_id);

	if(otd_flash_vsref!= FAIL)VSdetach(otd_flash_vsref);			/* Flash */
	if(otd_flash_vgroup_id!= FAIL)Vdetach(otd_flash_vgroup_id);

	if(otd_area_vsref!= FAIL)VSdetach(otd_area_vsref);			/* Area */
	if(otd_area_vgroup_id!= FAIL)Vdetach(otd_area_vgroup_id);
	

	if(otd_vector_vsref!= FAIL)VSdetach(otd_vector_vsref);			/* Browse */
	if(otd_image_vsref!= FAIL)VSdetach(otd_image_vsref);
	if(otd_browse_vsref!= FAIL)VSdetach(otd_browse_vsref);
	if(otd_browse_vgroup_id!= FAIL)Vdetach(otd_browse_vgroup_id);

	if(otd_orbit_flash_rate_vsref!= FAIL)VSdetach(otd_orbit_flash_rate_vsref);		/* Orbit Summary */
	if(otd_orbit_event_rate_vsref!= FAIL)VSdetach(otd_orbit_event_rate_vsref);	
	if(otd_ephemeris_vsref!= FAIL)VSdetach(otd_ephemeris_vsref);
	if(otd_orbit_summary_vsref!= FAIL)VSdetach(otd_orbit_summary_vsref);
	if(otd_orbit_summary_vgroup_id!= FAIL)Vdetach(otd_orbit_summary_vgroup_id);

	if(otd_view_vsref!= FAIL)VSdetach(otd_view_vsref);					/* Flash Desnity */
	if(otd_grid_25_vsref!= FAIL)VSdetach(otd_grid_25_vsref);					/* Flash Desnity */
	if(otd_grid_500_km_vsref!= FAIL)VSdetach(otd_grid_500_km_vsref);			
	if(otd_flash_density_vgroup_id!= FAIL)Vdetach(otd_flash_density_vgroup_id);

	if(SDsetattr(otd_image_sds_id,"total images",DFNT_INT32,1,&otd_image_srecord)==FAIL)
	return_status = FAIL;

	if(otd_image_sds_id!=FAIL)if(SDendaccess(otd_image_sds_id)==FAIL)return_status = FAIL;		/* image */
		 

 	if(otd_sd_id!=FAIL)if (SDend(otd_sd_id)==FAIL)return_status = FAIL;					/* SD id */

	if(otd_hdf_id!=FAIL)Vend(otd_hdf_id);

	if(otd_hdf_id !=FAIL)if(Hclose(otd_hdf_id)==FAIL)return_status = FAIL;	

	return (return_status);						/* success*/

}

long read_otd_IMAGE(short int *otd_image,long *vsrecord)

/****************************************************************************************/
/*											*/
/*	read Background image (SDS) from HDF file					*/
/*											*/
/*	returns the Srecord number of the image if successful				*/
/*	returns FAIL (-1) on error							*/
/*											*/
/*	Date last modified	6/1/96							*/
/****************************************************************************************/


{
	start[0] = *vsrecord;
	start[1]=start[2]=0;

	endr[0]=1;
	endr[1]=endr[2]=128;


	if(otd_image_count<*vsrecord)return (FAIL);
	if(SDreaddata(otd_image_sds_id,start,NULL,endr,otd_image)==FAIL)return (FAIL);
	return (*vsrecord);	
}





long read_otd_EVENT(void *databuffer,long *otd_event_vrecord)	
	{return (read_otd_VSGROUP(databuffer,otd_event_vsref,otd_event_vrecord));}
long read_otd_GROUP(void *databuffer,long *otd_group_vrecord) 
	{return (read_otd_VSGROUP(databuffer,otd_group_vsref,otd_group_vrecord));}
long read_otd_FLASH(void *databuffer,long *otd_flash_vrecord) 
	{return (read_otd_VSGROUP(databuffer,otd_flash_vsref,otd_flash_vrecord));}
long read_otd_AREA(void *databuffer,long *otd_area_vrecord)	
	{return (read_otd_VSGROUP(databuffer,otd_area_vsref,otd_area_vrecord));}
long read_otd_VECTOR_PRODUCTS(void *databuffer,long *otd_vector_vrecord) 
	{return (read_otd_VSGROUP(databuffer,otd_vector_vsref,otd_vector_vrecord));}
long read_otd_IMAGE_ATTRIBUTE(void *databuffer,long *otd_image_vrecord)  
	{return (read_otd_VSGROUP(databuffer,otd_image_vsref,otd_image_vrecord));}
long read_otd_ORBIT_DATA(void *databuffer,long *otd_orbit_vrecord)	
	{return (read_otd_VSGROUP(databuffer,otd_orbit_vsref,otd_orbit_vrecord));}
long read_otd_METADATA(void *databuffer,long *otd_metadata_vrecord)	
	{return (read_otd_VSGROUP(databuffer,otd_metadata_vsref,otd_metadata_vrecord));}
long read_otd_INVENTORY_METADATA(void *databuffer,long *otd_inventory_metadata_vrecord)	
	{return (read_otd_VSGROUP(databuffer,otd_inventory_metadata_vsref,otd_inventory_metadata_vrecord));}
long read_otd_ARCHIVED_METADATA(void *databuffer,long *otd_archived_metadata_vrecord)	
	{return (read_otd_VSGROUP(databuffer,otd_archived_metadata_vsref,otd_archived_metadata_vrecord));}
long read_otd_BROWSE_DATA(void *databuffer,long *otd_browse_data_vrecord) 
	{return (read_otd_VSGROUP(databuffer,otd_browse_vsref,otd_browse_data_vrecord));}

long read_otd_ORBIT_SUMMARY(void *databuffer,long *otd_orbit_summary_vrecord) 
	{return (read_otd_VSGROUP(databuffer,otd_orbit_summary_vsref,otd_orbit_summary_vrecord));}       
long read_otd_ORBIT_FLASH_RATE(void *databuffer,long *otd_orbit_flash_rate_vrecord)
	{return (read_otd_VSGROUP(databuffer,otd_orbit_flash_rate_vsref,otd_orbit_flash_rate_vrecord));}
long read_otd_ORBIT_EVENT_RATE(void *databuffer,long *otd_orbit_event_rate_vrecord)
	{return (read_otd_VSGROUP(databuffer,otd_orbit_event_rate_vsref,otd_orbit_event_rate_vrecord));}
long read_otd_EPHEMERIS(void *databuffer,long *otd_ephemeris_vrecord) 
	{return (read_otd_VSGROUP(databuffer,otd_ephemeris_vsref,otd_ephemeris_vrecord));}

long read_otd_VIEW_TIME(void *databuffer,long *otd_view_vrecord) 
	{return (read_otd_VSGROUP(databuffer,otd_view_vsref,otd_view_vrecord));}
long read_otd_GRID_25(void *databuffer,long *otd_grid_25_vrecord) 
	{return (read_otd_VSGROUP(databuffer,otd_grid_25_vsref,otd_grid_25_vrecord));}
long read_otd_GRID_500_KM(void *databuffer,long *otd_grid_500_km_vrecord) 
	{return (read_otd_VSGROUP(databuffer,otd_grid_500_km_vsref,otd_grid_500_km_vrecord));}
long read_otd_THRESHOLD(void *databuffer,long *otd_threshold_vrecord) 
	{return (read_otd_VSGROUP(databuffer,otd_threshold_vsref,otd_threshold_vrecord));}

long read_otd_VSGROUP(void *databuffer,long vsref,long *vsrecord)

/****************************************************************************************/
/*											*/
/*	read data to HDF file								*/
/*											*/
/*	returns Vdata record # if successful						*/
/*	returns FAIL (-1) on error							*/
/*											*/
/*	Date last modified	6/1/96							*/
/****************************************************************************************/


{int j;
	if(vsref==FAIL)return(FAIL);
	if (VSelts(vsref) <= *vsrecord) return (FAIL);
	if (VSseek(vsref,*vsrecord)==FAIL) return FAIL;
 	if((j=VSread(vsref,databuffer,1,FULL_INTERLACE))==FAIL) return (FAIL);
	return (*vsrecord);
}

long READ_otd_VSGROUPS(void *databuffer,long vsref,long *vsrecord,long numrec)

/****************************************************************************************/
/*											*/
/*	read data to HDF file								*/
/*											*/
/*	returns Vdata record # if successful						*/
/*	returns FAIL (-1) on error							*/
/*											*/
/*	Date last modified	6/1/96							*/
/****************************************************************************************/


{int j;
	if(vsref==FAIL)return(FAIL);
	if (VSelts(vsref) <= *vsrecord) return (FAIL);
	if (VSseek(vsref,*vsrecord)==FAIL) return FAIL;
 	if((j=VSread(vsref,databuffer,numrec,FULL_INTERLACE))==FAIL) return (FAIL);
	return (*vsrecord);
}

long get_total_otd_areas()		{return (get_otd_totals(otd_area_vsref));}
long get_total_otd_flashes()	{return (get_otd_totals(otd_flash_vsref));}
long get_total_otd_groups()		{return (get_otd_totals(otd_group_vsref));}
long get_total_otd_events()		{return (get_otd_totals(otd_event_vsref));}
long get_total_otd_metadata()	{return (get_otd_totals(otd_metadata_vsref));}
long get_total_otd_threshold()	{return (get_otd_totals(otd_threshold_vsref));}
long get_total_otd_browse()		{return (get_otd_totals(otd_browse_vsref));}
long get_total_otd_image()		{return (get_otd_totals(otd_image_vsref));}
long get_total_otd_25_grid()	{return (get_otd_totals(otd_grid_25_vsref));}
long get_total_otd_500_km()		{return (get_otd_totals(otd_grid_500_km_vsref));}
long get_total_otd_view_time()	{return (get_otd_totals(otd_view_vsref));}
long get_total_otd_ephemeris()	{return (get_otd_totals(otd_ephemeris_vsref));}
long get_total_otd_flash_rate()	{return (get_otd_totals(otd_orbit_flash_rate_vsref));}
long get_total_otd_event_rate()	{return (get_otd_totals(otd_orbit_event_rate_vsref));}
long get_total_otd_orbit_attribute(){return (get_otd_totals(otd_orbit_vsref));}
long get_total_otd_vector()		{return (get_otd_totals(otd_vector_vsref));}
long get_total_otd_orbit_summary()	{return (get_otd_totals(otd_orbit_summary_vsref));}

/****************************************************************************************/
/*											*/
/*	read vdata record number 							*/
/*											*/
/*	returns total Vdata record # if successful					*/
/*	returns FAIL (-1) on error							*/
/*											*/
/*	Date last modified	6/1/96							*/
/****************************************************************************************/

long get_otd_totals(long vsref)
{
 long return_flag;
 int32 record_count;
	if(vsref==FAIL)return (FAIL);
	return_flag = VSinquire(vsref,&record_count,NULL,NULL,NULL,NULL);
	if (return_flag == -1)
		return (return_flag);
	else
		return (record_count);
}

long read_otd_area_struct(struct otd_AREA *otd_area,long record){
	unsigned char buf[128];
		if(read_otd_AREA(buf,&record)==FAIL)return -1;
		memcpy(&otd_area->seq,&buf[0],4);
		memcpy(&otd_area->TAI93,&buf[4],8);
		memcpy(&otd_area->delta,&buf[12],4);
		memcpy(&otd_area->view,&buf[16],4);
		memcpy(&otd_area->s_z_a,&buf[20],2);
		memcpy(&otd_area->d_n_t,&buf[22],1);
		memcpy(&otd_area->end_status,&buf[23],1);
		memcpy(&otd_area->events,&buf[24],2);
		memcpy(&otd_area->cent[0],&buf[26],8);
		memcpy(&otd_area->stdev[0],&buf[34],8);
		memcpy(&otd_area->loc_count,&buf[42],2);
		memcpy(&otd_area->rad,&buf[44],4);
		memcpy(&otd_area->orbit_id,&buf[48],4);
		memcpy(&otd_area->day,&buf[52],4);
		memcpy(&otd_area->children,&buf[56],4);
		memcpy(&otd_area->child_seq,&buf[60],4);
		memcpy(&otd_area->child_rec,&buf[64],4);
		memcpy(&otd_area->QA[0],&buf[68],8);
		return 0;
}

long READ_otd_areas(struct otd_AREA *otd_area_var, long record, long numrec) {
	unsigned char *buf;
	long status,i;
	
	buf = malloc((size_t)(76*sizeof(unsigned char)*numrec));
	status = READ_otd_VSGROUPS(buf,otd_area_vsref,&record,numrec);
	for (i=0; i<numrec; i++) {
			memcpy(&otd_area_var[i].seq,&buf[i*76+0],4);
			memcpy(&otd_area_var[i].TAI93,&buf[i*76+4],8);
			memcpy(&otd_area_var[i].delta,&buf[i*76+12],4);
			memcpy(&otd_area_var[i].view,&buf[i*76+16],4);
			memcpy(&otd_area_var[i].s_z_a,&buf[i*76+20],2);
			memcpy(&otd_area_var[i].d_n_t,&buf[i*76+22],1);
			memcpy(&otd_area_var[i].end_status,&buf[i*76+23],1);
			memcpy(&otd_area_var[i].events,&buf[i*76+24],2);
			memcpy(&otd_area_var[i].cent[0],&buf[i*76+26],8);
			memcpy(&otd_area_var[i].stdev[0],&buf[i*76+34],8);
			memcpy(&otd_area_var[i].loc_count,&buf[i*76+42],2);
			memcpy(&otd_area_var[i].rad,&buf[i*76+44],4);
			memcpy(&otd_area_var[i].orbit_id,&buf[i*76+48],4);
			memcpy(&otd_area_var[i].day,&buf[i*76+52],4);
			memcpy(&otd_area_var[i].children,&buf[i*76+56],4);
			memcpy(&otd_area_var[i].child_seq,&buf[i*76+60],4);
			memcpy(&otd_area_var[i].child_rec,&buf[i*76+64],4);
			memcpy(&otd_area_var[i].QA[0],&buf[i*76+68],8);
	}
	free(buf);
	return 0;
}

long read_otd_flash_struct(struct otd_FLASH *otd_flash,long record){
	unsigned char buf[128];
		if(read_otd_FLASH(buf,&record)==FAIL)return -1;
		memcpy(&otd_flash->seq,&buf[0],4);
		memcpy(&otd_flash->TAI93,&buf[4],8);
		memcpy(&otd_flash->delta,&buf[12],4);
		memcpy(&otd_flash->view,&buf[16],4);
		memcpy(&otd_flash->s_z_a,&buf[20],2);
		memcpy(&otd_flash->d_n_t,&buf[22],1);
		memcpy(&otd_flash->end_status,&buf[23],1);
		memcpy(&otd_flash->events,&buf[24],2);
		memcpy(&otd_flash->cent[0],&buf[26],8);
		memcpy(&otd_flash->stdev[0],&buf[34],8);
		memcpy(&otd_flash->loc_count,&buf[42],2);
		memcpy(&otd_flash->rad,&buf[44],4);
		memcpy(&otd_flash->parent_seq,&buf[48],4);
		memcpy(&otd_flash->parent_rec,&buf[52],4);
		memcpy(&otd_flash->children,&buf[56],4);
		memcpy(&otd_flash->child_seq,&buf[60],4);
		memcpy(&otd_flash->child_rec,&buf[64],4);
		memcpy(&otd_flash->QA[0],&buf[68],8);
		return 0;
}

long READ_otd_flashes(struct otd_FLASH *otd_flash_var, long record, long numrec) {
	unsigned char *buf;
	long status,i;
	
	buf = malloc((size_t)(76*sizeof(unsigned char)*numrec));
	status = READ_otd_VSGROUPS(buf,otd_flash_vsref,&record,numrec);
	for (i=0; i<numrec; i++) {
			memcpy(&otd_flash_var[i].seq,&buf[i*76+0],4);
			memcpy(&otd_flash_var[i].TAI93,&buf[i*76+4],8);
			memcpy(&otd_flash_var[i].delta,&buf[i*76+12],4);
			memcpy(&otd_flash_var[i].view,&buf[i*76+16],4);
			memcpy(&otd_flash_var[i].s_z_a,&buf[i*76+20],2);
			memcpy(&otd_flash_var[i].d_n_t,&buf[i*76+22],1);
			memcpy(&otd_flash_var[i].end_status,&buf[i*76+23],1);
			memcpy(&otd_flash_var[i].events,&buf[i*76+24],2);
			memcpy(&otd_flash_var[i].cent[0],&buf[i*76+26],8);
			memcpy(&otd_flash_var[i].stdev[0],&buf[i*76+34],8);
			memcpy(&otd_flash_var[i].loc_count,&buf[i*76+42],2);
			memcpy(&otd_flash_var[i].rad,&buf[i*76+44],4);
			memcpy(&otd_flash_var[i].parent_seq,&buf[i*76+48],4);
			memcpy(&otd_flash_var[i].parent_rec,&buf[i*76+52],4);
			memcpy(&otd_flash_var[i].children,&buf[i*76+56],4);
			memcpy(&otd_flash_var[i].child_seq,&buf[i*76+60],4);
			memcpy(&otd_flash_var[i].child_rec,&buf[i*76+64],4);
			memcpy(&otd_flash_var[i].QA[0],&buf[i*76+68],8);
	}
	free(buf);
	return 0;
}

long read_otd_group_struct(struct otd_GROUP *otd_group,long record){
	unsigned char buf[128];
		if(read_otd_GROUP(buf,&record)==FAIL)return -1;
		memcpy(&otd_group->seq,&buf[0],4);
		memcpy(&otd_group->TAI93,&buf[4],8);
		memcpy(&otd_group->delta,&buf[12],4);
		memcpy(&otd_group->view,&buf[16],4);
		memcpy(&otd_group->s_z_a,&buf[20],2);
		memcpy(&otd_group->d_n_t,&buf[22],1);
		memcpy(&otd_group->end_status,&buf[23],1);
		memcpy(&otd_group->events,&buf[24],2);
		memcpy(&otd_group->cent[0],&buf[26],8);
		memcpy(&otd_group->stdev[0],&buf[34],8);
		memcpy(&otd_group->loc_count,&buf[42],2);
		memcpy(&otd_group->rad,&buf[44],4);
		memcpy(&otd_group->parent_seq,&buf[48],4);
		memcpy(&otd_group->parent_rec,&buf[52],4);
		memcpy(&otd_group->children,&buf[56],4);
		memcpy(&otd_group->child_seq,&buf[60],4);
		memcpy(&otd_group->child_rec,&buf[64],4);
		memcpy(&otd_group->QA[0],&buf[68],8);
		return 0;
}

long READ_otd_groups(struct otd_GROUP *otd_group_var, long record, long numrec) {
	unsigned char *buf;
	long status,i;
	
	buf = malloc((size_t)(76*sizeof(unsigned char)*numrec));
	status = READ_otd_VSGROUPS(buf,otd_group_vsref,&record,numrec);
	for (i=0; i<numrec; i++) {
			memcpy(&otd_group_var[i].seq,&buf[i*76+0],4);
			memcpy(&otd_group_var[i].TAI93,&buf[i*76+4],8);
			memcpy(&otd_group_var[i].delta,&buf[i*76+12],4);
			memcpy(&otd_group_var[i].view,&buf[i*76+16],4);
			memcpy(&otd_group_var[i].s_z_a,&buf[i*76+20],2);
			memcpy(&otd_group_var[i].d_n_t,&buf[i*76+22],1);
			memcpy(&otd_group_var[i].end_status,&buf[i*76+23],1);
			memcpy(&otd_group_var[i].events,&buf[i*76+24],2);
			memcpy(&otd_group_var[i].cent[0],&buf[i*76+26],8);
			memcpy(&otd_group_var[i].stdev[0],&buf[i*76+34],8);
			memcpy(&otd_group_var[i].loc_count,&buf[i*76+42],2);
			memcpy(&otd_group_var[i].rad,&buf[i*76+44],4);
			memcpy(&otd_group_var[i].parent_seq,&buf[i*76+48],4);
			memcpy(&otd_group_var[i].parent_rec,&buf[i*76+52],4);
			memcpy(&otd_group_var[i].children,&buf[i*76+56],4);
			memcpy(&otd_group_var[i].child_seq,&buf[i*76+60],4);
			memcpy(&otd_group_var[i].child_rec,&buf[i*76+64],4);
			memcpy(&otd_group_var[i].QA[0],&buf[i*76+68],8);
	}
	free(buf);
	return 0;
}

long read_otd_event_struct(struct otd_EVENT *otd_event,long record){
	unsigned char buf[128];
		if(read_otd_EVENT(buf,&record)==FAIL)return -1;
		memcpy(&otd_event->event_number,&buf[0],4);
		memcpy(&otd_event->seq_number,&buf[4],4);
		memcpy(&otd_event->TAI93,&buf[8],8);
		memcpy(&otd_event->s_z_a,&buf[16],2);
		memcpy(&otd_event->d_n_t,&buf[18],1);
		memcpy(&otd_event->x_pixel,&buf[19],2);
		memcpy(&otd_event->y_pixel,&buf[21],2);
		memcpy(&otd_event->raw_radiance,&buf[23],2);
		memcpy(&otd_event->cal_radiance,&buf[25],4);
		memcpy(&otd_event->parent_seq,&buf[29],4);
		memcpy(&otd_event->parent_rec,&buf[33],4);
		memcpy(&otd_event->QA[0],&buf[37],8);
		memcpy(&otd_event->location,&buf[45],8);
		return 0;
}

long READ_otd_events(struct otd_EVENT *otd_event_var, long record, long numrec) {
	unsigned char *buf;
	long status,i;
	
	buf = malloc((size_t)(53*sizeof(unsigned char)*numrec));
	status = READ_otd_VSGROUPS(buf,otd_event_vsref,&record,numrec);
	for (i=0; i<numrec; i++) {
			memcpy(&otd_event_var[i].event_number,&buf[i*53+0],4);
			memcpy(&otd_event_var[i].seq_number,&buf[i*53+4],4);
			memcpy(&otd_event_var[i].TAI93,&buf[i*53+8],8);
			memcpy(&otd_event_var[i].s_z_a,&buf[i*53+16],2);
			memcpy(&otd_event_var[i].d_n_t,&buf[i*53+18],1);
			memcpy(&otd_event_var[i].x_pixel,&buf[i*53+19],2);
			memcpy(&otd_event_var[i].y_pixel,&buf[i*53+21],2);
			memcpy(&otd_event_var[i].raw_radiance,&buf[i*53+23],2);
			memcpy(&otd_event_var[i].cal_radiance,&buf[i*53+25],4);
			memcpy(&otd_event_var[i].parent_seq,&buf[i*53+29],4);
			memcpy(&otd_event_var[i].parent_rec,&buf[i*53+33],4);
			memcpy(&otd_event_var[i].QA[0],&buf[i*53+37],8);
			memcpy(&otd_event_var[i].location,&buf[i*53+45],8);	
	}
	free(buf);
	return 0;
}
long read_otd_browse_struct(struct otd_BROWSE *otd_browse,long record){
	unsigned char buf[128];
		if(read_otd_BROWSE_DATA(buf,&record)==FAIL)return -1;
		memcpy(&otd_browse->location,&buf[0],8);
		memcpy(&otd_browse->EGFA,&buf[8],8);
		memcpy(&otd_browse->EGFA_cal_rad,&buf[16],16);
		memcpy(&otd_browse->max_rate,&buf[32],4);
		memcpy(&otd_browse->single_event_areas,&buf[36],2);
		memcpy(&otd_browse->highres_locs,&buf[38],2);
		memcpy(&otd_browse->first_highres,&buf[40],4);
		memcpy(&otd_browse->bkgnds,&buf[44],4);
		memcpy(&otd_browse->iloper,&buf[48],2);
		memcpy(&otd_browse->bkgnd_pointer,&buf[50],4);
		memcpy(&otd_browse->duration,&buf[54],2);
		return 0;
}
long read_otd_threshold_struct(struct otd_THRESHOLD *otd_threshold,long record){
	unsigned char buf[128];
		if(read_otd_THRESHOLD(buf,&record)==FAIL)return -1;
		memcpy(&otd_threshold->TAI93,&buf[0],8);
		memcpy(&otd_threshold->values,&buf[8],16);
		return 0;
}
long read_otd_image_attribute_struct(struct otd_IMAGE_ATTRIBUTE *otd_image,long record){
	unsigned char buf[128];
		if(read_otd_IMAGE_ATTRIBUTE(buf,&record)==FAIL)return -1;
		memcpy(&otd_image->TAI93,&buf[0],8);
		memcpy(&otd_image->NW_location,&buf[8],8);
		memcpy(&otd_image->NE_location,&buf[16],8);
		memcpy(&otd_image->SW_location,&buf[24],8);
		memcpy(&otd_image->SE_location,&buf[32],8);
		memcpy(&otd_image->nadir_location,&buf[40],8);
		memcpy(&otd_image->image_pointer,&buf[48],4);
		memcpy(&otd_image->calibration_number,&buf[52],4);
		memcpy(&otd_image->spice_pointer,&buf[56],4);
		return 0;
}

long read_otd_25_grid_struct(struct otd_DENSITY *otd_grid,long record){
	unsigned char buf[128];
		if(read_otd_GRID_25(buf,&record)==FAIL)return -1;
		memcpy(&otd_grid->location,&buf[0],8);
		memcpy(&otd_grid->flash_count,&buf[8],4);
		return 0;
}
long read_otd_500_km_grid_struct(struct otd_DENSITY *otd_grid,long record){
	unsigned char buf[128];
		if(read_otd_GRID_500_KM(buf,&record)==FAIL)return -1;
		memcpy(&otd_grid->location,&buf[0],8);
		memcpy(&otd_grid->flash_count,&buf[8],4);
		return 0;
}
long read_otd_orbit_attributes_struct(struct otd_Orbit_attributes *otd_oa,long record){
	unsigned char buf[128];
		if(read_otd_ORBIT_DATA(buf,&record)==FAIL)return -1;
			memcpy(&otd_oa->ID,buf,4);
			memcpy(&otd_oa->TAI93_start,&buf[4],8);
			memcpy(&otd_oa->start_location,&buf[12],8);
			memcpy(&otd_oa->TAI93_end,&buf[20],8);
			memcpy(&otd_oa->end_location,&buf[28],8);
		return 0;
}
long read_otd_view_time_struct(struct otd_View_time *otd_vt,long record){
	unsigned char buf[128];
		if(read_otd_VIEW_TIME(buf,&record)==FAIL)return -1;
			memcpy(&otd_vt->grid_point[0],buf,8);
			memcpy(&otd_vt->TAI93_start,&buf[8],8);
			memcpy(&otd_vt->duration,&buf[16],2);
		return 0;
}

long READ_otd_viewtimes(struct otd_View_time *otd_viewtime_var,long record,long numrec) {
	unsigned char *buf;
	long status,i;
	
	buf = malloc((size_t)(18*sizeof(unsigned char)*numrec));
	status = READ_otd_VSGROUPS(buf,otd_view_vsref,&record,numrec);
	for (i=0; i<numrec; i++) {
		memcpy(&otd_viewtime_var[i].grid_point,&buf[i*18+0],8);
		memcpy(&otd_viewtime_var[i].TAI93_start,&buf[i*18+8],8);
		memcpy(&otd_viewtime_var[i].duration,&buf[i*18+16],2);
	}
	free(buf);
	return 0;
}

long read_otd_ephemeris_struct(struct otd_EPHEMERIS *otd_eph,long record){
	unsigned char buf[128];
		if(read_otd_EPHEMERIS(buf,&record)==FAIL)return -1;
			memcpy(&otd_eph->GPS,buf,8);
			memcpy(&otd_eph->kernel,&buf[8],120);
		return 0;
}

long READ_otd_ephemerises(struct otd_EPHEMERIS *otd_ephemeris_var, long record, long numrec) {
	unsigned char *buf;
	long status,i;
	
	buf = malloc((size_t)(128*sizeof(unsigned char)*numrec));
	status = READ_otd_VSGROUPS(buf,otd_ephemeris_vsref,&record,numrec);
	for (i=0; i<numrec; i++) {
		memcpy(&otd_ephemeris_var[i].GPS,&buf[i*128+0],8);
		memcpy(&otd_ephemeris_var[i].kernel,&buf[i*128+8],120);
	}
	free(buf);
	return 0;
}
/* aardvark */

long create_otd_VGROUP(long otd_hdf_id,char *name,char *class);
long write_otd_VSGROUP(void *databuffer,long vsref,long *vsrecord);
long write_otd_VSGROUP(void *databuffer,long vsref,long *vsrecord);
long write_otd_SDS(short int *databuffer,long sds_id);


long create_OTD_HDF_FILE(char *hdf_fname)
{
 
/********************** initailize internal data structures/pointers ********************/
	otd_image_srecord =0;

	otd_image_vrecord = 0;
	otd_vector_vrecord = 0;

	otd_area_vrecord = 0;
	otd_flash_vrecord = 0;
	otd_group_vrecord = 0;
	otd_event_vrecord = 0;
	otd_view_vrecord = 0;
	otd_orbit_event_rate_vrecord = 0;
	otd_grid_25_vrecord = 0;
	otd_grid_500_km_vrecord = 0;
	otd_orbit_flash_rate_vrecord = 0;
	otd_orbit_event_rate_vrecord = 0;
	otd_orbit_summary_vrecord = 0;
	otd_threshold_vrecord = 0;

/********************************* initialize H interface *******************************/

	otd_hdf_id = (Hopen(hdf_fname,DFACC_CREATE,256)); 
	if(otd_hdf_id==FAIL)return otd_hdf_id;
	Vstart(otd_hdf_id);

/****************************** create vgroups ******************************************/

	otd_area_vgroup_id = create_otd_VGROUP(otd_hdf_id,"Area","statistics");
	otd_flash_vgroup_id = create_otd_VGROUP(otd_hdf_id,"Flash","statistics");
	otd_group_vgroup_id = create_otd_VGROUP(otd_hdf_id,"Group","statistics");
	otd_event_vgroup_id = create_otd_VGROUP(otd_hdf_id,"Event","statistics");
	otd_browse_vgroup_id = create_otd_VGROUP(otd_hdf_id,"Browse","statistics");
	otd_flash_density_vgroup_id = create_otd_VGROUP(otd_hdf_id,"Flash Density","statistics");

	/* added 1/13/95 */
	otd_orbit_summary_vgroup_id = create_otd_VGROUP(otd_hdf_id,"Orbit summary","statistics");	
	/* end mod 1/13/95 */

/*************************** create scientific data sets (SDS) **************************/

	otd_sd_id = SDstart(hdf_fname,DFACC_RDWR);
	otd_dimsize[0]=0;
	otd_dimsize[1]=128;
	otd_dimsize[2]=128;
	otd_image_sds_id = SDcreate(otd_sd_id,"Background images",DFNT_INT16,3,otd_dimsize); 
 
	otd_dimsize[0]=128;
	otd_dimsize[1]=128;

/************************* orbit lone vdata decsription *********************************/

	if ((otd_orbit_vsref = VSattach(otd_hdf_id,-1,"w"))==FAIL){
#ifdef DEBUG
printf("orbit_vsref VSattach failed\n");
#endif
		return (FAIL);
	}

	VSsetname(otd_orbit_vsref,"orbit");
	VSsetclass(otd_orbit_vsref,"Attributes");	

	VSfdefine(otd_orbit_vsref,"orbit ID",DFNT_INT32,1);
	VSfdefine(otd_orbit_vsref,"TAI93 start",DFNT_FLOAT64,1);
	VSfdefine(otd_orbit_vsref,"start location",DFNT_FLOAT32,2);
	VSfdefine(otd_orbit_vsref,"TAI93 end",DFNT_FLOAT64,1);
	VSfdefine(otd_orbit_vsref,"end location",DFNT_FLOAT32,2);
	if(VSsetfields(otd_orbit_vsref,"orbit ID,TAI93 start,start location,TAI93 end,end location")==FAIL){
#ifdef DEBUG
printf("orbit_vsref VSetfields failed\n");
#endif
		return(FAIL);
	}

 /************************ metadata lone vdata description*******************************/

	if ((otd_metadata_vsref = VSattach(otd_hdf_id,-1,"w"))==FAIL){
#ifdef DEBUG
printf("metadata_vsref VSattach failed\n");
#endif
		return (FAIL);   
	}
		
	VSsetname(otd_metadata_vsref,"metadata");
	VSsetclass(otd_metadata_vsref,"description");  
		 
        VSfdefine(otd_metadata_vsref,"text",DFNT_CHAR8,80);

	if(VSsetfields(otd_metadata_vsref,"text")==FAIL){
#ifdef DEBUG
printf("metadata_vsref VSetfields failed\n");
#endif
		return (FAIL);
	}   

 /************************ Threhsold lone vdata description*******************************/

	if ((otd_threshold_vsref = VSattach(otd_hdf_id,-1,"w"))==FAIL){
#ifdef DEBUG
printf("threshold_vsref VSattach failed\n");
#endif
		return (FAIL);
	}   
			
	VSsetname(otd_threshold_vsref,"threshold");
	VSsetclass(otd_threshold_vsref,"values");  
		 
        VSfdefine(otd_threshold_vsref,"TAI93",DFNT_FLOAT64,1);
	VSfdefine(otd_threshold_vsref,"values",DFNT_UINT8,16);

	if(VSsetfields(otd_threshold_vsref,"TAI93,values")==FAIL){
#ifdef DEBUG
printf("threshold_vsref VSetfields failed\n");
#endif
		return (FAIL);  
	} 
 
/******************************** area **************************************************/

	if ((otd_area_vsref = VSattach(otd_hdf_id,-1,"w"))==FAIL){
#ifdef DEBUG
printf("area_vsref VSattach failed\n");
#endif
		return (FAIL);   
	}
			
	VSsetname(otd_area_vsref,"area");
	VSsetclass(otd_area_vsref,"statistics");  
		 
        VSfdefine(otd_area_vsref,"seq",DFNT_INT32,1);	
			
	VSfdefine(otd_area_vsref,"TAI93",DFNT_FLOAT64,1);
	VSfdefine(otd_area_vsref,"delta",DFNT_FLOAT32,1);
	VSfdefine(otd_area_vsref,"view",DFNT_FLOAT32,1);
	VSfdefine(otd_area_vsref,"s-z-a",DFNT_INT16,1);
	VSfdefine(otd_area_vsref,"d-n-t",DFNT_CHAR8,1);
	VSfdefine(otd_area_vsref,"end status",DFNT_CHAR8,1);
	VSfdefine(otd_area_vsref,"events",DFNT_INT16,1);
	VSfdefine(otd_area_vsref,"cent",DFNT_FLOAT32,2);
	VSfdefine(otd_area_vsref,"stdev",DFNT_FLOAT32,2);
	VSfdefine(otd_area_vsref,"loc count",DFNT_INT16,1);
	VSfdefine(otd_area_vsref,"rad",DFNT_FLOAT32,1);
	VSfdefine(otd_area_vsref,"orbit id",DFNT_INT32,1);
	VSfdefine(otd_area_vsref,"day",DFNT_INT32,1);
	VSfdefine(otd_area_vsref,"children",DFNT_INT32,1);
	VSfdefine(otd_area_vsref,"child seq",DFNT_INT32,1);
	VSfdefine(otd_area_vsref,"child rec",DFNT_INT32,1);
	VSfdefine(otd_area_vsref,"QA",DFNT_INT16,4);

	if(VSsetfields(otd_area_vsref,
		"seq,TAI93,delta,view,s-z-a,d-n-t,end status,events,cent,stdev,loc count,rad,orbit id,day,children,child seq,child rec,QA")==FAIL){
#ifdef DEBUG
printf("area_vsref VSetfields failed\n");
#endif
		return (FAIL);   
	}

	 if(Vinsert(otd_area_vgroup_id,otd_area_vsref)==FAIL){
#ifdef DEBUG
printf("otd_area_vgroup_id Vinsert failed\n");
#endif
		return (FAIL); 
	}  

/******************************** Flash *************************************************/
	
	if ((otd_flash_vsref = VSattach(otd_hdf_id,-1,"w"))==FAIL){
#ifdef DEBUG
printf("flash_vsref VSetfields failed\n");
#endif
		return (FAIL); 
	}  
			
	VSsetname(otd_flash_vsref,"flash");
	VSsetclass(otd_flash_vsref,"Statistics");  
		 
        VSfdefine(otd_flash_vsref,"seq",DFNT_INT32,1);	
			
	VSfdefine(otd_flash_vsref,"TAI93",DFNT_FLOAT64,1);
	VSfdefine(otd_flash_vsref,"delta",DFNT_FLOAT32,1);
	VSfdefine(otd_flash_vsref,"view",DFNT_FLOAT32,1);
	VSfdefine(otd_flash_vsref,"s-z-a",DFNT_INT16,1);
	VSfdefine(otd_flash_vsref,"d-n-t",DFNT_CHAR8,1);
	VSfdefine(otd_flash_vsref,"end status",DFNT_CHAR8,1);
	VSfdefine(otd_flash_vsref,"events",DFNT_INT16,1);
	VSfdefine(otd_flash_vsref,"cent",DFNT_FLOAT32,2);
	VSfdefine(otd_flash_vsref,"stdev",DFNT_FLOAT32,2);
	VSfdefine(otd_flash_vsref,"loc count",DFNT_INT16,1);
	VSfdefine(otd_flash_vsref,"rad",DFNT_FLOAT32,1);
	VSfdefine(otd_flash_vsref,"parent seq",DFNT_INT32,1);
	VSfdefine(otd_flash_vsref,"parent rec",DFNT_INT32,1);
	VSfdefine(otd_flash_vsref,"children",DFNT_INT32,1);
	VSfdefine(otd_flash_vsref,"child seq",DFNT_INT32,1);
	VSfdefine(otd_flash_vsref,"child rec",DFNT_INT32,1);
	VSfdefine(otd_flash_vsref,"QA",DFNT_INT16,4);

	if(VSsetfields(otd_flash_vsref,
		"seq,TAI93,delta,view,s-z-a,d-n-t,end status,events,cent,stdev,loc count,rad,parent seq,parent rec,children,child seq,child rec,QA")==FAIL){
#ifdef DEBUG
printf("flash_vsref VSetfields failed\n");
#endif
		return (FAIL); 
	}  

	if(Vinsert(otd_flash_vgroup_id,otd_flash_vsref)==FAIL){
#ifdef DEBUG
printf("otd_flash_vgroup_id Vinsert failed\n");
#endif
		return (FAIL);
	}   
 
/************************************ Group *********************************************/
	
	if ((otd_group_vsref = VSattach(otd_hdf_id,-1,"w"))==FAIL){
#ifdef DEBUG
printf("group_vsref VSattach failed\n");
#endif
		return (FAIL); 
	}  

	VSsetname(otd_group_vsref,"group");
	VSsetclass(otd_group_vsref,"Statistics");  
		 
        VSfdefine(otd_group_vsref,"seq",DFNT_INT32,1);	
			
	VSfdefine(otd_group_vsref,"TAI93",DFNT_FLOAT64,1);
	VSfdefine(otd_group_vsref,"delta",DFNT_FLOAT32,1);
	VSfdefine(otd_group_vsref,"view",DFNT_FLOAT32,1);
	VSfdefine(otd_group_vsref,"s-z-a",DFNT_INT16,1);
	VSfdefine(otd_group_vsref,"d-n-t",DFNT_CHAR8,1);
	VSfdefine(otd_group_vsref,"end status",DFNT_CHAR8,1);
	VSfdefine(otd_group_vsref,"events",DFNT_INT16,1);
	VSfdefine(otd_group_vsref,"cent",DFNT_FLOAT32,2);
	VSfdefine(otd_group_vsref,"stdev",DFNT_FLOAT32,2);
	VSfdefine(otd_group_vsref,"loc count",DFNT_INT16,1);
	VSfdefine(otd_group_vsref,"rad",DFNT_FLOAT32,1);
	VSfdefine(otd_group_vsref,"parent seq",DFNT_INT32,1);
	VSfdefine(otd_group_vsref,"parent rec",DFNT_INT32,1);
	VSfdefine(otd_group_vsref,"children",DFNT_INT32,1);
	VSfdefine(otd_group_vsref,"child seq",DFNT_INT32,1);
	VSfdefine(otd_group_vsref,"child rec",DFNT_INT32,1);
	VSfdefine(otd_group_vsref,"QA",DFNT_INT16,4);

	if(VSsetfields(otd_group_vsref,
		"seq,TAI93,delta,view,s-z-a,d-n-t,end status,events,cent,stdev,loc count,rad,parent seq,parent rec,children,child seq,child rec,QA")==FAIL){
#ifdef DEBUG
printf("group_vsref VSetfields failed\n");
#endif
		return (FAIL);
	}   
	if(Vinsert(otd_group_vgroup_id,otd_group_vsref)==FAIL){
#ifdef DEBUG
printf("group_vgroup_id Vinsert failed\n");
#endif
		return (FAIL); 
	}  

/********************************* Events *********************************************/

	if ((otd_event_vsref = VSattach(otd_hdf_id,-1,"w"))==FAIL){
#ifdef DEBUG
printf("event_vsref VSattach failed\n");
#endif
		return (FAIL); 
	}  
			
	VSsetname(otd_event_vsref,"event");
	VSsetclass(otd_event_vsref,"Statistics");  
		 
	VSfdefine(otd_event_vsref,"event #",DFNT_INT32,1);
	VSfdefine(otd_event_vsref,"seq. #",DFNT_INT32,1);
	VSfdefine(otd_event_vsref,"TAI93",DFNT_FLOAT64,1);
	VSfdefine(otd_event_vsref,"s-z-a",DFNT_INT16,1);
	VSfdefine(otd_event_vsref,"d-n-t",DFNT_CHAR8,1);
	VSfdefine(otd_event_vsref,"x pixel",DFNT_INT16,1);
	VSfdefine(otd_event_vsref,"y pixel",DFNT_INT16,1);
	VSfdefine(otd_event_vsref,"raw radiance",DFNT_INT16,1);
	VSfdefine(otd_event_vsref,"cal radiance",DFNT_FLOAT32,1);

        VSfdefine(otd_event_vsref,"parent seq",DFNT_INT32,1);	/* parent */
        VSfdefine(otd_event_vsref,"parent rec",DFNT_INT32,1);	/* parent */
        
	VSfdefine(otd_event_vsref,"QA",DFNT_INT16,4);
	VSfdefine(otd_event_vsref,"location",DFNT_FLOAT32,2);

	if(VSsetfields(otd_event_vsref,
		"event #,seq. #,TAI93,s-z-a,d-n-t,x pixel,y pixel,raw radiance,cal radiance,parent seq,parent rec,QA,location")==FAIL){
#ifdef DEBUG
printf("event_vsref VSetfields failed\n");
#endif
		return (FAIL);   
	}

	if(Vinsert(otd_event_vgroup_id,otd_event_vsref)==FAIL){
#ifdef DEBUG
printf("event_vgroup_id Vinsert failed\n");
#endif
		return (FAIL);  
	}

/*********************************** browse *********************************************/
	
	if ((otd_browse_vsref = VSattach(otd_hdf_id,-1,"w"))==FAIL){
#ifdef DEBUG
printf("browse_vsref VSattach failed\n");
#endif
		return (FAIL);  
	} 
			
	VSsetname(otd_browse_vsref,"browse");
	VSsetclass(otd_browse_vsref,"area");  
		 
	VSfdefine(otd_browse_vsref,"location",DFNT_FLOAT32,2);
	VSfdefine(otd_browse_vsref,"EGFAs",DFNT_INT16,4);
	VSfdefine(otd_browse_vsref,"EGFA cal rad",DFNT_FLOAT32,4);
	VSfdefine(otd_browse_vsref,"max rate",DFNT_FLOAT32,1);
	VSfdefine(otd_browse_vsref,"1 event areas",DFNT_INT16,1);

	VSfdefine(otd_browse_vsref,"highres locs",DFNT_INT16,1);
	VSfdefine(otd_browse_vsref,"1st highres",DFNT_INT32,1);
	VSfdefine(otd_browse_vsref,"bkgnds",DFNT_INT32,1);
	VSfdefine(otd_browse_vsref,"iloper",DFNT_INT16,1);
	VSfdefine(otd_browse_vsref,"bkgnd pointer",DFNT_INT32,1);
	VSfdefine(otd_browse_vsref,"duration",DFNT_INT16,1);

	if(VSsetfields(otd_browse_vsref,
		"location,EGFAs,EGFA cal rad,max rate,1 event areas,highres locs,1st highres,bkgnds,iloper,bkgnd pointer,duration")==FAIL){
#ifdef DEBUG
printf("browse_vsref VSetfields failed\n");
#endif
		return (FAIL); 
	}

	if(Vinsert(otd_browse_vgroup_id,otd_browse_vsref)==FAIL){
#ifdef DEBUG
printf("browse_vsref Vinsert failed\n");
#endif
		return (FAIL); 
	}  
 
/********************************** vector products *************************************/

	if ((otd_vector_vsref = VSattach(otd_hdf_id,-1,"w"))==FAIL){
#ifdef DEBUG
printf("otd_vector_vsref VSattach failed\n");
#endif
		return (FAIL);
	}   
			
	VSsetname(otd_vector_vsref,"vector");
	VSsetclass(otd_vector_vsref,"statistics");  
		 
			
	VSfdefine(otd_vector_vsref,"location",DFNT_FLOAT32,2);
	VSfdefine(otd_vector_vsref,"EGFA counts",DFNT_INT16,4);
	VSfdefine(otd_vector_vsref,"EGFA cal radiances",DFNT_FLOAT32,4);
	VSfdefine(otd_vector_vsref,"max flash rate",DFNT_FLOAT32,1);
	VSfdefine(otd_vector_vsref,"single event areas",DFNT_INT16,1);

	if(VSsetfields(otd_vector_vsref,
		"location,EGFA counts,EGFA cal radiances,max flash rate,single event areas")==FAIL){
#ifdef DEBUG
printf("otd_vector_vsref VSetfields failed\n");
#endif
		return (FAIL); 
	}  
	if(Vinsert(otd_browse_vgroup_id,otd_vector_vsref)==FAIL){
#ifdef DEBUG
printf("otd_vector_vsref Vinsert failed\n");
#endif
		return (FAIL); 
	}  
 
/********************************* image attribute ***************************************/

	if ((otd_image_vsref = VSattach(otd_hdf_id,-1,"w"))==FAIL){
#ifdef DEBUG
printf("otd_image_vsref VSattach failed\n");
#endif
		return (FAIL); 
	} 
			
	VSsetname(otd_image_vsref,"image");
	VSsetclass(otd_image_vsref,"attributes");  
		 	
	VSfdefine(otd_image_vsref,"TAI93",DFNT_FLOAT64,1);
	VSfdefine(otd_image_vsref,"NW location",DFNT_FLOAT32,2);
	VSfdefine(otd_image_vsref,"NE location",DFNT_FLOAT32,2);
	VSfdefine(otd_image_vsref,"SW location",DFNT_FLOAT32,2);
	VSfdefine(otd_image_vsref,"SE location",DFNT_FLOAT32,2);
	VSfdefine(otd_image_vsref,"nadir location",DFNT_FLOAT32,2);
	VSfdefine(otd_image_vsref,"image pointer",DFNT_INT32,1);
	VSfdefine(otd_image_vsref,"calibration number",DFNT_INT32,1);
	VSfdefine(otd_image_vsref,"spice pointer",DFNT_INT32,1);

	if(VSsetfields(otd_image_vsref,
		"TAI93,NW location,NE location, SW location,SE location,nadir location,image pointer,calibration number,spice pointer")==FAIL){
#ifdef DEBUG
printf("image_vsref VSetfields failed\n");
#endif
		return (FAIL);
	}   

	if(Vinsert(otd_browse_vgroup_id,otd_image_vsref)==FAIL){
#ifdef DEBUG
printf("image_vsref Vinsert failed\n");
#endif

		return (FAIL);
	}   

/************************* orbit Summary 1/13/95 *********************************/

	otd_orbit_summary_vsref=VSattach(otd_hdf_id,-1,"w");
	if(otd_orbit_summary_vsref==FAIL){
#ifdef DEBUG
printf("orbit_summary_vsref VSattach failed\n");
#endif
		 return(FAIL);
	}

	VSsetname(otd_orbit_summary_vsref,"summary");
	VSsetclass(otd_orbit_summary_vsref,"data");

	VSfdefine(otd_orbit_summary_vsref,"areas",DFNT_INT32,1);
	VSfdefine(otd_orbit_summary_vsref,"flashes",DFNT_INT32,1);
	VSfdefine(otd_orbit_summary_vsref,"groups",DFNT_INT32,1);
	VSfdefine(otd_orbit_summary_vsref,"events",DFNT_INT32,1);
	VSfdefine(otd_orbit_summary_vsref,"backgrounds",DFNT_INT32,1);
	VSfdefine(otd_orbit_summary_vsref,"total cal radiance",DFNT_FLOAT32,1);
  
	if(VSsetfields(otd_orbit_summary_vsref,"areas,flashes,groups,events,backgrounds,total cal radiance")==FAIL){
#ifdef DEBUG
printf("otd_orbit_summary_vsref VSetfields failed\n");
#endif
		return (FAIL);
	}   

     if(Vinsert(otd_orbit_summary_vgroup_id,otd_orbit_summary_vsref)==FAIL){
#ifdef DEBUG
printf("otd_orbit_summary_vsref Vinsert failed\n");
#endif
		return (FAIL);
	}   

/************************* orbit area flash rate  1/13/95 *********************************/

	otd_orbit_flash_rate_vsref=VSattach(otd_hdf_id,-1,"w");
	if(otd_orbit_flash_rate_vsref==FAIL){
#ifdef DEBUG
printf("orbit_flash_rate_vsref VSattach failed\n");
#endif
		 return(FAIL);
	}

	VSsetname(otd_orbit_flash_rate_vsref,"rate_f");
	VSsetclass(otd_orbit_flash_rate_vsref,"data");

	VSfdefine(otd_orbit_flash_rate_vsref,"flash rate",DFNT_FLOAT32,1);
	
	if(VSsetfields(otd_orbit_flash_rate_vsref,"flash rate")==FAIL){
#ifdef DEBUG
printf("orbit_flash_rate_vsref VSetfields failed\n");
#endif
		return (FAIL);
	}   

	if(Vinsert(otd_orbit_summary_vgroup_id,otd_orbit_flash_rate_vsref)==FAIL){
#ifdef DEBUG
printf("orbit_flash_rate_vsref Vinsert failed\n");
#endif
		return (FAIL); 
	}  

/************************* orbit event rate  05/20/96 *********************************/

	otd_orbit_event_rate_vsref=VSattach(otd_hdf_id,-1,"w");
	if(otd_orbit_event_rate_vsref==FAIL){
#ifdef DEBUG
printf("orbit_event_rate_vsref VSattach failed\n");
#endif
		 return(FAIL);
	}

	VSsetname(otd_orbit_event_rate_vsref,"rate_e");
	VSsetclass(otd_orbit_event_rate_vsref,"data");

	VSfdefine(otd_orbit_event_rate_vsref,"count",DFNT_INT16,6100);
	
	if(VSsetfields(otd_orbit_event_rate_vsref,"count")==FAIL){
#ifdef DEBUG
printf("orbit_event_rate_vsref VSetfields failed\n");
#endif
		return (FAIL);   
	}

	if(Vinsert(otd_orbit_summary_vgroup_id,otd_orbit_event_rate_vsref)==FAIL){
#ifdef DEBUG
printf("orbit_event_rate_vsref Vinsert failed\n");
#endif
		return (FAIL); 
	}  

/***************************** Ephemeris data 6/8/95 ***********************************/

	otd_ephemeris_vsref = VSattach(otd_hdf_id,-1,"w");
	if(otd_ephemeris_vsref==FAIL){
#ifdef DEBUG
printf("epmemeris_vsref VSattach failed\n");
#endif
		return (FAIL);
	}
	
	VSsetname(otd_ephemeris_vsref,"ephemeris");
	VSsetclass(otd_ephemeris_vsref,"data");

	VSfdefine(otd_ephemeris_vsref,"GPS time",DFNT_FLOAT64,1);
	VSfdefine(otd_ephemeris_vsref,"spice kernel",DFNT_FLOAT64,15);

	if(VSsetfields(otd_ephemeris_vsref,"GPS time,spice kernel")==FAIL){
#ifdef DEBUG
printf("ephemeris_vsref VSetfields failed\n");
#endif
		return (FAIL); 
	}  

	if(Vinsert(otd_orbit_summary_vgroup_id,otd_ephemeris_vsref)==FAIL){
#ifdef DEBUG
printf("ephemeris_vsref Vinsert failed\n");
#endif
		return (FAIL); 
	}  

/************************* 2.5 deg x 2.5 deg grid 2/13/95 *********************************/

	otd_grid_25_vsref=VSattach(otd_hdf_id,-1,"w");
	if(otd_grid_25_vsref==FAIL){
#ifdef DEBUG
printf("grid_25_vsref VSattach failed\n");
#endif
 	return(FAIL);
	}

	VSsetname(otd_grid_25_vsref,"2.5 degree");
	VSsetclass(otd_grid_25_vsref,"grid");

	VSfdefine(otd_grid_25_vsref,"location",DFNT_FLOAT32,2);
	VSfdefine(otd_grid_25_vsref,"flash count",DFNT_INT32,1);
	
	if(VSsetfields(otd_grid_25_vsref,"location,flash count")==FAIL){
#ifdef DEBUG
printf("grid_25_vsref VSetfields failed\n");
#endif
		return (FAIL);
	}   

	if(Vinsert(otd_flash_density_vgroup_id,otd_grid_25_vsref)==FAIL){
#ifdef DEBUG
printf("grid_25_vsref Vinsert failed\n");
#endif
		return (FAIL);
	}   

/************************* view time .5 deg grid 10/4/95 *********************************/

	otd_view_vsref=VSattach(otd_hdf_id,-1,"w");
	if(otd_view_vsref==FAIL){
#ifdef DEBUG
printf("view_vsref VSattach failed\n");
#endif
		 return(FAIL);
	}

	VSsetname(otd_view_vsref,"view time");
	VSsetclass(otd_view_vsref,"grid");

	VSfdefine(otd_view_vsref,"grid point",DFNT_FLOAT32,2);
	VSfdefine(otd_view_vsref,"TAI93 start",DFNT_FLOAT64,1);
	VSfdefine(otd_view_vsref,"duration",DFNT_INT16,1);
	
	if(VSsetfields(otd_view_vsref,"grid point,TAI93 start,duration")==FAIL){
#ifdef DEBUG
printf("view_vsref VSetfields failed\n");
#endif
		return (FAIL);
	}   

	if(Vinsert(otd_flash_density_vgroup_id,otd_view_vsref)==FAIL){
#ifdef DEBUG
printf("view_vsref Vinsert failed\n");
#endif
		return (FAIL); 
	}  

/************************* 50 deg x 50 deg grid 2/13/95 *********************************/

	otd_grid_500_km_vsref=VSattach(otd_hdf_id,-1,"w");
	if(otd_grid_500_km_vsref==FAIL){
#ifdef DEBUG
printf("grid_500_km_vsref VSattach failed\n");
#endif
 		return(FAIL);
	}

	VSsetname(otd_grid_500_km_vsref,"500km");
	VSsetclass(otd_grid_500_km_vsref,"grid");

	VSfdefine(otd_grid_500_km_vsref,"location",DFNT_FLOAT32,2);
	VSfdefine(otd_grid_500_km_vsref,"flash count",DFNT_INT32,1);
	
	if(VSsetfields(otd_grid_500_km_vsref,"location,flash count")==FAIL){
#ifdef DEBUG
printf("grid_500_km_vsref VSetfields failed\n");
#endif
		return (FAIL);
	}   

	if(Vinsert(otd_flash_density_vgroup_id,otd_grid_500_km_vsref)==FAIL){
#ifdef DEBUG
printf("grid_500_km_vsref Vinsert failed\n");
#endif
		return (FAIL); 
	}  

	return (otd_hdf_id);

}


long create_otd_VGROUP(long otd_hdf_id,char *name,char *class)

/****************************************************************************************/
/*											*/
/*	create a vgroup in the HDF file							*/
/*											*/
/*	returns the reference number of the Vgroup if successful			*/
/*	returns FAIL (-1) on error							*/
/*											*/
/*	Date last modified	8/15/94							*/
/****************************************************************************************/

{
	long vgroup;
	vgroup= (long)Vattach(otd_hdf_id,-1,"w");
	if(vgroup == FAIL)return FAIL;	
	Vsetname(vgroup,name);
	Vsetclass(vgroup,class);
	return (vgroup);

}


 
long write_otd_IMAGE(short int *databuffer)

/****************************************************************************************/
/*											*/
/*	Write sds image data to HDF file						*/
/*											*/
/*	returns sds record # if successful						*/
/*	returns FAIL (-1) on error							*/
/*											*/
/*	Date last modified	8/15/94							*/
/****************************************************************************************/


{
	start[0]=otd_image_srecord++;
	start[1]=start[2]=0;

	endr[0]=1;
	endr[1]=endr[2]=128;

	if(SDwritedata(otd_image_sds_id,start,NULL,endr,databuffer)==FAIL){
		otd_image_srecord--;
		return (FAIL);}
	return (otd_image_srecord-1);
}




long write_otd_EVENT(void *databuffer)		{return (write_otd_VSGROUP(databuffer,otd_event_vsref,&otd_event_vrecord));}
long write_otd_RATE(void *databuffer)		{return (write_otd_VSGROUP(databuffer,otd_orbit_event_rate_vsref,&otd_orbit_event_rate_vrecord));}
long write_otd_GROUP(void *databuffer) 		{return (write_otd_VSGROUP(databuffer,otd_group_vsref,&otd_group_vrecord));}
long write_otd_FLASH(void *databuffer) 		{return (write_otd_VSGROUP(databuffer,otd_flash_vsref,&otd_flash_vrecord));}
long write_otd_AREA(void *databuffer)	 	{return (write_otd_VSGROUP(databuffer,otd_area_vsref,&otd_area_vrecord));}
long write_otd_VECTOR_PRODUCTS(void *databuffer)	{return (write_otd_VSGROUP(databuffer,otd_vector_vsref,&otd_vector_vrecord));}
long write_otd_IMAGE_ATTRIBUTE(void *databuffer)	{return (write_otd_VSGROUP(databuffer,otd_image_vsref,&otd_image_vrecord));}
long write_otd_ORBIT_DATA(void *databuffer)	{return (write_otd_VSGROUP(databuffer,otd_orbit_vsref,&otd_orbit_vrecord));}
long write_otd_METADATA(void *databuffer)		{return (write_otd_VSGROUP(databuffer,otd_metadata_vsref,&otd_metadata_vrecord));}
long write_otd_BROWSE_DATA(void *databuffer)	{return (write_otd_VSGROUP(databuffer,otd_browse_vsref,&otd_browse_data_vrecord));}
long write_otd_VIEW(void *databuffer)		{return (write_otd_VSGROUP(databuffer,otd_view_vsref,&otd_view_vrecord));}
long write_otd_GRID_25(void *databuffer)		{return (write_otd_VSGROUP(databuffer,otd_grid_25_vsref,&otd_grid_25_vrecord));}
long write_otd_GRID_500_KM(void *databuffer)	{return (write_otd_VSGROUP(databuffer,otd_grid_500_km_vsref,&otd_grid_500_km_vrecord));}

long write_otd_ORBIT_SUMMARY(void *databuffer)	{return (write_otd_VSGROUP(databuffer,otd_orbit_summary_vsref,&otd_orbit_summary_vrecord));}
long write_otd_AREA_FLASH_RATE(void *databuffer)	{return (write_otd_VSGROUP(databuffer,otd_orbit_flash_rate_vsref,&otd_orbit_flash_rate_vrecord));}
long write_otd_EPHEMERIS(void *databuffer)		{return (write_otd_VSGROUP(databuffer,otd_ephemeris_vsref,&otd_ephemeris_vrecord));}
long write_otd_THRESHOLD(void *databuffer)		{return (write_otd_VSGROUP(databuffer,otd_threshold_vsref,&otd_threshold_vrecord));}


long write_otd_VSGROUP(void *databuffer,long vsref,long *vsrecord)

/****************************************************************************************/
/*											*/
/*	Write data to HDF file								*/
/*											*/
/*	returns Vdata record # if successful						*/
/*	returns FAIL (-1) on error							*/
/*											*/
/*	Date last modified	8/15/94							*/
/****************************************************************************************/


{
	(*vsrecord)++;
 	if(VSwrite(vsref,databuffer,1,FULL_INTERLACE)==FAIL){
		(*vsrecord)--;
		return (FAIL);
	}
	return (*vsrecord-1);
}


/********************************************************************************************************/
/*													*/
/*	Compatible with HDF version 4.1r1									*/
/*													*/
/*													*/
/*													*/
/*	LIS/OTD HDF database access library								*/
/*													*/
/*	FUNCTION NAME			TYPE	DESCRIPTION						*/
/*	------------------------	----	------------------------------------------------------- */
/*	create_LIS_ORBIT_HDF		long	Create and initialize LIS/OTD orbit archive file	*/
/*	create_LIS_BACKGROUND_HDF	long	Create and initialize LIS/OTD background archive file	*/
/*	close_LIS_ORBIT_HDF		long	Gracefully detach and close orbit archive file		*/
/*	close_LIS_BACKGROUND_HDF	long	Gracefully detach and close background archive file	*/
/*													*/
/*	write_lis_AREA			long	Write areadata to archive file				*/
/*	write_lis_FLASH			long	Write flash data to archive file			*/
/*	write_lis_GROUP			long	Write group data to archive file			*/
/*	write_lis_EVENT			long	Write the event data to archive file			*/
/*	write_lis_POINT_SUMMARY		long	Write point_summary data to archive file		*/
/*	write_lis_VIEWTIME			long	Write the viewtime data to archive file			*/
/*	write_lis_BG_SUMMARY		long	Write the bg_summary data to archive file		*/
/*	write_lis_ONE_SECOND		long	Write the one_second data to archive file		*/
/*	write_lis_ORBIT_SUMMARY		long	Write the orbit_summary data to archive file		*/
/*													*/
/*													*/
/*	write_lis_BG_DATA_SUMMARY		long	Write the bg_data_summary data to archive file		*/
/*	write_lis_BG_INFO			long	Write the bg_info data to archive file			*/
/*	write_lis_BG_ORBIT_SUMMARY		long	Write the bg_orbit_summary data to archive file		*/
/*	write_lis_BACKGROUND_DATA		long	Write background image (SDS) to archive file		*/
/*													*/
/*													*/
/*	write_lis_VSGROUP			long	generic VSwrite routine	(hiddden from user)		*/
/*	write_lis_SDS			long    generic SDwritedata routine (hidden from user)		*/
/*													*/
/*													*/
/*	create_lis_VGROUP			long	create a vgroup in the HDF file	(hiddden from user)	*/
/*													*/
/*													*/
/*	*********************************************************					*/
/*	* NOTE!!					      	*					*/
/*	*	All functions return a negative number on error	*					*/
/*	*	see LIBLIS_err_defs.h for details		*					*/
/*	*********************************************************					*/
/*													*/
/********************************************************************************************************/


/****************************************************************************************/
/**************************************		*****************************************/
/************************************** HISTORY	*****************************************/
/**************************************		*****************************************/
/****************************************************************************************/
/*											*/
/*	Initial library development 9/29/1997	John Hall/CSC (jmh)			*/
/*											*/
/*											*/
/* Change History:									*/
/*											*/
/*											*/
/* Date		Module			Change description			initail */
/* --------	-------------------	-----------------------------------	-------	*/
/*											*/
/*											*/
/****************************************************************************************/

long create_lis_VGROUP(long lis_hdf_id,char *name,char *class);
long write_lis_VSGROUP(long file_number,void *databuffer,long vsref[],long vsrecord[]);
long write_lis_SDS(short int *databuffer,long sds_id);


long create_LIS_ORBIT_HDF(char *hdf_fname)
{
	long hid_tmp,i,file_number;
 

/********************************* initialize H interface *******************************/
	
	hid_tmp = (Hopen(hdf_fname,DFACC_CREATE,256)); 
	if(hid_tmp==FAIL)return hid_tmp;
	file_number = -1;
	for(i=0;i<MAX_FILE;i++){
		if(lis_hdf_id[i]==-1){
			file_number = i;
			break;
		}
	}
	if(file_number==-1){
		return(lis_operation_FAIL|lis_MAX_FILE_FAIL);
	}
	if(Vstart(hid_tmp)==FAIL) return lis_operation_FAIL|lis_Vstart_FAIL;
	lis_hdf_id[file_number] = hid_tmp;


/********************** initailize internal data structures/pointers ********************/

	lis_area_vrecord[file_number] = 0;
	lis_flash_vrecord[file_number] = 0;
	lis_group_vrecord[file_number] = 0;
	lis_event_vrecord[file_number] = 0;
	lis_viewtime_vrecord[file_number] = 0;
	lis_bg_summary_vrecord[file_number] = 0;
	lis_point_summary_vrecord[file_number] = 0;
	lis_orbit_summary_vrecord[file_number] = 0;
	lis_one_second_vrecord[file_number] = 0;

/****************************** create vgroups ******************************************/
	lis_orbit_data_vgroup_id[file_number] = create_lis_VGROUP(lis_hdf_id[file_number],"orbit","data");
	if(lis_orbit_data_vgroup_id[file_number]==FAIL){
		printf("create_lis_VGROUP orbit_data_vgroup_id failed\n");
		return lis_operation_FAIL|lis_Vgroup_Create_FAIL;
	}
	lis_point_vgroup_id[file_number] = create_lis_VGROUP(lis_hdf_id[file_number],"point","data");
	if(lis_point_vgroup_id[file_number]==FAIL){
		printf("create_lis_VGROUP point_vgroup_id failed\n");
		return lis_operation_FAIL|lis_Vgroup_Create_FAIL;
	}
	lis_lightning_vgroup_id[file_number] = create_lis_VGROUP(lis_hdf_id[file_number],"lightning","data");
	if(lis_lightning_vgroup_id[file_number]==FAIL){
		printf("create_lis_VGROUP point_vgroup_id failed\n");
		return lis_operation_FAIL|lis_Vgroup_Create_FAIL;
	}

	if(Vinsert(lis_orbit_data_vgroup_id[file_number],lis_point_vgroup_id[file_number])==FAIL){
		printf("point_vgroup_id Vinsert failed\n");

		return (lis_operation_FAIL|lis_Vinsert_FAIL); 
	}  
	if(Vinsert(lis_point_vgroup_id[file_number],lis_lightning_vgroup_id[file_number])==FAIL){
		printf("lightning_vgroup_id Vinsert failed\n");

		return (lis_operation_FAIL|lis_Vinsert_FAIL); 
	}  


/*************************** create scientific data sets (SDS) **************************/
/*
	sd_id = SDstart(hdf_fname,DFACC_RDWR);
	dimsize[0]=0;
	dimsize[1]=128;
	dimsize[2]=128;
	image_sds_id = SDcreate(sd_id,"bg_data",DFNT_INT16,3,dimsize); 
 
	dimsize[0]=128;
	dimsize[1]=128;
*/

/******************************** orbit summary **************************************************/

	if ((lis_orbit_summary_vsref[file_number] = VSattach(lis_hdf_id[file_number],-1,"w"))==FAIL){
		printf("orbit_summary_vsref VSattach failed\n");
		return (lis_operation_FAIL|lis_orbit_summary_VSattach_FAIL);   
	}
			
	VSsetname(lis_orbit_summary_vsref[file_number],"orbit_summary");
	VSsetclass(lis_orbit_summary_vsref[file_number],"data");  
		 
	VSfdefine(lis_orbit_summary_vsref[file_number],"id_number",DFNT_INT32,1);
	VSfdefine(lis_orbit_summary_vsref[file_number],"TAI93_start",DFNT_FLOAT64,1);
	VSfdefine(lis_orbit_summary_vsref[file_number],"UTC_start",DFNT_CHAR,28);
	VSfdefine(lis_orbit_summary_vsref[file_number],"GPS_start",DFNT_FLOAT64,1);
	VSfdefine(lis_orbit_summary_vsref[file_number],"TAI93_end",DFNT_FLOAT64,1);
	VSfdefine(lis_orbit_summary_vsref[file_number],"start_longitude",DFNT_FLOAT32,1);
	VSfdefine(lis_orbit_summary_vsref[file_number],"end_longitude",DFNT_FLOAT32,1);
	VSfdefine(lis_orbit_summary_vsref[file_number],"point_data_count",DFNT_INT16,1);
	VSfdefine(lis_orbit_summary_vsref[file_number],"point_data_address",DFNT_INT16,1);
	VSfdefine(lis_orbit_summary_vsref[file_number],"one_second_count",DFNT_INT32,1);
	VSfdefine(lis_orbit_summary_vsref[file_number],"one_second_address",DFNT_INT32,1);
	VSfdefine(lis_orbit_summary_vsref[file_number],"summary_image_count",DFNT_INT16,1);
	VSfdefine(lis_orbit_summary_vsref[file_number],"summary_image_address",DFNT_INT16,1);
	VSfdefine(lis_orbit_summary_vsref[file_number],"inspection_code",DFNT_UINT16,1);
	VSfdefine(lis_orbit_summary_vsref[file_number],"configuration_code",DFNT_UINT16,1);


	if(VSsetfields(lis_orbit_summary_vsref[file_number],
		"id_number,TAI93_start,UTC_start,GPS_start,TAI93_end,start_longitude,end_longitude,point_data_count,point_data_address,one_second_count,one_second_address,summary_image_count,summary_image_address,inspection_code,configuration_code")==FAIL){
		printf("orbit_summary_vsref VSetfields failed\n");
		return (lis_operation_FAIL|lis_VSsetfields_FAIL);   
	}


	if(Vinsert(lis_orbit_data_vgroup_id[file_number],lis_orbit_summary_vsref[file_number])==FAIL){
		printf("orbit_summary_vgroup_id Vinsert failed\n");
		return (lis_operation_FAIL| lis_Vinsert_FAIL); 
	}  

/******************************** one_second **************************************************/
	if ((lis_one_second_vsref[file_number] = VSattach(lis_hdf_id[file_number],-1,"w"))==FAIL){
		printf("one_second_vsref VSattach failed\n");
		return (lis_operation_FAIL|lis_one_second_VSattach_FAIL);   
	}
			
	VSsetname(lis_one_second_vsref[file_number],"one_second");
	VSsetclass(lis_one_second_vsref[file_number],"data");  
		 
	VSfdefine(lis_one_second_vsref[file_number],"TAI93_time",DFNT_FLOAT64,1);
	VSfdefine(lis_one_second_vsref[file_number],"alert_summary",DFNT_UINT8,1);
	VSfdefine(lis_one_second_vsref[file_number],"instrument_alert",DFNT_UINT8,1);
	VSfdefine(lis_one_second_vsref[file_number],"platform_alert",DFNT_UINT8,1);
	VSfdefine(lis_one_second_vsref[file_number],"external_alert",DFNT_UINT8,1);
	VSfdefine(lis_one_second_vsref[file_number],"processing_alert",DFNT_UINT8,1);
	VSfdefine(lis_one_second_vsref[file_number],"position_vector",DFNT_FLOAT32,3);
	VSfdefine(lis_one_second_vsref[file_number],"velocity_vector",DFNT_FLOAT32,3);
	VSfdefine(lis_one_second_vsref[file_number],"transform_matrix",DFNT_FLOAT32,9);
	VSfdefine(lis_one_second_vsref[file_number],"solar_vector",DFNT_FLOAT32,3);
	VSfdefine(lis_one_second_vsref[file_number],"ephemeris_quality_flag",DFNT_INT32,1);
	VSfdefine(lis_one_second_vsref[file_number],"attitude_quality_flag",DFNT_INT32,1);
	VSfdefine(lis_one_second_vsref[file_number],"boresight_threshold",DFNT_INT8,1);
	VSfdefine(lis_one_second_vsref[file_number],"thresholds",DFNT_INT8,16);
	VSfdefine(lis_one_second_vsref[file_number],"noise_index",DFNT_INT8,1);
	VSfdefine(lis_one_second_vsref[file_number],"event_count",DFNT_INT16,6);


	if(VSsetfields(lis_one_second_vsref[file_number],
		"TAI93_time,alert_summary,instrument_alert,platform_alert,external_alert,processing_alert,position_vector,velocity_vector,transform_matrix,solar_vector,ephemeris_quality_flag,attitude_quality_flag,boresight_threshold,thresholds,noise_index,event_count")==FAIL){
		printf("lis_one_second_vsref VSetfields failed\n");
		return (lis_operation_FAIL|lis_VSsetfields_FAIL);   
	}


	if(Vinsert(lis_orbit_data_vgroup_id[file_number],lis_one_second_vsref[file_number])==FAIL){
		printf("one_second_vgroup_id Vinsert failed\n");
		return (lis_operation_FAIL| lis_Vinsert_FAIL); 
	}  


/******************************** point summary **************************************************/

	if ((lis_point_summary_vsref[file_number] = VSattach(lis_hdf_id[file_number],-1,"w"))==FAIL){
		printf("lis_point_summary_vsref VSattach failed\n");
		return (lis_operation_FAIL|lis_operation_FAIL| lis_Vinsert_FAIL);   
	}
			
	VSsetname(lis_point_summary_vsref[file_number],"point_summary");
	VSsetclass(lis_point_summary_vsref[file_number],"data");  
		 
	VSfdefine(lis_point_summary_vsref[file_number],"parent_address",DFNT_INT32,1);
	VSfdefine(lis_point_summary_vsref[file_number],"event_count",DFNT_INT32,1);
	VSfdefine(lis_point_summary_vsref[file_number],"event_address",DFNT_INT32,1);
	VSfdefine(lis_point_summary_vsref[file_number],"group_count",DFNT_INT32,1);
	VSfdefine(lis_point_summary_vsref[file_number],"group_address",DFNT_INT32,1);
	VSfdefine(lis_point_summary_vsref[file_number],"flash_count",DFNT_INT32,1);
	VSfdefine(lis_point_summary_vsref[file_number],"flash_address",DFNT_INT32,1);
	VSfdefine(lis_point_summary_vsref[file_number],"area_count",DFNT_INT32,1);
	VSfdefine(lis_point_summary_vsref[file_number],"area_address",DFNT_INT32,1);
	VSfdefine(lis_point_summary_vsref[file_number],"bg_count",DFNT_INT32,1);
	VSfdefine(lis_point_summary_vsref[file_number],"bg_address",DFNT_INT32,1);
	VSfdefine(lis_point_summary_vsref[file_number],"vt_count",DFNT_INT32,1);
	VSfdefine(lis_point_summary_vsref[file_number],"vt_address",DFNT_INT32,1);


	if(VSsetfields(lis_point_summary_vsref[file_number],
		"parent_address,event_count,event_address,group_count,group_address,flash_count,flash_address,area_count,area_address,bg_count,bg_address,vt_count,vt_address")==FAIL){
		printf("lis_point_summary_vsref VSetfields failed\n");
		return (lis_operation_FAIL|lis_VSsetfields_FAIL);   
	}


	if(Vinsert(lis_point_vgroup_id[file_number],lis_point_summary_vsref[file_number])==FAIL){
		printf("lis_point_summary_vsref Vinsert failed\n");
		return (lis_operation_FAIL| lis_Vinsert_FAIL); 
	}  



/******************************** area **************************************************/

	if ((lis_area_vsref[file_number] = VSattach(lis_hdf_id[file_number],-1,"w"))==FAIL){
		printf("area_vsref VSattach failed\n");
		return (lis_operation_FAIL|lis_area_VSattach_FAIL);   
	}
			
	VSsetname(lis_area_vsref[file_number],"area");
	VSsetclass(lis_area_vsref[file_number],"data");  
		 
	VSfdefine(lis_area_vsref[file_number],"TAI93_time",DFNT_FLOAT64,1);
	VSfdefine(lis_area_vsref[file_number],"delta_time",DFNT_FLOAT32,1);
	VSfdefine(lis_area_vsref[file_number],"observe_time",DFNT_INT16,1);
	VSfdefine(lis_area_vsref[file_number],"location",DFNT_FLOAT32,2);/*25*/
	VSfdefine(lis_area_vsref[file_number],"net_radiance",DFNT_FLOAT32,1);
	VSfdefine(lis_area_vsref[file_number],"footprint",DFNT_FLOAT32,1);
	VSfdefine(lis_area_vsref[file_number],"address",DFNT_INT32,1);
	VSfdefine(lis_area_vsref[file_number],"parent_address",DFNT_INT32,1);/*55*/
	VSfdefine(lis_area_vsref[file_number],"child_address",DFNT_INT32,1);
	VSfdefine(lis_area_vsref[file_number],"child_count",DFNT_INT32,1);
	VSfdefine(lis_area_vsref[file_number],"grandchild_count",DFNT_INT32,1);
	VSfdefine(lis_area_vsref[file_number],"greatgrandchild_count",DFNT_INT32,1);
	VSfdefine(lis_area_vsref[file_number],"approx_threshold",DFNT_INT8,1);
	VSfdefine(lis_area_vsref[file_number],"alert_flag",DFNT_UINT8,1);
	VSfdefine(lis_area_vsref[file_number],"cluster_index",DFNT_INT8,1);/*100*/
	VSfdefine(lis_area_vsref[file_number],"density_index",DFNT_INT8,1);
	VSfdefine(lis_area_vsref[file_number],"noise_index",DFNT_INT8,1);
	VSfdefine(lis_area_vsref[file_number],"oblong_index",DFNT_FLOAT32,1);/*118*/
	VSfdefine(lis_area_vsref[file_number],"grouping_sequence",DFNT_INT32,1);
	VSfdefine(lis_area_vsref[file_number],"grouping_status",DFNT_INT8,1);

	if(VSsetfields(lis_area_vsref[file_number],
		"TAI93_time,delta_time,observe_time,location,net_radiance,footprint,address,parent_address,child_address,child_count,grandchild_count,greatgrandchild_count,approx_threshold,alert_flag,cluster_index,density_index,noise_index,oblong_index,grouping_sequence,grouping_status")==FAIL){
		printf("area_vsref VSetfields failed\n");
		return (lis_operation_FAIL|lis_VSsetfields_FAIL);   
	}
	if(Vinsert(lis_lightning_vgroup_id[file_number],lis_area_vsref[file_number])==FAIL){
		printf("area_vgroup_id Vinsert failed\n");
		return (lis_operation_FAIL| lis_Vinsert_FAIL); 
	}  
/******************************** flash **************************************************/

	if ((lis_flash_vsref[file_number] = VSattach(lis_hdf_id[file_number],-1,"w"))==FAIL){
		printf("flash_vsref VSattach failed\n");
		return (lis_operation_FAIL|lis_flash_VSattach_FAIL);   
	}
			
	VSsetname(lis_flash_vsref[file_number],"flash");
	VSsetclass(lis_flash_vsref[file_number],"data");  
		 
	VSfdefine(lis_flash_vsref[file_number],"TAI93_time",DFNT_FLOAT64,1);
	VSfdefine(lis_flash_vsref[file_number],"delta_time",DFNT_FLOAT32,1);
	VSfdefine(lis_flash_vsref[file_number],"observe_time",DFNT_INT16,1);
	VSfdefine(lis_flash_vsref[file_number],"location",DFNT_FLOAT32,2);/*25*/
	VSfdefine(lis_flash_vsref[file_number],"radiance",DFNT_FLOAT32,1);
	VSfdefine(lis_flash_vsref[file_number],"footprint",DFNT_FLOAT32,1);
	VSfdefine(lis_flash_vsref[file_number],"address",DFNT_INT32,1);
	VSfdefine(lis_flash_vsref[file_number],"parent_address",DFNT_INT32,1);/*55*/
	VSfdefine(lis_flash_vsref[file_number],"child_address",DFNT_INT32,1);
	VSfdefine(lis_flash_vsref[file_number],"child_count",DFNT_INT32,1);
	VSfdefine(lis_flash_vsref[file_number],"grandchild_count",DFNT_INT32,1);
	VSfdefine(lis_flash_vsref[file_number],"approx_threshold",DFNT_INT8,1);
	VSfdefine(lis_flash_vsref[file_number],"alert_flag",DFNT_UINT8,1);
	VSfdefine(lis_flash_vsref[file_number],"cluster_index",DFNT_INT8,1);/*100*/
	VSfdefine(lis_flash_vsref[file_number],"density_index",DFNT_INT8,1);
	VSfdefine(lis_flash_vsref[file_number],"noise_index",DFNT_INT8,1);
	VSfdefine(lis_flash_vsref[file_number],"glint_index",DFNT_FLOAT32,1);
	VSfdefine(lis_flash_vsref[file_number],"oblong_index",DFNT_FLOAT32,1);/*118*/
	VSfdefine(lis_flash_vsref[file_number],"grouping_sequence",DFNT_INT32,1);
	VSfdefine(lis_flash_vsref[file_number],"grouping_status",DFNT_INT8,1);

	if(VSsetfields(lis_flash_vsref[file_number],
		"TAI93_time,delta_time,observe_time,location,radiance,footprint,address,parent_address,child_address,child_count,grandchild_count,approx_threshold,alert_flag,cluster_index,density_index,noise_index,glint_index,oblong_index,grouping_sequence,grouping_status")==FAIL){
		printf("flash_vsref VSetfields failed\n");
		return (lis_operation_FAIL|lis_VSsetfields_FAIL);   
	}
	if(Vinsert(lis_lightning_vgroup_id[file_number],lis_flash_vsref[file_number])==FAIL){
		printf("flash_vgroup_id Vinsert failed\n");
		return (lis_operation_FAIL| lis_Vinsert_FAIL); 
	}  
/******************************** group **************************************************/

	if ((lis_group_vsref[file_number] = VSattach(lis_hdf_id[file_number],-1,"w"))==FAIL){
		printf("group_vsref VSattach failed\n");
		return (lis_operation_FAIL|lis_group_VSattach_FAIL);   
	}
			
	VSsetname(lis_group_vsref[file_number],"group");
	VSsetclass(lis_group_vsref[file_number],"data");  
		 
	VSfdefine(lis_group_vsref[file_number],"TAI93_time",DFNT_FLOAT64,1);
	VSfdefine(lis_group_vsref[file_number],"observe_time",DFNT_INT16,1);
	VSfdefine(lis_group_vsref[file_number],"location",DFNT_FLOAT32,2);/*25*/
	VSfdefine(lis_group_vsref[file_number],"radiance",DFNT_FLOAT32,1);
	VSfdefine(lis_group_vsref[file_number],"footprint",DFNT_FLOAT32,1);
	VSfdefine(lis_group_vsref[file_number],"address",DFNT_INT32,1);
	VSfdefine(lis_group_vsref[file_number],"parent_address",DFNT_INT32,1);/*55*/
	VSfdefine(lis_group_vsref[file_number],"child_address",DFNT_INT32,1);
	VSfdefine(lis_group_vsref[file_number],"child_count",DFNT_INT32,1);
	VSfdefine(lis_group_vsref[file_number],"approx_threshold",DFNT_INT8,1);
	VSfdefine(lis_group_vsref[file_number],"alert_flag",DFNT_UINT8,1);
	VSfdefine(lis_group_vsref[file_number],"cluster_index",DFNT_INT8,1);/*100*/
	VSfdefine(lis_group_vsref[file_number],"density_index",DFNT_INT8,1);
	VSfdefine(lis_group_vsref[file_number],"noise_index",DFNT_INT8,1);
	VSfdefine(lis_group_vsref[file_number],"glint_index",DFNT_FLOAT32,1);
	VSfdefine(lis_group_vsref[file_number],"oblong_index",DFNT_FLOAT32,1);/*118*/
	VSfdefine(lis_group_vsref[file_number],"grouping_sequence",DFNT_INT32,1);
	VSfdefine(lis_group_vsref[file_number],"grouping_status",DFNT_INT8,1);

	if(VSsetfields(lis_group_vsref[file_number],
		"TAI93_time,observe_time,location,radiance,footprint,address,parent_address,child_address,child_count,approx_threshold,alert_flag,cluster_index,density_index,noise_index,glint_index,oblong_index,grouping_sequence,grouping_status")==FAIL){
		printf("group_vsref VSetfields failed\n");
		return (lis_operation_FAIL|lis_VSsetfields_FAIL);   
	}
	if(Vinsert(lis_lightning_vgroup_id[file_number],lis_group_vsref[file_number])==FAIL){
		printf("group_vgroup_id Vinsert failed\n");
		return (lis_operation_FAIL| lis_Vinsert_FAIL); 
	}  
/******************************** event **************************************************/

	if ((lis_event_vsref[file_number] = VSattach(lis_hdf_id[file_number],-1,"w"))==FAIL){
		printf("event_vsref VSattach failed\n");
		return (lis_operation_FAIL|lis_event_VSattach_FAIL);   
	}
			
	VSsetname(lis_event_vsref[file_number],"event");
	VSsetclass(lis_event_vsref[file_number],"data");  
		 
	VSfdefine(lis_event_vsref[file_number],"TAI93_time",DFNT_FLOAT64,1);
	VSfdefine(lis_event_vsref[file_number],"observe_time",DFNT_INT16,1);
	VSfdefine(lis_event_vsref[file_number],"location",DFNT_FLOAT32,2);/*20*/
	VSfdefine(lis_event_vsref[file_number],"radiance",DFNT_FLOAT32,1);
	VSfdefine(lis_event_vsref[file_number],"footprint",DFNT_FLOAT32,1);
	VSfdefine(lis_event_vsref[file_number],"address",DFNT_INT32,1);
	VSfdefine(lis_event_vsref[file_number],"parent_address",DFNT_INT32,1);/*50*/
	VSfdefine(lis_event_vsref[file_number],"x_pixel",DFNT_INT8,1);
	VSfdefine(lis_event_vsref[file_number],"y_pixel",DFNT_INT8,1);
	VSfdefine(lis_event_vsref[file_number],"bg_value",DFNT_INT16,1);
	VSfdefine(lis_event_vsref[file_number],"bg_radiance",DFNT_INT16,1);
	VSfdefine(lis_event_vsref[file_number],"amplitude",DFNT_INT8,1);/*75*/
	VSfdefine(lis_event_vsref[file_number],"sza_index",DFNT_UINT8,1);
	VSfdefine(lis_event_vsref[file_number],"glint_index",DFNT_UINT8,1);
	VSfdefine(lis_event_vsref[file_number],"approx_threshold",DFNT_INT8,1);
	VSfdefine(lis_event_vsref[file_number],"alert_flag",DFNT_UINT8,1);/*100*/
	VSfdefine(lis_event_vsref[file_number],"cluster_index",DFNT_INT8,1);
	VSfdefine(lis_event_vsref[file_number],"density_index",DFNT_INT8,1);
	VSfdefine(lis_event_vsref[file_number],"noise_index",DFNT_INT8,1);/*119*/
	VSfdefine(lis_event_vsref[file_number],"bg_value_flag",DFNT_INT8,1);
	VSfdefine(lis_event_vsref[file_number],"grouping_sequence",DFNT_INT32,1);


	if(VSsetfields(lis_event_vsref[file_number],
		"TAI93_time,observe_time,location,radiance,footprint,address,parent_address,x_pixel,y_pixel,bg_value,bg_radiance,amplitude,sza_index,glint_index,approx_threshold,alert_flag,cluster_index,density_index,noise_index,bg_value_flag,grouping_sequence")==FAIL){
		printf("event_vsref VSetfields failed\n");
		return (lis_operation_FAIL|lis_VSsetfields_FAIL);   
	}


	if(Vinsert(lis_lightning_vgroup_id[file_number],lis_event_vsref[file_number])==FAIL){
		printf("event_vgroup_id Vinsert failed\n");
		return (lis_operation_FAIL| lis_Vinsert_FAIL); 
	}  
/******************************** viewtime **************************************************/

	if ((lis_viewtime_vsref[file_number] = VSattach(lis_hdf_id[file_number],-1,"w"))==FAIL){
		printf("viewtime_vsref VSattach failed\n");
		return (lis_operation_FAIL|lis_viewtime_VSattach_FAIL);   
	}
			
	VSsetname(lis_viewtime_vsref[file_number],"viewtime");
	VSsetclass(lis_viewtime_vsref[file_number],"data");  
		 
	VSfdefine(lis_viewtime_vsref[file_number],"location",DFNT_FLOAT32,2);
	VSfdefine(lis_viewtime_vsref[file_number],"TAI93_start",DFNT_INT32,1);
	VSfdefine(lis_viewtime_vsref[file_number],"TAI93_end",DFNT_INT32,1);
	VSfdefine(lis_viewtime_vsref[file_number],"effective_obs",DFNT_FLOAT32,1);
	VSfdefine(lis_viewtime_vsref[file_number],"alert_flag",DFNT_UINT8,1);
	VSfdefine(lis_viewtime_vsref[file_number],"approx_threshold",DFNT_INT8,1);


	if(VSsetfields(lis_viewtime_vsref[file_number],
		"location,TAI93_start,TAI93_end,effective_obs,alert_flag,approx_threshold")==FAIL){
		printf("viewtime_vsref VSetfields failed\n");
		return (lis_operation_FAIL|lis_VSsetfields_FAIL);   
	}


	if(Vinsert(lis_point_vgroup_id[file_number],lis_viewtime_vsref[file_number])==FAIL){
		printf("viewtime_vgroup_id Vinsert failed\n");
		return (lis_operation_FAIL| lis_Vinsert_FAIL); 
	}  

/******************************** background image summary **************************************************/

	if ((lis_bg_summary_vsref[file_number] = VSattach(lis_hdf_id[file_number],-1,"w"))==FAIL){
		printf("bg_summary_vsref VSattach failed\n");
		return (lis_operation_FAIL|lis_bg_summary_VSattach_FAIL);   
	}
			
	VSsetname(lis_bg_summary_vsref[file_number],"bg_summary");
	VSsetclass(lis_bg_summary_vsref[file_number],"data");  
		 
	VSfdefine(lis_bg_summary_vsref[file_number],"TAI93_time",DFNT_FLOAT64,1);
	VSfdefine(lis_bg_summary_vsref[file_number],"address",DFNT_INT32,1);
	VSfdefine(lis_bg_summary_vsref[file_number],"boresight",DFNT_FLOAT32,2);
	VSfdefine(lis_bg_summary_vsref[file_number],"corners",DFNT_FLOAT32,8);


	if(VSsetfields(lis_bg_summary_vsref[file_number],
		"TAI93_time,address,boresight,corners")==FAIL){
		printf("bg_summary_vsref VSetfields failed\n");
		return (lis_operation_FAIL|lis_VSsetfields_FAIL);   
	}


	if(Vinsert(lis_point_vgroup_id[file_number],lis_bg_summary_vsref[file_number])==FAIL){
		printf("bg_summary_vgroup_id Vinsert failed\n");
		return (lis_operation_FAIL| lis_Vinsert_FAIL); 
	}  

	return file_number;

}

long create_LIS_BACKGROUND_HDF(char *hdf_fname)
{
	long hid_tmp,i,file_number,sd_tmp,success_flag;
 

/********************************* initialize H interface *******************************/
	
	hid_tmp = (Hopen(hdf_fname,DFACC_CREATE,256)); 
	if(hid_tmp==FAIL){
#ifdef DEBUG
printf("Hopen failed\n");
#endif
		return (FAIL);   
	}
	file_number = -1;
	for(i=0;i<MAX_FILE;i++){
		if(lis_hdf_id[i]==-1){
			file_number = i;
			break;
		}
	}
	if(file_number==-1){
#ifdef DEBUG
printf("MAX_FILE exceeded\n");
#endif
		success_flag = lis_operation_FAIL|lis_MAX_FILE_FAIL;
		return(success_flag);
	}
	if(Vstart( hid_tmp)==FAIL){
#ifdef DEBUG
printf("Vstart  failed\n");
#endif
		success_flag = lis_operation_FAIL|lis_Vstart_FAIL;
		return(success_flag);
	}
	lis_hdf_id[file_number] = hid_tmp;

/****************************** create vgroups ******************************************/
	lis_bg_orbit_data_vgroup_id[file_number] = create_lis_VGROUP(lis_hdf_id[file_number],"orbit","data");



/********************** initailize internal data structures/pointers ********************/

	lis_image_srecord[file_number] = 0;
	lis_bg_data_summary_vrecord[file_number] = 0;
	lis_bg_info_vrecord[file_number] = 0;
	lis_bg_orbit_summary_vrecord[file_number] = 0;

/******************************** orbit summary **************************************************/

	if ((lis_bg_orbit_summary_vsref[file_number] = VSattach(lis_hdf_id[file_number],-1,"w"))==FAIL){
#ifdef DEBUG
printf("lis_bg_orbit_summary_vsref VSattach failed\n");
#endif
		return (lis_operation_FAIL|lis_bg_orbit_summary_VSattach_FAIL);   
	}
			
	VSsetname(lis_bg_orbit_summary_vsref[file_number],"bg_orbit_summary");
	VSsetclass(lis_bg_orbit_summary_vsref[file_number],"data");  
		 
	VSfdefine(lis_bg_orbit_summary_vsref[file_number],"id_number",DFNT_INT32,1);
	VSfdefine(lis_bg_orbit_summary_vsref[file_number],"TAI93_start",DFNT_FLOAT64,1);
	VSfdefine(lis_bg_orbit_summary_vsref[file_number],"UTC_start",DFNT_CHAR,28);
	VSfdefine(lis_bg_orbit_summary_vsref[file_number],"GPS_start",DFNT_FLOAT64,1);
	VSfdefine(lis_bg_orbit_summary_vsref[file_number],"TAI93_end",DFNT_FLOAT64,1);
	VSfdefine(lis_bg_orbit_summary_vsref[file_number],"start_longitude",DFNT_FLOAT32,1);
	VSfdefine(lis_bg_orbit_summary_vsref[file_number],"end_longitude",DFNT_FLOAT32,1);
	VSfdefine(lis_bg_orbit_summary_vsref[file_number],"image_summary_count",DFNT_INT16,1);
	VSfdefine(lis_bg_orbit_summary_vsref[file_number],"image_summary_address",DFNT_INT16,1);
	VSfdefine(lis_bg_orbit_summary_vsref[file_number],"image_info_count",DFNT_INT16,1);
	VSfdefine(lis_bg_orbit_summary_vsref[file_number],"image_info_address",DFNT_INT16,1);


	if(VSsetfields(lis_bg_orbit_summary_vsref[file_number],
		"id_number,TAI93_start,UTC_start,GPS_start,TAI93_end,start_longitude,end_longitude,image_summary_count,image_summary_address,image_info_count,image_info_address")==FAIL){
#ifdef DEBUG
printf("lis_bg_orbit_summary_vsref VSetfields failed\n");
#endif
		return (lis_operation_FAIL|lis_VSsetfields_FAIL);   
	}


	if(Vinsert(lis_bg_orbit_data_vgroup_id[file_number],lis_bg_orbit_summary_vsref[file_number])==FAIL){
#ifdef DEBUG
printf("bg_orbit_summary_vgroup_id Vinsert (lis_bg_orbit_summary_vsref) failed\n");
#endif
		return (lis_operation_FAIL|lis_Vinsert_FAIL); 
	}  


/******************************** bg_info **************************************************/
	if ((lis_bg_info_vsref[file_number] = VSattach(lis_hdf_id[file_number],-1,"w"))==FAIL){
#ifdef DEBUG
printf("lis_bg_info_vsref VSattach failed\n");
#endif
		return (lis_operation_FAIL| lis_bg_info_VSattach_FAIL);   
	}
			
	VSsetname(lis_bg_info_vsref[file_number],"bg_info");
	VSsetclass(lis_bg_info_vsref[file_number],"data");  
		 
	VSfdefine(lis_bg_info_vsref[file_number],"TAI93_time",DFNT_FLOAT64,1);
	VSfdefine(lis_bg_info_vsref[file_number],"alert_summary",DFNT_UINT8,1);
	VSfdefine(lis_bg_info_vsref[file_number],"instrument_alert",DFNT_UINT8,1);
	VSfdefine(lis_bg_info_vsref[file_number],"platform_alert",DFNT_UINT8,1);
	VSfdefine(lis_bg_info_vsref[file_number],"external_alert",DFNT_UINT8,1);
	VSfdefine(lis_bg_info_vsref[file_number],"processing_alert",DFNT_UINT8,1);
	VSfdefine(lis_bg_info_vsref[file_number],"position_vector",DFNT_FLOAT32,3);
	VSfdefine(lis_bg_info_vsref[file_number],"velocity_vector",DFNT_FLOAT32,3);
	VSfdefine(lis_bg_info_vsref[file_number],"transform_matrix",DFNT_FLOAT32,9);
	VSfdefine(lis_bg_info_vsref[file_number],"solar_vector",DFNT_FLOAT32,3);
	VSfdefine(lis_bg_info_vsref[file_number],"ephemeris_quality_flag",DFNT_INT32,1);
	VSfdefine(lis_bg_info_vsref[file_number],"attitude_quality_flag",DFNT_INT32,1);
	VSfdefine(lis_bg_info_vsref[file_number],"bg_value",DFNT_INT16,1);
	VSfdefine(lis_bg_info_vsref[file_number],"noise_index",DFNT_INT8,1);
	VSfdefine(lis_bg_info_vsref[file_number],"event_count",DFNT_INT16,6);


	if(VSsetfields(lis_bg_info_vsref[file_number],
		"TAI93_time,alert_summary,instrument_alert,platform_alert,external_alert,processing_alert,position_vector,velocity_vector,transform_matrix,solar_vector,ephemeris_quality_flag,attitude_quality_flag,bg_value,noise_index,event_count")==FAIL){
#ifdef DEBUG
printf("lis_bg_info_vsref VSetfields failed\n");
#endif
		return (lis_operation_FAIL|lis_VSsetfields_FAIL );   
	}

	if(Vinsert(lis_bg_orbit_data_vgroup_id[file_number],lis_bg_info_vsref[file_number])==FAIL){
#ifdef DEBUG
printf("bg_orbit_summary_vgroup_id Vinsert (lis_bg_info_vsref) failed\n");
#endif
		return (lis_operation_FAIL|lis_Vinsert_FAIL ); 
	}  




/******************************** background image summary **************************************************/

	if ((lis_bg_data_summary_vsref[file_number] = VSattach(lis_hdf_id[file_number],-1,"w"))==FAIL){
#ifdef DEBUG
printf("lis_bg_data_summary_vsref VSattach failed\n");
#endif
		return (lis_operation_FAIL|lis_bg_data_summary_VSattach_FAIL );   
	}
			
	VSsetname(lis_bg_data_summary_vsref[file_number],"bg_data_summary");
	VSsetclass(lis_bg_data_summary_vsref[file_number],"data");  
		 
	VSfdefine(lis_bg_data_summary_vsref[file_number],"TAI93_time",DFNT_FLOAT64,1);
	VSfdefine(lis_bg_data_summary_vsref[file_number],"address",DFNT_INT32,1);
	VSfdefine(lis_bg_data_summary_vsref[file_number],"boresight",DFNT_FLOAT32,2);
	VSfdefine(lis_bg_data_summary_vsref[file_number],"corners",DFNT_FLOAT32,8);


	if(VSsetfields(lis_bg_data_summary_vsref[file_number],
		"TAI93_time,address,boresight,corners")==FAIL){
#ifdef DEBUG
printf("lis_bg_data_summary_vsref VSetfields failed\n");
#endif
		return (lis_operation_FAIL|lis_VSsetfields_FAIL);   
	}
	if(Vinsert(lis_bg_orbit_data_vgroup_id[file_number],lis_bg_data_summary_vsref[file_number])==FAIL){
#ifdef DEBUG
printf("bg_orbit_summary_vgroup_id Vinsert (lis_bg_data_summary_vsref) failed\n");
#endif
		return (lis_operation_FAIL|lis_Vinsert_FAIL); 
	}  



/*************************** create scientific data sets (SDS) **************************/

	sd_tmp = SDstart(hdf_fname,DFACC_RDWR);
	if(sd_tmp==-1){
#ifdef DEBUG
printf("SDstart failed\n");
#endif
		return (FAIL);   
	}
	lis_sd_id[file_number] = sd_tmp;
	lis_dimsize[0]=0;
	lis_dimsize[1]=128;
	lis_dimsize[2]=128;
	lis_image_sds_id[file_number] = SDcreate(lis_sd_id[file_number],"bg_data",DFNT_INT16,3,lis_dimsize); 
 
	lis_dimsize[0]=128;
	lis_dimsize[1]=128;


	return file_number;
}

long close_LIS_ORBIT_HDF(int32 file_number)

/****************************************************************************************/
/*                                                                             		*/
/*	close_hdf_file									*/
/*											*/
/*	returns 0 if all vgroups, vsets and the file itself are successfully    	*/
/*	closed										*/
/*	returns FAIL(-1) on error							*/
/*											*/
/*	Date last modified 9/7/94							*/
/****************************************************************************************/

{
	long return_status;
	return_status = 0;
	

        if(lis_event_vsref[file_number]!= FAIL)VSdetach(lis_event_vsref[file_number]);			/* Events */

	if(lis_group_vsref[file_number]!= FAIL)VSdetach(lis_group_vsref[file_number]);			/* Group */

	if(lis_flash_vsref[file_number]!= FAIL)VSdetach(lis_flash_vsref[file_number]);			/* Flash */

	if(lis_area_vsref[file_number]!= FAIL)VSdetach(lis_area_vsref[file_number]);			/* Area */

	if(lis_lightning_vgroup_id[file_number]!= FAIL)Vdetach(lis_lightning_vgroup_id[file_number]);

	
	if(lis_viewtime_vsref[file_number]!= FAIL)VSdetach(lis_viewtime_vsref[file_number]);		/* viewtime */
	
	if(lis_point_summary_vsref[file_number]!= FAIL)VSdetach(lis_point_summary_vsref[file_number]);		/* Point summary */

	if(lis_bg_summary_vsref[file_number]!= FAIL)VSdetach(lis_bg_summary_vsref[file_number]);		/* background summary */

	if(lis_one_second_vsref[file_number]!= FAIL)VSdetach(lis_one_second_vsref[file_number]);		/* one second summary */

	if(lis_orbit_summary_vsref[file_number]!= FAIL)VSdetach(lis_orbit_summary_vsref[file_number]);		/* orbit summary */

	if(lis_point_vgroup_id[file_number]!= FAIL)Vdetach(lis_point_vgroup_id[file_number]);
	if(lis_orbit_data_vgroup_id[file_number]!= FAIL)Vdetach(lis_orbit_data_vgroup_id[file_number]);
		

	if(lis_hdf_id[file_number]!=FAIL)Vend(lis_hdf_id[file_number]);

	if(lis_hdf_id[file_number] !=FAIL)if(Hclose(lis_hdf_id[file_number])==FAIL)return_status = FAIL;	

	lis_hdf_id[file_number] = -1;

	if (return_status != lis_operation_SUCCESSFUL) {
		fprintf(stderr,"Error closing HDF file.\n");
	}
	return (return_status);		/* success*/




}

long close_LIS_BACKGROUND_HDF(int32 file_number)

/****************************************************************************************/
/*                                                                             		*/
/*	close_hdf_file									*/
/*											*/
/*	returns 0 if all vgroups, vsets and the file itself are successfully    	*/
/*	closed										*/
/*	returns FAIL(-1) on error							*/
/*											*/
/*	Date last modified 9/7/94							*/
/****************************************************************************************/

{
	long return_status;
	return_status = 0;
	
	if(lis_bg_orbit_summary_vsref[file_number]!= FAIL)VSdetach(lis_bg_orbit_summary_vsref[file_number]);		/* bg orbit data summary */
	if(lis_bg_info_vsref[file_number]!= FAIL)VSdetach(lis_bg_info_vsref[file_number]);			/* bg info summary */
	if(lis_bg_data_summary_vsref[file_number]!= FAIL)VSdetach(lis_bg_data_summary_vsref[file_number]);		/* background summary */
	if(lis_bg_orbit_data_vgroup_id[file_number]!=FAIL)Vdetach(lis_bg_orbit_data_vgroup_id[file_number]);

 /*
	if(SDsetattr(image_sds_id[file_number],"total images",DFNT_INT32,1,&image_srecord[file_number])==FAIL)
	return_status = FAIL;
*/

	if(lis_image_sds_id[file_number]!=FAIL)if(SDendaccess(lis_image_sds_id[file_number])==FAIL)return_status = FAIL;	/* image */
		 

 	if(lis_sd_id[file_number]!=FAIL)if (SDend(lis_sd_id[file_number])==FAIL)return_status = FAIL;			/* SD id */

	if(lis_hdf_id[file_number]!=FAIL)Vend(lis_hdf_id[file_number]);

	if(lis_hdf_id[file_number] !=FAIL)if(Hclose(lis_hdf_id[file_number])==FAIL)return_status = FAIL;	

	lis_hdf_id[file_number] = -1;
	return return_status;
}


long create_lis_VGROUP(long lis_hdf_id,char *name,char *class)

/****************************************************************************************/
/*											*/
/*	create a vgroup in the HDF file							*/
/*											*/
/*	returns the reference number of the Vgroup if successful			*/
/*	returns FAIL (-1) on error							*/
/*											*/
/*	Date last modified	8/15/94							*/
/****************************************************************************************/

{
	long vgroup;
	vgroup= (long)Vattach(lis_hdf_id,-1,"w");
	if(vgroup == FAIL)return FAIL;	
	Vsetname(vgroup,name);
	Vsetclass(vgroup,class);
	return (vgroup);

}


 
long write_lis_BACKGROUND_DATA(long file_number,short int *databuffer)

/****************************************************************************************/
/*											*/
/*	Write sds image data to HDF file						*/
/*											*/
/*	returns sds record # if successful						*/
/*	returns FAIL (-1) on error							*/
/*											*/
/*	Date last modified	8/15/94							*/
/****************************************************************************************/


{

	start[0]=lis_image_srecord[file_number]++;
	start[1]=start[2]=0;

	endr[0]=1;
	endr[1]=endr[2]=128;

	if(SDwritedata(lis_image_sds_id[file_number],start,NULL,endr,databuffer)==FAIL){
		lis_image_srecord[file_number]--;
		return (lis_operation_FAIL|lis_SDwritedata_FAIL);}
	return (lis_image_srecord[file_number]-1);
}




long write_lis_EVENT(long file_number,void *databuffer)		{return (write_lis_VSGROUP(file_number,databuffer,lis_event_vsref,lis_event_vrecord));}
long write_lis_GROUP(long file_number,void *databuffer) 		{return (write_lis_VSGROUP(file_number,databuffer,lis_group_vsref,lis_group_vrecord));}
long write_lis_FLASH(long file_number,void *databuffer) 		{return (write_lis_VSGROUP(file_number,databuffer,lis_flash_vsref,lis_flash_vrecord));}
long write_lis_AREA(long file_number,void *databuffer)	 	{return (write_lis_VSGROUP(file_number,databuffer,lis_area_vsref,lis_area_vrecord));}
long write_lis_POINT_SUMMARY(long file_number,void *databuffer)	{return (write_lis_VSGROUP(file_number,databuffer,lis_point_summary_vsref,lis_point_summary_vrecord));}
long write_lis_VIEWTIME(long file_number,void *databuffer)	 	{return (write_lis_VSGROUP(file_number,databuffer,lis_viewtime_vsref,lis_viewtime_vrecord));}
long write_lis_BG_SUMMARY(long file_number,void *databuffer)	{return (write_lis_VSGROUP(file_number,databuffer,lis_bg_summary_vsref,lis_bg_summary_vrecord));}
long write_lis_ONE_SECOND(long file_number,void *databuffer)	{return (write_lis_VSGROUP(file_number,databuffer,lis_one_second_vsref,lis_one_second_vrecord));}
long write_lis_ORBIT_SUMMARY(long file_number,void *databuffer)	{return (write_lis_VSGROUP(file_number,databuffer,lis_orbit_summary_vsref,lis_orbit_summary_vrecord));}
long write_lis_BG_DATA_SUMMARY(long file_number,void *databuffer)	{return (write_lis_VSGROUP(file_number,databuffer,lis_bg_data_summary_vsref,lis_bg_data_summary_vrecord));}
long write_lis_BG_INFO(long file_number,void *databuffer)		{return (write_lis_VSGROUP(file_number,databuffer,lis_bg_info_vsref,lis_bg_info_vrecord));}
long write_lis_BG_ORBIT_SUMMARY(long file_number,void *databuffer)	{return (write_lis_VSGROUP(file_number,databuffer,lis_bg_orbit_summary_vsref,lis_bg_orbit_summary_vrecord));}


long write_lis_VSGROUP(long file_number,void *databuffer,long vsref[],long vsrecord[])

/****************************************************************************************/
/*											*/
/*	Write data to HDF file								*/
/*											*/
/*	returns Vdata record # if successful						*/
/*	returns VSwrite_FAIL on error							*/
/*											*/
/*	Date last modified	8/15/94							*/
/****************************************************************************************/


{
	vsrecord[file_number]++;
 	if(VSwrite(vsref[file_number],databuffer,1,FULL_INTERLACE)==FAIL){
		vsrecord[file_number]--;
		return (lis_operation_FAIL|lis_VSwrite_FAIL);
	}
	return vsrecord[file_number]-1;
}
