#define DEBUG 1
/****************************************************************************************/
/*											*/
/*	Compatible with HDF version 4.0						*/
/*											*/
/*											*/
/*											*/
/*	LIS/OTD HDF database access library						*/
/*											*/
/*	FUNCTION NAME		TYPE	DESCRIPTION					*/
/*	-------------------	----	----------------------------------------------- */
/*	create_OTD_HDF_FILE	long	Create and initialize LIS/OTD archive file	*/
/*	close_OTD_HDF_FILE	long	Gracefully detach and close archive file	*/
/*											*/
/*	write_otd_IMAGE		long	Write background image (SDS) to archive file	*/
/*	write_otd_ORBIT_DATA	long	Write TRMM orbit data to archive file		*/
/*	write_otd_METADATA		long	Write the Metadata to archive file		*/
/*	write_otd_ORBIT_SUMMARY	long	Write the orbit summary data to archive file	*/
/*	write_otd_VIEW		long	Write the view time data to archive file	*/
/*	write_otd_GRID_25		long	Write the 2.5 deg. data to archive file		*/
/*	write_otd_GRID_500_KM	long	Write the 500 km data to archive file		*/
/*	write_otd_AREA		long	Write AREA data to archive file			*/
/*	write_otd_FLASH		long	Write FLASH data to archive file		*/
/*	write_otd_GROUP		long	Write GROUP data to archive file		*/
/*	write_otd_EVENT		long	Write the EVENT data to archive file		*/
/*	write_otd_RATE		long	Write the EVENT rate data to archive file	*/
/*	write_otd_VECTOR_PRODUCTS	long	Write VECTOR PRODUCTS data to archive file	*/
/*	write_otd_IMAGE_ATTRIBUTE	long	Write background image data to archive file	*/
/*	write_otd_BROWSE_DATA	long	Write BROWSE DATA to archive file		*/
/*	write_otd_EPHEMERIS		long	Write SPICE ephemeris data to archive file	*/
/*	write_otd_THRESHOLD		long	Write sensor threshold data to archive file	*/
/*											*/
/*											*/
/*											*/
/*	write_otd_VSGROUP		long	generic VSwrite routine	(hiddden from user)	*/
/*	write_otd_SDS		long    generic SDwritedata routine (hidden from user)	*/
/*											*/
/*											*/
/*											*/
/*											*/
/*	*********************************************************			*/
/*	* NOTE!!					      	*			*/
/*	*	All functions return FAIL (-1) on error		*			*/
/*	*********************************************************			*/
/*											*/
/****************************************************************************************/


/****************************************************************************************/
/**************************************		*****************************************/
/************************************** HISTORY	*****************************************/
/**************************************		*****************************************/
/****************************************************************************************/
/*											*/
/*	Initial library development 9/1/94	John Hall/CSC (jmh)			*/
/*						e-mail:					*/
/*						       	jhall@deltaq.msfc.nasa.gov	*/
/*						       	johnny.hall@msfc.nasa.gov	*/
/*						phone:					*/
/*						  	(205) 922-5972			*/
/*											*/
/*											*/
/* Change History:									*/
/*											*/
/*											*/
/* Date		Module			Change description			initail */
/* --------	-------------------	-----------------------------------	-------	*/
/* 1/13/95	Create/write funcs.	Major change request from D Mach	jmh	*/
/*					virtual re-write of code!			*/
/*											*/
/* 2/22/95	Create/write funcs	Created orbit_summary,area_flash_rate,	jmh	*/
/*					unique_event_location,2.5_grid,			*/
/*					500km_grid,Vdatas				*/
/*											*/
/* 10/3/95	Removed			unique_event_location			jmh	*/
/*					removed locations and pointers from		*/
/*					groups, flashes, and areas 			*/
/*											*/
/*		Added			QA,TAI,location to event vdata		jmh	*/
/*											*/
/*											*/
/* 10/4/95	Added			View time vset				jmh	*/
/*											*/
/*											*/
/*											*/
/*											*/
/*											*/
/*											*/
/*											*/
/****************************************************************************************/

#include "hdf.h"
#include "liblisotd_hdf.h"
#include "liblisotd_err_defs.h"
#include <fcntl.h>
#include <stdlib.h>
long create_otd_VGROUP(long otd_hdf_id,char *name,char *class);
long write_otd_VSGROUP(void *databuffer,long vsref,long *vsrecord);
long write_otd_VSGROUP(void *databuffer,long vsref,long *vsrecord);
long write_otd_SDS(short int *databuffer,long sds_id);


long create_OTD_HDF_FILE(char *hdf_fname)
{
 
/********************** initailize internal data structures/pointers ********************/
	otd_image_srecord =0;

	otd_image_vrecord = 0;
	otd_vector_vrecord = 0;

	otd_area_vrecord = 0;
	otd_flash_vrecord = 0;
	otd_group_vrecord = 0;
	otd_event_vrecord = 0;
	otd_view_vrecord = 0;
	otd_orbit_event_rate_vrecord = 0;
	otd_grid_25_vrecord = 0;
	otd_grid_500_km_vrecord = 0;
	otd_orbit_flash_rate_vrecord = 0;
	otd_orbit_event_rate_vrecord = 0;
	otd_orbit_summary_vrecord = 0;
	otd_threshold_vrecord = 0;

/********************************* initialize H interface *******************************/

	otd_hdf_id = (Hopen(hdf_fname,DFACC_CREATE,256)); 
	if(otd_hdf_id==FAIL)return otd_hdf_id;
	Vstart(otd_hdf_id);

/****************************** create vgroups ******************************************/

	otd_area_vgroup_id = create_otd_VGROUP(otd_hdf_id,"Area","statistics");
	otd_flash_vgroup_id = create_otd_VGROUP(otd_hdf_id,"Flash","statistics");
	otd_group_vgroup_id = create_otd_VGROUP(otd_hdf_id,"Group","statistics");
	otd_event_vgroup_id = create_otd_VGROUP(otd_hdf_id,"Event","statistics");
	otd_browse_vgroup_id = create_otd_VGROUP(otd_hdf_id,"Browse","statistics");
	otd_flash_density_vgroup_id = create_otd_VGROUP(otd_hdf_id,"Flash Density","statistics");

	/* added 1/13/95 */
	otd_orbit_summary_vgroup_id = create_otd_VGROUP(otd_hdf_id,"Orbit summary","statistics");	
	/* end mod 1/13/95 */

/*************************** create scientific data sets (SDS) **************************/

	otd_sd_id = SDstart(hdf_fname,DFACC_RDWR);
	otd_dimsize[0]=0;
	otd_dimsize[1]=128;
	otd_dimsize[2]=128;
	otd_image_sds_id = SDcreate(otd_sd_id,"Background images",DFNT_INT16,3,otd_dimsize); 
 
	otd_dimsize[0]=128;
	otd_dimsize[1]=128;

/************************* orbit lone vdata decsription *********************************/

	if ((otd_orbit_vsref = VSattach(otd_hdf_id,-1,"w"))==FAIL){
#ifdef DEBUG
printf("orbit_vsref VSattach failed\n");
#endif
		return (FAIL);
	}

	VSsetname(otd_orbit_vsref,"orbit");
	VSsetclass(otd_orbit_vsref,"Attributes");	

	VSfdefine(otd_orbit_vsref,"orbit ID",DFNT_INT32,1);
	VSfdefine(otd_orbit_vsref,"TAI93 start",DFNT_FLOAT64,1);
	VSfdefine(otd_orbit_vsref,"start location",DFNT_FLOAT32,2);
	VSfdefine(otd_orbit_vsref,"TAI93 end",DFNT_FLOAT64,1);
	VSfdefine(otd_orbit_vsref,"end location",DFNT_FLOAT32,2);
	if(VSsetfields(otd_orbit_vsref,"orbit ID,TAI93 start,start location,TAI93 end,end location")==FAIL){
#ifdef DEBUG
printf("orbit_vsref VSetfields failed\n");
#endif
		return(FAIL);
	}

 /************************ metadata lone vdata description*******************************/

	if ((otd_metadata_vsref = VSattach(otd_hdf_id,-1,"w"))==FAIL){
#ifdef DEBUG
printf("metadata_vsref VSattach failed\n");
#endif
		return (FAIL);   
	}
		
	VSsetname(otd_metadata_vsref,"metadata");
	VSsetclass(otd_metadata_vsref,"description");  
		 
        VSfdefine(otd_metadata_vsref,"text",DFNT_CHAR8,80);

	if(VSsetfields(otd_metadata_vsref,"text")==FAIL){
#ifdef DEBUG
printf("metadata_vsref VSetfields failed\n");
#endif
		return (FAIL);
	}   

 /************************ Threhsold lone vdata description*******************************/

	if ((otd_threshold_vsref = VSattach(otd_hdf_id,-1,"w"))==FAIL){
#ifdef DEBUG
printf("threshold_vsref VSattach failed\n");
#endif
		return (FAIL);
	}   
			
	VSsetname(otd_threshold_vsref,"threshold");
	VSsetclass(otd_threshold_vsref,"values");  
		 
        VSfdefine(otd_threshold_vsref,"TAI93",DFNT_FLOAT64,1);
	VSfdefine(otd_threshold_vsref,"values",DFNT_UINT8,16);

	if(VSsetfields(otd_threshold_vsref,"TAI93,values")==FAIL){
#ifdef DEBUG
printf("threshold_vsref VSetfields failed\n");
#endif
		return (FAIL);  
	} 
 
/******************************** area **************************************************/

	if ((otd_area_vsref = VSattach(otd_hdf_id,-1,"w"))==FAIL){
#ifdef DEBUG
printf("area_vsref VSattach failed\n");
#endif
		return (FAIL);   
	}
			
	VSsetname(otd_area_vsref,"area");
	VSsetclass(otd_area_vsref,"statistics");  
		 
        VSfdefine(otd_area_vsref,"seq",DFNT_INT32,1);	
			
	VSfdefine(otd_area_vsref,"TAI93",DFNT_FLOAT64,1);
	VSfdefine(otd_area_vsref,"delta",DFNT_FLOAT32,1);
	VSfdefine(otd_area_vsref,"view",DFNT_FLOAT32,1);
	VSfdefine(otd_area_vsref,"s-z-a",DFNT_INT16,1);
	VSfdefine(otd_area_vsref,"d-n-t",DFNT_CHAR8,1);
	VSfdefine(otd_area_vsref,"end status",DFNT_CHAR8,1);
	VSfdefine(otd_area_vsref,"events",DFNT_INT16,1);
	VSfdefine(otd_area_vsref,"cent",DFNT_FLOAT32,2);
	VSfdefine(otd_area_vsref,"stdev",DFNT_FLOAT32,2);
	VSfdefine(otd_area_vsref,"loc count",DFNT_INT16,1);
	VSfdefine(otd_area_vsref,"rad",DFNT_FLOAT32,1);
	VSfdefine(otd_area_vsref,"orbit id",DFNT_INT32,1);
	VSfdefine(otd_area_vsref,"day",DFNT_INT32,1);
	VSfdefine(otd_area_vsref,"children",DFNT_INT32,1);
	VSfdefine(otd_area_vsref,"child seq",DFNT_INT32,1);
	VSfdefine(otd_area_vsref,"child rec",DFNT_INT32,1);
	VSfdefine(otd_area_vsref,"QA",DFNT_INT16,4);

	if(VSsetfields(otd_area_vsref,
		"seq,TAI93,delta,view,s-z-a,d-n-t,end status,events,cent,stdev,loc count,rad,orbit id,day,children,child seq,child rec,QA")==FAIL){
#ifdef DEBUG
printf("area_vsref VSetfields failed\n");
#endif
		return (FAIL);   
	}

	 if(Vinsert(otd_area_vgroup_id,otd_area_vsref)==FAIL){
#ifdef DEBUG
printf("otd_area_vgroup_id Vinsert failed\n");
#endif
		return (FAIL); 
	}  

/******************************** Flash *************************************************/
	
	if ((otd_flash_vsref = VSattach(otd_hdf_id,-1,"w"))==FAIL){
#ifdef DEBUG
printf("flash_vsref VSetfields failed\n");
#endif
		return (FAIL); 
	}  
			
	VSsetname(otd_flash_vsref,"flash");
	VSsetclass(otd_flash_vsref,"Statistics");  
		 
        VSfdefine(otd_flash_vsref,"seq",DFNT_INT32,1);	
			
	VSfdefine(otd_flash_vsref,"TAI93",DFNT_FLOAT64,1);
	VSfdefine(otd_flash_vsref,"delta",DFNT_FLOAT32,1);
	VSfdefine(otd_flash_vsref,"view",DFNT_FLOAT32,1);
	VSfdefine(otd_flash_vsref,"s-z-a",DFNT_INT16,1);
	VSfdefine(otd_flash_vsref,"d-n-t",DFNT_CHAR8,1);
	VSfdefine(otd_flash_vsref,"end status",DFNT_CHAR8,1);
	VSfdefine(otd_flash_vsref,"events",DFNT_INT16,1);
	VSfdefine(otd_flash_vsref,"cent",DFNT_FLOAT32,2);
	VSfdefine(otd_flash_vsref,"stdev",DFNT_FLOAT32,2);
	VSfdefine(otd_flash_vsref,"loc count",DFNT_INT16,1);
	VSfdefine(otd_flash_vsref,"rad",DFNT_FLOAT32,1);
	VSfdefine(otd_flash_vsref,"parent seq",DFNT_INT32,1);
	VSfdefine(otd_flash_vsref,"parent rec",DFNT_INT32,1);
	VSfdefine(otd_flash_vsref,"children",DFNT_INT32,1);
	VSfdefine(otd_flash_vsref,"child seq",DFNT_INT32,1);
	VSfdefine(otd_flash_vsref,"child rec",DFNT_INT32,1);
	VSfdefine(otd_flash_vsref,"QA",DFNT_INT16,4);

	if(VSsetfields(otd_flash_vsref,
		"seq,TAI93,delta,view,s-z-a,d-n-t,end status,events,cent,stdev,loc count,rad,parent seq,parent rec,children,child seq,child rec,QA")==FAIL){
#ifdef DEBUG
printf("flash_vsref VSetfields failed\n");
#endif
		return (FAIL); 
	}  

	if(Vinsert(otd_flash_vgroup_id,otd_flash_vsref)==FAIL){
#ifdef DEBUG
printf("otd_flash_vgroup_id Vinsert failed\n");
#endif
		return (FAIL);
	}   
 
/************************************ Group *********************************************/
	
	if ((otd_group_vsref = VSattach(otd_hdf_id,-1,"w"))==FAIL){
#ifdef DEBUG
printf("group_vsref VSattach failed\n");
#endif
		return (FAIL); 
	}  

	VSsetname(otd_group_vsref,"group");
	VSsetclass(otd_group_vsref,"Statistics");  
		 
        VSfdefine(otd_group_vsref,"seq",DFNT_INT32,1);	
			
	VSfdefine(otd_group_vsref,"TAI93",DFNT_FLOAT64,1);
	VSfdefine(otd_group_vsref,"delta",DFNT_FLOAT32,1);
	VSfdefine(otd_group_vsref,"view",DFNT_FLOAT32,1);
	VSfdefine(otd_group_vsref,"s-z-a",DFNT_INT16,1);
	VSfdefine(otd_group_vsref,"d-n-t",DFNT_CHAR8,1);
	VSfdefine(otd_group_vsref,"end status",DFNT_CHAR8,1);
	VSfdefine(otd_group_vsref,"events",DFNT_INT16,1);
	VSfdefine(otd_group_vsref,"cent",DFNT_FLOAT32,2);
	VSfdefine(otd_group_vsref,"stdev",DFNT_FLOAT32,2);
	VSfdefine(otd_group_vsref,"loc count",DFNT_INT16,1);
	VSfdefine(otd_group_vsref,"rad",DFNT_FLOAT32,1);
	VSfdefine(otd_group_vsref,"parent seq",DFNT_INT32,1);
	VSfdefine(otd_group_vsref,"parent rec",DFNT_INT32,1);
	VSfdefine(otd_group_vsref,"children",DFNT_INT32,1);
	VSfdefine(otd_group_vsref,"child seq",DFNT_INT32,1);
	VSfdefine(otd_group_vsref,"child rec",DFNT_INT32,1);
	VSfdefine(otd_group_vsref,"QA",DFNT_INT16,4);

	if(VSsetfields(otd_group_vsref,
		"seq,TAI93,delta,view,s-z-a,d-n-t,end status,events,cent,stdev,loc count,rad,parent seq,parent rec,children,child seq,child rec,QA")==FAIL){
#ifdef DEBUG
printf("group_vsref VSetfields failed\n");
#endif
		return (FAIL);
	}   
	if(Vinsert(otd_group_vgroup_id,otd_group_vsref)==FAIL){
#ifdef DEBUG
printf("group_vgroup_id Vinsert failed\n");
#endif
		return (FAIL); 
	}  

/********************************* Events *********************************************/

	if ((otd_event_vsref = VSattach(otd_hdf_id,-1,"w"))==FAIL){
#ifdef DEBUG
printf("event_vsref VSattach failed\n");
#endif
		return (FAIL); 
	}  
			
	VSsetname(otd_event_vsref,"event");
	VSsetclass(otd_event_vsref,"Statistics");  
		 
	VSfdefine(otd_event_vsref,"event #",DFNT_INT32,1);
	VSfdefine(otd_event_vsref,"seq. #",DFNT_INT32,1);
	VSfdefine(otd_event_vsref,"TAI93",DFNT_FLOAT64,1);
	VSfdefine(otd_event_vsref,"s-z-a",DFNT_INT16,1);
	VSfdefine(otd_event_vsref,"d-n-t",DFNT_CHAR8,1);
	VSfdefine(otd_event_vsref,"x pixel",DFNT_INT16,1);
	VSfdefine(otd_event_vsref,"y pixel",DFNT_INT16,1);
	VSfdefine(otd_event_vsref,"raw radiance",DFNT_INT16,1);
	VSfdefine(otd_event_vsref,"cal radiance",DFNT_FLOAT32,1);

        VSfdefine(otd_event_vsref,"parent seq",DFNT_INT32,1);	/* parent */
        VSfdefine(otd_event_vsref,"parent rec",DFNT_INT32,1);	/* parent */
        
	VSfdefine(otd_event_vsref,"QA",DFNT_INT16,4);
	VSfdefine(otd_event_vsref,"location",DFNT_FLOAT32,2);

	if(VSsetfields(otd_event_vsref,
		"event #,seq. #,TAI93,s-z-a,d-n-t,x pixel,y pixel,raw radiance,cal radiance,parent seq,parent rec,QA,location")==FAIL){
#ifdef DEBUG
printf("event_vsref VSetfields failed\n");
#endif
		return (FAIL);   
	}

	if(Vinsert(otd_event_vgroup_id,otd_event_vsref)==FAIL){
#ifdef DEBUG
printf("event_vgroup_id Vinsert failed\n");
#endif
		return (FAIL);  
	}

/*********************************** browse *********************************************/
	
	if ((otd_browse_vsref = VSattach(otd_hdf_id,-1,"w"))==FAIL){
#ifdef DEBUG
printf("browse_vsref VSattach failed\n");
#endif
		return (FAIL);  
	} 
			
	VSsetname(otd_browse_vsref,"browse");
	VSsetclass(otd_browse_vsref,"area");  
		 
	VSfdefine(otd_browse_vsref,"location",DFNT_FLOAT32,2);
	VSfdefine(otd_browse_vsref,"EGFAs",DFNT_INT16,4);
	VSfdefine(otd_browse_vsref,"EGFA cal rad",DFNT_FLOAT32,4);
	VSfdefine(otd_browse_vsref,"max rate",DFNT_FLOAT32,1);
	VSfdefine(otd_browse_vsref,"1 event areas",DFNT_INT16,1);

	VSfdefine(otd_browse_vsref,"highres locs",DFNT_INT16,1);
	VSfdefine(otd_browse_vsref,"1st highres",DFNT_INT32,1);
	VSfdefine(otd_browse_vsref,"bkgnds",DFNT_INT32,1);
	VSfdefine(otd_browse_vsref,"iloper",DFNT_INT16,1);
	VSfdefine(otd_browse_vsref,"bkgnd pointer",DFNT_INT32,1);
	VSfdefine(otd_browse_vsref,"duration",DFNT_INT16,1);

	if(VSsetfields(otd_browse_vsref,
		"location,EGFAs,EGFA cal rad,max rate,1 event areas,highres locs,1st highres,bkgnds,iloper,bkgnd pointer,duration")==FAIL){
#ifdef DEBUG
printf("browse_vsref VSetfields failed\n");
#endif
		return (FAIL); 
	}

	if(Vinsert(otd_browse_vgroup_id,otd_browse_vsref)==FAIL){
#ifdef DEBUG
printf("browse_vsref Vinsert failed\n");
#endif
		return (FAIL); 
	}  
 
/********************************** vector products *************************************/

	if ((otd_vector_vsref = VSattach(otd_hdf_id,-1,"w"))==FAIL){
#ifdef DEBUG
printf("otd_vector_vsref VSattach failed\n");
#endif
		return (FAIL);
	}   
			
	VSsetname(otd_vector_vsref,"vector");
	VSsetclass(otd_vector_vsref,"statistics");  
		 
			
	VSfdefine(otd_vector_vsref,"location",DFNT_FLOAT32,2);
	VSfdefine(otd_vector_vsref,"EGFA counts",DFNT_INT16,4);
	VSfdefine(otd_vector_vsref,"EGFA cal radiances",DFNT_FLOAT32,4);
	VSfdefine(otd_vector_vsref,"max flash rate",DFNT_FLOAT32,1);
	VSfdefine(otd_vector_vsref,"single event areas",DFNT_INT16,1);

	if(VSsetfields(otd_vector_vsref,
		"location,EGFA counts,EGFA cal radiances,max flash rate,single event areas")==FAIL){
#ifdef DEBUG
printf("otd_vector_vsref VSetfields failed\n");
#endif
		return (FAIL); 
	}  
	if(Vinsert(otd_browse_vgroup_id,otd_vector_vsref)==FAIL){
#ifdef DEBUG
printf("otd_vector_vsref Vinsert failed\n");
#endif
		return (FAIL); 
	}  
 
/********************************* image attribute ***************************************/

	if ((otd_image_vsref = VSattach(otd_hdf_id,-1,"w"))==FAIL){
#ifdef DEBUG
printf("otd_image_vsref VSattach failed\n");
#endif
		return (FAIL); 
	} 
			
	VSsetname(otd_image_vsref,"image");
	VSsetclass(otd_image_vsref,"attributes");  
		 	
	VSfdefine(otd_image_vsref,"TAI93",DFNT_FLOAT64,1);
	VSfdefine(otd_image_vsref,"NW location",DFNT_FLOAT32,2);
	VSfdefine(otd_image_vsref,"NE location",DFNT_FLOAT32,2);
	VSfdefine(otd_image_vsref,"SW location",DFNT_FLOAT32,2);
	VSfdefine(otd_image_vsref,"SE location",DFNT_FLOAT32,2);
	VSfdefine(otd_image_vsref,"nadir location",DFNT_FLOAT32,2);
	VSfdefine(otd_image_vsref,"image pointer",DFNT_INT32,1);
	VSfdefine(otd_image_vsref,"calibration number",DFNT_INT32,1);
	VSfdefine(otd_image_vsref,"spice pointer",DFNT_INT32,1);

	if(VSsetfields(otd_image_vsref,
		"TAI93,NW location,NE location, SW location,SE location,nadir location,image pointer,calibration number,spice pointer")==FAIL){
#ifdef DEBUG
printf("image_vsref VSetfields failed\n");
#endif
		return (FAIL);
	}   

	if(Vinsert(otd_browse_vgroup_id,otd_image_vsref)==FAIL){
#ifdef DEBUG
printf("image_vsref Vinsert failed\n");
#endif

		return (FAIL);
	}   

/************************* orbit Summary 1/13/95 *********************************/

	otd_orbit_summary_vsref=VSattach(otd_hdf_id,-1,"w");
	if(otd_orbit_summary_vsref==FAIL){
#ifdef DEBUG
printf("orbit_summary_vsref VSattach failed\n");
#endif
		 return(FAIL);
	}

	VSsetname(otd_orbit_summary_vsref,"summary");
	VSsetclass(otd_orbit_summary_vsref,"data");

	VSfdefine(otd_orbit_summary_vsref,"areas",DFNT_INT32,1);
	VSfdefine(otd_orbit_summary_vsref,"flashes",DFNT_INT32,1);
	VSfdefine(otd_orbit_summary_vsref,"groups",DFNT_INT32,1);
	VSfdefine(otd_orbit_summary_vsref,"events",DFNT_INT32,1);
	VSfdefine(otd_orbit_summary_vsref,"backgrounds",DFNT_INT32,1);
	VSfdefine(otd_orbit_summary_vsref,"total cal radiance",DFNT_FLOAT32,1);
  
	if(VSsetfields(otd_orbit_summary_vsref,"areas,flashes,groups,events,backgrounds,total cal radiance")==FAIL){
#ifdef DEBUG
printf("otd_orbit_summary_vsref VSetfields failed\n");
#endif
		return (FAIL);
	}   

     if(Vinsert(otd_orbit_summary_vgroup_id,otd_orbit_summary_vsref)==FAIL){
#ifdef DEBUG
printf("otd_orbit_summary_vsref Vinsert failed\n");
#endif
		return (FAIL);
	}   

/************************* orbit area flash rate  1/13/95 *********************************/

	otd_orbit_flash_rate_vsref=VSattach(otd_hdf_id,-1,"w");
	if(otd_orbit_flash_rate_vsref==FAIL){
#ifdef DEBUG
printf("orbit_flash_rate_vsref VSattach failed\n");
#endif
		 return(FAIL);
	}

	VSsetname(otd_orbit_flash_rate_vsref,"rate_f");
	VSsetclass(otd_orbit_flash_rate_vsref,"data");

	VSfdefine(otd_orbit_flash_rate_vsref,"flash rate",DFNT_FLOAT32,1);
	
	if(VSsetfields(otd_orbit_flash_rate_vsref,"flash rate")==FAIL){
#ifdef DEBUG
printf("orbit_flash_rate_vsref VSetfields failed\n");
#endif
		return (FAIL);
	}   

	if(Vinsert(otd_orbit_summary_vgroup_id,otd_orbit_flash_rate_vsref)==FAIL){
#ifdef DEBUG
printf("orbit_flash_rate_vsref Vinsert failed\n");
#endif
		return (FAIL); 
	}  

/************************* orbit event rate  05/20/96 *********************************/

	otd_orbit_event_rate_vsref=VSattach(otd_hdf_id,-1,"w");
	if(otd_orbit_event_rate_vsref==FAIL){
#ifdef DEBUG
printf("orbit_event_rate_vsref VSattach failed\n");
#endif
		 return(FAIL);
	}

	VSsetname(otd_orbit_event_rate_vsref,"rate_e");
	VSsetclass(otd_orbit_event_rate_vsref,"data");

	VSfdefine(otd_orbit_event_rate_vsref,"count",DFNT_INT16,6100);
	
	if(VSsetfields(otd_orbit_event_rate_vsref,"count")==FAIL){
#ifdef DEBUG
printf("orbit_event_rate_vsref VSetfields failed\n");
#endif
		return (FAIL);   
	}

	if(Vinsert(otd_orbit_summary_vgroup_id,otd_orbit_event_rate_vsref)==FAIL){
#ifdef DEBUG
printf("orbit_event_rate_vsref Vinsert failed\n");
#endif
		return (FAIL); 
	}  

/***************************** Ephemeris data 6/8/95 ***********************************/

	otd_ephemeris_vsref = VSattach(otd_hdf_id,-1,"w");
	if(otd_ephemeris_vsref==FAIL){
#ifdef DEBUG
printf("epmemeris_vsref VSattach failed\n");
#endif
		return (FAIL);
	}
	
	VSsetname(otd_ephemeris_vsref,"ephemeris");
	VSsetclass(otd_ephemeris_vsref,"data");

	VSfdefine(otd_ephemeris_vsref,"GPS time",DFNT_FLOAT64,1);
	VSfdefine(otd_ephemeris_vsref,"spice kernel",DFNT_FLOAT64,15);

	if(VSsetfields(otd_ephemeris_vsref,"GPS time,spice kernel")==FAIL){
#ifdef DEBUG
printf("ephemeris_vsref VSetfields failed\n");
#endif
		return (FAIL); 
	}  

	if(Vinsert(otd_orbit_summary_vgroup_id,otd_ephemeris_vsref)==FAIL){
#ifdef DEBUG
printf("ephemeris_vsref Vinsert failed\n");
#endif
		return (FAIL); 
	}  

/************************* 2.5 deg x 2.5 deg grid 2/13/95 *********************************/

	otd_grid_25_vsref=VSattach(otd_hdf_id,-1,"w");
	if(otd_grid_25_vsref==FAIL){
#ifdef DEBUG
printf("grid_25_vsref VSattach failed\n");
#endif
 	return(FAIL);
	}

	VSsetname(otd_grid_25_vsref,"2.5 degree");
	VSsetclass(otd_grid_25_vsref,"grid");

	VSfdefine(otd_grid_25_vsref,"location",DFNT_FLOAT32,2);
	VSfdefine(otd_grid_25_vsref,"flash count",DFNT_INT32,1);
	
	if(VSsetfields(otd_grid_25_vsref,"location,flash count")==FAIL){
#ifdef DEBUG
printf("grid_25_vsref VSetfields failed\n");
#endif
		return (FAIL);
	}   

	if(Vinsert(otd_flash_density_vgroup_id,otd_grid_25_vsref)==FAIL){
#ifdef DEBUG
printf("grid_25_vsref Vinsert failed\n");
#endif
		return (FAIL);
	}   

/************************* view time .5 deg grid 10/4/95 *********************************/

	otd_view_vsref=VSattach(otd_hdf_id,-1,"w");
	if(otd_view_vsref==FAIL){
#ifdef DEBUG
printf("view_vsref VSattach failed\n");
#endif
		 return(FAIL);
	}

	VSsetname(otd_view_vsref,"view time");
	VSsetclass(otd_view_vsref,"grid");

	VSfdefine(otd_view_vsref,"grid point",DFNT_FLOAT32,2);
	VSfdefine(otd_view_vsref,"TAI93 start",DFNT_FLOAT64,1);
	VSfdefine(otd_view_vsref,"duration",DFNT_INT16,1);
	
	if(VSsetfields(otd_view_vsref,"grid point,TAI93 start,duration")==FAIL){
#ifdef DEBUG
printf("view_vsref VSetfields failed\n");
#endif
		return (FAIL);
	}   

	if(Vinsert(otd_flash_density_vgroup_id,otd_view_vsref)==FAIL){
#ifdef DEBUG
printf("view_vsref Vinsert failed\n");
#endif
		return (FAIL); 
	}  

/************************* 50 deg x 50 deg grid 2/13/95 *********************************/

	otd_grid_500_km_vsref=VSattach(otd_hdf_id,-1,"w");
	if(otd_grid_500_km_vsref==FAIL){
#ifdef DEBUG
printf("grid_500_km_vsref VSattach failed\n");
#endif
 		return(FAIL);
	}

	VSsetname(otd_grid_500_km_vsref,"500km");
	VSsetclass(otd_grid_500_km_vsref,"grid");

	VSfdefine(otd_grid_500_km_vsref,"location",DFNT_FLOAT32,2);
	VSfdefine(otd_grid_500_km_vsref,"flash count",DFNT_INT32,1);
	
	if(VSsetfields(otd_grid_500_km_vsref,"location,flash count")==FAIL){
#ifdef DEBUG
printf("grid_500_km_vsref VSetfields failed\n");
#endif
		return (FAIL);
	}   

	if(Vinsert(otd_flash_density_vgroup_id,otd_grid_500_km_vsref)==FAIL){
#ifdef DEBUG
printf("grid_500_km_vsref Vinsert failed\n");
#endif
		return (FAIL); 
	}  

	return (otd_hdf_id);

}


long close_OTD_HDF_FILE()

/****************************************************************************************/
/*                                                                             		*/
/*	close_hdf_file									*/
/*											*/
/*	returns 0 if all vgroups, vsets and the file itself are successfully    	*/
/*	closed										*/
/*	returns FAIL(-1) on error							*/
/*											*/
/*	Date last modified 9/7/94							*/
/****************************************************************************************/

{
	long return_status;
	return_status = 0;

        if(otd_orbit_vsref!= FAIL)VSdetach(otd_orbit_vsref);			/* orbit */
 
        if(otd_metadata_vsref!= FAIL)VSdetach(otd_metadata_vsref);		/* metadata */

        if(otd_threshold_vsref!= FAIL)VSdetach(otd_threshold_vsref);		/* threshold */
 
        if(otd_event_vsref!= FAIL)VSdetach(otd_event_vsref);			/* Events */
	if(otd_event_vgroup_id!= FAIL)Vdetach(otd_event_vgroup_id);

	if(otd_group_vsref!= FAIL)VSdetach(otd_group_vsref);			/* Group */
	if(otd_group_vgroup_id!= FAIL)Vdetach(otd_group_vgroup_id);

	if(otd_flash_vsref!= FAIL)VSdetach(otd_flash_vsref);			/* Flash */
	if(otd_flash_vgroup_id!= FAIL)Vdetach(otd_flash_vgroup_id);

	if(otd_area_vsref!= FAIL)VSdetach(otd_area_vsref);			/* Area */
	if(otd_area_vgroup_id!= FAIL)Vdetach(otd_area_vgroup_id);
	

	if(otd_vector_vsref!= FAIL)VSdetach(otd_vector_vsref);			/* Browse */
	if(otd_image_vsref!= FAIL)VSdetach(otd_image_vsref);
	if(otd_browse_vsref!= FAIL)VSdetach(otd_browse_vsref);
	if(otd_browse_vgroup_id!= FAIL)Vdetach(otd_browse_vgroup_id);

	if(otd_orbit_flash_rate_vsref!= FAIL)VSdetach(otd_orbit_flash_rate_vsref);	/* Orbit Summary */
	if(otd_orbit_event_rate_vsref!= FAIL)VSdetach(otd_orbit_event_rate_vsref);	
	if(otd_ephemeris_vsref!= FAIL)VSdetach(otd_ephemeris_vsref);
	if(otd_orbit_summary_vsref!= FAIL)VSdetach(otd_orbit_summary_vsref);
	if(otd_orbit_summary_vgroup_id!= FAIL)Vdetach(otd_orbit_summary_vgroup_id);

	if(otd_view_vsref!= FAIL)VSdetach(otd_view_vsref);					/* Flash Desnity */
	if(otd_grid_25_vsref!= FAIL)VSdetach(otd_grid_25_vsref);				/* Flash Desnity */
	if(otd_grid_500_km_vsref!= FAIL)VSdetach(otd_grid_500_km_vsref);			
	if(otd_flash_density_vgroup_id!= FAIL)Vdetach(otd_flash_density_vgroup_id);

	if(SDsetattr(otd_image_sds_id,"total images",DFNT_INT32,1,&otd_image_srecord)==FAIL)
	return_status = FAIL;

	if(otd_image_sds_id!=FAIL)if(SDendaccess(otd_image_sds_id)==FAIL)return_status = FAIL;	/* image */
		 

 	if(otd_sd_id!=FAIL)if (SDend(otd_sd_id)==FAIL)return_status = FAIL;			/* SD id */

	if(otd_hdf_id!=FAIL)Vend(otd_hdf_id);

	if(otd_hdf_id !=FAIL)if(Hclose(otd_hdf_id)==FAIL)return_status = FAIL;	

	return (return_status);		/* success*/

}


long create_otd_VGROUP(long otd_hdf_id,char *name,char *class)

/****************************************************************************************/
/*											*/
/*	create a vgroup in the HDF file							*/
/*											*/
/*	returns the reference number of the Vgroup if successful			*/
/*	returns FAIL (-1) on error							*/
/*											*/
/*	Date last modified	8/15/94							*/
/****************************************************************************************/

{
	long vgroup;
	vgroup= (long)Vattach(otd_hdf_id,-1,"w");
	if(vgroup == FAIL)return FAIL;	
	Vsetname(vgroup,name);
	Vsetclass(vgroup,class);
	return (vgroup);

}


 
long write_otd_IMAGE(short int *databuffer)

/****************************************************************************************/
/*											*/
/*	Write sds image data to HDF file						*/
/*											*/
/*	returns sds record # if successful						*/
/*	returns FAIL (-1) on error							*/
/*											*/
/*	Date last modified	8/15/94							*/
/****************************************************************************************/


{
	start[0]=otd_image_srecord++;
	start[1]=start[2]=0;

	endr[0]=1;
	endr[1]=endr[2]=128;

	if(SDwritedata(otd_image_sds_id,start,NULL,endr,databuffer)==FAIL){
		otd_image_srecord--;
		return (FAIL);}
	return (otd_image_srecord-1);
}




long write_otd_EVENT(void *databuffer)		{return (write_otd_VSGROUP(databuffer,otd_event_vsref,&otd_event_vrecord));}
long write_otd_RATE(void *databuffer)		{return (write_otd_VSGROUP(databuffer,otd_orbit_event_rate_vsref,&otd_orbit_event_rate_vrecord));}
long write_otd_GROUP(void *databuffer) 		{return (write_otd_VSGROUP(databuffer,otd_group_vsref,&otd_group_vrecord));}
long write_otd_FLASH(void *databuffer) 		{return (write_otd_VSGROUP(databuffer,otd_flash_vsref,&otd_flash_vrecord));}
long write_otd_AREA(void *databuffer)	 	{return (write_otd_VSGROUP(databuffer,otd_area_vsref,&otd_area_vrecord));}
long write_otd_VECTOR_PRODUCTS(void *databuffer)	{return (write_otd_VSGROUP(databuffer,otd_vector_vsref,&otd_vector_vrecord));}
long write_otd_IMAGE_ATTRIBUTE(void *databuffer)	{return (write_otd_VSGROUP(databuffer,otd_image_vsref,&otd_image_vrecord));}
long write_otd_ORBIT_DATA(void *databuffer)	{return (write_otd_VSGROUP(databuffer,otd_orbit_vsref,&otd_orbit_vrecord));}
long write_otd_METADATA(void *databuffer)		{return (write_otd_VSGROUP(databuffer,otd_metadata_vsref,&otd_metadata_vrecord));}
long write_otd_BROWSE_DATA(void *databuffer)	{return (write_otd_VSGROUP(databuffer,otd_browse_vsref,&otd_browse_data_vrecord));}
long write_otd_VIEW(void *databuffer)		{return (write_otd_VSGROUP(databuffer,otd_view_vsref,&otd_view_vrecord));}
long write_otd_GRID_25(void *databuffer)		{return (write_otd_VSGROUP(databuffer,otd_grid_25_vsref,&otd_grid_25_vrecord));}
long write_otd_GRID_500_KM(void *databuffer)	{return (write_otd_VSGROUP(databuffer,otd_grid_500_km_vsref,&otd_grid_500_km_vrecord));}

long write_otd_ORBIT_SUMMARY(void *databuffer)	{return (write_otd_VSGROUP(databuffer,otd_orbit_summary_vsref,&otd_orbit_summary_vrecord));}
long write_otd_AREA_FLASH_RATE(void *databuffer)	{return (write_otd_VSGROUP(databuffer,otd_orbit_flash_rate_vsref,&otd_orbit_flash_rate_vrecord));}
long write_otd_EPHEMERIS(void *databuffer)		{return (write_otd_VSGROUP(databuffer,otd_ephemeris_vsref,&otd_ephemeris_vrecord));}
long write_otd_THRESHOLD(void *databuffer)		{return (write_otd_VSGROUP(databuffer,otd_threshold_vsref,&otd_threshold_vrecord));}


long write_otd_VSGROUP(void *databuffer,long vsref,long *vsrecord)

/****************************************************************************************/
/*											*/
/*	Write data to HDF file								*/
/*											*/
/*	returns Vdata record # if successful						*/
/*	returns FAIL (-1) on error							*/
/*											*/
/*	Date last modified	8/15/94							*/
/****************************************************************************************/


{
	(*vsrecord)++;
 	if(VSwrite(vsref,databuffer,1,FULL_INTERLACE)==FAIL){
		(*vsrecord)--;
		return (FAIL);
	}
	return (*vsrecord-1);
}


/********************************************************************************************************/
/*													*/
/*	Compatible with HDF version 4.1r1									*/
/*													*/
/*													*/
/*													*/
/*	LIS/OTD HDF database access library								*/
/*													*/
/*	FUNCTION NAME			TYPE	DESCRIPTION						*/
/*	------------------------	----	------------------------------------------------------- */
/*	create_LIS_ORBIT_HDF		long	Create and initialize LIS/OTD orbit archive file	*/
/*	create_LIS_BACKGROUND_HDF	long	Create and initialize LIS/OTD background archive file	*/
/*	close_LIS_ORBIT_HDF		long	Gracefully detach and close orbit archive file		*/
/*	close_LIS_BACKGROUND_HDF	long	Gracefully detach and close background archive file	*/
/*													*/
/*	write_lis_AREA			long	Write areadata to archive file				*/
/*	write_lis_FLASH			long	Write flash data to archive file			*/
/*	write_lis_GROUP			long	Write group data to archive file			*/
/*	write_lis_EVENT			long	Write the event data to archive file			*/
/*	write_lis_POINT_SUMMARY		long	Write point_summary data to archive file		*/
/*	write_lis_VIEWTIME			long	Write the viewtime data to archive file			*/
/*	write_lis_BG_SUMMARY		long	Write the bg_summary data to archive file		*/
/*	write_lis_ONE_SECOND		long	Write the one_second data to archive file		*/
/*	write_lis_ORBIT_SUMMARY		long	Write the orbit_summary data to archive file		*/
/*													*/
/*													*/
/*	write_lis_BG_DATA_SUMMARY		long	Write the bg_data_summary data to archive file		*/
/*	write_lis_BG_INFO			long	Write the bg_info data to archive file			*/
/*	write_lis_BG_ORBIT_SUMMARY		long	Write the bg_orbit_summary data to archive file		*/
/*	write_lis_BACKGROUND_DATA		long	Write background image (SDS) to archive file		*/
/*													*/
/*													*/
/*	write_lis_VSGROUP			long	generic VSwrite routine	(hiddden from user)		*/
/*	write_lis_SDS			long    generic SDwritedata routine (hidden from user)		*/
/*													*/
/*													*/
/*	create_lis_VGROUP			long	create a vgroup in the HDF file	(hiddden from user)	*/
/*													*/
/*													*/
/*	*********************************************************					*/
/*	* NOTE!!					      	*					*/
/*	*	All functions return a negative number on error	*					*/
/*	*	see LIBLIS_err_defs.h for details		*					*/
/*	*********************************************************					*/
/*													*/
/********************************************************************************************************/


/****************************************************************************************/
/**************************************		*****************************************/
/************************************** HISTORY	*****************************************/
/**************************************		*****************************************/
/****************************************************************************************/
/*											*/
/*	Initial library development 9/29/1997	John Hall/CSC (jmh)			*/
/*											*/
/*											*/
/* Change History:									*/
/*											*/
/*											*/
/* Date		Module			Change description			initail */
/* --------	-------------------	-----------------------------------	-------	*/
/*											*/
/*											*/
/****************************************************************************************/

long create_lis_VGROUP(long lis_hdf_id,char *name,char *class);
long write_lis_VSGROUP(long file_number,void *databuffer,long vsref[],long vsrecord[]);
long write_lis_SDS(short int *databuffer,long sds_id);


long create_LIS_ORBIT_HDF(char *hdf_fname)
{
	long hid_tmp,i,file_number;
 

/********************************* initialize H interface *******************************/
	
	hid_tmp = (Hopen(hdf_fname,DFACC_CREATE,256)); 
	if(hid_tmp==FAIL)return hid_tmp;
	file_number = -1;
	for(i=0;i<MAX_FILE;i++){
		if(lis_hdf_id[i]==-1){
			file_number = i;
			break;
		}
	}
	if(file_number==-1){
		return(lis_operation_FAIL|lis_MAX_FILE_FAIL);
	}
	if(Vstart(hid_tmp)==FAIL) return lis_operation_FAIL|lis_Vstart_FAIL;
	lis_hdf_id[file_number] = hid_tmp;


/********************** initailize internal data structures/pointers ********************/

	lis_area_vrecord[file_number] = 0;
	lis_flash_vrecord[file_number] = 0;
	lis_group_vrecord[file_number] = 0;
	lis_event_vrecord[file_number] = 0;
	lis_viewtime_vrecord[file_number] = 0;
	lis_bg_summary_vrecord[file_number] = 0;
	lis_point_summary_vrecord[file_number] = 0;
	lis_orbit_summary_vrecord[file_number] = 0;
	lis_one_second_vrecord[file_number] = 0;

/****************************** create vgroups ******************************************/
	lis_orbit_data_vgroup_id[file_number] = create_lis_VGROUP(lis_hdf_id[file_number],"orbit","data");
	if(lis_orbit_data_vgroup_id[file_number]==FAIL){
		printf("create_lis_VGROUP orbit_data_vgroup_id failed\n");
		return lis_operation_FAIL|lis_Vgroup_Create_FAIL;
	}
	lis_point_vgroup_id[file_number] = create_lis_VGROUP(lis_hdf_id[file_number],"point","data");
	if(lis_point_vgroup_id[file_number]==FAIL){
		printf("create_lis_VGROUP point_vgroup_id failed\n");
		return lis_operation_FAIL|lis_Vgroup_Create_FAIL;
	}
	lis_lightning_vgroup_id[file_number] = create_lis_VGROUP(lis_hdf_id[file_number],"lightning","data");
	if(lis_lightning_vgroup_id[file_number]==FAIL){
		printf("create_lis_VGROUP point_vgroup_id failed\n");
		return lis_operation_FAIL|lis_Vgroup_Create_FAIL;
	}

	if(Vinsert(lis_orbit_data_vgroup_id[file_number],lis_point_vgroup_id[file_number])==FAIL){
		printf("point_vgroup_id Vinsert failed\n");

		return (lis_operation_FAIL|lis_Vinsert_FAIL); 
	}  
	if(Vinsert(lis_point_vgroup_id[file_number],lis_lightning_vgroup_id[file_number])==FAIL){
		printf("lightning_vgroup_id Vinsert failed\n");

		return (lis_operation_FAIL|lis_Vinsert_FAIL); 
	}  


/*************************** create scientific data sets (SDS) **************************/
/*
	sd_id = SDstart(hdf_fname,DFACC_RDWR);
	dimsize[0]=0;
	dimsize[1]=128;
	dimsize[2]=128;
	image_sds_id = SDcreate(sd_id,"bg_data",DFNT_INT16,3,dimsize); 
 
	dimsize[0]=128;
	dimsize[1]=128;
*/

/******************************** orbit summary **************************************************/

	if ((lis_orbit_summary_vsref[file_number] = VSattach(lis_hdf_id[file_number],-1,"w"))==FAIL){
		printf("orbit_summary_vsref VSattach failed\n");
		return (lis_operation_FAIL|lis_orbit_summary_VSattach_FAIL);   
	}
			
	VSsetname(lis_orbit_summary_vsref[file_number],"orbit_summary");
	VSsetclass(lis_orbit_summary_vsref[file_number],"data");  
		 
	VSfdefine(lis_orbit_summary_vsref[file_number],"id_number",DFNT_INT32,1);
	VSfdefine(lis_orbit_summary_vsref[file_number],"TAI93_start",DFNT_FLOAT64,1);
	VSfdefine(lis_orbit_summary_vsref[file_number],"UTC_start",DFNT_CHAR,28);
	VSfdefine(lis_orbit_summary_vsref[file_number],"GPS_start",DFNT_FLOAT64,1);
	VSfdefine(lis_orbit_summary_vsref[file_number],"TAI93_end",DFNT_FLOAT64,1);
	VSfdefine(lis_orbit_summary_vsref[file_number],"start_longitude",DFNT_FLOAT32,1);
	VSfdefine(lis_orbit_summary_vsref[file_number],"end_longitude",DFNT_FLOAT32,1);
	VSfdefine(lis_orbit_summary_vsref[file_number],"point_data_count",DFNT_INT16,1);
	VSfdefine(lis_orbit_summary_vsref[file_number],"point_data_address",DFNT_INT16,1);
	VSfdefine(lis_orbit_summary_vsref[file_number],"one_second_count",DFNT_INT32,1);
	VSfdefine(lis_orbit_summary_vsref[file_number],"one_second_address",DFNT_INT32,1);
	VSfdefine(lis_orbit_summary_vsref[file_number],"summary_image_count",DFNT_INT16,1);
	VSfdefine(lis_orbit_summary_vsref[file_number],"summary_image_address",DFNT_INT16,1);
	VSfdefine(lis_orbit_summary_vsref[file_number],"inspection_code",DFNT_UINT16,1);
	VSfdefine(lis_orbit_summary_vsref[file_number],"configuration_code",DFNT_UINT16,1);


	if(VSsetfields(lis_orbit_summary_vsref[file_number],
		"id_number,TAI93_start,UTC_start,GPS_start,TAI93_end,start_longitude,end_longitude,point_data_count,point_data_address,one_second_count,one_second_address,summary_image_count,summary_image_address,inspection_code,configuration_code")==FAIL){
		printf("orbit_summary_vsref VSetfields failed\n");
		return (lis_operation_FAIL|lis_VSsetfields_FAIL);   
	}


	if(Vinsert(lis_orbit_data_vgroup_id[file_number],lis_orbit_summary_vsref[file_number])==FAIL){
		printf("orbit_summary_vgroup_id Vinsert failed\n");
		return (lis_operation_FAIL| lis_Vinsert_FAIL); 
	}  

/******************************** one_second **************************************************/
	if ((lis_one_second_vsref[file_number] = VSattach(lis_hdf_id[file_number],-1,"w"))==FAIL){
		printf("one_second_vsref VSattach failed\n");
		return (lis_operation_FAIL|lis_one_second_VSattach_FAIL);   
	}
			
	VSsetname(lis_one_second_vsref[file_number],"one_second");
	VSsetclass(lis_one_second_vsref[file_number],"data");  
		 
	VSfdefine(lis_one_second_vsref[file_number],"TAI93_time",DFNT_FLOAT64,1);
	VSfdefine(lis_one_second_vsref[file_number],"alert_summary",DFNT_UINT8,1);
	VSfdefine(lis_one_second_vsref[file_number],"instrument_alert",DFNT_UINT8,1);
	VSfdefine(lis_one_second_vsref[file_number],"platform_alert",DFNT_UINT8,1);
	VSfdefine(lis_one_second_vsref[file_number],"external_alert",DFNT_UINT8,1);
	VSfdefine(lis_one_second_vsref[file_number],"processing_alert",DFNT_UINT8,1);
	VSfdefine(lis_one_second_vsref[file_number],"position_vector",DFNT_FLOAT32,3);
	VSfdefine(lis_one_second_vsref[file_number],"velocity_vector",DFNT_FLOAT32,3);
	VSfdefine(lis_one_second_vsref[file_number],"transform_matrix",DFNT_FLOAT32,9);
	VSfdefine(lis_one_second_vsref[file_number],"solar_vector",DFNT_FLOAT32,3);
	VSfdefine(lis_one_second_vsref[file_number],"ephemeris_quality_flag",DFNT_INT32,1);
	VSfdefine(lis_one_second_vsref[file_number],"attitude_quality_flag",DFNT_INT32,1);
	VSfdefine(lis_one_second_vsref[file_number],"boresight_threshold",DFNT_INT8,1);
	VSfdefine(lis_one_second_vsref[file_number],"thresholds",DFNT_INT8,16);
	VSfdefine(lis_one_second_vsref[file_number],"noise_index",DFNT_INT8,1);
	VSfdefine(lis_one_second_vsref[file_number],"event_count",DFNT_INT16,6);


	if(VSsetfields(lis_one_second_vsref[file_number],
		"TAI93_time,alert_summary,instrument_alert,platform_alert,external_alert,processing_alert,position_vector,velocity_vector,transform_matrix,solar_vector,ephemeris_quality_flag,attitude_quality_flag,boresight_threshold,thresholds,noise_index,event_count")==FAIL){
		printf("lis_one_second_vsref VSetfields failed\n");
		return (lis_operation_FAIL|lis_VSsetfields_FAIL);   
	}


	if(Vinsert(lis_orbit_data_vgroup_id[file_number],lis_one_second_vsref[file_number])==FAIL){
		printf("one_second_vgroup_id Vinsert failed\n");
		return (lis_operation_FAIL| lis_Vinsert_FAIL); 
	}  


/******************************** point summary **************************************************/

	if ((lis_point_summary_vsref[file_number] = VSattach(lis_hdf_id[file_number],-1,"w"))==FAIL){
		printf("lis_point_summary_vsref VSattach failed\n");
		return (lis_operation_FAIL|lis_operation_FAIL| lis_Vinsert_FAIL);   
	}
			
	VSsetname(lis_point_summary_vsref[file_number],"point_summary");
	VSsetclass(lis_point_summary_vsref[file_number],"data");  
		 
	VSfdefine(lis_point_summary_vsref[file_number],"parent_address",DFNT_INT32,1);
	VSfdefine(lis_point_summary_vsref[file_number],"event_count",DFNT_INT32,1);
	VSfdefine(lis_point_summary_vsref[file_number],"event_address",DFNT_INT32,1);
	VSfdefine(lis_point_summary_vsref[file_number],"group_count",DFNT_INT32,1);
	VSfdefine(lis_point_summary_vsref[file_number],"group_address",DFNT_INT32,1);
	VSfdefine(lis_point_summary_vsref[file_number],"flash_count",DFNT_INT32,1);
	VSfdefine(lis_point_summary_vsref[file_number],"flash_address",DFNT_INT32,1);
	VSfdefine(lis_point_summary_vsref[file_number],"area_count",DFNT_INT32,1);
	VSfdefine(lis_point_summary_vsref[file_number],"area_address",DFNT_INT32,1);
	VSfdefine(lis_point_summary_vsref[file_number],"bg_count",DFNT_INT32,1);
	VSfdefine(lis_point_summary_vsref[file_number],"bg_address",DFNT_INT32,1);
	VSfdefine(lis_point_summary_vsref[file_number],"vt_count",DFNT_INT32,1);
	VSfdefine(lis_point_summary_vsref[file_number],"vt_address",DFNT_INT32,1);


	if(VSsetfields(lis_point_summary_vsref[file_number],
		"parent_address,event_count,event_address,group_count,group_address,flash_count,flash_address,area_count,area_address,bg_count,bg_address,vt_count,vt_address")==FAIL){
		printf("lis_point_summary_vsref VSetfields failed\n");
		return (lis_operation_FAIL|lis_VSsetfields_FAIL);   
	}


	if(Vinsert(lis_point_vgroup_id[file_number],lis_point_summary_vsref[file_number])==FAIL){
		printf("lis_point_summary_vsref Vinsert failed\n");
		return (lis_operation_FAIL| lis_Vinsert_FAIL); 
	}  



/******************************** area **************************************************/

	if ((lis_area_vsref[file_number] = VSattach(lis_hdf_id[file_number],-1,"w"))==FAIL){
		printf("area_vsref VSattach failed\n");
		return (lis_operation_FAIL|lis_area_VSattach_FAIL);   
	}
			
	VSsetname(lis_area_vsref[file_number],"area");
	VSsetclass(lis_area_vsref[file_number],"data");  
		 
	VSfdefine(lis_area_vsref[file_number],"TAI93_time",DFNT_FLOAT64,1);
	VSfdefine(lis_area_vsref[file_number],"delta_time",DFNT_FLOAT32,1);
	VSfdefine(lis_area_vsref[file_number],"observe_time",DFNT_INT16,1);
	VSfdefine(lis_area_vsref[file_number],"location",DFNT_FLOAT32,2);/*25*/
	VSfdefine(lis_area_vsref[file_number],"net_radiance",DFNT_FLOAT32,1);
	VSfdefine(lis_area_vsref[file_number],"footprint",DFNT_FLOAT32,1);
	VSfdefine(lis_area_vsref[file_number],"address",DFNT_INT32,1);
	VSfdefine(lis_area_vsref[file_number],"parent_address",DFNT_INT32,1);/*55*/
	VSfdefine(lis_area_vsref[file_number],"child_address",DFNT_INT32,1);
	VSfdefine(lis_area_vsref[file_number],"child_count",DFNT_INT32,1);
	VSfdefine(lis_area_vsref[file_number],"grandchild_count",DFNT_INT32,1);
	VSfdefine(lis_area_vsref[file_number],"greatgrandchild_count",DFNT_INT32,1);
	VSfdefine(lis_area_vsref[file_number],"approx_threshold",DFNT_INT8,1);
	VSfdefine(lis_area_vsref[file_number],"alert_flag",DFNT_UINT8,1);
	VSfdefine(lis_area_vsref[file_number],"cluster_index",DFNT_INT8,1);/*100*/
	VSfdefine(lis_area_vsref[file_number],"density_index",DFNT_INT8,1);
	VSfdefine(lis_area_vsref[file_number],"noise_index",DFNT_INT8,1);
	VSfdefine(lis_area_vsref[file_number],"oblong_index",DFNT_FLOAT32,1);/*118*/
	VSfdefine(lis_area_vsref[file_number],"grouping_sequence",DFNT_INT32,1);
	VSfdefine(lis_area_vsref[file_number],"grouping_status",DFNT_INT8,1);

	if(VSsetfields(lis_area_vsref[file_number],
		"TAI93_time,delta_time,observe_time,location,net_radiance,footprint,address,parent_address,child_address,child_count,grandchild_count,greatgrandchild_count,approx_threshold,alert_flag,cluster_index,density_index,noise_index,oblong_index,grouping_sequence,grouping_status")==FAIL){
		printf("area_vsref VSetfields failed\n");
		return (lis_operation_FAIL|lis_VSsetfields_FAIL);   
	}
	if(Vinsert(lis_lightning_vgroup_id[file_number],lis_area_vsref[file_number])==FAIL){
		printf("area_vgroup_id Vinsert failed\n");
		return (lis_operation_FAIL| lis_Vinsert_FAIL); 
	}  
/******************************** flash **************************************************/

	if ((lis_flash_vsref[file_number] = VSattach(lis_hdf_id[file_number],-1,"w"))==FAIL){
		printf("flash_vsref VSattach failed\n");
		return (lis_operation_FAIL|lis_flash_VSattach_FAIL);   
	}
			
	VSsetname(lis_flash_vsref[file_number],"flash");
	VSsetclass(lis_flash_vsref[file_number],"data");  
		 
	VSfdefine(lis_flash_vsref[file_number],"TAI93_time",DFNT_FLOAT64,1);
	VSfdefine(lis_flash_vsref[file_number],"delta_time",DFNT_FLOAT32,1);
	VSfdefine(lis_flash_vsref[file_number],"observe_time",DFNT_INT16,1);
	VSfdefine(lis_flash_vsref[file_number],"location",DFNT_FLOAT32,2);/*25*/
	VSfdefine(lis_flash_vsref[file_number],"radiance",DFNT_FLOAT32,1);
	VSfdefine(lis_flash_vsref[file_number],"footprint",DFNT_FLOAT32,1);
	VSfdefine(lis_flash_vsref[file_number],"address",DFNT_INT32,1);
	VSfdefine(lis_flash_vsref[file_number],"parent_address",DFNT_INT32,1);/*55*/
	VSfdefine(lis_flash_vsref[file_number],"child_address",DFNT_INT32,1);
	VSfdefine(lis_flash_vsref[file_number],"child_count",DFNT_INT32,1);
	VSfdefine(lis_flash_vsref[file_number],"grandchild_count",DFNT_INT32,1);
	VSfdefine(lis_flash_vsref[file_number],"approx_threshold",DFNT_INT8,1);
	VSfdefine(lis_flash_vsref[file_number],"alert_flag",DFNT_UINT8,1);
	VSfdefine(lis_flash_vsref[file_number],"cluster_index",DFNT_INT8,1);/*100*/
	VSfdefine(lis_flash_vsref[file_number],"density_index",DFNT_INT8,1);
	VSfdefine(lis_flash_vsref[file_number],"noise_index",DFNT_INT8,1);
	VSfdefine(lis_flash_vsref[file_number],"glint_index",DFNT_FLOAT32,1);
	VSfdefine(lis_flash_vsref[file_number],"oblong_index",DFNT_FLOAT32,1);/*118*/
	VSfdefine(lis_flash_vsref[file_number],"grouping_sequence",DFNT_INT32,1);
	VSfdefine(lis_flash_vsref[file_number],"grouping_status",DFNT_INT8,1);

	if(VSsetfields(lis_flash_vsref[file_number],
		"TAI93_time,delta_time,observe_time,location,radiance,footprint,address,parent_address,child_address,child_count,grandchild_count,approx_threshold,alert_flag,cluster_index,density_index,noise_index,glint_index,oblong_index,grouping_sequence,grouping_status")==FAIL){
		printf("flash_vsref VSetfields failed\n");
		return (lis_operation_FAIL|lis_VSsetfields_FAIL);   
	}
	if(Vinsert(lis_lightning_vgroup_id[file_number],lis_flash_vsref[file_number])==FAIL){
		printf("flash_vgroup_id Vinsert failed\n");
		return (lis_operation_FAIL| lis_Vinsert_FAIL); 
	}  
/******************************** group **************************************************/

	if ((lis_group_vsref[file_number] = VSattach(lis_hdf_id[file_number],-1,"w"))==FAIL){
		printf("group_vsref VSattach failed\n");
		return (lis_operation_FAIL|lis_group_VSattach_FAIL);   
	}
			
	VSsetname(lis_group_vsref[file_number],"group");
	VSsetclass(lis_group_vsref[file_number],"data");  
		 
	VSfdefine(lis_group_vsref[file_number],"TAI93_time",DFNT_FLOAT64,1);
	VSfdefine(lis_group_vsref[file_number],"observe_time",DFNT_INT16,1);
	VSfdefine(lis_group_vsref[file_number],"location",DFNT_FLOAT32,2);/*25*/
	VSfdefine(lis_group_vsref[file_number],"radiance",DFNT_FLOAT32,1);
	VSfdefine(lis_group_vsref[file_number],"footprint",DFNT_FLOAT32,1);
	VSfdefine(lis_group_vsref[file_number],"address",DFNT_INT32,1);
	VSfdefine(lis_group_vsref[file_number],"parent_address",DFNT_INT32,1);/*55*/
	VSfdefine(lis_group_vsref[file_number],"child_address",DFNT_INT32,1);
	VSfdefine(lis_group_vsref[file_number],"child_count",DFNT_INT32,1);
	VSfdefine(lis_group_vsref[file_number],"approx_threshold",DFNT_INT8,1);
	VSfdefine(lis_group_vsref[file_number],"alert_flag",DFNT_UINT8,1);
	VSfdefine(lis_group_vsref[file_number],"cluster_index",DFNT_INT8,1);/*100*/
	VSfdefine(lis_group_vsref[file_number],"density_index",DFNT_INT8,1);
	VSfdefine(lis_group_vsref[file_number],"noise_index",DFNT_INT8,1);
	VSfdefine(lis_group_vsref[file_number],"glint_index",DFNT_FLOAT32,1);
	VSfdefine(lis_group_vsref[file_number],"oblong_index",DFNT_FLOAT32,1);/*118*/
	VSfdefine(lis_group_vsref[file_number],"grouping_sequence",DFNT_INT32,1);
	VSfdefine(lis_group_vsref[file_number],"grouping_status",DFNT_INT8,1);

	if(VSsetfields(lis_group_vsref[file_number],
		"TAI93_time,observe_time,location,radiance,footprint,address,parent_address,child_address,child_count,approx_threshold,alert_flag,cluster_index,density_index,noise_index,glint_index,oblong_index,grouping_sequence,grouping_status")==FAIL){
		printf("group_vsref VSetfields failed\n");
		return (lis_operation_FAIL|lis_VSsetfields_FAIL);   
	}
	if(Vinsert(lis_lightning_vgroup_id[file_number],lis_group_vsref[file_number])==FAIL){
		printf("group_vgroup_id Vinsert failed\n");
		return (lis_operation_FAIL| lis_Vinsert_FAIL); 
	}  
/******************************** event **************************************************/

	if ((lis_event_vsref[file_number] = VSattach(lis_hdf_id[file_number],-1,"w"))==FAIL){
		printf("event_vsref VSattach failed\n");
		return (lis_operation_FAIL|lis_event_VSattach_FAIL);   
	}
			
	VSsetname(lis_event_vsref[file_number],"event");
	VSsetclass(lis_event_vsref[file_number],"data");  
		 
	VSfdefine(lis_event_vsref[file_number],"TAI93_time",DFNT_FLOAT64,1);
	VSfdefine(lis_event_vsref[file_number],"observe_time",DFNT_INT16,1);
	VSfdefine(lis_event_vsref[file_number],"location",DFNT_FLOAT32,2);/*20*/
	VSfdefine(lis_event_vsref[file_number],"radiance",DFNT_FLOAT32,1);
	VSfdefine(lis_event_vsref[file_number],"footprint",DFNT_FLOAT32,1);
	VSfdefine(lis_event_vsref[file_number],"address",DFNT_INT32,1);
	VSfdefine(lis_event_vsref[file_number],"parent_address",DFNT_INT32,1);/*50*/
	VSfdefine(lis_event_vsref[file_number],"x_pixel",DFNT_INT8,1);
	VSfdefine(lis_event_vsref[file_number],"y_pixel",DFNT_INT8,1);
	VSfdefine(lis_event_vsref[file_number],"bg_value",DFNT_INT16,1);
	VSfdefine(lis_event_vsref[file_number],"bg_radiance",DFNT_INT16,1);
	VSfdefine(lis_event_vsref[file_number],"amplitude",DFNT_INT8,1);/*75*/
	VSfdefine(lis_event_vsref[file_number],"sza_index",DFNT_UINT8,1);
	VSfdefine(lis_event_vsref[file_number],"glint_index",DFNT_UINT8,1);
	VSfdefine(lis_event_vsref[file_number],"approx_threshold",DFNT_INT8,1);
	VSfdefine(lis_event_vsref[file_number],"alert_flag",DFNT_UINT8,1);/*100*/
	VSfdefine(lis_event_vsref[file_number],"cluster_index",DFNT_INT8,1);
	VSfdefine(lis_event_vsref[file_number],"density_index",DFNT_INT8,1);
	VSfdefine(lis_event_vsref[file_number],"noise_index",DFNT_INT8,1);/*119*/
	VSfdefine(lis_event_vsref[file_number],"bg_value_flag",DFNT_INT8,1);
	VSfdefine(lis_event_vsref[file_number],"grouping_sequence",DFNT_INT32,1);


	if(VSsetfields(lis_event_vsref[file_number],
		"TAI93_time,observe_time,location,radiance,footprint,address,parent_address,x_pixel,y_pixel,bg_value,bg_radiance,amplitude,sza_index,glint_index,approx_threshold,alert_flag,cluster_index,density_index,noise_index,bg_value_flag,grouping_sequence")==FAIL){
		printf("event_vsref VSetfields failed\n");
		return (lis_operation_FAIL|lis_VSsetfields_FAIL);   
	}


	if(Vinsert(lis_lightning_vgroup_id[file_number],lis_event_vsref[file_number])==FAIL){
		printf("event_vgroup_id Vinsert failed\n");
		return (lis_operation_FAIL| lis_Vinsert_FAIL); 
	}  
/******************************** viewtime **************************************************/

	if ((lis_viewtime_vsref[file_number] = VSattach(lis_hdf_id[file_number],-1,"w"))==FAIL){
		printf("viewtime_vsref VSattach failed\n");
		return (lis_operation_FAIL|lis_viewtime_VSattach_FAIL);   
	}
			
	VSsetname(lis_viewtime_vsref[file_number],"viewtime");
	VSsetclass(lis_viewtime_vsref[file_number],"data");  
		 
	VSfdefine(lis_viewtime_vsref[file_number],"location",DFNT_FLOAT32,2);
	VSfdefine(lis_viewtime_vsref[file_number],"TAI93_start",DFNT_INT32,1);
	VSfdefine(lis_viewtime_vsref[file_number],"TAI93_end",DFNT_INT32,1);
	VSfdefine(lis_viewtime_vsref[file_number],"effective_obs",DFNT_FLOAT32,1);
	VSfdefine(lis_viewtime_vsref[file_number],"alert_flag",DFNT_UINT8,1);
	VSfdefine(lis_viewtime_vsref[file_number],"approx_threshold",DFNT_INT8,1);


	if(VSsetfields(lis_viewtime_vsref[file_number],
		"location,TAI93_start,TAI93_end,effective_obs,alert_flag,approx_threshold")==FAIL){
		printf("viewtime_vsref VSetfields failed\n");
		return (lis_operation_FAIL|lis_VSsetfields_FAIL);   
	}


	if(Vinsert(lis_point_vgroup_id[file_number],lis_viewtime_vsref[file_number])==FAIL){
		printf("viewtime_vgroup_id Vinsert failed\n");
		return (lis_operation_FAIL| lis_Vinsert_FAIL); 
	}  

/******************************** background image summary **************************************************/

	if ((lis_bg_summary_vsref[file_number] = VSattach(lis_hdf_id[file_number],-1,"w"))==FAIL){
		printf("bg_summary_vsref VSattach failed\n");
		return (lis_operation_FAIL|lis_bg_summary_VSattach_FAIL);   
	}
			
	VSsetname(lis_bg_summary_vsref[file_number],"bg_summary");
	VSsetclass(lis_bg_summary_vsref[file_number],"data");  
		 
	VSfdefine(lis_bg_summary_vsref[file_number],"TAI93_time",DFNT_FLOAT64,1);
	VSfdefine(lis_bg_summary_vsref[file_number],"address",DFNT_INT32,1);
	VSfdefine(lis_bg_summary_vsref[file_number],"boresight",DFNT_FLOAT32,2);
	VSfdefine(lis_bg_summary_vsref[file_number],"corners",DFNT_FLOAT32,8);


	if(VSsetfields(lis_bg_summary_vsref[file_number],
		"TAI93_time,address,boresight,corners")==FAIL){
		printf("bg_summary_vsref VSetfields failed\n");
		return (lis_operation_FAIL|lis_VSsetfields_FAIL);   
	}


	if(Vinsert(lis_point_vgroup_id[file_number],lis_bg_summary_vsref[file_number])==FAIL){
		printf("bg_summary_vgroup_id Vinsert failed\n");
		return (lis_operation_FAIL| lis_Vinsert_FAIL); 
	}  

	return file_number;

}

long create_LIS_BACKGROUND_HDF(char *hdf_fname)
{
	long hid_tmp,i,file_number,sd_tmp,success_flag;
 

/********************************* initialize H interface *******************************/
	
	hid_tmp = (Hopen(hdf_fname,DFACC_CREATE,256)); 
	if(hid_tmp==FAIL){
#ifdef DEBUG
printf("Hopen failed\n");
#endif
		return (FAIL);   
	}
	file_number = -1;
	for(i=0;i<MAX_FILE;i++){
		if(lis_hdf_id[i]==-1){
			file_number = i;
			break;
		}
	}
	if(file_number==-1){
#ifdef DEBUG
printf("MAX_FILE exceeded\n");
#endif
		success_flag = lis_operation_FAIL|lis_MAX_FILE_FAIL;
		return(success_flag);
	}
	if(Vstart( hid_tmp)==FAIL){
#ifdef DEBUG
printf("Vstart  failed\n");
#endif
		success_flag = lis_operation_FAIL|lis_Vstart_FAIL;
		return(success_flag);
	}
	lis_hdf_id[file_number] = hid_tmp;

/****************************** create vgroups ******************************************/
	lis_bg_orbit_data_vgroup_id[file_number] = create_lis_VGROUP(lis_hdf_id[file_number],"orbit","data");



/********************** initailize internal data structures/pointers ********************/

	lis_image_srecord[file_number] = 0;
	lis_bg_data_summary_vrecord[file_number] = 0;
	lis_bg_info_vrecord[file_number] = 0;
	lis_bg_orbit_summary_vrecord[file_number] = 0;

/******************************** orbit summary **************************************************/

	if ((lis_bg_orbit_summary_vsref[file_number] = VSattach(lis_hdf_id[file_number],-1,"w"))==FAIL){
#ifdef DEBUG
printf("lis_bg_orbit_summary_vsref VSattach failed\n");
#endif
		return (lis_operation_FAIL|lis_bg_orbit_summary_VSattach_FAIL);   
	}
			
	VSsetname(lis_bg_orbit_summary_vsref[file_number],"bg_orbit_summary");
	VSsetclass(lis_bg_orbit_summary_vsref[file_number],"data");  
		 
	VSfdefine(lis_bg_orbit_summary_vsref[file_number],"id_number",DFNT_INT32,1);
	VSfdefine(lis_bg_orbit_summary_vsref[file_number],"TAI93_start",DFNT_FLOAT64,1);
	VSfdefine(lis_bg_orbit_summary_vsref[file_number],"UTC_start",DFNT_CHAR,28);
	VSfdefine(lis_bg_orbit_summary_vsref[file_number],"GPS_start",DFNT_FLOAT64,1);
	VSfdefine(lis_bg_orbit_summary_vsref[file_number],"TAI93_end",DFNT_FLOAT64,1);
	VSfdefine(lis_bg_orbit_summary_vsref[file_number],"start_longitude",DFNT_FLOAT32,1);
	VSfdefine(lis_bg_orbit_summary_vsref[file_number],"end_longitude",DFNT_FLOAT32,1);
	VSfdefine(lis_bg_orbit_summary_vsref[file_number],"image_summary_count",DFNT_INT16,1);
	VSfdefine(lis_bg_orbit_summary_vsref[file_number],"image_summary_address",DFNT_INT16,1);
	VSfdefine(lis_bg_orbit_summary_vsref[file_number],"image_info_count",DFNT_INT16,1);
	VSfdefine(lis_bg_orbit_summary_vsref[file_number],"image_info_address",DFNT_INT16,1);


	if(VSsetfields(lis_bg_orbit_summary_vsref[file_number],
		"id_number,TAI93_start,UTC_start,GPS_start,TAI93_end,start_longitude,end_longitude,image_summary_count,image_summary_address,image_info_count,image_info_address")==FAIL){
#ifdef DEBUG
printf("lis_bg_orbit_summary_vsref VSetfields failed\n");
#endif
		return (lis_operation_FAIL|lis_VSsetfields_FAIL);   
	}


	if(Vinsert(lis_bg_orbit_data_vgroup_id[file_number],lis_bg_orbit_summary_vsref[file_number])==FAIL){
#ifdef DEBUG
printf("bg_orbit_summary_vgroup_id Vinsert (lis_bg_orbit_summary_vsref) failed\n");
#endif
		return (lis_operation_FAIL|lis_Vinsert_FAIL); 
	}  


/******************************** bg_info **************************************************/
	if ((lis_bg_info_vsref[file_number] = VSattach(lis_hdf_id[file_number],-1,"w"))==FAIL){
#ifdef DEBUG
printf("lis_bg_info_vsref VSattach failed\n");
#endif
		return (lis_operation_FAIL| lis_bg_info_VSattach_FAIL);   
	}
			
	VSsetname(lis_bg_info_vsref[file_number],"bg_info");
	VSsetclass(lis_bg_info_vsref[file_number],"data");  
		 
	VSfdefine(lis_bg_info_vsref[file_number],"TAI93_time",DFNT_FLOAT64,1);
	VSfdefine(lis_bg_info_vsref[file_number],"alert_summary",DFNT_UINT8,1);
	VSfdefine(lis_bg_info_vsref[file_number],"instrument_alert",DFNT_UINT8,1);
	VSfdefine(lis_bg_info_vsref[file_number],"platform_alert",DFNT_UINT8,1);
	VSfdefine(lis_bg_info_vsref[file_number],"external_alert",DFNT_UINT8,1);
	VSfdefine(lis_bg_info_vsref[file_number],"processing_alert",DFNT_UINT8,1);
	VSfdefine(lis_bg_info_vsref[file_number],"position_vector",DFNT_FLOAT32,3);
	VSfdefine(lis_bg_info_vsref[file_number],"velocity_vector",DFNT_FLOAT32,3);
	VSfdefine(lis_bg_info_vsref[file_number],"transform_matrix",DFNT_FLOAT32,9);
	VSfdefine(lis_bg_info_vsref[file_number],"solar_vector",DFNT_FLOAT32,3);
	VSfdefine(lis_bg_info_vsref[file_number],"ephemeris_quality_flag",DFNT_INT32,1);
	VSfdefine(lis_bg_info_vsref[file_number],"attitude_quality_flag",DFNT_INT32,1);
	VSfdefine(lis_bg_info_vsref[file_number],"bg_value",DFNT_INT16,1);
	VSfdefine(lis_bg_info_vsref[file_number],"noise_index",DFNT_INT8,1);
	VSfdefine(lis_bg_info_vsref[file_number],"event_count",DFNT_INT16,6);


	if(VSsetfields(lis_bg_info_vsref[file_number],
		"TAI93_time,alert_summary,instrument_alert,platform_alert,external_alert,processing_alert,position_vector,velocity_vector,transform_matrix,solar_vector,ephemeris_quality_flag,attitude_quality_flag,bg_value,noise_index,event_count")==FAIL){
#ifdef DEBUG
printf("lis_bg_info_vsref VSetfields failed\n");
#endif
		return (lis_operation_FAIL|lis_VSsetfields_FAIL );   
	}

	if(Vinsert(lis_bg_orbit_data_vgroup_id[file_number],lis_bg_info_vsref[file_number])==FAIL){
#ifdef DEBUG
printf("bg_orbit_summary_vgroup_id Vinsert (lis_bg_info_vsref) failed\n");
#endif
		return (lis_operation_FAIL|lis_Vinsert_FAIL ); 
	}  




/******************************** background image summary **************************************************/

	if ((lis_bg_data_summary_vsref[file_number] = VSattach(lis_hdf_id[file_number],-1,"w"))==FAIL){
#ifdef DEBUG
printf("lis_bg_data_summary_vsref VSattach failed\n");
#endif
		return (lis_operation_FAIL|lis_bg_data_summary_VSattach_FAIL );   
	}
			
	VSsetname(lis_bg_data_summary_vsref[file_number],"bg_data_summary");
	VSsetclass(lis_bg_data_summary_vsref[file_number],"data");  
		 
	VSfdefine(lis_bg_data_summary_vsref[file_number],"TAI93_time",DFNT_FLOAT64,1);
	VSfdefine(lis_bg_data_summary_vsref[file_number],"address",DFNT_INT32,1);
	VSfdefine(lis_bg_data_summary_vsref[file_number],"boresight",DFNT_FLOAT32,2);
	VSfdefine(lis_bg_data_summary_vsref[file_number],"corners",DFNT_FLOAT32,8);


	if(VSsetfields(lis_bg_data_summary_vsref[file_number],
		"TAI93_time,address,boresight,corners")==FAIL){
#ifdef DEBUG
printf("lis_bg_data_summary_vsref VSetfields failed\n");
#endif
		return (lis_operation_FAIL|lis_VSsetfields_FAIL);   
	}
	if(Vinsert(lis_bg_orbit_data_vgroup_id[file_number],lis_bg_data_summary_vsref[file_number])==FAIL){
#ifdef DEBUG
printf("bg_orbit_summary_vgroup_id Vinsert (lis_bg_data_summary_vsref) failed\n");
#endif
		return (lis_operation_FAIL|lis_Vinsert_FAIL); 
	}  



/*************************** create scientific data sets (SDS) **************************/

	sd_tmp = SDstart(hdf_fname,DFACC_RDWR);
	if(sd_tmp==-1){
#ifdef DEBUG
printf("SDstart failed\n");
#endif
		return (FAIL);   
	}
	lis_sd_id[file_number] = sd_tmp;
	lis_dimsize[0]=0;
	lis_dimsize[1]=128;
	lis_dimsize[2]=128;
	lis_image_sds_id[file_number] = SDcreate(lis_sd_id[file_number],"bg_data",DFNT_INT16,3,lis_dimsize); 
 
	lis_dimsize[0]=128;
	lis_dimsize[1]=128;


	return file_number;
}

long close_LIS_ORBIT_HDF(int32 file_number)

/****************************************************************************************/
/*                                                                             		*/
/*	close_hdf_file									*/
/*											*/
/*	returns 0 if all vgroups, vsets and the file itself are successfully    	*/
/*	closed										*/
/*	returns FAIL(-1) on error							*/
/*											*/
/*	Date last modified 9/7/94							*/
/****************************************************************************************/

{
	long return_status;
	return_status = 0;
	

        if(lis_event_vsref[file_number]!= FAIL)VSdetach(lis_event_vsref[file_number]);			/* Events */

	if(lis_group_vsref[file_number]!= FAIL)VSdetach(lis_group_vsref[file_number]);			/* Group */

	if(lis_flash_vsref[file_number]!= FAIL)VSdetach(lis_flash_vsref[file_number]);			/* Flash */

	if(lis_area_vsref[file_number]!= FAIL)VSdetach(lis_area_vsref[file_number]);			/* Area */

	if(lis_lightning_vgroup_id[file_number]!= FAIL)Vdetach(lis_lightning_vgroup_id[file_number]);

	
	if(lis_viewtime_vsref[file_number]!= FAIL)VSdetach(lis_viewtime_vsref[file_number]);		/* viewtime */
	
	if(lis_point_summary_vsref[file_number]!= FAIL)VSdetach(lis_point_summary_vsref[file_number]);		/* Point summary */

	if(lis_bg_summary_vsref[file_number]!= FAIL)VSdetach(lis_bg_summary_vsref[file_number]);		/* background summary */

	if(lis_one_second_vsref[file_number]!= FAIL)VSdetach(lis_one_second_vsref[file_number]);		/* one second summary */

	if(lis_orbit_summary_vsref[file_number]!= FAIL)VSdetach(lis_orbit_summary_vsref[file_number]);		/* orbit summary */

	if(lis_point_vgroup_id[file_number]!= FAIL)Vdetach(lis_point_vgroup_id[file_number]);
	if(lis_orbit_data_vgroup_id[file_number]!= FAIL)Vdetach(lis_orbit_data_vgroup_id[file_number]);
		

	if(lis_hdf_id[file_number]!=FAIL)Vend(lis_hdf_id[file_number]);

	if(lis_hdf_id[file_number] !=FAIL)if(Hclose(lis_hdf_id[file_number])==FAIL)return_status = FAIL;	

	lis_hdf_id[file_number] = -1;

	if (return_status != lis_operation_SUCCESSFUL) {
		fprintf(stderr,"Error closing HDF file.\n");
	}
	return (return_status);		/* success*/




}

long close_LIS_BACKGROUND_HDF(int32 file_number)

/****************************************************************************************/
/*                                                                             		*/
/*	close_hdf_file									*/
/*											*/
/*	returns 0 if all vgroups, vsets and the file itself are successfully    	*/
/*	closed										*/
/*	returns FAIL(-1) on error							*/
/*											*/
/*	Date last modified 9/7/94							*/
/****************************************************************************************/

{
	long return_status;
	return_status = 0;
	
	if(lis_bg_orbit_summary_vsref[file_number]!= FAIL)VSdetach(lis_bg_orbit_summary_vsref[file_number]);		/* bg orbit data summary */
	if(lis_bg_info_vsref[file_number]!= FAIL)VSdetach(lis_bg_info_vsref[file_number]);			/* bg info summary */
	if(lis_bg_data_summary_vsref[file_number]!= FAIL)VSdetach(lis_bg_data_summary_vsref[file_number]);		/* background summary */
	if(lis_bg_orbit_data_vgroup_id[file_number]!=FAIL)Vdetach(lis_bg_orbit_data_vgroup_id[file_number]);

 /*
	if(SDsetattr(image_sds_id[file_number],"total images",DFNT_INT32,1,&image_srecord[file_number])==FAIL)
	return_status = FAIL;
*/

	if(lis_image_sds_id[file_number]!=FAIL)if(SDendaccess(lis_image_sds_id[file_number])==FAIL)return_status = FAIL;	/* image */
		 

 	if(lis_sd_id[file_number]!=FAIL)if (SDend(lis_sd_id[file_number])==FAIL)return_status = FAIL;			/* SD id */

	if(lis_hdf_id[file_number]!=FAIL)Vend(lis_hdf_id[file_number]);

	if(lis_hdf_id[file_number] !=FAIL)if(Hclose(lis_hdf_id[file_number])==FAIL)return_status = FAIL;	

	lis_hdf_id[file_number] = -1;
	return return_status;
}


long create_lis_VGROUP(long lis_hdf_id,char *name,char *class)

/****************************************************************************************/
/*											*/
/*	create a vgroup in the HDF file							*/
/*											*/
/*	returns the reference number of the Vgroup if successful			*/
/*	returns FAIL (-1) on error							*/
/*											*/
/*	Date last modified	8/15/94							*/
/****************************************************************************************/

{
	long vgroup;
	vgroup= (long)Vattach(lis_hdf_id,-1,"w");
	if(vgroup == FAIL)return FAIL;	
	Vsetname(vgroup,name);
	Vsetclass(vgroup,class);
	return (vgroup);

}


 
long write_lis_BACKGROUND_DATA(long file_number,short int *databuffer)

/****************************************************************************************/
/*											*/
/*	Write sds image data to HDF file						*/
/*											*/
/*	returns sds record # if successful						*/
/*	returns FAIL (-1) on error							*/
/*											*/
/*	Date last modified	8/15/94							*/
/****************************************************************************************/


{

	start[0]=lis_image_srecord[file_number]++;
	start[1]=start[2]=0;

	endr[0]=1;
	endr[1]=endr[2]=128;

	if(SDwritedata(lis_image_sds_id[file_number],start,NULL,endr,databuffer)==FAIL){
		lis_image_srecord[file_number]--;
		return (lis_operation_FAIL|lis_SDwritedata_FAIL);}
	return (lis_image_srecord[file_number]-1);
}




long write_lis_EVENT(long file_number,void *databuffer)		{return (write_lis_VSGROUP(file_number,databuffer,lis_event_vsref,lis_event_vrecord));}
long write_lis_GROUP(long file_number,void *databuffer) 		{return (write_lis_VSGROUP(file_number,databuffer,lis_group_vsref,lis_group_vrecord));}
long write_lis_FLASH(long file_number,void *databuffer) 		{return (write_lis_VSGROUP(file_number,databuffer,lis_flash_vsref,lis_flash_vrecord));}
long write_lis_AREA(long file_number,void *databuffer)	 	{return (write_lis_VSGROUP(file_number,databuffer,lis_area_vsref,lis_area_vrecord));}
long write_lis_POINT_SUMMARY(long file_number,void *databuffer)	{return (write_lis_VSGROUP(file_number,databuffer,lis_point_summary_vsref,lis_point_summary_vrecord));}
long write_lis_VIEWTIME(long file_number,void *databuffer)	 	{return (write_lis_VSGROUP(file_number,databuffer,lis_viewtime_vsref,lis_viewtime_vrecord));}
long write_lis_BG_SUMMARY(long file_number,void *databuffer)	{return (write_lis_VSGROUP(file_number,databuffer,lis_bg_summary_vsref,lis_bg_summary_vrecord));}
long write_lis_ONE_SECOND(long file_number,void *databuffer)	{return (write_lis_VSGROUP(file_number,databuffer,lis_one_second_vsref,lis_one_second_vrecord));}
long write_lis_ORBIT_SUMMARY(long file_number,void *databuffer)	{return (write_lis_VSGROUP(file_number,databuffer,lis_orbit_summary_vsref,lis_orbit_summary_vrecord));}
long write_lis_BG_DATA_SUMMARY(long file_number,void *databuffer)	{return (write_lis_VSGROUP(file_number,databuffer,lis_bg_data_summary_vsref,lis_bg_data_summary_vrecord));}
long write_lis_BG_INFO(long file_number,void *databuffer)		{return (write_lis_VSGROUP(file_number,databuffer,lis_bg_info_vsref,lis_bg_info_vrecord));}
long write_lis_BG_ORBIT_SUMMARY(long file_number,void *databuffer)	{return (write_lis_VSGROUP(file_number,databuffer,lis_bg_orbit_summary_vsref,lis_bg_orbit_summary_vrecord));}


long write_lis_VSGROUP(long file_number,void *databuffer,long vsref[],long vsrecord[])

/****************************************************************************************/
/*											*/
/*	Write data to HDF file								*/
/*											*/
/*	returns Vdata record # if successful						*/
/*	returns VSwrite_FAIL on error							*/
/*											*/
/*	Date last modified	8/15/94							*/
/****************************************************************************************/


{
	vsrecord[file_number]++;
 	if(VSwrite(vsref[file_number],databuffer,1,FULL_INTERLACE)==FAIL){
		vsrecord[file_number]--;
		return (lis_operation_FAIL|lis_VSwrite_FAIL);
	}
	return vsrecord[file_number]-1;
}
