/* Basic includes you should have in any program 
   using the low-level API */

#include <stdio.h>
#include <math.h>
#include "hdf.h"
#include "liblisotd_err_defs.h"
#include "liblisotd_readhdf.h"

main(int argc, char *argv[]) {

	long fid,i;
	long lis_events, lis_one_seconds, lis_areas;
	struct lis_area          *lis_area_var;
	struct lis_event         *lis_event_var;
	struct lis_one_second    *lis_one_second_var;
	char fname[132];

	/* This is the file we'll access; it lives in the
	   same directory as the executable. */
	   
	sprintf(fname,"TRMM_LIS_SC.03.4_1998.028.00970");

	/* Get the file ID of the LIS HDF data file.
	   Some orbits may not have metadata inserted 
	     into them, or may legitimately have no 
	     point vdatas.  Errors are encoded in the 
	     file ID's if this occurs.  We mask them
	      out in order to continue. */
	
	fid = OPEN_lis_orbit_hdf(fname);
	
    if(((fid&0xffffff00)==lis_NoMetadataMissingVdata)) {
    			fid = fid^lis_NoMetadataMissingVdata;
    } else if(((fid&0xffffff00)==lis_NoMetadata)) {
				fid = fid^lis_NoMetadata;
    } else if(((fid&0xffffff00)==lis_MissingVdata)) {
    			fid = fid^lis_MissingVdata;
    }
    
    /* Should have a valid file ID at this point.  
        If not, we're in trouble. */
    	
    if (fid < 0) {
    		printf("Unrecoverable error opening %s\n",fname);
            if(CLOSE_lis_orbit_hdf(fid)!=lis_operation_SUCCESSFUL)
            	printf("close %s file failed\n",fname);
   			exit(-1);
    }

	/* Get the number of area vdata records */
	
	lis_areas = lis_area_COUNT(fid);
	
	/* Allocate memory accordingly */
	
	lis_area_var = (struct lis_area *)
		malloc((size_t)(lis_areas*sizeof(struct lis_area)));
	
	/* Read the area vdata i from the file.  It will be 
	  	contained in the variable lis_area_var, which
	  	follows the conventional structure definition.
	  	See Appendix C of the documentation or the file
	  	liblisotd_readhdf.h for details. */
	  	
	i = READ_lis_areas(fid,lis_area_var,0,lis_areas);

	/* Exactly the same for events */
	
	lis_events = lis_event_COUNT(fid);
	lis_event_var = (struct lis_event *)
		malloc((size_t)(lis_events*sizeof(struct lis_event)));
	i = READ_lis_events(fid,lis_event_var,0,lis_events);

	/* And for one second data, etc. */
	
	lis_one_seconds = lis_one_second_COUNT(fid);
	lis_one_second_var = (struct lis_one_second *)
		malloc((size_t)(lis_one_seconds*sizeof(struct lis_one_second)));
	i = READ_lis_one_seconds(fid,lis_one_second_var,0,lis_one_seconds);

	/* You get the idea.  Now close out the orbit nicely and
		free the allocated memory. */
		
   	if(CLOSE_lis_orbit_hdf(fid)!=lis_operation_SUCCESSFUL)
   		printf("close %s file failed\n",fname);
   		
   	free(lis_area_var);
   	free(lis_event_var);
   	free(lis_one_second_var);
}
