@lisapp_GlobalRoutines.pro

; EXAMPLE IDL COMMANDS TO RUN THIS PROGRAM
;
; IDL> .run lisapp.pro
; IDL> fname='TRMM_LIS_SC.03.4_1998.025.00919'
; IDL> read_orbit,fname,orbit
; IDL> .run overpass
; IDL> zoom_overpass,  159858990.0d0, 159859000.0d0, 0.002d0
;

PRO ZOOM_OVERPASS, start_time, end_time, delta_time, outpath

  common datafiles, orbit, fname
  COMMON datablock, array,xsize,ysize

  usersym,sin(findgen(21)/10.*!pi),cos(findgen(21)/10.*!pi),/fill
  load_my_color_tables
  ocean_color=15
  land_color=12
  grid_color=0
  equator_color=104
  fov_border_color=3*0+174
  fov_fill_color=17*0+192
  prev_fov_color=16
  continent_color=145
  black=1
  white=0
  tvlct,r,g,b,/get

  xsize=420
  ysize=420
  xsize=320
  ysize=320
  window,0,colors=220,xs=xsize,ys=ysize
  erase,black
  !P.position=[.05,.05,.95,.95]

  time_offset_index= long(orbit.one_second(0).TAI93_time)

  ;determine where to put the center of the map
  set_map_size, map_lat, map_lon,dy,dx , start_time

  map_set,map_lat,map_lon,/horiz,/orthographic,color=0,$
    limit=[map_lat-dy,map_lon-dx,map_lat+dy,map_lon+dx]
  ; make ocean blue
  tv,intarr(xsize*.90,ysize*.90)+ocean_color,xsize*.05,ysize*.05
  array=tvrd(0,0,xsize,ysize)


  ; determine the places that the instrument has already seen
  ; from the beginning of the orbit.
  ;prior_observation_path, new_location, time_offset_index

  ;---------------------------------------------------------------
  Epos=convert_coord([xsize/2.], [ysize/2.], /device,/to_data)
  Ex=cos(Epos(0,*)*!dtor)*cos(Epos(1,*)*!dtor)
  Ey=sin(Epos(0,*)*!dtor)*cos(Epos(1,*)*!dtor)
  Ez=sin(Epos(1,*)*!dtor)

  Sx=orbit.one_second(start_time-time_offset_index).solar_vector(0)
  Sy=orbit.one_second(start_time-time_offset_index).solar_vector(1)
  Sz=orbit.one_second(start_time-time_offset_index).solar_vector(2)

  factor=fix((Ex*Sx+Ey*Sy+Ez*Sz)*32+64)
  index=where(array eq ocean_color)
  array_ocean=array*0+1
  array_ocean(index)=array(index)*0 +(factor > 64)
  ;---------------------------------------------------------------

  ; step through time making plot of what the instrument sees
  
  ; print,long((end_time-start_time+delta_time/2.)/delta_time)
  
  for itime=0L, long((end_time-start_time+delta_time/2.)/delta_time),1L do begin
    time=start_time+delta_time*itime*1.0D0
    ; print,time,format='(f14.4)'

    set_map_size, map_lat, map_lon, dy , dx , time+0*3.00D0
    map_set,map_lat,map_lon,/orthographic,$
      limit=[map_lat-dy,map_lon-dx,map_lat+dy,map_lon+dx]
  ;---------------------------------------------------------------
  Epos=convert_coord([xsize/2.], [ysize/2.], /device,/to_data)
  Ex=cos(Epos(0,*)*!dtor)*cos(Epos(1,*)*!dtor)
  Ey=sin(Epos(0,*)*!dtor)*cos(Epos(1,*)*!dtor)
  Ez=sin(Epos(1,*)*!dtor)

  Sx=orbit.one_second(start_time-time_offset_index).solar_vector(0)
  Sy=orbit.one_second(start_time-time_offset_index).solar_vector(1)
  Sz=orbit.one_second(start_time-time_offset_index).solar_vector(2)

  factor=fix((Ex*Sx+Ey*Sy+Ez*Sz)*32+64)
  index=where(array eq ocean_color)
  array_ocean=array*0+1
  array_ocean(index)=array(index)*0 +(factor > 64)
  ;---------------------------------------------------------------
    erase,black
;-------------
    map_continents,/cont,/fill,color=land_color
    array_land=tvrd(0,0,xsize,ysize)

    array_land=array_land*(array/ocean_color) > 1

    land_index=where(array_land eq land_color)
    array_land=array_ocean
    if (land_index(0) ne -1) then begin
      array_land(land_index)=array_ocean(land_index)-32
    endif
    tv,array_land
;-------------
;   tv,array,0,0
; ; map_grid,latdel=5,londel=5,glinesty=1,color=grid_color
;   oplot,findgen(362)-180.,0*findgen(362),color=equator_color,thick=1
;   map_continents,/fill,color=land_color
    map_continents,/usa,mlinethick=1,color=continent_color
; ; map_continents,/coast,color=black,mlinethick=1
    map_continents,/cont,/hires,color=continent_color,mlinethick=1+xsize/400
    lower=fix(map_lon-dx)/5*5-5
    upper=fix(map_lon+dx)/5*5+10
    for m=lower,upper,5 do oplot,intarr(901)+m,findgen(901)/5.-90,psym=3,color=grid_color
    lower=fix(map_lat-dy)/5*5-5
    upper=fix(map_lat+dy)/5*5+10
    for m=lower,upper,5 do oplot,findgen(1801)/5.-180,intarr(1801)+m,psym=3,color=grid_color
    tv,intarr(xsize,fix(ysize*.05))+black,0,0
    tv,intarr(xsize,fix(ysize*.05))+black,0,ysize*.95
    tv,intarr(fix(xsize*.05),ysize)+black,0,0
    tv,intarr(fix(xsize*.05),ysize)+black,xsize*.95,0

  ; map_grid,latdel=5,londel=5,glinesty=1,color=grid_color
    map_set,map_lat,map_lon,/orthographic,$
      limit=[map_lat-dy,map_lon-dx,map_lat+dy,map_lon+dx],/noerase
    

;   display previous orbits
;   for k=4,1,-1 do begin 
;     tmp=new_location[*,1]+16.0*k
;     index=where(tmp gt 180)
;     if (index(0) ne -1) then tmp(index)=tmp(index)-360
;     for m=0,n_elements(tmp)-1,5 do $
;       polyfill,tmp(m:4+m),new_location[m:4+m,0],color=5+k
;   endfor


    ; plot prior observations from this orbit
    px=[1,126,126,1,1]
    py=[1,1,126,126,1]
    num=5

;   px=[0,63,120,63,0,0]
;   py=[2,-3,63,130,125,2]
;   num=6
;   this_sensor='lis'
;   for j=time_offset_index,time-35,10 do begin
;     get_vector_pointing_vector,this_sensor,px,py,look_vector
;     get_vector_earth_intersection,look_vector, $
;      orbit.one_second(j-time_offset_index).TAI93_time*(intarr(num)+1),$
;        location,return_code
;     polyfill,location[*,1],location[*,0],color=prev_fov_color
;   endfor


    ; plot current observations for this moment in time
    px=[0,30,53,73,98,127,127,127,127,127,127, 98, 73, 53, $
        30,  0, 0, 0, 0, 0,0]
    py=[0, 0, 0, 0, 0,  0, 30, 53, 73, 98,127,127,127,127,$
       127,127,98,73,53,30,0]
    num=n_elements(px)
    this_sensor='lis'
    get_vector_pointing_vector,this_sensor,px,py,look_vector
    get_vector_earth_intersection_new,look_vector, $
     time*(dblarr(num)+1.D0)+0*3.00D0, location,return_code
    polyfill,location[*,1],location[*,0],color=fov_fill_color

    ; display the background images 
     plot_backgrounds,xsize,ysize,fov_fill_color,time,2.0

    ;plot previous events
    event_index=where(orbit.point.lightning.event.tai93_time ge start_time and $
       orbit.point.lightning.event.tai93_time lt time)
    if (event_index(0) ne -1) then begin
      ppx=orbit.point.lightning.event(event_index).x_pixel
      ppy=orbit.point.lightning.event(event_index).y_pixel
      get_vector_pointing_vector,this_sensor,ppx,ppy,look_vector
      get_vector_earth_intersection_new,look_vector, $
       orbit.point.lightning.event(event_index).tai93_time+0*3.00D0, $
       e_location,return_code
      oplot,e_location[*,1],e_location[*,0],psym=8,$
        color=11,symsize=.05+fix(xsize/200)*.10
    endif
    ;plot current events
    event_index=where(orbit.point.lightning.event.tai93_time ge time and $
       orbit.point.lightning.event.tai93_time lt time+delta_time)
    if (event_index(0) ne -1) then begin
      ppx=orbit.point.lightning.event(event_index).x_pixel
      ppy=orbit.point.lightning.event(event_index).y_pixel
      get_vector_pointing_vector,this_sensor,ppx,ppy,look_vector
      get_vector_earth_intersection_new,look_vector, $
       orbit.point.lightning.event(event_index).tai93_time+0*3.00D0, $
       e_location,return_code
      oplot,e_location[*,1],e_location[*,0],psym=8,$
        color=2,symsize=.2+fix(xsize/200)*.10
      oplot,e_location[*,1],e_location[*,0],psym=8,$
        color=9,symsize=.05+fix(xsize/200)*.10
    endif

    oplot,location[*,1],location[*,0],color=fov_border_color,$
          thick=2+max([xsize,ysize])/400

  ; set_map_size, map_lat, map_lon, dy , dx , time
  ; dy=6
  ; dx=6/max([abs(cos((map_lat-dy/2.)*!dtor)),abs(cos((map_lat+dy/2.)*!dtor))])
    map_set,map_lat,map_lon,/horiz,/orthographic,/noerase,color=white,$
      limit=[map_lat-dy,map_lon-dx,map_lat+dy,map_lon+dx]

    image=tvrd(0,0,xsize,ysize)
    filename= outpath
    filename = filename + 'pass_'+string(itime,format='(i4.4)')+'.gif'
    write_gif,filename,image,r,g,b
    wait,1
  endfor
  
END

;-----------------------------------------------------------------------
PRO ORTHO_OVERPASS, start_time, end_time, delta_time

  common datafiles, orbit, fname
  COMMON datablock, array,xsize,ysize
  usersym,sin(findgen(21)/10.*!pi),cos(findgen(21)/10.*!pi),/fill

  if (start_time lt orbit.orbit_summary.TAI93_START) then begin
   print,'Start time before beginning of orbit at ',$
     orbit.orbit_summary.TAI93_START,format='(a,f14.4)'
   return
  endif
  if (end_time gt orbit.orbit_summary.TAI93_END) then begin
   print,'End time after end of orbit at ',$
     orbit.orbit_summary.TAI93_END,format='(a,f14.4)'
   return
  endif

  xsize=210
  ysize=210
  window,0,colors=220,xs=xsize,ys=ysize

  load_my_color_tables
  ocean_color=15
  land_color=12
  grid_color=6
  equator_color=104
  fov_border_color=3
  fov_fill_color=17
  prev_fov_color=16
  continent_color=145
  black=1
  white=0

  erase,black

  time_offset_index= long(orbit.one_second(0).TAI93_time)

  ;determine where to put the center of the map

  get_nadir_location,$
    orbit.one_second(start_time-time_offset_index).TAI93_time,location
  map_lat=0
  map_lon=location[1]

  !P.position=[.05,.05,.95,.95]
  erase,black
  map_set,map_lat,map_lon,/horiz,/orthographic,color=white,/noerase
  map_set,map_lat,map_lon,/horizon,/orthographic,/noerase,$
   e_horizon={Fill:2,color:ocean_color}
  tv,intarr(xsize,2)+black,0,ysize*.05
  tv,intarr(xsize,2)+black,0,ysize*.95
  tv,intarr(2,ysize)+black,xsize*.05,0
  tv,intarr(2,ysize)+black,xsize*.95,0
  array=tvrd(0,0,xsize,ysize)

 ;print,'painting oceans blue...'
 ;FLOODFILL,xsize/2,ysize/2,black,ocean_color
 ;print,'...finished painting oceans'
  index=where(array eq white)
  array(index)=black

  tvlct,r,g,b,/get

  this_sensor='lis'
  px=[5,122,122,5,5]
  py=[5,5,122,122,5]
  for j=time_offset_index+3000,$
        time_offset_index+orbit.orbit_summary.one_second_count,10 do begin
    get_vector_pointing_vector,this_sensor,px,py,look_vector
    get_vector_earth_intersection,look_vector, $
     orbit.one_second(j-time_offset_index).TAI93_time*(intarr(5)+1),$
       location,return_code
    if( j eq time_offset_index+3000) then begin
      new_location=location
    endif else begin
      new_location=[new_location,location]
    endelse
  endfor

 ;for time=start_time, end_time, delta_time do begin
  for itime=0, (end_time-start_time+delta_time/2.)/delta_time do begin
    time=start_time+delta_time*itime*1.0D0
    ; print,time,format='(f14.4)'
    get_nadir_location,$
      orbit.one_second(time-time_offset_index).TAI93_time,location
    map_lat=0
    map_lon=location[1]
    map_set,map_lat,map_lon,/orthographic
    erase,black
    index=where(array eq ocean_color)

    Epos=convert_coord(index mod xsize, index/xsize, /device,/to_data)
    Ex=cos(Epos(0,*)*!dtor)*cos(Epos(1,*)*!dtor)
    Ey=sin(Epos(0,*)*!dtor)*cos(Epos(1,*)*!dtor)
    Ez=sin(Epos(1,*)*!dtor)
    Sx=orbit.one_second(time-time_offset_index).solar_vector(0)
    Sy=orbit.one_second(time-time_offset_index).solar_vector(1)
    Sz=orbit.one_second(time-time_offset_index).solar_vector(2)

    factor=(Ex*Sx+Ey*Sy+Ez*Sz)*32+64
    e_color=intarr(n_elements(index))
    e_color(*)=factor(0,*)
    e_color=e_color > 64
    array_ocean=array*0 +1

    for i=0L,n_elements(index)-1 do array_ocean(index(i))=e_color(i)
    tv,array_ocean
   ;map_grid,latdel=10,londel=10,glinesty=0,color=grid_color
   ;oplot,findgen(362)-180.,0*findgen(362),color=equator_color,thick=1
    array_ocean_and_grid=tvrd(0,0,xsize,ysize)
    erase,1
    map_continents,/cont,/fill,color=land_color
    array_land=tvrd(0,0,xsize,ysize)

    array_land=array_land*(array/ocean_color) > 1

    land_index=where(array_land eq land_color)
    array_land=array_ocean_and_grid
    array_land(land_index)=array_ocean(land_index)-32
    tv,array_land

   ;;map_continents,/cont,/fill,color=land_color
    map_set,map_lat,map_lon,/orthographic,/noerase,/cont,color=continent_color
   ;map_set,map_lat,map_lon,/horiz,/orthographic,/noerase,color=white
    tv,intarr(xsize,2)+black,0,ysize*.05
    tv,intarr(xsize,2)+black,0,ysize*.95
    tv,intarr(2,ysize)+black,xsize*.05,0
    tv,intarr(2,ysize)+black,xsize*.95,0

   ;For plotting previous orbit's field-of-view
    for k=1,1 do begin 
      tmp=new_location[*,1]+16.0*k
      index=where(tmp gt 180)
      if (index(0) ne -1) then tmp(index)=tmp(index)-360
      for m=0,n_elements(tmp)-1,5 do $
        polyfill,tmp(m:4+m),new_location[m:4+m,0],color=prev_fov_color
    endfor


    px=[3,125,125,3,3]
    py=[3,3,125,125,3]
    this_sensor='lis'
  ; for j=time_offset_index,time,10 do begin
    for j=max([time_offset_index,time-3000]),time,10 do begin
      get_vector_pointing_vector,this_sensor,px,py,look_vector
      get_vector_earth_intersection,look_vector, $
       orbit.one_second(j-time_offset_index).TAI93_time*(intarr(5)+1),$
         location,return_code
      polyfill,location[*,1],location[*,0],color=prev_fov_color
    endfor


   ;px=[0,30,63,98,127,127,127,127,127,98,63,30,0,0,0,0,0]
   ;py=[0,0,0,0,0,30,63,98,127,127,127,127,127,98,63,30,0]
    px=[0,30,53,73,98,127,127,127,127,127,127, 98, 73, 53, $
        30,  0, 0, 0, 0, 0,0]
    py=[0, 0, 0, 0, 0,  0, 30, 53, 73, 98,127,127,127,127,$
       127,127,98,73,53,30,0]
    num=n_elements(px)
    this_sensor='lis'
    get_vector_pointing_vector,this_sensor,px,py,look_vector
    get_vector_earth_intersection,look_vector, $
     orbit.one_second(time-time_offset_index).TAI93_time*(intarr(num)+1),$
       location,return_code
    polyfill,location[*,1],location[*,0],color=fov_fill_color
 ; plot backgrounds within field-of-view
    plot_backgrounds,xsize,ysize,fov_fill_color,time,.5

    ;plot events
  ; event_index=where(orbit.point.lightning.event.tai93_time le time)
  ; if (event_index(0) ne -1) then begin
  ;   oplot,orbit.point.lightning.event(event_index).location[1],$
  ;         orbit.point.lightning.event(event_index).location[0],psym=8,$
  ;   color=2,symsize=.25
  ;   oplot,orbit.point.lightning.event(event_index).location[1],$
  ;         orbit.point.lightning.event(event_index).location[0],psym=8,$
  ;   color=9,symsize=.1
  ; endif

    oplot,location[*,1],location[*,0],color=fov_border_color,$
          thick=1+max([xsize,ysize])/400
   ;oplot,location[*,1],location[*,0],color=fov_border_color
    image=tvrd(0,0,xsize,ysize)
    filename='over_'+string(itime+6000,format='(i4.4)')+'.gif'
   ;filename='over_'+string((time-start_time)/delta_time,format='(i3.3)')+'.gif'
    write_gif,filename,image,r,g,b
  endfor

  
; this_sensor='lis'
; for time=start_time, end_time, delta_time do begin

;   get_vector_pointing_vector,this_sensor,px,py,look_vector
;   time_index=time-time_offset_index
;   get_vector_earth_intersection,look_vector, $
;    orbit.one_second(time_index).TAI93_time*[1,1,1,1,1,1,1,1,1],$
;      location,return_code

; endfor
  
END

;-----------------------------------------------------------------------
PRO get_pointing_vector, sensor, xpix, ypix, look_vector

  coeff = dblarr(4)

  if (sensor eq 'lis') then begin
    coeff = [1.4754537D, -.36224695D, -.088939824D, -.28203806D]
    x = (63.5-xpix)/127.
    y = (ypix-63.5)/127.
  endif else begin
    if (sensor eq 'otd') then begin
      coeff = [1.4037314D, .015948348D, -.86702375D, .24905820D]
      x=(xpix-63.5)/127.
      y=(63.5-ypix)/127.
    endif else begin
      print, 'Unknown sensor '+sensor
      stop
    endelse
  endelse

  xy = sqrt(x*x+y*y)

  convert = coeff(0) + xy*(coeff(1)+xy*(coeff(2)+xy*(coeff(3))))

  look_vector = dblarr(3)

  look_vector(0) = x*convert
  look_vector(1) = y*convert
  look_vector(2) = sqrt(1.-(look_vector(0)*look_vector(0) + $
                        look_vector(1)*look_vector(1)))

end

;-----------------------------------------------------------------------
PRO get_vector_pointing_vector, sensor, xpix, ypix, look_vector

  coeff = dblarr(4)

  if (sensor eq 'lis') then begin
    coeff = [1.4754537D, -.36224695D, -.088939824D, -.28203806D]
    x = (63.5-xpix)/127.
    y = (ypix-63.5)/127.
  endif else begin
    if (sensor eq 'otd') then begin
      coeff = [1.4037314D, .015948348D, -.86702375D, .24905820D]
      x=(xpix-63.5)/127.
      y=(63.5-ypix)/127.
    endif else begin
      print, 'Unknown sensor '+sensor
      stop
    endelse
  endelse

  xy = sqrt(x*x+y*y)

  convert = coeff(0) + xy*(coeff(1)+xy*(coeff(2)+xy*(coeff(3))))

  vsize = size(xpix)
  vlen = vsize(1)

  look_vector = dblarr(vlen,3)

  look_vector(*,0) = x*convert
  look_vector(*,1) = y*convert
  look_vector(*,2) = sqrt(1.-(look_vector(*,0)*look_vector(*,0) + $
                          look_vector(*,1)*look_vector(*,1)))

END

;-----------------------------------------------------------------------
PRO get_array_pointing_vector, sensor, array_look_vector

  array_look_vector = dblarr(128,128,3)

  coeff = dblarr(4)

  xpix = fltarr(128,128)
  ypix = fltarr(128,128)

  for i=0,127,1 do begin
    xpix(i,*) = float(i)
    ypix(*,i) = float(i)
  endfor

  if (sensor eq 'lis') then begin
    coeff = [1.4754537D, -.36224695D, -.088939824D, -.28203806D]
    x = (63.5-xpix)/127.
    y = (ypix-63.5)/127.
  endif else begin
    if (sensor eq 'otd') then begin
      coeff = [1.4037314D, .015948348D, -.86702375D, .24905820D]
      x=(xpix-63.5)/127.
      y=(63.5-ypix)/127.
    endif else begin
      print, 'Unknown sensor '+sensor
      stop
    endelse
  endelse

  xy = sqrt(x*x+y*y)

  convert = coeff(0) + xy*(coeff(1)+xy*(coeff(2)+xy*(coeff(3))))

  array_look_vector = dblarr(128,128,3)

  array_look_vector(*,*,0) = x*convert
  array_look_vector(*,*,1) = y*convert
  array_look_vector(*,*,2) =  $
    sqrt(1.-(array_look_vector(*,*,0)*array_look_vector(*,*,0) + $
         array_look_vector(*,*,1)*array_look_vector(*,*,1)))

end
;-----------------------------------------------------------------------
PRO get_earth_intersection_new, look_vector, time, location, return_code

  common datafiles, orbit, fname

  lookunitvec = dblarr(3)
  LookECR = dblarr(3)
  location = dblarr(2)

  axis_A = 6378137.0D
  axis_C = 6356752.31D
  axrat  = 0.99664714532D

  axis_SqA = axis_A*axis_A
  axis_SqB = axis_SqA
  axis_SqC = axis_C*axis_C

  tidx0=time-orbit.one_second(0).TAI93_time
  tidx1= long(time-orbit.one_second(0).TAI93_time+.5)
  tidx2= long(time-orbit.one_second(0).TAI93_time+.5)+1
  if (tidx2 gt orbit.orbit_summary.one_second_count) then $
    tidx2=long(time-orbit.one_second(0).TAI93_time+.5)-1

  p1x=orbit.one_second(tidx1).position_vector(0)
  p1y=orbit.one_second(tidx1).position_vector(1)
  p1z=orbit.one_second(tidx1).position_vector(2)
  p2x=orbit.one_second(tidx2).position_vector(0)
  p2y=orbit.one_second(tidx2).position_vector(1)
  p2z=orbit.one_second(tidx2).position_vector(2)

  x_pos_vect=(tidx0-tidx1)/(tidx2-tidx1)*(p2x-p1x)+p1x
  y_pos_vect=(tidx0-tidx1)/(tidx2-tidx1)*(p2y-p1y)+p1y
  z_pos_vect=(tidx0-tidx1)/(tidx2-tidx1)*(p2z-p1z)+p1z

  lookunitvec(0) = orbit.one_second(tidx1).transform_matrix(0)*look_vector(0)+ $
    orbit.one_second(tidx1).transform_matrix(1) * look_vector(1) + $
    orbit.one_second(tidx1).transform_matrix(2) * look_vector(2)
  lookunitvec(1) = orbit.one_second(tidx1).transform_matrix(3)*look_vector(0)+ $
    orbit.one_second(tidx1).transform_matrix(4) * look_vector(1) + $
    orbit.one_second(tidx1).transform_matrix(5) * look_vector(2)
  lookunitvec(2) = orbit.one_second(tidx1).transform_matrix(6)*look_vector(0)+ $
    orbit.one_second(tidx1).transform_matrix(7) * look_vector(1) + $
    orbit.one_second(tidx1).transform_matrix(8) * look_vector(2)

  normlookunitvec = sqrt(lookunitvec(0)*lookunitvec(0) + $
    lookunitvec(1)*lookunitvec(1) + lookunitvec(2)*lookunitvec(2))

  lookunitvec(0) = lookunitvec(0) / normlookunitvec
  lookunitvec(1) = lookunitvec(1) / normlookunitvec
  lookunitvec(2) = lookunitvec(2) / normlookunitvec

  aCoef = lookunitvec(0)*lookunitvec(0) / axis_SqA + $
          lookunitvec(1)*lookunitvec(1) / axis_SqB + $
          lookunitvec(2)*lookunitvec(2) / axis_SqC

; bCoef = 2.0D* $
;        (orbit.one_second(tidx1).position_vector(0)*lookunitvec(0)/axis_SqA+ $
;         orbit.one_second(tidx1).position_vector(1)*lookunitvec(1)/axis_SqB+ $
;         orbit.one_second(tidx1).position_vector(2)*lookunitvec(2)/axis_SqC)
  bCoef = 2.0D* $
         (x_pos_vect*lookunitvec(0)/axis_SqA+ $
          y_pos_vect*lookunitvec(1)/axis_SqB+ $
          z_pos_vect*lookunitvec(2)/axis_SqC)

; cCoef = -1.0D + $
;         orbit.one_second(tidx1).position_vector(0)* $
;         orbit.one_second(tidx1).position_vector(0)/axis_SqA + $
;         orbit.one_second(tidx1).position_vector(1)* $
;         orbit.one_second(tidx1).position_vector(1)/axis_SqB + $
;         orbit.one_second(tidx1).position_vector(2)* $
;         orbit.one_second(tidx1).position_vector(2) / axis_SqC
  cCoef = -1.0D + $
          x_pos_vect* x_pos_vect /axis_SqA + $
          y_pos_vect* y_pos_vect /axis_SqB + $
          z_pos_vect* z_pos_vect / axis_SqC

  radical = bCoef*bCoef - 4.0D*aCoef*cCoef

  if ((aCoef le 0.0) or (radical lt 0.0)) then begin
    slantRange = 0D
    location(0) = 90D
    location(1) = 90D
    return_code = -1
  endif else begin
    slantRange = (-1.0D*bCoef - sqrt(radical))/(2.0D*aCoef)
  ; LookECR(0) = orbit.one_second(tidx1).position_vector(0) + $
  ;              slantRange * lookunitvec(0)
  ; LookECR(1) = orbit.one_second(tidx1).position_vector(1) + $
  ;              slantRange * lookunitvec(1)
  ; LookECR(2) = orbit.one_second(tidx1).position_vector(2) + $
  ;              slantRange * lookunitvec(2)
    LookECR(0) = x_pos_vect + $
                 slantRange * lookunitvec(0)
    LookECR(1) = y_pos_vect + $
                 slantRange * lookunitvec(1)
    LookECR(2) = z_pos_vect + $
                 slantRange * lookunitvec(2)
    location(1) = 180.0D / double(!PI) * atan(LookECR(1),LookECR(0))
    location(0) = 180.0D / double(!PI) * atan(LookECR(2), $
                  axrat*axrat * sqrt(LookECR(0)*LookECR(0) + $
                  LookECR(1)*LookECR(1)))
    return_code = 0
  endelse

END

;-----------------------------------------------------------------------
PRO get_vector_earth_intersection, look_vector, time, vector_location, $
                                    return_code

  common datafiles, orbit, fname

  vsize = size(look_vector)
  vlen = vsize(1)

  lookunitvec = dblarr(vlen,3)
  LookECR = dblarr(vlen,3)
  vector_location = dblarr(vlen,2)
  tidx = lonarr(vlen)

  axis_A = 6378137.0D
  axis_C = 6356752.31D
  axrat  = 0.99664714532D

  axis_SqA = axis_A*axis_A
  axis_SqB = axis_SqA
  axis_SqC = axis_C*axis_C

  tidx = long(time-orbit.one_second(0).TAI93_time+.5)

  goodtimes = where(tidx ge 0 and $
                    tidx lt orbit.orbit_summary.one_second_count,goodct)
  if (goodct lt vlen) then begin
    badtimes = where(tidx lt 0 or tidx ge orbit.orbit_summary.one_second_count)
    vector_location(badtimes,0) = 90D
    vector_location(badtimes,1) = 90D
  endif
  if (goodct gt 0) then begin
    lookunitvec(goodtimes,0) = $
      orbit.one_second(tidx(goodtimes)).transform_matrix(0) * $
      look_vector(goodtimes,0) + $
      orbit.one_second(tidx(goodtimes)).transform_matrix(1) * $
      look_vector(goodtimes,1) + $
      orbit.one_second(tidx(goodtimes)).transform_matrix(2) * $
      look_vector(goodtimes,2)
    lookunitvec(goodtimes,1) = $
      orbit.one_second(tidx(goodtimes)).transform_matrix(3) * $
      look_vector(goodtimes,0) + $
      orbit.one_second(tidx(goodtimes)).transform_matrix(4) * $
      look_vector(goodtimes,1) + $
      orbit.one_second(tidx(goodtimes)).transform_matrix(5) * $
      look_vector(goodtimes,2)
    lookunitvec(goodtimes,2) = $
      orbit.one_second(tidx(goodtimes)).transform_matrix(6) * $
      look_vector(goodtimes,0) + $
      orbit.one_second(tidx(goodtimes)).transform_matrix(7) * $
      look_vector(goodtimes,1) + $
      orbit.one_second(tidx(goodtimes)).transform_matrix(8) * $
      look_vector(goodtimes,2)

    normlookunitvec = $
      sqrt(lookunitvec(goodtimes,0)*lookunitvec(goodtimes,0) + $
      lookunitvec(goodtimes,1)*lookunitvec(goodtimes,1) + $
      lookunitvec(goodtimes,2)*lookunitvec(goodtimes,2))

    lookunitvec(goodtimes,0) = lookunitvec(goodtimes,0) / normlookunitvec
    lookunitvec(goodtimes,1) = lookunitvec(goodtimes,1) / normlookunitvec
    lookunitvec(goodtimes,2) = lookunitvec(goodtimes,2) / normlookunitvec

    aCoef = lookunitvec(goodtimes,0)*lookunitvec(goodtimes,0) / axis_SqA + $
            lookunitvec(goodtimes,1)*lookunitvec(goodtimes,1) / axis_SqB + $
            lookunitvec(goodtimes,2)*lookunitvec(goodtimes,2) / axis_SqC

    bCoef = 2.0D * $
           (orbit.one_second(tidx(goodtimes)).position_vector(0) * $
            lookunitvec(goodtimes,0) / axis_SqA + $
            orbit.one_second(tidx(goodtimes)).position_vector(1) * $
            lookunitvec(goodtimes,1) / axis_SqB + $
            orbit.one_second(tidx(goodtimes)).position_vector(2) * $
            lookunitvec(goodtimes,2) / axis_SqC)

    cCoef = -1.0D + $
            orbit.one_second(tidx(goodtimes)).position_vector(0) * $
            orbit.one_second(tidx(goodtimes)).position_vector(0)/axis_SqA + $
            orbit.one_second(tidx(goodtimes)).position_vector(1)* $
            orbit.one_second(tidx(goodtimes)).position_vector(1)/axis_SqB + $
            orbit.one_second(tidx(goodtimes)).position_vector(2)* $
            orbit.one_second(tidx(goodtimes)).position_vector(2)/axis_SqC

    radical = bCoef*bCoef - 4.0D*aCoef*cCoef

    badgeo = where(aCoef le 0.0 or radical lt 0.0,badgct)
    goodgeo = where(aCoef gt 0.0 and radical ge 0.0,goodgct)

    if (goodgct lt goodct) then begin
      slantRange = 0D
      vector_location(goodtimes(badgeo),0) = 90D
      vector_location(goodtimes(badgeo),1) = 90D
      return_code = -1
    endif
    if (goodgct gt 0) then begin
      slantRange = (-1.0D*bCoef - sqrt(radical))/(2.0D*aCoef)
      LookECR(goodtimes(goodgeo),0) = $
        orbit.one_second(tidx(goodtimes(goodgeo))).position_vector(0) + $
        slantRange * lookunitvec(goodtimes(goodgeo),0)
      LookECR(goodtimes(goodgeo),1) = $
        orbit.one_second(tidx(goodtimes(goodgeo))).position_vector(1) + $
        slantRange * lookunitvec(goodtimes(goodgeo),1)
      LookECR(goodtimes(goodgeo),2) = $
        orbit.one_second(tidx(goodtimes(goodgeo))).position_vector(2) + $
        slantRange * lookunitvec(goodtimes(goodgeo),2)
      vector_location(goodtimes(goodgeo),1) = 180.0D / double(!PI) * $
        atan(LookECR(goodtimes(goodgeo),1),LookECR(goodtimes(goodgeo),0))
      vector_location(goodtimes(goodgeo),0) = 180.0D / double(!PI) * $
        atan(LookECR(goodtimes(goodgeo),2), axrat*axrat * $
        sqrt(LookECR(goodtimes(goodgeo),0)*LookECR(goodtimes(goodgeo),0) + $
        LookECR(goodtimes(goodgeo),1)*LookECR(goodtimes(goodgeo),1)))
      return_code = 0

    endif
  endif
END
;-----------------------------------------------------------------------
PRO get_vector_earth_intersection_new, look_vector, time, vector_location, $
                                    return_code

  common datafiles, orbit, fname

  vsize = size(look_vector)
  vlen = vsize(1)

  lookunitvec = dblarr(vlen,3)
  LookECR = dblarr(vlen,3)
  vector_location = dblarr(vlen,2)
  tidx = lonarr(vlen)

  axis_A = 6378137.0D
  axis_C = 6356752.31D
  axrat  = 0.99664714532D

  axis_SqA = axis_A*axis_A
  axis_SqB = axis_SqA
  axis_SqC = axis_C*axis_C

  tidx0=time-orbit.one_second(0).TAI93_time
  tidx1= long(time-orbit.one_second(0).TAI93_time+.5)
  tidx2= long(time-orbit.one_second(0).TAI93_time+.5)+1
  index=where(tidx2 gt orbit.orbit_summary.one_second_count)
  if (index(0) ne -1) then $
    tidx2(index)=long(time(index)-orbit.one_second(0).TAI93_time+.5)-1

  p1x=orbit.one_second(tidx1).position_vector(0)
  p1y=orbit.one_second(tidx1).position_vector(1)
  p1z=orbit.one_second(tidx1).position_vector(2)
  p2x=orbit.one_second(tidx2).position_vector(0)
  p2y=orbit.one_second(tidx2).position_vector(1)
  p2z=orbit.one_second(tidx2).position_vector(2)

  x_pos_vect=(tidx0-tidx1)/(tidx2-tidx1)*(p2x-p1x)+p1x
  y_pos_vect=(tidx0-tidx1)/(tidx2-tidx1)*(p2y-p1y)+p1y
  z_pos_vect=(tidx0-tidx1)/(tidx2-tidx1)*(p2z-p1z)+p1z

  goodtimes = where(tidx1 ge 0 and $
                    tidx1 lt orbit.orbit_summary.one_second_count,goodct)
  if (goodct lt vlen) then begin
    badtimes = where(tidx1 lt 0 or tidx1 ge orbit.orbit_summary.one_second_count)
    vector_location(badtimes,0) = 90D
    vector_location(badtimes,1) = 90D
  endif
  if (goodct gt 0) then begin
    lookunitvec(goodtimes,0) = $
      orbit.one_second(tidx1(goodtimes)).transform_matrix(0) * $
      look_vector(goodtimes,0) + $
      orbit.one_second(tidx1(goodtimes)).transform_matrix(1) * $
      look_vector(goodtimes,1) + $
      orbit.one_second(tidx1(goodtimes)).transform_matrix(2) * $
      look_vector(goodtimes,2)
    lookunitvec(goodtimes,1) = $
      orbit.one_second(tidx1(goodtimes)).transform_matrix(3) * $
      look_vector(goodtimes,0) + $
      orbit.one_second(tidx1(goodtimes)).transform_matrix(4) * $
      look_vector(goodtimes,1) + $
      orbit.one_second(tidx1(goodtimes)).transform_matrix(5) * $
      look_vector(goodtimes,2)
    lookunitvec(goodtimes,2) = $
      orbit.one_second(tidx1(goodtimes)).transform_matrix(6) * $
      look_vector(goodtimes,0) + $
      orbit.one_second(tidx1(goodtimes)).transform_matrix(7) * $
      look_vector(goodtimes,1) + $
      orbit.one_second(tidx1(goodtimes)).transform_matrix(8) * $
      look_vector(goodtimes,2)

    normlookunitvec = $
      sqrt(lookunitvec(goodtimes,0)*lookunitvec(goodtimes,0) + $
      lookunitvec(goodtimes,1)*lookunitvec(goodtimes,1) + $
      lookunitvec(goodtimes,2)*lookunitvec(goodtimes,2))

    lookunitvec(goodtimes,0) = lookunitvec(goodtimes,0) / normlookunitvec
    lookunitvec(goodtimes,1) = lookunitvec(goodtimes,1) / normlookunitvec
    lookunitvec(goodtimes,2) = lookunitvec(goodtimes,2) / normlookunitvec

    aCoef = lookunitvec(goodtimes,0)*lookunitvec(goodtimes,0) / axis_SqA + $
            lookunitvec(goodtimes,1)*lookunitvec(goodtimes,1) / axis_SqB + $
            lookunitvec(goodtimes,2)*lookunitvec(goodtimes,2) / axis_SqC

    bCoef = 2.0D * $
           (x_pos_vect(goodtimes) * $
            lookunitvec(goodtimes,0) / axis_SqA + $
            y_pos_vect(goodtimes) * $
            lookunitvec(goodtimes,1) / axis_SqB + $
            z_pos_vect(goodtimes) * $
            lookunitvec(goodtimes,2) / axis_SqC)

    cCoef = -1.0D + $
            x_pos_vect(goodtimes) * x_pos_vect(goodtimes) /axis_SqA + $
            y_pos_vect(goodtimes) * y_pos_vect(goodtimes) /axis_SqB + $
            z_pos_vect(goodtimes) * z_pos_vect(goodtimes) /axis_SqC

    radical = bCoef*bCoef - 4.0D*aCoef*cCoef

    badgeo = where(aCoef le 0.0 or radical lt 0.0,badgct)
    goodgeo = where(aCoef gt 0.0 and radical ge 0.0,goodgct)

    if (goodgct lt goodct) then begin
      slantRange = 0D
      vector_location(goodtimes(badgeo),0) = 90D
      vector_location(goodtimes(badgeo),1) = 90D
      return_code = -1
    endif
    if (goodgct gt 0) then begin
      slantRange = (-1.0D*bCoef - sqrt(radical))/(2.0D*aCoef)
      LookECR(goodtimes(goodgeo),0) = $
        x_pos_vect(goodtimes(goodgeo)) + $
        slantRange * lookunitvec(goodtimes(goodgeo),0)
      LookECR(goodtimes(goodgeo),1) = $
        y_pos_vect(goodtimes(goodgeo)) + $
        slantRange * lookunitvec(goodtimes(goodgeo),1)
      LookECR(goodtimes(goodgeo),2) = $
        z_pos_vect(goodtimes(goodgeo)) + $
        slantRange * lookunitvec(goodtimes(goodgeo),2)
      vector_location(goodtimes(goodgeo),1) = 180.0D / double(!PI) * $
        atan(LookECR(goodtimes(goodgeo),1),LookECR(goodtimes(goodgeo),0))
      vector_location(goodtimes(goodgeo),0) = 180.0D / double(!PI) * $
        atan(LookECR(goodtimes(goodgeo),2), axrat*axrat * $
        sqrt(LookECR(goodtimes(goodgeo),0)*LookECR(goodtimes(goodgeo),0) + $
        LookECR(goodtimes(goodgeo),1)*LookECR(goodtimes(goodgeo),1)))
      return_code = 0

    endif
  endif
END
;-----------------------------------------------------------------------
PRO get_array_earth_intersection, look_vector, time, array_location, $
                                  return_code

  common datafiles, orbit, fname

  lookunitvec = dblarr(128,128,3)
  LookECR = dblarr(128,128,3)
  array_location = dblarr(128,128,2)

  axis_A = 6378137.0D
  axis_C = 6356752.31D
  axrat  = 0.99664714532D

  axis_SqA = axis_A*axis_A
  axis_SqB = axis_SqA
  axis_SqC = axis_C*axis_C

  tidx = long(time-orbit.one_second(0).TAI93_time)

  if (tidx lt 0 or tidx ge orbit.orbit_summary.one_second_count) then begin
    slantRange = 0D
    array_location(0) = 90D
    array_location(1) = 90D
    return_code = -1
  endif else begin
    lookunitvec(*,*,0) = $
      orbit.one_second(tidx).transform_matrix(0) * look_vector(*,*,0) + $
      orbit.one_second(tidx).transform_matrix(1) * look_vector(*,*,1) + $
      orbit.one_second(tidx).transform_matrix(2) * look_vector(*,*,2)
    lookunitvec(*,*,1) = $
      orbit.one_second(tidx).transform_matrix(3) * look_vector(*,*,0) + $
      orbit.one_second(tidx).transform_matrix(4) * look_vector(*,*,1) + $
      orbit.one_second(tidx).transform_matrix(5) * look_vector(*,*,2)
    lookunitvec(*,*,2) = $
      orbit.one_second(tidx).transform_matrix(6) * look_vector(*,*,0) + $
      orbit.one_second(tidx).transform_matrix(7) * look_vector(*,*,1) + $
      orbit.one_second(tidx).transform_matrix(8) * look_vector(*,*,2)

    normlookunitvec = $
      sqrt(lookunitvec(*,*,0)*lookunitvec(*,*,0) + $
           lookunitvec(*,*,1)*lookunitvec(*,*,1) + $
           lookunitvec(*,*,2)*lookunitvec(*,*,2))

    lookunitvec(*,*,0) = lookunitvec(*,*,0) / normlookunitvec
    lookunitvec(*,*,1) = lookunitvec(*,*,1) / normlookunitvec
    lookunitvec(*,*,2) = lookunitvec(*,*,2) / normlookunitvec

    aCoef = lookunitvec(*,*,0)*lookunitvec(*,*,0) / axis_SqA + $
            lookunitvec(*,*,1)*lookunitvec(*,*,1) / axis_SqB + $
            lookunitvec(*,*,2)*lookunitvec(*,*,2) / axis_SqC

    bCoef = 2.0D * $
      (orbit.one_second(tidx).position_vector(0)*lookunitvec(*,*,0)/axis_SqA+ $
      orbit.one_second(tidx).position_vector(1)*lookunitvec(*,*,1)/axis_SqB+ $
      orbit.one_second(tidx).position_vector(2)*lookunitvec(*,*,2)/axis_SqC)

    cCoef = -1.0D + $
      orbit.one_second(tidx).position_vector(0)* $
      orbit.one_second(tidx).position_vector(0) / axis_SqA + $
      orbit.one_second(tidx).position_vector(1)* $
      orbit.one_second(tidx).position_vector(1) / axis_SqB + $
      orbit.one_second(tidx).position_vector(2)* $
      orbit.one_second(tidx).position_vector(2) / axis_SqC

    radical = bCoef*bCoef - 4.0D*aCoef*cCoef

    if ((min(aCoef) le 0.0) or (min(radical) lt 0.0)) then begin
      slantRange = 0D
      array_location(0) = 90D
      array_location(1) = 90D
      return_code = -1
    endif else begin
      slantRange = (-1.0D*bCoef - sqrt(radical))/(2.0D*aCoef)
      LookECR(*,*,0) = orbit.one_second(tidx).position_vector(0) + $
                       slantRange * lookunitvec(*,*,0)
      LookECR(*,*,1) = orbit.one_second(tidx).position_vector(1) + $
                       slantRange * lookunitvec(*,*,1)
      LookECR(*,*,2) = orbit.one_second(tidx).position_vector(2) + $
                       slantRange * lookunitvec(*,*,2)
      array_location(*,*,1) =180.0D/double(!PI)*atan(LookECR(*,*,1),$
                             LookECR(*,*,0))
      array_location(*,*,0) = 180.0D/double(!PI)*atan(LookECR(*,*,2), $
        axrat*axrat * sqrt(LookECR(*,*,0)*LookECR(*,*,0) + $
        LookECR(*,*,1)*LookECR(*,*,1)))

      return_code = 0
    endelse
  endelse
END

;-----------------------------------------------------------------------
PRO get_nadir_location, time, location

  common datafiles, orbit, fname

  location = dblarr(2)

  axrat = .99664714532D

  tidx = long(time-orbit.one_second(0).TAI93_time)

  location(1) = 180.0D / double(!PI) * $
                atan(orbit.one_second(tidx).position_vector(1), $
                orbit.one_second(tidx).position_vector(0))
  location(0) = 180.0D / double(!PI) * $
                atan(orbit.one_second(tidx).position_vector(2), $
                axrat*axrat*sqrt(orbit.one_second(tidx).position_vector(0)* $
                orbit.one_second(tidx).position_vector(0) + $ 
                orbit.one_second(tidx).position_vector(1)*$
                orbit.one_second(tidx).position_vector(1)))

END
;-----------------------------------------------------------------------
PRO get_array_earth_intersection_new, look_vector, time, array_location, $
                                  return_code

  common datafiles, orbit, fname

  lookunitvec = dblarr(128,128,3)
  LookECR = dblarr(128,128,3)
  array_location = dblarr(128,128,2)

  axis_A = 6378137.0D
  axis_C = 6356752.31D
  axrat  = 0.99664714532D

  axis_SqA = axis_A*axis_A
  axis_SqB = axis_SqA
  axis_SqC = axis_C*axis_C

  tidx0=time-orbit.one_second(0).TAI93_time
  tidx1= long(time-orbit.one_second(0).TAI93_time+.5)
  tidx2= long(time-orbit.one_second(0).TAI93_time+.5)+1
  index=where(tidx2 gt orbit.orbit_summary.one_second_count)
  if (index(0) ne -1) then $
    tidx2(index)=long(time(index)-orbit.one_second(0).TAI93_time+.5)-1

  p1x=orbit.one_second(tidx1).position_vector(0)
  p1y=orbit.one_second(tidx1).position_vector(1)
  p1z=orbit.one_second(tidx1).position_vector(2)
  p2x=orbit.one_second(tidx2).position_vector(0)
  p2y=orbit.one_second(tidx2).position_vector(1)
  p2z=orbit.one_second(tidx2).position_vector(2)

  x_pos_vect=(tidx0-tidx1)/(tidx2-tidx1)*(p2x-p1x)+p1x
  y_pos_vect=(tidx0-tidx1)/(tidx2-tidx1)*(p2y-p1y)+p1y
  z_pos_vect=(tidx0-tidx1)/(tidx2-tidx1)*(p2z-p1z)+p1z

  if (tidx1 lt 0 or tidx1 ge orbit.orbit_summary.one_second_count) then begin
    slantRange = 0D
    array_location(0) = 90D
    array_location(1) = 90D
    return_code = -1
  endif else begin
    lookunitvec(*,*,0) = $
      orbit.one_second(tidx1).transform_matrix(0) * look_vector(*,*,0) + $
      orbit.one_second(tidx1).transform_matrix(1) * look_vector(*,*,1) + $
      orbit.one_second(tidx1).transform_matrix(2) * look_vector(*,*,2)
    lookunitvec(*,*,1) = $
      orbit.one_second(tidx1).transform_matrix(3) * look_vector(*,*,0) + $
      orbit.one_second(tidx1).transform_matrix(4) * look_vector(*,*,1) + $
      orbit.one_second(tidx1).transform_matrix(5) * look_vector(*,*,2)
    lookunitvec(*,*,2) = $
      orbit.one_second(tidx1).transform_matrix(6) * look_vector(*,*,0) + $
      orbit.one_second(tidx1).transform_matrix(7) * look_vector(*,*,1) + $
      orbit.one_second(tidx1).transform_matrix(8) * look_vector(*,*,2)

    normlookunitvec = $
      sqrt(lookunitvec(*,*,0)*lookunitvec(*,*,0) + $
           lookunitvec(*,*,1)*lookunitvec(*,*,1) + $
           lookunitvec(*,*,2)*lookunitvec(*,*,2))

    lookunitvec(*,*,0) = lookunitvec(*,*,0) / normlookunitvec
    lookunitvec(*,*,1) = lookunitvec(*,*,1) / normlookunitvec
    lookunitvec(*,*,2) = lookunitvec(*,*,2) / normlookunitvec

    aCoef = lookunitvec(*,*,0)*lookunitvec(*,*,0) / axis_SqA + $
            lookunitvec(*,*,1)*lookunitvec(*,*,1) / axis_SqB + $
            lookunitvec(*,*,2)*lookunitvec(*,*,2) / axis_SqC

    bCoef = 2.0D * $
      (x_pos_vect*lookunitvec(*,*,0)/axis_SqA+ $
       y_pos_vect*lookunitvec(*,*,1)/axis_SqB+ $
       z_pos_vect*lookunitvec(*,*,2)/axis_SqC)

    cCoef = -1.0D + $
      x_pos_vect*x_pos_vect/axis_SqA + $
      y_pos_vect*y_pos_vect/axis_SqB + $
      z_pos_vect*z_pos_vect/axis_SqC

    radical = bCoef*bCoef - 4.0D*aCoef*cCoef

    if ((min(aCoef) le 0.0) or (min(radical) lt 0.0)) then begin
      slantRange = 0D
      array_location(0) = 90D
      array_location(1) = 90D
      return_code = -1
    endif else begin
      slantRange = (-1.0D*bCoef - sqrt(radical))/(2.0D*aCoef)
      LookECR(*,*,0) = x_pos_vect + $
                       slantRange * lookunitvec(*,*,0)
      LookECR(*,*,1) = y_pos_vect + $
                       slantRange * lookunitvec(*,*,1)
      LookECR(*,*,2) = z_pos_vect + $
                       slantRange * lookunitvec(*,*,2)
      array_location(*,*,1) =180.0D/double(!PI)*atan(LookECR(*,*,1),$
                             LookECR(*,*,0))
      array_location(*,*,0) = 180.0D/double(!PI)*atan(LookECR(*,*,2), $
        axrat*axrat * sqrt(LookECR(*,*,0)*LookECR(*,*,0) + $
        LookECR(*,*,1)*LookECR(*,*,1)))

      return_code = 0
    endelse
  endelse
END

;-----------------------------------------------------------------------
PRO get_nadir_location, time, location

  common datafiles, orbit, fname

  location = dblarr(2)

  axrat = .99664714532D

  tidx = long(time-orbit.one_second(0).TAI93_time)

  location(1) = 180.0D / double(!PI) * $
                atan(orbit.one_second(tidx).position_vector(1), $
                orbit.one_second(tidx).position_vector(0))
  location(0) = 180.0D / double(!PI) * $
                atan(orbit.one_second(tidx).position_vector(2), $
                axrat*axrat*sqrt(orbit.one_second(tidx).position_vector(0)* $
                orbit.one_second(tidx).position_vector(0) + $ 
                orbit.one_second(tidx).position_vector(1)*$
                orbit.one_second(tidx).position_vector(1)))

END
;-----------------------------------------------------------------------
PRO get_nadir_location_new, time, location

  common datafiles, orbit, fname

  location = dblarr(2)

  axrat = .99664714532D

  tidx0=time-orbit.one_second(0).TAI93_time
  tidx1= long(time-orbit.one_second(0).TAI93_time+.5)
  tidx2= long(time-orbit.one_second(0).TAI93_time+.5)+1
  index=where(tidx2 gt orbit.orbit_summary.one_second_count)
  if (index(0) ne -1) then $
    tidx2(index)=long(time(index)-orbit.one_second(0).TAI93_time+.5)-1

  p1x=orbit.one_second(tidx1).position_vector(0)
  p1y=orbit.one_second(tidx1).position_vector(1)
  p1z=orbit.one_second(tidx1).position_vector(2)
  p2x=orbit.one_second(tidx2).position_vector(0)
  p2y=orbit.one_second(tidx2).position_vector(1)
  p2z=orbit.one_second(tidx2).position_vector(2)

  x_pos_vect=(tidx0-tidx1)/(tidx2-tidx1)*(p2x-p1x)+p1x
  y_pos_vect=(tidx0-tidx1)/(tidx2-tidx1)*(p2y-p1y)+p1y
  z_pos_vect=(tidx0-tidx1)/(tidx2-tidx1)*(p2z-p1z)+p1z

  location(1) = 180.0D / double(!PI) * $
                atan(y_pos_vect,x_pos_vect)
  location(0) = 180.0D / double(!PI) * atan(z_pos_vect,axrat*axrat* $
                sqrt(x_pos_vect*x_pos_vect + y_pos_vect*y_pos_vect))

END

PRO LOAD_MY_COLOR_TABLES
    r=bytarr(256)
    g=bytarr(256)
    b=bytarr(256)
    r(0:15)=[255,000,255,128,000,000,000,000,255,255,255,101,159,209,102,000]
    g(0:15)=[255,000,000,000,216,128,194,000,255,122,000,097,109,163,181,106]
    b(0:15)=[255,000,000,000,000,000,255,255,000,000,255,198,068,118,207,129]
    r(16:95)=[193,176,160,144,128,112,096,080,064,048,032,016,000,000,000,000, $
              000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000, $
              000,019,039,058,078,098,117,137,156,176,196,215,235,255,255,255, $
              255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255, $
              250,246,241,237,232,228,223,219,215,210,206,201,197,192,188,184]
    g(16:95)=[146,150,154,159,163,167,172,176,180,185,189,193,198,189,180,172, $
              163,155,146,137,129,120,112,103,095,101,108,115,122,129,136,143, $
              150,158,166,174,182,190,198,206,214,222,230,238,246,255,241,227, $
              213,199,185,172,158,144,130,116,103,098,094,089,085,080,076,071, $
              066,062,057,053,048,044,039,035,031,026,022,017,013,008,004,000]
    b(16:95)=[255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255, $
              255,255,255,255,255,255,255,255,255,223,191,159,127,095,063,031, $
              000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000, $
              000,000,000,000,000,000,000,000,000,003,007,011,015,019,022,026, $
              029,032,036,039,042,046,049,053,056,059,063,066,069,073,076,080]
    r(96:96+15)=  [000,017,034,051,068,085,102,119,136,$
                   153,170,187,204,221,238,255]
    g(96:96+15)=  [177,182,187,192,197,203,208,213,218,$
                   223,229,234,239,244,249,255]
    b(96:96+15)=  [208,211,214,217,220,223,226,229,233,$
                   236,239,242,245,248,251,255]
    r(112:112+15)=[209,212,215,218,221,224,227,230,233,$
                   236,239,242,245,248,251,255]
    g(112:112+15)=[164,170,176,182,188,194,200,206,212,$
                   218,224,230,236,242,248,255]
    b(112:112+15)=[124,132,141,150,158,167,176,185,193,$
                   202,211,220,228,237,246,255]

  ;  r(128:255)=bindgen(128)*2
  ;  g(128:255)=bindgen(128)*2
  ;  b(128:255)=bindgen(128)*2
	
	; Modify for platforms that can only squeeze out 220 colors
	
    r(128:219)=fix(bindgen(92)*1.39*2.)
    g(128:219)=fix(bindgen(92)*1.39*2.)
    b(128:219)=fix(bindgen(92)*1.39*2.)
    r(220:255) = 255
    g(220:255) = 255
    b(220:255) = 255

    r(32:63)=[083,087,092,096,101,106,110,115,119,124,129,133,138,142,$
              147,152,156,161,166,170,175,179,184,189,193,198,202,207,$
              212,216,221,226]
    g(32:63)=[054,058,062,066,070,074,078,082,086,090,094,098,102,106,$
              110,114,118,122,126,130,134,138,142,146,150,154,158,162,$
              166,170,174,179]
    b(32:63)=[028,031,035,039,042,046,050,053,057,061,064,068,072,075,$
              079,083,086,090,094,097,101,105,108,112,116,119,123,127,$
              130,134,138,142]
    r(64:95)=[000,000,000,000,000,000,000,000,000,000,000,000,000,000,$
              000,000,000,000,000,000,000,000,000,000,000,000,000,000,$
              000,000,000,000]
    g(64:95)=[051,056,061,066,071,076,081,086,091,096,101,106,111,116,$
              121,126,132,137,142,147,152,157,162,167,172,177,182,187,$
              192,197,202,208]
    b(64:95)=[084,088,093,098,103,108,113,117,122,127,132,137,142,146,$
              151,156,161,166,171,175,180,185,190,195,200,204,209,214,$
              219,224,229,234]
    r(16)=112 & g(16)=112 & b(16)=0
    r(17)=20  & g(17)=20  & b(17)=20
    tvlct,r,g,b

END

;-----------------------------------------------------------------------
PRO FLOODFILL,x,y,old_val,new_val
  COMMON datablock, array,xsize,ysize

  if (array(x,y) eq old_val) then begin
    array(x mod xsize,y mod ysize)=new_val
    FLOODFILL,(x) ,(y-1+ysize) mod ysize,old_val,new_val
    FLOODFILL,(x) ,(y+1) mod ysize,old_val,new_val
    FLOODFILL,(x-1+xsize) mod xsize,(y),old_val,new_val
    FLOODFILL,(x+1) mod xsize,(y),old_val,new_val
  endif
; if (array(0,0) ne 0) then print,'ERROR'
END

;-----------------------------------------------------------------------
PRO prior_observation_path, new_location, time_offset_index

  common datafiles, orbit, fname
  this_sensor='lis'
  px=[5,122,122,5,5]
  py=[5,5,122,122,5]
    px=[3,125,125,3,3]
    py=[3,3,125,125,3]
  for j=time_offset_index,$
        time_offset_index+orbit.orbit_summary.one_second_count,10 do begin
    get_vector_pointing_vector,this_sensor,px,py,look_vector
    get_vector_earth_intersection,look_vector, $
     orbit.one_second(j-time_offset_index).TAI93_time*(intarr(5)+1),$
       location,return_code
    if( j eq time_offset_index) then begin
      new_location=location
    endif else begin
      new_location=[new_location,location]
    endelse
  endfor

END
;-----------------------------------------------------------------------
PRO set_map_size, map_lat_ctr, map_lon_ctr, dy , dx , time
  common datafiles, orbit, fname
  common sensor_read, this_sensor, file_format
  get_nadir_location_new,time, location
  map_lat_ctr=location[0]
  map_lon_ctr=location[1]
  dy=6
  if (this_sensor eq 'otd') then begin
  	dy = dy * 1.5
  endif
  tmp=[abs(cos((map_lat_ctr-dy/2.)*!dtor)),abs(cos((map_lat_ctr+dy/2.)*!dtor))]
  dx=dy/max( tmp)
END

function get_max_val, image_numbers
  common datafiles, orbit, fname
  common bg_info, fnamestub, candobgs, bgname, bgpathtouse

  ins = size(image_numbers)
  case ins[0] of
  	0: begin
  		numim = 1
  	   end
  	1: begin
  		numim = ins[1]
  	   end
  endcase
  firstimage = image_numbers[0]
  lastimage = image_numbers[numim-1]

  
	bfname = bgname
  
     fileid=HDF_OPEN(bfname,/read)
 ;printf,uid," FILENAME       : ",bfname

  sd_id=HDF_SD_START(bfname,/read)
  HDF_SD_FILEINFO,sd_id,nmfsds,nglobatts

  for i=0,nmfsds-1 do begin
    sds_id=HDF_SD_SELECT(sd_id,i)
    HDF_SD_GETINFO,sds_id,name=n,ndims=r,type=t,natts=nats,$
         hdf_type=h,unit=u

    if r gt 0 then HDF_SD_GETINFO,sds_id,dims=dims else dims=0

    for j=0,nats-1 do begin
      HDF_SD_ATTRINFO,sds_id,j,name=n,type=t,count=c,data=d,$
            hdf_type=h
    endfor

    inbg = intarr(128,128,numim)
    temp = intarr(128,128)

    for kk=firstimage,lastimage,1 do begin
	hdf_sd_getdata,sds_id,temp,start=[0,0,kk],count=[128,128,1]
	inbg(*,*,kk-firstimage) = temp
    endfor

	;    hdf_sd_getdata,sds_id,inbg,start=[0,0,firstimage],$
       ;count=[128,128,numim]

    HDF_SD_ENDACCESS,sds_id
  endfor

  for i=0,nglobatts-1 do begin
    HDF_SD_ATTRINFO,sd_id,i,name=n,type=t,count=c,data=d
  endfor

  HDF_SD_END,sd_id

  val = max(inbg[*,*,*])
  inbg = 0
  return,val
end


;-----------------------------------------------------------------------
PRO read_lis_background, image_number, image, max_val=max_val

  common datafiles, orbit, fname
  common bg_info, fnamestub, candobgs, bgname, bgpathtouse

;  bfname= strmid(fname,0,strpos(fname,'TRMM_LIS_'))+'TRMM_LIS_BG'+ $
;    strmid(fname,strpos(fname,'TRMM_LIS_')+11,999)
  ; print,bfname
;  uid=-1
;  if HDF_ishdf(bfname) ne 1 then begin
;    message,"File "+bfname+" is not an HDF file."
;    return
;  endif


	bfname = bgname
  
     fileid=HDF_OPEN(bfname,/read)
 ;printf,uid," FILENAME       : ",bfname

  sd_id=HDF_SD_START(bfname,/read)
  HDF_SD_FILEINFO,sd_id,nmfsds,nglobatts

  for i=0,nmfsds-1 do begin
    sds_id=HDF_SD_SELECT(sd_id,i)
    HDF_SD_GETINFO,sds_id,name=n,ndims=r,type=t,natts=nats,$
         hdf_type=h,unit=u

    if r gt 0 then HDF_SD_GETINFO,sds_id,dims=dims else dims=0

    for j=0,nats-1 do begin
      HDF_SD_ATTRINFO,sds_id,j,name=n,type=t,count=c,data=d,$
            hdf_type=h
    endfor

    inbg = intarr(128,128)
    hdf_sd_getdata,sds_id,inbg,start=[0,0,image_number],$
       count=[128,128,1]

    HDF_SD_ENDACCESS,sds_id
  endfor

  for i=0,nglobatts-1 do begin
    HDF_SD_ATTRINFO,sd_id,i,name=n,type=t,count=c,data=d
  endfor

  HDF_SD_END,sd_id

  bg_image=intarr(128,128)
  tmp=fltarr(128,128)

  bg_image=inbg
  raw_image=bg_image
  
  if keyword_set(max_val) then begin
  	mv = max_val
  endif else begin
  	mv =float(max(bg_image))
  endelse
  
;  if (max_val lt 1000) then begin
;  	max_val=1000
;  endif 
;  index=where(bg_image gt max_val)
;  if(index(0) ne -1) then bg_image(index)=max_val
 ; bg_image=fix(bg_image*127./max_val+128.)
  bg_image=fix(bg_image*74./mv+146.)

  image=bg_image

end

;-----------------------------------------------------------------------
PRO plot_backgrounds,xsize,ysize,fov_fill_color,time,symbol_size
  common datafiles, orbit, fname
  common sensor_read, this_sensor, file_format
	common otd_boom, boom

  ;this_sensor='lis'
case this_sensor of
	'lis':begin
			sensor_scale = 1.0
	      end
	'otd':begin
		  	sensor_scale = 2.0
		end
endcase
  image=tvrd(0,0,xsize,ysize)
  fill_index=where(image eq fov_fill_color)
  if (fill_index(0) eq -1) then return
  mask=image*0
  mask(fill_index)=1
  result=convert_coord(fill_index mod xsize, fill_index/xsize, /device,/to_data)
  index=where(abs(time-orbit.point.bg_summary.tai93_time) le 95)
  if (index(0) eq -1) then return
 ;index2=reverse(sort(abs(time-orbit.point.bg_summary(index).tai93_time)))
  index2=(sort(abs(time-orbit.point.bg_summary(index).tai93_time)))

  usersym,sin(findgen(21)/10.*!pi),cos(findgen(21)/10.*!pi),/fill
  for i=0,n_elements(index)-1 do begin
    image_number=index(index2(i))
	; print,'about to read image number ',i
    read_lis_background, image_number, image
    bg_time=orbit.point.bg_summary(image_number).tai93_time
    get_array_pointing_vector, this_sensor, look_vector
    get_array_earth_intersection_new, look_vector, bg_time*1.0D0+0*3.00D0, location, $
                                  return_code
    pix_result=convert_coord(location(*,*,1),location(*,*,0),/to_device,/data)
  ;  for j=0L,128*128-1 do $
   ;   if (pix_result(0,j) gt 0 and pix_result(0,j) lt xsize and $
   ;       pix_result(1,j) gt 0 and pix_result(1,j) lt ysize) then $
   ;   if (mask(pix_result(0,j),pix_result(1,j)) eq 1) then begin
   ;     plots,location(j mod 128,j/128,1),location(j mod 128,j/128,0), $
   ;     psym=8,color=image(j mod 128,j/128),$
   ;     symsize=(.1+max([xsize,ysize])*.0003)*symbol_size*sensor_scale
    ;    mask(pix_result(0,j),pix_result(1,j))=2
   ;   endif

    ssz = (.1+max([xsize,ysize])*.0003)*symbol_size*sensor_scale
    xvals = fltarr(16384) & yvals = xvals
    xvals(*) = pix_result(0,*) & yvals(*) = pix_result(1,*) & idxs = long(yvals)*320L+long(xvals)
    maskvals = mask(idxs)
    tocheck1 = ((xvals gt 0) and (xvals lt xsize) and (yvals gt 0) and (yvals lt ysize) and (maskvals eq 1))
    ;print,min(tocheck1),max(tocheck1)
    touse1 = where(tocheck1 eq 1, touse1ct)
    ;tocheck2 = (maskvals eq 1)
    ;touse2 = where(tocheck2 eq 1, touse2ct)
    ;help,pix_result,xvals,yvals,idxs,mask,maskvals,tocheck1,touse1,tocheck2,touse2
    if (touse1ct gt 0) then begin
    	mask(idxs(touse1)) = 2
		xvals2 = touse1 mod 128
		yvals2 = touse1/128
		xvals = 0 & yvals = 0 & idxs = 0 & maskvals = 0 & tocheck1 = 0 & touse1 = 0
		colidx = yvals2*128 + xvals2
		xvals3 = location(128*128 + colidx)
		yvals3 = location(colidx)
		cols3 = image(colidx)
		if (this_sensor eq 'otd') then begin
			noboom = where(boom(colidx) ne 0,noboomct)
			if (noboomct gt 0) then begin
				xvals2 = 0 & yvals2 = 0 & colidx = 0
				plots,xvals3(noboom),yvals3(noboom),color=cols3(noboom),symsize=ssz,psym=8
			endif
		endif else begin
			xvals2 = 0 & yvals2 = 0 & colidx = 0
			plots,xvals3,yvals3,color=cols3,symsize=ssz,psym=8
		endelse
		xvals3 = 0 & yvals3 = 0 & cols3 = 0
   	endif
    		
    
    image=tvrd(0,0,xsize,ysize)
    fill_index=where(image eq fov_fill_color)
    if (fill_index(0) eq -1) then return
    mask=image*0
    mask(fill_index+1)=1
    mask(fill_index-1)=1
    mask(fill_index+xsize)=1
    mask(fill_index-xsize)=1

  ;  ;print,location(j,k,1),location(j,k,0)
  endfor
  
END

PRO ZOOM_OVERPASS_WID, start_time, end_time, delta_time, backgrounds=backgrounds, preview=preview

  common datafiles, orbit, fname
  COMMON datablock, array,xsize,ysize
common movie_widgs, mb, timeseries_id, draw_id, mb_start_slid, mb_cur_slid, mb_end_slid, $
	mb_step_slid, mb_compression_droplist, mb_images_droplist, mb_bit_depth_droplist, mb_size_slid, mb_test_but, $
	mb_proc_time_slid, mb_outdir_field, mb_prefix_field, mb_play_but, mb_quit_but, mb_quality_droplist, mb_status_lab
common movie_params, movie_start, movie_stop, frame_size, movie_dur, movie_comp, movie_step, $
	movie_frames, movie_size, frame_proctime, movie_proctime, comp_vals,bit_depth,quality
	common movie_params2, window_start, window_stop, mb_frame_lab
	common movie_com4, proctimes, proctested, frame_sizes
common mb_com_2, mb_remain_lab

  widget_control,mb_cur_slid,get_value=curslid_init
  widget_control,mb_images_droplist,get_uvalue=use_bgs
  widget_control,mb_outdir_field,get_value=path
  widget_control,mb_prefix_field,get_value=stub
  
  systime_start = systime(1)

  usersym,sin(findgen(21)/10.*!pi),cos(findgen(21)/10.*!pi),/fill
  load_my_color_tables
  ocean_color=15
  land_color=12
  grid_color=0
  equator_color=104
  fov_border_color=3*0+174
  fov_fill_color=17*0+192
  prev_fov_color=16
  continent_color=145
  black=1
  white=0
  tvlct,r,g,b,/get

  xsize=420
  ysize=420
  xsize=320
  ysize=320
  if keyword_set(preview) then begin
  	wset, draw_id
  endif else begin
  	window,0,colors=220,xs=xsize,ys=ysize,/pixmap
  endelse
  erase,black
  !P.position=[.05,.05,.95,.95]

  time_offset_index= long(orbit.one_second(0).TAI93_time)

  ;determine where to put the center of the map
  set_map_size, map_lat, map_lon,dy,dx , start_time

  map_set,map_lat,map_lon,/horiz,/orthographic,color=0,$
    limit=[map_lat-dy,map_lon-dx,map_lat+dy,map_lon+dx]
  ; make ocean blue
  tv,intarr(xsize*.90,ysize*.90)+ocean_color,xsize*.05,ysize*.05
  array=tvrd(0,0,xsize,ysize)


  ; determine the places that the instrument has already seen
  ; from the beginning of the orbit.
  ;prior_observation_path, new_location, time_offset_index

  ;---------------------------------------------------------------
  Epos=convert_coord([xsize/2.], [ysize/2.], /device,/to_data)
  Ex=cos(Epos(0,*)*!dtor)*cos(Epos(1,*)*!dtor)
  Ey=sin(Epos(0,*)*!dtor)*cos(Epos(1,*)*!dtor)
  Ez=sin(Epos(1,*)*!dtor)

  Sx=orbit.one_second(start_time-time_offset_index).solar_vector(0)
  Sy=orbit.one_second(start_time-time_offset_index).solar_vector(1)
  Sz=orbit.one_second(start_time-time_offset_index).solar_vector(2)

  factor=fix((Ex*Sx+Ey*Sy+Ez*Sz)*32+64)
  index=where(array eq ocean_color)
  array_ocean=array*0+1
  array_ocean(index)=array(index)*0 +(factor > 64)
  ;---------------------------------------------------------------

  ; step through time making plot of what the instrument sees
  
  ; print,long((end_time-start_time+delta_time/2.)/delta_time)
  
  loop_end = long((end_time-start_time+delta_time/2.)/delta_time)

  for itime=0L, loop_end ,1L do begin
  	widget_control,mb_cur_slid,set_value=curslid_init+itime*delta_time
  	
    time=start_time+delta_time*itime*1.0D0
    ; print,time,format='(f14.4)'

    set_map_size, map_lat, map_lon, dy , dx , time+0*3.00D0
    map_set,map_lat,map_lon,/orthographic,$
      limit=[map_lat-dy,map_lon-dx,map_lat+dy,map_lon+dx]
  ;---------------------------------------------------------------
  Epos=convert_coord([xsize/2.], [ysize/2.], /device,/to_data)
  Ex=cos(Epos(0,*)*!dtor)*cos(Epos(1,*)*!dtor)
  Ey=sin(Epos(0,*)*!dtor)*cos(Epos(1,*)*!dtor)
  Ez=sin(Epos(1,*)*!dtor)

  Sx=orbit.one_second(start_time-time_offset_index).solar_vector(0)
  Sy=orbit.one_second(start_time-time_offset_index).solar_vector(1)
  Sz=orbit.one_second(start_time-time_offset_index).solar_vector(2)

  factor=fix((Ex*Sx+Ey*Sy+Ez*Sz)*32+64)
  index=where(array eq ocean_color)
  array_ocean=array*0+1
  array_ocean(index)=array(index)*0 +(factor > 64)
  ;---------------------------------------------------------------
    erase,black
;-------------
    map_continents,/cont,/fill,color=land_color
    array_land=tvrd(0,0,xsize,ysize)

    array_land=array_land*(array/ocean_color) > 1

    land_index=where(array_land eq land_color)
    array_land=array_ocean
    if (land_index(0) ne -1) then begin
      array_land(land_index)=array_ocean(land_index)-32
    endif
    tv,array_land
;-------------
;   tv,array,0,0
; ; map_grid,latdel=5,londel=5,glinesty=1,color=grid_color
;   oplot,findgen(362)-180.,0*findgen(362),color=equator_color,thick=1
;   map_continents,/fill,color=land_color
    map_continents,/usa,mlinethick=1,color=continent_color
; ; map_continents,/coast,color=black,mlinethick=1
    map_continents,/cont,/hires,color=continent_color,mlinethick=1+xsize/400
    lower=fix(map_lon-dx)/5*5-5
    upper=fix(map_lon+dx)/5*5+10
    for m=lower,upper,5 do oplot,intarr(901)+m,findgen(901)/5.-90,psym=3,color=grid_color
    lower=fix(map_lat-dy)/5*5-5
    upper=fix(map_lat+dy)/5*5+10
    for m=lower,upper,5 do oplot,findgen(1801)/5.-180,intarr(1801)+m,psym=3,color=grid_color
    tv,intarr(xsize,fix(ysize*.05))+black,0,0
    tv,intarr(xsize,fix(ysize*.05))+black,0,ysize*.95
    tv,intarr(fix(xsize*.05),ysize)+black,0,0
    tv,intarr(fix(xsize*.05),ysize)+black,xsize*.95,0

  ; map_grid,latdel=5,londel=5,glinesty=1,color=grid_color
    map_set,map_lat,map_lon,/orthographic,$
      limit=[map_lat-dy,map_lon-dx,map_lat+dy,map_lon+dx],/noerase
    

;   display previous orbits
;   for k=4,1,-1 do begin 
;     tmp=new_location[*,1]+16.0*k
;     index=where(tmp gt 180)
;     if (index(0) ne -1) then tmp(index)=tmp(index)-360
;     for m=0,n_elements(tmp)-1,5 do $
;       polyfill,tmp(m:4+m),new_location[m:4+m,0],color=5+k
;   endfor


    ; plot prior observations from this orbit
    px=[1,126,126,1,1]
    py=[1,1,126,126,1]
    num=5

;   px=[0,63,120,63,0,0]
;   py=[2,-3,63,130,125,2]
;   num=6
;   this_sensor='lis'
;   for j=time_offset_index,time-35,10 do begin
;     get_vector_pointing_vector,this_sensor,px,py,look_vector
;     get_vector_earth_intersection,look_vector, $
;      orbit.one_second(j-time_offset_index).TAI93_time*(intarr(num)+1),$
;        location,return_code
;     polyfill,location[*,1],location[*,0],color=prev_fov_color
;   endfor


    ; plot current observations for this moment in time
    px=[0,30,53,73,98,127,127,127,127,127,127, 98, 73, 53, $
        30,  0, 0, 0, 0, 0,0]
    py=[0, 0, 0, 0, 0,  0, 30, 53, 73, 98,127,127,127,127,$
       127,127,98,73,53,30,0]
    num=n_elements(px)
    this_sensor='lis'
    get_vector_pointing_vector,this_sensor,px,py,look_vector
    get_vector_earth_intersection_new,look_vector, $
     time*(dblarr(num)+1.D0)+0*3.00D0, location,return_code
    polyfill,location[*,1],location[*,0],color=fov_fill_color

    ; display the background images 
     if keyword_set(backgrounds) then begin
		; print,'plotting backgrounds',xsize,ysize,fov_fill_color,time
     		plot_backgrounds,xsize,ysize,fov_fill_color,time,2.0
	endif
	
    ;plot previous events
    event_index=where(orbit.point.lightning.event.tai93_time ge start_time and $
       orbit.point.lightning.event.tai93_time lt time)
    if (event_index(0) ne -1) then begin
      ppx=orbit.point.lightning.event(event_index).x_pixel
      ppy=orbit.point.lightning.event(event_index).y_pixel
      get_vector_pointing_vector,this_sensor,ppx,ppy,look_vector
      get_vector_earth_intersection_new,look_vector, $
       orbit.point.lightning.event(event_index).tai93_time+0*3.00D0, $
       e_location,return_code
      oplot,e_location[*,1],e_location[*,0],psym=8,$
        color=11,symsize=.05+fix(xsize/200)*.10
    endif
    ;plot current events
    event_index=where(orbit.point.lightning.event.tai93_time ge time and $
       orbit.point.lightning.event.tai93_time lt time+delta_time)
    if (event_index(0) ne -1) then begin
      ppx=orbit.point.lightning.event(event_index).x_pixel
      ppy=orbit.point.lightning.event(event_index).y_pixel
      get_vector_pointing_vector,this_sensor,ppx,ppy,look_vector
      get_vector_earth_intersection_new,look_vector, $
       orbit.point.lightning.event(event_index).tai93_time+0*3.00D0, $
       e_location,return_code
      oplot,e_location[*,1],e_location[*,0],psym=8,$
        color=2,symsize=.2+fix(xsize/200)*.10
      oplot,e_location[*,1],e_location[*,0],psym=8,$
        color=9,symsize=.05+fix(xsize/200)*.10
    endif

    oplot,location[*,1],location[*,0],color=fov_border_color,$
          thick=2+max([xsize,ysize])/400

  ; set_map_size, map_lat, map_lon, dy , dx , time
  ; dy=6
  ; dx=6/max([abs(cos((map_lat-dy/2.)*!dtor)),abs(cos((map_lat+dy/2.)*!dtor))])
    map_set,map_lat,map_lon,/horiz,/orthographic,/noerase,color=white,$
      limit=[map_lat-dy,map_lon-dx,map_lat+dy,map_lon+dx]

    image=tvrd(0,0,xsize,ysize)
    ;filename= "Data:"
    ;filename = filename + 'test_'+string(itime,format='(i4.4)')+'.gif'
    filename = path[0]+stub[0]+string(itime,format='(i4.4)')+'.gif'
    write_gif,filename[0],image,r,g,b
    ; wait,1
    systime_now = systime(1)
    frame_proctime = 1./((float(itime+1)/float((systime_now)-float(systime_start)))*60.)
	proctimes[use_bgs] = frame_proctime
	proctested[use_bgs] = 1
	movie_proctime = movie_frames*frame_proctime
	widget_control,mb_proc_time_slid,set_value=long(movie_proctime)
	remaining_proctime_min = float(loop_end-itime)*frame_proctime
	remaining_proctime_hour = fix(remaining_proctime_min/60.)
	remaining_proctime_min = remaining_proctime_min - float(remaining_proctime_hour*60)
	remaining_proctime_minute = fix(remaining_proctime_min)
	remaining_proctime_second = (remaining_proctime_min-float(remaining_proctime_minute))*60.0
	remainstr = string(remaining_proctime_hour,remaining_proctime_minute,remaining_proctime_second, $
				format='("Time remaining: ",(I2.2),":",(I2.2),":",(F4.1))')
	widget_control,mb_remain_lab,set_value=remainstr
  endfor
  
  widget_control,mb_cur_slid,set_value=curslid_init

END

pro browse_orbit_summary, filename, orbit
  
  create_color_tables
  window,0,colors=220,xs=1200,ys=350
  erase,0

  browse_map, orbit
  browse_viewtime, orbit
  browse_events, orbit
  xyouts,.5,.96,filename,charsize=.6,align=.5,/norm,color=1

end

pro create_color_tables
    r=bytarr(256)
    g=bytarr(256)
    b=bytarr(256)
    r(0:15)=[255,000,255,128,000,000,000,000,255,255,255,101,159,237,166,000]
    g(0:15)=[255,000,000,000,216,128,194,000,255,122,000,097,109,209,217,106]
    b(0:15)=[255,000,000,000,000,000,255,255,000,000,255,198,068,186,248,129]
    r(16:95)=[193,176,160,144,128,112,096,080,064,048,032,016,000,000,000,000, $
              000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000, $
              000,019,039,058,078,098,117,137,156,176,196,215,235,255,255,255, $
              255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255, $
              250,246,241,237,232,228,223,219,215,210,206,201,197,192,188,184]
    g(16:95)=[146,150,154,159,163,167,172,176,180,185,189,193,198,189,180,172, $
              163,155,146,137,129,120,112,103,095,101,108,115,122,129,136,143, $
              150,158,166,174,182,190,198,206,214,222,230,238,246,255,241,227, $
              213,199,185,172,158,144,130,116,103,098,094,089,085,080,076,071, $
              066,062,057,053,048,044,039,035,031,026,022,017,013,008,004,000]
    b(16:95)=[255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255, $
              255,255,255,255,255,255,255,255,255,223,191,159,127,095,063,031, $
              000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000, $
              000,000,000,000,000,000,000,000,000,003,007,011,015,019,022,026, $
              029,032,036,039,042,046,049,053,056,059,063,066,069,073,076,080]
    r(96:96+15)=[000,017,034,051,068,085,102,119,136,153,170,187,204,221,238,255]
    g(96:96+15)=[177,182,187,192,197,203,208,213,218,223,229,234,239,244,249,255]
    b(96:96+15)=[208,211,214,217,220,223,226,229,233,236,239,242,245,248,251,255]
    r(112:112+15)=[209,212,215,218,221,224,227,230,233,236,239,242,245,248,251,255]
    g(112:112+15)=[164,170,176,182,188,194,200,206,212,218,224,230,236,242,248,255]
    b(112:112+15)=[124,132,141,150,158,167,176,185,193,202,211,220,228,237,246,255]

    r(135:220)=fix(bindgen(86)*255./86.)
    g(135:220)=fix(bindgen(86)*255./86.)
    b(135:220)=fix(bindgen(86)*255./86.)
    
    r(221:255) = r(220) & g(221:255) = g(220) & b(221:255) = b(220)
    
    ; Kludge for systems with < 256 colors available; we will try to keep things
    ;    to 220 colors, which seems to work across platforms
    
    r(128) = 154 & g(128) = 154 & b(128) = 154 ; Ocean
    r(129) = 204 & g(129) = 204 & b(129) = 204 ; Land
    r(130) = 32 & g(130) = 32 & b(130) = 32
    r(131) = 96 & g(131) = 96 & b(131) = 96
    r(132) = 174 & g(132) = 174 & b(132) = 174
    r(133) = 224 & g(133) = 224 & b(133) = 224
    r(134) = 255 & g(134) = 255 & b(134) = 255
    
 ;   i = indgen(16)
 ;   r(130+i) = (i+1)*16
 ;   g(130+i) = (i+1)*16
 ;   b(130+i) = (i+1)*16
 ;   r(145) = 255 & g(145) = 255 & b(145) = 255
    
;	print,r(205),g(205),b(205)
;	print,r(230),g(230),b(230)
    tvlct,r,g,b

	
end

pro fancy_plot, filename, orbit, ctr_lat, ctr_lon
	show_density_map,filename,orbit,ctr_lat,ctr_lon,0
end

;--------------------------------------------------------------------------------------------
pro show_density_map, filename, orbit,ctr_lat,ctr_lon,leader
;--------------------------------------------------------------------------------------------
	common ctsave, rold,gold,bold
	common fpcom, fpdid, fpbut1, fpbut2, fpbut3, fpbut4, fpb
	common fpcom2, theimage
	common lisbase, lisb
	common fontcom, fontsize
		
	; Store previous color table

	waitb = widget_base(title='Working',tlb_frame_attr=0,scroll=0,/modal,column=1,group_leader=lisb)
	waitm = widget_label(waitb,value='Preparing lightning density plot ... this may take a moment...',/align_center)
	widget_control,waitb,/realize
	
	
	tvlct,rold,gold,bold,/get
    
     create_color_tables
	if (!version.os_family eq 'MacOS') then begin
		fontsize = 2.0
	endif else begin 
		if (!version.os_family eq 'unix') then begin
			fontsize = 0.8
		endif else begin
			fontsize = 1.0
		endelse
	endelse

    
    win_size=900
    w_s_plus_100=win_size+100

    window,1,colors=220,xs=win_size+100,ys=win_size+100,/pixmap
    erase,0

;   latmin=22.  & latmax=42.  & lonmin=-90. &  lonmax=-70.

    latmin=fix(ctr_lat/2.)*2-10.  
    latmax=fix(ctr_lat/2.)*2+10.
    lonmin=fix(ctr_lon/2.)*2-10.
    lonmax=fix(ctr_lon/2.)*2+10.


    scale=float(win_size)/max([lonmax-lonmin,latmax-latmin])

    density_array=fltarr((lonmax-lonmin)*scale+100,(latmax-latmin)*scale+100)


    index=where(orbit.point.lightning.flash.location[1] ge lonmin and $
                orbit.point.lightning.flash.location[1] le lonmax and $
                orbit.point.lightning.flash.location[0] ge latmin and $
                orbit.point.lightning.flash.location[0] le latmax)
    ; print,'Num of flashes ',n_elements(index)

    index=where(orbit.point.lightning.group.location[1] ge lonmin and $
                orbit.point.lightning.group.location[1] le lonmax and $
                orbit.point.lightning.group.location[0] ge latmin and $
                orbit.point.lightning.group.location[0] le latmax)

    ; print,'Num of groups ',n_elements(index)

    if (index(0) ne -1) then begin
      first_group_time=min(orbit.point.lightning.group(index).tai93_time)
      last_group_time=max(orbit.point.lightning.group(index).tai93_time)
      s_index=sort(orbit.point.lightning.group(index).tai93_time)
      k=n_elements(index) /2
      middle_group_time=orbit.point.lightning.group(index(s_index(k))).tai93_time
    endif else begin
      bg_in_view=intarr(orbit.point.point_summary.bg_count)
      for i=0,3 do begin
       kindex = where(orbit.point.bg_summary.corners[1+i*2] gt lonmin and $
                     orbit.point.bg_summary.corners[1+i*2] lt lonmax and $
                     orbit.point.bg_summary.corners[0+i*2] gt latmin and $
                     orbit.point.bg_summary.corners[0+i*2] lt latmax)
       if (kindex(0) ne -1) then bg_in_view(kindex)=1
      endfor
      image_numbers=where(bg_in_view eq 1)
      first_group_time=min(orbit.point.bg_summary(image_numbers).tai93_time)
      last_group_time=max(orbit.point.bg_summary(image_numbers).tai93_time)
      s_index=sort(orbit.point.bg_summary(image_numbers).tai93_time)
      k=n_elements(image_numbers) /2
      middle_group_time=orbit.point.bg_summary(image_numbers(s_index(k))).tai93_time
    endelse


    ; determine time of middle group -- used for title

    ss=strmid(string(orbit.orbit_summary.UTC_start),17,2)
    mm=strmid(string(orbit.orbit_summary.UTC_start),14,2)
    hh=strmid(string(orbit.orbit_summary.UTC_start),11,2)
    date=strmid(string(orbit.orbit_summary.UTC_start),0,10)

    leap_second=long(orbit.orbit_summary.TAI93_start mod 86400)-ss-mm*60L-hh*3600L

    ref_days=long((middle_group_time-leap_second)/86400.)
    ref_time=(middle_group_time-leap_second-ref_days*86400.D0)
    offset_mm=long(long(ref_time(0))/60.)

    start_ss=ref_time-offset_mm*60L
    start_mm=offset_mm mod 60
    start_hh=offset_mm/60

    diam=fix(180./scale) +1.

	widget_control,waitm,set_value='Preparing density grid ...'
	
    if (index(0) ne -1) then begin
    min_cluster=10
    for i=0L, n_elements(index)-1 do begin
      event_index=orbit.point.lightning.group(index(i)).child_address
      xlower=999
      xupper=-999
      ylower=999
      yupper=-999
      flag=0
      for m=0,orbit.point.lightning.group(index(i)).child_count-1 do begin
        
        if (orbit.point.lightning.event(event_index+m).cluster_index ge 0) then begin
          px=fix((orbit.point.lightning.event(event_index+m).location[1]-lonmin)*scale+.5) 
          py=fix((orbit.point.lightning.event(event_index+m).location[0]-latmin)*scale+.5)
          polyfill,diam*sin(findgen(21)/10.*!pi)+px+50,diam*cos(findgen(21)/10.*!pi)+py+50,/dev,color=1
          xlower=min([xlower,px-diam-1+50])
          xupper=max([xupper,px+diam+1+50])
          ylower=min([ylower,py-diam-1+50])
          yupper=max([yupper,py+diam+1+50])
          flag=1
        endif
      endfor
      if (flag eq 1) then begin
        image=tvrd(xlower,ylower,xupper-xlower+1,yupper-ylower+1)
        density_array(xlower:xupper,ylower:yupper)=$
             density_array(xlower:xupper,ylower:yupper)+image(*,*)
        tv,intarr(xupper-xlower+1,yupper-ylower+1),xlower,ylower
      endif
    endfor
    endif

;   tv,density_array

    ; white out the edges of the density array

    tv,intarr(50,(latmax-latmin)*scale+100),0,0
    tv,intarr(50,(latmax-latmin)*scale+100),(lonmax-lonmin)*scale+50,0
    tv,intarr((lonmax-lonmin)*scale+100,50),0,0
    tv,intarr((lonmax-lonmin)*scale+100,50),0,(latmax-latmin)*scale+50

	widget_control,waitm,set_value='Adjusting for viewtime ...'



    viewtime_array=fltarr((lonmax-lonmin)*scale+100,(latmax-latmin)*scale+100)

    world_grid_05x05=fltarr(720,360)

    ; loop thru viewtime, keeping accumulated viewtime in grid

    for i=0L,orbit.point.point_summary.vt_count-1 do begin
      px=fix((orbit.point.viewtime(i).location[1]-.25+180)*2)
      py=fix((orbit.point.viewtime(i).location[0]-.25+90)*2)
      world_grid_05x05(px,py)=world_grid_05x05(px,py)+ $
                              orbit.point.viewtime(i).effective_obs
    endfor
	maxwvg = max(world_grid_05x05(*,*))
	    
    ; transfer viewtime information to viewtime array for the region of interest
    for i=(lonmin+180.)*2,(lonmax+180)*2 do begin
      for j=(latmin+90)*2,(latmax+90)*2 do begin
          loc_lat=j/2.-90.
          loc_lon=i/2.-180.
          px=fix((loc_lon-lonmin)*scale+.5) 
          py=fix((loc_lat-latmin)*scale+.5)
 ;         polyfill,[px-1,px+.5*scale+1,px+.5*scale+1,px-1,px-1]+50, $
 ;                 [py-1,py-1,py+.5*scale+1,py+.5*scale+1,py-1]+50,$
 ;                   /dev,color=world_grid_05x05(i,j)
          polyfill,[px-1,px+.5*scale+1,px+.5*scale+1,px-1,px-1]+50, $
                   [py-1,py-1,py+.5*scale+1,py+.5*scale+1,py-1]+50,$
                    /dev,color=135+fix(world_grid_05x05(i,j)*86./maxwvg)
	
            endfor
    endfor
    viewtime_array =tvrd(0,0,!d.x_size,!d.y_size)
;    viewtime_array = (viewtime_array-32.)*80./192.
	viewtime_array = viewtime_array*90./255.
	    
	; print,max(viewtime_array),min(viewtime_array)
	
  viewtime_array=smooth((viewtime_array > 20),20)
  net_density_array=float(density_array)*100./float(viewtime_array)
        
	; print,min(net_density_array),max(net_density_array)

	widget_control,waitm,set_value='Preparing composite image ...'


    index=where(net_density_array gt 1)
    if (index(0) ne -1) then begin 
      max_log=max(alog(net_density_array(index)))
      color_scale = 10.
    ; print,'color_scale=',color_scale
  
      net_density_array(index) = (alog(net_density_array(index))*color_scale+16)
      index=where(net_density_array gt (16+79))
      if(index(0) ne -1) then net_density_array(index)=79+16
    endif

    index=where(net_density_array gt 0)
    if (index(0) ne -1) then begin
      for i=0L,n_elements(index)-1 do begin
        if (net_density_array(index(i)-1) eq 0) then net_density_array(index(i)-1)=1
        if (net_density_array(index(i)+1) eq 0) then net_density_array(index(i)+1)=1
        if (net_density_array(index(i)-w_s_plus_100) eq 0) then $
          net_density_array(index(i)-w_s_plus_100)=1
        if (net_density_array(index(i)+w_s_plus_100) eq 0) then $
          net_density_array(index(i)+w_s_plus_100)=1
    
        if (net_density_array(index(i)-2) eq 0) then net_density_array(index(i)-2)=1
        if (net_density_array(index(i)+2) eq 0) then net_density_array(index(i)+2)=1
        if (net_density_array(index(i)-2*w_s_plus_100) eq 0) then $
          net_density_array(index(i)-2*w_s_plus_100)=1
        if (net_density_array(index(i)+2*w_s_plus_100) eq 0) then $
          net_density_array(index(i)+2*w_s_plus_100)=1
      endfor
    endif

    net_density_array=net_density_array(50:(lonmax-lonmin)*scale-1+50,50:(latmax-latmin)*scale-1+50)


    window,4,colors=220,xs=win_size,ys=win_size,/pixmap
    !p.position=[.15,.16,.95,.96]
    ; plot map in window
    map_set,limit=[latmin,lonmin,latmax,lonmax],/noborder,color=1
    lower_time=max([first_group_time-230,orbit.orbit_summary.tai93_start])
    upper_time=min([last_group_time+230,orbit.orbit_summary.tai93_end])
    index=where(orbit.one_second.TAI93_time gt lower_time and $
                orbit.one_second.TAI93_time lt upper_time )

    corner_lat=fltarr(4)
    corner_lon=fltarr(4)

    this_sensor='lis'
    px=[0,63,127,127,127,63,0,0,0]
    py=[0,0,0,63,127,127,127,63,0]
    
    if (index(0) ne -1) then begin
    for m=0L,n_elements(index)-1 do begin
      get_vector_pointing_vector,this_sensor,px,py,look_vector
      get_vector_earth_intersection,look_vector, $
         orbit.one_second(index(m)).TAI93_time*[1,1,1,1,1,1,1,1,1],location,10.,return_code
      polyfill,location(*,1),location(*,0),color=1
    endfor

    result_1=convert_coord(lonmin,latmin,/data,/to_dev)
    result_2=convert_coord(lonmax,latmax,/data,/to_dev)
    coverage_mask=tvrd(result_1(0),result_1(1),result_2(0)-result_1(0),result_2(1)-result_1(1))
    endif



	widget_control,waitm,set_value='Adding geographic overlays ...'

   ; window,1,xs=win_size,ys=win_size
    !p.position=[.15,.16,.95,.96]
    ; plot map in window
    map_set,limit=[latmin,lonmin,latmax,lonmax],/noborder,color=1
    erase,0

       
    ; make ocean gray
;    polyfill,[lonmin,lonmax,lonmax,lonmin,lonmin],[latmin,latmin,latmax,latmax,latmin],color=210+15-20
    polyfill,[lonmin,lonmax,lonmax,lonmin,lonmin],[latmin,latmin,latmax,latmax,latmin],color=128

    ; color in continents with gray color, and put black border arount continents
;    map_continents,/cont,/hires,/fill,color=240+10-20
    map_continents,/usa,/cont,/hires,/fill,color=129
    map_mask=tvrd(result_1(0),result_1(1),result_2(0)-result_1(0),result_2(1)-result_1(1))
    tv,map_mask + coverage_mask*4,result_1(0),result_1(1)
    map_continents,/usa,/cont,/hires,color=1

    ; put dotted grid on map
    map_grid,latdel=2,londel=2,color=1,glinestyle=1

    index=where(net_density_array gt 0)
    if (index(0) ne -1) then begin
      for i=0L,n_elements(index)-1 do begin
        px=(index(i) mod win_size)/scale+lonmin
        py=(index(i)/win_size)/scale+latmin
        plots,px,py,psym=3,color=net_density_array(index(i))
      endfor
    endif

    ; make border around plot 
    dt=(lonmax-lonmin)/80.
    oplot,[lonmin-dt,lonmax+dt,lonmax+dt,lonmin-dt,lonmin-dt],$
          [latmin-dt,latmin-dt,latmax+dt,latmax+dt,latmin-dt],$
          color=1,thick=0
    oplot,[lonmin,lonmax,lonmax,lonmin,lonmin],[latmin,latmin,latmax,latmax,latmin],$
     color=1,thick=3

    num=fix((lonmax-lonmin)/4.)
    
    for i=0,num do begin
      if (lonmin+4.*i ge 0) then begin
        result=convert_coord(lonmin+4*i,latmin-.03*(latmax-latmin),/data,/to_dev)
        xyouts,result(0),result(1),$
          string(lonmin+4.*i,format='(f5.1)')+'!eo!N E',charsize=fontsize,color=1,align=.5 ,/dev
        xyouts,result(0)+1,result(1),$
          string(lonmin+4.*i,format='(f5.1)')+'!eo!N E',charsize=fontsize,color=1,align=.5 ,/dev
        xyouts,result(0),result(1)+1,$
          string(lonmin+4.*i,format='(f5.1)')+'!eo!N E',charsize=fontsize,color=1,align=.5 ,/dev
        xyouts,result(0)+1,result(1)+1,$
          string(lonmin+4.*i,format='(f5.1)')+'!eo!N E',charsize=fontsize,color=1,align=.5 ,/dev
      endif else if (lonmin+4.*i eq 0) then begin
        result=convert_coord(lonmin+4*i,latmin-.03*(latmax-latmin),/data,/to_dev)
        xyouts,result(0),result(1), '0.0!eo!N',charsize=fontsize,color=1,align=.5,/dev
        xyouts,result(0)+1,result(1), '0.0!eo!N',charsize=fontsize,color=1,align=.5,/dev
        xyouts,result(0),result(1)+1, '0.0!eo!N',charsize=fontsize,color=1,align=.5,/dev
        xyouts,result(0)+1,result(1)+1, '0.0!eo!N',charsize=fontsize,color=1,align=.5,/dev
      endif else begin
        result=convert_coord(lonmin+4*i,latmin-.03*(latmax-latmin),/data,/to_dev)
        xyouts,result(0),result(1),$
          string(abs(lonmin+4.*i),format='(f5.1)')+'!eo!N W',charsize=fontsize,color=1,align=.5,/dev
        xyouts,result(0)+1,result(1),$
          string(abs(lonmin+4.*i),format='(f5.1)')+'!eo!N W',charsize=fontsize,color=1,align=.5,/dev
        xyouts,result(0),result(1)+1,$
          string(abs(lonmin+4.*i),format='(f5.1)')+'!eo!N W',charsize=fontsize,color=1,align=.5,/dev
        xyouts,result(0)+1,result(1)+1,$
          string(abs(lonmin+4.*i),format='(f5.1)')+'!eo!N W',charsize=fontsize,color=1,align=.5,/dev
      endelse
    endfor

    num=fix((latmax-latmin)/4.)
    for i=0,num do begin
      if (latmin+4.*i gt 0) then begin
        result=convert_coord(lonmin-.02*(lonmax-lonmin),latmin+4*i,/data,/to_dev)
        xyouts,result(0),result(1),$
          string(latmin+4.*i,format='(f4.1)')+'!eo!N N',charsize=fontsize,color=1,align=1.0 ,/dev
        xyouts,result(0)+1,result(1),$
          string(latmin+4.*i,format='(f4.1)')+'!eo!N N',charsize=fontsize,color=1,align=1.0 ,/dev
        xyouts,result(0),result(1)+1,$
          string(latmin+4.*i,format='(f4.1)')+'!eo!N N',charsize=fontsize,color=1,align=1.0 ,/dev
        xyouts,result(0)+1,result(1)+1,$
          string(latmin+4.*i,format='(f4.1)')+'!eo!N N',charsize=fontsize,color=1,align=1.0 ,/dev
      endif else if (latmin+4.*i eq 0) then begin
        result=convert_coord(lonmin-.02*(lonmax-lonmin),latmin+4*i,/data,/to_dev)
        xyouts,result(0),result(1), '0.0!eo!N',charsize=fontsize,color=1,align=1.0 ,/dev
        xyouts,result(0)+1,result(1), '0.0!eo!N',charsize=fontsize,color=1,align=1.0 ,/dev
        xyouts,result(0),result(1)+1, '0.0!eo!N',charsize=fontsize,color=1,align=1.0 ,/dev
        xyouts,result(0)+1,result(1)+1, '0.0!eo!N',charsize=fontsize,color=1,align=1.0 ,/dev
      endif else begin
        result=convert_coord(lonmin-.02*(lonmax-lonmin),latmin+4*i,/data,/to_dev)
        xyouts,result(0),result(1),$
          string(abs(latmin+4.*i),format='(f4.1)')+'!eo!N S',charsize=fontsize,color=1,align=1.,/dev
        xyouts,result(0)+1,result(1),$
          string(abs(latmin+4.*i),format='(f4.1)')+'!eo!N S',charsize=fontsize,color=1,align=1.,/dev
        xyouts,result(0),result(1)+1,$
          string(abs(latmin+4.*i),format='(f4.1)')+'!eo!N S',charsize=fontsize,color=1,align=1.,/dev
        xyouts,result(0)+1,result(1)+1,$
          string(abs(latmin+4.*i),format='(f4.1)')+'!eo!N S',charsize=fontsize,color=1,align=1.,/dev
      endelse
    endfor
    
    title=string(date,start_hh,start_mm,format='(a,3h   ,i2.2,1h:,i2.2,7h:00 UTC)')

    result=convert_coord((lonmax+lonmin)/2.,latmax+.01*(latmax-latmin),/data,/to_dev)
     xyouts,result(0),result(1),title,charsize=1.25*fontsize,align=.5,color=1,/dev
     xyouts,result(0),result(1)+1,title,charsize=1.25*fontsize,align=.5,color=1,/dev
     xyouts,result(0)+1,result(1),title,charsize=1.25*fontsize,align=.5,color=1,/dev
     xyouts,result(0)+1,result(1)+1,title,charsize=1.25*fontsize,align=.5,color=1,/dev

    !p.position=[.20,.029,.90,.069]
    plot,[0,1],[0,1],/nodata,charsize=.2,color=0,/noerase

    result=convert_coord(.50,1.25,/data,/to_dev)
     xyouts,result(0),result(1),'Lightning Activity',charsize=1.25*fontsize,align=.5,color=1,/dev
     xyouts,result(0)+1,result(1),'Lightning Activity',charsize=1.25*fontsize,align=.5,color=1,/dev
     xyouts,result(0),result(1)+1,'Lightning Activity',charsize=1.25*fontsize,align=.5,color=1,/dev
     xyouts,result(0)+1,result(1)+1,'Lightning Activity',charsize=1.25*fontsize,align=.5,color=1,/dev


    result=convert_coord(.10,-.5,/data,/to_dev)
     xyouts,result(0),result(1),'Low',charsize=fontsize,align=.5,color=1,/dev
     xyouts,result(0)+1,result(1),'Low',charsize=fontsize,align=.5,color=1,/dev
     xyouts,result(0),result(1)+1,'Low',charsize=fontsize,align=.5,color=1,/dev
     xyouts,result(0)+1,result(1)+1,'Low',charsize=fontsize,align=.5,color=1,/dev

    result=convert_coord(.50,-.5,/data,/to_dev)
     xyouts,result(0),result(1),'Medium',charsize=fontsize,align=.5,color=1,/dev
     xyouts,result(0)+1,result(1),'Medium',charsize=fontsize,align=.5,color=1,/dev
     xyouts,result(0),result(1)+1,'Medium',charsize=fontsize,align=.5,color=1,/dev
     xyouts,result(0)+1,result(1)+1,'Medium',charsize=fontsize,align=.5,color=1,/dev

    result=convert_coord(.90,-.5,/data,/to_dev)
     xyouts,result(0),result(1),'High',charsize=fontsize,align=.5,color=1,/dev
     xyouts,result(0)+1,result(1),'High',charsize=fontsize,align=.5,color=1,/dev
     xyouts,result(0),result(1)+1,'High',charsize=fontsize,align=.5,color=1,/dev
     xyouts,result(0)+1,result(1)+1,'High',charsize=fontsize,align=.5,color=1,/dev

    for i=0,79 do begin
      polyfill,[i,i+1.,i+1,i,i]/80.,[0,0,1,1,0],color=i+16
    endfor
    oplot,[0.0001,.999,.999,0.0001,0.0001],[0.001,0.001,.999,.999,0.001],thick=3,color=1

    !p.position=[0.,0.,0.,0.]
    
    theimage=tvrd(0,0,!d.x_size,!d.y_size)
    
    
	fpb = widget_base(row=2,title='Lightning Density Map (half size)',scroll=0)
	fpbb = widget_base(fpb,col=4)
	fpbut1 = widget_button(fpbb,value='Edit & Print',xsize=113)
	fpbut3 = widget_button(fpbb,value='Show Full Size Image',xsize=113)
	fpbut2 = widget_button(fpbb,value='Save Full GIF',xsize=113)
	fpbut4 = widget_button(fpbb,value='Done',xsize=113)
	fpdb = widget_base(fpb,col=1)
	fpdraw = widget_draw(fpdb,colors=220,retain=2,frame=3,xsize=!d.x_size/2,ysize=!d.y_size/2,/align_center)

	widget_control,fpb,/realize
	widget_control,fpdraw,get_value=fpdid
      wset,fpdid
	im2 = congrid(theimage,!d.x_size,!d.y_size,/interp)
	tv,im2
	
    ;tvlct,r,g,b,/get
    ;gifname='Data:map_x.gif'
    ;write_gif, gifname,image,r,g,b
	
	widget_control,waitb,/destroy
	
		xmanager,'Fancy Plot',fpb,event_handler='fplothandler',/no_block

end

pro fplothandler, event

	common ctsave, rold,gold,bold
	common fpcom, fpdid, fpbut1, fpbut2, fpbut3, fpbut4, fpb
	common fullpaths, app_path, data_path, bg_path, output_path
	common paths, lastpath, currentpath, origpath, shlbfile
	common fpcom2, theimage

		case 1 of 
			(event.id eq fpbut1): begin
					wset,fpdid
					; tmpimg = tvrd(0,0,!d.x_size,!d.y_size)
					tmpimg = theimage
					tvlct,thisred,thisgreen,thisblue,/get
					live_image,tmpimg,red=thisred,green=thisgreen,blue=thisblue
					end
			(event.id eq fpbut2): begin
					wset,fpdid
					cd,output_path[0]
					gif_save_name = dialog_pickfile(/noconfirm,/write,path=output_path[0])
					cd,currentpath
					if (gif_save_name) then begin
						; gifimage = tvrd(0,0,!d.x_size,!d.y_size)
						gifimage = theimage
						tvlct,r,g,b,/get
						write_gif,gif_save_name,gifimage,r,g,b
					endif
					end
			(event.id eq fpbut3):begin
					window,12,xsize=723,ysize=722,retain=2
					wset,12
					tv,theimage(135:858,142:864)
					end
			(event.id eq fpbut4): begin
					theimage = 0
					widget_control, fpb, /destroy
					tvlct,rold,gold,bold
					end
		endcase
end

pro ev_table, event, evlist

	common datafiles, orbit, fname
	common lisbase, lisb
	common evtab_com, evexpstr, numev, evidx, evtabexp, evtabdone, evtabb, evtabglob

	waitb = widget_base(title='Working',tlb_frame_attr=0,scroll=0,/modal,column=1,group_leader=lisb)
	waitm = widget_label(waitb,value='Preparing table ... this may take a moment...',/align_center)
	widget_control,waitb,/realize

	template_event = {TAI93_time:double(0), observe_time:fix(0), location0:float(0), location1:float(0),radiance:float(0),       $
				footprint:float(0), address:long(0), parent_address:long(0), x_pixel:byte(0), y_pixel:byte(0),  $
				bg_value:fix(0), bg_radiance:fix(0), amplitude:byte(0), sza_index:byte(0), glint_index:byte(0), $
				approx_threshold:byte(0), alert_flag:byte(0), cluster_index:byte(0), density_index:                      $
				byte(0), noise_index:byte(0), bg_value_flag:byte(0), grouping_sequence:long(0)}
	
	evlsz = size(evlist)
	case evlsz[0] of 
		0:begin 
			numev = 1
		  end
		1: begin
			numev = evlsz[1]
		   end
	endcase
	evidx = evlist

		flat_events = replicate(template_event,numev)
	flat_events(*).TAI93_time = orbit.point.lightning.event(evlist).TAI93_time
	flat_events(*).observe_time = orbit.point.lightning.event(evlist).observe_time
	flat_events(*).location0 = orbit.point.lightning.event(evlist).location(0)
	flat_events(*).location1 = orbit.point.lightning.event(evlist).location(1)
	flat_events(*).radiance = orbit.point.lightning.event(evlist).radiance
	flat_events(*).footprint = orbit.point.lightning.event(evlist).footprint
	flat_events(*).address = orbit.point.lightning.event(evlist).address
	flat_events(*).parent_address = orbit.point.lightning.event(evlist).parent_address
	flat_events(*).x_pixel = orbit.point.lightning.event(evlist).x_pixel
	flat_events(*).y_pixel = orbit.point.lightning.event(evlist).y_pixel
	flat_events(*).bg_value = orbit.point.lightning.event(evlist).bg_value
	flat_events(*).bg_radiance = orbit.point.lightning.event(evlist).bg_radiance
	flat_events(*).amplitude = orbit.point.lightning.event(evlist).amplitude
	flat_events(*).sza_index = orbit.point.lightning.event(evlist).sza_index
	flat_events(*).glint_index = orbit.point.lightning.event(evlist).glint_index
	flat_events(*).approx_threshold = orbit.point.lightning.event(evlist).approx_threshold
	flat_events(*).alert_flag = orbit.point.lightning.event(evlist).alert_flag
	flat_events(*).cluster_index = orbit.point.lightning.event(evlist).cluster_index
	flat_events(*).density_index = orbit.point.lightning.event(evlist).density_index
	flat_events(*).noise_index = orbit.point.lightning.event(evlist).noise_index
	flat_events(*).bg_value_flag = orbit.point.lightning.event(evlist).bg_value_flag
	flat_events(*).grouping_sequence = orbit.point.lightning.event(evlist).grouping_sequence
	
	onefmtstr = ['(D13.3)','(I3)','(F6.2)','(F7.2)','(E9.3)','(F5.1)','(I5)','(I4)','(I3)','(I3)','(I4)','(I3)','(I3)', $
		 '(I3)','(I3)','(I2)','(I3)','(I3)','(I3)','(I3)','(I1)','(I5)']
	
	evexpstr = '((D13.3)," ",(I3)," ",(F6.2)," ",(F7.2)," ",(E9.3)," ",(F5.1)," ",(I5)," ",(I4)," ",(I3)," ",(I3)," ",(I4)," ",(I3)," ",(I3)," "'+ $
		 '(I3)," ",(I3)," ",(I2)," ",(I3)," ",(I3)," ",(I3)," ",(I3)," ",(I1)," ",(I5))'
	fmtstr = strarr(22,numev)
	for i = 0,21,1 do begin
		fmtstr(i,*) = onefmtstr(i)
	endfor
	
	evtabb = widget_base(title='Events',group_leader=lisb,row=2)
	evtab1 = widget_base(evtabb,column=3)
	evtabexp = widget_button(evtab1,value='Save as ASCII...')
	evtabglob = widget_button(evtab1,value='Store in global struct')
	evtabdone = widget_button(evtab1,value='Done')
	evtab2 = widget_base(evtabb,column=1)
	evtab = widget_table(evtab2,value=flat_events(*),/scroll,/editable,/resizeable_columns,xsize=22,ysize=numev,x_scroll_size=12,y_scroll_size=24,column_labels= $
		['TAI93_time','observe_time','location[0]','location[1]','radiance','footprint','address','parent_address','x_pixel', $
		 'y_pixel','bg_value','bg_radiance','amplitude','sza_index','glint_index','approx_threshold','alert_flag','cluster_index', $
		 'density_index','noise_index','bg_value_flag','grouping_sequence'],/alignment,format= $
		 fmtstr)
		 
	widget_control,waitb,/destroy
	widget_control,evtabb,/realize
	xmanager, 'EV Table', evtabb, event='ev_table_event', /no_block
	; 
end

pro ev_table_event, event
	common evtab_com, evexpstr, numev, evidx, evtabexp, evtabdone, evtabb, evtabglob
	common datafiles, orbit, fname
	common lisbase, lisb
	common fullpaths, app_path, data_path, bg_path, output_path
	common paths, lastpath, currentpath, origpath, shlbfile
	common global_subsets, subset_events, subset_flashes, subset_groups, subset_areas, $
			subset_viewtimes, subset_one_seconds

	case 1 of 
		(event.id eq evtabdone):begin
				widget_control, evtabb, /destroy
			end
		(event.id eq evtabglob):begin
				subset_events = replicate(orbit.point.lightning.event[0],numev)
				subset_events(*) = orbit.point.lightning.event[evidx]
				widget_control,evtabglob,sensitive=0
			end
		(event.id eq evtabexp):begin
				cd,output_path[0]
				fnameasc = dialog_pickfile(/noconfirm,/write,path=output_path[0])
				if (fnameasc ne '') then begin 
					openw,lun,fnameasc,/get_lun
					for i=0L,numev-1L,1L do begin
						printf,lun,orbit.point.lightning.event[evidx[i]],format=evexpstr
					endfor
					close,lun
					free_lun,lun
					widget_control,evtabexp,sensitive=0
				endif
				cd,currentpath
			end
	endcase		
end

pro vt_table, event, vtlist

	common datafiles, orbit, fname
	common lisbase, lisb
	common vttab_com, vtexpstr, numvt, vtidx, vttabexp, vttabdone, vttabb, vttabglob
	
	waitb = widget_base(title='Working',tlb_frame_attr=0,scroll=0,/modal,column=1,group_leader=lisb)
	waitm = widget_label(waitb,value='Preparing table ... this may take a moment...',/align_center)
	widget_control,waitb,/realize
	
	template_vt = {location0:float(0),location1:float(0),TAI93_start:long(0),TAI93_end:long(0), $
			     effective_obs:float(0), alert_flag:byte(0), approx_threshold:byte(0)}
	
	vtlsz = size(vtlist)
	case vtlsz[0] of 
		0:begin 
			numvt = 1
		  end
		1: begin
			numvt = vtlsz[1]
		   end
	endcase
	vtidx = vtlist
	
	flat_vts = replicate(template_vt,numvt)
	
	flat_vts(*).location0 = orbit.point.viewtime(vtlist).location(0)
	flat_vts(*).location1 = orbit.point.viewtime(vtlist).location(1)
	flat_vts(*).TAI93_start = orbit.point.viewtime(vtlist).TAI93_start
	flat_vts(*).TAI93_end = orbit.point.viewtime(vtlist).TAI93_end
	flat_vts(*).effective_obs = orbit.point.viewtime(vtlist).effective_obs
	flat_vts(*).alert_flag = orbit.point.viewtime(vtlist).alert_flag
	flat_vts(*).approx_threshold = orbit.point.viewtime(vtlist).approx_threshold
		
	onefmtstr = ['(F6.2)','(F7.2)','(I13)','(I13)','(F7.3)','(I3)','(I3)']
	
	vtexpstr = '((F6.2)," ",(F7.2)," ",(I13)," ",(I13)," ",(F7.3)," ",(I3)," ",(I3))'
	
	fmtstr = strarr(7,numvt)
	for i = 0,6,1 do begin
		fmtstr(i,*) = onefmtstr(i)
	endfor
	
	vttabb = widget_base(title='Viewtimes',group_leader=lisb,row=2)
	vttab1 = widget_base(vttabb,column=3)
	vttabexp = widget_button(vttab1,value='Save as ASCII...')
	vttabglob = widget_button(vttab1,value='Store in global struct')
	vttabdone = widget_button(vttab1,value='Done')
	vttab2 = widget_base(vttabb,column=1)
	vttab = widget_table(vttab2,value=flat_vts(*),/scroll,/editable,/resizeable_columns,xsize=7,ysize=numvt,$
					x_scroll_size=7,y_scroll_size=15,column_labels= $
		['Location[0]','Location[1]','TAI93_start','TAI93_end','effective_obs','alert_flag','approx_threshold'],/alignment,format= $
		 fmtstr)
	widget_control,waitb,/destroy
	widget_control,vttabb,/realize
	xmanager, 'VT Table', vttabb, event='vt_table_event', /no_block
	; 
end

pro vt_table_event, event
	common vttab_com, vtexpstr, numvt, vtidx, vttabexp, vttabdone, vttabb, vttabglob
	common datafiles, orbit, fname
	common lisbase, lisb
	common fullpaths, app_path, data_path, bg_path, output_path
	common paths, lastpath, currentpath, origpath, shlbfile
	common global_subsets, subset_events, subset_flashes, subset_groups, subset_areas, $
			subset_viewtimes, subset_one_seconds

	case 1 of 
		(event.id eq vttabdone):begin
				widget_control, vttabb, /destroy
			end
		(event.id eq vttabglob):begin
				subset_viewtimes = replicate(orbit.point.viewtime[0],numvt)
				subset_viewtimes[*] = orbit.point.viewtime[vtidx]
				widget_control,vttabglob,sensitive=0
			end
		(event.id eq vttabexp):begin
					cd,output_path[0]
				fnameasc = dialog_pickfile(/noconfirm,/write,path=output_path[0])
				if (fnameasc ne '') then begin 
					openw,lun,fnameasc,/get_lun
					for i=0L,numvt-1L,1L do begin
						printf,lun,orbit.point.viewtime[vtidx[i]],format=vtexpstr
					endfor
					close,lun
					free_lun,lun
					widget_control,vttabexp,sensitive=0
				endif
				cd,currentpath
			end
	endcase		
end

pro os_table, event, oslist

	common datafiles, orbit, fname
	common lisbase, lisb
	common ostab_com, osexpstr, numos, osidx, ostabexp, ostabdone, ostabb, ostabglob

	waitb = widget_base(title='Working',tlb_frame_attr=0,scroll=0,/modal,column=1,group_leader=lisb)
	waitm = widget_label(waitb,value='Preparing table ... this may take a moment...',/align_center)
	widget_control,waitb,/realize

	template_os = {TAI93_time:double(0),alert_summary:byte(0),instrument_alert:byte(0),platform_alert:byte(0),external_alert:byte(0), $
			     processing_alert:byte(0),position_vector0:float(0),position_vector1:float(0),position_vector2:float(0),velocity_vector0:float(0), $
			     velocity_vector1:float(0),velocity_vector2:float(0),transform_matrix0:float(0),transform_matrix1:float(0),transform_matrix2:float(0), $
			     transform_matrix3:float(0),transform_matrix4:float(0),transform_matrix5:float(0),transform_matrix6:float(0), $
			     transform_matrix7:float(0),transform_matrix8:float(0),solar_vector0:float(0),solar_vector1:float(0),solar_vector2:float(0), $
			     ephemeris_quality_flag:byte(0),attitude_quality_flag:byte(0),boresight_threshold:byte(0), $
			     thresholds0:byte(0),thresholds1:byte(0),thresholds2:byte(0),thresholds3:byte(0),thresholds4:byte(0),thresholds5:byte(0), $
			     thresholds6:byte(0),thresholds7:byte(0),thresholds8:byte(0),thresholds9:byte(0),thresholds10:byte(0),thresholds11:byte(0), $
			     noise_index:byte(0),event_count0:fix(0),event_count1:fix(0),event_count2:fix(0),event_count3:fix(0),event_count4:fix(0), $
			     event_count5:fix(0)}

	oslsz = size(oslist)
	case oslsz[0] of 
		0:begin 
			numos = 1
		  end
		1: begin
			numos = oslsz[1]
		   end
	endcase
	osidx = oslist
	
	flat_oss = replicate(template_os,numos)
	
	flat_oss(*).TAI93_time 		= orbit.one_second(oslist).TAI93_time
	flat_oss(*).alert_summary 		= orbit.one_second(oslist).alert_summary
	flat_oss(*).instrument_alert	= orbit.one_second(oslist).instrument_alert
	flat_oss(*).platform_alert		= orbit.one_second(oslist).platform_alert
	flat_oss(*).external_alert		= orbit.one_second(oslist).external_alert
	flat_oss(*).processing_alert	= orbit.one_second(oslist).processing_alert
	flat_oss(*).position_vector0 	= orbit.one_second(oslist).position_vector(0)
	flat_oss(*).position_vector1 	= orbit.one_second(oslist).position_vector(1)
	flat_oss(*).position_vector2 	= orbit.one_second(oslist).position_vector(2)
	flat_oss(*).velocity_vector0 	= orbit.one_second(oslist).velocity_vector(0)
	flat_oss(*).velocity_vector1	= orbit.one_second(oslist).velocity_vector(1)
	flat_oss(*).velocity_vector2 	= orbit.one_second(oslist).velocity_vector(2)
	flat_oss(*).transform_matrix0 	= orbit.one_second(oslist).transform_matrix(0)
	flat_oss(*).transform_matrix1	= orbit.one_second(oslist).transform_matrix(1)
	flat_oss(*).transform_matrix2	= orbit.one_second(oslist).transform_matrix(2)
	flat_oss(*).transform_matrix3	= orbit.one_second(oslist).transform_matrix(3)
	flat_oss(*).transform_matrix4	= orbit.one_second(oslist).transform_matrix(4)
	flat_oss(*).transform_matrix5	= orbit.one_second(oslist).transform_matrix(5)
	flat_oss(*).transform_matrix6	= orbit.one_second(oslist).transform_matrix(6)
	flat_oss(*).transform_matrix7	= orbit.one_second(oslist).transform_matrix(7)
	flat_oss(*).transform_matrix8	= orbit.one_second(oslist).transform_matrix(8)
	flat_oss(*).solar_vector0 		= orbit.one_second(oslist).solar_vector(0)
	flat_oss(*).solar_vector1 		= orbit.one_second(oslist).solar_vector(1)
	flat_oss(*).solar_vector2 		= orbit.one_second(oslist).solar_vector(2)
	flat_oss(*).ephemeris_quality_flag = orbit.one_second(oslist).ephemeris_quality_flag
	flat_oss(*).attitude_quality_flag = orbit.one_second(oslist).attitude_quality_flag
	flat_oss(*).boresight_threshold = orbit.one_second(oslist).boresight_threshold
	flat_oss(*).thresholds0 		= orbit.one_second(oslist).thresholds(0)
	flat_oss(*).thresholds1 		= orbit.one_second(oslist).thresholds(1)
	flat_oss(*).thresholds2 		= orbit.one_second(oslist).thresholds(2)
	flat_oss(*).thresholds3 		= orbit.one_second(oslist).thresholds(3)
	flat_oss(*).thresholds4 		= orbit.one_second(oslist).thresholds(4)
	flat_oss(*).thresholds5 		= orbit.one_second(oslist).thresholds(5)
	flat_oss(*).thresholds6 		= orbit.one_second(oslist).thresholds(6)
	flat_oss(*).thresholds7 		= orbit.one_second(oslist).thresholds(7)
	flat_oss(*).thresholds8 		= orbit.one_second(oslist).thresholds(8)
	flat_oss(*).thresholds9 		= orbit.one_second(oslist).thresholds(9)
	flat_oss(*).thresholds10 		= orbit.one_second(oslist).thresholds(10)
	flat_oss(*).thresholds11		= orbit.one_second(oslist).thresholds(11)
	flat_oss(*).noise_index 		= orbit.one_second(oslist).noise_index
	flat_oss(*).event_count0 		= orbit.one_second(oslist).event_count(0)
	flat_oss(*).event_count1 		= orbit.one_second(oslist).event_count(1)
	flat_oss(*).event_count2 		= orbit.one_second(oslist).event_count(2)
	flat_oss(*).event_count3 		= orbit.one_second(oslist).event_count(3)
	flat_oss(*).event_count4 		= orbit.one_second(oslist).event_count(4)
	flat_oss(*).event_count5 		= orbit.one_second(oslist).event_count(5)
	

	onefmtstr = ['(F13.3)','(I3)','(I3)','(I3)','(I3)','(I3)','(F10.1)','(F10.1)','(F10.1)','(F7.1)','(F7.1)','(F7.1)', $
			  '(F7.4)','(F7.4)','(F7.4)','(F7.4)','(F7.4)','(F7.4)','(F7.4)','(F7.4)','(F7.4)','(F7.3)','(F7.3)','(F7.3)', $
			  '(I3)','(I3)','(I2)','(I2)','(I2)','(I2)','(I2)','(I2)','(I2)','(I2)','(I2)','(I2)','(I2)','(I2)','(I2)','(I3)', $
			  '(I6)','(I6)','(I6)','(I6)','(I6)','(I6)']
			  	
	fmtstr = strarr(46,numos)
	for i = 0,45,1 do begin
		fmtstr(i,*) = onefmtstr(i)
	endfor
	
	ostabb = widget_base(title='One Seconds',group_leader=lisb,row=2)
	ostab1 = widget_base(ostabb,column=3)
	ostabexp = widget_button(ostab1,value='Save as ASCII...')
	ostabglob = widget_button(ostab1,value='Store in global struct')
	ostabdone = widget_button(ostab1,value='Done')
	ostab2 = widget_base(ostabb,column=1)
	ostab = widget_table(ostab2,value=flat_oss(*),/scroll,/editable,/resizeable_columns,xsize=46,ysize=numos,$
			x_scroll_size=12,y_scroll_size=15,column_labels= $
		['TAI93_time','alert_summary','instrument_alert','platform_alert','external_alert','processing_alert','position_vector[0]', $
		 'position_vector[1]','position_vector[2]','velocity_vector[0]','velocity_vector[1]','velocity_vector[2]','transform_matrix[0]', $
		 'transform_matrix[1]','transform_matrix[2]','transform_matrix[3]','transform_matrix[4]','transform_matrix[5]','transform_matrix[6]', $
		 'transform_matrix[7]','transform_matrix[8]','solar_vector[0]', $
		 'solar_vector[1]','solar_vector[2]','ephemeris_quality_flag','attitude_quality_flag','boresight_threshold','thresholds[0]', $
		 'thresholds[1]','thresholds[2]','thresholds[3]','thresholds[4]','thresholds[5]','thresholds[6]','thresholds[7]','thresholds[8]', $
		 'thresholds[9]','thresholds[10]','thresholds[11]','noise_index','event_count[0]','event_count[1]','event_count[2]','event_count[3]', $
		 'event_count[4]','event_count[5]'], $
		/alignment,format=fmtstr)
	widget_control,waitb,/destroy
	widget_control,ostabb,/realize
	
	osexpstr = '((F13.3)," ",5((I3)," "),3((F10.1)," "),3((F7.1)," "),9((F7.4)," "),3((F7.3)," "),'+$
			  '2((I3)," "),(I2)," ",16((I2)," "),(I3)," ",6((I6)," "))'
	xmanager, 'OS Table', ostabb, event='os_table_event', /no_block
end

pro os_table_event, event
	common ostab_com, osexpstr, numos, osidx, ostabexp, ostabdone, ostabb, ostabglob
	common datafiles, orbit, fname
	common lisbase, lisb
	common fullpaths, app_path, data_path, bg_path, output_path
	common paths, lastpath, currentpath, origpath, shlbfile
	common global_subsets, subset_events, subset_flashes, subset_groups, subset_areas, $
			subset_viewtimes, subset_one_seconds

	case 1 of 
		(event.id eq ostabdone):begin
				widget_control, ostabb, /destroy
			end
		(event.id eq ostabglob):begin
				subset_one_seconds = replicate(orbit.one_second[0],numos)
				subset_one_seconds[*] = orbit.one_second[osidx]
				widget_control,ostabglob,sensitive=0
			end
		(event.id eq ostabexp):begin
					cd,output_path[0]
				fnameasc = dialog_pickfile(/noconfirm,/write,path=output_path[0])
				if (fnameasc ne '') then begin 
					openw,lun,fnameasc,/get_lun
					for i=0L,numos-1L,1L do begin
						printf,lun,orbit.one_second[osidx[i]],format=osexpstr
					endfor
					close,lun
					free_lun,lun
					widget_control,ostabexp,sensitive=0
				endif
				cd,currentpath
			end
	endcase		
end

pro gr_table, event, grlist

	common datafiles, orbit, fname
	common lisbase, lisb
	common grtab_com, grexpstr, numgr, gridx, grtabexp, grtabdone, grtabb, grtabglob

	waitb = widget_base(title='Working',tlb_frame_attr=0,scroll=0,/modal,column=1,group_leader=lisb)
	waitm = widget_label(waitb,value='Preparing table ... this may take a moment...',/align_center)
	widget_control,waitb,/realize

	this_os = !version.os
	if (this_os ne 'MacOS') then begin
		template_group = {TAI93_time:double(0), observe_time:fix(0), location0:float(0), location1:float(0),radiance:float(0),       $
					footprint:float(0), address:long(0), parent_address:long(0), child_address:long(0), child_count:long(0), $
					approx_threshold:byte(0), alert_flag:byte(0), cluster_index:byte(0), density_index:                      $
					byte(0), noise_index:byte(0), glint_index:float(0), oblong_index:float(0), grouping_sequence:long(0), grouping_status:byte(0)}
	endif else begin
		template_group = {TAI93_time:double(0), observe_time:fix(0), location0:float(0), location1:float(0),radiance:float(0),       $
			footprint:float(0), address:long(0), parent_address:long(0), child_address:long(0), child_count:long(0), $
			approx_threshold:byte(0), alert_flag:byte(0), cluster_index:byte(0), density_index:                      $
			byte(0), noise_index:byte(0), glint_index:float(0), grouping_sequence:long(0), grouping_status:byte(0)}
	endelse
	
	grlsz = size(grlist)
	case grlsz[0] of 
		0:begin 
			numgr = 1
		  end
		1: begin
			numgr = grlsz[1]
		   end
	endcase
	gridx = grlist

	flat_groups = replicate(template_group,numgr)
	flat_groups(*).TAI93_time = orbit.point.lightning.group(grlist).TAI93_time
	flat_groups(*).observe_time = orbit.point.lightning.group(grlist).observe_time
	flat_groups(*).location0 = orbit.point.lightning.group(grlist).location(0)
	flat_groups(*).location1 = orbit.point.lightning.group(grlist).location(1)
	flat_groups(*).radiance = orbit.point.lightning.group(grlist).radiance
	flat_groups(*).footprint = orbit.point.lightning.group(grlist).footprint
	flat_groups(*).address = orbit.point.lightning.group(grlist).address
	flat_groups(*).parent_address = orbit.point.lightning.group(grlist).parent_address
	flat_groups(*).child_address = orbit.point.lightning.group(grlist).child_address
	flat_groups(*).child_count = orbit.point.lightning.group(grlist).child_count
	flat_groups(*).approx_threshold = orbit.point.lightning.group(grlist).approx_threshold
	flat_groups(*).alert_flag = orbit.point.lightning.group(grlist).alert_flag
	flat_groups(*).cluster_index = orbit.point.lightning.group(grlist).cluster_index
	flat_groups(*).density_index = orbit.point.lightning.group(grlist).density_index
	flat_groups(*).noise_index = orbit.point.lightning.group(grlist).noise_index
	flat_groups(*).glint_index = orbit.point.lightning.group(grlist).glint_index
	flat_groups(*).grouping_sequence = orbit.point.lightning.group(grlist).grouping_sequence
	flat_groups(*).grouping_status = orbit.point.lightning.group(grlist).grouping_status
	
	if (this_os ne 'MacOS') then begin
		flat_groups(*).oblong_index = orbit.point.lightning.group(grlist).oblong_index
		onefmtstr = ['(D13.3)','(I3)','(F6.2)','(F7.2)','(E9.3)','(F6.1)','(I4)','(I4)','(I5)','(I3)','(I3)','(I3)','(I3)', $
			 '(I3)','(I3)','(F5.1)','(F5.1)','(I4)','(I1)']
		
		grexpstr = '((D13.3)," ",(I3)," ",(F6.2)," ",(F7.2)," ",(E9.3)," ",(F5.1)," ",(I4)," ",(I4)," ",(I5)," ",(I3)," ",(I3)," ",(I3)," ",(I3)," ",'+$
			 '(I3)," ",(I3)," ",(F5.1)," ",(F5.1)," ",(I4)," ",(I1))'
		collab = ['TAI93_time','observe_time','location[0]','location[1]','radiance','footprint','address','parent_address','child_address', 'child_count', $
		 'approx_threshold','alert_flag','cluster_index', $
		 'density_index','noise_index','glint_index','oblong_index','grouping_sequence','grouping_status']
		grxsz = 19
	endif else begin
		onefmtstr = ['(D13.3)','(I3)','(F6.2)','(F7.2)','(E9.3)','(F6.1)','(I4)','(I4)','(I5)','(I3)','(I3)','(I3)','(I3)', $
			 '(I3)','(I3)','(F5.1)','(I4)','(I1)']
		
		grexpstr = '((D13.3)," ",(I3)," ",(F6.2)," ",(F7.2)," ",(E9.3)," ",(F5.1)," ",(I4)," ",(I4)," ",(I5)," ",(I3)," ",(I3)," ",(I3)," ",(I3)," ",'+$
			 '(I3)," ",(I3)," ",(F5.1)," ",(I4)," ",(I1))'
		collab=['TAI93_time','observe_time','location[0]','location[1]','radiance','footprint','address','parent_address','child_address', 'child_count', $
		 'approx_threshold','alert_flag','cluster_index', $
		 'density_index','noise_index','glint_index','grouping_sequence','grouping_status']
		 grxsz = 18
	endelse

	fmtstr = strarr(grxsz,numgr)
	for i = 0,grxsz-1,1 do begin
		fmtstr(i,*) = onefmtstr(i)
	endfor
	
	grtabb = widget_base(title='Groups',group_leader=lisb,row=2)
	grtab1 = widget_base(grtabb,column=3)
	grtabexp = widget_button(grtab1,value='Save as ASCII...')
	grtabglob = widget_button(grtab1,value='Store in global struct')
	grtabdone = widget_button(grtab1,value='Done')
	grtab2 = widget_base(grtabb,column=1)
	grtab = widget_table(grtab2,value=flat_groups(*),/scroll,/editable,/resizeable_columns,xsize=grxsz,ysize=numgr,$
					x_scroll_size=12,y_scroll_size=15,column_labels=collab $
		,/alignment,format= $
		 fmtstr)
		 
	widget_control,waitb,/destroy
	widget_control,grtabb,/realize
	xmanager, 'GR Table', grtabb, event='gr_table_event', /no_block
	; 
end


pro gr_table_event, event
	common grtab_com, grexpstr, numgr, gridx, grtabexp, grtabdone, grtabb, grtabglob
	common datafiles, orbit, fname
	common lisbase, lisb
	common fullpaths, app_path, data_path, bg_path, output_path
	common paths, lastpath, currentpath, origpath, shlbfile
	common global_subsets, subset_events, subset_flashes, subset_groups, subset_areas, $
			subset_viewtimes, subset_one_seconds

	case 1 of 
		(event.id eq grtabdone):begin
				widget_control, grtabb, /destroy
			end
		(event.id eq grtabglob):begin
				subset_groups = replicate(orbit.point.lightning.group[0],numgr)
				subset_groups[*] = orbit.point.lightning.group[gridx]
				widget_control,grtabglob,sensitive=0
			end
		(event.id eq grtabexp):begin
					cd,output_path[0]
				fnameasc = dialog_pickfile(/noconfirm,/write,path=output_path[0])
				if (fnameasc ne '') then begin 
					openw,lun,fnameasc,/get_lun
					for i=0L,numgr-1L,1L do begin
						printf,lun,orbit.point.lightning.group[gridx[i]],format=grexpstr
					endfor
					close,lun
					free_lun,lun
					widget_control,grtabexp,sensitive=0
				endif
				cd,currentpath
			end
	endcase		
end

pro fl_table, event, fllist

	common datafiles, orbit, fname
	common lisbase, lisb
	common fltab_com, flexpstr, numfl, flidx, fltabexp, fltabdone, fltabb, fltabglob

	waitb = widget_base(title='Working',tlb_frame_attr=0,scroll=0,/modal,column=1,group_leader=lisb)
	waitm = widget_label(waitb,value='Preparing table ... this may take a moment...',/align_center)
	widget_control,waitb,/realize

	this_os = !version.os
	if (this_os ne 'MacOS') then begin
		template_flash = {TAI93_time:double(0), delta_time:float(0), observe_time:fix(0), location0:float(0),location1:float(0) ,radiance:float(0), $
				footprint:float(0), address:long(0), parent_address:long(0), child_address:long(0), child_count:long(0),      $
				grandchild_count:long(0), approx_threshold:byte(0), alert_flag:byte(0), cluster_index:byte(0), density_index: $
				byte(0), noise_index:byte(0), glint_index:float(0), oblong_index:float(0), grouping_sequence:long(0),         $
				grouping_status:byte(0)}
	endif else begin
		template_flash = {TAI93_time:0D, delta_time:0.0, observe_time:0, location0:float(0),location1:float(0) ,radiance:0.0, $
				footprint:0.0, address:0L, parent_address:0L, child_address:0L, child_count:0L,      $
				grandchild_count:0L, approx_threshold:byte(0), alert_flag:byte(0), cluster_index:byte(0), density_index: $
				byte(0), noise_index:byte(0), glint_index:0.0, grouping_sequence:0L,       $
				grouping_status:byte(0)}
	endelse
	
	fllsz = size(fllist)
	case fllsz[0] of 
		0:begin 
			numfl = 1
		  end
		1: begin
			numfl = fllsz[1]
		   end
	endcase
	flidx = fllist

	flat_flashs = replicate(template_flash,numfl)
	flat_flashs(*).TAI93_time = orbit.point.lightning.flash(fllist).TAI93_time
	flat_flashs(*).delta_time = orbit.point.lightning.flash(fllist).delta_time
	flat_flashs(*).observe_time = orbit.point.lightning.flash(fllist).observe_time
	flat_flashs(*).location0 = orbit.point.lightning.flash(fllist).location(0)
	flat_flashs(*).location1 = orbit.point.lightning.flash(fllist).location(1)
	flat_flashs(*).radiance = orbit.point.lightning.flash(fllist).radiance
	flat_flashs(*).footprint = orbit.point.lightning.flash(fllist).footprint
	flat_flashs(*).address = orbit.point.lightning.flash(fllist).address
	flat_flashs(*).parent_address = orbit.point.lightning.flash(fllist).parent_address
	flat_flashs(*).child_address = orbit.point.lightning.flash(fllist).child_address
	flat_flashs(*).child_count = orbit.point.lightning.flash(fllist).child_count
	flat_flashs(*).grandchild_count = orbit.point.lightning.flash(fllist).grandchild_count
	flat_flashs(*).approx_threshold = orbit.point.lightning.flash(fllist).approx_threshold
	flat_flashs(*).alert_flag = orbit.point.lightning.flash(fllist).alert_flag
	flat_flashs(*).cluster_index = orbit.point.lightning.flash(fllist).cluster_index
	flat_flashs(*).density_index = orbit.point.lightning.flash(fllist).density_index
	flat_flashs(*).noise_index = orbit.point.lightning.flash(fllist).noise_index
	flat_flashs(*).glint_index = orbit.point.lightning.flash(fllist).glint_index
	flat_flashs(*).grouping_sequence = orbit.point.lightning.flash(fllist).grouping_sequence
	flat_flashs(*).grouping_status = orbit.point.lightning.flash(fllist).grouping_status

	if (this_os ne 'MacOS') then begin
		flat_flashs(*).oblong_index = orbit.point.lightning.flash(fllist).oblong_index
		onefmtstr = ['(D13.3)','(F7.3)','(I3)','(F6.2)','(F7.2)','(E9.3)','(F6.1)','(I4)','(I4)','(I5)','(I4)','(I4)','(I3)','(I3)','(I3)','(I3)', $
			 '(I3)','(F5.1)','(F5.1)','(I4)','(I1)']
		
		flexpstr = '((D13.3)," ",(F7.3)," ",(I3)," ",(F6.2)," ",(F7.2)," ",(E9.3)," ",(F6.1)," ",(I4),'+$
		'" ",(I4)," ",(I5)," ",(I4)," ",(I4)," ",'+$
		'(I3)," ",(I3)," ",(I3)," ",'+$
			 '(I3)," ",(I3)," ",(F5.1)," ",(F5.1)," ",(I4)," ",(I1))'
		collab = ['TAI93_time','delta_time','observe_time','location[0]','location[1]','radiance','footprint','address',$
		 'parent_address','child_address', 'child_count', 'grandchild_count',$
		 'approx_threshold','alert_flag','cluster_index', $
		 'density_index','noise_index','glint_index','oblong_index','grouping_sequence','grouping_status']
		flxsz = 21
	endif else begin
		onefmtstr = ['(D13.3)','(F7.3)','(I3)','(F6.2)','(F7.2)','(E9.3)','(F6.1)','(I4)','(I4)','(I5)','(I4)','(I4)','(I3)','(I3)','(I3)','(I3)', $
			 '(I3)','(F5.1)','(I4)','(I1)']
		
		flexpstr = '((D13.3)," ",(F7.3)," ",(I3)," ",(F6.2)," ",(F7.2)," ",(E9.3)," ",(F6.1)," ",(I4),'+$
		'" ",(I4)," ",(I5)," ",(I4)," ",(I4)," ",'+$
		'(I3)," ",(I3)," ",(I3)," ",'+$
			 '(I3)," ",(I3)," ",(F5.1)," ",(I4)," ",(I1))'
		collab = ['TAI93_time','delta_time','observe_time','location[0]','location[1]','radiance','footprint','address',$
		 'parent_address','child_address', 'child_count', 'grandchild_count',$
		 'approx_threshold','alert_flag','cluster_index', $
		 'density_index','noise_index','glint_index','grouping_sequence','grouping_status']
		 flxsz = 20
	endelse

	fmtstr = strarr(flxsz,numfl)
	for i = 0,flxsz-1,1 do begin
		fmtstr(i,*) = onefmtstr(i)
	endfor
	
	fltabb = widget_base(title='Flashes',group_leader=lisb,row=2)
	fltab1 = widget_base(fltabb,column=3)
	fltabexp = widget_button(fltab1,value='Save as ASCII...')
	fltabglob = widget_button(fltab1,value='Store in global struct')
	fltabdone = widget_button(fltab1,value='Done')
	fltab2 = widget_base(fltabb,column=1)
	fltab = widget_table(fltab2,value=flat_flashs(*),/scroll,/editable,/resizeable_columns,xsize=flxsz,ysize=numfl,$
					x_scroll_size=12,y_scroll_size=15,column_labels=collab $
		,/alignment,format= $
		 fmtstr)
		 
	widget_control,waitb,/destroy
	widget_control,fltabb,/realize
	xmanager, 'FL Table', fltabb, event='fl_table_event', /no_block
	; 
end


pro fl_table_event, event
	common fltab_com, flexpstr, numfl, flidx, fltabexp, fltabdone, fltabb, fltabglob
	common datafiles, orbit, fname
	common lisbase, lisb
	common fullpaths, app_path, data_path, bg_path, output_path
	common paths, lastpath, currentpath, origpath, shlbfile
	common global_subsets, subset_events, subset_flashes, subset_groups, subset_areas, $
			subset_viewtimes, subset_one_seconds

	case 1 of 
		(event.id eq fltabdone):begin
				widget_control, fltabb, /destroy
			end
		(event.id eq fltabglob):begin
				subset_flashes = replicate(orbit.point.lightning.flash[0],numfl)
				subset_flashes[*] = orbit.point.lightning.flash[flidx]
				widget_control,fltabglob,sensitive=0
			end
		(event.id eq fltabexp):begin
					cd,output_path[0]
				fnameasc = dialog_pickfile(/noconfirm,/write,path=output_path[0])
				if (fnameasc ne '') then begin 
					openw,lun,fnameasc,/get_lun
					for i=0L,numfl-1L,1L do begin
						printf,lun,orbit.point.lightning.flash[flidx[i]],format=flexpstr
					endfor
					close,lun
					free_lun,lun
					widget_control,fltabexp,sensitive=0
				endif
				cd,currentpath
			end
	endcase		
end

pro ar_table, event, arlist

	common datafiles, orbit, fname
	common lisbase, lisb
	common artab_com, arexpstr, numar, aridx, artabexp, artabdone, artabb, artabglob

	waitb = widget_base(title='Working',tlb_frame_attr=0,scroll=0,/modal,column=1,group_leader=lisb)
	waitm = widget_label(waitb,value='Preparing table ... this may take a moment...',/align_center)
	widget_control,waitb,/realize

	this_os = !version.os
	if (this_os ne 'MacOS') then begin
		template_area = {TAI93_time:double(0),delta_time:float(0),observe_time:fix(0),location0:0.0,location1:0.0,net_radiance:float(0), $
				footprint:float(0),address:long(0),parent_address:long(0),child_address:long(0),child_count:long(0),       $
				grandchild_count:long(0),greatgrandchild_count:long(0),approx_threshold:byte(0),alert_flag:byte(0),cluster_index: $
				byte(0),density_index:byte(0),noise_index:byte(0),oblong_index:float(0),grouping_sequence:long(0),         $
				grouping_status:byte(0)}
	endif else begin
		template_area = {TAI93_time:0D,delta_time:0.0,observe_time:0,location0:0.0,location1:0.0,net_radiance:0.0, $
				footprint:0.0,address:0L,parent_address:0L,child_address:0L,child_count:0L,       $
				grandchild_count:0L,greatgrandchild_count:0L,approx_threshold:byte(0),alert_flag:byte(0),cluster_index: $
				byte(0),density_index:byte(0),noise_index:byte(0),         $
				grouping_status:byte(0)}
	endelse
	
	arlsz = size(arlist)
	case arlsz[0] of 
		0:begin 
			numar = 1
		  end
		1: begin
			numar = arlsz[1]
		   end
	endcase
	aridx = arlist

	flat_areas = replicate(template_area,numar)
	flat_areas(*).TAI93_time = orbit.point.lightning.area(arlist).TAI93_time
	flat_areas(*).delta_time = orbit.point.lightning.area(arlist).delta_time
	flat_areas(*).observe_time = orbit.point.lightning.area(arlist).observe_time
	flat_areas(*).location0 = orbit.point.lightning.area(arlist).location(0)
	flat_areas(*).location1 = orbit.point.lightning.area(arlist).location(1)
	flat_areas(*).net_radiance = orbit.point.lightning.area(arlist).net_radiance
	flat_areas(*).footprint = orbit.point.lightning.area(arlist).footprint
	flat_areas(*).address = orbit.point.lightning.area(arlist).address
	flat_areas(*).parent_address = orbit.point.lightning.area(arlist).parent_address
	flat_areas(*).child_address = orbit.point.lightning.area(arlist).child_address
	flat_areas(*).child_count = orbit.point.lightning.area(arlist).child_count
	flat_areas(*).grandchild_count = orbit.point.lightning.area(arlist).grandchild_count
	flat_areas(*).greatgrandchild_count = orbit.point.lightning.area(arlist).greatgrandchild_count
	flat_areas(*).approx_threshold = orbit.point.lightning.area(arlist).approx_threshold
	flat_areas(*).alert_flag = orbit.point.lightning.area(arlist).alert_flag
	flat_areas(*).cluster_index = orbit.point.lightning.area(arlist).cluster_index
	flat_areas(*).density_index = orbit.point.lightning.area(arlist).density_index
	flat_areas(*).noise_index = orbit.point.lightning.area(arlist).noise_index
	flat_areas(*).grouping_status = orbit.point.lightning.area(arlist).grouping_status

	if (this_os ne 'MacOS') then begin
		flat_areas(*).oblong_index = orbit.point.lightning.area(arlist).oblong_index
		flat_areas(*).grouping_sequence = orbit.point.lightning.area(arlist).grouping_sequence
		onefmtstr = ['(D13.3)','(F7.3)','(I3)','(F6.2)','(F7.2)','(E9.3)','(F6.1)','(I4)','(I4)','(I4)','(I4)','(I4)','(I4)','(I3)','(I3)','(I3)','(I3)', $
			 '(I3)','(F5.1)','(I4)','(I1)']
		
		arexpstr = '((D13.3)," ",(F7.3)," ",(I3)," ",(F6.2)," ",(F7.2)," ",(E9.3)," ",(F6.1)," ",(I4),'+$
		'" ",(I4)," ",(I4)," ",(I4)," ",(I4)," ",(I4)," ",'+$
		'(I3)," ",(I3)," ",(I3)," ",'+$
			 '(I3)," ",(I3)," ",(F5.1)," ",(I4)," ",(I1))'
		collab = ['TAI93_time','delta_time','observe_time','location[0]','location[1]','radiance','footprint','address',$
		 'parent_address','child_address', 'child_count', 'grandchild_count','greatgrandchild_count',$
		 'approx_threshold','alert_flag','cluster_index', $
		 'density_index','noise_index','oblong_index','grouping_sequence','grouping_status']
		arxsz = 21
	endif else begin
		onefmtstr = ['(D13.3)','(F7.3)','(I3)','(F6.2)','(F7.2)','(E9.3)','(F6.1)','(I4)','(I4)','(I4)','(I4)','(I4)','(I4)','(I3)','(I3)','(I3)','(I3)', $
			 '(I3)','(I1)']
		
		arexpstr = '((D13.3)," ",(F7.3)," ",(I3)," ",(F6.2)," ",(F7.2)," ",(E9.3)," ",(F6.1)," ",(I4),'+$
		'" ",(I4)," ",(I4)," ",(I4)," ",(I4)," ",(I4)," ",'+$
		'(I3)," ",(I3)," ",(I3)," ",'+$
			 '(I3)," ",(I3)," ",(I1))'
		collab = ['TAI93_time','delta_time','observe_time','location[0]','location[1]','radiance','footprint','address',$
		 'parent_address','child_address', 'child_count', 'grandchild_count','greatgrandchild_count',$
		 'approx_threshold','alert_flag','cluster_index', $
		 'density_index','noise_index','grouping_status']
		 arxsz = 19
	endelse

	fmtstr = strarr(arxsz,numar)
	for i = 0,arxsz-1,1 do begin
		fmtstr(i,*) = onefmtstr(i)
	endfor
	
	artabb = widget_base(title='Areas',group_leader=lisb,row=2)
	artab1 = widget_base(artabb,column=3)
	artabexp = widget_button(artab1,value='Save as ASCII...')
	artabglob = widget_button(artab1,value='Store in global struct')
	artabdone = widget_button(artab1,value='Done')
	artab2 = widget_base(artabb,column=1)
	artab = widget_table(artab2,value=flat_areas(*),/scroll,/editable,/resizeable_columns,xsize=arxsz,ysize=numar,$
					x_scroll_size=12,y_scroll_size=15,column_labels=collab $
		,/alignment,format= $
		 fmtstr)
		 
	widget_control,waitb,/destroy
	widget_control,artabb,/realize
	xmanager, 'AR Table', artabb, event='ar_table_event', /no_block
	; 
end


pro ar_table_event, event
	common artab_com, arexpstr, numar, aridx, artabexp, artabdone, artabb, artabglob
	common datafiles, orbit, fname
	common fullpaths, app_path, data_path, bg_path, output_path
	common paths, lastpath, currentpath, origpath, shlbfile
	common lisbase, lisb
	common global_subsets, subset_events, subset_flashes, subset_groups, subset_areas, $
			subset_viewtimes, subset_one_seconds

	case 1 of 
		(event.id eq artabdone):begin
				widget_control, artabb, /destroy
			end
		(event.id eq artabglob):begin
				subset_areas = replicate(orbit.point.lightning.area[0],numar)
				subset_areas[*] = orbit.point.lightning.area[aridx]
				widget_control,artabglob,sensitive=0
			end
		(event.id eq artabexp):begin
					cd,output_path[0]
				fnameasc = dialog_pickfile(/noconfirm,/write,path=output_path[0])
				if (fnameasc ne '') then begin 
					openw,lun,fnameasc,/get_lun
					for i=0L,numar-1L,1L do begin
						printf,lun,orbit.point.lightning.area[aridx[i]],format=arexpstr
					endfor
					close,lun
					free_lun,lun
					widget_control,artabexp,sensitive=0
				endif
				cd,currentpath
			end
	endcase		
end

pro select_bgs, initial_selection

	common datafiles, orbit, fname
	common lisbase, lisb
	common select_bg_ids, bgs_selected, sbg_draw_id, sbg_select, sbg_base
	common bglists, bglist, sbg_list
	common lisapp_colors, red,green,blue
	
	bgs_selected = bytarr(orbit.point.point_summary.bg_count)
	bgs_selected = initial_selection
	
	dtarr = new_datetime_structure(orbit.point.point_summary.bg_count)
	dtarr(*).TAI93 = orbit.point.bg_summary(*).TAI93_time
	calc_datetime, dtarr
	bglist = strarr(orbit.point.point_summary.bg_count)
	
	for i=0,orbit.point.point_summary.bg_count-1,1 do begin
		bglist(i) = string(format='("  ",(I3.3)," ",(I2.2),":",(I2.2),":",(I2.2)," ",(F6.2)," ",(F7.2))', $
			i, dtarr(i).utc.hour, dtarr(i).utc.minute, fix(dtarr(i).utc.second), 			  $
			orbit.point.bg_summary(i).boresight(0), orbit.point.bg_summary(i).boresight(1))
	endfor
		
	sbg_base = widget_base(column=1,tlb_frame_attr=8, xsize=800, ysize=620, scroll=0,/align_center,/base_align_center,group_leader=lisb,$
					title='Select Backgrounds to Plot')
	sbg_sel_base = widget_base(sbg_base,column=2,/align_center,xsize=790,ysize=240,grid_layout=0)
	sbg_plt_base = widget_base(sbg_base,column=1,/align_center,xsize=790,ysize=440)
	sbg_list1_base = widget_base(sbg_sel_base,column=1,/align_center,xsize=415,ysize=220)
	; sbg_list2_base = widget_base(sbg_list1_base,row=1,/align_center)
	sbg_list = widget_list(sbg_list1_base,event_pro='handle_sbg_list',ysize=7,value=bglist,/align_center,xsize=31)
	sbg_button_base1 = widget_base(column=1,sbg_sel_base,/align_right,xsize=340)
	sbg_button_base2 = widget_base(row=2,sbg_button_base1)
	sbg_br1_base = widget_base(sbg_button_base2,column=2)
	sbg_br1c1_base = widget_base(sbg_br1_base,column=1,/align_right,xsize=160)
	sbg_br1c2_base = widget_base(sbg_br1_base,column=1,/align_right,xsize=160)
	sbg_br2_base = widget_base(sbg_button_base2,column=2,/align_right)
	sbg_select = cw_bgroup(sbg_br1c1_base,['Select','Deselect'],/exclusive,event_func='select_bgs_toggle',/no_release,xsize=155)
	sbg_select_all = widget_button(sbg_br1c2_base,value='Select All',/align_center,xsize=155)
	sbg_deselect_all = widget_button(sbg_br1c2_base,value='Deselect All',/align_center,xsize=155)
	sbg_plot = widget_button(sbg_br2_base,value='Plot',/align_left,xsize=140)
	sbg_quit = widget_button(sbg_br2_base,value='Cancel',/align_right,xsize=140)
	sbg_draw = widget_draw(sbg_plt_base,colors=220,xsize=740,ysize=370,/align_center,retain=2)
	
	widget_control,sbg_base,/realize
	widget_control,sbg_select,sensitive=0
	
	Widget_Control, sbg_draw, get_value=sbg_draw_id
	
	wset,sbg_draw_id & 	if (not iswindows(0)) then device,pseu=8
	
	tvlct,red,green,blue
	
	; map_set,/continents,E_Continents={countries:1},/usa,xmargin=[0,0],ymargin=[0,0],/noerase,/noborder
	map_set,/continents,xmargin=[0,0],ymargin=[0,0],/noerase,/noborder,color=64
	nadirs = fltarr(2,orbit.orbit_summary.one_second_count)
	get_nadir_locations, orbit.one_second(*).TAI93_time, nadirs
	thrcol = bytarr(orbit.orbit_summary.one_second_count)
	thrcol = 8+fix(4.31*float(orbit.one_second(*).boresight_threshold))
	usersym,sin(findgen(21)/10.*!pi),cos(findgen(21)/10.*!pi),/fill
	uscale = usersym_scale(0)
	ypos = fltarr(orbit.orbit_summary.one_second_count)
	ypos(*) = -87.0
	ssize = .05+fix(740/200)*.10*uscale
	plots,nadirs[1,*],ypos,color=thrcol,psym=8,symsize=ssize
	plots,nadirs[1,*],-1*ypos,color=thrcol,psym=8,symsize=ssize
	flcol = 133
	twocol = [110,88]
	plotcol = bytarr(orbit.point.point_summary.bg_count)
	for i =0, orbit.point.point_summary.bg_count-1,1 do begin
		plotcol[i] = twocol[bgs_selected[i]]
		if (bgs_selected[i] eq 1) then begin
			toggle_list_entry, i
		endif
	endfor
	plots,orbit.point.bg_summary(*).boresight(1),orbit.point.bg_summary(*).boresight(0),psym=4,color=plotcol
	plots,orbit.point.lightning.flash(*).location(1),orbit.point.lightning.flash(*).location(0),psym=3,color=flcol

	xmanager, 'select_bgs', sbg_base, event='select_bgs_event',/no_block
	
end

function select_bgs_toggle, event

	common datafiles, orbit, fname
	common lisbase, lisb
	common select_bg_ids, bgs_selected, sbg_draw_id, sbg_select, sbg_base
	common bglists, bglist, sbg_list
	common lisapp_colors,red,green,blue
	
	widget_control, event.id, get_value=buttonvalue
	widget_control, event.id, get_uvalue=bgidx
	
	wset,sbg_draw_id & 	if (not iswindows(0)) then device,pseu=8
	map_set,xmargin=[0,0],ymargin=[0,0],/noerase,limit=[-90,-180,90,180],/noborder,color=64
	
	tvlct,red,green,blue

	if (bgs_selected(bgidx) ne (1-buttonvalue)) then begin
		bgs_selected(bgidx) = 1-buttonvalue
		wset,sbg_draw_id & 	if (not iswindows(0)) then device,pseu=8
		plots,orbit.point.bg_summary(bgidx).boresight(1),orbit.point.bg_summary(bgidx).boresight(0),psym=4,color=88+(22*(1-bgs_selected(bgidx)))	
		toggle_list_entry,bgidx
		widget_control,sbg_list,set_list_select=bgidx
	endif
	return, bgidx
end

pro select_bgs_event, event

	common datafiles, orbit, fname
	common lisbase, lisb
	common select_bg_ids, bgs_selected, sbg_draw_id, sbg_select, sbg_base
	common bglists, bglist, sbg_list
	common lisapp_colors,red,green,blue
	
	widget_control, event.id, get_value=buttonvalue
	widget_control, sbg_select,get_uvalue=bgidx
	wset,sbg_draw_id & 	if (not iswindows(0)) then device,pseu=8
	map_set,xmargin=[0,0],ymargin=[0,0],/noerase,limit=[-90,-180,90,180],/noborder,color=64
	
	tvlct,red,green,blue
	
	case buttonvalue of
		'Plot':	begin
						widget_control, sbg_base, sensitive=0
						widget_control, sbg_list, sensitive=0
						plot_bgs,orbit.point.bg_summary(*),bgs_selected,'select_bgs'
				end
		'Cancel':	begin
						bglist = 0 & sbg_list = 0 & bgs_selected = 0 
						select_bg_ids = 0 & sbg_select = 0 
						widget_control,sbg_base,/destroy
				end
		'Select All': 	begin
						bgs_selected(*)=1
						plots,orbit.point.bg_summary(*).boresight(1),orbit.point.bg_summary(*).boresight(0),psym=4,color=88
						widget_control,sbg_select,set_value=0
						for i=0,orbit.point.point_summary.bg_count-1,1 do begin
							entry = bglist(i)
							retval = string(entry)
							bretval = byte(retval)
							bretval(0) = 42
							entry = string(bretval)
							bglist(i)=entry
						endfor
						widget_control,sbg_list,set_value=bglist							
					end
		'Deselect All': begin
						bgs_selected(*)=0
						plots,orbit.point.bg_summary(*).boresight(1),orbit.point.bg_summary(*).boresight(0),psym=4,color=110
						widget_control,sbg_select,set_value=1
						for i=0,orbit.point.point_summary.bg_count-1,1 do begin
							entry = bglist(i)
							retval = string(entry)
							bretval = byte(retval)
							bretval(0) = 32
							entry = string(bretval)
							bglist(i)=entry
						endfor
						widget_control,sbg_list,set_value=bglist							
					end
	endcase
	
end	

pro handle_sbg_list, event

	common datafiles, orbit, fname
	common lisbase, lisb
	common select_bg_ids, bgs_selected, sbg_draw_id, sbg_select, sbg_base
	common bglists, bglist, sbg_list
	common lisapp_colors, red,green,blue
	
	wset,sbg_draw_id & 	if (not iswindows(0)) then device,pseu=8
	map_set,xmargin=[0,0],ymargin=[0,0],/noerase,limit=[-90,-180,90,180],/noborder,color=64
	
	tvlct,red,green,blue

	bgidx = event.index
	bgclix = event.clicks
	
	widget_control,sbg_select,/sensitive
	
	if (bgclix eq 1) then begin
	
		widget_control,sbg_select,set_value=1-bgs_selected(bgidx)
		widget_control,sbg_select,set_uvalue=bgidx
	
	endif else begin
	
		bgs_selected(bgidx) = 1-bgs_selected(bgidx)
		widget_control,sbg_select,set_value=1-bgs_selected(bgidx)
		toggle_list_entry,bgidx
		widget_control,sbg_list,set_list_select=bgidx
		wset,sbg_draw_id & 	if (not iswindows(0)) then device,pseu=8
		plots,orbit.point.bg_summary(bgidx).boresight(1),orbit.point.bg_summary(bgidx).boresight(0),psym=4,color=88+(22*(1-bgs_selected(bgidx)))
		widget_control,sbg_select,set_uvalue=bgidx
		
	endelse

end

pro change_entry_string, entry

		retval = string(entry)
		bretval = byte(retval)
		if (bretval(0) eq 42) then begin
			bretval(0) = 32
		endif else begin
			bretval(0) = 42
		endelse
		entry = string(bretval)
end

pro toggle_list_entry, idx

	common bglists, bglist, sbg_list
	
	entry = bglist(idx)
	change_entry_string, entry
	bglist(idx) = entry
	widget_control,sbg_list,set_value=bglist
	
end

pro plot_bgs, allbginfo, selected, callpro
	common sensor_read, this_sensor, file_format
	common datafiles, orbit, fname
	common otd_boom, boom
	common bgcommon, bgplotb, bgplotwin, bgimg, bgoverlay
	common bgcommon2, bg_grid, flatmin, flatmax, flonmin, flonmax
	common bgcommon3, overlay_indices, bgplotoverlay1, bgplotoverlay2, lightning_overlays, diagnostic_overlays
	common caller, callingfunction 
	
	common lisapp_colors,red,green,blue
	common prefs, readparams, readparamsotd, setparams, setparamsotd, gotparams, gotparamsotd
	common lisapp_prefs, app_prefs
	common bg_info, fnamestub, candobgs, bgname, bgpathtouse
	
	callingfunction=callpro
	
	bginfo = allbginfo(where(selected eq 1,count))
	; print,count,' selected'
	
	if (count lt 1) then begin
		print, "No backgrounds were selected!"
		stop
	endif else begin
	
		bgwaitb = widget_base(title='Working',tlb_frame_attr=0,scroll=0,column=2,group_leader=lisb)
		bgwaittop = widget_base(bgwaitb,column=1,/base_align_center)
		bgwaitbot = widget_base(bgwaitb,column=1,xsize=150,ysize=150,/align_center)
		bgwaitm = widget_label(bgwaittop,value='Preparing image ... this may take a moment...',/align_center)
		bgwaitdraw = widget_draw(bgwaitbot,colors=220,xsize=128,ysize=128,retain=2,/align_center)
		
		widget_control,bgwaitb,/realize
		Widget_Control, bgwaitdraw, get_value=bgwaitdraw_id

		num_requested = count
		bgs_requested = intarr(128,128,num_requested)
	     	bg_indices = where(selected eq 1,count)
		widget_control,bgwaitm,set_value='Loading image data...'

		filename = bgname
				
		fileid=HDF_OPEN(filename,/read)
	
		sd_id=HDF_SD_START(filename,/read)
		HDF_SD_FILEINFO,sd_id,nmfsds,nglobatts
		
		wset,bgwaitdraw_id & 	if (not iswindows(0)) then device,pseu=8
		
		tvlct,red,green,blue
		
		for i=0,nmfsds-1 do begin
			sds_id=HDF_SD_SELECT(sd_id,i)
		     	HDF_SD_GETINFO,sds_id,name=n,ndims=r,type=t,natts=nats,$
		                         hdf_type=h,unit=u
		          	          	
		     	if r gt 0 then HDF_SD_GETINFO,sds_id,dims=dims else dims=0
		
			for j=0,nats-1 do begin
			 	HDF_SD_ATTRINFO,sds_id,j,name=n,type=t,count=c,data=d,$
		                            hdf_type=h
		     	endfor
		       		  
	          	inbg = intarr(128,128)
	          	for iii=0,num_requested-1,1 do begin
	          		; print,iii
	          		hdf_sd_getdata,sds_id,inbg,start=[0,0,bg_indices(iii)],count=[128,128,1]
				bgs_requested(*,*,iii) = inbg	
				msgstr = string(format='("Loading image ",(I3)," of ",(I3))',iii+1,num_requested)
				widget_control,bgwaitm,set_value=msgstr
	          		tv,fix(64.*float(inbg)/float(max(inbg)))
	          	endfor
	
			HDF_SD_ENDACCESS,sds_id
		endfor
		for i=0,nglobatts-1 do begin
			HDF_SD_ATTRINFO,sd_id,i,name=n,type=t,count=c,data=d
		endfor
				   
		HDF_SD_END,sd_id
	    
		
		minlat = fltarr(4)
		minlat(0) = min(bginfo(*).corners(0))
		minlat(1) = min(bginfo(*).corners(2))
		minlat(2) = min(bginfo(*).corners(4))
		minlat(3) = min(bginfo(*).corners(6))
		minminlat = min(minlat)
		maxlat = fltarr(4)
		maxlat(0) = max(bginfo(*).corners(0))
		maxlat(1) = max(bginfo(*).corners(2))
		maxlat(2) = max(bginfo(*).corners(4))
		maxlat(3) = max(bginfo(*).corners(6))
		maxmaxlat = max(maxlat)
		minlon = fltarr(4)
		minlon(0) = min(bginfo(*).corners(1))
		minlon(1) = min(bginfo(*).corners(3))
		minlon(2) = min(bginfo(*).corners(5))
		minlon(3) = min(bginfo(*).corners(7))
		minminlon = min(minlon)
		maxlon = fltarr(4)
		maxlon(0) = max(bginfo(*).corners(1))
		maxlon(1) = max(bginfo(*).corners(3))
		maxlon(2) = max(bginfo(*).corners(5))
		maxlon(3) = max(bginfo(*).corners(7))
		maxmaxlon = max(maxlon)
		
		if (this_sensor eq 'otd') then begin
			slop = 4.
		endif else begin
			slop = 2.
		endelse
		
		ilonmin = max([fix(minminlon-slop),-180])
		ilonmax = min([fix(maxmaxlon+slop),180])
		ilatmin = max([fix(minminlat-slop),-90])
		ilatmax = min([fix(maxmaxlat+slop),90])
		
		flonmin = float(ilonmin) & flonmax = float(ilonmax)
		flatmin = float(ilatmin) & flatmax = float(ilatmax)
		
		; Compute resolution to yield a target screen size of no larger than (targetsize) pixels
		
		target_size = float(app_prefs.graphics.targetsize)
		
		dellat = flatmax-flatmin
		dellon = flonmax-flonmin
		
		if (dellat ge dellon) then begin
			; Portrait-ish
			cellsperdeg = fix(target_size/dellat)
		endif else begin
			; Landscape-ish
			cellsperdeg = fix(target_size/dellon)
		endelse
		
			
		; cellsperdeg = 10
		gridsize = 1.0/float(cellsperdeg)
		
		gxsize = (ilonmax-ilonmin)*cellsperdeg
		gysize = (ilatmax-ilatmin)*cellsperdeg
		
		bg_grid = intarr(gxsize,gysize)
				
		get_array_pointing_vector,this_sensor,array_look_vector
		
		for i = 0, num_requested-1, 1 do begin
			; print,i
			msgstr = string(format='("Geolocating image ",(I3)," of ",(I3))',i+1,num_requested)
			widget_control,bgwaitm,set_value=msgstr
			tv,fix(64.*float(bgs_requested(*,*,i))/float(max(bgs_requested(*,*,i))))
			get_array_earth_intersection,array_look_vector,bginfo(i).TAI93_time,array_location, 0.0D, return_code
			if (return_code eq 0) then begin
				xlocns0 = reform(fix((array_location(*,*,1)-flonmin)*cellsperdeg),1,16384)
				ylocns0 = reform(fix((array_location(*,*,0)-flatmin)*cellsperdeg),1,16384)
				
				; *** Kludge for LIS
				
				if (this_sensor eq 'lis') then begin
					; Kludge ...  lingrid0 = reform(rotate(bgs_requested(*,*,i),2),1,16384)
					lingrid0 = reform(bgs_requested(*,*,i),1,16384)
				endif else begin
					lingrid0 = reform(bgs_requested(*,*,i),1,16384)
				endelse
				
				if (this_sensor eq 'otd') then begin
					xlocns = xlocns0(where(boom ne 0))
					ylocns = ylocns0(where(boom ne 0))
					lingrid = lingrid0(where(boom ne 0))
				endif else begin
					xlocns = xlocns0
					ylocns = ylocns0
					lingrid = lingrid0
				endelse
				xp1 = xlocns+1
				xm1 = xlocns-1
				yp1 = ylocns+1
				ym1 = ylocns-1
				bg_grid(xlocns,ylocns) = lingrid
				bg_grid(xm1,ym1) = lingrid
				bg_grid(xlocns,ym1) = lingrid
				bg_grid(xp1,ym1) = lingrid
				bg_grid(xm1,ylocns) = lingrid
				bg_grid(xp1,ylocns) = lingrid
				bg_grid(xm1,yp1) = lingrid
				bg_grid(xlocns,yp1) = lingrid
				bg_grid(xp1,yp1) = lingrid
				
			endif
		endfor
		
		poss_overlays = total(gotparams(0:3))+1
		lightning_overlays = strarr(poss_overlays)
		overlay_indices = intarr(poss_overlays)
		
		lightning_overlays(0)='(None)'
		if (poss_overlays gt 1) then begin
			i=1
			if (gotparams(0) ne 0) then begin
				lightning_overlays(i) = 'Areas'
				overlay_indices(i)=1
				i = i+1
			endif
			if (gotparams(1) ne 0) then begin
				lightning_overlays(i) = 'Flashes'
				overlay_indices(i)=2
				i = i+1
			endif
			if (gotparams(2) ne 0) then begin
				lightning_overlays(i) = 'Groups'
				overlay_indices(i)=3
				i = i+1
			endif
			if (gotparams(3) ne 0) then begin
				lightning_overlays(i) = 'Events'
				overlay_indices(i)=4
				i = i+1
			endif
		endif
		if (gotparams(5) ne 0) then begin
			diagnostic_overlays=['(None)','Alert Summary','Instrument Alert','Platform Alert','External Alert','Processing Alert', $
									'Ephemeris Quality Flag','Attitude Quality Flag','Noise Index']
		endif else begin
			diagnostic_overlays=['(None Available)']
		endelse
		bgplotb = widget_base(group_leader=lisb,row=2)
		bgplotcontrolb = widget_base(bgplotb,column=2)
		bgplotbutb = widget_base(bgplotcontrolb,row=3,/base_align_center,/align_center)
		bgplotbutlive = widget_button(bgplotbutb,value='Edit & Print')
		bgplotbutsave = widget_button(bgplotbutb,value='Save as GIF')
		bgplotbutdone = widget_button(bgplotbutb,value='Done')
		bgplotpulldownb = widget_base(bgplotcontrolb,row=2)
		bgplotoverlay1 = widget_droplist(bgplotpulldownb,title='Lightning Overlay',value=lightning_overlays,event_pro='ltgoverlay_handler')
		bgplotoverlay2 = widget_droplist(bgplotpulldownb,title='Diagnostic Overlay',value=diagnostic_overlays,event_pro='diagoverlay_handler')
		bgplotwinb = widget_base(bgplotb,column=1,/align_center,/base_align_center,xsize=max([gxsize,600]),ysize=gysize)
		bgplotwind = widget_draw(bgplotwinb,colors=220,xsize=gxsize,ysize=gysize,/align_center,retain=2)
		
		widget_control,bgwaitb,/destroy	
		
		widget_control,bgplotb,/realize
		widget_control,bgplotwind,get_value=bgplotwin
		widget_control,bgplotoverlay1,set_droplist_select=0
		widget_control,bgplotoverlay1,set_uvalue=lightning_overlays(0)
		widget_control,bgplotoverlay2,set_droplist_select=0
		widget_control,bgplotoverlay2,set_uvalue=diagnostic_overlays(0)
		if (gotparams(5) eq 0) then begin
			widget_control,bgplotoverlay2,sensitive=0
		endif
		if ((flatmax-flatmin gt 30) or (flonmax-flonmin gt 30)) then begin
			dellat = 2 
			dellon = 2
		endif else begin
			dellat = 1
			dellon = 1
		endelse	

		if (app_prefs.graphics.fill eq 1) then begin
			window,colors=220,16,xsize=gxsize,ysize=gysize,retain=2,/pixmap
			tvlct,red,green,blue
			basebaseimg = tvrd()
			basebaseimg(*,*) = 134
			tv,basebaseimg
				map_set,/continents,e_continents={usa:1,hires:app_prefs.graphics.hires,coasts:app_prefs.graphics.coasts,fill:app_prefs.graphics.fill,color:135},limit=$
				[flatmin,flonmin,flatmax,flonmax],xmargin=[0,0],ymargin=[0,0],/noerase,/noborder,color=46,label=1,latdel=dellat,londel=dellon
			basebaseimg = tvrd()
			window,colors=220,15,xsize=gxsize,ysize=gysize,retain=2,/pixmap
			if (not iswindows(0)) then device,pseu=8
			tvlct,red,green,blue
			gridimage = fix(64.*float(bg_grid)/float(max(bg_grid)))
			withvals = where(bg_grid ne 0)
			basebaseimg(withvals) = gridimage(withvals)
			tv,basebaseimg		
			map_set,/continents,e_continents={usa:1,hires:app_prefs.graphics.hires,coasts:app_prefs.graphics.coasts},limit=[flatmin,$
							flonmin,flatmax,flonmax],xmargin=[0,0],ymargin=[0,0],/noerase,/noborder,color=46,label=1,latdel=dellat,londel=dellon
		endif else begin
			window,colors=220,15,xsize=gxsize,ysize=gysize,retain=2,/pixmap
			if (not iswindows(0)) then device,pseu=8
			tvlct,red,green,blue
			gridimage = fix(64.*float(bg_grid)/float(max(bg_grid)))
			tv,gridimage		
			map_set,/continents,e_continents={usa:1,hires:app_prefs.graphics.hires,coasts:app_prefs.graphics.coasts},$
							limit=[flatmin,flonmin,flatmax,flonmax],xmargin=[0,0],ymargin=[0,0],/noerase,/noborder,color=64,label=1,latdel=dellat,londel=dellon
		endelse
		
		bgimg = tvrd()

		draw_bgimg_base
		
		xmanager,'Geolocated Background Plot',bgplotb,event_handler='bgplothandler',/no_block
		
	endelse
end

pro draw_bgimg_base
	common bgcommon, bgplotb, bgplotwin, bgimg, bgoverlay
	common bgcommon2, bg_grid, flatmin, flatmax, flonmin, flonmax
	common bgcommon3, overlay_indices, bgplotoverlay1, bgplotoverlay2, lightning_overlays, diagnostic_overlays
	common lisapp_colors, red, green, blue
	
	wset,bgplotwin & 	if (not iswindows(0)) then device,pseu=8
	tvlct,red,green,blue
	tv,bgimg
	map_set,/noerase,limit=[flatmin,flonmin,flatmax,flonmax],xmargin=[0,0],ymargin=[0,0],/noborder
end


pro ltgoverlay_handler, event
	common bgcommon, bgplotb, bgplotwin, bgimg, bgoverlay
	common bgcommon2, bg_grid, flatmin, flatmax, flonmin, flonmax
	common bgcommon3, overlay_indices, bgplotoverlay1, bgplotoverlay2, lightning_overlays, diagnostic_overlays

	widget_control,bgplotoverlay1,set_uvalue=lightning_overlays(event.index)
	ltgoverlay_selection = lightning_overlays(event.index)
	widget_control,bgplotoverlay2,get_uvalue=diagoverlay_selection
	
	draw_bgimg_base
	add_ltg_overlay, ltgoverlay_selection
	add_diag_overlay, diagoverlay_selection
end

pro add_ltg_overlay, selection
	common bgcommon, bgplotb, bgplotwin, bgimg, bgoverlay
	common bgcommon2, bg_grid, flatmin, flatmax, flonmin, flonmax
	common bgcommon3, overlay_indices, bgplotoverlay1, bgplotoverlay2, lightning_overlays, diagnostic_overlays
	common datafiles, orbit, fname

	if (selection ne '(None)') then begin
		case selection of
			'Areas' : begin
						items = orbit.point.lightning.area(*)
					end
			'Flashes': begin
						items = orbit.point.lightning.flash(*)
					end
			'Groups' : begin
						items = orbit.point.lightning.group(*)
					end
			'Events' : begin
						items = orbit.point.lightning.event(*)
					end
		endcase
		item_lats = items(*).location(0)
		item_lons = items(*).location(1)
		inplot = where( (item_lats ge flatmin) and (item_lats le flatmax) and $
					(item_lons ge flonmin) and (item_lons le flonmax), inplot_count)
		if (inplot_count gt 0) then begin
    			usersym,sin(findgen(21)/10.*!pi),cos(findgen(21)/10.*!pi),/fill
			uscale = usersym_scale(0)
			items_inplot = items(inplot)
			use_lats = items_inplot(*).location(0)
			use_lons = items_inplot(*).location(1)
			case selection of 
				'Areas' : begin
							colorbase = items_inplot(*).net_radiance
							sorted = sort(colorbase)
							cbmin = alog10(max([min( colorbase(where(colorbase ne 0))  ),4e4]))
							cbmax = alog10(max([max( colorbase(where(colorbase ne 0))  ),4e4]))
							cbdel = cbmax-cbmin
							usecolors = fix(65.+63.*(alog10(colorbase(sorted))-cbmin)/cbdel)
							scaler = 2.0
						end
				'Flashes': begin
							colorbase = items_inplot(*).radiance
							sorted = sort(colorbase)
							cbmin = alog10(max([min( colorbase(where(colorbase ne 0))  ),4e4]))
							cbmax = alog10(max([max( colorbase(where(colorbase ne 0))  ),4e4]))
							cbdel = cbmax-cbmin
							usecolors = fix(65.+63.*(alog10(colorbase(sorted))-cbmin)/cbdel)
							scaler = 1.5
						end
				'Groups' : begin
							colorbase = items_inplot(*).radiance
							sorted = sort(colorbase)
							cbmin = alog10(max([min( colorbase(where(colorbase ne 0))  ),4e4]))
							cbmax = alog10(max([max( colorbase(where(colorbase ne 0))  ),4e4]))
							cbdel = cbmax-cbmin
							usecolors = fix(65.+63.*(alog10(colorbase(sorted))-cbmin)/cbdel)
							scaler = 1.25
						end
				'Events' : begin
							colorbase = float(items_inplot(*).amplitude)
							sorted = sort(colorbase)
							usecolors = fix(65.+63.*colorbase(sorted)/128.)
							scaler = 1.0
						end
			endcase
			if (inplot_count gt 0) then begin
			; Markup
				ssize = .05+fix(800/200)*.10*scaler*uscale
				plots,use_lons(sorted),use_lats(sorted),psym=8,color=usecolors,symsize=ssize
				
			endif
		endif
	endif
end

pro add_diag_overlay, selection
	common bgcommon, bgplotb, bgplotwin, bgimg, bgoverlay
	common bgcommon2, bg_grid, flatmin, flatmax, flonmin, flonmax
	common bgcommon3, overlay_indices, bgplotoverlay1, bgplotoverlay2, lightning_overlays, diagnostic_overlays
	common datafiles, orbit, fname
	
	nadirs = fltarr(2,orbit.orbit_summary.one_second_count)
	get_nadir_locations, orbit.one_second(*).TAI93_time, nadirs
	lats = fltarr(orbit.orbit_summary.one_second_count)
	lats(0:orbit.orbit_summary.one_second_count-1) = nadirs(0,*)
	lons = fltarr(orbit.orbit_summary.one_second_count)
	lons(0:orbit.orbit_summary.one_second_count-1) = nadirs(1,*)
	
	inplot = where((lats ge flatmin) and (lats le flatmax) and (lons ge flonmin) and (lons le flonmax),in_count)
	ready = 'implemented'
	if (selection ne '(None)') then begin
		case selection of 
			'Alert Summary':begin
				good = where(orbit.one_second(inplot).alert_summary eq 0,good_count)
				fatal = where( ((orbit.one_second(inplot).alert_summary and 1) ne 0) or $
						   ((orbit.one_second(inplot).alert_summary and 4) ne 0) or $
						   ((orbit.one_second(inplot).alert_summary and 16) ne 0) or $
						   ((orbit.one_second(inplot).alert_summary and 64) ne 0), fatal_count)
				warn = where(  ((orbit.one_second(inplot).alert_summary and 2) ne 0) or $
							((orbit.one_second(inplot).alert_summary and 8) ne 0) or $
							((orbit.one_second(inplot).alert_summary and 32) ne 0) or $
							((orbit.one_second(inplot).alert_summary and 128) ne 0), warn_count)
				usecolors = bytarr(in_count)
				if (good_count gt 0) then begin
					usecolors(good) = 129
				endif
				if (warn_count gt 0) then begin
					usecolors(warn) = 130
				endif
				if (fatal_count gt 0) then begin
					usecolors(fatal) = 131
				endif		
			    end
			'Instrument Alert':begin
				good = where(orbit.one_second(inplot).instrument_alert eq 0,good_count)
				warn = where( ((orbit.one_second(inplot).instrument_alert and 8) ne 0), warn_count)
				fatal = where(  ((orbit.one_second(inplot).instrument_alert and 1) ne 0) or $
							((orbit.one_second(inplot).instrument_alert and 2) ne 0) or $
							((orbit.one_second(inplot).instrument_alert and 64) ne 0), fatal_count)
				indifferent = where( ((orbit.one_second(inplot).instrument_alert and 128) ne 0), indifferent_count)
				warnfatal = where(  ((orbit.one_second(inplot).instrument_alert and 2) ne 0) or $
								((orbit.one_second(inplot).instrument_alert and 4) ne 0) or $
								((orbit.one_second(inplot).instrument_alert and 16) ne 0) or $
								((orbit.one_second(inplot).instrument_alert and 32) ne 0), warnfatal_count)
				usecolors = bytarr(in_count)
				if (good_count gt 0) then begin
					usecolors(good) = 129
				endif
				if (indifferent_count gt 0) then begin
					usecolors(indifferent) = 133
				endif
				if (warn_count gt 0) then begin
					usecolors(warn) = 130
				endif
				if (warnfatal_count gt 0) then begin
					usecolors(warnfatal) = 132
				endif
				if (fatal_count gt 0) then begin
					usecolors(fatal) = 131
				endif					    
			    end
			'Platform Alert':begin
				good = where(orbit.one_second(inplot).platform_alert eq 0,good_count)
				warn = where(   ((orbit.one_second(inplot).platform_alert and 1) ne 0) or $
						  	((orbit.one_second(inplot).platform_alert and 4) ne 0) or $
						  	((orbit.one_second(inplot).platform_alert and 16) ne 0) or $
						  	((orbit.one_second(inplot).platform_alert and 64) ne 0), warn_count)
				fatal = where(  ((orbit.one_second(inplot).platform_alert and 2) ne 0) or $
							((orbit.one_second(inplot).platform_alert and 8) ne 0) or $
							((orbit.one_second(inplot).platform_alert and 32) ne 0) or $
							((orbit.one_second(inplot).platform_alert and 128) ne 0), fatal_count)
				usecolors = bytarr(in_count)
				if (good_count gt 0) then begin
					usecolors(good) = 129
				endif
				if (warn_count gt 0) then begin
					usecolors(warn) = 130
				endif
				if (fatal_count gt 0) then begin
					usecolors(fatal) = 131
				endif					    
			    end
			'External Alert':begin
				good = where(orbit.one_second(inplot).external_alert eq 0,good_count)
				warn = where( ((orbit.one_second(inplot).external_alert and 1) ne 0) or $
							((orbit.one_second(inplot).external_alert and 2) ne 0) or $
							((orbit.one_second(inplot).external_alert and 4) ne 0), warn_count)
				indifferent = where( ((orbit.one_second(inplot).external_alert and 8) ne 0) or $
								((orbit.one_second(inplot).external_alert and 16) ne 0) or $
								((orbit.one_second(inplot).external_alert and 32) ne 0) or $
								((orbit.one_second(inplot).external_alert and 64) ne 0), indifferent_count)
				usecolors = bytarr(in_count)
				if (good_count gt 0) then begin
					usecolors(good) = 129
				endif
				if (indifferent_count gt 0) then begin
					usecolors(indifferent) = 133
				endif
				if (warn_count gt 0) then begin
					usecolors(warn) = 130
				endif
			    end
			'Processing Alert':begin
				good = where(orbit.one_second(inplot).processing_alert eq 0,good_count)
				warn = where(   ((orbit.one_second(inplot).instrument_alert and 1) ne 0) or $
							((orbit.one_second(inplot).instrument_alert and 32) ne 0) or $
							((orbit.one_second(inplot).instrument_alert and 64) ne 0), warn_count)
				fatal = where(  ((orbit.one_second(inplot).instrument_alert and 2) ne 0) or $
							((orbit.one_second(inplot).instrument_alert and 4) ne 0) or $
							((orbit.one_second(inplot).instrument_alert and 8) ne 0), fatal_count)
				warnfatal = where(  ((orbit.one_second(inplot).instrument_alert and 16) ne 0), warnfatal_count)
				usecolors = bytarr(in_count)
				if (good_count gt 0) then begin
					usecolors(good) = 129
				endif
				if (warn_count gt 0) then begin
					usecolors(warn) = 130
				endif
				if (warnfatal_count gt 0) then begin
					usecolors(warnfatal) = 132
				endif
				if (fatal_count gt 0) then begin
					usecolors(fatal) = 131
				endif					    
			    end
			'Ephemeris Quality Flag':begin
					ready = 'nyi'
				end
			'Attitude Quality Flag':begin
					ready = 'nyi'
				end
			'Noise Index':begin
					ready = 'nyi'
				end
		endcase	
		if (ready ne 'nyi') then begin
			; print,usecolors(inplot)
			plots,lons(inplot),lats(inplot),psym=3,color=usecolors
		endif

	endif

end

pro diagoverlay_handler, event
	common bgcommon, bgplotb, bgplotwin, bgimg, bgoverlay
	common bgcommon2, bg_grid, flatmin, flatmax, flonmin, flonmax
	common bgcommon3, overlay_indices, bgplotoverlay1, bgplotoverlay2, lightning_overlays, diagnostic_overlays

	widget_control,bgplotoverlay1,get_uvalue=ltgoverlay_selection
	widget_control,bgplotoverlay2,set_uvalue=diagnostic_overlays(event.index)
	diagoverlay_selection = diagnostic_overlays(event.index)
	
	draw_bgimg_base
	add_ltg_overlay, ltgoverlay_selection
	add_diag_overlay, diagoverlay_selection
	
end


pro bgplothandler, event
	common sensor_read, this_sensor, file_format
	common datafiles, orbit, fname
	common otd_boom, boom
	common lisbase, lisb
	common lisapp_colors,red,green,blue
	common bgcommon, bgplotb, bgplotwin, bgimg, bgoverlay
	common sysinfo, version
	common select_bg_ids, bgs_selected, sbg_draw_id, sbg_select, sbg_base
	common bglists, bglist, sbg_list
	common caller, callingfunction 
	common fullpaths, app_path, data_path, bg_path, output_path
	common paths, lastpath, currentpath, origpath, shlbfile
	
	widget_control, event.id, get_value=buttonvalue
	thisred = bytarr(256) & thisgreen = bytarr(256) & thisblue = bytarr(256)
	ctsize = size(red)
	ctlen = ctsize(2)
	thisred(0:ctlen-1) = red & thisblue(0:ctlen-1) = blue & thisgreen(0:ctlen-1) = green
	
	case buttonvalue of
		'Edit & Print' :  begin
				wset,bgplotwin & 	if (not iswindows(0)) then device,pseu=8
				bgtempimg=tvrd()
				tvlct,thisred,thisgreen,thisblue,/get
				help,bgtempimg,thisred,thisgreen,thisblue
				live_image,bgtempimg,red=thisred,green=thisgreen,blue=thisblue
			  end
		'Save as GIF' : begin
				wset,bgplotwin & 	if (not iswindows(0)) then device,pseu=8
				; print,origpath
					cd,output_path[0]
				gif_save_name = dialog_pickfile(/noconfirm,/write,path=output_path[0])
				if (gif_save_name) then begin
					thisred(255) = 255 & thisgreen(255) = 255 & thisblue(255) = 255
					write_gif,gif_save_name,tvrd(),thisred,thisgreen,thisblue
				endif
				cd,currentpath
				end
		'Done': begin
				widget_control,bgplotb,/destroy
				if (callingfunction eq 'select_bgs') then begin
					widget_control,sbg_base,/sensitive
					widget_control,sbg_list,/sensitive
				endif
			  end
	endcase
end


pro movie_prefs, base, start, stop

	common datafiles, orbit, fname
	common movie_widgs, mb, timeseries_id, draw_id, mb_start_slid, mb_cur_slid, mb_end_slid, $
		mb_step_slid, mb_compression_droplist, mb_images_droplist, mb_bit_depth_droplist, mb_size_slid, mb_test_but, $
		mb_proc_time_slid, mb_outdir_field, mb_prefix_field, mb_play_but, mb_quit_but, mb_quality_droplist, mb_status_lab
	common movie_params, movie_start, movie_stop, frame_size, movie_dur, movie_comp, movie_step, $
		movie_frames, movie_size, frame_proctime, movie_proctime, comp_vals,bit_depth,quality
	common movie_params2, window_start, window_stop, mb_frame_lab
	common movie_widgs2, mb_build_but
	common movie_com3, timeseries_baseimg
	common movie_com4, proctimes, proctested, frame_sizes
	common ctsave, rold,gold,bold
	common mb_com_2, mb_remain_lab
	common fullpaths, app_path, data_path, bg_path, output_path
	common bg_info, fnamestub, candobgs, bgname, bgpathtouse

	tvlct,rold,gold,bold,/get
		
	window_start = start & window_stop = stop
	
	delt = long(stop)-long(start)
	
	mb = widget_base(group_leader=lisb,row=3,title='Make Movie Frames',/align_center)
	mb1 = widget_base(mb,row=2)
	mb4 = widget_base(mb,column=3,frame=1)
	
	mb1b = widget_base(mb1,row=1,frame=1)
	mb1t = widget_base(mb1,col=2)
	mb1tl = widget_base(mb1t,row=9,frame=1)
	mb1tr = widget_base(mb1t,row=4)
	

	comptypes = ['GIF89a','QuickTime/Graphics','QuickTime/Animation (RLE)','QuickTime/Photo (MJPEG-A)','QuickTime/Video',$
		'QuickTime/Video (Cinepak)','QuickTime/Video (Sorenson)','SGI/None','SGI/Animation (RLE)',$
		'SGI/Photo (JPEG)','SGI/Video (MVC1)']
		cv1 = [[[1,1,1],[1.175,1.175,1.175],[4.842,4.842,4.842],[14.149,6.823,4.517],[3.403,3.239,2.641],[7.521,7.447,7.447],[2.743,2.639,2.460],[113.068,113.068,113.068],[15.866,15.866,15.866],[10.600,3.450,2.543],[12.586,12.586,12.586]],$
		    	  	  [[1,1,1],[1.175,1.175,1.175],[3.207,3.207,3.207],[14.149,6.823,4.517],[3.403,3.239,2.641],[7.578,7.516,6.521],[2.743,2.639,2.460],[113.068,113.068,113.068],[15.866,15.866,15.866],[10.600,3.450,2.543],[12.586,12.586,12.586]]]
		cv2 = [[[1,1,1],[.552,.552,.552],[1.898,1.898,1.898],[4.049,1.975,1.284],[1.308,1.190,1.036],[1.852,1.549,1.281],[.981,.874,.781],[27.371,27.371,27.371],[4.703,4.703,4.703],[3.128,0.981,0.704],[3.081,3.081,3.081]],$
			       [[1,1,1],[.552,.552,.552],[2.052,2.052,2.052],[4.049,1.974,1.284],[1.308,1.190,1.036],[1.863,1.440,1.169],[.981,.874,.781],[27.371,27.371,27.371],[4.703,4.703,4.703],[3.128,0.981,0.704],[3.081,3.081,3.081]]]
		comp_vals = fltarr(3,11,2,2)
		comp_vals(*,*,*,0) = cv1 & comp_vals(*,*,*,1) = cv2
		
		qualtypes = ['Best','High','Medium']
	
	case candobgs of 
		0: begin
			wantbg = ['(BG File Not Found)']
		   end
		1: begin
			wantbg = ['No','Yes']
		   end
	endcase
	
	bitdepths = ['8 bit','16/24 bit']
	
	mb_timeseries_pane = widget_draw(mb1b,colors=220,xsize=640,ysize=128,retain=2)
	
	mb_frame_lab = widget_label(mb1tl,value='Number of Frames:      ',xsize=300)
	mb_start_slid = widget_slider(mb1tl,/drag,minimum=0,maximum=delt,title='Start',value=0,scroll=1,xsize=300)
	mb_cur_slid = widget_slider(mb1tl,/drag,minimum=0,maximum=delt,title='Current',value=0,scroll=1,xsize=300)
	mb_end_slid = widget_slider(mb1tl,/drag,minimum=0,maximum=delt,title='End',value=delt,scroll=1,xsize=300)
	mb_step_slid = widget_slider(mb1tl,/drag,minimum=2,maximum=1000,title='Frame Step (ms):',value=200,scroll=1,xsize=300)
	
	
	mb_size_slid = widget_slider(mb1tl,/drag,minimum=0,maximum=200,title='Movie Size (MB):',value=0,scroll=1,xsize=300)
	mb_proc_time_slid = widget_slider(mb1tl,/drag,minimum=1,maximum=1440,title='Est Proc Time (min):',value=10,scroll=1,xsize=300)
	mb_droplists = widget_base(mb1tl,row=4,frame=1)
	mb_compression_droplist = 	widget_droplist(mb_droplists,value=comptypes,title='Compression:',xsize=300)
	mb_quality_droplist = widget_droplist(mb_droplists,value=qualtypes,title='Quality:',xsize=300)
	mb_bit_depth_droplist = widget_droplist(mb_droplists,value=bitdepths,title='Bit Depth:',xsize=300)
	mb_images_droplist = widget_droplist(mb_droplists,value=wantbg,title='Plot BGs: ',xsize=300)
	mb_test_but = widget_button(mb1tl,value='Speed Test',xsize=300)
	
	mb_draw_pane = widget_draw(mb1tr,colors=220,xsize=320,ysize=320,retain=2,frame=1)
	mb_status_lab = widget_label(mb1tr,value='---------- Idle ----------')
	mb_remain_lab = widget_label(mb1tr,value='Remaining Time: --:--:--.- ')
	
	mb_field = widget_base(mb1tr,row=2,frame=1)
	mb_outdir_field = cw_field(mb_field,title='Output Path:',value=output_path[0])
	mb_prefix_field = cw_field(mb_field,title='File Prefix:',value='frm_                           ')
	mb_build_but = widget_button(mb4,value='Build Movie',xsize=220)
	; mb_play_but = widget_button(mb4,value='Play Movie',xsize=220)
	mb_quit_but = widget_button(mb4,value='Done',xsize=220)
	
	widget_control,mb,/realize
	
	bit_depth=0
	movie_start = double(start) & movie_stop = double(stop)
	frame_size = 4200.0/1024./1024.
	movie_dur  = float(movie_stop-movie_start)
	movie_comp = 1.0
	movie_step = 200./1000.
	movie_frames = movie_dur/movie_step
	movie_size = movie_frames*frame_size*movie_comp
	frame_proctime = 6./60.
	movie_proctime = movie_frames*frame_proctime
	proctimes = [6./60.,23./60.]
	proctested = [0,0]
	frame_sizes = [4200.0/1024./1024.,15000./1024./1024.]
	
	widget_control,mb_cur_slid,sensitive=0
	widget_control,mb_proc_time_slid,sensitive=0
	mbv = string(long(movie_frames))
	mbvs = 'Number of Frames: '+mbv
	widget_control,mb_frame_lab,set_value=mbvs
	widget_control,mb_size_slid,set_value=long(movie_size)
	widget_control,mb_proc_time_slid,set_value=long(movie_proctime)
	
	widget_control, mb_timeseries_pane, get_value = timeseries_id
	widget_control, mb_draw_pane, get_value = draw_id
	
	widget_control, mb_compression_droplist,set_uvalue=0
	widget_control, mb_quality_droplist,set_uvalue=0
	widget_control, mb_bit_depth_droplist,set_uvalue=0
	widget_control, mb_images_droplist,set_uvalue=0
	
	; widget_control,mb_play_but,sensitive=0
	
	load_my_color_tables
	wset,timeseries_id
	erase,31
	inwin = where((orbit.point.lightning.event(*).TAI93_time ge start) and $
			   (orbit.point.lightning.event(*).TAI93_time le stop),inct)
	if (inct gt 0) then begin
		times = 640*double(orbit.point.lightning.event(inwin).TAI93_time-start)/double(stop-start)
		amps = 128*float(orbit.point.lightning.event(inwin).amplitude)/127.
		plot,findgen(2),findgen(2),/noerase,/nodata,xrange=[0,640],yrange=[0,128],position=[0,0,1,1],xstyle=5,ystyle=5,xmargin=[0,0],ymargin=[0,0]
		plots,times,amps,psym=3,color=32
	endif
	timeseries_baseimg = tvrd()
	
	xmanager,'Movie Frames',mb,event='movie_handler',/no_block
end
		
pro movie_handler, event
	common datafiles, orbit, fname
	common movie_widgs, mb, timeseries_id, draw_id, mb_start_slid, mb_cur_slid, mb_end_slid, $
		mb_step_slid, mb_compression_droplist, mb_images_droplist, mb_bit_depth_droplist, mb_size_slid, mb_test_but, $
		mb_proc_time_slid, mb_outdir_field, mb_prefix_field, mb_play_but, mb_quit_but, mb_quality_droplist, mb_status_lab
	common movie_params, movie_start, movie_stop, frame_size, movie_dur, movie_comp, movie_step, $
		movie_frames, movie_size, frame_proctime, movie_proctime, comp_vals,bit_depth,quality
	common movie_params2, window_start, window_stop, mb_frame_lab
	common movie_widgs2, mb_build_but
	common movie_com3, timeseries_baseimg
	common movie_com4, proctimes, proctested, frame_sizes
	common ctsave, rold,gold,bold
	common mb_com_2, mb_remain_lab
	
	case 1 of 
		(event.id eq mb_start_slid): begin
			widget_control,mb_start_slid,get_value=new_start_idx
			new_start = window_start+double(new_start_idx)
			movie_start = long(new_start)
			widget_control,mb_end_slid,get_value=old_end_idx
			old_end = window_start+double(old_end_idx)
			if (new_start ge old_end) then begin
				new_end_idx = long(new_start+1L)-long(window_start)
				widget_control,mb_end_slid,set_value=new_end_idx
				new_end = window_start+double(new_end_idx)
			endif else begin 
				new_end = old_end
			endelse
			movie_dur = double(movie_stop-movie_start)
			movie_frames = movie_dur/movie_step
			movie_size = movie_frames*frame_size*movie_comp
			movie_proctime = movie_frames*frame_proctime
			mbv = string(long(movie_frames))
			mbvs = 'Number of Frames: '+mbv
			widget_control,mb_frame_lab,set_value=mbvs
			widget_control,mb_size_slid,set_value=long(movie_size)
			widget_control,mb_proc_time_slid,set_value=long(movie_proctime)
			widget_control,mb_cur_slid,set_value=new_start_idx

			winxmin = 640*float(new_start-window_start)/float(window_stop-window_start)
			winxmax = 640*float(new_end-window_start)/float(window_stop-window_start)
			xvect = [winxmin,winxmin,winxmax,winxmax,winxmin]
			yvect = [0,128,128,0,0]			
			sbounds=polyfillv(xvect,yvect,640,128)
			newimage = timeseries_baseimg
			if (sbounds(0) ne -1) then begin
				newimage(sbounds) = newimage(sbounds) + 32
			endif
			wset,timeseries_id
			tv,newimage
			end
		(event.id eq mb_end_slid): begin
			widget_control,mb_end_slid,get_value=new_end_idx
			new_end = window_start+double(new_end_idx)
			movie_stop = long(new_end)
			widget_control,mb_start_slid,get_value=old_start_idx
			old_start = window_start+double(old_start_idx)
			if (new_end lt old_start) then begin
				new_start_idx = max([long(new_end-1L)-long(window_start),0L])
				widget_control,mb_start_slid,set_value=new_start_idx
				widget_control,mb_cur_slid,set_value=new_start_idx
				new_start = window_start+double(new_start_idx)
			endif else begin
				new_start = old_start
			endelse
			movie_dur = double(movie_stop-movie_start)
			movie_frames = movie_dur/movie_step
			movie_size = movie_frames*frame_size*movie_comp
			movie_proctime = movie_frames*frame_proctime
			mbv = string(long(movie_frames))
			mbvs = 'Number of Frames: '+mbv
			widget_control,mb_frame_lab,set_value=mbvs
			widget_control,mb_size_slid,set_value=long(movie_size)
			widget_control,mb_proc_time_slid,set_value=long(movie_proctime)

			winxmin = 640*float(new_start-window_start)/float(window_stop-window_start)
			winxmax = 640*float(new_end-window_start)/float(window_stop-window_start)
			xvect = [winxmin,winxmin,winxmax,winxmax,winxmin]
			yvect = [0,128,128,0,0]			
			sbounds=polyfillv(xvect,yvect,640,128)
			newimage = timeseries_baseimg
			if (sbounds(0) ne -1) then begin
				newimage(sbounds) = newimage(sbounds) + 32
			endif
			wset,timeseries_id
			tv,newimage
			end
		(event.id eq mb_step_slid):begin
			widget_control,mb_step_slid,get_value=new_step
			movie_step = float(new_step)/1000.
			movie_frames = movie_dur/movie_step
			movie_size = movie_frames*frame_size*movie_comp
			movie_proctime = movie_frames*frame_proctime
			mbv = string(long(movie_frames))
			mbvs = 'Number of Frames: '+mbv
			widget_control,mb_frame_lab,set_value=mbvs
			widget_control,mb_size_slid,set_value=long(movie_size)
			widget_control,mb_proc_time_slid,set_value=long(movie_proctime)
			end
		(event.id eq mb_size_slid):begin
			widget_control,mb_size_slid,get_value=new_size
			movie_size = float(new_size)
			new_step = (movie_dur*frame_size*movie_comp/movie_size)*1000.
			movie_step = float(new_step)/1000.
			widget_control,mb_step_slid,set_value=new_step
			movie_frames = movie_dur/movie_step
			movie_proctime = movie_frames*frame_proctime
			mbv = string(long(movie_frames))
			mbvs = 'Number of Frames: '+mbv
			widget_control,mb_frame_lab,set_value=mbvs
			widget_control,mb_proc_time_slid,set_value=long(movie_proctime)
			end
		(event.id eq mb_proc_time_slid):begin
			widget_control,mb_proc_time_slid,get_value=new_proctime
			new_frames = long(float(new_proctime)/frame_proctime)
			movie_frames=new_frames
			movie_step = float(movie_dur)/float(movie_frames)
			movie_size = movie_frames*frame_size*movie_comp
			widget_control,mb_step_slid,set_value=movie_step*1000
			mbv = string(long(movie_frames))
			mbvs = 'Number of Frames: '+mbv
			widget_control,mb_frame_lab,set_value=mbvs
			widget_control,mb_size_slid,set_value=long(movie_size)
			; print,new_proctime,new_frames,movie_frames,movie_dur,movie_step,movie_size,frame_size,frame_proctime			
			end
		(event.id eq mb_compression_droplist):begin
			widget_control,mb_compression_droplist,set_uvalue=event.index
			widget_control,mb_quality_droplist,get_uvalue=qual_setting
			widget_control,mb_bit_depth_droplist,get_uvalue=bit_depth
			widget_control,mb_images_droplist,get_uvalue=use_bgs
			comp_setting = event.index
			movie_comp = comp_vals(qual_setting,comp_setting,bit_depth,use_bgs)
			movie_size = movie_frames*frame_size*movie_comp
			widget_control,mb_size_slid,set_value=long(movie_size)
			; print,movie_comp
			end		
		(event.id eq mb_quality_droplist):begin
			widget_control,mb_quality_droplist,set_uvalue=event.index
			widget_control,mb_compression_droplist,get_uvalue=comp_setting
			widget_control,mb_bit_depth_droplist,get_uvalue=bit_depth
			widget_control,mb_images_droplist,get_uvalue=use_bgs
			qual_setting = event.index
			movie_comp = comp_vals(qual_setting,comp_setting,bit_depth,use_bgs)
			movie_size = movie_frames*frame_size*movie_comp
			widget_control,mb_size_slid,set_value=long(movie_size)
			end		
		(event.id eq mb_bit_depth_droplist):begin
			widget_control,mb_bit_depth_droplist,set_uvalue=event.index
			widget_control,mb_compression_droplist,get_uvalue=comp_setting
			widget_control,mb_quality_droplist,get_uvalue=qual_setting
			widget_control,mb_images_droplist,get_uvalue=use_bgs
			bit_depth = event.index
			movie_comp = comp_vals(qual_setting,comp_setting,bit_depth,use_bgs)
			movie_size = movie_frames*frame_size*movie_comp
			widget_control,mb_size_slid,set_value=long(movie_size)
			end
		(event.id eq mb_images_droplist):begin
			widget_control,mb_images_droplist,set_uvalue=event.index
			widget_control,mb_compression_droplist,get_uvalue=comp_setting
			widget_control,mb_quality_droplist,get_uvalue=qual_setting
			widget_control,mb_bit_depth_droplist,get_uvalue=bit_depth
			use_bgs = event.index
			frame_size = frame_sizes[use_bgs]
			frame_proctime = proctimes[use_bgs]
			movie_comp = comp_vals(qual_setting,comp_setting,bit_depth,use_bgs)
			movie_size = movie_frames*frame_size*movie_comp
			movie_proctime = movie_frames*frame_proctime
			widget_control,mb_size_slid,set_value=long(movie_size)
			widget_control,mb_proc_time_slid,set_value=long(movie_proctime)
			if (proctested[use_bgs] eq 0) then begin
				widget_control,mb_proc_time_slid,sensitive=0
			endif else begin
				widget_control,mb_proc_time_slid,sensitive=1
			endelse
			end
		(event.id eq mb_test_but):begin
			widget_control,mb_status_lab,set_value='Testing speed...'
			widget_control,mb_start_slid,sensitive=0
			widget_control,mb_end_slid,sensitive=0
			widget_control,mb_step_slid,sensitive=0
			widget_control,mb_proc_time_slid,sensitive=0
			widget_control,mb_size_slid,sensitive=0
			widget_control,mb_build_but,sensitive=0
			widget_control,mb_quit_but,sensitive=0
			widget_control,mb_test_but,sensitive=0
			widget_control,mb_images_droplist,get_uvalue=use_bgs
			 ; print,movie_start,movie_stop,movie_step
			if (use_bgs eq 0) then begin
				test_start = systime(1)
				sample_start = double(movie_start+movie_stop)/2.
				sample_stop = double(sample_start+max([15.*movie_step,1.]))
				sample_frames = ((sample_stop-sample_start)/movie_step)
				zoom_overpass_wid,sample_start,sample_stop,movie_step,/preview
				test_end = systime(1)
				frame_proctime = 1./((sample_frames/float((test_end)-float(test_start)))*60.)
			endif else begin
				test_start = systime(1)
				sample_start = double(movie_start+movie_stop)/2.
				sample_stop = double(sample_start+max([10.*movie_step,1.]))
				sample_frames = ((sample_stop-sample_start)/movie_step)
				zoom_overpass_wid,sample_start,sample_stop,movie_step,/preview,/backgrounds
				test_end = systime(1)
				frame_proctime = 1./((sample_frames/float((test_end)-float(test_start)))*60.)
			endelse
			widget_control,mb_images_droplist,get_uvalue=use_bgs
			proctimes[use_bgs] = frame_proctime
			proctested[use_bgs] = 1
			movie_proctime = movie_frames*frame_proctime
			widget_control,mb_proc_time_slid,set_value=long(movie_proctime)
			; print, frame_proctime
			widget_control,mb_start_slid,sensitive=1
			widget_control,mb_end_slid,sensitive=1
			widget_control,mb_step_slid,sensitive=1
			widget_control,mb_proc_time_slid,sensitive=1
			widget_control,mb_size_slid,sensitive=1
			widget_control,mb_build_but,sensitive=1
			widget_control,mb_quit_but,sensitive=1
			widget_control,mb_test_but,sensitive=1
			widget_control,mb_status_lab,set_value='---------- Idle ----------'
			end
		(event.id eq mb_build_but):begin
			widget_control,mb_status_lab,set_value='Building frames...'
			widget_control,mb_start_slid,sensitive=0
			widget_control,mb_end_slid,sensitive=0
			widget_control,mb_step_slid,sensitive=0
			widget_control,mb_proc_time_slid,sensitive=0
			widget_control,mb_size_slid,sensitive=0
			widget_control,mb_build_but,sensitive=0
			widget_control,mb_quit_but,sensitive=0
			widget_control,mb_images_droplist,get_uvalue=use_bgs
			widget_control,mb_test_but,sensitive=0
			 ; print,movie_start,movie_stop,movie_step
			if (use_bgs eq 0) then begin
				test_start = systime(1)
				zoom_overpass_wid,movie_start,movie_stop,movie_step,/preview
				test_end = systime(1)
				frame_proctime = 1./((float(movie_frames)/float((test_end)-float(test_start)))*60.)
			endif else begin
				test_start = systime(1)
				zoom_overpass_wid,movie_start,movie_stop,movie_step,/backgrounds
				test_end = systime(1)
				frame_proctime = 1./((float(movie_frames)/float((test_end)-float(test_start)))*60.)
			endelse
			movie_proctime = movie_frames*frame_proctime
			widget_control,mb_proc_time_slid,set_value=long(movie_proctime)
			; print, frame_proctime
			widget_control,mb_test_but,sensitive=1
			widget_control,mb_start_slid,sensitive=1
			widget_control,mb_end_slid,sensitive=1
			widget_control,mb_step_slid,sensitive=1
			widget_control,mb_proc_time_slid,sensitive=1
			widget_control,mb_size_slid,sensitive=1
			widget_control,mb_build_but,sensitive=1
			widget_control,mb_quit_but,sensitive=1
			widget_control,mb_status_lab,set_value='---------- Idle ----------'
			end	
		(event.id eq mb_quit_but):begin
			timeseries_baseimg = 0
			widget_control,mb,/destroy
			tvlct,rold,gold,bold
			end		
	endcase
end

;--------------------------------------------------------------------------------------------
pro show_both_map, ctr_lat,ctr_lon
;--------------------------------------------------------------------------------------------
  common datafiles, orbit, fname
  common sensor_read, this_sensor, file_format
	common otd_boom, boom
	common ctsave, rold,gold,bold
	common fpbgcom, fpbgdid, fpbgbut1, fpbgbut2, fpbgbut3, fpbgbut4, fpbgb
	common fpbgcom2, bgimage, image
	common lisbase, lisb
	common fontcom, fontsize

  	waitb = widget_base(title='Working',tlb_frame_attr=0,scroll=0,/modal,column=1,group_leader=lisb)
	waitm = widget_label(waitb,value='Preparing lightning density and background plot ... this may take a moment...',/align_center)
	widget_control,waitb,/realize
  
	tvlct,rold,gold,bold,/get
     create_color_tables
	if (!version.os_family eq 'MacOS') then begin
		fontsize = 2.0
	endif else begin 
		if (!version.os_family eq 'unix') then begin
			fontsize = 0.8
		endif else begin
			fontsize = 1.0
		endelse
	endelse
	case this_sensor of
		'lis':begin
				sensor_scale = 1.0
		      end
		'otd':begin
			  	sensor_scale = 2.0
			end
	endcase

    win_size=900
    w_s_plus_100=win_size+100

    window,1,xs=win_size+100,ys=win_size+100,/pixmap
    erase,0

    latmin=fix(ctr_lat/2.)*2-10.  
    latmax=fix(ctr_lat/2.)*2+10.
    lonmin=fix(ctr_lon/2.)*2-10.
    lonmax=fix(ctr_lon/2.)*2+10.


    scale=float(win_size)/max([lonmax-lonmin,latmax-latmin])

    density_array=fltarr((lonmax-lonmin)*scale+100,(latmax-latmin)*scale+100)


    index=where(orbit.point.lightning.flash.location[1] ge lonmin and $
                orbit.point.lightning.flash.location[1] le lonmax and $
                orbit.point.lightning.flash.location[0] ge latmin and $
                orbit.point.lightning.flash.location[0] le latmax)
    ;print,'Num of flashes ',n_elements(index)

    index=where(orbit.point.lightning.group.location[1] ge lonmin and $
                orbit.point.lightning.group.location[1] le lonmax and $
                orbit.point.lightning.group.location[0] ge latmin and $
                orbit.point.lightning.group.location[0] le latmax)

    ;print,'Num of groups ',n_elements(index)

    if (index(0) ne -1) then begin
      first_group_time=min(orbit.point.lightning.group(index).tai93_time)
      last_group_time=max(orbit.point.lightning.group(index).tai93_time)
      s_index=sort(orbit.point.lightning.group(index).tai93_time)
      k=n_elements(index) /2
      middle_group_time=orbit.point.lightning.group(index(s_index(k))).tai93_time
    endif else begin
      bg_in_view=intarr(orbit.point.point_summary.bg_count)
      for i=0,3 do begin
       kindex = where(orbit.point.bg_summary.corners[1+i*2] gt lonmin and $
                     orbit.point.bg_summary.corners[1+i*2] lt lonmax and $
                     orbit.point.bg_summary.corners[0+i*2] gt latmin and $
                     orbit.point.bg_summary.corners[0+i*2] lt latmax)
       if (kindex(0) ne -1) then bg_in_view(kindex)=1
      endfor
      image_numbers=where(bg_in_view eq 1)
      first_group_time=min(orbit.point.bg_summary(image_numbers).tai93_time)
      last_group_time=max(orbit.point.bg_summary(image_numbers).tai93_time)
      s_index=sort(orbit.point.bg_summary(image_numbers).tai93_time)
      k=n_elements(image_numbers) /2
      middle_group_time=orbit.point.bg_summary(image_numbers(s_index(k))).tai93_time
    endelse


    ; determine time of middle group -- used for title

    ss=strmid(string(orbit.orbit_summary.UTC_start),17,2)
    mm=strmid(string(orbit.orbit_summary.UTC_start),14,2)
    hh=strmid(string(orbit.orbit_summary.UTC_start),11,2)
    date=strmid(string(orbit.orbit_summary.UTC_start),0,10)

    leap_second=long(orbit.orbit_summary.TAI93_start mod 86400)-ss-mm*60L-hh*3600L

    ref_days=long((middle_group_time-leap_second)/86400.)
    ref_time=(middle_group_time-leap_second-ref_days*86400.D0)
    offset_mm=long(long(ref_time(0))/60.)

    start_ss=ref_time-offset_mm*60L
    start_mm=offset_mm mod 60
    start_hh=offset_mm/60

    diam=fix(180./scale) +1.

	widget_control,waitm,set_value='Preparing density grid...'
	
    if (index(0) ne -1) then begin
    min_cluster=10
    for i=0L, n_elements(index)-1 do begin
      event_index=orbit.point.lightning.group(index(i)).child_address
      xlower=999
      xupper=-999
      ylower=999
      yupper=-999
      flag=0
      for m=0,orbit.point.lightning.group(index(i)).child_count-1 do begin
        
        if (orbit.point.lightning.event(event_index+m).cluster_index gt min_cluster) then begin
          px=fix((orbit.point.lightning.event(event_index+m).location[1]-lonmin)*scale+.5) 
          py=fix((orbit.point.lightning.event(event_index+m).location[0]-latmin)*scale+.5)
          polyfill,diam*sin(findgen(21)/10.*!pi)+px+50,diam*cos(findgen(21)/10.*!pi)+py+50,/dev,color=1
          xlower=min([xlower,px-diam-1+50])
          xupper=max([xupper,px+diam+1+50])
          ylower=min([ylower,py-diam-1+50])
          yupper=max([yupper,py+diam+1+50])
          flag=1
        endif
      endfor
      if (flag eq 1) then begin
        image=tvrd(xlower,ylower,xupper-xlower+1,yupper-ylower+1)
        density_array(xlower:xupper,ylower:yupper)=$
             density_array(xlower:xupper,ylower:yupper)+image(*,*)
        tv,intarr(xupper-xlower+1,yupper-ylower+1),xlower,ylower
      endif
    endfor
    endif

;   tv,density_array

    ; white out the edges of the density array

    tv,intarr(50,(latmax-latmin)*scale+100),0,0
    tv,intarr(50,(latmax-latmin)*scale+100),(lonmax-lonmin)*scale+50,0
    tv,intarr((lonmax-lonmin)*scale+100,50),0,0
    tv,intarr((lonmax-lonmin)*scale+100,50),0,(latmax-latmin)*scale+50

	widget_control,waitm,set_value='Adjusting for viewtime...'

       viewtime_array=fltarr((lonmax-lonmin)*scale+100,(latmax-latmin)*scale+100)

    world_grid_05x05=fltarr(720,360)

    ; loop thru viewtime, keeping accumulated viewtime in grid

    for i=0L,orbit.point.point_summary.vt_count-1 do begin
      px=fix((orbit.point.viewtime(i).location[1]-.25+180)*2)
      py=fix((orbit.point.viewtime(i).location[0]-.25+90)*2)
      world_grid_05x05(px,py)=world_grid_05x05(px,py)+ $
                              orbit.point.viewtime(i).effective_obs
    endfor
	maxwvg = max(world_grid_05x05(*,*))
    
    ; transfer viewtime information to viewtime array for the region of interest
    for i=(lonmin+180.)*2,(lonmax+180)*2 do begin
      for j=(latmin+90)*2,(latmax+90)*2 do begin
          loc_lat=j/2.-90.
          loc_lon=i/2.-180.
          px=fix((loc_lon-lonmin)*scale+.5) 
          py=fix((loc_lat-latmin)*scale+.5)
;          polyfill,[px-1,px+.5*scale+1,px+.5*scale+1,px-1,px-1]+50, $
 ;                  [py-1,py-1,py+.5*scale+1,py+.5*scale+1,py-1]+50,$
  ;                  /dev,color=world_grid_05x05(i,j)
          polyfill,[px-1,px+.5*scale+1,px+.5*scale+1,px-1,px-1]+50, $
                   [py-1,py-1,py+.5*scale+1,py+.5*scale+1,py-1]+50,$
                    /dev,color=135+fix(world_grid_05x05(i,j)*86./maxwvg)

       endfor
    endfor
    viewtime_array =tvrd(0,0,!d.x_size,!d.y_size)
;    viewtime_array = (viewtime_array-32.)*80./192.
; Viewtime array color values run from 0 to 255 in about 86 bins.  Kevin is
;   expecting them to run from 0 to 90.

	viewtime_array = (viewtime_array)*90./255.
	
  viewtime_array=smooth((viewtime_array > 20),20)
  net_density_array=float(density_array)*100./float(viewtime_array)
        
;-------------------
;   window,1,xs=win_size,ys=win_size

	widget_control,waitm,set_value='Preparing composite density image...'

    index=where(net_density_array gt 1)
    if (index(0) ne -1) then begin 
      max_log=max(alog(net_density_array(index)))
      color_scale = 10.
    ; print,'color_scale=',color_scale
  
      net_density_array(index) = (alog(net_density_array(index))*color_scale+16)
      index=where(net_density_array gt (16+79))
      if(index(0) ne -1) then net_density_array(index)=79+16
    endif

    index=where(net_density_array gt 0)
    if (index(0) ne -1) then begin
      for i=0L,n_elements(index)-1 do begin
        if (net_density_array(index(i)-1) eq 0) then net_density_array(index(i)-1)=1
        if (net_density_array(index(i)+1) eq 0) then net_density_array(index(i)+1)=1
        if (net_density_array(index(i)-w_s_plus_100) eq 0) then $
          net_density_array(index(i)-w_s_plus_100)=1
        if (net_density_array(index(i)+w_s_plus_100) eq 0) then $
          net_density_array(index(i)+w_s_plus_100)=1
    
        if (net_density_array(index(i)-2) eq 0) then net_density_array(index(i)-2)=1
        if (net_density_array(index(i)+2) eq 0) then net_density_array(index(i)+2)=1
        if (net_density_array(index(i)-2*w_s_plus_100) eq 0) then $
          net_density_array(index(i)-2*w_s_plus_100)=1
        if (net_density_array(index(i)+2*w_s_plus_100) eq 0) then $
          net_density_array(index(i)+2*w_s_plus_100)=1
      endfor
    endif

    net_density_array=net_density_array(50:(lonmax-lonmin)*scale-1+50,50:(latmax-latmin)*scale-1+50)


    window,4,xs=win_size,ys=win_size,/pixmap,colors=220
    !p.position=[.15,.16,.95,.96]
    ; plot map in window
    map_set,limit=[latmin,lonmin,latmax,lonmax],/noborder,color=1
    lower_time=max([first_group_time-230,orbit.orbit_summary.tai93_start])
    upper_time=min([last_group_time+230,orbit.orbit_summary.tai93_end])
    index=where(orbit.one_second.TAI93_time gt lower_time and $
                orbit.one_second.TAI93_time lt upper_time )

    corner_lat=fltarr(4)
    corner_lon=fltarr(4)

   ;  this_sensor='lis'
    px=[0,63,127,127,127,63,0,0,0]
    py=[0,0,0,63,127,127,127,63,0]
    
    if (index(0) ne -1) then begin
    for m=0L,n_elements(index)-1 do begin
      get_vector_pointing_vector,this_sensor,px,py,look_vector
      get_vector_earth_intersection,look_vector, $
         orbit.one_second(index(m)).TAI93_time*[1,1,1,1,1,1,1,1,1],location,0., return_code
      polyfill,location(*,1),location(*,0),color=1
    endfor

    result_1=convert_coord(lonmin,latmin,/data,/to_dev)
    result_2=convert_coord(lonmax,latmax,/data,/to_dev)
    coverage_mask=tvrd(result_1(0),result_1(1),result_2(0)-result_1(0),result_2(1)-result_1(1))
    endif

    window,1,xs=win_size,ys=win_size
    !p.position=[.15,.16,.95,.96]
    ; plot map in window
    map_set,limit=[latmin,lonmin,latmax,lonmax],/noborder,color=1
    erase,0

       
    ; make ocean gray
;    polyfill,[lonmin,lonmax,lonmax,lonmin,lonmin],[latmin,latmin,latmax,latmax,latmin],color=210+15-20
    polyfill,[lonmin,lonmax,lonmax,lonmin,lonmin],[latmin,latmin,latmax,latmax,latmin],color=128

    ; color in continents with gray color, and put black border arount continents
;    map_continents,/cont,/hires,/fill,color=240+10-20
     map_continents,/usa,/cont,/hires,/fill,color=129
   map_mask=tvrd(result_1(0),result_1(1),result_2(0)-result_1(0),result_2(1)-result_1(1))
    tv,map_mask + coverage_mask*4,result_1(0),result_1(1)
    map_continents,/cont,/hires,color=1

    ; put dotted grid on map
;   map_grid,latdel=1,londel=1,color=1,glinestyle=1
    map_continents,/usa,color=1
    map_grid,latdel=2,londel=2,color=1,glinestyle=1

    index=where(net_density_array gt 0)
    if (index(0) ne -1) then begin
      for i=0L,n_elements(index)-1 do begin
        px=(index(i) mod win_size)/scale+lonmin
        py=(index(i)/win_size)/scale+latmin
        plots,px,py,psym=3,color=net_density_array(index(i))
      endfor
    endif

    ; make border around plot 
    dt=(lonmax-lonmin)/80.
    oplot,[lonmin-dt,lonmax+dt,lonmax+dt,lonmin-dt,lonmin-dt],$
          [latmin-dt,latmin-dt,latmax+dt,latmax+dt,latmin-dt],$
          color=1,thick=0
    oplot,[lonmin,lonmax,lonmax,lonmin,lonmin],[latmin,latmin,latmax,latmax,latmin],$
     color=1,thick=3

    num=fix((lonmax-lonmin)/4.)
    
    for i=0,num do begin
      if (lonmin+4.*i ge 0) then begin
        result=convert_coord(lonmin+4*i,latmin-.03*(latmax-latmin),/data,/to_dev)
        xyouts,result(0),result(1),$
          string(lonmin+4.*i,format='(f5.1)')+'!eo!N E',charsize=fontsize,color=1,align=.5 ,/dev
        xyouts,result(0)+1,result(1),$
          string(lonmin+4.*i,format='(f5.1)')+'!eo!N E',charsize=fontsize,color=1,align=.5 ,/dev
        xyouts,result(0),result(1)+1,$
          string(lonmin+4.*i,format='(f5.1)')+'!eo!N E',charsize=fontsize,color=1,align=.5 ,/dev
        xyouts,result(0)+1,result(1)+1,$
          string(lonmin+4.*i,format='(f5.1)')+'!eo!N E',charsize=fontsize,color=1,align=.5 ,/dev
      endif else if (lonmin+4.*i eq 0) then begin
        result=convert_coord(lonmin+4*i,latmin-.03*(latmax-latmin),/data,/to_dev)
        xyouts,result(0),result(1), '0.0!eo!N',charsize=fontsize,color=1,align=.5,/dev
        xyouts,result(0)+1,result(1), '0.0!eo!N',charsize=fontsize,color=1,align=.5,/dev
        xyouts,result(0),result(1)+1, '0.0!eo!N',charsize=fontsize,color=1,align=.5,/dev
        xyouts,result(0)+1,result(1)+1, '0.0!eo!N',charsize=fontsize,color=1,align=.5,/dev
      endif else begin
        result=convert_coord(lonmin+4*i,latmin-.03*(latmax-latmin),/data,/to_dev)
        xyouts,result(0),result(1),$
          string(abs(lonmin+4.*i),format='(f5.1)')+'!eo!N W',charsize=fontsize,color=1,align=.5,/dev
        xyouts,result(0)+1,result(1),$
          string(abs(lonmin+4.*i),format='(f5.1)')+'!eo!N W',charsize=fontsize,color=1,align=.5,/dev
        xyouts,result(0),result(1)+1,$
          string(abs(lonmin+4.*i),format='(f5.1)')+'!eo!N W',charsize=fontsize,color=1,align=.5,/dev
        xyouts,result(0)+1,result(1)+1,$
          string(abs(lonmin+4.*i),format='(f5.1)')+'!eo!N W',charsize=fontsize,color=1,align=.5,/dev
      endelse
    endfor

    num=fix((latmax-latmin)/4.)
    for i=0,num do begin
      if (latmin+4.*i gt 0) then begin
        result=convert_coord(lonmin-.02*(lonmax-lonmin),latmin+4*i,/data,/to_dev)
        xyouts,result(0),result(1),$
          string(latmin+4.*i,format='(f4.1)')+'!eo!N N',charsize=fontsize,color=1,align=1.0 ,/dev
        xyouts,result(0)+1,result(1),$
          string(latmin+4.*i,format='(f4.1)')+'!eo!N N',charsize=fontsize,color=1,align=1.0 ,/dev
        xyouts,result(0),result(1)+1,$
          string(latmin+4.*i,format='(f4.1)')+'!eo!N N',charsize=fontsize,color=1,align=1.0 ,/dev
        xyouts,result(0)+1,result(1)+1,$
          string(latmin+4.*i,format='(f4.1)')+'!eo!N N',charsize=fontsize,color=1,align=1.0 ,/dev
      endif else if (latmin+4.*i eq 0) then begin
        result=convert_coord(lonmin-.02*(lonmax-lonmin),latmin+4*i,/data,/to_dev)
        xyouts,result(0),result(1), '0.0!eo!N',charsize=fontsize,color=1,align=1.0 ,/dev
        xyouts,result(0)+1,result(1), '0.0!eo!N',charsize=fontsize,color=1,align=1.0 ,/dev
        xyouts,result(0),result(1)+1, '0.0!eo!N',charsize=fontsize,color=1,align=1.0 ,/dev
        xyouts,result(0)+1,result(1)+1, '0.0!eo!N',charsize=fontsize,color=1,align=1.0 ,/dev
      endif else begin
        result=convert_coord(lonmin-.02*(lonmax-lonmin),latmin+4*i,/data,/to_dev)
        xyouts,result(0),result(1),$
          string(abs(latmin+4.*i),format='(f4.1)')+'!eo!N S',charsize=fontsize,color=1,align=1.,/dev
        xyouts,result(0)+1,result(1),$
          string(abs(latmin+4.*i),format='(f4.1)')+'!eo!N S',charsize=fontsize,color=1,align=1.,/dev
        xyouts,result(0),result(1)+1,$
          string(abs(latmin+4.*i),format='(f4.1)')+'!eo!N S',charsize=fontsize,color=1,align=1.,/dev
        xyouts,result(0)+1,result(1)+1,$
          string(abs(latmin+4.*i),format='(f4.1)')+'!eo!N S',charsize=fontsize,color=1,align=1.,/dev
      endelse
    endfor
    
    title=string(date,start_hh,start_mm,format='(a,3h   ,i2.2,1h:,i2.2,7h:00 UTC)')

    result=convert_coord((lonmax+lonmin)/2.,latmax+.01*(latmax-latmin),/data,/to_dev)
     xyouts,result(0),result(1),title,charsize=1.25*fontsize,align=.5,color=1,/dev
     xyouts,result(0),result(1)+1,title,charsize=1.25*fontsize,align=.5,color=1,/dev
     xyouts,result(0)+1,result(1),title,charsize=1.25*fontsize,align=.5,color=1,/dev
     xyouts,result(0)+1,result(1)+1,title,charsize=1.25*fontsize,align=.5,color=1,/dev

    !p.position=[.20,.029,.90,.069]
    plot,[0,1],[0,1],/nodata,charsize=.25*fontsize,color=0,/noerase

    result=convert_coord(.50,1.25,/data,/to_dev)
     xyouts,result(0),result(1),'Lightning Activity',charsize=1.25*fontsize,align=.5,color=1,/dev
     xyouts,result(0)+1,result(1),'Lightning Activity',charsize=1.25*fontsize,align=.5,color=1,/dev
     xyouts,result(0),result(1)+1,'Lightning Activity',charsize=1.25*fontsize,align=.5,color=1,/dev
     xyouts,result(0)+1,result(1)+1,'Lightning Activity',charsize=1.25*fontsize,align=.5,color=1,/dev


    result=convert_coord(.10,-.5,/data,/to_dev)
     xyouts,result(0),result(1),'Low',charsize=fontsize,align=.5,color=1,/dev
     xyouts,result(0)+1,result(1),'Low',charsize=fontsize,align=.5,color=1,/dev
     xyouts,result(0),result(1)+1,'Low',charsize=fontsize,align=.5,color=1,/dev
     xyouts,result(0)+1,result(1)+1,'Low',charsize=fontsize,align=.5,color=1,/dev

    result=convert_coord(.50,-.5,/data,/to_dev)
     xyouts,result(0),result(1),'Medium',charsize=fontsize,align=.5,color=1,/dev
     xyouts,result(0)+1,result(1),'Medium',charsize=fontsize,align=.5,color=1,/dev
     xyouts,result(0),result(1)+1,'Medium',charsize=fontsize,align=.5,color=1,/dev
     xyouts,result(0)+1,result(1)+1,'Medium',charsize=fontsize,align=.5,color=1,/dev

    result=convert_coord(.90,-.5,/data,/to_dev)
     xyouts,result(0),result(1),'High',charsize=fontsize,align=.5,color=1,/dev
     xyouts,result(0)+1,result(1),'High',charsize=fontsize,align=.5,color=1,/dev
     xyouts,result(0),result(1)+1,'High',charsize=fontsize,align=.5,color=1,/dev
     xyouts,result(0)+1,result(1)+1,'High',charsize=fontsize,align=.5,color=1,/dev

    for i=0,79 do begin
      polyfill,[i,i+1.,i+1,i,i]/80.,[0,0,1,1,0],color=i+16
    endfor
    oplot,[0.0001,.999,.999,0.0001,0.0001],[0.001,0.001,.999,.999,0.001],thick=3,color=1

    !p.position=[0.,0.,0.,0.]
    
    save_image=tvrd(0,0,!d.x_size,!d.y_size)
    tvlct,r,g,b,/get

    ; NOW PLOT BACKGROUNDS

	widget_control,waitm,set_value='Preparing background plot...'

    window,4,xs=win_size,ys=win_size,/pixmap
    !p.position=[.15,.16,.95,.96]
    ; plot map in window
    map_set,limit=[latmin,lonmin,latmax,lonmax],/noborder
    lower_time=max([first_group_time-230,orbit.orbit_summary.tai93_start])
    upper_time=min([last_group_time+230,orbit.orbit_summary.tai93_end])
    index=where(orbit.one_second.TAI93_time gt lower_time and $
                orbit.one_second.TAI93_time lt upper_time )

    corner_lat=fltarr(4)
    corner_lon=fltarr(4)

    ; this_sensor='lis'
    px=[0,63,127,127,127,63,0,0,0]
    py=[0,0,0,63,127,127,127,63,0]
   
    if (index(0) ne -1) then begin
    for m=0L,n_elements(index)-1 do begin
      get_vector_pointing_vector,this_sensor,px,py,look_vector
      get_vector_earth_intersection,look_vector, $
         orbit.one_second(index(m)).TAI93_time*[1,1,1,1,1,1,1,1,1],location,0., return_code
      polyfill,location(*,1),location(*,0),color=1
    endfor

    result_1=convert_coord(lonmin,latmin,/data,/to_dev)
    result_2=convert_coord(lonmax,latmax,/data,/to_dev)
    coverage_mask=tvrd(result_1(0),result_1(1),result_2(0)-result_1(0),result_2(1)-result_1(1))
    endif

    window,0,xs=win_size,ys=win_size
    !p.position=[.15,.16,.95,.96]
    ; plot map in window
    map_set,limit=[latmin,lonmin,latmax,lonmax],/noborder

    ; make ocean gray
;    polyfill,[lonmin,lonmax,lonmax,lonmin,lonmin],[latmin,latmin,latmax,latmax,latmin],color=210+15-20
    polyfill,[lonmin,lonmax,lonmax,lonmin,lonmin],[latmin,latmin,latmax,latmax,latmin],color=128

    ; color in continents with gray color, and put black border around continents
 ;   map_continents,/cont,/hires,/fill,color=240+10-20
    map_continents,/usa,/cont,/hires,/fill,color=129
    map_mask=tvrd(result_1(0),result_1(1),result_2(0)-result_1(0),result_2(1)-result_1(1))
    tv,map_mask + coverage_mask*4,result_1(0),result_1(1)
    map_continents,/cont,/hires,color=1
    map_continents,/usa,/hires,color=1

    ; put dotted grid on map
    map_grid,latdel=2,londel=2,color=1,glinestyle=1

    ; make border around plot 
    oplot,[lonmin,lonmax,lonmax,lonmin,lonmin],[latmin,latmin,latmax,latmax,latmin],$
     color=1,thick=3

    bg_in_view=intarr(orbit.point.point_summary.bg_count)
    for i=0,3 do begin 
     index = where(orbit.point.bg_summary.corners[1+i*2] gt lonmin and $
                   orbit.point.bg_summary.corners[1+i*2] lt lonmax and $
                   orbit.point.bg_summary.corners[0+i*2] gt latmin and $
                   orbit.point.bg_summary.corners[0+i*2] lt latmax)
     if (index(0) ne -1) then bg_in_view(index)=1
    endfor

    image_numbers=where(bg_in_view eq 1)
    ; print,'Examining background images : ',image_numbers

    usersym,.35*cos(findgen(25)/12.*!pi),.35*sin(findgen(25)/12.*!pi),/fill

	ivals = intarr(128,128) & jvals = intarr(128,128)
	kvals = bindgen(128)
	for i=0,127,1 do begin
		ivals(i,*) = kvals
		jvals(*,i) = kvals
	endfor
	symsizes = (ivals-63.5)^2 + (jvals-63.5)^2
	thissymsize = fontsize*sensor_scale*(1.1+.7*sqrt(symsizes)/90.)/2.
	onesymsize = fontsize*sensor_scale*(1.1+.7*sqrt((63.5^2+63.5^2))/90.)/2.
	
       if (image_numbers(0) ne -1) then begin

      for k=0,n_elements(image_numbers)-1 do begin
        ;print,'Next'
        msgval = string(format='("Adding background ",(I2)," of ",(I2))',k+1,$
        		n_elements(image_numbers))
      widget_control,waitm,set_value=msgval

        mv = get_max_val(image_numbers)
        read_lis_background, image_numbers(k), image, max_val=mv

        time=orbit.point.bg_summary(image_numbers(k)).tai93_time
        get_array_pointing_vector, this_sensor, array_look_vector
        get_array_earth_intersection, array_look_vector, time, array_location, 15000., return_code

	;	print,systime()
       ; for i=0,127 do for j=0,127 do begin
       ;   symsize=(i-63.5)^2 + (j-63.5)^2
       ;   if (array_location(i,j,1) gt lonmin and array_location(i,j,1) lt lonmax and $
       ;       array_location(i,j,0) gt latmin and array_location(i,j,0) lt latmax) then $
       ;       plots,array_location(i,j,1),array_location(i,j,0),psym=8,color=image(i,j),$
        ;         symsize=fontsize*sensor_scale*(1.1+.7*sqrt(symsize)/90.)/2.
       ; endfor
	;	print,systime()
		lonvals = array_location(*,*,1) & latvals = array_location(*,*,0)
		case this_sensor of
			'lis':begin
				inplot = where(lonvals gt lonmin and lonvals lt lonmax and $
						    latvals gt latmin and latvals lt latmax, inct)
				 end
			'otd':begin
				inplot = where(lonvals gt lonmin and lonvals lt lonmax and $
						    latvals gt latmin and latvals lt latmax and $
						    boom ne 0, inct)
				end
		endcase

		if (inct gt 0) then begin
			plots,lonvals(inplot),latvals(inplot),psym=8,color=image(inplot),$
				symsize=onesymsize
			print,min(image(inplot)),max(image(inplot))
		endif
	;	print,systime()
      endfor

    endif
    

    ; put dotted grid on map
    map_grid,latdel=2,londel=2,color=1,glinestyle=1

    ; make border around plot 
    oplot,[lonmin,lonmax,lonmax,lonmin,lonmin],[latmin,latmin,latmax,latmax,latmin],$
     color=1,thick=3

    index=where(orbit.point.lightning.group.location[1] ge lonmin and $
                orbit.point.lightning.group.location[1] le lonmax and $
                orbit.point.lightning.group.location[0] ge latmin and $
                orbit.point.lightning.group.location[0] le latmax)

    !p.position=[0,0,0,0]
    ; make border around plot 
    dt=(lonmax-lonmin)/80.
    oplot,[lonmin-dt,lonmax+dt,lonmax+dt,lonmin-dt,lonmin-dt],$
          [latmin-dt,latmin-dt,latmax+dt,latmax+dt,latmin-dt],$
          color=1,thick=0
    oplot,[lonmin,lonmax,lonmax,lonmin,lonmin],[latmin,latmin,latmax,latmax,latmin],$
     color=1,thick=3

    num=fix((lonmax-lonmin)/4.)
    
    for i=0,num do begin
      if (lonmin+4.*i ge 0) then begin
        result=convert_coord(lonmin+4*i,latmin-.03*(latmax-latmin),/data,/to_dev)
        xyouts,result(0),result(1),$
          string(lonmin+4.*i,format='(f5.1)')+'!eo!N E',charsize=fontsize,color=1,align=.5 ,/dev
        xyouts,result(0)+1,result(1),$
          string(lonmin+4.*i,format='(f5.1)')+'!eo!N E',charsize=fontsize,color=1,align=.5 ,/dev
        xyouts,result(0),result(1)+1,$
          string(lonmin+4.*i,format='(f5.1)')+'!eo!N E',charsize=fontsize,color=1,align=.5 ,/dev
        xyouts,result(0)+1,result(1)+1,$
          string(lonmin+4.*i,format='(f5.1)')+'!eo!N E',charsize=fontsize,color=1,align=.5 ,/dev
      endif else if (lonmin+4.*i eq 0) then begin
        result=convert_coord(lonmin+4*i,latmin-.03*(latmax-latmin),/data,/to_dev)
        xyouts,result(0),result(1), '0.0!eo!N',charsize=fontsize,color=1,align=.5,/dev
        xyouts,result(0)+1,result(1), '0.0!eo!N',charsize=fontsize,color=1,align=.5,/dev
        xyouts,result(0),result(1)+1, '0.0!eo!N',charsize=fontsize,color=1,align=.5,/dev
        xyouts,result(0)+1,result(1)+1, '0.0!eo!N',charsize=fontsize,color=1,align=.5,/dev
      endif else begin
        result=convert_coord(lonmin+4*i,latmin-.03*(latmax-latmin),/data,/to_dev)
        xyouts,result(0),result(1),$
          string(abs(lonmin+4.*i),format='(f5.1)')+'!eo!N W',charsize=fontsize,color=1,align=.5,/dev
        xyouts,result(0)+1,result(1),$
          string(abs(lonmin+4.*i),format='(f5.1)')+'!eo!N W',charsize=fontsize,color=1,align=.5,/dev
        xyouts,result(0),result(1)+1,$
          string(abs(lonmin+4.*i),format='(f5.1)')+'!eo!N W',charsize=fontsize,color=1,align=.5,/dev
        xyouts,result(0)+1,result(1)+1,$
          string(abs(lonmin+4.*i),format='(f5.1)')+'!eo!N W',charsize=fontsize,color=1,align=.5,/dev
      endelse
    endfor

    num=fix((latmax-latmin)/4.)
    for i=0,num do begin
      if (latmin+4.*i gt 0) then begin
        result=convert_coord(lonmin-.02*(lonmax-lonmin),latmin+4*i,/data,/to_dev)
        xyouts,result(0),result(1),$
          string(latmin+4.*i,format='(f4.1)')+'!eo!N N',charsize=fontsize,color=1,align=1.0 ,/dev
        xyouts,result(0)+1,result(1),$
          string(latmin+4.*i,format='(f4.1)')+'!eo!N N',charsize=fontsize,color=1,align=1.0 ,/dev
        xyouts,result(0),result(1)+1,$
          string(latmin+4.*i,format='(f4.1)')+'!eo!N N',charsize=fontsize,color=1,align=1.0 ,/dev
        xyouts,result(0)+1,result(1)+1,$
          string(latmin+4.*i,format='(f4.1)')+'!eo!N N',charsize=fontsize,color=1,align=1.0 ,/dev
      endif else if (latmin+4.*i eq 0) then begin
        result=convert_coord(lonmin-.02*(lonmax-lonmin),latmin+4*i,/data,/to_dev)
        xyouts,result(0),result(1), '0.0!eo!N',charsize=fontsize,color=1,align=1.0 ,/dev
        xyouts,result(0)+1,result(1), '0.0!eo!N',charsize=fontsize,color=1,align=1.0 ,/dev
        xyouts,result(0),result(1)+1, '0.0!eo!N',charsize=fontsize,color=1,align=1.0 ,/dev
        xyouts,result(0)+1,result(1)+1, '0.0!eo!N',charsize=fontsize,color=1,align=1.0 ,/dev
      endif else begin
        result=convert_coord(lonmin-.02*(lonmax-lonmin),latmin+4*i,/data,/to_dev)
        xyouts,result(0),result(1),$
          string(abs(latmin+4.*i),format='(f4.1)')+'!eo!N S',charsize=fontsize,color=1,align=1.,/dev
        xyouts,result(0)+1,result(1),$
          string(abs(latmin+4.*i),format='(f4.1)')+'!eo!N S',charsize=fontsize,color=1,align=1.,/dev
        xyouts,result(0),result(1)+1,$
          string(abs(latmin+4.*i),format='(f4.1)')+'!eo!N S',charsize=fontsize,color=1,align=1.,/dev
        xyouts,result(0)+1,result(1)+1,$
          string(abs(latmin+4.*i),format='(f4.1)')+'!eo!N S',charsize=fontsize,color=1,align=1.,/dev
      endelse
    endfor
    
    title=string(date,start_hh,start_mm,format='(a,3h   ,i2.2,1h:,i2.2,7h:00 UTC)')

    result=convert_coord((lonmax+lonmin)/2.,latmax+.01*(latmax-latmin),/data,/to_dev)
     xyouts,result(0),result(1),title,charsize=1.25*fontsize,align=.5,color=1,/dev
     xyouts,result(0),result(1)+1,title,charsize=1.25*fontsize,align=.5,color=1,/dev
     xyouts,result(0)+1,result(1),title,charsize=1.25*fontsize,align=.5,color=1,/dev
     xyouts,result(0)+1,result(1)+1,title,charsize=1.25*fontsize,align=.5,color=1,/dev

	widget_control,waitm,set_value='Compositing the plots...'

    bgimage=tvrd(0,0,!d.x_size,!d.y_size)
    image = bgimage
    
    index=where(save_image lt 96)
    if (index(0) ne -1) then image(index)=save_image(index)
    tv,image
    tvlct,r,g,b,/get

;    gifname='map_b.gif'

;    write_gif, gifname,image,r,g,b
	fpbgb = widget_base(row=2,title='Lightning Density Map with Backgrounds (half size)',scroll=0)
	fpbgbb = widget_base(fpbgb,col=4)
	fpbgbut1 = widget_button(fpbgbb,value='Edit & Print',xsize=225)
	fpbgbut3 = widget_button(fpbgbb,value='Show Full Size Images',xsize=225)
	fpbgbut2 = widget_button(fpbgbb,value='Save as Full Size GIF',xsize=225)
	fpbgbut4 = widget_button(fpbgbb,value='Done',xsize=225)
	fpbgdb = widget_base(fpbgb,col=2)
	fpbgdraw = widget_draw(fpbgdb,colors=220,retain=2,frame=3,xsize=!d.x_size/2,ysize=!d.y_size/2,/align_center)
	fpbgdraw2 = widget_draw(fpbgdb,colors=220,retain=2,frame=3,xsize=!d.x_size/2,ysize=!d.y_size/2,/align_center)

	widget_control,waitb,/destroy
		
	widget_control,fpbgb,/realize
	widget_control,fpbgdraw,get_value=fpbgdid2
      wset,fpbgdid2
	im2 = congrid(bgimage,!d.x_size,!d.y_size,/interp,cubic=-.5)
	tv,im2
	widget_control,fpbgdraw2,get_value=fpbgdid
	wset,fpbgdid
	im3 = congrid(image,!d.x_size,!d.y_size,/interp,cubic=-.5)
	tv,im3
	wdelete,0
	wdelete,1
		xmanager,'Fancy Plot with BG',fpbgb,event_handler='fplothandler2',/no_block

end

pro fplothandler2, event

	common ctsave, rold,gold,bold
	common fpbgcom, fpbgdid, fpbgbut1, fpbgbut2, fpbgbut3, fpbgbut4, fpbgb
	common fpbgcom2, bgimage, image
	common fullpaths, app_path, data_path, bg_path, output_path
	common paths, lastpath, currentpath, origpath, shlbfile

		case 1 of 
			(event.id eq fpbgbut1): begin
					wset,fpbgdid
					; tmpimg = tvrd(0,0,!d.x_size,!d.y_size)
					tmpimg = image
					tvlct,thisred,thisgreen,thisblue,/get
					live_image,tmpimg,red=thisred,green=thisgreen,blue=thisblue
					end
			(event.id eq fpbgbut2): begin
					wset,fpbgdid
					cd,output_path[0]
					gif_save_name = dialog_pickfile(/noconfirm,/write,path=output_path[0])
					cd,currentpath
					if (gif_save_name) then begin
						; gifimage = tvrd(0,0,!d.x_size,!d.y_size)
						gifimage = image
						tvlct,r,g,b,/get
						write_gif,gif_save_name,gifimage,r,g,b
					endif
					end
			(event.id eq fpbgbut3):begin
					window,12,xsize=723,ysize=722,retain=2
					wset,12
					tv,bgimage(135:858,142:864)
					window,13,xsize=723,ysize=722,retain=2
					wset,13
					tv,image(135:858,142:864)
					end
			(event.id eq fpbgbut4): begin
					bgimage = 0 & image = 0
					widget_control, fpbgb, /destroy
					tvlct,rold,gold,bold
					end
		endcase
end
